/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/graph/DepthFirstPreorderIterator.java#2 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.graph;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Stack;

/**
 * Iterator over the graph nodes in order of a depth-first <b>preorder</b> traversal.
 */
class DepthFirstPreorderIterator<T> implements Iterator<T> {

    private Graph<T> graph;

    private Stack<T> nodesToVisit;

    private Set<T> nodesAlreadyVisited;

    DepthFirstPreorderIterator(T startNode, Graph<T> graph) {
        this.graph = graph;
        this.nodesToVisit = new Stack<T>();
        this.nodesAlreadyVisited = new HashSet<T>();

        this.nodesToVisit.push(startNode);
    }

    @Override
    public boolean hasNext() {
        return !this.nodesToVisit.isEmpty();
    }

    @Override
    public T next() {
        T node = this.nodesToVisit.pop();
        this.nodesAlreadyVisited.add(node);

        for (T successor : this.graph.getSuccessors(node)) {
            if (!this.nodesAlreadyVisited.contains(successor)) {
                this.nodesToVisit.push(successor);
            }
        }

        return node;
    }

    @Override
    public void remove() {
        throw new UnsupportedOperationException();
    }

}
