/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/graph/HierarchicalObjectGraphFactory.java#2 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.graph;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.netapp.common.util.NoOpTransformer;
import com.netapp.common.util.Transformer;

public class HierarchicalObjectGraphFactory {

    public <S extends HierarchicalObject<S>, G extends Graph<S>> void buildGraph(G graph, S entryNode, S exitNode, Set<? extends S> rootHierarchicalObjects) {
        this.buildGraph(
            graph,
            entryNode,
            exitNode,
            rootHierarchicalObjects,
            new NoOpTransformer<S>()
        );
    }

    public <S extends HierarchicalObject<S>, T, G extends Graph<T>> void buildGraph(G graph, T entryNode, T exitNode, Set<? extends S> rootHierarchicalObjects, Transformer<S, T> nodeTransformer) {
        this.buildGraphHelper(
            graph,
            nodeTransformer,
            entryNode,
            rootHierarchicalObjects,
            new HashMap<S, T>(),
            exitNode
        );
    }

    private <S extends HierarchicalObject<S>, T> void buildGraphHelper(Graph<T> graph, Transformer<S, T> nodeTransformer, T predecessorNode, Set<? extends S> hierarchicalObjects, Map<S, T> builtNodes, T exitNode) {

        for (S hierarchicalObject : hierarchicalObjects) {
            T node = builtNodes.get(hierarchicalObject);

            if (node == null) {
                node = nodeTransformer.transform(hierarchicalObject);
                builtNodes.put(hierarchicalObject, node);
            }

            graph.addEdge(predecessorNode, node);

            this.buildGraphHelper(graph, nodeTransformer, node, hierarchicalObject.getSuccessors(), builtNodes, exitNode);

            if (hierarchicalObject.getSuccessors().isEmpty() && exitNode != null) {
                graph.addEdge(node, exitNode);
            }
        }

    }

}
