/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/logging/SLF4JLoggerAdapter.java#3 $
 *
 * Copyright (c) 2010 NetApp, Inc.
 * All rights reserved.
 */

package com.netapp.common.logging;

import org.slf4j.Logger;

/**
 * @author thomaswb
 *         Date: May 19, 2010
 */
public class SLF4JLoggerAdapter implements LogAdapter {
    private final Logger logger;

    SLF4JLoggerAdapter(Logger logger) {
        this.logger = logger;
    }

    @Override
    public void log(LogMessage msg) {
        dispatch(msg.getLevel(), msg.localizedMessage());
    }

    @Override
    public void log(LogMessage msg, Throwable t) {
        dispatch(msg.getLevel(), msg.localizedMessage(), t);
    }

    @Override
    public void log(LogLevel level, String msg) {
        dispatch(level, msg);
    }

    @Override
    public void log(LogLevel level, String msg, Throwable t) {
        dispatch(level, msg, t);
    }

    private void dispatch(LogLevel level, String msg) {
        switch (level) {
            case FATAL:
            case ERROR:
                logger.error(msg);
                break;
            case WARN:
                logger.warn(msg);
                break;
            case INFO:
            case PROGRESS:
                logger.info(msg);
                break;
            case DEBUG:
                logger.debug(msg);
                break;
            case TRACE:
                logger.trace(msg);
                break;
        }
    }

    private void dispatch(LogLevel level, String msg, Throwable t) {
        switch (level) {
            case FATAL:
            case ERROR:
                logger.error(msg, t);
                break;
            case WARN:
                logger.warn(msg, t);
                break;
            case INFO:
            case PROGRESS:
                logger.info(msg, t);
                break;
            case DEBUG:
                logger.debug(msg, t);
                break;
            case TRACE:
                logger.trace(msg, t);
                break;
        }
    }

    @Override
    public final boolean isLogLevelEnabled(LogLevel level) {
        switch (level) {
            case FATAL:
            case ERROR:
                return logger.isErrorEnabled();
            case WARN:
                return logger.isWarnEnabled();
            case INFO:
            case PROGRESS:
                return logger.isInfoEnabled();
            case DEBUG:
                return logger.isDebugEnabled();
            case TRACE:
                return logger.isTraceEnabled();
        }
        return true;
    }
}
