/*
 * $Id: //depot/prod/newport_news/main/modules/nn-platform/src/main/java/com/netapp/common/platform/darwin/OSXDeviceAnalyzer.java#3 $
 *
 * Copyright (c) 2008 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.platform.darwin;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.netapp.common.logging.NALogger;
import com.netapp.common.platform.DeviceAnalyzer;
import com.netapp.common.platform.DeviceType;
import com.netapp.common.platform.MsgKey;
import com.netapp.common.platform.PlatformException;
import com.netapp.common.platform.RawDeviceMapping;

/**
 * @author sunday
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public class OSXDeviceAnalyzer implements DeviceAnalyzer {
    private static final NALogger logger = NALogger.getLogger(OSXDeviceAnalyzer.class);

    private static final Pattern DEV_PATTERN = Pattern.compile("(/dev/(r)?disk\\d+)(s(\\d+))?");
    private static final int BASE_DEVICE_GROUP = 1;
    private static final int RAW_GROUP = 2;
    private static final int PARTITION_GROUP = 3;
    private static final int PARTITION_NUM_GROUP = 4;

    @Override
    public DeviceType determineDeviceType(File file) {
        Matcher m = DEV_PATTERN.matcher(file.getAbsolutePath());
        if (m.matches()) {
            if (m.group(RAW_GROUP) != null) {
                return DeviceType.RAW_DEVICE;
            } else if (m.group(PARTITION_GROUP) != null) {
                return DeviceType.PARTITION;
            }
            return DeviceType.BASE_DEVICE;
        }
        return DeviceType.NON_DEVICE;
    }

    @Override
    public File deviceForPartition(File disk, int partitionNumber) {
        String partition = String.format("s%s", partitionNumber);
        return this.deviceForPartition(disk, partition);
    }

    @Override
    public File deviceForPartition(File disk, String partition) {
        return new File(disk.getAbsolutePath() + partition);
    }

    @Override
    public String extractFullDiskPathElement(File file) {
        Matcher m = DEV_PATTERN.matcher(file.getAbsolutePath());
        if (!m.matches()) {
            logger.internal("Disk %s is not a valid path for this platform", file.getAbsolutePath());
            String error = logger.error(MsgKey.PLATFORM_RAW_DEVICE_MAPPER_UNSUPPORTED_OPERATION);
            throw new PlatformException(error);
        }
        return m.group(BASE_DEVICE_GROUP);
    }

    @Override
    public int extractPartitionNumber(File file) {
        Matcher m = DEV_PATTERN.matcher(file.getAbsolutePath());
        if (!m.matches() || m.group(PARTITION_NUM_GROUP) == null) {
            logger.internal("Partition %s is not a valid path for this platform", file.getAbsolutePath());
            String error = logger.error(MsgKey.PLATFORM_RAW_DEVICE_MAPPER_UNSUPPORTED_OPERATION);
            throw new PlatformException(error);
        }
        return Integer.parseInt(m.group(PARTITION_NUM_GROUP));
    }

    @Override
    public String extractPartitionPathElement(File file) {
        Matcher m = DEV_PATTERN.matcher(file.getAbsolutePath());
        if (!m.matches() || m.group(PARTITION_NUM_GROUP) == null) {
            logger.internal("Partition %s is not a valid path for this platform", file.getAbsolutePath());
            String error = logger.error(MsgKey.PLATFORM_RAW_DEVICE_MAPPER_UNSUPPORTED_OPERATION);
            throw new PlatformException(error);
        }
        return m.group(PARTITION_GROUP);
    }

    @Override
    public RawDeviceMapping lookupMappingByDisk(File disk) {
        File rawDevice = new File(disk.getAbsolutePath().replaceFirst("/dev/disk", "/dev/rdisk"));
        return new RawDeviceMapping(rawDevice, disk);
    }

    @Override
    public RawDeviceMapping lookupMappingByRawDevice(File rawDevice) {
        File disk = new File(rawDevice.getAbsolutePath().replaceFirst("/dev/rdisk", "/dev/disk"));
        return new RawDeviceMapping(rawDevice, disk);
    }
}
