/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/signal/SignalHandlerImpl.java#3 $
 *
 * Copyright (c) 2007 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.signal;

//CHECKSTYLE:OFF IllegalImport
import sun.misc.Signal;
import sun.misc.SignalHandler;

import com.netapp.common.MsgKey;
import com.netapp.common.logging.NALogger;

//CHECKSTYLE:ON IllegalImport

/**
 * @author taylor
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public class SignalHandlerImpl implements SignalHandler {

    protected SignalHandler oldHandler;
    protected boolean isFatal;
    protected String signalName;

    // Enumeration to tell the caller
    public static enum ProgramType { SERVER, GUI, CLI }
    protected ProgramType programType;

    private static final NALogger logger = NALogger.getLogger(SignalHandlerImpl.class);

    /**
     * @param signalName Signal to catch.
     * @param isFatal Should we exit after catching the signal?
     * @param programType Calling program type (Server, CLI, or GUI)
     */
    public SignalHandlerImpl(String signalName, boolean isFatal, ProgramType programType) {
        super();
        this.isFatal = isFatal;
        this.programType = programType;
        this.signalName = signalName;
    }

    /**
     * Install our signal handler and to replace the previous one.
     */
    public void replace() {
        replace(false);
    }

    /**
     * Install our signal handler
     * @param chain If true, call the former handler after our processing is complete.
     */
    public void replace(boolean chain) {
        logger.internal("Replacing signal handler for 'SIG" + this.signalName + "'");

        /*
         * Sometimes the VM won't let us register a handler.
         * I used to do this farther out and let the client worry about it,
         * but we don't really care, and some places weren't handling it, so
         * it's easier to just do here.
         */
        try {
            SignalHandler last = Signal.handle(new Signal(signalName), this);
            this.oldHandler = chain ? last : null;
        } catch (Throwable t) {
            logger.internal("Error replacing 'SIG" + this.signalName + "': " + t.getLocalizedMessage());
        }

    }

    /**
     * @see sun.misc.SignalHandler#handle(sun.misc.Signal)
     */
    @Override
    @edu.umd.cs.findbugs.annotations.SuppressWarnings("Dm")
    public void handle(Signal sig) {
        if (this.isFatal) {
            logger.error(MsgKey.SIGNAL_FATAL_SIGNAL, sig);
        } else {
            logger.warn(MsgKey.SIGNAL_NONFATAL_SIGNAL, sig);
        }
        try {
            // Chain back to previous handler, if one exists
            if (this.oldHandler != null && this.oldHandler != SIG_DFL && this.oldHandler != SIG_IGN) {
                logger.internal("Calling original handler for signal " + sig);
                this.oldHandler.handle(sig);
            }

        } catch (Throwable t) {
            // Don't want to let an exception here bubble up to the root of this handler thread.
            logger.internal("Original signal handler failed, reason " + t);
        }
        logger.internal("Handler for signal " + sig + " finished.");
        if (this.isFatal) {
            // We are in our own thread, so throwing an exception does not do anything.
            System.exit(1);
        }

    }

}
