/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/util/NumberUtils.java#4 $
 *
 * Copyright (c) 2011 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

/**
 * Utilities for Number classes
 *
 * @author millerm
 * @version $Revision: #4 $ $Date: 2012/04/16 $
 */

public class NumberUtils {

    private NumberUtils() { }

    /**
     * Null safe conversion of Integer to Long
     *
     * @param integer
     * @return null if integer is null, Long.valueOf(integer.longValue) otherwise
     */
    public static Long toLong(Integer integer) {
        return (integer == null) ? null : integer.longValue();
    }

    /**
     * Null-safe, and divide by zero safe, percent calculation.
     *
     * @return non-null percent ((numerator / denominator) * 100) if both input values are non null, and division by zero did not occur.
     *         percent will be between 0 and 100
     */
    public static Double calculatePercent(Number numerator, Number denominator) {
        if (numerator != null && denominator != null) {
            double percent = (numerator.doubleValue() / denominator.doubleValue()) * 100d;
            // check for division by zero
            if (!(Double.isNaN(percent) || Double.isInfinite(percent))) {
                return percent;
            }
        }
        return null;
    }


}
