/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/util/Size.java#6 $
 *
 * Copyright (c) 2008 Network Appliance, Inc.
 * All rights reserved.
 */
package com.netapp.common.util;

import java.io.Serializable;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.builder.HashCodeBuilder;

public class Size implements Serializable, Comparable<Size> {

    private static final Pattern SIZE_STRING_PATTERN = Pattern.compile("(\\d[\\d,]+)\\s*([a-zA-Z]{0,2})");

    private long value;

    private Unit unit;

    public Size(long value, Unit unit) {
        this.value = value;
        this.unit = unit;
    }

    public long getValue() {
        return this.value;
    }

    public Unit getUnit() {
        return this.unit;
    }

    public Size convertTo(Unit destUnit) {
        return new Size(this.valueIn(destUnit), destUnit);
    }

    public long valueIn(Unit destUnit) {
        return (this.value * this.unit.multiplier) / destUnit.multiplier;
    }

    @Override
    public boolean equals(Object object) {
        if (!(object instanceof Size)) {
            return false;
        }

        Size rhs = (Size) object;
        return this.value == rhs.valueIn(this.unit);
    }

    @Override
    public int hashCode() {
        return new HashCodeBuilder()
            .append(this.valueIn(Size.Unit.B))
            .toHashCode();
    }

    @Override
    public String toString() {
        return String.format("%,d%s", this.value, this.unit);
    }

    /**
     * Returns the size in a format expected by ONTAP.
     * The format to use is: < number > k|m|g|t
     * where "k" means kilobytes, "m" means megabytes,
     * "g" means gigabytes, and "t" means terabytes.
     * For example, "50 m" for 50 MB.
     */
    public String toOntapString() {
        return String.format("%d %c", this.value, this.unit.toString().toLowerCase().charAt(0));
    }

    public enum Unit {
        B(1),
        KB(1024),
        MB(1024 * KB.multiplier),
        GB(1024 * MB.multiplier),
        TB(1024 * GB.multiplier),
        PB(1024 * TB.multiplier),
        EB(1024 * PB.multiplier);

        private long multiplier;

        Unit(long multiplier) {
            this.multiplier = multiplier;
        }
    }

    @Override
    public int compareTo(Size other) {
        return (int) (this.valueIn(Unit.B) - other.valueIn(Unit.B));
    }

    public static Size fromString(String string) {
        Matcher matcher = SIZE_STRING_PATTERN.matcher(string);

        if (!matcher.matches()) {
            throw new IllegalArgumentException(string);
        }

        return new Size(
            Long.parseLong(matcher.group(1).replace(",", "")),
            matcher.group(2).isEmpty()
                ? Unit.B
                : Unit.valueOf(matcher.group(2).toUpperCase())
        );
    }
}
