/*
 * $Id: //depot/prod/newport_news/main/modules/nn-core/src/main/java/com/netapp/common/xml/DocumentNamespaceContext.java#3 $
 *
 * Copyright (c) 2009 NetApp, Inc.
 * All rights reserved.
 */
package com.netapp.common.xml;

import java.util.Iterator;

import javax.xml.XMLConstants;
import javax.xml.namespace.NamespaceContext;

import org.w3c.dom.Document;

/**
 * Resolves namespace prefixes and URIs against an underlying {@link Document DOM document}.
 *
 * Useful for executing XPath queries against a DOM document.
 * <p>
 * For example:
 * <blockquote><pre>
 * DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
 * <b>documentBuilderFactory.setNamespaceAware(true);</b>
 *
 * DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
 * Document document = documentBuilder.parse(...);
 *
 * XPath xPath = XPathFactory.newInstance().newXPath();
 * xPath.setNamespaceContext(<b>new DocumentNamespaceContext(document)</b>);
 * xPath.evaluate("/ns1:foo/ns2:bar", document);
 * </pre></blockquote>
 *
 * @author sirianni
 * @version $Revision: #3 $ $Date: 2011/06/24 $
 */
public class DocumentNamespaceContext implements NamespaceContext {

    private Document document;

    public DocumentNamespaceContext(Document document) {
        this.document = document;
    }

    @Override
    public String getNamespaceURI(String prefix) {
        if (prefix.equals(XMLConstants.DEFAULT_NS_PREFIX)) {
            return this.document.lookupNamespaceURI(null);
        } else {
            return this.document.lookupNamespaceURI(prefix);
        }
    }

    @Override
    public String getPrefix(String namespaceURI) {
        return this.document.lookupPrefix(namespaceURI);
    }

    @Override
    public Iterator<String> getPrefixes(String namespaceURI) {
        /*
         * This method is not called by the vast majority of the clients of the NamespaceContext interface.
         * It is not supported by most other NamespaceContext implementations either.
         */
        throw new UnsupportedOperationException();
    }

}
