package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.CatalogItemType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class CatalogItem extends VcloudEntity<CatalogItemType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType catalogReference;

  public CatalogItem(VcloudClient client, CatalogItemType catalogItemType_v1_5)
  {
    super(client, catalogItemType_v1_5);
    sortReferences_v1_5();
  }

  public ReferenceType getCatalogReference()
    throws VCloudException
  {
    if (this.catalogReference != null)
      return this.catalogReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public ReferenceType getEntityReference()
    throws VCloudException
  {
    if (((CatalogItemType)getResource()).getEntity() != null) {
      return ((CatalogItemType)getResource()).getEntity();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortReferences_v1_5()
  {
    for (LinkType catalogItemLink : ((CatalogItemType)getResource()).getLink())
      if ((catalogItemLink.getRel().equals("up")) && (catalogItemLink.getType().equals("application/vnd.vmware.vcloud.catalog+xml")))
      {
        this.catalogReference = catalogItemLink;
      }
  }

  public static CatalogItem getCatalogItemByReference(VcloudClient client, ReferenceType catalogItemRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + catalogItemRef.getHref());

    return new CatalogItem(client, (CatalogItemType)getResourceByReference(client, catalogItemRef));
  }

  public static CatalogItem getCatalogItemById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new CatalogItem(client, (CatalogItemType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.catalogItem+xml"));
  }

  public CatalogItem updateCatalogItem(CatalogItemType catalogItemType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String catalogItemUpdateUrl = ((CatalogItemType)getResource()).getHref();
    String catalogItemRequestXmlString = JAXBUtil.marshal(of.createCatalogItem(catalogItemType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + catalogItemUpdateUrl);

    return new CatalogItem(getVcloudClient(), (CatalogItemType)SdkUtil.put(getVcloudClient(), catalogItemUpdateUrl, catalogItemRequestXmlString, "application/vnd.vmware.vcloud.catalogItem+xml", 200));
  }

  public void delete()
    throws VCloudException
  {
    String catalogItemUrl = getReference().getHref();
    deleteCatalogItem(getVcloudClient(), catalogItemUrl);
  }

  public static void delete(VcloudClient client, ReferenceType catalogItemRef)
    throws VCloudException
  {
    deleteCatalogItem(client, catalogItemRef.getHref());
  }

  private static void deleteCatalogItem(VcloudClient client, String catalogItemrl)
    throws VCloudException
  {
    SdkUtil.delete(client, catalogItemrl, 204);
  }
}