package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.AllocatedIpAddressType;
import com.vmware.vcloud.api.rest.schema.AllocatedIpAddressesType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.NetworkConfigurationType;
import com.vmware.vcloud.api.rest.schema.OrgVdcNetworkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class OrgVdcNetwork extends VcloudEntity<OrgVdcNetworkType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType vdcReference;

  OrgVdcNetwork(VcloudClient client, OrgVdcNetworkType orgVdcNetworkType)
  {
    super(client, orgVdcNetworkType);
    sortRefs();
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortRefs()
  {
    for (LinkType vdcNetworkLink : ((OrgVdcNetworkType)getResource()).getLink())
      if ((vdcNetworkLink.getRel().equals("up")) && (vdcNetworkLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = vdcNetworkLink;
      }
  }

  public NetworkConfigurationType getConfiguration()
  {
    return ((OrgVdcNetworkType)getResource()).getConfiguration();
  }

  public static OrgVdcNetwork getOrgVdcNetworkByReference(VcloudClient client, ReferenceType orgVdcNetworkRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + orgVdcNetworkRef.getHref());

    return new OrgVdcNetwork(client, (OrgVdcNetworkType)getResourceByReference(client, orgVdcNetworkRef));
  }

  public static OrgVdcNetwork getOrgVdcNetworkById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new OrgVdcNetwork(client, (OrgVdcNetworkType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.network+xml"));
  }

  public List<AllocatedIpAddress> getAllocatedAddresses()
    throws VCloudException
  {
    String allocatedAddressesHref = getReference().getHref() + "/allocatedAddresses";

    AllocatedIpAddressesType allocatedIpAddressesType = (AllocatedIpAddressesType)SdkUtil.get(getVcloudClient(), allocatedAddressesHref, 200);

    List allocatedIpAddresses = new ArrayList();
    for (AllocatedIpAddressType allocatedIpAddressType : allocatedIpAddressesType.getIpAddress())
    {
      allocatedIpAddresses.add(new AllocatedIpAddress(getVcloudClient(), allocatedIpAddressType));
    }

    return allocatedIpAddresses;
  }
}