package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.ComplianceResultType;
import com.vmware.vcloud.api.rest.schema.DiskAttachOrDetachParamsType;
import com.vmware.vcloud.api.rest.schema.GuestCustomizationSectionType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.MediaInsertOrEjectParamsType;
import com.vmware.vcloud.api.rest.schema.NetworkConnectionSectionType;
import com.vmware.vcloud.api.rest.schema.NetworkConnectionType;
import com.vmware.vcloud.api.rest.schema.RasdItemsListType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.RelocateParamsType;
import com.vmware.vcloud.api.rest.schema.RuntimeInfoSectionType;
import com.vmware.vcloud.api.rest.schema.ScreenTicketType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema.VmCapabilitiesType;
import com.vmware.vcloud.api.rest.schema.VmPendingQuestionType;
import com.vmware.vcloud.api.rest.schema.VmQuestionAnswerType;
import com.vmware.vcloud.api.rest.schema.VmType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefType;
import com.vmware.vcloud.api.rest.schema.extension.VmVimInfoType;
import com.vmware.vcloud.api.rest.schema.ovf.CimString;
import com.vmware.vcloud.api.rest.schema.ovf.OperatingSystemSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.RASDType;
import com.vmware.vcloud.api.rest.schema.ovf.ResourceType;
import com.vmware.vcloud.api.rest.schema.ovf.SectionType;
import com.vmware.vcloud.api.rest.schema.ovf.VirtualHardwareSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.environment.EnvironmentType;
import com.vmware.vcloud.api.rest.schema.ovf.environment.PlatformSectionType;
import com.vmware.vcloud.api.rest.schema.ovf.environment.PropertySectionType;
import com.vmware.vcloud.sdk.constants.IpAddressAllocationModeType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.VMStatus;
import java.io.InputStream;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class VM extends AbstractVapp<VmType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private VirtualHardwareSectionType virtualHardwareSection;
  private OperatingSystemSectionType operatingSystemSection;
  private NetworkConnectionSectionType networkConnectionSection;
  private GuestCustomizationSectionType guestCustomizationSection;
  private RuntimeInfoSectionType runtimeInfoSection;
  private VirtualCpu virtualCpu;
  private VirtualMemory virtualMemory;
  private List<VirtualDisk> virtualDisks;
  private List<VirtualNetworkCard> networkCards;
  private List<VirtualMedia> medias;
  private ReferenceType parentVappReference;
  private VmVimInfoType vmVimInfoType;

  VM(VcloudClient client, VmType vmType)
  {
    super(client, vmType);
    sortOvfSectionsAndReferences_v1_5();
  }

  public static VM getVMByReference(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + vmRef.getHref());

    VmType vmType = (VmType)getResourceByReference(client, vmRef);
    return new VM(client, vmType);
  }

  public static VM getVMById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    VmType vmType = (VmType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.vm+xml");
    return new VM(client, vmType);
  }

  public VMStatus getVMStatus()
  {
    return VMStatus.fromValue(((VmType)getResource()).getStatus().intValue());
  }

  public Task updateVM(VmType vmType)
    throws VCloudException
  {
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    String vmUpdateUrl = getReference().getHref();
    String vmRequestXmlString = JAXBUtil.marshal(of.createVm(vmType));
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + vmUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), vmUpdateUrl, vmRequestXmlString, "application/vnd.vmware.vcloud.vm+xml", 202));
  }

  public GuestCustomizationSectionType getGuestCustomizationSection()
    throws VCloudException
  {
    if (this.guestCustomizationSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.guestCustomizationSection;
  }

  public static GuestCustomizationSectionType getGuestCustomizationSection(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (GuestCustomizationSectionType)SdkUtil.get(client, vmRef.getHref() + "/guestCustomizationSection/", 200);
  }

  public VirtualHardwareSectionType getVirtualHardwareSection()
    throws VCloudException
  {
    if (this.virtualHardwareSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.virtualHardwareSection;
  }

  public static VirtualHardwareSectionType getVirtualHardwareSection(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (VirtualHardwareSectionType)SdkUtil.get(client, vmRef.getHref() + "/virtualHardwareSection/", 200);
  }

  public OperatingSystemSectionType getOperatingSystemSection()
    throws VCloudException
  {
    if (this.operatingSystemSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.operatingSystemSection;
  }

  public static OperatingSystemSectionType getOperatingSystemSection(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (OperatingSystemSectionType)SdkUtil.get(client, vmRef.getHref() + "/operatingSystemSection/", 200);
  }

  public RuntimeInfoSectionType getRuntimeInfoSection()
    throws VCloudException
  {
    if (this.runtimeInfoSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.runtimeInfoSection;
  }

  public static RuntimeInfoSectionType getRuntimeInfoSection(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (RuntimeInfoSectionType)SdkUtil.get(client, vmRef.getHref() + "/runtimeInfoSection/", 200);
  }

  public Task updateSection(SectionType sectionType)
    throws VCloudException
  {
    com.vmware.vcloud.api.rest.schema.ovf.ObjectFactory ovfObjectFactory = new com.vmware.vcloud.api.rest.schema.ovf.ObjectFactory();
    com.vmware.vcloud.api.rest.schema.ObjectFactory vCloudObjectFactory_v1_5 = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    String sectionUrl = getReference().getHref();
    String sectionContentType = null;
    String sectionRequestString = null;
    if ((sectionType instanceof VirtualHardwareSectionType)) {
      JAXBElement section = ovfObjectFactory.createVirtualHardwareSection((VirtualHardwareSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/virtualHardwareSection/";

      sectionContentType = "application/vnd.vmware.vcloud.virtualHardwareSection+xml";
    } else if ((sectionType instanceof OperatingSystemSectionType)) {
      JAXBElement section = ovfObjectFactory.createOperatingSystemSection((OperatingSystemSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/operatingSystemSection/";

      sectionContentType = "application/vnd.vmware.vcloud.operatingSystemSection+xml";
    } else if ((sectionType instanceof GuestCustomizationSectionType)) {
      JAXBElement section = vCloudObjectFactory_v1_5.createGuestCustomizationSection((GuestCustomizationSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/guestCustomizationSection/";

      sectionContentType = "application/vnd.vmware.vcloud.guestCustomizationSection+xml";
    } else if ((sectionType instanceof NetworkConnectionSectionType)) {
      JAXBElement section = vCloudObjectFactory_v1_5.createNetworkConnectionSection((NetworkConnectionSectionType)sectionType);

      sectionRequestString = JAXBUtil.marshal(section);
      sectionUrl = sectionUrl + "/networkConnectionSection/";

      sectionContentType = "application/vnd.vmware.vcloud.networkConnectionSection+xml";
    } else {
      logger.log(Level.INFO, sectionType.getClass().getName() + " - " + SdkUtil.getI18nString(SdkMessage.NOT_VALID_SECTION_MSG));

      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NOT_VALID_SECTION_MSG));
    }

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), sectionUrl, sectionRequestString, sectionContentType, 202));
  }

  public Task insertMedia(MediaInsertOrEjectParamsType mediaInsertOrEjectParamsType)
    throws VCloudException
  {
    String url = getReference().getHref() + "/media/action/insertMedia";

    com.vmware.vcloud.api.rest.schema.ObjectFactory vcloudObjectFactory = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    String requestString = JAXBUtil.marshal(vcloudObjectFactory.createMediaInsertOrEjectParams(mediaInsertOrEjectParamsType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.POST_URL_MSG) + " - " + url);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.mediaInsertOrEjectParams+xml", 202));
  }

  public Task ejectMedia(MediaInsertOrEjectParamsType mediaInsertOrEjectParamsType)
    throws VCloudException
  {
    String url = getReference().getHref() + "/media/action/ejectMedia";

    com.vmware.vcloud.api.rest.schema.ObjectFactory vcloudObjectFactory = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    String requestString = JAXBUtil.marshal(vcloudObjectFactory.createMediaInsertOrEjectParams(mediaInsertOrEjectParamsType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.POST_URL_MSG) + " - " + url);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.mediaInsertOrEjectParams+xml", 202));
  }

  public BigInteger getVmSize()
    throws VCloudException
  {
    BigInteger size = BigInteger.ZERO;
    for (VirtualDisk disk : getDisks()) {
      String rasdItemType = disk.getItemResource().getResourceType().getValue();

      if ("17".equals(rasdItemType))
      {
        BigInteger diskSize = disk.getHardDiskSize();
        if (diskSize != null) {
          size = size.add(diskSize);
        }
      }
    }
    return size;
  }

  public VirtualCpu getCpu()
    throws VCloudException
  {
    if ((this.virtualCpu == null) && 
      (this.virtualHardwareSection != null)) {
      List items = this.virtualHardwareSection.getItem();
      if (items != null) {
        for (RASDType item : items)
          if (item.getResourceType().getValue().equals("3"))
          {
            this.virtualCpu = new VirtualCpu(item);
          }
      }
    }
    if (this.virtualCpu != null) {
      return this.virtualCpu;
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NO_VIRTUAL_CPU_MSG));
  }

  public static VirtualCpu getCpu(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/virtualHardwareSection/cpu";

    VirtualCpu virtualCpu = getCpuByUrl(client, url);
    return virtualCpu;
  }

  private static VirtualCpu getCpuByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    return new VirtualCpu((RASDType)SdkUtil.get(client, url, 200));
  }

  public Task updateCpu(VirtualCpu virtualCpuItem)
    throws VCloudException
  {
    String virtualCpuUpdateUrl = getReference().getHref() + "/virtualHardwareSection/cpu";

    String virtulCpuRequestXmlString = null;
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    virtulCpuRequestXmlString = JAXBUtil.marshal(of.createItem(virtualCpuItem.getItemResource()));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + virtualCpuUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), virtualCpuUpdateUrl, virtulCpuRequestXmlString, "application/vnd.vmware.vcloud.rasdItem+xml", 202));
  }

  public VirtualMemory getMemory()
    throws VCloudException
  {
    if ((this.virtualMemory == null) && 
      (this.virtualHardwareSection != null)) {
      List items = this.virtualHardwareSection.getItem();
      if (items != null) {
        for (RASDType item : items)
          if (item.getResourceType().getValue().equals("4"))
          {
            this.virtualMemory = new VirtualMemory(item);
          }
      }
    }
    if (this.virtualMemory != null) {
      return this.virtualMemory;
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NO_VIRTUAL_MEMORY_MSG));
  }

  public static VirtualMemory getMemory(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/virtualHardwareSection/memory";

    VirtualMemory virtualMemory = getMemoryByUrl(client, url);
    return virtualMemory;
  }

  private static VirtualMemory getMemoryByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    return new VirtualMemory((RASDType)SdkUtil.get(client, url, 200));
  }

  public Task updateMemory(VirtualMemory virtualMemoryItem)
    throws VCloudException
  {
    String virtualMemoryUpdateUrl = getReference().getHref() + "/virtualHardwareSection/memory";

    String virtulMemoryRequestXmlString = null;
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    virtulMemoryRequestXmlString = JAXBUtil.marshal(of.createItem(virtualMemoryItem.getItemResource()));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + virtualMemoryUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), virtualMemoryUpdateUrl, virtulMemoryRequestXmlString, "application/vnd.vmware.vcloud.rasdItem+xml", 202));
  }

  /** @deprecated */
  public List<SerialPort> getSerialPorts()
    throws VCloudException
  {
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NOT_SUPPORTED_API));
  }

  /** @deprecated */
  public static List<SerialPort> getSerialPorts(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NOT_SUPPORTED_API));
  }

  /** @deprecated */
  public Task updateSerialPorts(List<SerialPort> serialPorts)
    throws VCloudException
  {
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NOT_SUPPORTED_API));
  }

  public List<VirtualDisk> getDisks()
    throws VCloudException
  {
    if (this.virtualDisks == null) {
      this.virtualDisks = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items)
            if ((item.getResourceType().getValue().equals("17")) || (item.getResourceType().getValue().equals("6")) || (item.getResourceType().getValue().equals("5")))
            {
              this.virtualDisks.add(new VirtualDisk(item));
            }
        }
      }
    }
    return this.virtualDisks;
  }

  public static List<VirtualDisk> getDisks(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/virtualHardwareSection/disks";

    return getDisksByUrl(client, url);
  }

  private static List<VirtualDisk> getDisksByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    List hardwareItems = new ArrayList();
    RasdItemsListType rasdItems = (RasdItemsListType)SdkUtil.get(client, url, 200);

    if (rasdItems != null) {
      for (RASDType item : rasdItems.getItem()) {
        hardwareItems.add(new VirtualDisk(item));
      }
    }
    return hardwareItems;
  }

  private CimString getBusAddress(VirtualDisk disk, List<VirtualDisk> disks)
    throws VCloudException
  {
    for (VirtualDisk bus : disks) {
      if ((bus.getItemResource().getResourceType().getValue().equals("6")) && (disk.getHostResourceAttributeValue("busType").equals("6")))
      {
        if ((bus.getItemResource().getResourceSubType().getValue().equals(disk.getHardDiskBusType())) && (bus.getItemResource().getAddress().getValue().equals(disk.getItemResource().getAddress().getValue())))
        {
          return bus.getItemResource().getInstanceID();
        }
      } else if ((bus.getItemResource().getResourceType().getValue().equals("5")) && (disk.getHostResourceAttributeValue("busType").equals("5")))
      {
        if (bus.getItemResource().getAddress().getValue().equals(disk.getItemResource().getAddress().getValue()))
        {
          return bus.getItemResource().getInstanceID();
        }
      }
    }
    return null;
  }

  private VirtualDisk createController(VirtualDisk disk)
    throws VCloudException
  {
    RASDType busType = new RASDType();

    CimString any = new CimString();
    any.setValue("");

    CimString resourceSubType = new CimString();
    ResourceType resourceType = new ResourceType();

    if (disk.getHostResourceAttributeValue("busType").equals("5"))
    {
      resourceType.setValue("5");
    }
    else {
      resourceSubType.setValue(disk.getHardDiskBusType());
      resourceType.setValue("6");

      busType.setResourceSubType(resourceSubType);
    }

    busType.setElementName(any);
    busType.setDescription(any);
    busType.setInstanceID(any);
    busType.setAddress(disk.getItemResource().getAddress());
    busType.setResourceType(resourceType);
    busType.setInstanceID(disk.getItemResource().getParent());
    VirtualDisk bus = new VirtualDisk(busType);
    return bus;
  }

  private void processDisks(List<VirtualDisk> disks)
    throws VCloudException
  {
    for (int i = 0; i < disks.size(); i++)
      if ((((VirtualDisk)disks.get(i)).isHardDisk().booleanValue()) && (((VirtualDisk)disks.get(i)).getItemResource().getAddressOnParent() != null) && (((VirtualDisk)disks.get(i)).getItemResource().getAddress() != null))
      {
        CimString busAddress = getBusAddress((VirtualDisk)disks.get(i), disks);
        if (busAddress != null)
          ((VirtualDisk)disks.get(i)).getItemResource().setParent(busAddress);
        else
          disks.add(createController((VirtualDisk)disks.get(i)));
      }
  }

  public Task updateDisks(List<VirtualDisk> virtualDisks)
    throws VCloudException
  {
    processDisks(virtualDisks);
    String virtualDisksUpdateUrl = getReference().getHref() + "/virtualHardwareSection/disks";

    String virtulDisksRequestXmlString = null;
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    virtulDisksRequestXmlString = JAXBUtil.marshal(of.createRasdItemsList(getRASDItemsList(virtualDisks)));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + virtualDisksUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), virtualDisksUpdateUrl, virtulDisksRequestXmlString, "application/vnd.vmware.vcloud.rasdItemsList+xml", 202));
  }

  public List<VirtualNetworkCard> getNetworkCards()
    throws VCloudException
  {
    if (this.networkCards == null) {
      this.networkCards = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items)
            if (item.getResourceType().getValue().equals("10"))
            {
              this.networkCards.add(new VirtualNetworkCard(item));
            }
        }
      }
    }
    return this.networkCards;
  }

  public static List<VirtualNetworkCard> getNetworkCards(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/virtualHardwareSection/networkCards";

    return getNetworkCardsByUrl(client, url);
  }

  private static List<VirtualNetworkCard> getNetworkCardsByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    List hardwareItems = new ArrayList();
    RasdItemsListType rasdItems = (RasdItemsListType)SdkUtil.get(client, url, 200);

    if (rasdItems != null) {
      for (RASDType item : rasdItems.getItem()) {
        hardwareItems.add(new VirtualNetworkCard(item));
      }
    }

    return hardwareItems;
  }

  public Task updateNetworkCards(List<VirtualNetworkCard> virtualNetworkCards)
    throws VCloudException
  {
    String virtualNetworkCardsUpdateUrl = getReference().getHref() + "/virtualHardwareSection/networkCards";

    String virtulNetworkCardsRequestXmlString = null;
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    virtulNetworkCardsRequestXmlString = JAXBUtil.marshal(of.createRasdItemsList(getRASDItemsList(virtualNetworkCards)));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + virtualNetworkCardsUpdateUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), virtualNetworkCardsUpdateUrl, virtulNetworkCardsRequestXmlString, "application/vnd.vmware.vcloud.rasdItemsList+xml", 202));
  }

  public List<VirtualMedia> getMedias()
    throws VCloudException
  {
    if (this.medias == null) {
      this.medias = new ArrayList();
      if (this.virtualHardwareSection != null) {
        List items = this.virtualHardwareSection.getItem();
        if (items != null) {
          for (RASDType item : items)
            if ((item.getResourceType().getValue().equals("14")) || (item.getResourceType().getValue().equals("15")) || (item.getResourceType().getValue().equals("5")))
            {
              this.medias.add(new VirtualMedia(item));
            }
        }
      }
    }
    return this.medias;
  }

  public static List<VirtualMedia> getMedias(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/virtualHardwareSection/media";

    return getMediasByUrl(client, url);
  }

  private static List<VirtualMedia> getMediasByUrl(VcloudClient client, String url)
    throws VCloudException
  {
    List hardwareItems = new ArrayList();
    RasdItemsListType rasdItems = (RasdItemsListType)SdkUtil.get(client, url, 200);

    if (rasdItems != null) {
      for (RASDType item : rasdItems.getItem()) {
        hardwareItems.add(new VirtualMedia(item));
      }
    }

    return hardwareItems;
  }

  private RasdItemsListType getRASDItemsList(List<? extends HardwareItem> items)
  {
    RasdItemsListType rasdItemsListType = new RasdItemsListType();
    List rasdItems = rasdItemsListType.getItem();
    for (HardwareItem item : items) {
      rasdItems.add(item.getItemResource());
    }
    return rasdItemsListType;
  }

  private void sortOvfSectionsAndReferences_v1_5()
  {
    List sections = ((VmType)getResource()).getSection();

    Iterator it = sections.iterator();
    while (it.hasNext()) {
      JAXBElement sec = (JAXBElement)it.next();
      if ((sec.getValue() instanceof VirtualHardwareSectionType)) {
        this.virtualHardwareSection = ((VirtualHardwareSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof OperatingSystemSectionType)) {
        this.operatingSystemSection = ((OperatingSystemSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof NetworkConnectionSectionType)) {
        this.networkConnectionSection = ((NetworkConnectionSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof GuestCustomizationSectionType)) {
        this.guestCustomizationSection = ((GuestCustomizationSectionType)sec.getValue());
      }
      else if ((sec.getValue() instanceof RuntimeInfoSectionType))
        this.runtimeInfoSection = ((RuntimeInfoSectionType)sec.getValue());
      logger.log(Level.INFO, ((SectionType)sec.getValue()).getClass().getName());
    }

    for (LinkType vmLink : ((VmType)getResource()).getLink()) {
      if ((vmLink.getRel().equals("up")) && (vmLink.getType().equals("application/vnd.vmware.vcloud.vApp+xml")))
      {
        this.parentVappReference = vmLink;
      }
    }

    for (VCloudExtensionType vcloudExtension : ((VmType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        if ((((JAXBElement)jaxbObject).getValue() instanceof VmVimInfoType))
          this.vmVimInfoType = ((VmVimInfoType)((JAXBElement)jaxbObject).getValue());
      }
    }
    Iterator i$;
  }

  public ReferenceType getParentVappReference()
    throws VCloudException
  {
    if (this.parentVappReference != null)
      return this.parentVappReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public VimObjectRefType getVMVimRef()
    throws VCloudException
  {
    if ((this.vmVimInfoType != null) && (this.vmVimInfoType.getVmVimObjectRef() != null)) {
      return this.vmVimInfoType.getVmVimObjectRef();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public VimObjectRefType getVMDatastoreVimRef()
    throws VCloudException
  {
    if ((this.vmVimInfoType != null) && (this.vmVimInfoType.getDatastoreVimObjectRef() != null))
    {
      return this.vmVimInfoType.getDatastoreVimObjectRef();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public VimObjectRefType getVMHostVimRef()
    throws VCloudException
  {
    if ((this.vmVimInfoType != null) && (this.vmVimInfoType.getHostVimObjectRef() != null))
    {
      return this.vmVimInfoType.getHostVimObjectRef();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public Integer getVMDiskChainLength()
    throws VCloudException
  {
    if (this.vmVimInfoType != null) {
      return Integer.valueOf(this.vmVimInfoType.getVirtualDisksMaxChainLength());
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
  }

  public Boolean isVMwareToolsInstalled()
    throws VCloudException
  {
    return Boolean.valueOf(getRuntimeInfoSection().getVMWareTools() != null);
  }

  public PlatformSectionType getPlatformSection()
    throws VCloudException
  {
    if (((VmType)getResource()).getEnvironment() != null)
    {
      for (JAXBElement section : ((VmType)getResource()).getEnvironment().getSection()) {
        if ((section.getValue() instanceof PlatformSectionType)) {
          return (PlatformSectionType)section.getValue();
        }
      }
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
  }

  public PropertySectionType getPropertySection()
    throws VCloudException
  {
    if (((VmType)getResource()).getEnvironment() != null) {
      for (JAXBElement section : ((VmType)getResource()).getEnvironment().getSection())
      {
        if ((section.getValue() instanceof PropertySectionType)) {
          return (PropertySectionType)section.getValue();
        }
      }
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
  }

  public NetworkConnectionSectionType getNetworkConnectionSection()
    throws VCloudException
  {
    if (this.networkConnectionSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.networkConnectionSection;
  }

  public static NetworkConnectionSectionType getNetworkConnectionSection(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (NetworkConnectionSectionType)SdkUtil.get(client, vmRef.getHref() + "/networkConnectionSection/", 200);
  }

  public Collection<NetworkConnectionType> getNetworkConnections()
    throws VCloudException
  {
    if (this.networkConnectionSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return this.networkConnectionSection.getNetworkConnection();
  }

  public HashMap<Integer, String> getIpAddressesById()
    throws VCloudException
  {
    if (this.networkConnectionSection == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    HashMap ipAddresses = new HashMap();
    for (NetworkConnectionType networkConnection : this.networkConnectionSection.getNetworkConnection())
    {
      if (networkConnection.getIpAddress() != null) {
        ipAddresses.put(Integer.valueOf(networkConnection.getNetworkConnectionIndex()), networkConnection.getIpAddress());
      }
      else if (networkConnection.getIpAddressAllocationMode().equals(IpAddressAllocationModeType.DHCP.value()))
      {
        ipAddresses.put(Integer.valueOf(networkConnection.getNetworkConnectionIndex()), IpAddressAllocationModeType.DHCP.value());
      }
    }

    return ipAddresses;
  }

  public VmPendingQuestionType getVmQuestion()
    throws VCloudException
  {
    String url = getReference().getHref() + "/question";

    return (VmPendingQuestionType)SdkUtil.get(getVcloudClient(), url, 200);
  }

  public static VmPendingQuestionType getVmQuestion(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    String url = vmRef.getHref() + "/question";
    return (VmPendingQuestionType)SdkUtil.get(client, url, 200);
  }

  private static String createVmQuestionAnswerBody(int choiceId, String questionId)
  {
    com.vmware.vcloud.api.rest.schema.ObjectFactory of = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    VmQuestionAnswerType vmQuestionAnswerType = new VmQuestionAnswerType();
    vmQuestionAnswerType.setChoiceId(choiceId);
    vmQuestionAnswerType.setQuestionId(questionId);
    JAXBElement vmQuestionAnswer = of.createVmQuestionAnswer(vmQuestionAnswerType);

    return JAXBUtil.marshal(vmQuestionAnswer);
  }

  public void answerVmQuestion(int choiceId, String questionId)
    throws VCloudException
  {
    String url = getReference().getHref() + "/question/action/answer";

    String requestString = createVmQuestionAnswerBody(choiceId, questionId);
    SdkUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.vcloud.vmPendingAnswer+xml", 204);
  }

  public InputStream getScreen()
    throws VCloudException
  {
    String url = getReference().getHref() + "/screen";

    return RestUtil.downloadFile(getVcloudClient(), url);
  }

  public ScreenTicketType acquireTicket()
    throws VCloudException
  {
    String vmUrl = getReference().getHref();
    String url = vmUrl + "/screen/action/acquireTicket";
    return (ScreenTicketType)SdkUtil.post(getVcloudClient(), url, null, null, 200);
  }

  public Task installVMwareTools()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/installVMwareTools", null, null, 202));
  }

  public Task upgradeHardware()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/upgradeHardwareVersion", null, null, 202));
  }

  public Task consolidate()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/consolidate", null, null, 202));
  }

  public static Task installVMwareTools(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/installVMwareTools", null, null, 202));
  }

  public static Task upgradeHardware(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/upgradeHardwareVersion", null, null, 202));
  }

  public static Task consolidate(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/consolidate", null, null, 202));
  }

  /** @deprecated */
  public static Task relocate(VcloudClient client, ReferenceType vmRef, ReferenceType datastoreReference)
    throws VCloudException
  {
    RelocateParamsType paramsType = new RelocateParamsType();
    paramsType.setDatastore(datastoreReference);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createRelocateParams(paramsType));

    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/relocate", request, "application/vnd.vmware.vcloud.relocateVmParams+xml", 202));
  }

  /** @deprecated */
  public Task relocate(ReferenceType datastoreReference)
    throws VCloudException
  {
    RelocateParamsType paramsType = new RelocateParamsType();
    paramsType.setDatastore(datastoreReference);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createRelocateParams(paramsType));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/relocate", request, "application/vnd.vmware.vcloud.relocateVmParams+xml", 202));
  }

  public Task attachDisk(ReferenceType diskRef)
    throws VCloudException
  {
    DiskAttachOrDetachParamsType diskAttachOrDetachParamsType = new DiskAttachOrDetachParamsType();
    diskAttachOrDetachParamsType.setDisk(diskRef);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createDiskAttachOrDetachParams(diskAttachOrDetachParamsType));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/disk/action/attach", request, "application/vnd.vmware.vcloud.diskAttachOrDetachParams+xml", 202));
  }

  public Task attachDisk(ReferenceType diskRef, Integer busNumber, Integer unitNumber)
    throws VCloudException
  {
    DiskAttachOrDetachParamsType diskAttachOrDetachParamsType = new DiskAttachOrDetachParamsType();
    diskAttachOrDetachParamsType.setDisk(diskRef);
    diskAttachOrDetachParamsType.setBusNumber(busNumber);
    diskAttachOrDetachParamsType.setUnitNumber(unitNumber);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createDiskAttachOrDetachParams(diskAttachOrDetachParamsType));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/disk/action/attach", request, "application/vnd.vmware.vcloud.diskAttachOrDetachParams+xml", 202));
  }

  public Task detachDisk(ReferenceType diskRef)
    throws VCloudException
  {
    DiskAttachOrDetachParamsType detachDiskParams = new DiskAttachOrDetachParamsType();
    detachDiskParams.setDisk(diskRef);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createDiskAttachOrDetachParams(detachDiskParams));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/disk/action/detach", request, "application/vnd.vmware.vcloud.diskAttachOrDetachParams+xml", 202));
  }

  public Task detachDisk(ReferenceType diskRef, Integer busNumber, Integer unitNumber)
    throws VCloudException
  {
    DiskAttachOrDetachParamsType detachDiskParams = new DiskAttachOrDetachParamsType();
    detachDiskParams.setDisk(diskRef);
    detachDiskParams.setBusNumber(busNumber);
    detachDiskParams.setUnitNumber(unitNumber);
    String request = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createDiskAttachOrDetachParams(detachDiskParams));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/disk/action/detach", request, "application/vnd.vmware.vcloud.diskAttachOrDetachParams+xml", 202));
  }

  public List<ReferenceType> getAttachedDisks()
    throws VCloudException
  {
    List attachedDisks = new ArrayList();
    List vmDisks = getDisks();
    for (VirtualDisk vmDisk : vmDisks) {
      if (vmDisk.isHardDisk().booleanValue()) {
        try {
          ReferenceType diskRef = new ReferenceType();
          diskRef.setHref(vmDisk.getHostResourceAttributeValue("disk"));

          diskRef.setType("application/vnd.vmware.vcloud.disk+xml");
          attachedDisks.add(diskRef);
        } catch (VCloudException e) {
          logger.log(Level.INFO, vmDisk.getItemResource().getElementName().getValue() + " - not an independent disk attached to the vm");
        }
      }

    }

    return attachedDisks;
  }

  public Task enableNestedHypervisor()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/enableNestedHypervisor", null, null, 202));
  }

  public static Task enableNestedHypervisor(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/enableNestedHypervisor", null, null, 202));
  }

  public Task disableNestedHypervisor()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/disableNestedHypervisor", null, null, 202));
  }

  public static Task disableNestedHypervisor(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/disableNestedHypervisor", null, null, 202));
  }

  public Task updateHotAdd(boolean memoryHotAdd, boolean cpuHotAdd)
    throws VCloudException
  {
    return updateHotAdd(getVcloudClient(), getReference(), memoryHotAdd, cpuHotAdd);
  }

  public static Task updateHotAdd(VcloudClient client, ReferenceType vmRef, boolean memoryHotAdd, boolean cpuHotAdd)
    throws VCloudException
  {
    VmCapabilitiesType vmCapabilities = new VmCapabilitiesType();
    vmCapabilities.setCpuHotAddEnabled(Boolean.valueOf(cpuHotAdd));
    vmCapabilities.setMemoryHotAddEnabled(Boolean.valueOf(memoryHotAdd));
    String vmCapabilitiesString = JAXBUtil.marshal(new com.vmware.vcloud.api.rest.schema.ObjectFactory().createVmCapabilities(vmCapabilities));

    return new Task(client, (TaskType)SdkUtil.put(client, vmRef.getHref() + "/vmCapabilities", vmCapabilitiesString, "application/vnd.vmware.vcloud.vmCapabilitiesSection+xml", 202));
  }

  public Task checkCompliance()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), ((VmType)getResource()).getHref() + "/action/checkCompliance", null, null, 202));
  }

  public static Task checkCompliance(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.post(client, vmRef.getHref() + "/action/checkCompliance", null, null, 202));
  }

  public ComplianceResultType complianceResult()
    throws VCloudException
  {
    return (ComplianceResultType)SdkUtil.get(getVcloudClient(), ((VmType)getResource()).getHref() + "/complianceResult", 200);
  }

  public static ComplianceResultType complianceResult(VcloudClient client, ReferenceType vmRef)
    throws VCloudException
  {
    return (ComplianceResultType)SdkUtil.get(client, vmRef.getHref() + "/complianceResult", 200);
  }

  public Task reconfigureVM(VmType vmType)
    throws VCloudException
  {
    String reconfigureVMUrl = getReference().getHref() + "/action/reconfigureVm";

    com.vmware.vcloud.api.rest.schema.ObjectFactory objectFactory_v5_1 = new com.vmware.vcloud.api.rest.schema.ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createVm(vmType));
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), reconfigureVMUrl, requestBodyXml, "application/vnd.vmware.vcloud.vm+xml", 202));
  }
}