package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.EntityType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.MetadataType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.TasksInProgressType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public abstract class VcloudEntity<T extends EntityType> extends VcloudResource<T>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType referenceType;
  private List<Task> tasks;

  protected VcloudEntity(VcloudClient client, T entityType)
  {
    super(client, entityType);
    setTasks(client, entityType);
    setReference(entityType);
  }

  public List<Task> getTasks()
  {
    return this.tasks;
  }

  public ReferenceType getReference()
  {
    return this.referenceType;
  }

  private void setTasks(VcloudClient client, EntityType entityType)
  {
    if (this.tasks == null) {
      this.tasks = new ArrayList();
      if (entityType.getTasks() != null)
        for (TaskType taskType : entityType.getTasks().getTask())
          this.tasks.add(new Task(client, taskType));
    }
  }

  private void setReference(EntityType entity)
  {
    this.referenceType = new ReferenceType();
    this.referenceType.setHref(entity.getHref());
    this.referenceType.setName(entity.getName());
    this.referenceType.setType(entity.getType());
    this.referenceType.setId(entity.getId());
  }

  protected static EntityType getEntityById(VcloudClient client, String vCloudId, String mediaType)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + client.getVcloudApiURL() + "/entity" + "/" + vCloudId);

    EntityType entity = (EntityType)SdkUtil.get(client, client.getVcloudApiURL() + "/entity" + "/" + vCloudId, 200);

    for (LinkType link : entity.getLink()) {
      if (link.getType().equals(mediaType)) {
        return (EntityType)SdkUtil.get(client, link.getHref(), 200);
      }
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public Metadata getMetadata()
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + getReference().getHref() + "/metadata");

    return new Metadata(getVcloudClient(), (MetadataType)SdkUtil.get(getVcloudClient(), getReference().getHref() + "/metadata", 200));
  }
}