package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema.CaptureVAppParamsType;
import com.vmware.vcloud.api.rest.schema.CatalogItemType;
import com.vmware.vcloud.api.rest.schema.CatalogType;
import com.vmware.vcloud.api.rest.schema.CloneMediaParamsType;
import com.vmware.vcloud.api.rest.schema.CloneVAppParamsType;
import com.vmware.vcloud.api.rest.schema.CloneVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema.ComposeVAppParamsType;
import com.vmware.vcloud.api.rest.schema.DiskCreateParamsType;
import com.vmware.vcloud.api.rest.schema.DiskType;
import com.vmware.vcloud.api.rest.schema.InstantiateVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.MediaType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.ResourceEntitiesType;
import com.vmware.vcloud.api.rest.schema.ResourceReferenceType;
import com.vmware.vcloud.api.rest.schema.UploadVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema.VAppTemplateType;
import com.vmware.vcloud.api.rest.schema.VAppType;
import com.vmware.vcloud.api.rest.schema.VCloudExtensionType;
import com.vmware.vcloud.api.rest.schema.VdcStorageProfilesType;
import com.vmware.vcloud.api.rest.schema.VdcType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefType;
import com.vmware.vcloud.sdk.constants.ImageType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.VappTemplateStatus;
import com.vmware.vcloud.sdk.constants.VimObjectTypeEnum;
import com.vmware.vcloud.sdk.exception.DuplicateNameException;
import com.vmware.vcloud.sdk.exception.MissingPropertyException;
import com.vmware.vcloud.sdk.exception.ResourceNotAddedException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeoutException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.bind.JAXBElement;

public class Vdc extends VcloudEntity<VdcType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private Collection<ReferenceType> vappTemplateRefs;
  private Collection<ReferenceType> mediaRefs;
  private Collection<ReferenceType> storageProfileRefs;
  private Collection<ReferenceType> diskRefs;
  private HashMap<String, ReferenceType> vappRefsByName;
  private HashMap<String, ReferenceType> availableNetworkRefsByName;
  private ReferenceType orgReference;

  Vdc(VcloudClient client, VdcType vdcType_v1_5)
  {
    super(client, vdcType_v1_5);
    sortEntityRefs_v1_5();
  }

  public static Vdc getVdcByReference(VcloudClient client, ReferenceType vdcRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + vdcRef.getHref());

    return new Vdc(client, (VdcType)getResourceByReference(client, vdcRef));
  }

  public static Vdc getVdcById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new Vdc(client, (VdcType)getEntityById(client, vCloudId, "application/vnd.vmware.vcloud.vdc+xml"));
  }

  public VimObjectRefType getResourcePoolVimRef()
    throws VCloudException
  {
    for (VCloudExtensionType vcloudExtension : ((VdcType)getResource()).getVCloudExtension())
    {
      for (i$ = vcloudExtension.getAny().iterator(); i$.hasNext(); ) { Object jaxbObject = i$.next();
        if (((((JAXBElement)jaxbObject).getValue() instanceof VimObjectRefType)) && 
          (((VimObjectRefType)((JAXBElement)jaxbObject).getValue()).getVimObjectType().equals(VimObjectTypeEnum.RESOURCE_POOL.value())))
        {
          return (VimObjectRefType)((JAXBElement)jaxbObject).getValue();
        }
      }
    }
    Iterator i$;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  public ReferenceType getOrgReference()
    throws VCloudException
  {
    if (this.orgReference != null)
      return this.orgReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortEntityRefs_v1_5()
  {
    this.vappTemplateRefs = new ArrayList();
    this.mediaRefs = new ArrayList();
    this.vappRefsByName = new HashMap();
    this.availableNetworkRefsByName = new HashMap();
    this.storageProfileRefs = new ArrayList();
    this.diskRefs = new ArrayList();

    if (((VdcType)getResource()).getResourceEntities() != null) {
      ResourceEntitiesType resEntities = ((VdcType)getResource()).getResourceEntities();

      List entities = resEntities.getResourceEntity();

      for (ResourceReferenceType entity : entities) {
        if (entity.getType().equals("application/vnd.vmware.vcloud.vAppTemplate+xml"))
        {
          this.vappTemplateRefs.add(entity);
        } else if (entity.getType().equals("application/vnd.vmware.vcloud.media+xml"))
        {
          this.mediaRefs.add(entity);
        } else if (entity.getType().equals("application/vnd.vmware.vcloud.vApp+xml"))
        {
          this.vappRefsByName.put(entity.getName(), entity);
        } else if (entity.getType().equals("application/vnd.vmware.vcloud.disk+xml"))
        {
          this.diskRefs.add(entity);
        }
        else logger.log(Level.WARNING, SdkUtil.getI18nString(SdkMessage.UNKNOWN_REF_TYPE_MSG) + " - ", entity.getType());

      }

    }

    if (((VdcType)getResource()).getAvailableNetworks() != null) {
      AvailableNetworksType availableNetworksType = ((VdcType)getResource()).getAvailableNetworks();

      for (ReferenceType networkRef : availableNetworksType.getNetwork()) {
        this.availableNetworkRefsByName.put(networkRef.getName(), networkRef);
      }

    }

    for (LinkType vdcLink : ((VdcType)getResource()).getLink()) {
      if ((vdcLink.getRel().equals("up")) && (vdcLink.getType().equals("application/vnd.vmware.vcloud.org+xml")))
      {
        this.orgReference = vdcLink;
      }
    }
    if (((VdcType)getResource()).getVdcStorageProfiles() != null)
      for (ReferenceType ref : ((VdcType)getResource()).getVdcStorageProfiles().getVdcStorageProfile())
      {
        this.storageProfileRefs.add(ref);
      }
  }

  public HashMap<String, ReferenceType> getAvailableNetworkRefsByName()
  {
    return this.availableNetworkRefsByName;
  }

  public ReferenceType getAvailableNetworkRefByName(String name)
  {
    return (ReferenceType)this.availableNetworkRefsByName.get(name);
  }

  public Collection<ReferenceType> getAvailableNetworkRefs()
  {
    return this.availableNetworkRefsByName.values();
  }

  public Collection<ReferenceType> getVappTemplateRefs()
  {
    return this.vappTemplateRefs;
  }

  public Collection<ReferenceType> getMediaRefs()
  {
    return this.mediaRefs;
  }

  public Collection<ReferenceType> getDiskRefs()
  {
    return this.diskRefs;
  }

  public HashMap<String, ReferenceType> getVappRefsByName()
  {
    return this.vappRefsByName;
  }

  public Collection<ReferenceType> getVappTemplateRefsByName(String vAppTemplateName)
  {
    Collection vappTemplateRefs = new ArrayList();
    for (ReferenceType vappTemplateRef : getVappTemplateRefs()) {
      if (vappTemplateRef.getName().equalsIgnoreCase(vAppTemplateName)) {
        vappTemplateRefs.add(vappTemplateRef);
      }
    }
    return vappTemplateRefs;
  }

  public Collection<ReferenceType> getMediaRefsByName(String mediaName)
  {
    Collection mediaRefs = new ArrayList();
    for (ReferenceType mediaRef : getMediaRefs()) {
      if (mediaRef.getName().equalsIgnoreCase(mediaName)) {
        mediaRefs.add(mediaRef);
      }
    }

    return mediaRefs;
  }

  public ReferenceType getVappRefByName(String name)
  {
    return (ReferenceType)this.vappRefsByName.get(name);
  }

  public Collection<ReferenceType> getVappRefs()
  {
    return this.vappRefsByName.values();
  }

  public VappTemplate createVappTemplate(UploadVAppTemplateParamsType vappTemplParams)
    throws VCloudException
  {
    String url = getReference().getHref() + "/action/uploadVAppTemplate";

    JAXBElement params = new ObjectFactory().createUploadVAppTemplateParams(vappTemplParams);

    String reqPayload = JAXBUtil.marshal(params);

    return executeVappTemplUpload(getVcloudClient(), url, reqPayload, "application/vnd.vmware.vcloud.uploadVAppTemplateParams+xml");
  }

  public VappTemplate uploadVappTemplate(String vAppTemplateName, String vAppTemplateDesc, String localOvfFileLocation, Boolean manifestRequired, ReferenceType vdcStorageRef, ReferenceType catalogRef)
    throws VCloudException, DuplicateNameException, ResourceNotAddedException, TimeoutException
  {
    Catalog catalog = checkCatalogForDuplicates(catalogRef, vAppTemplateName);

    UploadVAppTemplateParamsType vappTemplParams = new UploadVAppTemplateParamsType();
    vappTemplParams.setDescription(vAppTemplateDesc);
    vappTemplParams.setName(vAppTemplateName);
    vappTemplParams.setManifestRequired(manifestRequired);
    vappTemplParams.setVdcStorageProfile(vdcStorageRef);
    VappTemplate vappTemplate = createVappTemplate(vappTemplParams);

    File ovfFile = new File(localOvfFileLocation);
    String ovfFileName = ovfFile.getName();
    FileInputStream ovfFileInputStream;
    try
    {
      ovfFileInputStream = new FileInputStream(ovfFile);
    } catch (FileNotFoundException e) {
      throw new VCloudRuntimeException(e);
    }
    vappTemplate.uploadOVFFile(ovfFileInputStream, ovfFile.length());
    vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());

    while (!((VAppTemplateType)vappTemplate.getResource()).isOvfDescriptorUploaded().booleanValue()) {
      vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());
    }
    try
    {
      ovfFileInputStream.close();
    } catch (IOException e) {
      throw new VCloudRuntimeException(e);
    }

    Set vmdkFileNames = vappTemplate.getUploadFileNames();
    for (String vmdkFileName : vmdkFileNames)
    {
      String vmdkFileLocation = localOvfFileLocation.replace(ovfFileName, vmdkFileName);

      File vmdkFile = new File(vmdkFileLocation);
      FileInputStream vmdkFileInputStream;
      try
      {
        vmdkFileInputStream = new FileInputStream(vmdkFile);
      } catch (FileNotFoundException e) {
        throw new VCloudRuntimeException(e);
      }
      vappTemplate.uploadFile(vmdkFileName, vmdkFileInputStream, vmdkFile.length());
      try
      {
        vmdkFileInputStream.close();
      } catch (IOException e) {
        throw new VCloudRuntimeException(e);
      }
    }

    vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());

    while (((VAppTemplateType)vappTemplate.getResource()).getStatus().intValue() == VappTemplateStatus.UNRESOLVED.value())
    {
      vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());
    }

    addResourceToCatalog(vappTemplate.getReference(), catalog);

    vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());

    return vappTemplate;
  }

  private Catalog checkCatalogForDuplicates(ReferenceType catalogRef, String resourceName)
    throws DuplicateNameException, VCloudException
  {
    Catalog catalog = Catalog.getCatalogByReference(getVcloudClient(), catalogRef);

    if (catalog.getCatalogItemRefsByName().containsKey(resourceName)) {
      throw new DuplicateNameException(MessageFormat.format(SdkUtil.getI18nString(SdkMessage.DUPLICATE_NAME), new Object[] { resourceName, ((CatalogType)catalog.getResource()).getName(), ((CatalogType)catalog.getResource()).getHref() }));
    }

    return catalog;
  }

  private void addResourceToCatalog(ReferenceType resourceRef, Catalog catalog)
    throws ResourceNotAddedException, VCloudException, TimeoutException
  {
    CatalogItemType catalogItemType = new CatalogItemType();
    catalogItemType.setName(resourceRef.getName());
    catalogItemType.setEntity(resourceRef);
    CatalogItem catalogItem;
    try
    {
      catalogItem = catalog.addCatalogItem(catalogItemType);
    } catch (VCloudException e) {
      throw new ResourceNotAddedException(e, resourceRef);
    }
    if (catalogItem.getTasks().size() > 0)
      ((Task)catalogItem.getTasks().get(0)).waitForTask(0L);
  }

  public Media uploadMedia(String mediaName, String mediaDescription, ImageType mediaType, String localMediaFile, ReferenceType vdcStorageProfileRef, ReferenceType catalogRef)
    throws VCloudException, DuplicateNameException, ResourceNotAddedException, TimeoutException
  {
    Catalog catalog = checkCatalogForDuplicates(catalogRef, mediaName);

    File mediaFile = new File(localMediaFile);
    FileInputStream mediaInputStream;
    try
    {
      mediaInputStream = new FileInputStream(mediaFile);
    } catch (FileNotFoundException e) {
      throw new VCloudRuntimeException(e);
    }

    MediaType mediaParams = new MediaType();
    mediaParams.setName(mediaName);
    mediaParams.setDescription(mediaDescription);
    mediaParams.setImageType(mediaType.value());
    mediaParams.setSize(mediaFile.length());
    mediaParams.setVdcStorageProfile(vdcStorageProfileRef);

    Media media = createMedia(mediaParams);

    String uploadMediaFileName = (String)media.getUploadFileNames().iterator().next();

    media.uploadFile(uploadMediaFileName, mediaInputStream, mediaFile.length());

    while (((MediaType)media.getResource()).getStatus().intValue() == 0) {
      media = Media.getMediaByReference(getVcloudClient(), media.getReference());
    }

    addResourceToCatalog(media.getReference(), catalog);

    media = Media.getMediaByReference(getVcloudClient(), media.getReference());

    return media;
  }

  public Media createMedia(MediaType mediaParams)
    throws VCloudException
  {
    String url = getReference().getHref() + "/media/";

    JAXBElement params = new ObjectFactory().createMedia(mediaParams);

    String reqPayload = JAXBUtil.marshal(params);

    return new Media(getVcloudClient(), (MediaType)SdkUtil.post(getVcloudClient(), url, reqPayload, "application/vnd.vmware.vcloud.media+xml", 201));
  }

  private static VappTemplate executeVappTemplUpload(VcloudClient client, String vdcActionUrl, String reqPayload, String contentType)
    throws VCloudException
  {
    return new VappTemplate(client, (VAppTemplateType)SdkUtil.post(client, vdcActionUrl, reqPayload, contentType, 201));
  }

  public Vapp composeVapp(ComposeVAppParamsType composeVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement composeVappParams = of.createComposeVAppParams(composeVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(composeVappParams);

    String postURL = getReference().getHref() + "/action/composeVApp";

    return new Vapp(getVcloudClient(), (VAppType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.composeVAppParams+xml", 201));
  }

  public VappTemplate captureVapp(CaptureVAppParamsType captureVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement captureVappParams = of.createCaptureVAppParams(captureVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(captureVappParams);

    String postURL = getReference().getHref() + "/action/captureVApp";

    return new VappTemplate(getVcloudClient(), (VAppTemplateType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.captureVAppParams+xml", 201));
  }

  public VappTemplate captureVapp(CaptureVAppParamsType captureVappParamsType, ReferenceType catalogRef)
    throws VCloudException, DuplicateNameException, ResourceNotAddedException, TimeoutException, MissingPropertyException
  {
    if (captureVappParamsType.getName() == null) {
      throw new MissingPropertyException(MessageFormat.format(SdkUtil.getI18nString(SdkMessage.MISSING_PROPERTY), new Object[] { "name", CaptureVAppParamsType.class.getSimpleName() }));
    }

    Catalog catalog = checkCatalogForDuplicates(catalogRef, captureVappParamsType.getName());

    VappTemplate vappTemplate = captureVapp(captureVappParamsType);

    addResourceToCatalog(vappTemplate.getReference(), catalog);

    vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());

    return vappTemplate;
  }

  public Vapp instantiateVappTemplate(InstantiateVAppTemplateParamsType instVappTemplParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();

    JAXBElement instVappParams = of.createInstantiateVAppTemplateParams(instVappTemplParamsType);

    String requestBodyXml = JAXBUtil.marshal(instVappParams);

    String postURL = getReference().getHref() + "/action/instantiateVAppTemplate";

    return new Vapp(getVcloudClient(), (VAppType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.instantiateVAppTemplateParams+xml", 201));
  }

  public Vapp cloneVapp(CloneVAppParamsType cloneVappParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneVappParams = of.createCloneVAppParams(cloneVappParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneVappParams);
    String postURL = getReference().getHref() + "/action/cloneVApp";

    return new Vapp(getVcloudClient(), (VAppType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneVAppParams+xml", 201));
  }

  public VappTemplate cloneVappTemplate(CloneVAppTemplateParamsType cloneVappTemplateParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneVappParams = of.createCloneVAppTemplateParams(cloneVappTemplateParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneVappParams);
    String postURL = getReference().getHref() + "/action/cloneVAppTemplate";

    return new VappTemplate(getVcloudClient(), (VAppTemplateType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneVAppTemplateParams+xml", 201));
  }

  public Media cloneMedia(CloneMediaParamsType cloneMediaParamsType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    JAXBElement cloneMediaParams = of.createCloneMediaParams(cloneMediaParamsType);

    String requestBodyXml = JAXBUtil.marshal(cloneMediaParams);
    String postURL = getReference().getHref() + "/action/cloneMedia";

    return new Media(getVcloudClient(), (MediaType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.cloneMediaParams+xml", 201));
  }

  public VappTemplate cloneVappTemplate(CloneVAppTemplateParamsType cloneVappTemplateParamsType, ReferenceType catalogRef)
    throws VCloudException, DuplicateNameException, MissingPropertyException, ResourceNotAddedException, TimeoutException
  {
    if (cloneVappTemplateParamsType.getName() == null) {
      throw new MissingPropertyException(MessageFormat.format(SdkUtil.getI18nString(SdkMessage.MISSING_PROPERTY), new Object[] { "name", CloneVAppTemplateParamsType.class.getSimpleName() }));
    }

    Catalog catalog = checkCatalogForDuplicates(catalogRef, cloneVappTemplateParamsType.getName());

    VappTemplate vappTemplate = cloneVappTemplate(cloneVappTemplateParamsType);

    addResourceToCatalog(vappTemplate.getReference(), catalog);

    vappTemplate = VappTemplate.getVappTemplateByReference(getVcloudClient(), vappTemplate.getReference());

    return vappTemplate;
  }

  public Media cloneMedia(CloneMediaParamsType cloneMediaParamsType, ReferenceType catalogRef)
    throws VCloudException, DuplicateNameException, MissingPropertyException, ResourceNotAddedException, TimeoutException
  {
    if (cloneMediaParamsType.getName() == null) {
      throw new MissingPropertyException(MessageFormat.format(SdkUtil.getI18nString(SdkMessage.MISSING_PROPERTY), new Object[] { "name", CloneMediaParamsType.class.getSimpleName() }));
    }

    Catalog catalog = checkCatalogForDuplicates(catalogRef, cloneMediaParamsType.getName());

    Media media = cloneMedia(cloneMediaParamsType);

    addResourceToCatalog(media.getReference(), catalog);

    media = Media.getMediaByReference(getVcloudClient(), media.getReference());

    return media;
  }

  public Collection<ReferenceType> getVdcStorageProfileRefs()
  {
    return this.storageProfileRefs;
  }

  public Disk createDisk(DiskCreateParamsType diskCreateParams)
    throws VCloudException
  {
    JAXBElement diskMediaParams = new ObjectFactory().createDiskCreateParams(diskCreateParams);

    String requestBodyXml = JAXBUtil.marshal(diskMediaParams);
    String postURL = getReference().getHref() + "/disk";

    return new Disk(getVcloudClient(), (DiskType)SdkUtil.post(getVcloudClient(), postURL, requestBodyXml, "application/vnd.vmware.vcloud.diskCreateParams+xml", 201));
  }
}