package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.VdcStorageProfileType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VdcStorageProfile extends VcloudEntity<VdcStorageProfileType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType vdcReference;

  VdcStorageProfile(VcloudClient client, VdcStorageProfileType vdcStorageProfileType)
  {
    super(client, vdcStorageProfileType);
    sortVdcStorageProfileReferences();
  }

  public static VdcStorageProfile getVdcStorageProfileByReference(VcloudClient client, ReferenceType vdcStorageProfileRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + vdcStorageProfileRef.getHref());

    return new VdcStorageProfile(client, (VdcStorageProfileType)getResourceByReference(client, vdcStorageProfileRef));
  }

  private void sortVdcStorageProfileReferences()
  {
    for (LinkType storageProfileLink : ((VdcStorageProfileType)getResource()).getLink())
      if ((storageProfileLink.getRel().equals("up")) && (storageProfileLink.getType().equals("application/vnd.vmware.vcloud.vdc+xml")))
      {
        this.vdcReference = storageProfileLink;
      }
  }

  public ReferenceType getVdcReference()
    throws VCloudException
  {
    if (this.vdcReference != null)
      return this.vdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }
}