package com.vmware.vcloud.sdk;

import com.vmware.vcloud.api.rest.schema.ovf.CimString;
import com.vmware.vcloud.api.rest.schema.ovf.RASDType;
import com.vmware.vcloud.api.rest.schema.ovf.ResourceType;
import com.vmware.vcloud.sdk.constants.BusSubType;
import com.vmware.vcloud.sdk.constants.BusType;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.math.BigInteger;
import java.util.List;
import java.util.Map;
import javax.xml.namespace.QName;

public class VirtualDisk extends HardwareItem
{
  public VirtualDisk(BigInteger diskSize, BusType busType, BusSubType busSubType)
  {
    super(new RASDType());
    CimString diskConfig = new CimString();
    Map cimAttributes = diskConfig.getOtherAttributes();
    if (busType.value().equals(BusType.IDE.value())) {
      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busType", "vcloud"), "5");
    }
    else
    {
      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busSubType", "vcloud"), busSubType.value());

      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busType", "vcloud"), "6");
    }

    cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "capacity", "vcloud"), diskSize.toString());

    CimString elementName = new CimString();
    elementName.setValue("");
    CimString instanceId = new CimString();
    instanceId.setValue("");
    ResourceType diskResourceType = new ResourceType();
    diskResourceType.setValue("17");

    RASDType disk = getItemResource();
    disk.setElementName(elementName);
    disk.setInstanceID(instanceId);
    disk.setResourceType(diskResourceType);
    List diskAttributes = disk.getHostResource();
    diskAttributes.add(diskConfig);
  }

  public VirtualDisk(BigInteger diskSize, BusType busType, BusSubType busSubType, Integer busNumber, Integer unitNumber)
  {
    super(new RASDType());
    CimString diskConfig = new CimString();
    Map cimAttributes = diskConfig.getOtherAttributes();
    if (busType.value().equals(BusType.IDE.value())) {
      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busType", "vcloud"), "5");
    }
    else
    {
      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busSubType", "vcloud"), busSubType.value());

      cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "busType", "vcloud"), "6");
    }

    cimAttributes.put(new QName("http://www.vmware.com/vcloud/v1.5", "capacity", "vcloud"), diskSize.toString());

    CimString elementName = new CimString();
    elementName.setValue("");
    CimString instanceID = new CimString();
    instanceID.setValue("");
    CimString unitNo = new CimString();
    unitNo.setValue(unitNumber.toString());
    CimString busNo = new CimString();
    busNo.setValue(busNumber.toString());
    CimString busAddress = new CimString();
    busAddress.setValue(busNumber + busSubType.value() + unitNumber);
    ResourceType diskResourceType = new ResourceType();
    diskResourceType.setValue("17");

    RASDType disk = getItemResource();
    disk.setElementName(elementName);
    disk.setInstanceID(instanceID);
    disk.setResourceType(diskResourceType);
    disk.setAddressOnParent(unitNo);
    disk.setAddress(busNo);
    disk.setParent(busAddress);
    List diskAttributes = disk.getHostResource();
    diskAttributes.add(diskConfig);
  }

  public VirtualDisk(RASDType virtualDiskItem)
  {
    super(virtualDiskItem);
  }

  public BigInteger getHardDiskSize()
    throws VCloudException
  {
    return new BigInteger(getHostResourceAttributeValue("capacity"));
  }

  public void updateHardDiskSize(BigInteger hardDiskSize)
    throws VCloudException
  {
    Map hostResourceAttributes = ((CimString)getItemResource().getHostResource().get(0)).getOtherAttributes();

    for (QName attr : hostResourceAttributes.keySet())
      if (attr.getLocalPart().equals("capacity"))
      {
        hostResourceAttributes.put(attr, hardDiskSize.toString());
      }
  }

  public String getHardDiskBusType()
    throws VCloudException
  {
    return getHostResourceAttributeValue("busSubType");
  }

  public Boolean isHardDisk()
  {
    return Boolean.valueOf(getItemResource().getResourceType().getValue().equals("17"));
  }

  String getHostResourceAttributeValue(String attributeName)
    throws VCloudException
  {
    if (getItemResource().getResourceType().getValue().equals("17"))
    {
      List hostResources = getItemResource().getHostResource();
      if (hostResources.size() > 0)
      {
        return getCimStringAttributeValue((CimString)getItemResource().getHostResource().get(0), attributeName);
      }
    }

    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.NOT_HARD_DISK_MSG) + " - " + getItemResource().getElementName().getValue());
  }
}