package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.AdminVdcStorageProfileType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminVdcStorageProfile extends VcloudEntity<AdminVdcStorageProfileType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminVdcReference;

  AdminVdcStorageProfile(VcloudClient client, AdminVdcStorageProfileType adminVdcStorageProfileType)
  {
    super(client, adminVdcStorageProfileType);
    sortAdminVdcStorageProfileReferences();
  }

  public static AdminVdcStorageProfile getAdminVdcStorageProfileByReference(VcloudClient client, ReferenceType adminVdcStorageProfileRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + adminVdcStorageProfileRef.getHref());

    return new AdminVdcStorageProfile(client, (AdminVdcStorageProfileType)getResourceByReference(client, adminVdcStorageProfileRef));
  }

  private void sortAdminVdcStorageProfileReferences()
  {
    for (LinkType storageProfileLink : ((AdminVdcStorageProfileType)getResource()).getLink())
      if ((storageProfileLink.getRel().equals("up")) && (storageProfileLink.getType().equals("application/vnd.vmware.admin.vdc+xml")))
      {
        this.adminVdcReference = storageProfileLink;
      }
  }

  public ReferenceType getAdminVdcReference()
    throws VCloudException
  {
    if (this.adminVdcReference != null)
      return this.adminVdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public AdminVdcStorageProfile updateAdminVdcStorageProfile(AdminVdcStorageProfileType adminVdcStorageProfileResource)
    throws VCloudException
  {
    String requestString = JAXBUtil.marshal(new ObjectFactory().createAdminVdcStorageProfile(adminVdcStorageProfileResource));

    return new AdminVdcStorageProfile(getVcloudClient(), (AdminVdcStorageProfileType)SdkUtil.put(getVcloudClient(), getReference().getHref(), requestString, "application/vnd.vmware.admin.vdcStorageProfile+xml", 200));
  }
}