package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.GatewayFeaturesType;
import com.vmware.vcloud.api.rest.schema.GatewayType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class EdgeGateway extends VcloudEntity<GatewayType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminVdcReference;

  EdgeGateway(VcloudClient client, GatewayType gatewayType)
  {
    super(client, gatewayType);
    sortRefs();
  }

  private void sortRefs()
  {
    for (LinkType adminVdcNetworkLink : ((GatewayType)getResource()).getLink())
      if ((adminVdcNetworkLink.getRel().equals("up")) && (adminVdcNetworkLink.getType().equals("application/vnd.vmware.admin.vdc+xml")))
      {
        this.adminVdcReference = adminVdcNetworkLink;
      }
  }

  public ReferenceType getAdminVdcReference()
    throws VCloudException
  {
    if (this.adminVdcReference != null)
      return this.adminVdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public static EdgeGateway getEdgeGatewayByReference(VcloudClient client, ReferenceType edgeGatewayTypeRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + edgeGatewayTypeRef.getHref());

    return new EdgeGateway(client, (GatewayType)getResourceByReference(client, edgeGatewayTypeRef));
  }

  public static EdgeGateway getEdgeGatewayById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new EdgeGateway(client, (GatewayType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.edgeGateway+xml"));
  }

  public Task updateEdgeGateway(GatewayType gatewayParams)
    throws VCloudException
  {
    String requestXml = JAXBUtil.marshal(new ObjectFactory().createEdgeGateway(gatewayParams));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.put(getVcloudClient(), ((GatewayType)getResource()).getHref(), requestXml, "application/vnd.vmware.admin.edgeGateway+xml", 202));
  }

  public Task reapplyServices()
    throws VCloudException
  {
    String reapplyServicesURL = getReference().getHref() + "/action/reapplyServices";

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), reapplyServicesURL, null, null, 202));
  }

  public Task redeploy()
    throws VCloudException
  {
    String redeployURL = getReference().getHref() + "/action/redeploy";

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), redeployURL, null, null, 202));
  }

  public Task syncSyslogServer()
    throws VCloudException
  {
    String suncServerSysURL = getReference().getHref() + "/action/syncSyslogServerSettings";

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), suncServerSysURL, null, null, 202));
  }

  public Task upgradeConfig()
    throws VCloudException
  {
    String upgradeConfigURL = getReference().getHref() + "/action/upgradeConfig";

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), upgradeConfigURL, null, null, 202));
  }

  public static Task delete(VcloudClient client, ReferenceType edgeGatewayRef)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.delete(client, edgeGatewayRef.getHref(), 202));
  }

  public Task delete()
    throws VCloudException
  {
    return new Task(getVcloudClient(), (TaskType)SdkUtil.delete(getVcloudClient(), ((GatewayType)getResource()).getHref(), 202));
  }

  public Task configureServices(GatewayFeaturesType gatewayFeaturesType)
    throws VCloudException
  {
    String configureServicesUrl = getReference().getHref() + "/action/configureServices";

    ObjectFactory objectFactory_v5_1 = new ObjectFactory();
    String requestBodyXml = JAXBUtil.marshal(objectFactory_v5_1.createEdgeGatewayServiceConfiguration(gatewayFeaturesType));
    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), configureServicesUrl, requestBodyXml, "application/vnd.vmware.admin.edgeGatewayServiceConfiguration+xml", 202));
  }
}