package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.GroupType;
import com.vmware.vcloud.api.rest.schema.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.UsersListType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Group extends VcloudEntity<GroupType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private HashMap<String, ReferenceType> userRefsByName;

  Group(VcloudClient client, GroupType groupType_v1_5)
  {
    super(client, groupType_v1_5);
    sortRefs_v1_5();
  }

  public static Group getGroupByReference(VcloudClient client, ReferenceType groupRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + groupRef.getHref());

    return new Group(client, (GroupType)getResourceByReference(client, groupRef));
  }

  public static Group getGroupById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new Group(client, (GroupType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.group+xml"));
  }

  public Group updateGroup(GroupType groupType)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    String groupUpdateUrl = getReference().getHref();
    String groupUpdateRequestXmlString = JAXBUtil.marshal(of.createGroup(groupType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + groupUpdateUrl);

    return new Group(getVcloudClient(), (GroupType)SdkUtil.put(getVcloudClient(), groupUpdateUrl, groupUpdateRequestXmlString, "application/vnd.vmware.admin.group+xml", 200));
  }

  public void delete()
    throws VCloudException
  {
    String groupUrl = getReference().getHref();
    deleteGroup(getVcloudClient(), groupUrl);
  }

  public static void delete(VcloudClient client, ReferenceType groupRef)
    throws VCloudException
  {
    deleteGroup(client, groupRef.getHref());
  }

  private static void deleteGroup(VcloudClient client, String groupUrl)
    throws VCloudException
  {
    SdkUtil.delete(client, groupUrl, 204);
  }

  private void sortRefs_v1_5()
  {
    this.userRefsByName = new HashMap();
    if (((GroupType)getResource()).getUsersList() != null)
      for (ReferenceType userRef : ((GroupType)getResource()).getUsersList().getUserReference())
      {
        this.userRefsByName.put(userRef.getName(), userRef);
      }
  }

  public ReferenceType getRoleReference()
    throws VCloudException
  {
    if (((GroupType)getResource()).getRole() != null) {
      return ((GroupType)getResource()).getRole();
    }
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public HashMap<String, ReferenceType> getUserRefsByName()
  {
    return this.userRefsByName;
  }

  public Collection<ReferenceType> getUserRefs()
  {
    return this.userRefsByName.values();
  }

  public ReferenceType getUserRefByName(String userName)
  {
    return (ReferenceType)this.userRefsByName.get(userName);
  }
}