package com.vmware.vcloud.sdk.admin;

import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.RightType;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Right extends VcloudEntity<RightType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");

  public Right(VcloudClient client, RightType rightType)
  {
    super(client, rightType);
  }

  public static Right getRightByReference(VcloudClient client, ReferenceType rightRef)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + rightRef.getHref());

    return new Right(client, (RightType)getResourceByReference(client, rightRef));
  }

  public static Right getRightById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new Right(client, (RightType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.right+xml"));
  }

  public static void delete(VcloudClient client, ReferenceType rightRef)
    throws VCloudException
  {
    SdkUtil.delete(client, rightRef.getHref(), 204);
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), ((RightType)getResource()).getHref(), 204);
  }
}