package com.vmware.vcloud.sdk.admin.extensions;

import com.vmware.vcloud.api.rest.schema.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.NetworkPoolReferencesType;
import com.vmware.vcloud.api.rest.schema.ProviderVdcStorageProfilesType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.TaskType;
import com.vmware.vcloud.api.rest.schema.VdcReferencesType;
import com.vmware.vcloud.api.rest.schema.extension.MigrateParamsType;
import com.vmware.vcloud.api.rest.schema.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.extension.ProviderVdcMergeParamsType;
import com.vmware.vcloud.api.rest.schema.extension.UpdateProviderVdcStorageProfilesParamsType;
import com.vmware.vcloud.api.rest.schema.extension.UpdateResourcePoolSetParamsType;
import com.vmware.vcloud.api.rest.schema.extension.VMWHostReferencesType;
import com.vmware.vcloud.api.rest.schema.extension.VMWProviderVdcResourcePoolSetType;
import com.vmware.vcloud.api.rest.schema.extension.VMWProviderVdcResourcePoolType;
import com.vmware.vcloud.api.rest.schema.extension.VMWProviderVdcType;
import com.vmware.vcloud.api.rest.schema.extension.VMWStorageProfileType;
import com.vmware.vcloud.api.rest.schema.extension.VMWStorageProfilesType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefType;
import com.vmware.vcloud.api.rest.schema.extension.VimObjectRefsType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.Task;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWProviderVdc extends VcloudEntity<VMWProviderVdcType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private HashMap<String, ReferenceType> externalNetworkRefsByName;
  private HashMap<String, ReferenceType> vmwNetworkPoolRefsByName;
  private HashMap<String, ReferenceType> vmwHostRefsByName;
  private ReferenceType providerVdcReference;
  private ReferenceType vdcReferences;
  private HashMap<String, VMWProviderVdcResourcePoolType> vmwProvVdcResPoolsByMoref;

  VMWProviderVdc(VcloudClient client, VMWProviderVdcType vmwProviderVdcType_v1_5)
  {
    super(client, vmwProviderVdcType_v1_5);
    sortReferences_v1_5();
  }

  public static VMWProviderVdc getVMWProviderVdcByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new VMWProviderVdc(client, (VMWProviderVdcType)getResourceByReference(client, reference));
  }

  public static VMWProviderVdc getVMWProviderVdcById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new VMWProviderVdc(client, (VMWProviderVdcType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.vmwprovidervdc+xml"));
  }

  public ReferenceType getProviderVdcReference()
    throws VCloudException
  {
    if (this.providerVdcReference != null)
      return this.providerVdcReference;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  private void sortReferences_v1_5()
  {
    this.externalNetworkRefsByName = new HashMap();
    this.vmwNetworkPoolRefsByName = new HashMap();
    this.vmwHostRefsByName = new HashMap();

    if (((VMWProviderVdcType)getResource()).getAvailableNetworks() != null) {
      for (ReferenceType externalNetworkRef : ((VMWProviderVdcType)getResource()).getAvailableNetworks().getNetwork())
      {
        this.externalNetworkRefsByName.put(externalNetworkRef.getName(), externalNetworkRef);
      }

    }

    if (((VMWProviderVdcType)getResource()).getNetworkPoolReferences() != null) {
      for (ReferenceType vmwNetworkPoolRef : ((VMWProviderVdcType)getResource()).getNetworkPoolReferences().getNetworkPoolReference())
      {
        this.vmwNetworkPoolRefsByName.put(vmwNetworkPoolRef.getName(), vmwNetworkPoolRef);
      }
    }

    if (((VMWProviderVdcType)getResource()).getHostReferences() != null) {
      for (ReferenceType vmwHostRef : ((VMWProviderVdcType)getResource()).getHostReferences().getHostReference())
      {
        this.vmwHostRefsByName.put(vmwHostRef.getName(), vmwHostRef);
      }
    }
    for (LinkType vmwProviderVdcLink : ((VMWProviderVdcType)getResource()).getLink()) {
      if ((vmwProviderVdcLink.getRel().equals("alternate")) && (vmwProviderVdcLink.getType().equals("application/vnd.vmware.admin.providervdc+xml")))
      {
        this.providerVdcReference = vmwProviderVdcLink;
      }if ((vmwProviderVdcLink.getRel().equals("down")) && (vmwProviderVdcLink.getType().equals("application/vnd.vmware.admin.vdcReferences+xml")))
      {
        this.vdcReferences = vmwProviderVdcLink;
      }
    }
  }

  public VMWProviderVdc updateVMWProviderVdc(VMWProviderVdcType vmwProviderVdcType)
    throws VCloudException
  {
    ObjectFactory ofFactory = new ObjectFactory();
    String vmwProviderVdcUpdateUrl = getReference().getHref();
    String vmwProviderVdcUpdateRequest = JAXBUtil.marshal(ofFactory.createVMWProviderVdc(vmwProviderVdcType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.PUT_URL_MSG) + " - " + vmwProviderVdcUpdateUrl);

    return new VMWProviderVdc(getVcloudClient(), (VMWProviderVdcType)SdkUtil.put(getVcloudClient(), vmwProviderVdcUpdateUrl, vmwProviderVdcUpdateRequest, "application/vnd.vmware.admin.vmwprovidervdc+xml", 200));
  }

  public Task delete()
    throws VCloudException
  {
    String vmwProviderVdcUrl = getReference().getHref();
    return deleteVMWProviderVdc(getVcloudClient(), vmwProviderVdcUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    return deleteVMWProviderVdc(client, vmwProviderVdcRef.getHref());
  }

  private static Task deleteVMWProviderVdc(VcloudClient client, String vmwProviderVdcUrl)
    throws VCloudException
  {
    return new Task(client, (TaskType)SdkUtil.delete(client, vmwProviderVdcUrl, 202));
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = getReference().getHref() + "/action/disable";

    SdkUtil.post(getVcloudClient(), disableUrl, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    String disableUrl = vmwProviderVdcRef.getHref() + "/action/disable";

    SdkUtil.post(client, disableUrl, null, null, 204);
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = getReference().getHref() + "/action/enable";

    SdkUtil.post(getVcloudClient(), enableURL, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    String enableURL = vmwProviderVdcRef.getHref() + "/action/enable";

    SdkUtil.post(client, enableURL, null, null, 204);
  }

  public HashMap<String, ReferenceType> getAdminVdcRefsByName()
    throws VCloudException
  {
    return getAdminVdcRefs(getVcloudClient(), this.vdcReferences.getHref());
  }

  private static HashMap<String, ReferenceType> getAdminVdcRefs(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + url);

    VdcReferencesType vdcReferencesType = (VdcReferencesType)SdkUtil.get(client, url, 200);

    HashMap adminVdcRefs = new HashMap();
    for (ReferenceType vdcReference : vdcReferencesType.getVdcReference()) {
      adminVdcRefs.put(vdcReference.getName(), vdcReference);
    }
    return adminVdcRefs;
  }

  public List<ReferenceType> getVMWVimServerRefs()
  {
    return ((VMWProviderVdcType)getResource()).getVimServer();
  }

  public List<VimObjectRefType> getResourcePoolVimRefs()
    throws VCloudException
  {
    if (((VMWProviderVdcType)getResource()).getResourcePoolRefs() != null)
      return ((VMWProviderVdcType)getResource()).getResourcePoolRefs().getVimObjectRef();
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG));
  }

  private static HashMap<String, VMWProviderVdcResourcePoolType> getResourcePools(VcloudClient client, ReferenceType vmwProvVdcRef)
    throws VCloudException
  {
    VMWProviderVdcResourcePoolSetType vmwProvVdcResPoolSet = (VMWProviderVdcResourcePoolSetType)SdkUtil.get(client, vmwProvVdcRef.getHref() + "/resourcePools", 200);

    HashMap resourcePools = new HashMap();
    for (VMWProviderVdcResourcePoolType vmwProvVdcResPool : vmwProvVdcResPoolSet.getVMWProviderVdcResourcePool())
    {
      resourcePools.put(vmwProvVdcResPool.getResourcePoolVimObjectRef().getMoRef(), vmwProvVdcResPool);
    }

    return resourcePools;
  }

  public HashMap<String, VMWProviderVdcResourcePoolType> getResourcePoolsByMoref()
    throws VCloudException
  {
    if (this.vmwProvVdcResPoolsByMoref == null) {
      this.vmwProvVdcResPoolsByMoref = getResourcePools(getVcloudClient(), getReference());
    }
    return this.vmwProvVdcResPoolsByMoref;
  }

  public static HashMap<String, VMWProviderVdcResourcePoolType> getResourcePoolsByMoref(VcloudClient client, ReferenceType vmwProvVdcRef)
    throws VCloudException
  {
    return getResourcePools(client, vmwProvVdcRef);
  }

  public Task updateResourcePools(List<VimObjectRefType> resourcePoolVimRefs, List<ReferenceType> resourcePoolRefs)
    throws VCloudException
  {
    UpdateResourcePoolSetParamsType resourcePoolSetParams = new UpdateResourcePoolSetParamsType();
    List addResourcePoolVimRefs = resourcePoolSetParams.getAddItem();

    addResourcePoolVimRefs.addAll(resourcePoolVimRefs);
    List deleteResourcePoolRefs = resourcePoolSetParams.getDeleteItem();

    deleteResourcePoolRefs.addAll(resourcePoolRefs);
    String updateResPoolRequestXML = JAXBUtil.marshal(new ObjectFactory().createUpdateResourcePoolSetParams(resourcePoolSetParams));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), getReference().getHref() + "/action/updateResourcePools", updateResPoolRequestXML, "application/vnd.vmware.admin.resourcePoolSetUpdateParams+xml", 202));
  }

  public HashMap<String, VimObjectRefType> getDatastoreVimRefsByMoref()
  {
    HashMap datastoreRefsByMorefName = new HashMap();
    if (((VMWProviderVdcType)getResource()).getDataStoreRefs() != null)
      for (VimObjectRefType datastoreRef : ((VMWProviderVdcType)getResource()).getDataStoreRefs().getVimObjectRef())
      {
        datastoreRefsByMorefName.put(datastoreRef.getMoRef(), datastoreRef);
      }
    return datastoreRefsByMorefName;
  }

  public void disableResourcePool(String morefValue)
    throws VCloudException
  {
    if (!getResourcePoolsByMoref().containsKey(morefValue)) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    String disableResPoolUrl = ((VMWProviderVdcResourcePoolType)getResourcePoolsByMoref().get(morefValue)).getResourcePoolRef().getHref() + "/action/disable";

    SdkUtil.post(getVcloudClient(), disableResPoolUrl, null, null, 204);
  }

  public void enableResourcePool(String morefValue)
    throws VCloudException
  {
    if (!getResourcePoolsByMoref().containsKey(morefValue)) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    String disableResPoolUrl = ((VMWProviderVdcResourcePoolType)getResourcePoolsByMoref().get(morefValue)).getResourcePoolRef().getHref() + "/action/enable";

    SdkUtil.post(getVcloudClient(), disableResPoolUrl, null, null, 204);
  }

  public Task mergeProviderVdcs(List<ReferenceType> vmwProvVdcRefs)
    throws VCloudException
  {
    ProviderVdcMergeParamsType mergeParams = new ProviderVdcMergeParamsType();
    mergeParams.getProviderVdcReference().addAll(vmwProvVdcRefs);
    String mergeProvVdcsUrl = ((VMWProviderVdcType)getResource()).getHref() + "/action/merge";

    String mergeString = JAXBUtil.marshal(new ObjectFactory().createProviderVdcMergeParams(mergeParams));

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), mergeProvVdcsUrl, mergeString, "application/vnd.vmware.admin.providerVdcMergeParams+xml", 202));
  }

  public HashMap<String, ReferenceType> getExternalNetworkRefsByName()
  {
    return this.externalNetworkRefsByName;
  }

  public ReferenceType getExternalNetworkRefByName(String name)
  {
    return (ReferenceType)this.externalNetworkRefsByName.get(name);
  }

  public Collection<ReferenceType> getExternalNetworkRefs()
  {
    return this.externalNetworkRefsByName.values();
  }

  public HashMap<String, ReferenceType> getVMWNetworkPoolRefsByName()
  {
    return this.vmwNetworkPoolRefsByName;
  }

  public ReferenceType getVMWNetworkPoolRefByName(String name)
  {
    return (ReferenceType)this.vmwNetworkPoolRefsByName.get(name);
  }

  public Collection<ReferenceType> getVMWNetworkPoolRefs()
  {
    return this.vmwNetworkPoolRefsByName.values();
  }

  public HashMap<String, ReferenceType> getVMWHostRefsByName()
  {
    return this.vmwHostRefsByName;
  }

  public ReferenceType getVMWHostRefByName(String name)
  {
    return (ReferenceType)this.vmwHostRefsByName.get(name);
  }

  public Collection<ReferenceType> getVMWHostRefs()
  {
    return this.vmwHostRefsByName.values();
  }

  public List<ReferenceType> getVMWProviderVdcStorageProfileRefs()
    throws VCloudException
  {
    if (((VMWProviderVdcType)getResource()).getStorageProfiles() == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.DATA_NOT_FOUND));
    }

    return ((VMWProviderVdcType)getResource()).getStorageProfiles().getProviderVdcStorageProfile();
  }

  public List<VMWStorageProfileType> getAvailableVMWProviderVdcStorageProfiles()
    throws VCloudException
  {
    return ((VMWStorageProfilesType)SdkUtil.get(getVcloudClient(), getReference().getHref() + "/availableStorageProfiles", 200)).getVMWStorageProfile();
  }

  public Task updateVMWProviderVdcStorageProfiles(List<String> addStorageProfiles, List<ReferenceType> removeStorageProfiles)
    throws VCloudException
  {
    ObjectFactory of = new ObjectFactory();
    UpdateProviderVdcStorageProfilesParamsType updateStorageProfilesType = new UpdateProviderVdcStorageProfilesParamsType();
    updateStorageProfilesType.getAddStorageProfile().addAll(addStorageProfiles);

    updateStorageProfilesType.getRemoveStorageProfile().addAll(removeStorageProfiles);

    String updateVdcStprageProfilesUrl = getReference().getHref() + "/storageProfiles";

    String adminVdcUpdateRequestXmlString = JAXBUtil.marshal(of.createUpdateProviderVdcStorageProfiles(updateStorageProfilesType));

    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.POST_URL_MSG) + " - " + updateVdcStprageProfilesUrl);

    return new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), updateVdcStprageProfilesUrl, adminVdcUpdateRequestXmlString, "application/vnd.vmware.admin.updateProviderVdcStorageProfiles+xml", 202));
  }

  public Task migrateVms(String sourceResourcePoolMoref, List<ReferenceType> vmRefs, VimObjectRefType resourcePoolVimRef)
    throws VCloudException
  {
    MigrateParamsType migrateParams = new MigrateParamsType();
    migrateParams.setResourcePoolRef(resourcePoolVimRef);
    migrateParams.getVmRef().addAll(vmRefs);
    String requestXmlString = JAXBUtil.marshal(new ObjectFactory().createMigrateParams(migrateParams));

    if (!getResourcePoolsByMoref().containsKey(sourceResourcePoolMoref))
    {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.VIM_OBJECT_REF_NOT_FOUND_MSG) + " - " + sourceResourcePoolMoref);
    }

    String migrateVmsUrl = ((VMWProviderVdcResourcePoolType)getResourcePoolsByMoref().get(sourceResourcePoolMoref)).getResourcePoolRef().getHref() + "/action/migrateVms";

    Task task = new Task(getVcloudClient(), (TaskType)SdkUtil.post(getVcloudClient(), migrateVmsUrl, requestXmlString, "application/vnd.vmware.admin.migrateVmParams+xml", 202));

    return task;
  }

  public ReferenceResult getVMs(String resourcePoolMoref)
    throws VCloudException
  {
    if (getResourcePoolsByMoref().get(resourcePoolMoref) == null) {
      throw new VCloudException(SdkUtil.getI18nString(SdkMessage.MOREF_NOT_FOUND));
    }
    String query = ((VMWProviderVdcResourcePoolType)getResourcePoolsByMoref().get(resourcePoolMoref)).getResourcePoolRef().getHref() + "/vmList" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }
}