package com.vmware.vcloud.sdk.admin.extensions.service;

import com.vmware.vcloud.api.rest.schema.LinkType;
import com.vmware.vcloud.api.rest.schema.ReferenceType;
import com.vmware.vcloud.api.rest.schema.extension.AclRuleType;
import com.vmware.vcloud.api.rest.schema.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema.extension.ResourceClassActionType;
import com.vmware.vcloud.sdk.JAXBUtil;
import com.vmware.vcloud.sdk.QueryService;
import com.vmware.vcloud.sdk.ReferenceResult;
import com.vmware.vcloud.sdk.SdkUtil;
import com.vmware.vcloud.sdk.VCloudException;
import com.vmware.vcloud.sdk.VcloudClient;
import com.vmware.vcloud.sdk.VcloudEntity;
import com.vmware.vcloud.sdk.constants.SdkMessage;
import com.vmware.vcloud.sdk.constants.query.FormatType;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AdminResourceClassAction extends VcloudEntity<ResourceClassActionType>
{
  private static Logger logger = Logger.getLogger("com.vmware.vcloud.sdk");
  private ReferenceType adminResourceClassRef;

  public AdminResourceClassAction(VcloudClient client, ResourceClassActionType resourceClassActionParams)
  {
    super(client, resourceClassActionParams);
    sortAdminResourceClassActionLinkRefs();
  }

  private void sortAdminResourceClassActionLinkRefs()
  {
    if (getResource() != null)
      for (LinkType link : ((ResourceClassActionType)getResource()).getLink())
        if ((link.getRel().equals("up")) && (link.getType().equals("application/vnd.vmware.admin.resourceClass+xml")))
        {
          this.adminResourceClassRef = link;
        }
  }

  public static AdminResourceClassAction getAdminResourceClassActionByReference(VcloudClient client, ReferenceType reference)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_URL_MSG) + " - " + reference.getHref());

    return new AdminResourceClassAction(client, (ResourceClassActionType)getResourceByReference(client, reference));
  }

  public static AdminResourceClassAction getAdminResourceClassActionById(VcloudClient client, String vCloudId)
    throws VCloudException
  {
    logger.log(Level.INFO, SdkUtil.getI18nString(SdkMessage.GET_VCLOUD_ID_MSG) + " - " + vCloudId);

    return new AdminResourceClassAction(client, (ResourceClassActionType)getEntityById(client, vCloudId, "application/vnd.vmware.admin.resourceClassAction+xml"));
  }

  public void delete()
    throws VCloudException
  {
    SdkUtil.delete(getVcloudClient(), getReference().getHref(), 204);
  }

  public ReferenceType getAdminResourceClassReference()
    throws VCloudException
  {
    if (this.adminResourceClassRef != null)
      return this.adminResourceClassRef;
    throw new VCloudException(SdkUtil.getI18nString(SdkMessage.REFERENCE_NOT_FOUND_MSG));
  }

  public AdminAclRule registerAdminAclRule(AclRuleType aclRuleParams)
    throws VCloudException
  {
    String requestXML = JAXBUtil.marshal(new ObjectFactory().createAclRule(aclRuleParams));

    return new AdminAclRule(getVcloudClient(), (AclRuleType)SdkUtil.post(getVcloudClient(), ((ResourceClassActionType)getResource()).getHref() + "/aclrules", requestXML, "application/vnd.vmware.admin.aclRule+xml", 201));
  }

  public ReferenceResult getAdminAclRuleRefs()
    throws VCloudException
  {
    String query = getReference().getHref() + "/aclrules" + "?" + getVcloudClient().getQueryService().buildQuery(FormatType.REFERENCE_VIEW);

    return (ReferenceResult)getVcloudClient().getQueryService().executeQuery(query);
  }
}