package com.vmware.vcloud.sdk_v1_0;

import com.vmware.vcloud.api.rest.schema_v1_0.EntityType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.TasksInProgressType;
import java.util.ArrayList;
import java.util.List;

public abstract class VcloudEntity<T extends EntityType> extends VcloudResource<T>
{
  private ReferenceType referenceType;
  private List<Task> tasks;

  protected VcloudEntity(VcloudClient client, T entityType)
  {
    super(client, entityType);
    setTasks(client, entityType);
    setReference(entityType);
  }

  public List<Task> getTasks()
  {
    return this.tasks;
  }

  public ReferenceType getReference()
  {
    return this.referenceType;
  }

  private void setTasks(VcloudClient client, EntityType entityType)
  {
    if (this.tasks == null) {
      this.tasks = new ArrayList();
      if (entityType.getTasks() != null)
        for (TaskType taskType : entityType.getTasks().getTask())
          this.tasks.add(new Task(client, taskType));
    }
  }

  private void setReference(EntityType entity)
  {
    this.referenceType = new ReferenceType();
    this.referenceType.setHref(entity.getHref());
    this.referenceType.setName(entity.getName());
    this.referenceType.setType(entity.getType());
  }
}