package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWExternalNetworkType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWExternalNetwork extends VcloudEntity<VMWExternalNetworkType>
{
  private static Logger logger = Logger.getLogger(VMWExternalNetwork.class.getSimpleName());
  private ReferenceType externalNetworkReference;

  VMWExternalNetwork(VcloudClient client, VMWExternalNetworkType vmwExternalNetworkType)
  {
    super(client, vmwExternalNetworkType);
    sortReferences();
  }

  public static VMWExternalNetwork getVMWExternalNetworkByReference(VcloudClient client, ReferenceType referenceType)
    throws VCloudException
  {
    return getVMWExternalNetwork(client, referenceType.getHref());
  }

  private static VMWExternalNetwork getVMWExternalNetwork(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VMWExternalNetwork  " + url);
    Response response = RestUtil.get(client, url);
    VMWExternalNetworkType vmwExternalNetworkType = null;

    if (response.isExpected(200)) {
      vmwExternalNetworkType = (VMWExternalNetworkType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    VMWExternalNetwork vmwExternalNetwork = new VMWExternalNetwork(client, vmwExternalNetworkType);

    return vmwExternalNetwork;
  }

  public ReferenceType getExternalNetworkReference()
    throws VCloudException
  {
    if (this.externalNetworkReference != null)
      return this.externalNetworkReference;
    throw new VCloudException("Reference Not Found");
  }

  private void sortReferences()
  {
    for (LinkType vmwProviderVdcLink : ((VMWExternalNetworkType)getResource()).getLink())
      if ((vmwProviderVdcLink.getRel().equals(RelationType.ALTERNATE)) && (vmwProviderVdcLink.getType().equals("application/vnd.vmware.admin.network+xml")))
      {
        this.externalNetworkReference = vmwProviderVdcLink;
      }
  }

  public VimObjectRefType getPortGroupRef()
  {
    return ((VMWExternalNetworkType)getResource()).getVimPortGroupRef();
  }

  public VMWExternalNetwork updateVMWExternalNetwork(VMWExternalNetworkType vmwExternalNetworkType)
    throws VCloudException
  {
    ObjectFactory ofFactory = new ObjectFactory();
    String vmwExternalNetworkUpdateUrl = ((VMWExternalNetworkType)getResource()).getHref();
    String vmwExternalNetworkUpdateRequest = JAXBUtil.marshal(ofFactory.createVMWExternalNetwork(vmwExternalNetworkType));

    logger.log(Level.INFO, "Update VMWExternalNetwork URL : " + vmwExternalNetworkUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), vmwExternalNetworkUpdateUrl, vmwExternalNetworkUpdateRequest, "application/vnd.vmware.admin.vmwexternalnet+xml", null);

    VMWExternalNetworkType updatedVMWExternalNetworkType = null;

    if (response.isExpected(200)) {
      updatedVMWExternalNetworkType = (VMWExternalNetworkType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    return new VMWExternalNetwork(getVcloudClient(), updatedVMWExternalNetworkType);
  }

  public Task delete()
    throws VCloudException
  {
    String vmwExternalNetworkUrl = ((VMWExternalNetworkType)getResource()).getHref();
    return deleteVMWExternalNetwork(getVcloudClient(), vmwExternalNetworkUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType vmwExternalNetworkRef)
    throws VCloudException
  {
    return deleteVMWExternalNetwork(client, vmwExternalNetworkRef.getHref());
  }

  private static Task deleteVMWExternalNetwork(VcloudClient client, String vmwExternalNetworkUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, vmwExternalNetworkUrl);
    TaskType taskType = null;
    if (response.isExpected(202))
      taskType = (TaskType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(client, taskType);

    return task;
  }
}