package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.AvailableNetworksType;
import com.vmware.vcloud.api.rest.schema_v1_0.LinkType;
import com.vmware.vcloud.api.rest.schema_v1_0.NetworkPoolReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.RelationType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.VdcReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWHostReferencesType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VMWProviderVdcType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimObjectRefsType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.Collection;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWProviderVdc extends VcloudEntity<VMWProviderVdcType>
{
  private static Logger logger = Logger.getLogger(VMWProviderVdc.class.getSimpleName());
  private HashMap<String, ReferenceType> externalNetworkRefsByName;
  private HashMap<String, ReferenceType> vmwNetworkPoolRefsByName;
  private HashMap<String, ReferenceType> vmwHostRefsByName;
  private HashMap<String, VimObjectRefType> datastoreRefsByMorefName;
  private ReferenceType providerVdcReference;
  private ReferenceType vdcReferences;

  VMWProviderVdc(VcloudClient client, VMWProviderVdcType vmwProviderVdcType)
  {
    super(client, vmwProviderVdcType);
    sortReferences();
  }

  public static VMWProviderVdc getVMWProviderVdcByReference(VcloudClient client, ReferenceType referenceType)
    throws VCloudException
  {
    return getVMWProviderVdc(client, referenceType.getHref());
  }

  private static VMWProviderVdc getVMWProviderVdc(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VMWProvider Vdc  " + url);
    Response response = RestUtil.get(client, url);
    VMWProviderVdcType vmwProviderVdcType = null;

    if (response.isExpected(200))
      vmwProviderVdcType = (VMWProviderVdcType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    VMWProviderVdc vmwProviderVdc = new VMWProviderVdc(client, vmwProviderVdcType);

    return vmwProviderVdc;
  }

  public ReferenceType getProviderVdcReference()
    throws VCloudException
  {
    if (this.providerVdcReference != null)
      return this.providerVdcReference;
    throw new VCloudException("Reference Not Found");
  }

  private void sortReferences()
  {
    this.externalNetworkRefsByName = new HashMap();
    this.vmwNetworkPoolRefsByName = new HashMap();
    this.vmwHostRefsByName = new HashMap();
    this.datastoreRefsByMorefName = new HashMap();

    if (((VMWProviderVdcType)getResource()).getAvailableNetworks() != null) {
      for (ReferenceType externalNetworkRef : ((VMWProviderVdcType)getResource()).getAvailableNetworks().getNetwork())
      {
        this.externalNetworkRefsByName.put(externalNetworkRef.getName(), externalNetworkRef);
      }

    }

    if (((VMWProviderVdcType)getResource()).getNetworkPoolReferences() != null) {
      for (ReferenceType vmwNetworkPoolRef : ((VMWProviderVdcType)getResource()).getNetworkPoolReferences().getNetworkPoolReference())
      {
        this.vmwNetworkPoolRefsByName.put(vmwNetworkPoolRef.getName(), vmwNetworkPoolRef);
      }
    }

    if (((VMWProviderVdcType)getResource()).getHostReferences() != null) {
      for (ReferenceType vmwHostRef : ((VMWProviderVdcType)getResource()).getHostReferences().getHostReference())
      {
        this.vmwHostRefsByName.put(vmwHostRef.getName(), vmwHostRef);
      }
    }
    if (((VMWProviderVdcType)getResource()).getDataStoreRefs() != null) {
      for (VimObjectRefType datastoreRef : ((VMWProviderVdcType)getResource()).getDataStoreRefs().getVimObjectRef())
      {
        this.datastoreRefsByMorefName.put(datastoreRef.getMoRef(), datastoreRef);
      }
    }
    for (LinkType vmwProviderVdcLink : ((VMWProviderVdcType)getResource()).getLink()) {
      if ((vmwProviderVdcLink.getRel().equals(RelationType.ALTERNATE)) && (vmwProviderVdcLink.getType().equals("application/vnd.vmware.admin.providervdc+xml")))
      {
        this.providerVdcReference = vmwProviderVdcLink;
      }if (vmwProviderVdcLink.getRel().equals(RelationType.DOWN))
        this.vdcReferences = vmwProviderVdcLink;
    }
  }

  public VMWProviderVdc updateVMWProviderVdc(VMWProviderVdcType vmwProviderVdcType)
    throws VCloudException
  {
    ObjectFactory ofFactory = new ObjectFactory();
    String vmwProviderVdcUpdateUrl = ((VMWProviderVdcType)getResource()).getHref();
    String vmwProviderVdcUpdateRequest = JAXBUtil.marshal(ofFactory.createVMWProviderVdc(vmwProviderVdcType));

    logger.log(Level.INFO, "Update VMWProviderVdc URL : " + vmwProviderVdcUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), vmwProviderVdcUpdateUrl, vmwProviderVdcUpdateRequest, "application/vnd.vmware.admin.vmwprovidervdc+xml", null);

    VMWProviderVdcType updatedVMWProviderVdcType = null;

    if (response.isExpected(200)) {
      updatedVMWProviderVdcType = (VMWProviderVdcType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    return new VMWProviderVdc(getVcloudClient(), updatedVMWProviderVdcType);
  }

  public Task delete()
    throws VCloudException
  {
    String vmwProviderVdcUrl = ((VMWProviderVdcType)getResource()).getHref();
    return deleteVMWProviderVdc(getVcloudClient(), vmwProviderVdcUrl);
  }

  public static Task delete(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    return deleteVMWProviderVdc(client, vmwProviderVdcRef.getHref());
  }

  private static Task deleteVMWProviderVdc(VcloudClient client, String vmwProviderVdcUrl)
    throws VCloudException
  {
    Response response = RestUtil.delete(client, vmwProviderVdcUrl);
    TaskType taskType = null;
    if (response.isExpected(202))
      taskType = (TaskType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    Task task = new Task(client, taskType);

    return task;
  }

  public void disable()
    throws VCloudException
  {
    String disableUrl = ((VMWProviderVdcType)getResource()).getHref() + "/action/disable";

    executeVMWProviderVdcAction(getVcloudClient(), disableUrl, null, null, null, 204);
  }

  public static void disable(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    String disableUrl = vmwProviderVdcRef.getHref() + "/action/disable";

    executeVMWProviderVdcAction(client, disableUrl, null, null, null, 204);
  }

  public void enable()
    throws VCloudException
  {
    String enableURL = ((VMWProviderVdcType)getResource()).getHref() + "/action/enable";

    executeVMWProviderVdcAction(getVcloudClient(), enableURL, null, null, null, 204);
  }

  public static void enable(VcloudClient client, ReferenceType vmwProviderVdcRef)
    throws VCloudException
  {
    String enableURL = vmwProviderVdcRef.getHref() + "/action/enable";

    executeVMWProviderVdcAction(client, enableURL, null, null, null, 204);
  }

  private static Task executeVMWProviderVdcAction(VcloudClient client, String vmwProviderVdcActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    Response response = RestUtil.post(client, vmwProviderVdcActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }

  public HashMap<String, ReferenceType> getAdminVdcRefsByName()
    throws VCloudException
  {
    return getAdminVdcRefs(getVcloudClient(), this.vdcReferences.getHref());
  }

  private static HashMap<String, ReferenceType> getAdminVdcRefs(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET Admin Vdc References " + url);
    Response response = RestUtil.get(client, url);
    VdcReferencesType vdcReferencesType = null;

    if (response.isExpected(200))
      vdcReferencesType = (VdcReferencesType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    HashMap adminVdcRefs = new HashMap();
    for (ReferenceType vdcReference : vdcReferencesType.getVdcReference())
      adminVdcRefs.put(vdcReference.getName(), vdcReference);
    return adminVdcRefs;
  }

  public ReferenceType getVMWVimServerRef()
  {
    return ((VMWProviderVdcType)getResource()).getVimServer();
  }

  public VimObjectRefType getResourcePoolRef()
  {
    return ((VMWProviderVdcType)getResource()).getResourcePoolRef();
  }

  public HashMap<String, VimObjectRefType> getDatastoreRefsByMorefName()
  {
    return this.datastoreRefsByMorefName;
  }

  public VimObjectRefType getDatastoreRefByMorefName(String morefName)
  {
    return (VimObjectRefType)this.datastoreRefsByMorefName.get(morefName);
  }

  public Collection<VimObjectRefType> getDatastoreRefs()
  {
    return this.datastoreRefsByMorefName.values();
  }

  public HashMap<String, ReferenceType> getExternalNetworkRefsByName()
  {
    return this.externalNetworkRefsByName;
  }

  public ReferenceType getExternalNetworkRefByName(String name)
  {
    return (ReferenceType)this.externalNetworkRefsByName.get(name);
  }

  public Collection<ReferenceType> getExternalNetworkRefs()
  {
    return this.externalNetworkRefsByName.values();
  }

  public HashMap<String, ReferenceType> getVMWNetworkPoolRefsByName()
  {
    return this.vmwNetworkPoolRefsByName;
  }

  public ReferenceType getVMWNetworkPoolRefByName(String name)
  {
    return (ReferenceType)this.vmwNetworkPoolRefsByName.get(name);
  }

  public Collection<ReferenceType> getVMWNetworkPoolRefs()
  {
    return this.vmwNetworkPoolRefsByName.values();
  }

  public HashMap<String, ReferenceType> getVMWHostRefsByName()
  {
    return this.vmwHostRefsByName;
  }

  public ReferenceType getVMWHostRefByName(String name)
  {
    return (ReferenceType)this.vmwHostRefsByName.get(name);
  }

  public Collection<ReferenceType> getVMWHostRefs()
  {
    return this.vmwHostRefsByName.values();
  }
}