package com.vmware.vcloud.sdk_v1_0.admin.extensions;

import com.vmware.vcloud.api.rest.schema_v1_0.ReferenceType;
import com.vmware.vcloud.api.rest.schema_v1_0.TaskType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppTemplateType;
import com.vmware.vcloud.api.rest.schema_v1_0.VAppType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ImportVmAsVAppParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ImportVmAsVAppTemplateParamsType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ObjectFactory;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ResourcePoolListType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.ResourcePoolType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VimServerType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VmObjectRefType;
import com.vmware.vcloud.api.rest.schema_v1_0.extension.VmObjectRefsListType;
import com.vmware.vcloud.sdk_v1_0.JAXBUtil;
import com.vmware.vcloud.sdk_v1_0.Response;
import com.vmware.vcloud.sdk_v1_0.RestUtil;
import com.vmware.vcloud.sdk_v1_0.Task;
import com.vmware.vcloud.sdk_v1_0.VCloudException;
import com.vmware.vcloud.sdk_v1_0.Vapp;
import com.vmware.vcloud.sdk_v1_0.VappTemplate;
import com.vmware.vcloud.sdk_v1_0.VcloudClient;
import com.vmware.vcloud.sdk_v1_0.VcloudEntity;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class VMWVimServer extends VcloudEntity<VimServerType>
{
  private static Logger logger = Logger.getLogger(VMWVimServer.class.getSimpleName());

  VMWVimServer(VcloudClient client, VimServerType vimServerType)
  {
    super(client, vimServerType);
  }

  public static VMWVimServer getVMWVimServerByReference(VcloudClient client, ReferenceType vmwVimServerRef)
    throws VCloudException
  {
    return getVMWVimServer(client, vmwVimServerRef.getHref());
  }

  private static VMWVimServer getVMWVimServer(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VMWVimServer " + url);
    Response response = RestUtil.get(client, url);
    VimServerType vimServerType = null;

    if (response.isExpected(200))
      vimServerType = (VimServerType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new VMWVimServer(client, vimServerType);
  }

  public Task updateVMWVimServer(VimServerType vimServerType)
    throws VCloudException
  {
    ObjectFactory extensionFactory = new ObjectFactory();
    String vmwVimServerUpdateUrl = ((VimServerType)getResource()).getHref();
    String vmwVimServerUpdateRequestXmlString = JAXBUtil.marshal(extensionFactory.createVimServer(vimServerType));

    logger.log(Level.INFO, "Update Vim Server URL : " + vmwVimServerUpdateUrl);

    Response response = RestUtil.put(getVcloudClient(), vmwVimServerUpdateUrl, vmwVimServerUpdateRequestXmlString, "application/vnd.vmware.admin.vmwvirtualcenter+xml", null);

    TaskType taskType = null;

    if (response.isExpected(202))
      taskType = (TaskType)response.getResource();
    else {
      response.handleUnExpectedResponse();
    }
    return new Task(getVcloudClient(), taskType);
  }

  public void forceVMWVimServerReconnect()
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/action/forcevimserverreconnect";

    executeHostAction(getVcloudClient(), url, null, null, null, 204);
  }

  public static void forceVMWVimServerReconnect(VcloudClient client, ReferenceType vmwVimServerRef)
    throws VCloudException
  {
    executeHostAction(client, vmwVimServerRef.getHref() + "/action/forcevimserverreconnect", null, null, null, 204);
  }

  public Task unregisterVMWVimServer()
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/action/unregister";

    return executeHostAction(getVcloudClient(), url, null, null, null, 202);
  }

  public static Task unregisterVMWVimServer(VcloudClient client, ReferenceType vmwVimServerRef)
    throws VCloudException
  {
    return executeHostAction(client, vmwVimServerRef.getHref() + "/action/unregister", null, null, null, 202);
  }

  private static Task executeHostAction(VcloudClient client, String vimServerActionUrl, String content, String contentType, String charset, int statusCode)
    throws VCloudException
  {
    logger.log(Level.INFO, "POST: VimServer Action URL " + vimServerActionUrl);

    Response response = RestUtil.post(client, vimServerActionUrl, content, contentType, charset);

    boolean expectedStatus = response.isExpected(statusCode);
    if (statusCode == 204)
      if (!expectedStatus)
        response.handleUnExpectedResponse();
      else
        return null;
    if (!expectedStatus)
      response.handleUnExpectedResponse();
    TaskType taskType = (TaskType)response.getResource();
    Task task = new Task(client, taskType);

    return task;
  }

  public static HashMap<String, ResourcePoolType> getResourcePools(VcloudClient client, ReferenceType vimServerRef)
    throws VCloudException
  {
    String url = vimServerRef.getHref() + "/resourcePoolList";

    return getResourcePools(client, url);
  }

  public HashMap<String, ResourcePoolType> getResourcePools()
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/resourcePoolList";

    return getResourcePools(getVcloudClient(), url);
  }

  private static HashMap<String, ResourcePoolType> getResourcePools(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET VimServer ResourcePool list " + url);
    Response response = RestUtil.get(client, url);
    ResourcePoolListType resourcePoolListType = null;

    if (response.isExpected(200)) {
      resourcePoolListType = (ResourcePoolListType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    HashMap resourcePoolsByName = new HashMap();
    for (ResourcePoolType resourcePoolType : resourcePoolListType.getResourcePool())
    {
      resourcePoolsByName.put(resourcePoolType.getName(), resourcePoolType);
    }
    return resourcePoolsByName;
  }

  public HashMap<String, VmObjectRefType> getVms()
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/vmsList";

    return getVms(getVcloudClient(), url);
  }

  public HashMap<String, VmObjectRefType> getVms(int page, int pageSize)
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/vmsList" + "?page=" + page + "&pageSize=" + pageSize;

    return getVms(getVcloudClient(), url);
  }

  public HashMap<String, VmObjectRefType> getVms(int page)
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/vmsList" + "?page=" + page;

    return getVms(getVcloudClient(), url);
  }

  private static HashMap<String, VmObjectRefType> getVms(VcloudClient client, String url)
    throws VCloudException
  {
    logger.log(Level.INFO, "GET: VimServer VM's List " + url);
    Response response = RestUtil.get(client, url);
    VmObjectRefsListType vmObjectRefsListType = null;

    if (response.isExpected(200)) {
      vmObjectRefsListType = (VmObjectRefsListType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }
    HashMap vmObjectRefByName = new HashMap();
    if (vmObjectRefsListType != null) {
      for (VmObjectRefType vmObjectRef : vmObjectRefsListType.getVmObjectRef())
      {
        vmObjectRefByName.put(vmObjectRef.getName(), vmObjectRef);
      }
    }
    return vmObjectRefByName;
  }

  public static HashMap<String, VmObjectRefType> getVms(VcloudClient client, ReferenceType vimServerRef)
    throws VCloudException
  {
    return getVms(client, vimServerRef.getHref() + "/vmsList");
  }

  public static HashMap<String, VmObjectRefType> getVms(VcloudClient client, ReferenceType vimServerRef, int page)
    throws VCloudException
  {
    return getVms(client, vimServerRef.getHref() + "/vmsList" + "?page=" + page);
  }

  public static HashMap<String, VmObjectRefType> getVms(VcloudClient client, ReferenceType vimServerRef, int page, int pageSize)
    throws VCloudException
  {
    return getVms(client, vimServerRef.getHref() + "/vmsList" + "?page=" + page + "&pageSize=" + pageSize);
  }

  public Vapp importVmAsVApp(ImportVmAsVAppParamsType importVmAsVAppParamsType)
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/importVmAsVApp";

    logger.log(Level.INFO, "POST: VimServer Import VM as VApp " + url);
    ObjectFactory extensionFactory = new ObjectFactory();
    String requestString = JAXBUtil.marshal(extensionFactory.createImportVmAsVAppParams(importVmAsVAppParamsType));

    Response response = RestUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.admin.importVmAsVAppParams+xml", null);

    VAppType vAppType = null;

    if (response.isExpected(201)) {
      vAppType = (VAppType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    return new Vapp(getVcloudClient(), vAppType);
  }

  public VappTemplate importVmAsVAppTemplate(ImportVmAsVAppTemplateParamsType importVmAsVAppTemplateParamsType)
    throws VCloudException
  {
    String url = ((VimServerType)getResource()).getHref() + "/importVmAsVAppTemplate";

    logger.log(Level.INFO, "POST: VimServer Import VM as VAppTemplate " + url);

    ObjectFactory extensionFactory = new ObjectFactory();
    String requestString = JAXBUtil.marshal(extensionFactory.createImportVmAsVAppTemplateParams(importVmAsVAppTemplateParamsType));

    Response response = RestUtil.post(getVcloudClient(), url, requestString, "application/vnd.vmware.admin.importVmAsVAppTemplateParams+xml", null);

    VAppTemplateType vAppTemplateType = null;

    if (response.isExpected(201)) {
      vAppTemplateType = (VAppTemplateType)response.getResource();
    }
    else {
      response.handleUnExpectedResponse();
    }

    return new VappTemplate(getVcloudClient(), vAppTemplateType);
  }
}