/*      */ package org.codehaus.stax2.ri.dom;
/*      */ 
/*      */ import java.io.IOException;
/*      */ import java.io.Writer;
/*      */ import java.math.BigDecimal;
/*      */ import java.math.BigInteger;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collections;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.NoSuchElementException;
/*      */ import javax.xml.namespace.NamespaceContext;
/*      */ import javax.xml.namespace.QName;
/*      */ import javax.xml.stream.Location;
/*      */ import javax.xml.stream.XMLStreamConstants;
/*      */ import javax.xml.stream.XMLStreamException;
/*      */ import javax.xml.transform.dom.DOMSource;
/*      */ import org.codehaus.stax2.AttributeInfo;
/*      */ import org.codehaus.stax2.DTDInfo;
/*      */ import org.codehaus.stax2.LocationInfo;
/*      */ import org.codehaus.stax2.XMLStreamLocation2;
/*      */ import org.codehaus.stax2.XMLStreamReader2;
/*      */ import org.codehaus.stax2.ri.EmptyIterator;
/*      */ import org.codehaus.stax2.ri.EmptyNamespaceContext;
/*      */ import org.codehaus.stax2.ri.SingletonIterator;
/*      */ import org.codehaus.stax2.ri.Stax2Util;
/*      */ import org.codehaus.stax2.ri.Stax2Util.ByteAggregator;
/*      */ import org.codehaus.stax2.ri.Stax2Util.TextBuffer;
/*      */ import org.codehaus.stax2.ri.typed.StringBase64Decoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.BaseArrayDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.BooleanDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.DecimalDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.DoubleArrayDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.DoubleDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.FloatArrayDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.FloatDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.IntArrayDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.IntDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.IntegerDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.LongArrayDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.LongDecoder;
/*      */ import org.codehaus.stax2.ri.typed.ValueDecoderFactory.QNameDecoder;
/*      */ import org.codehaus.stax2.typed.Base64Variant;
/*      */ import org.codehaus.stax2.typed.Base64Variants;
/*      */ import org.codehaus.stax2.typed.TypedArrayDecoder;
/*      */ import org.codehaus.stax2.typed.TypedValueDecoder;
/*      */ import org.codehaus.stax2.typed.TypedXMLStreamException;
/*      */ import org.codehaus.stax2.validation.DTDValidationSchema;
/*      */ import org.codehaus.stax2.validation.ValidationProblemHandler;
/*      */ import org.codehaus.stax2.validation.XMLValidationSchema;
/*      */ import org.codehaus.stax2.validation.XMLValidator;
/*      */ import org.w3c.dom.Attr;
/*      */ import org.w3c.dom.DocumentType;
/*      */ import org.w3c.dom.Element;
/*      */ import org.w3c.dom.NamedNodeMap;
/*      */ import org.w3c.dom.Node;
/*      */ 
/*      */ public abstract class DOMWrappingReader
/*      */   implements XMLStreamReader2, AttributeInfo, DTDInfo, LocationInfo, NamespaceContext, XMLStreamConstants
/*      */ {
/*      */   protected static final int INT_SPACE = 32;
/*      */   private static final int MASK_GET_TEXT = 6768;
/*      */   private static final int MASK_GET_TEXT_XXX = 4208;
/*      */   private static final int MASK_GET_ELEMENT_TEXT = 4688;
/*      */   protected static final int MASK_TYPED_ACCESS_BINARY = 4178;
/*      */   protected static final int ERR_STATE_NOT_START_ELEM = 1;
/*      */   protected static final int ERR_STATE_NOT_ELEM = 2;
/*      */   protected static final int ERR_STATE_NOT_PI = 3;
/*      */   protected static final int ERR_STATE_NOT_TEXTUAL = 4;
/*      */   protected static final int ERR_STATE_NOT_TEXTUAL_XXX = 5;
/*      */   protected static final int ERR_STATE_NOT_TEXTUAL_OR_ELEM = 6;
/*      */   protected static final int ERR_STATE_NO_LOCALNAME = 7;
/*      */   protected final String _systemId;
/*      */   protected final Node _rootNode;
/*      */   protected final boolean _cfgNsAware;
/*      */   protected final boolean _coalescing;
/*  157 */   protected boolean _cfgInternNames = false;
/*      */ 
/*  163 */   protected boolean _cfgInternNsURIs = false;
/*      */ 
/*  167 */   protected int _currEvent = 7;
/*      */   protected Node _currNode;
/*  175 */   protected int _depth = 0;
/*      */   protected String _coalescedText;
/*  188 */   protected Stax2Util.TextBuffer _textBuffer = new Stax2Util.TextBuffer();
/*      */ 
/*  204 */   protected List _attrList = null;
/*      */ 
/*  213 */   protected List _nsDeclList = null;
/*      */   protected ValueDecoderFactory _decoderFactory;
/*  224 */   protected StringBase64Decoder _base64Decoder = null;
/*      */ 
/*      */   protected DOMWrappingReader(DOMSource paramDOMSource, boolean paramBoolean1, boolean paramBoolean2)
/*      */     throws XMLStreamException
/*      */   {
/*  244 */     Node localNode = paramDOMSource.getNode();
/*  245 */     if (localNode == null) {
/*  246 */       throw new IllegalArgumentException("Can not pass null Node for constructing a DOM-based XMLStreamReader");
/*      */     }
/*  248 */     this._cfgNsAware = paramBoolean1;
/*  249 */     this._coalescing = paramBoolean2;
/*  250 */     this._systemId = paramDOMSource.getSystemId();
/*      */ 
/*  255 */     switch (localNode.getNodeType())
/*      */     {
/*      */     case 1:
/*      */     case 9:
/*      */     case 11:
/*  266 */       break;
/*      */     default:
/*  269 */       throw new XMLStreamException("Can not create an XMLStreamReader for a DOM node of type " + localNode.getClass());
/*      */     }
/*  271 */     this._rootNode = (this._currNode = localNode);
/*      */   }
/*      */   protected void setInternNames(boolean paramBoolean) {
/*  274 */     this._cfgInternNames = paramBoolean; } 
/*  275 */   protected void setInternNsURIs(boolean paramBoolean) { this._cfgInternNsURIs = paramBoolean; }
/*      */ 
/*      */ 
/*      */   protected abstract void throwStreamException(String paramString, Location paramLocation)
/*      */     throws XMLStreamException;
/*      */ 
/*      */   public String getCharacterEncodingScheme()
/*      */   {
/*  300 */     return null;
/*      */   }
/*      */ 
/*      */   public String getEncoding()
/*      */   {
/*  313 */     return getCharacterEncodingScheme();
/*      */   }
/*      */ 
/*      */   public String getVersion()
/*      */   {
/*  321 */     return null;
/*      */   }
/*      */ 
/*      */   public boolean isStandalone()
/*      */   {
/*  328 */     return false;
/*      */   }
/*      */ 
/*      */   public boolean standaloneSet()
/*      */   {
/*  335 */     return false;
/*      */   }
/*      */ 
/*      */   public abstract Object getProperty(String paramString);
/*      */ 
/*      */   public abstract boolean isPropertySupported(String paramString);
/*      */ 
/*      */   public abstract boolean setProperty(String paramString, Object paramObject);
/*      */ 
/*      */   public int getAttributeCount()
/*      */   {
/*  369 */     if (this._currEvent != 1) {
/*  370 */       reportWrongState(1);
/*      */     }
/*  372 */     if (this._attrList == null) {
/*  373 */       _calcNsAndAttrLists(true);
/*      */     }
/*  375 */     return this._attrList.size();
/*      */   }
/*      */ 
/*      */   public String getAttributeLocalName(int paramInt)
/*      */   {
/*  380 */     if (this._currEvent != 1) {
/*  381 */       reportWrongState(1);
/*      */     }
/*  383 */     if (this._attrList == null) {
/*  384 */       _calcNsAndAttrLists(true);
/*      */     }
/*  386 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  387 */       handleIllegalAttrIndex(paramInt);
/*  388 */       return null;
/*      */     }
/*  390 */     Attr localAttr = (Attr)this._attrList.get(paramInt);
/*  391 */     return _internName(_safeGetLocalName(localAttr));
/*      */   }
/*      */ 
/*      */   public QName getAttributeName(int paramInt)
/*      */   {
/*  396 */     if (this._currEvent != 1) {
/*  397 */       reportWrongState(1);
/*      */     }
/*  399 */     if (this._attrList == null) {
/*  400 */       _calcNsAndAttrLists(true);
/*      */     }
/*  402 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  403 */       handleIllegalAttrIndex(paramInt);
/*  404 */       return null;
/*      */     }
/*  406 */     Attr localAttr = (Attr)this._attrList.get(paramInt);
/*  407 */     return _constructQName(localAttr.getNamespaceURI(), _safeGetLocalName(localAttr), localAttr.getPrefix());
/*      */   }
/*      */ 
/*      */   public String getAttributeNamespace(int paramInt)
/*      */   {
/*  412 */     if (this._currEvent != 1) {
/*  413 */       reportWrongState(1);
/*      */     }
/*  415 */     if (this._attrList == null) {
/*  416 */       _calcNsAndAttrLists(true);
/*      */     }
/*  418 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  419 */       handleIllegalAttrIndex(paramInt);
/*  420 */       return null;
/*      */     }
/*  422 */     Attr localAttr = (Attr)this._attrList.get(paramInt);
/*  423 */     return _internNsURI(localAttr.getNamespaceURI());
/*      */   }
/*      */ 
/*      */   public String getAttributePrefix(int paramInt)
/*      */   {
/*  428 */     if (this._currEvent != 1) {
/*  429 */       reportWrongState(1);
/*      */     }
/*  431 */     if (this._attrList == null) {
/*  432 */       _calcNsAndAttrLists(true);
/*      */     }
/*  434 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  435 */       handleIllegalAttrIndex(paramInt);
/*  436 */       return null;
/*      */     }
/*  438 */     Attr localAttr = (Attr)this._attrList.get(paramInt);
/*  439 */     return _internName(localAttr.getPrefix());
/*      */   }
/*      */ 
/*      */   public String getAttributeType(int paramInt)
/*      */   {
/*  444 */     if (this._currEvent != 1) {
/*  445 */       reportWrongState(1);
/*      */     }
/*  447 */     if (this._attrList == null) {
/*  448 */       _calcNsAndAttrLists(true);
/*      */     }
/*  450 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  451 */       handleIllegalAttrIndex(paramInt);
/*  452 */       return null;
/*      */     }
/*      */ 
/*  468 */     return "CDATA";
/*      */   }
/*      */ 
/*      */   public String getAttributeValue(int paramInt)
/*      */   {
/*  473 */     if (this._currEvent != 1) {
/*  474 */       reportWrongState(1);
/*      */     }
/*  476 */     if (this._attrList == null) {
/*  477 */       _calcNsAndAttrLists(true);
/*      */     }
/*  479 */     if ((paramInt >= this._attrList.size()) || (paramInt < 0)) {
/*  480 */       handleIllegalAttrIndex(paramInt);
/*  481 */       return null;
/*      */     }
/*  483 */     Attr localAttr = (Attr)this._attrList.get(paramInt);
/*  484 */     return localAttr.getValue();
/*      */   }
/*      */ 
/*      */   public String getAttributeValue(String paramString1, String paramString2)
/*      */   {
/*  489 */     if (this._currEvent != 1) {
/*  490 */       reportWrongState(1);
/*      */     }
/*  492 */     Element localElement = (Element)this._currNode;
/*  493 */     NamedNodeMap localNamedNodeMap = localElement.getAttributes();
/*      */ 
/*  499 */     if ((paramString1 != null) && (paramString1.length() == 0)) {
/*  500 */       paramString1 = null;
/*      */     }
/*  502 */     Attr localAttr = (Attr)localNamedNodeMap.getNamedItemNS(paramString1, paramString2);
/*  503 */     return localAttr == null ? null : localAttr.getValue();
/*      */   }
/*      */ 
/*      */   public String getElementText()
/*      */     throws XMLStreamException
/*      */   {
/*  520 */     if (this._currEvent != 1)
/*      */     {
/*  525 */       reportParseProblem(1);
/*      */     }
/*      */ 
/*  528 */     if (this._coalescing) {
/*  529 */       String str = null;
/*      */       while (true)
/*      */       {
/*  532 */         int j = next();
/*  533 */         if (j == 2) {
/*      */           break;
/*      */         }
/*  536 */         if ((j != 5) && (j != 3))
/*      */         {
/*  539 */           if ((1 << j & 0x1250) == 0) {
/*  540 */             reportParseProblem(4);
/*      */           }
/*  542 */           if (str == null)
/*  543 */             str = getText();
/*      */           else
/*  545 */             str = str + getText();
/*      */         }
/*      */       }
/*  548 */       return str == null ? "" : str;
/*      */     }
/*  550 */     this._textBuffer.reset();
/*      */     while (true)
/*      */     {
/*  553 */       int i = next();
/*  554 */       if (i == 2) {
/*      */         break;
/*      */       }
/*  557 */       if ((i != 5) && (i != 3))
/*      */       {
/*  560 */         if ((1 << i & 0x1250) == 0) {
/*  561 */           reportParseProblem(4);
/*      */         }
/*  563 */         this._textBuffer.append(getText());
/*      */       }
/*      */     }
/*  565 */     return this._textBuffer.get();
/*      */   }
/*      */ 
/*      */   public int getEventType()
/*      */   {
/*  574 */     return this._currEvent;
/*      */   }
/*      */ 
/*      */   public String getLocalName()
/*      */   {
/*  579 */     if ((this._currEvent == 1) || (this._currEvent == 2)) {
/*  580 */       return _internName(_safeGetLocalName(this._currNode));
/*      */     }
/*  582 */     if (this._currEvent != 9) {
/*  583 */       reportWrongState(7);
/*      */     }
/*  585 */     return _internName(this._currNode.getNodeName());
/*      */   }
/*      */ 
/*      */   public final Location getLocation() {
/*  589 */     return getStartLocation();
/*      */   }
/*      */ 
/*      */   public QName getName()
/*      */   {
/*  594 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  595 */       reportWrongState(1);
/*      */     }
/*  597 */     return _constructQName(this._currNode.getNamespaceURI(), _safeGetLocalName(this._currNode), this._currNode.getPrefix());
/*      */   }
/*      */ 
/*      */   public NamespaceContext getNamespaceContext()
/*      */   {
/*  603 */     return this;
/*      */   }
/*      */ 
/*      */   public int getNamespaceCount()
/*      */   {
/*  608 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  609 */       reportWrongState(2);
/*      */     }
/*  611 */     if (this._nsDeclList == null) {
/*  612 */       if (!this._cfgNsAware) {
/*  613 */         return 0;
/*      */       }
/*  615 */       _calcNsAndAttrLists(this._currEvent == 1);
/*      */     }
/*  617 */     return this._nsDeclList.size() / 2;
/*      */   }
/*      */ 
/*      */   public String getNamespacePrefix(int paramInt)
/*      */   {
/*  626 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  627 */       reportWrongState(2);
/*      */     }
/*  629 */     if (this._nsDeclList == null) {
/*  630 */       if (!this._cfgNsAware) {
/*  631 */         handleIllegalNsIndex(paramInt);
/*      */       }
/*  633 */       _calcNsAndAttrLists(this._currEvent == 1);
/*      */     }
/*  635 */     if ((paramInt < 0) || (paramInt + paramInt >= this._nsDeclList.size())) {
/*  636 */       handleIllegalNsIndex(paramInt);
/*      */     }
/*      */ 
/*  639 */     return (String)this._nsDeclList.get(paramInt + paramInt);
/*      */   }
/*      */ 
/*      */   public String getNamespaceURI() {
/*  643 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  644 */       reportWrongState(2);
/*      */     }
/*  646 */     return _internNsURI(this._currNode.getNamespaceURI());
/*      */   }
/*      */ 
/*      */   public String getNamespaceURI(int paramInt) {
/*  650 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  651 */       reportWrongState(2);
/*      */     }
/*  653 */     if (this._nsDeclList == null) {
/*  654 */       if (!this._cfgNsAware) {
/*  655 */         handleIllegalNsIndex(paramInt);
/*      */       }
/*  657 */       _calcNsAndAttrLists(this._currEvent == 1);
/*      */     }
/*  659 */     if ((paramInt < 0) || (paramInt + paramInt >= this._nsDeclList.size())) {
/*  660 */       handleIllegalNsIndex(paramInt);
/*      */     }
/*      */ 
/*  663 */     return (String)this._nsDeclList.get(paramInt + paramInt + 1);
/*      */   }
/*      */ 
/*      */   public String getPIData()
/*      */   {
/*  670 */     if (this._currEvent != 3) {
/*  671 */       reportWrongState(3);
/*      */     }
/*  673 */     return this._currNode.getNodeValue();
/*      */   }
/*      */ 
/*      */   public String getPITarget() {
/*  677 */     if (this._currEvent != 3) {
/*  678 */       reportWrongState(3);
/*      */     }
/*  680 */     return _internName(this._currNode.getNodeName());
/*      */   }
/*      */ 
/*      */   public String getPrefix() {
/*  684 */     if ((this._currEvent != 1) && (this._currEvent != 2)) {
/*  685 */       reportWrongState(2);
/*      */     }
/*  687 */     return _internName(this._currNode.getPrefix());
/*      */   }
/*      */ 
/*      */   public String getText()
/*      */   {
/*  692 */     if (this._coalescedText != null) {
/*  693 */       return this._coalescedText;
/*      */     }
/*  695 */     if ((1 << this._currEvent & 0x1A70) == 0) {
/*  696 */       reportWrongState(4);
/*      */     }
/*  698 */     return this._currNode.getNodeValue();
/*      */   }
/*      */ 
/*      */   public char[] getTextCharacters()
/*      */   {
/*  703 */     String str = getText();
/*  704 */     return str.toCharArray();
/*      */   }
/*      */ 
/*      */   public int getTextCharacters(int paramInt1, char[] paramArrayOfChar, int paramInt2, int paramInt3)
/*      */   {
/*  709 */     if ((1 << this._currEvent & 0x1070) == 0) {
/*  710 */       reportWrongState(5);
/*      */     }
/*  712 */     String str = getText();
/*  713 */     if (paramInt3 > str.length()) {
/*  714 */       paramInt3 = str.length();
/*      */     }
/*  716 */     str.getChars(paramInt1, paramInt1 + paramInt3, paramArrayOfChar, paramInt2);
/*  717 */     return paramInt3;
/*      */   }
/*      */ 
/*      */   public int getTextLength()
/*      */   {
/*  722 */     if ((1 << this._currEvent & 0x1070) == 0) {
/*  723 */       reportWrongState(5);
/*      */     }
/*  725 */     return getText().length();
/*      */   }
/*      */ 
/*      */   public int getTextStart()
/*      */   {
/*  730 */     if ((1 << this._currEvent & 0x1070) == 0) {
/*  731 */       reportWrongState(5);
/*      */     }
/*  733 */     return 0;
/*      */   }
/*      */ 
/*      */   public boolean hasName() {
/*  737 */     return (this._currEvent == 1) || (this._currEvent == 2);
/*      */   }
/*      */ 
/*      */   public boolean hasNext() {
/*  741 */     return this._currEvent != 8;
/*      */   }
/*      */ 
/*      */   public boolean hasText() {
/*  745 */     return (1 << this._currEvent & 0x1A70) != 0;
/*      */   }
/*      */ 
/*      */   public boolean isAttributeSpecified(int paramInt)
/*      */   {
/*  750 */     if (this._currEvent != 1) {
/*  751 */       reportWrongState(1);
/*      */     }
/*  753 */     Element localElement = (Element)this._currNode;
/*  754 */     Attr localAttr = (Attr)localElement.getAttributes().item(paramInt);
/*  755 */     if (localAttr == null) {
/*  756 */       handleIllegalAttrIndex(paramInt);
/*  757 */       return false;
/*      */     }
/*  759 */     return localAttr.getSpecified();
/*      */   }
/*      */ 
/*      */   public boolean isCharacters()
/*      */   {
/*  764 */     return this._currEvent == 4;
/*      */   }
/*      */ 
/*      */   public boolean isEndElement() {
/*  768 */     return this._currEvent == 2;
/*      */   }
/*      */ 
/*      */   public boolean isStartElement() {
/*  772 */     return this._currEvent == 1;
/*      */   }
/*      */ 
/*      */   public boolean isWhiteSpace()
/*      */   {
/*  777 */     if ((this._currEvent == 4) || (this._currEvent == 12)) {
/*  778 */       String str = getText();
/*  779 */       int i = 0; for (int j = str.length(); i < j; i++)
/*      */       {
/*  783 */         if (str.charAt(i) > ' ') {
/*  784 */           return false;
/*      */         }
/*      */       }
/*  787 */       return true;
/*      */     }
/*  789 */     return this._currEvent == 6;
/*      */   }
/*      */ 
/*      */   public void require(int paramInt, String paramString1, String paramString2)
/*      */     throws XMLStreamException
/*      */   {
/*  795 */     int i = this._currEvent;
/*      */ 
/*  801 */     if (i != paramInt) {
/*  802 */       if (i == 12)
/*  803 */         i = 4;
/*  804 */       else if (i == 6) {
/*  805 */         i = 4;
/*      */       }
/*      */     }
/*      */ 
/*  809 */     if (paramInt != i)
/*  810 */       throwStreamException("Required type " + Stax2Util.eventTypeDesc(paramInt) + ", current type " + Stax2Util.eventTypeDesc(i));
/*      */     String str;
/*  815 */     if (paramString2 != null) {
/*  816 */       if ((i != 1) && (i != 2) && (i != 9))
/*      */       {
/*  818 */         throwStreamException("Required a non-null local name, but current token not a START_ELEMENT, END_ELEMENT or ENTITY_REFERENCE (was " + Stax2Util.eventTypeDesc(this._currEvent) + ")");
/*      */       }
/*  820 */       str = getLocalName();
/*  821 */       if ((str != paramString2) && (!str.equals(paramString2))) {
/*  822 */         throwStreamException("Required local name '" + paramString2 + "'; current local name '" + str + "'.");
/*      */       }
/*      */     }
/*  825 */     if (paramString1 != null) {
/*  826 */       if ((i != 1) && (i != 2)) {
/*  827 */         throwStreamException("Required non-null NS URI, but current token not a START_ELEMENT or END_ELEMENT (was " + Stax2Util.eventTypeDesc(i) + ")");
/*      */       }
/*      */ 
/*  830 */       str = getNamespaceURI();
/*      */ 
/*  832 */       if (paramString1.length() == 0) {
/*  833 */         if ((str != null) && (str.length() > 0)) {
/*  834 */           throwStreamException("Required empty namespace, instead have '" + str + "'.");
/*      */         }
/*      */       }
/*  837 */       else if ((paramString1 != str) && (!paramString1.equals(str)))
/*  838 */         throwStreamException("Required namespace '" + paramString1 + "'; have '" + str + "'.");
/*      */     }
/*      */   }
/*      */ 
/*      */   public int next()
/*      */     throws XMLStreamException
/*      */   {
/*  855 */     this._coalescedText = null;
/*      */ 
/*  861 */     switch (this._currEvent)
/*      */     {
/*      */     case 7:
/*  867 */       switch (this._currNode.getNodeType())
/*      */       {
/*      */       case 9:
/*      */       case 11:
/*  871 */         this._currNode = this._currNode.getFirstChild();
/*      */ 
/*  873 */         if (this._currNode != null) break;
/*  874 */         return this._currEvent = 8;
/*      */       case 1:
/*  880 */         return this._currEvent = 1;
/*      */       default:
/*  883 */         throw new XMLStreamException("Internal error: unexpected DOM root node type " + this._currNode.getNodeType() + " for node '" + this._currNode + "'");
/*      */       }
/*      */ 
/*      */       break;
/*      */     case 8:
/*  888 */       throw new NoSuchElementException("Can not call next() after receiving END_DOCUMENT");
/*      */     case 1:
/*  891 */       this._depth += 1;
/*  892 */       this._attrList = null;
/*      */ 
/*  894 */       localNode = this._currNode.getFirstChild();
/*  895 */       if (localNode == null)
/*      */       {
/*  899 */         return this._currEvent = 2;
/*      */       }
/*  901 */       this._nsDeclList = null;
/*      */ 
/*  906 */       this._currNode = localNode;
/*  907 */       break;
/*      */     case 2:
/*  912 */       this._depth -= 1;
/*      */ 
/*  914 */       this._attrList = null;
/*  915 */       this._nsDeclList = null;
/*      */ 
/*  922 */       if (this._currNode == this._rootNode) {
/*  923 */         return this._currEvent = 8;
/*      */       }
/*      */       break;
/*      */     case 3:
/*      */     case 4:
/*      */     case 5:
/*      */     case 6:
/*      */     }
/*      */ 
/*  932 */     Node localNode = this._currNode.getNextSibling();
/*      */ 
/*  934 */     if (localNode != null) {
/*  935 */       this._currNode = localNode;
/*      */     }
/*      */     else
/*      */     {
/*  942 */       this._currNode = this._currNode.getParentNode();
/*  943 */       int i = this._currNode.getNodeType();
/*  944 */       if (i == 1) {
/*  945 */         return this._currEvent = 2;
/*      */       }
/*      */ 
/*  948 */       if ((this._currNode != this._rootNode) || ((i != 9) && (i != 11)))
/*      */       {
/*  950 */         throw new XMLStreamException("Internal error: non-element parent node (" + i + ") that is not the initial root node");
/*      */       }
/*  952 */       return this._currEvent = 8;
/*      */     }
/*      */ 
/*  957 */     switch (this._currNode.getNodeType()) {
/*      */     case 4:
/*  959 */       if (this._coalescing)
/*  960 */         coalesceText(12);
/*      */       else {
/*  962 */         this._currEvent = 12;
/*      */       }
/*  964 */       break;
/*      */     case 8:
/*  966 */       this._currEvent = 5;
/*  967 */       break;
/*      */     case 10:
/*  969 */       this._currEvent = 11;
/*  970 */       break;
/*      */     case 1:
/*  972 */       this._currEvent = 1;
/*  973 */       break;
/*      */     case 5:
/*  975 */       this._currEvent = 9;
/*  976 */       break;
/*      */     case 7:
/*  978 */       this._currEvent = 3;
/*  979 */       break;
/*      */     case 3:
/*  981 */       if (this._coalescing)
/*  982 */         coalesceText(4);
/*      */       else {
/*  984 */         this._currEvent = 4;
/*      */       }
/*  986 */       break;
/*      */     case 2:
/*      */     case 6:
/*      */     case 12:
/*  992 */       throw new XMLStreamException("Internal error: unexpected DOM node type " + this._currNode.getNodeType() + " (attr/entity/notation?), for node '" + this._currNode + "'");
/*      */     case 9:
/*      */     case 11:
/*      */     default:
/*  995 */       throw new XMLStreamException("Internal error: unrecognized DOM node type " + this._currNode.getNodeType() + ", for node '" + this._currNode + "'");
/*      */     }
/*      */ 
/*  998 */     return this._currEvent;
/*      */   }
/*      */ 
/*      */   public int nextTag() throws XMLStreamException
/*      */   {
/*      */     while (true)
/*      */     {
/* 1005 */       int i = next();
/*      */ 
/* 1007 */       switch (i) {
/*      */       case 3:
/*      */       case 5:
/*      */       case 6:
/* 1011 */         break;
/*      */       case 4:
/*      */       case 12:
/* 1014 */         if (isWhiteSpace()) {
/*      */           continue;
/*      */         }
/* 1017 */         throwStreamException("Received non-all-whitespace CHARACTERS or CDATA event in nextTag().");
/* 1018 */         break;
/*      */       case 1:
/*      */       case 2:
/* 1021 */         return i;
/*      */       case 7:
/*      */       case 8:
/*      */       case 9:
/*      */       case 10:
/* 1023 */       case 11: } throwStreamException("Received event " + Stax2Util.eventTypeDesc(i) + ", instead of START_ELEMENT or END_ELEMENT.");
/*      */     }
/*      */   }
/*      */ 
/*      */   public void close()
/*      */     throws XMLStreamException
/*      */   {
/*      */   }
/*      */ 
/*      */   public String getNamespaceURI(String paramString)
/*      */   {
/* 1060 */     Node localNode1 = this._currNode;
/* 1061 */     int i = (paramString == null) || (paramString.length() == 0) ? 1 : 0;
/*      */ 
/* 1063 */     while (localNode1 != null) {
/* 1064 */       NamedNodeMap localNamedNodeMap = localNode1.getAttributes();
/* 1065 */       if (localNamedNodeMap != null) {
/* 1066 */         int j = 0; for (int k = localNamedNodeMap.getLength(); j < k; j++) {
/* 1067 */           Node localNode2 = localNamedNodeMap.item(j);
/* 1068 */           String str = localNode2.getPrefix();
/* 1069 */           if ((str == null) || (str.length() == 0)) {
/* 1070 */             if ((i != 0) && ("xmlns".equals(localNode2.getLocalName())))
/* 1071 */               return localNode2.getNodeValue();
/*      */           }
/* 1073 */           else if ((i == 0) && ("xmlns".equals(str)) && 
/* 1074 */             (paramString.equals(localNode2.getLocalName()))) {
/* 1075 */             return localNode2.getNodeValue();
/*      */           }
/*      */         }
/*      */       }
/*      */ 
/* 1080 */       localNode1 = localNode1.getParentNode();
/*      */     }
/* 1082 */     return null;
/*      */   }
/*      */ 
/*      */   public String getPrefix(String paramString)
/*      */   {
/* 1101 */     Node localNode1 = this._currNode;
/* 1102 */     if (paramString == null) {
/* 1103 */       paramString = "";
/*      */     }
/*      */ 
/* 1106 */     while (localNode1 != null) {
/* 1107 */       NamedNodeMap localNamedNodeMap = localNode1.getAttributes();
/* 1108 */       int i = 0; for (int j = localNamedNodeMap.getLength(); i < j; i++) {
/* 1109 */         Node localNode2 = localNamedNodeMap.item(i);
/* 1110 */         String str = localNode2.getPrefix();
/* 1111 */         if ((str == null) || (str.length() == 0)) {
/* 1112 */           if (("xmlns".equals(localNode2.getLocalName())) && (paramString.equals(localNode2.getNodeValue())))
/* 1113 */             return "";
/*      */         }
/* 1115 */         else if (("xmlns".equals(str)) && 
/* 1116 */           (paramString.equals(localNode2.getNodeValue()))) {
/* 1117 */           return localNode2.getLocalName();
/*      */         }
/*      */       }
/*      */ 
/* 1121 */       localNode1 = localNode1.getParentNode();
/*      */     }
/* 1123 */     return null;
/*      */   }
/*      */ 
/*      */   public Iterator getPrefixes(String paramString)
/*      */   {
/* 1128 */     String str = getPrefix(paramString);
/* 1129 */     if (str == null) {
/* 1130 */       return EmptyIterator.getInstance();
/*      */     }
/* 1132 */     return new SingletonIterator(str);
/*      */   }
/*      */ 
/*      */   public boolean getElementAsBoolean()
/*      */     throws XMLStreamException
/*      */   {
/* 1143 */     ValueDecoderFactory.BooleanDecoder localBooleanDecoder = _decoderFactory().getBooleanDecoder();
/* 1144 */     getElementAs(localBooleanDecoder);
/* 1145 */     return localBooleanDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public int getElementAsInt() throws XMLStreamException
/*      */   {
/* 1150 */     ValueDecoderFactory.IntDecoder localIntDecoder = _decoderFactory().getIntDecoder();
/* 1151 */     getElementAs(localIntDecoder);
/* 1152 */     return localIntDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public long getElementAsLong() throws XMLStreamException
/*      */   {
/* 1157 */     ValueDecoderFactory.LongDecoder localLongDecoder = _decoderFactory().getLongDecoder();
/* 1158 */     getElementAs(localLongDecoder);
/* 1159 */     return localLongDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public float getElementAsFloat() throws XMLStreamException
/*      */   {
/* 1164 */     ValueDecoderFactory.FloatDecoder localFloatDecoder = _decoderFactory().getFloatDecoder();
/* 1165 */     getElementAs(localFloatDecoder);
/* 1166 */     return localFloatDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public double getElementAsDouble() throws XMLStreamException
/*      */   {
/* 1171 */     ValueDecoderFactory.DoubleDecoder localDoubleDecoder = _decoderFactory().getDoubleDecoder();
/* 1172 */     getElementAs(localDoubleDecoder);
/* 1173 */     return localDoubleDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public BigInteger getElementAsInteger() throws XMLStreamException
/*      */   {
/* 1178 */     ValueDecoderFactory.IntegerDecoder localIntegerDecoder = _decoderFactory().getIntegerDecoder();
/* 1179 */     getElementAs(localIntegerDecoder);
/* 1180 */     return localIntegerDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public BigDecimal getElementAsDecimal() throws XMLStreamException
/*      */   {
/* 1185 */     ValueDecoderFactory.DecimalDecoder localDecimalDecoder = _decoderFactory().getDecimalDecoder();
/* 1186 */     getElementAs(localDecimalDecoder);
/* 1187 */     return localDecimalDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public QName getElementAsQName() throws XMLStreamException
/*      */   {
/* 1192 */     ValueDecoderFactory.QNameDecoder localQNameDecoder = _decoderFactory().getQNameDecoder(getNamespaceContext());
/* 1193 */     getElementAs(localQNameDecoder);
/* 1194 */     return localQNameDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public byte[] getElementAsBinary() throws XMLStreamException
/*      */   {
/* 1199 */     return getElementAsBinary(Base64Variants.getDefaultVariant());
/*      */   }
/*      */ 
/*      */   public byte[] getElementAsBinary(Base64Variant paramBase64Variant)
/*      */     throws XMLStreamException
/*      */   {
/* 1205 */     Stax2Util.ByteAggregator localByteAggregator = _base64Decoder().getByteAggregator();
/* 1206 */     byte[] arrayOfByte = localByteAggregator.startAggregation();
/*      */     while (true) {
/* 1208 */       int i = 0;
/* 1209 */       int j = arrayOfByte.length;
/*      */       do {
/* 1211 */         int k = readElementAsBinary(arrayOfByte, i, j, paramBase64Variant);
/* 1212 */         if (k < 1) {
/* 1213 */           return localByteAggregator.aggregateAll(arrayOfByte, i);
/*      */         }
/* 1215 */         i += k;
/* 1216 */         j -= k;
/* 1217 */       }while (j > 0);
/* 1218 */       arrayOfByte = localByteAggregator.addFullBlock(arrayOfByte);
/*      */     }
/*      */   }
/*      */ 
/*      */   public void getElementAs(TypedValueDecoder paramTypedValueDecoder) throws XMLStreamException
/*      */   {
/* 1224 */     String str = getElementText();
/* 1225 */     str = Stax2Util.trimSpaces(str);
/*      */     try {
/* 1227 */       if (str == null)
/* 1228 */         paramTypedValueDecoder.handleEmptyValue();
/*      */       else
/* 1230 */         paramTypedValueDecoder.decode(str);
/*      */     }
/*      */     catch (IllegalArgumentException localIllegalArgumentException) {
/* 1233 */       throw _constructTypeException(localIllegalArgumentException, str);
/*      */     }
/*      */   }
/*      */ 
/*      */   public int readElementAsIntArray(int[] paramArrayOfInt, int paramInt1, int paramInt2) throws XMLStreamException
/*      */   {
/* 1239 */     return readElementAsArray(_decoderFactory().getIntArrayDecoder(paramArrayOfInt, paramInt1, paramInt2));
/*      */   }
/*      */ 
/*      */   public int readElementAsLongArray(long[] paramArrayOfLong, int paramInt1, int paramInt2) throws XMLStreamException
/*      */   {
/* 1244 */     return readElementAsArray(_decoderFactory().getLongArrayDecoder(paramArrayOfLong, paramInt1, paramInt2));
/*      */   }
/*      */ 
/*      */   public int readElementAsFloatArray(float[] paramArrayOfFloat, int paramInt1, int paramInt2) throws XMLStreamException
/*      */   {
/* 1249 */     return readElementAsArray(_decoderFactory().getFloatArrayDecoder(paramArrayOfFloat, paramInt1, paramInt2));
/*      */   }
/*      */ 
/*      */   public int readElementAsDoubleArray(double[] paramArrayOfDouble, int paramInt1, int paramInt2) throws XMLStreamException
/*      */   {
/* 1254 */     return readElementAsArray(_decoderFactory().getDoubleArrayDecoder(paramArrayOfDouble, paramInt1, paramInt2));
/*      */   }
/*      */ 
/*      */   public int readElementAsArray(TypedArrayDecoder paramTypedArrayDecoder)
/*      */     throws XMLStreamException
/*      */   {
/* 1262 */     if (this._currEvent == 1)
/*      */     {
/* 1264 */       localObject1 = this._currNode.getFirstChild();
/* 1265 */       if (localObject1 == null) {
/* 1266 */         this._currEvent = 2;
/* 1267 */         return -1;
/*      */       }
/* 1269 */       this._coalescedText = coalesceTypedText((Node)localObject1);
/* 1270 */       this._currEvent = 4;
/* 1271 */       this._currNode = this._currNode.getLastChild();
/*      */     } else {
/* 1273 */       if ((this._currEvent != 4) && (this._currEvent != 12))
/*      */       {
/* 1275 */         if (this._currEvent == 2) {
/* 1276 */           return -1;
/*      */         }
/* 1278 */         reportWrongState(6);
/*      */       }
/*      */ 
/* 1286 */       if (this._coalescedText == null) {
/* 1287 */         throw new IllegalStateException("First call to readElementAsArray() must be for a START_ELEMENT, not directly for a textual event");
/*      */       }
/*      */ 
/*      */     }
/*      */ 
/* 1294 */     Object localObject1 = this._coalescedText;
/* 1295 */     int i = ((String)localObject1).length();
/* 1296 */     int j = 0;
/* 1297 */     int k = 0;
/* 1298 */     String str = null;
/*      */     try
/*      */     {
/* 1302 */       while (j < i)
/*      */       {
/* 1304 */         while (((String)localObject1).charAt(j) <= ' ') {
/* 1305 */           j++; if (j >= i) {
/* 1306 */             break label218;
/*      */           }
/*      */         }
/*      */ 
/* 1310 */         m = j;
/* 1311 */         j++;
/* 1312 */         while ((j < i) && (((String)localObject1).charAt(j) > ' ')) {
/* 1313 */           j++;
/*      */         }
/* 1315 */         k++;
/*      */ 
/* 1317 */         str = ((String)localObject1).substring(m, j);
/*      */ 
/* 1319 */         j++;
/* 1320 */         if (paramTypedArrayDecoder.decodeValue(str))
/*      */           break;
/*      */       }
/*      */     }
/*      */     catch (IllegalArgumentException localIllegalArgumentException)
/*      */     {
/*      */       int m;
/* 1329 */       label218: Location localLocation = getLocation();
/* 1330 */       throw new TypedXMLStreamException(str, localIllegalArgumentException.getMessage(), localLocation, localIllegalArgumentException);
/*      */     } finally {
/* 1332 */       int n = i - j;
/* 1333 */       this._coalescedText = (n < 1 ? "" : ((String)localObject1).substring(j));
/*      */     }
/*      */ 
/* 1336 */     if (k < 1) {
/* 1337 */       this._currEvent = 2;
/* 1338 */       this._currNode = this._currNode.getParentNode();
/* 1339 */       return -1;
/*      */     }
/* 1341 */     return k;
/*      */   }
/*      */ 
/*      */   private String coalesceTypedText(Node paramNode)
/*      */     throws XMLStreamException
/*      */   {
/* 1351 */     this._textBuffer.reset();
/* 1352 */     this._attrList = null;
/*      */ 
/* 1354 */     for (Node localNode = paramNode; localNode != null; localNode = localNode.getNextSibling()) {
/* 1355 */       switch (localNode.getNodeType())
/*      */       {
/*      */       case 1:
/* 1358 */         throwStreamException("Element content can not contain child START_ELEMENT when using Typed Access methods");
/*      */       case 3:
/*      */       case 4:
/* 1361 */         this._textBuffer.append(localNode.getNodeValue());
/* 1362 */         break;
/*      */       case 7:
/*      */       case 8:
/* 1366 */         break;
/*      */       case 2:
/*      */       case 5:
/*      */       case 6:
/*      */       default:
/* 1370 */         throwStreamException("Unexpected DOM node type (" + localNode.getNodeType() + ") when trying to decode Typed content");
/*      */       }
/*      */     }
/* 1373 */     return this._textBuffer.get();
/*      */   }
/*      */ 
/*      */   public int readElementAsBinary(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
/*      */     throws XMLStreamException
/*      */   {
/* 1385 */     return readElementAsBinary(paramArrayOfByte, paramInt1, paramInt2, Base64Variants.getDefaultVariant());
/*      */   }
/*      */ 
/*      */   public int readElementAsBinary(byte[] paramArrayOfByte, int paramInt1, int paramInt2, Base64Variant paramBase64Variant)
/*      */     throws XMLStreamException
/*      */   {
/* 1392 */     if (paramArrayOfByte == null) {
/* 1393 */       throw new IllegalArgumentException("resultBuffer is null");
/*      */     }
/* 1395 */     if (paramInt1 < 0) {
/* 1396 */       throw new IllegalArgumentException("Illegal offset (" + paramInt1 + "), must be [0, " + paramArrayOfByte.length + "[");
/*      */     }
/* 1398 */     if ((paramInt2 < 1) || (paramInt1 + paramInt2 > paramArrayOfByte.length)) {
/* 1399 */       if (paramInt2 == 0) {
/* 1400 */         return 0;
/*      */       }
/* 1402 */       throw new IllegalArgumentException("Illegal maxLength (" + paramInt2 + "), has to be positive number, and offset+maxLength can not exceed" + paramArrayOfByte.length);
/*      */     }
/*      */ 
/* 1405 */     StringBase64Decoder localStringBase64Decoder = _base64Decoder();
/* 1406 */     int i = this._currEvent;
/*      */ 
/* 1408 */     if ((1 << i & 0x1052) == 0) {
/* 1409 */       if (i == 2)
/*      */       {
/* 1411 */         if (!localStringBase64Decoder.hasData())
/* 1412 */           return -1;
/*      */       }
/*      */       else {
/* 1415 */         reportWrongState(6);
/*      */       }
/*      */ 
/*      */     }
/*      */ 
/* 1420 */     if (i == 1)
/*      */     {
/*      */       do {
/* 1423 */         i = next();
/* 1424 */         if (i == 2)
/*      */         {
/* 1426 */           return -1;
/*      */         }
/*      */       }
/* 1428 */       while ((i == 5) || (i == 3));
/*      */ 
/* 1431 */       if ((1 << i & 0x1250) == 0) {
/* 1432 */         reportParseProblem(4);
/*      */       }
/* 1434 */       localStringBase64Decoder.init(paramBase64Variant, true, getText());
/*      */     }
/*      */ 
/* 1439 */     int j = 0;
/*      */     while (true)
/*      */     {
/*      */       int k;
/*      */       try
/*      */       {
/* 1446 */         k = localStringBase64Decoder.decode(paramArrayOfByte, paramInt1, paramInt2);
/*      */       } catch (IllegalArgumentException localIllegalArgumentException) {
/* 1448 */         throw _constructTypeException(localIllegalArgumentException, "");
/*      */       }
/* 1450 */       paramInt1 += k;
/* 1451 */       j += k;
/* 1452 */       paramInt2 -= k;
/*      */ 
/* 1457 */       if ((paramInt2 < 1) || (this._currEvent == 2))
/*      */       {
/*      */         break;
/*      */       }
/*      */       do
/* 1462 */         i = next();
/* 1463 */       while ((i == 5) || (i == 3) || (i == 6));
/*      */ 
/* 1467 */       if (i == 2)
/*      */       {
/* 1474 */         int m = localStringBase64Decoder.endOfContent();
/* 1475 */         if (m < 0)
/* 1476 */           throw _constructTypeException("Incomplete base64 triplet at the end of decoded content", "");
/* 1477 */         if (m <= 0) {
/*      */           break;
/*      */         }
/*      */       }
/*      */       else
/*      */       {
/* 1483 */         if ((1 << i & 0x1250) == 0) {
/* 1484 */           reportParseProblem(4);
/*      */         }
/* 1486 */         localStringBase64Decoder.init(paramBase64Variant, false, getText());
/*      */       }
/*      */ 
/*      */     }
/*      */ 
/* 1492 */     return j > 0 ? j : -1;
/*      */   }
/*      */ 
/*      */   public int getAttributeIndex(String paramString1, String paramString2)
/*      */   {
/* 1503 */     return findAttributeIndex(paramString1, paramString2);
/*      */   }
/*      */ 
/*      */   public boolean getAttributeAsBoolean(int paramInt) throws XMLStreamException
/*      */   {
/* 1508 */     ValueDecoderFactory.BooleanDecoder localBooleanDecoder = _decoderFactory().getBooleanDecoder();
/* 1509 */     getAttributeAs(paramInt, localBooleanDecoder);
/* 1510 */     return localBooleanDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public int getAttributeAsInt(int paramInt) throws XMLStreamException
/*      */   {
/* 1515 */     ValueDecoderFactory.IntDecoder localIntDecoder = _decoderFactory().getIntDecoder();
/* 1516 */     getAttributeAs(paramInt, localIntDecoder);
/* 1517 */     return localIntDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public long getAttributeAsLong(int paramInt) throws XMLStreamException
/*      */   {
/* 1522 */     ValueDecoderFactory.LongDecoder localLongDecoder = _decoderFactory().getLongDecoder();
/* 1523 */     getAttributeAs(paramInt, localLongDecoder);
/* 1524 */     return localLongDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public float getAttributeAsFloat(int paramInt) throws XMLStreamException
/*      */   {
/* 1529 */     ValueDecoderFactory.FloatDecoder localFloatDecoder = _decoderFactory().getFloatDecoder();
/* 1530 */     getAttributeAs(paramInt, localFloatDecoder);
/* 1531 */     return localFloatDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public double getAttributeAsDouble(int paramInt) throws XMLStreamException
/*      */   {
/* 1536 */     ValueDecoderFactory.DoubleDecoder localDoubleDecoder = _decoderFactory().getDoubleDecoder();
/* 1537 */     getAttributeAs(paramInt, localDoubleDecoder);
/* 1538 */     return localDoubleDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public BigInteger getAttributeAsInteger(int paramInt) throws XMLStreamException
/*      */   {
/* 1543 */     ValueDecoderFactory.IntegerDecoder localIntegerDecoder = _decoderFactory().getIntegerDecoder();
/* 1544 */     getAttributeAs(paramInt, localIntegerDecoder);
/* 1545 */     return localIntegerDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public BigDecimal getAttributeAsDecimal(int paramInt) throws XMLStreamException
/*      */   {
/* 1550 */     ValueDecoderFactory.DecimalDecoder localDecimalDecoder = _decoderFactory().getDecimalDecoder();
/* 1551 */     getAttributeAs(paramInt, localDecimalDecoder);
/* 1552 */     return localDecimalDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public QName getAttributeAsQName(int paramInt) throws XMLStreamException
/*      */   {
/* 1557 */     ValueDecoderFactory.QNameDecoder localQNameDecoder = _decoderFactory().getQNameDecoder(getNamespaceContext());
/* 1558 */     getAttributeAs(paramInt, localQNameDecoder);
/* 1559 */     return localQNameDecoder.getValue();
/*      */   }
/*      */ 
/*      */   public final void getAttributeAs(int paramInt, TypedValueDecoder paramTypedValueDecoder) throws XMLStreamException
/*      */   {
/* 1564 */     String str = getAttributeValue(paramInt);
/* 1565 */     str = Stax2Util.trimSpaces(str);
/*      */     try {
/* 1567 */       if (str == null)
/* 1568 */         paramTypedValueDecoder.handleEmptyValue();
/*      */       else
/* 1570 */         paramTypedValueDecoder.decode(str);
/*      */     }
/*      */     catch (IllegalArgumentException localIllegalArgumentException) {
/* 1573 */       throw _constructTypeException(localIllegalArgumentException, str);
/*      */     }
/*      */   }
/*      */ 
/*      */   public int[] getAttributeAsIntArray(int paramInt) throws XMLStreamException
/*      */   {
/* 1579 */     ValueDecoderFactory.IntArrayDecoder localIntArrayDecoder = _decoderFactory().getIntArrayDecoder();
/* 1580 */     _getAttributeAsArray(localIntArrayDecoder, getAttributeValue(paramInt));
/* 1581 */     return localIntArrayDecoder.getValues();
/*      */   }
/*      */ 
/*      */   public long[] getAttributeAsLongArray(int paramInt) throws XMLStreamException
/*      */   {
/* 1586 */     ValueDecoderFactory.LongArrayDecoder localLongArrayDecoder = _decoderFactory().getLongArrayDecoder();
/* 1587 */     _getAttributeAsArray(localLongArrayDecoder, getAttributeValue(paramInt));
/* 1588 */     return localLongArrayDecoder.getValues();
/*      */   }
/*      */ 
/*      */   public float[] getAttributeAsFloatArray(int paramInt) throws XMLStreamException
/*      */   {
/* 1593 */     ValueDecoderFactory.FloatArrayDecoder localFloatArrayDecoder = _decoderFactory().getFloatArrayDecoder();
/* 1594 */     _getAttributeAsArray(localFloatArrayDecoder, getAttributeValue(paramInt));
/* 1595 */     return localFloatArrayDecoder.getValues();
/*      */   }
/*      */ 
/*      */   public double[] getAttributeAsDoubleArray(int paramInt) throws XMLStreamException
/*      */   {
/* 1600 */     ValueDecoderFactory.DoubleArrayDecoder localDoubleArrayDecoder = _decoderFactory().getDoubleArrayDecoder();
/* 1601 */     _getAttributeAsArray(localDoubleArrayDecoder, getAttributeValue(paramInt));
/* 1602 */     return localDoubleArrayDecoder.getValues();
/*      */   }
/*      */ 
/*      */   public int getAttributeAsArray(int paramInt, TypedArrayDecoder paramTypedArrayDecoder) throws XMLStreamException
/*      */   {
/* 1607 */     return _getAttributeAsArray(paramTypedArrayDecoder, getAttributeValue(paramInt));
/*      */   }
/*      */ 
/*      */   protected int _getAttributeAsArray(TypedArrayDecoder paramTypedArrayDecoder, String paramString) throws XMLStreamException
/*      */   {
/* 1612 */     int i = 0;
/* 1613 */     int j = 0;
/* 1614 */     int k = paramString.length();
/* 1615 */     String str = null;
/* 1616 */     int m = 0;
/*      */     try
/*      */     {
/* 1620 */       while (i < k)
/*      */       {
/* 1622 */         while (paramString.charAt(i) <= ' ') {
/* 1623 */           i++; if (i >= k) {
/* 1624 */             break label115;
/*      */           }
/*      */         }
/*      */ 
/* 1628 */         j = i;
/* 1629 */         i++;
/* 1630 */         while ((i < k) && (paramString.charAt(i) > ' ')) {
/* 1631 */           i++;
/*      */         }
/* 1633 */         int n = i;
/* 1634 */         i++;
/*      */ 
/* 1636 */         str = paramString.substring(j, n);
/* 1637 */         m++;
/* 1638 */         if ((paramTypedArrayDecoder.decodeValue(str)) && 
/* 1639 */           (!checkExpand(paramTypedArrayDecoder))) {
/*      */           break;
/*      */         }
/*      */       }
/*      */     }
/*      */     catch (IllegalArgumentException localIllegalArgumentException)
/*      */     {
/* 1646 */       label115: Location localLocation = getLocation();
/* 1647 */       throw new TypedXMLStreamException(str, localIllegalArgumentException.getMessage(), localLocation, localIllegalArgumentException);
/*      */     }
/* 1649 */     return m;
/*      */   }
/*      */ 
/*      */   private final boolean checkExpand(TypedArrayDecoder paramTypedArrayDecoder)
/*      */   {
/* 1660 */     if ((paramTypedArrayDecoder instanceof ValueDecoderFactory.BaseArrayDecoder)) {
/* 1661 */       ((ValueDecoderFactory.BaseArrayDecoder)paramTypedArrayDecoder).expand();
/* 1662 */       return true;
/*      */     }
/* 1664 */     return false;
/*      */   }
/*      */ 
/*      */   public byte[] getAttributeAsBinary(int paramInt) throws XMLStreamException
/*      */   {
/* 1669 */     return getAttributeAsBinary(paramInt, Base64Variants.getDefaultVariant());
/*      */   }
/*      */ 
/*      */   public byte[] getAttributeAsBinary(int paramInt, Base64Variant paramBase64Variant) throws XMLStreamException
/*      */   {
/* 1674 */     String str = getAttributeValue(paramInt);
/* 1675 */     StringBase64Decoder localStringBase64Decoder = _base64Decoder();
/* 1676 */     localStringBase64Decoder.init(paramBase64Variant, true, str);
/*      */     try {
/* 1678 */       return localStringBase64Decoder.decodeCompletely();
/*      */     } catch (IllegalArgumentException localIllegalArgumentException) {
/* 1680 */       throw _constructTypeException(localIllegalArgumentException, str);
/*      */     }
/*      */   }
/*      */ 
/*      */   public Object getFeature(String paramString)
/*      */   {
/* 1695 */     throw new IllegalArgumentException("Unrecognized feature \"" + paramString + "\"");
/*      */   }
/*      */ 
/*      */   public void setFeature(String paramString, Object paramObject)
/*      */   {
/* 1700 */     throw new IllegalArgumentException("Unrecognized feature \"" + paramString + "\"");
/*      */   }
/*      */ 
/*      */   public void skipElement()
/*      */     throws XMLStreamException
/*      */   {
/* 1707 */     if (this._currEvent != 1) {
/* 1708 */       reportWrongState(1);
/*      */     }
/* 1710 */     int i = 1;
/*      */     while (true)
/*      */     {
/* 1713 */       int j = next();
/* 1714 */       if (j == 1) {
/* 1715 */         i++;
/* 1716 */       } else if (j == 2) {
/* 1717 */         i--; if (i == 0)
/*      */           break;
/*      */       }
/*      */     }
/*      */   }
/*      */ 
/*      */   public AttributeInfo getAttributeInfo()
/*      */     throws XMLStreamException
/*      */   {
/* 1728 */     if (this._currEvent != 1) {
/* 1729 */       reportWrongState(1);
/*      */     }
/* 1731 */     return this;
/*      */   }
/*      */ 
/*      */   public int findAttributeIndex(String paramString1, String paramString2)
/*      */   {
/* 1740 */     if (this._currEvent != 1) {
/* 1741 */       reportWrongState(1);
/*      */     }
/* 1743 */     Element localElement = (Element)this._currNode;
/* 1744 */     NamedNodeMap localNamedNodeMap = localElement.getAttributes();
/* 1745 */     if ((paramString1 != null) && (paramString1.length() == 0)) {
/* 1746 */       paramString1 = null;
/*      */     }
/*      */ 
/* 1749 */     int i = 0; for (int j = localNamedNodeMap.getLength(); i < j; i++) {
/* 1750 */       Node localNode = localNamedNodeMap.item(i);
/* 1751 */       String str1 = _safeGetLocalName(localNode);
/* 1752 */       if (paramString2.equals(str1)) {
/* 1753 */         String str2 = localNode.getNamespaceURI();
/* 1754 */         int k = (str2 == null) || (str2.length() == 0) ? 1 : 0;
/* 1755 */         if (paramString1 == null) {
/* 1756 */           if (k != 0) {
/* 1757 */             return i;
/*      */           }
/*      */         }
/* 1760 */         else if ((k == 0) && (paramString1.equals(str2))) {
/* 1761 */           return i;
/*      */         }
/*      */       }
/*      */     }
/*      */ 
/* 1766 */     return -1;
/*      */   }
/*      */ 
/*      */   public int getIdAttributeIndex()
/*      */   {
/* 1775 */     return -1;
/*      */   }
/*      */ 
/*      */   public int getNotationAttributeIndex()
/*      */   {
/* 1784 */     return -1;
/*      */   }
/*      */ 
/*      */   public DTDInfo getDTDInfo()
/*      */     throws XMLStreamException
/*      */   {
/* 1798 */     if (this._currEvent != 11) {
/* 1799 */       return null;
/*      */     }
/* 1801 */     return this;
/*      */   }
/*      */ 
/*      */   public final LocationInfo getLocationInfo()
/*      */   {
/* 1810 */     return this;
/*      */   }
/*      */ 
/*      */   public int getText(Writer paramWriter, boolean paramBoolean)
/*      */     throws IOException, XMLStreamException
/*      */   {
/* 1840 */     String str = getText();
/* 1841 */     paramWriter.write(str);
/* 1842 */     return str.length();
/*      */   }
/*      */ 
/*      */   public int getDepth()
/*      */   {
/* 1852 */     return this._depth;
/*      */   }
/*      */ 
/*      */   public boolean isEmptyElement()
/*      */     throws XMLStreamException
/*      */   {
/* 1863 */     return false;
/*      */   }
/*      */ 
/*      */   public NamespaceContext getNonTransientNamespaceContext()
/*      */   {
/* 1873 */     return EmptyNamespaceContext.getInstance();
/*      */   }
/*      */ 
/*      */   public String getPrefixedName()
/*      */   {
/* 1878 */     switch (this._currEvent)
/*      */     {
/*      */     case 1:
/*      */     case 2:
/* 1882 */       String str1 = this._currNode.getPrefix();
/* 1883 */       String str2 = _safeGetLocalName(this._currNode);
/*      */ 
/* 1885 */       if (str1 == null) {
/* 1886 */         return _internName(str2);
/*      */       }
/* 1888 */       StringBuffer localStringBuffer = new StringBuffer(str2.length() + 1 + str1.length());
/* 1889 */       localStringBuffer.append(str1);
/* 1890 */       localStringBuffer.append(':');
/* 1891 */       localStringBuffer.append(str2);
/* 1892 */       return _internName(localStringBuffer.toString());
/*      */     case 9:
/* 1895 */       return getLocalName();
/*      */     case 3:
/* 1897 */       return getPITarget();
/*      */     case 11:
/* 1899 */       return getDTDRootName();
/*      */     case 4:
/*      */     case 5:
/*      */     case 6:
/*      */     case 7:
/*      */     case 8:
/* 1902 */     case 10: } throw new IllegalStateException("Current state (" + Stax2Util.eventTypeDesc(this._currEvent) + ") not START_ELEMENT, END_ELEMENT, ENTITY_REFERENCE, PROCESSING_INSTRUCTION or DTD");
/*      */   }
/*      */ 
/*      */   public void closeCompletely()
/*      */     throws XMLStreamException
/*      */   {
/*      */   }
/*      */ 
/*      */   public Object getProcessedDTD()
/*      */   {
/* 1917 */     return null;
/*      */   }
/*      */ 
/*      */   public String getDTDRootName() {
/* 1921 */     if (this._currEvent == 11) {
/* 1922 */       return _internName(((DocumentType)this._currNode).getName());
/*      */     }
/* 1924 */     return null;
/*      */   }
/*      */ 
/*      */   public String getDTDPublicId() {
/* 1928 */     if (this._currEvent == 11) {
/* 1929 */       return ((DocumentType)this._currNode).getPublicId();
/*      */     }
/* 1931 */     return null;
/*      */   }
/*      */ 
/*      */   public String getDTDSystemId() {
/* 1935 */     if (this._currEvent == 11) {
/* 1936 */       return ((DocumentType)this._currNode).getSystemId();
/*      */     }
/* 1938 */     return null;
/*      */   }
/*      */ 
/*      */   public String getDTDInternalSubset()
/*      */   {
/* 1950 */     return null;
/*      */   }
/*      */ 
/*      */   public DTDValidationSchema getProcessedDTDSchema()
/*      */   {
/* 1956 */     return null;
/*      */   }
/*      */ 
/*      */   public long getStartingByteOffset()
/*      */   {
/* 1969 */     return -1L;
/*      */   }
/*      */ 
/*      */   public long getStartingCharOffset()
/*      */   {
/* 1974 */     return 0L;
/*      */   }
/*      */ 
/*      */   public long getEndingByteOffset()
/*      */     throws XMLStreamException
/*      */   {
/* 1980 */     return -1L;
/*      */   }
/*      */ 
/*      */   public long getEndingCharOffset()
/*      */     throws XMLStreamException
/*      */   {
/* 1986 */     return -1L;
/*      */   }
/*      */ 
/*      */   public XMLStreamLocation2 getStartLocation()
/*      */   {
/* 1993 */     return XMLStreamLocation2.NOT_AVAILABLE;
/*      */   }
/*      */ 
/*      */   public XMLStreamLocation2 getCurrentLocation()
/*      */   {
/* 1998 */     return XMLStreamLocation2.NOT_AVAILABLE;
/*      */   }
/*      */ 
/*      */   public final XMLStreamLocation2 getEndLocation()
/*      */     throws XMLStreamException
/*      */   {
/* 2004 */     return XMLStreamLocation2.NOT_AVAILABLE;
/*      */   }
/*      */ 
/*      */   public XMLValidator validateAgainst(XMLValidationSchema paramXMLValidationSchema)
/*      */     throws XMLStreamException
/*      */   {
/* 2017 */     return null;
/*      */   }
/*      */ 
/*      */   public XMLValidator stopValidatingAgainst(XMLValidationSchema paramXMLValidationSchema)
/*      */     throws XMLStreamException
/*      */   {
/* 2024 */     return null;
/*      */   }
/*      */ 
/*      */   public XMLValidator stopValidatingAgainst(XMLValidator paramXMLValidator)
/*      */     throws XMLStreamException
/*      */   {
/* 2031 */     return null;
/*      */   }
/*      */ 
/*      */   public ValidationProblemHandler setValidationProblemHandler(ValidationProblemHandler paramValidationProblemHandler)
/*      */   {
/* 2037 */     return null;
/*      */   }
/*      */ 
/*      */   protected void coalesceText(int paramInt)
/*      */   {
/* 2048 */     this._textBuffer.reset();
/* 2049 */     this._textBuffer.append(this._currNode.getNodeValue());
/*      */     Node localNode;
/* 2052 */     while ((localNode = this._currNode.getNextSibling()) != null) {
/* 2053 */       int i = localNode.getNodeType();
/* 2054 */       if ((i != 3) && (i != 4)) {
/*      */         break;
/*      */       }
/* 2057 */       this._currNode = localNode;
/* 2058 */       this._textBuffer.append(this._currNode.getNodeValue());
/*      */     }
/* 2060 */     this._coalescedText = this._textBuffer.get();
/*      */ 
/* 2063 */     this._currEvent = 4;
/*      */   }
/*      */ 
/*      */   private QName _constructQName(String paramString1, String paramString2, String paramString3)
/*      */   {
/* 2075 */     return new QName(_internNsURI(paramString1), _internName(paramString2), _internName(paramString3));
/*      */   }
/*      */ 
/*      */   private void _calcNsAndAttrLists(boolean paramBoolean)
/*      */   {
/* 2084 */     NamedNodeMap localNamedNodeMap = this._currNode.getAttributes();
/*      */ 
/* 2087 */     int i = localNamedNodeMap.getLength();
/* 2088 */     if (i == 0) {
/* 2089 */       this._attrList = (this._nsDeclList = Collections.EMPTY_LIST);
/* 2090 */       return;
/*      */     }
/*      */ 
/* 2093 */     if (!this._cfgNsAware) {
/* 2094 */       this._attrList = new ArrayList(i);
/* 2095 */       for (int j = 0; j < i; j++) {
/* 2096 */         this._attrList.add(localNamedNodeMap.item(j));
/*      */       }
/* 2098 */       this._nsDeclList = Collections.EMPTY_LIST;
/* 2099 */       return;
/*      */     }
/*      */ 
/* 2103 */     ArrayList localArrayList1 = null;
/* 2104 */     ArrayList localArrayList2 = null;
/*      */ 
/* 2106 */     for (int k = 0; k < i; k++) {
/* 2107 */       Node localNode = localNamedNodeMap.item(k);
/* 2108 */       String str = localNode.getPrefix();
/*      */ 
/* 2111 */       if ((str == null) || (str.length() == 0))
/*      */       {
/* 2113 */         if (!"xmlns".equals(localNode.getLocalName())) {
/* 2114 */           if (!paramBoolean) continue;
/* 2115 */           if (localArrayList1 == null) {
/* 2116 */             localArrayList1 = new ArrayList(i - k);
/*      */           }
/* 2118 */           localArrayList1.add(localNode); continue;
/*      */         }
/*      */ 
/* 2122 */         str = null;
/*      */       } else {
/* 2124 */         if (!"xmlns".equals(str)) {
/* 2125 */           if (!paramBoolean) continue;
/* 2126 */           if (localArrayList1 == null) {
/* 2127 */             localArrayList1 = new ArrayList(i - k);
/*      */           }
/* 2129 */           localArrayList1.add(localNode); continue;
/*      */         }
/*      */ 
/* 2133 */         str = localNode.getLocalName();
/*      */       }
/* 2135 */       if (localArrayList2 == null) {
/* 2136 */         localArrayList2 = new ArrayList((i - k) * 2);
/*      */       }
/* 2138 */       localArrayList2.add(_internName(str));
/* 2139 */       localArrayList2.add(_internNsURI(localNode.getNodeValue()));
/*      */     }
/*      */ 
/* 2142 */     this._attrList = (localArrayList1 == null ? Collections.EMPTY_LIST : localArrayList1);
/* 2143 */     this._nsDeclList = (localArrayList2 == null ? Collections.EMPTY_LIST : localArrayList2);
/*      */   }
/*      */ 
/*      */   private void handleIllegalAttrIndex(int paramInt)
/*      */   {
/* 2148 */     Element localElement = (Element)this._currNode;
/* 2149 */     NamedNodeMap localNamedNodeMap = localElement.getAttributes();
/* 2150 */     int i = localNamedNodeMap.getLength();
/* 2151 */     String str = "Illegal attribute index " + paramInt + "; element <" + localElement.getNodeName() + "> has " + (i == 0 ? "no" : String.valueOf(i)) + " attributes";
/* 2152 */     throw new IllegalArgumentException(str);
/*      */   }
/*      */ 
/*      */   private void handleIllegalNsIndex(int paramInt)
/*      */   {
/* 2157 */     String str = "Illegal namespace declaration index " + paramInt + " (has " + getNamespaceCount() + " ns declarations)";
/* 2158 */     throw new IllegalArgumentException(str);
/*      */   }
/*      */ 
/*      */   private String _safeGetLocalName(Node paramNode)
/*      */   {
/* 2170 */     String str = paramNode.getLocalName();
/* 2171 */     if (str == null) {
/* 2172 */       str = paramNode.getNodeName();
/*      */     }
/* 2174 */     return str;
/*      */   }
/*      */ 
/*      */   protected void reportWrongState(int paramInt)
/*      */   {
/* 2185 */     throw new IllegalStateException(findErrorDesc(paramInt, this._currEvent));
/*      */   }
/*      */ 
/*      */   protected void reportParseProblem(int paramInt)
/*      */     throws XMLStreamException
/*      */   {
/* 2191 */     throwStreamException(findErrorDesc(paramInt, this._currEvent));
/*      */   }
/*      */ 
/*      */   protected void throwStreamException(String paramString)
/*      */     throws XMLStreamException
/*      */   {
/* 2197 */     throwStreamException(paramString, getErrorLocation());
/*      */   }
/*      */ 
/*      */   protected Location getErrorLocation()
/*      */   {
/* 2202 */     Object localObject = getCurrentLocation();
/* 2203 */     if (localObject == null) {
/* 2204 */       localObject = getLocation();
/*      */     }
/* 2206 */     return localObject;
/*      */   }
/*      */ 
/*      */   protected TypedXMLStreamException _constructTypeException(IllegalArgumentException paramIllegalArgumentException, String paramString)
/*      */   {
/* 2219 */     String str = paramIllegalArgumentException.getMessage();
/* 2220 */     if (str == null) {
/* 2221 */       str = "";
/*      */     }
/* 2223 */     XMLStreamLocation2 localXMLStreamLocation2 = getStartLocation();
/* 2224 */     if (localXMLStreamLocation2 == null) {
/* 2225 */       return new TypedXMLStreamException(paramString, str, paramIllegalArgumentException);
/*      */     }
/* 2227 */     return new TypedXMLStreamException(paramString, str, localXMLStreamLocation2);
/*      */   }
/*      */ 
/*      */   protected TypedXMLStreamException _constructTypeException(String paramString1, String paramString2)
/*      */   {
/* 2232 */     XMLStreamLocation2 localXMLStreamLocation2 = getStartLocation();
/* 2233 */     if (localXMLStreamLocation2 == null) {
/* 2234 */       return new TypedXMLStreamException(paramString2, paramString1);
/*      */     }
/* 2236 */     return new TypedXMLStreamException(paramString2, paramString1, localXMLStreamLocation2);
/*      */   }
/*      */ 
/*      */   protected ValueDecoderFactory _decoderFactory()
/*      */   {
/* 2247 */     if (this._decoderFactory == null) {
/* 2248 */       this._decoderFactory = new ValueDecoderFactory();
/*      */     }
/* 2250 */     return this._decoderFactory;
/*      */   }
/*      */ 
/*      */   protected StringBase64Decoder _base64Decoder()
/*      */   {
/* 2255 */     if (this._base64Decoder == null) {
/* 2256 */       this._base64Decoder = new StringBase64Decoder();
/*      */     }
/* 2258 */     return this._base64Decoder;
/*      */   }
/*      */ 
/*      */   protected String findErrorDesc(int paramInt1, int paramInt2)
/*      */   {
/* 2268 */     String str = Stax2Util.eventTypeDesc(paramInt2);
/* 2269 */     switch (paramInt1) {
/*      */     case 1:
/* 2271 */       return "Current event " + str + ", needs to be START_ELEMENT";
/*      */     case 2:
/* 2273 */       return "Current event " + str + ", needs to be START_ELEMENT or END_ELEMENT";
/*      */     case 7:
/* 2275 */       return "Current event (" + str + ") has no local name";
/*      */     case 3:
/* 2277 */       return "Current event (" + str + ") needs to be PROCESSING_INSTRUCTION";
/*      */     case 4:
/* 2280 */       return "Current event (" + str + ") not a textual event";
/*      */     case 6:
/* 2282 */       return "Current event (" + str + " not START_ELEMENT, END_ELEMENT, CHARACTERS or CDATA";
/*      */     case 5:
/* 2284 */       return "Current event " + str + ", needs to be one of CHARACTERS, CDATA, SPACE or COMMENT";
/*      */     }
/*      */ 
/* 2287 */     return "Internal error (unrecognized error type: " + paramInt1 + ")";
/*      */   }
/*      */ 
/*      */   protected String _internName(String paramString)
/*      */   {
/* 2296 */     if (paramString == null) {
/* 2297 */       return "";
/*      */     }
/* 2299 */     return this._cfgInternNames ? paramString.intern() : paramString;
/*      */   }
/*      */ 
/*      */   protected String _internNsURI(String paramString)
/*      */   {
/* 2304 */     if (paramString == null) {
/* 2305 */       return "";
/*      */     }
/* 2307 */     return this._cfgInternNsURIs ? paramString.intern() : paramString;
/*      */   }
/*      */ }

/* Location:           C:\Program Files\NetApp\Snap_Creator_Framework\scServer4.1.2\engine\work\jetty-0.0.0.0-8443-ui.war-_ui-any-\webapp\WEB-INF\lib\stax2-api-3.1.4.jar
 * Qualified Name:     org.codehaus.stax2.ri.dom.DOMWrappingReader
 * JD-Core Version:    0.6.2
 */