package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.ElementValuePair;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SourceTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public abstract class Annotation extends Expression
{
  static final MemberValuePair[] NoValuePairs = new MemberValuePair[0];
  public int declarationSourceEnd;
  public Binding recipient;
  public TypeReference type;
  private AnnotationBinding compilerAnnotation = null;

  public static long getRetentionPolicy(char[] policyName) {
    if ((policyName == null) || (policyName.length == 0))
      return 0L;
    switch (policyName[0]) {
    case 'C':
      if (CharOperation.equals(policyName, TypeConstants.UPPER_CLASS))
        return 35184372088832L;
      break;
    case 'S':
      if (CharOperation.equals(policyName, TypeConstants.UPPER_SOURCE))
        return 17592186044416L;
      break;
    case 'R':
      if (CharOperation.equals(policyName, TypeConstants.UPPER_RUNTIME))
        return 52776558133248L;
      break;
    }
    return 0L;
  }

  public static long getTargetElementType(char[] elementName) {
    if ((elementName == null) || (elementName.length == 0))
      return 0L;
    switch (elementName[0]) {
    case 'A':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_ANNOTATION_TYPE))
        return 4398046511104L;
      break;
    case 'C':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_CONSTRUCTOR))
        return 1099511627776L;
      break;
    case 'F':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_FIELD))
        return 137438953472L;
      break;
    case 'L':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_LOCAL_VARIABLE))
        return 2199023255552L;
      break;
    case 'M':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_METHOD))
        return 274877906944L;
      break;
    case 'P':
      if (CharOperation.equals(elementName, TypeConstants.UPPER_PARAMETER))
        return 549755813888L;
      if (CharOperation.equals(elementName, TypeConstants.UPPER_PACKAGE))
        return 8796093022208L;
      break;
    case 'T':
      if (CharOperation.equals(elementName, TypeConstants.TYPE))
        return 68719476736L;
      break;
    }
    return 0L;
  }

  public ElementValuePair[] computeElementValuePairs() {
    return Binding.NO_ELEMENT_VALUE_PAIRS;
  }

  private long detectStandardAnnotation(Scope scope, ReferenceBinding annotationType, MemberValuePair valueAttribute)
  {
    long tagBits = 0L;
    switch (annotationType.id)
    {
    case 48:
      if (valueAttribute != null) {
        Expression expr = valueAttribute.value;
        if ((expr.bits & 0x3) == 1) {
          FieldBinding field = ((Reference)expr).fieldBinding();
          if ((field != null) && (field.declaringClass.id == 51)) {
            tagBits |= getRetentionPolicy(field.name);
          }
        }
      }
      break;
    case 50:
      tagBits |= 34359738368L;
      if (valueAttribute != null) {
        Expression expr = valueAttribute.value;
        if ((expr instanceof ArrayInitializer)) {
          ArrayInitializer initializer = (ArrayInitializer)expr;
          Expression[] expressions = initializer.expressions;
          if (expressions != null) {
            int i = 0; for (int length = expressions.length; i < length; i++) {
              Expression initExpr = expressions[i];
              if ((initExpr.bits & 0x3) == 1) {
                FieldBinding field = ((Reference)initExpr).fieldBinding();
                if ((field != null) && (field.declaringClass.id == 52)) {
                  long element = getTargetElementType(field.name);
                  if ((tagBits & element) != 0L)
                    scope.problemReporter().duplicateTargetInTargetAnnotation(annotationType, (NameReference)initExpr);
                  else
                    tagBits |= element;
                }
              }
            }
          }
        }
        else if ((expr.bits & 0x3) == 1) {
          FieldBinding field = ((Reference)expr).fieldBinding();
          if ((field != null) && (field.declaringClass.id == 52)) {
            tagBits |= getTargetElementType(field.name);
          }
        }
      }
      break;
    case 44:
      tagBits |= 70368744177664L;
      break;
    case 45:
      tagBits |= 140737488355328L;
      break;
    case 46:
      tagBits |= 281474976710656L;
      break;
    case 47:
      tagBits |= 562949953421312L;
      break;
    case 49:
      tagBits |= 1125899906842624L;
    }

    return tagBits;
  }

  public AnnotationBinding getCompilerAnnotation() {
    return this.compilerAnnotation;
  }

  public abstract MemberValuePair[] memberValuePairs();

  public StringBuffer printExpression(int indent, StringBuffer output) {
    output.append('@');
    this.type.printExpression(0, output);
    return output;
  }

  public void recordSuppressWarnings(Scope scope, int startSuppresss, int endSuppress, boolean isSuppressingWarnings) {
    long suppressWarningIrritants = 0L;
    MemberValuePair[] pairs = memberValuePairs();
    int i = 0; for (int length = pairs.length; i < length; i++) {
      MemberValuePair pair = pairs[i];
      if (CharOperation.equals(pair.name, TypeConstants.VALUE)) {
        Expression value = pair.value;
        if ((value instanceof ArrayInitializer)) {
          ArrayInitializer initializer = (ArrayInitializer)value;
          Expression[] inits = initializer.expressions;
          if (inits == null) break;
          int j = 0; for (int initsLength = inits.length; j < initsLength; j++) {
            Constant cst = inits[j].constant;
            if ((cst != Constant.NotAConstant) && (cst.typeID() == 11)) {
              long irritant = CompilerOptions.warningTokenToIrritant(cst.stringValue());
              if (irritant != 0L) {
                suppressWarningIrritants |= irritant;
                if ((suppressWarningIrritants ^ 0xFFFFFFFF) == 0L) break; 
              }
              else { scope.problemReporter().unhandledWarningToken(inits[j]); }

            }
          }
          break;
        }

        Constant cst = value.constant;
        if ((cst == Constant.NotAConstant) || (cst.typeID() != 11)) break;
        long irritant = CompilerOptions.warningTokenToIrritant(cst.stringValue());
        if (irritant != 0L) {
          suppressWarningIrritants |= irritant;
          if ((suppressWarningIrritants ^ 0xFFFFFFFF) != 0L) break; break;
        }
        scope.problemReporter().unhandledWarningToken(value);

        break;
      }
    }
    if ((isSuppressingWarnings) && (suppressWarningIrritants != 0L))
      scope.referenceCompilationUnit().compilationResult.recordSuppressWarnings(suppressWarningIrritants, startSuppresss, endSuppress);
  }

  public TypeBinding resolveType(BlockScope scope)
  {
    if (this.compilerAnnotation != null)
      return this.resolvedType;
    this.constant = Constant.NotAConstant;

    TypeBinding typeBinding = this.type.resolveType(scope);
    if (typeBinding == null) {
      return null;
    }
    this.resolvedType = typeBinding;

    if (!typeBinding.isAnnotationType()) {
      scope.problemReporter().typeMismatchError(typeBinding, scope.getJavaLangAnnotationAnnotation(), this.type);
      return null;
    }

    ReferenceBinding annotationType = (ReferenceBinding)this.resolvedType;
    MethodBinding[] methods = annotationType.methods();

    MemberValuePair[] originalValuePairs = memberValuePairs();
    MemberValuePair valueAttribute = null;

    int pairsLength = originalValuePairs.length;
    MemberValuePair[] pairs;
    if (pairsLength > 0)
    {
      MemberValuePair[] pairs;
      System.arraycopy(originalValuePairs, 0, pairs = new MemberValuePair[pairsLength], 0, pairsLength);
    } else {
      pairs = originalValuePairs;
    }

    int i = 0; for (int requiredLength = methods.length; i < requiredLength; i++) {
      MethodBinding method = methods[i];
      char[] selector = method.selector;
      boolean foundValue = false;
      for (int j = 0; j < pairsLength; j++) {
        MemberValuePair pair = pairs[j];
        if (pair != null) {
          char[] name = pair.name;
          if (CharOperation.equals(name, selector)) {
            if ((valueAttribute == null) && (CharOperation.equals(name, TypeConstants.VALUE))) {
              valueAttribute = pair;
            }
            pair.binding = method;
            pair.resolveTypeExpecting(scope, method.returnType);
            pairs[j] = null;
            foundValue = true;

            boolean foundDuplicate = false;
            for (int k = j + 1; k < pairsLength; k++) {
              MemberValuePair otherPair = pairs[k];
              if ((otherPair != null) && 
                (CharOperation.equals(otherPair.name, selector))) {
                foundDuplicate = true;
                scope.problemReporter().duplicateAnnotationValue(annotationType, otherPair);
                otherPair.binding = method;
                otherPair.resolveTypeExpecting(scope, method.returnType);
                pairs[k] = null;
              }
            }
            if (foundDuplicate) {
              scope.problemReporter().duplicateAnnotationValue(annotationType, pair);
              break;
            }
          }
        }
      }
      if ((!foundValue) && ((method.modifiers & 0x20000) == 0)) {
        scope.problemReporter().missingValueForAnnotationMember(this, selector);
      }
    }

    for (int i = 0; i < pairsLength; i++) {
      if (pairs[i] != null) {
        scope.problemReporter().undefinedAnnotationValue(annotationType, pairs[i]);
        pairs[i].resolveTypeExpecting(scope, null);
      }
    }

    this.compilerAnnotation = scope.environment().createAnnotation((ReferenceBinding)this.resolvedType, computeElementValuePairs());

    long tagBits = detectStandardAnnotation(scope, annotationType, valueAttribute);

    scope.referenceCompilationUnit().compilationResult.recordSuppressWarnings(256L, this.sourceStart, this.declarationSourceEnd);
    if (this.recipient != null) {
      if (tagBits != 0L)
      {
        switch (this.recipient.kind()) {
        case 16:
          ((PackageBinding)this.recipient).tagBits |= tagBits;
          break;
        case 4:
        case 2052:
          SourceTypeBinding sourceType = (SourceTypeBinding)this.recipient;
          sourceType.tagBits |= tagBits;
          if ((tagBits & 0x0) != 0L) {
            TypeDeclaration typeDeclaration = sourceType.scope.referenceContext;
            int start;
            int start;
            if (scope.referenceCompilationUnit().types[0] == typeDeclaration)
              start = 0;
            else {
              start = typeDeclaration.declarationSourceStart;
            }
            recordSuppressWarnings(scope, start, typeDeclaration.declarationSourceEnd, scope.compilerOptions().suppressWarnings);
          }
          break;
        case 8:
          MethodBinding sourceMethod = (MethodBinding)this.recipient;
          sourceMethod.tagBits |= tagBits;
          if ((tagBits & 0x0) != 0L) {
            SourceTypeBinding sourceType = (SourceTypeBinding)sourceMethod.declaringClass;
            AbstractMethodDeclaration methodDeclaration = sourceType.scope.referenceContext.declarationOf(sourceMethod);
            recordSuppressWarnings(scope, methodDeclaration.declarationSourceStart, methodDeclaration.declarationSourceEnd, scope.compilerOptions().suppressWarnings);
          }
          break;
        case 1:
          FieldBinding sourceField = (FieldBinding)this.recipient;
          sourceField.tagBits |= tagBits;
          if ((tagBits & 0x0) != 0L) {
            SourceTypeBinding sourceType = (SourceTypeBinding)sourceField.declaringClass;
            FieldDeclaration fieldDeclaration = sourceType.scope.referenceContext.declarationOf(sourceField);
            recordSuppressWarnings(scope, fieldDeclaration.declarationSourceStart, fieldDeclaration.declarationSourceEnd, scope.compilerOptions().suppressWarnings);
          }
          break;
        case 2:
          LocalVariableBinding variable = (LocalVariableBinding)this.recipient;
          variable.tagBits |= tagBits;
          if ((tagBits & 0x0) != 0L) {
            LocalDeclaration localDeclaration = variable.declaration;
            recordSuppressWarnings(scope, localDeclaration.declarationSourceStart, localDeclaration.declarationSourceEnd, scope.compilerOptions().suppressWarnings);
          }
          break;
        }

      }

      long metaTagBits = annotationType.getAnnotationTagBits();
      if ((metaTagBits & 0x0) != 0L)
      {
        switch (this.recipient.kind()) {
        case 16:
          if ((metaTagBits & 0x0) == 0L) break;
          break;
        case 4:
        case 2052:
          if (((ReferenceBinding)this.recipient).isAnnotationType()) {
            if ((metaTagBits & 0x0) != 0L)
              break label1151;
          } else if ((metaTagBits & 0x0) == 0L)
              break; break;
        case 8:
          if (((MethodBinding)this.recipient).isConstructor()) {
            if ((metaTagBits & 0x0) != 0L)
              break label1151;
          } else if ((metaTagBits & 0x0) == 0L)
              break; break;
        case 1:
          if ((metaTagBits & 0x0) == 0L) break;
          break;
        case 2:
          if ((((LocalVariableBinding)this.recipient).tagBits & 0x400) != 0L ? 
            (metaTagBits & 0x0) != 0L : 
            (annotationType.tagBits & 0x0) != 0L) {
            break label1151;
          }
        }
        scope.problemReporter().disallowedTargetForAnnotation(this);
      }
    }
    label1151: return this.resolvedType;
  }

  public abstract void traverse(ASTVisitor paramASTVisitor, BlockScope paramBlockScope);
}