package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class JavadocSingleTypeReference extends SingleTypeReference
{
  public int tagSourceStart;
  public int tagSourceEnd;
  public PackageBinding packageBinding;

  public JavadocSingleTypeReference(char[] source, long pos, int tagStart, int tagEnd)
  {
    super(source, pos);
    this.tagSourceStart = tagStart;
    this.tagSourceEnd = tagEnd;
    this.bits |= 32768;
  }

  protected void reportInvalidType(Scope scope) {
    scope.problemReporter().javadocInvalidType(this, this.resolvedType, scope.getDeclarationModifiers());
  }
  protected void reportDeprecatedType(TypeBinding type, Scope scope) {
    scope.problemReporter().javadocDeprecatedType(type, this, scope.getDeclarationModifiers());
  }

  public void traverse(ASTVisitor visitor, BlockScope scope)
  {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }

  public void traverse(ASTVisitor visitor, ClassScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }

  TypeBinding internalResolveType(Scope scope)
  {
    this.constant = Constant.NotAConstant;
    if (this.resolvedType != null) {
      return this.resolvedType.isValidBinding() ? this.resolvedType : null;
    }
    this.resolvedType = getTypeBinding(scope);
    if (!this.resolvedType.isValidBinding()) {
      char[][] tokens = { this.token };
      Binding binding = scope.getTypeOrPackage(tokens);
      if ((binding instanceof PackageBinding)) {
        this.packageBinding = ((PackageBinding)binding);
      } else {
        if (this.resolvedType.problemId() == 7) {
          ReferenceBinding closestMatch = ((ProblemReferenceBinding)this.resolvedType).closestMatch();
          if ((closestMatch != null) && (closestMatch.isTypeVariable())) {
            this.resolvedType = closestMatch;
            return this.resolvedType;
          }
        }
        reportInvalidType(scope);
      }
      return null;
    }
    if (isTypeUseDeprecated(this.resolvedType, scope))
      reportDeprecatedType(this.resolvedType, scope);
    if ((this.resolvedType instanceof ParameterizedTypeBinding)) {
      this.resolvedType = ((ParameterizedTypeBinding)this.resolvedType).genericType();
    }
    return this.resolvedType;
  }

  public TypeBinding resolveType(BlockScope blockScope, boolean checkBounds)
  {
    return internalResolveType(blockScope);
  }

  public TypeBinding resolveType(ClassScope classScope) {
    return internalResolveType(classScope);
  }
}