package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.PackageBinding;
import org.eclipse.jdt.internal.compiler.lookup.ParameterizedTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeVariableBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class ParameterizedQualifiedTypeReference extends ArrayQualifiedTypeReference
{
  public TypeReference[][] typeArguments;

  public ParameterizedQualifiedTypeReference(char[][] tokens, TypeReference[][] typeArguments, int dim, long[] positions)
  {
    super(tokens, dim, positions);
    this.typeArguments = typeArguments;
  }
  public void checkBounds(Scope scope) {
    if (this.resolvedType == null) return;

    checkBounds(
      (ReferenceBinding)this.resolvedType.leafComponentType(), 
      scope, 
      this.typeArguments.length - 1);
  }

  public void checkBounds(ReferenceBinding type, Scope scope, int index) {
    if ((index > 0) && (type.enclosingType() != null)) {
      checkBounds(type.enclosingType(), scope, index - 1);
    }
    if (type.isParameterizedType()) {
      ParameterizedTypeBinding parameterizedType = (ParameterizedTypeBinding)type;
      ReferenceBinding currentType = parameterizedType.genericType();
      TypeVariableBinding[] typeVariables = currentType.typeVariables();
      TypeBinding[] argTypes = parameterizedType.arguments;
      if ((argTypes != null) && (typeVariables != null))
        parameterizedType.boundCheck(scope, this.typeArguments[index]);
    }
  }

  public TypeReference copyDims(int dim) {
    return new ParameterizedQualifiedTypeReference(this.tokens, this.typeArguments, dim, this.sourcePositions);
  }

  public char[][] getParameterizedTypeName()
  {
    int length = this.tokens.length;
    char[][] qParamName = new char[length][];
    for (int i = 0; i < length; i++) {
      TypeReference[] arguments = this.typeArguments[i];
      if (arguments == null) {
        qParamName[i] = this.tokens[i];
      } else {
        StringBuffer buffer = new StringBuffer(5);
        buffer.append(this.tokens[i]);
        buffer.append('<');
        int j = 0; for (int argLength = arguments.length; j < argLength; j++) {
          if (j > 0) buffer.append(',');
          buffer.append(CharOperation.concatWith(arguments[j].getParameterizedTypeName(), '.'));
        }
        buffer.append('>');
        int nameLength = buffer.length();
        qParamName[i] = new char[nameLength];
        buffer.getChars(0, nameLength, qParamName[i], 0);
      }
    }
    int dim = this.dimensions;
    if (dim > 0) {
      char[] dimChars = new char[dim * 2];
      for (int i = 0; i < dim; i++) {
        int index = i * 2;
        dimChars[index] = '[';
        dimChars[(index + 1)] = ']';
      }
      qParamName[(length - 1)] = CharOperation.concat(qParamName[(length - 1)], dimChars);
    }
    return qParamName;
  }

  protected TypeBinding getTypeBinding(Scope scope)
  {
    return null;
  }

  private TypeBinding internalResolveType(Scope scope, boolean checkBounds)
  {
    this.constant = Constant.NotAConstant;
    if ((this.bits & 0x40000) != 0) {
      if ((this.resolvedType != null) && (!this.resolvedType.isValidBinding()))
        return null;
      return this.resolvedType;
    }
    this.bits |= 262144;
    boolean isClassScope = scope.kind == 3;
    Binding binding = scope.getPackage(this.tokens);
    if ((binding != null) && (!binding.isValidBinding())) {
      this.resolvedType = ((ReferenceBinding)binding);
      reportInvalidType(scope);

      int i = 0; for (int max = this.tokens.length; i < max; i++) {
        TypeReference[] args = this.typeArguments[i];
        if (args != null) {
          int argLength = args.length;
          for (int j = 0; j < argLength; j++) {
            TypeReference typeArgument = args[j];
            if (isClassScope)
              typeArgument.resolveType((ClassScope)scope);
            else {
              typeArgument.resolveType((BlockScope)scope, checkBounds);
            }
          }
        }
      }
      return null;
    }

    PackageBinding packageBinding = binding == null ? null : (PackageBinding)binding;
    boolean typeIsConsistent = true;
    ReferenceBinding qualifiedType = null;
    int i = packageBinding == null ? 0 : packageBinding.compoundName.length; for (int max = this.tokens.length; i < max; i++) {
      findNextTypeBinding(i, scope, packageBinding);
      if (!this.resolvedType.isValidBinding()) {
        reportInvalidType(scope);

        for (int j = i; j < max; j++) {
          TypeReference[] args = this.typeArguments[j];
          if (args != null) {
            int argLength = args.length;
            for (int k = 0; k < argLength; k++) {
              TypeReference typeArgument = args[k];
              if (isClassScope)
                typeArgument.resolveType((ClassScope)scope);
              else {
                typeArgument.resolveType((BlockScope)scope);
              }
            }
          }
        }
        return null;
      }
      ReferenceBinding currentType = (ReferenceBinding)this.resolvedType;
      if (qualifiedType == null) {
        qualifiedType = currentType.enclosingType();
        if ((qualifiedType != null) && ((qualifiedType.isGenericType()) || (qualifiedType.isParameterizedType()))) {
          qualifiedType = currentType.isStatic() ? 
            (ReferenceBinding)scope.environment().convertToRawType(qualifiedType) : 
            scope.environment().convertToParameterizedType(qualifiedType);
        }
      }
      if ((typeIsConsistent) && (currentType.isStatic()) && (qualifiedType != null) && ((qualifiedType.isParameterizedType()) || (qualifiedType.isGenericType()))) {
        scope.problemReporter().staticMemberOfParameterizedType(this, scope.environment().createParameterizedType((ReferenceBinding)currentType.erasure(), null, qualifiedType));
        typeIsConsistent = false;
      }

      TypeReference[] args = this.typeArguments[i];
      if (args != null) {
        TypeReference keep = null;
        if (isClassScope) {
          keep = ((ClassScope)scope).superTypeReference;
          ((ClassScope)scope).superTypeReference = null;
        }
        int argLength = args.length;
        TypeBinding[] argTypes = new TypeBinding[argLength];
        boolean argHasError = false;
        for (int j = 0; j < argLength; j++) {
          TypeReference arg = args[j];
          TypeBinding argType = isClassScope ? 
            arg.resolveTypeArgument((ClassScope)scope, currentType, j) : 
            arg.resolveTypeArgument((BlockScope)scope, currentType, j);
          if (argType == null)
            argHasError = true;
          else {
            argTypes[j] = argType;
          }
        }
        if (argHasError) {
          return null;
        }
        if (isClassScope) {
          ((ClassScope)scope).superTypeReference = keep;
          if (((ClassScope)scope).detectHierarchyCycle(currentType, this)) {
            return null;
          }
        }
        TypeVariableBinding[] typeVariables = currentType.typeVariables();
        if (typeVariables == Binding.NO_TYPE_VARIABLES) {
          scope.problemReporter().nonGenericTypeCannotBeParameterized(i, this, currentType, argTypes);
          return null;
        }if (argLength != typeVariables.length) {
          scope.problemReporter().incorrectArityForParameterizedType(this, currentType, argTypes);
          return null;
        }

        if ((typeIsConsistent) && (!currentType.isStatic())) {
          ReferenceBinding actualEnclosing = currentType.enclosingType();
          if ((actualEnclosing != null) && (actualEnclosing.isRawType())) {
            scope.problemReporter().rawMemberTypeCannotBeParameterized(
              this, scope.environment().createRawType((ReferenceBinding)currentType.erasure(), actualEnclosing), argTypes);
            typeIsConsistent = false;
          }
        }
        ParameterizedTypeBinding parameterizedType = scope.environment().createParameterizedType((ReferenceBinding)currentType.erasure(), argTypes, qualifiedType);

        if (checkBounds)
          parameterizedType.boundCheck(scope, args);
        qualifiedType = parameterizedType;
      } else {
        if ((isClassScope) && 
          (((ClassScope)scope).detectHierarchyCycle(currentType, this)))
          return null;
        ReferenceBinding currentErasure = (ReferenceBinding)currentType.erasure();
        if (currentErasure.isGenericType()) {
          if ((typeIsConsistent) && (qualifiedType != null) && (qualifiedType.isParameterizedType())) {
            scope.problemReporter().parameterizedMemberTypeMissingArguments(this, scope.environment().createParameterizedType(currentErasure, null, qualifiedType));
            typeIsConsistent = false;
          }
          qualifiedType = scope.environment().createRawType(currentErasure, qualifiedType);
        } else {
          qualifiedType = (qualifiedType != null) && (qualifiedType.isParameterizedType()) ? 
            scope.environment().createParameterizedType(currentErasure, null, qualifiedType) : 
            currentType;
        }
      }
      if (isTypeUseDeprecated(qualifiedType, scope))
        reportDeprecatedType(qualifiedType, scope);
      this.resolvedType = qualifiedType;
    }

    if (this.dimensions > 0) {
      if (this.dimensions > 255)
        scope.problemReporter().tooManyDimensions(this);
      this.resolvedType = scope.createArrayType(this.resolvedType, this.dimensions);
    }
    return this.resolvedType;
  }

  public StringBuffer printExpression(int indent, StringBuffer output) {
    int length = this.tokens.length;
    for (int i = 0; i < length - 1; i++) {
      output.append(this.tokens[i]);
      TypeReference[] typeArgument = this.typeArguments[i];
      if (typeArgument != null) {
        output.append('<');
        int max = typeArgument.length - 1;
        for (int j = 0; j < max; j++) {
          typeArgument[j].print(0, output);
          output.append(", ");
        }
        typeArgument[max].print(0, output);
        output.append('>');
      }
      output.append('.');
    }
    output.append(this.tokens[(length - 1)]);
    TypeReference[] typeArgument = this.typeArguments[(length - 1)];
    if (typeArgument != null) {
      output.append('<');
      int max = typeArgument.length - 1;
      for (int j = 0; j < max; j++) {
        typeArgument[j].print(0, output);
        output.append(", ");
      }
      typeArgument[max].print(0, output);
      output.append('>');
    }
    if ((this.bits & 0x4000) != 0) {
      for (int i = 0; i < this.dimensions - 1; i++) {
        output.append("[]");
      }
      output.append("...");
    } else {
      for (int i = 0; i < this.dimensions; i++) {
        output.append("[]");
      }
    }
    return output;
  }

  public TypeBinding resolveType(BlockScope scope, boolean checkBounds) {
    return internalResolveType(scope, checkBounds);
  }
  public TypeBinding resolveType(ClassScope scope) {
    return internalResolveType(scope, false);
  }
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    if (visitor.visit(this, scope)) {
      int i = 0; for (int max = this.typeArguments.length; i < max; i++) {
        if (this.typeArguments[i] != null) {
          int j = 0; for (int max2 = this.typeArguments[i].length; j < max2; j++) {
            this.typeArguments[i][j].traverse(visitor, scope);
          }
        }
      }
    }
    visitor.endVisit(this, scope);
  }

  public void traverse(ASTVisitor visitor, ClassScope scope) {
    if (visitor.visit(this, scope)) {
      int i = 0; for (int max = this.typeArguments.length; i < max; i++) {
        if (this.typeArguments[i] != null) {
          int j = 0; for (int max2 = this.typeArguments[i].length; j < max2; j++) {
            this.typeArguments[i][j].traverse(visitor, scope);
          }
        }
      }
    }
    visitor.endVisit(this, scope);
  }
}