package org.eclipse.jdt.internal.compiler.codegen;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ClassFile;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;
import org.eclipse.jdt.internal.compiler.lookup.MethodBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.SyntheticArgumentBinding;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class StackMapFrameCodeStream extends CodeStream
{
  public StackMapFrame currentFrame;
  public ArrayList frames;
  public Set framePositions;
  public ArrayList variablesModificationsPositions;
  public int[] stateIndexes;
  public int stateIndexesCounter;

  public StackMapFrameCodeStream(ClassFile givenClassFile)
  {
    super(givenClassFile);
  }
  public void aaload() {
    super.aaload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void aastore() { super.aastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void aconst_null()
  {
    super.aconst_null();
    this.currentFrame.addStackItem(TypeBinding.NULL);
  }

  public void addDefinitelyAssignedVariables(Scope scope, int initStateIndex) {
    for (int i = 0; i < this.visibleLocalsCount; i++) {
      LocalVariableBinding localBinding = this.visibleLocals[i];
      if (localBinding != null)
      {
        boolean isDefinitelyAssigned = isDefinitelyAssigned(scope, initStateIndex, localBinding);
        if (!isDefinitelyAssigned) {
          if (this.stateIndexes != null) {
            int j = 0; for (int max = this.stateIndexesCounter; j < max; j++)
              if (isDefinitelyAssigned(scope, this.stateIndexes[j], localBinding)) {
                this.currentFrame.putLocal(localBinding.resolvedPosition, new VerificationTypeInfo(localBinding.type));
                if ((localBinding.initializationCount != 0) && (localBinding.initializationPCs[((localBinding.initializationCount - 1 << 1) + 1)] == -1))
                {
                  break;
                }

                localBinding.recordInitializationStartPC(this.position);

                break;
              }
          }
        }
        else {
          this.currentFrame.putLocal(localBinding.resolvedPosition, new VerificationTypeInfo(localBinding.type));
          if ((localBinding.initializationCount == 0) || (localBinding.initializationPCs[((localBinding.initializationCount - 1 << 1) + 1)] != -1))
          {
            localBinding.recordInitializationStartPC(this.position);
          }
        }
      }
    }
    Integer newValue = new Integer(this.position);
    if ((this.variablesModificationsPositions.size() == 0) || (!this.variablesModificationsPositions.get(this.variablesModificationsPositions.size() - 1).equals(newValue))) {
      this.variablesModificationsPositions.add(newValue);
    }
    storeStackMapFrame();
  }
  public void addVariable(LocalVariableBinding localBinding) {
    this.currentFrame.putLocal(localBinding.resolvedPosition, new VerificationTypeInfo(localBinding.type));
    storeStackMapFrame();
    super.addVariable(localBinding);
  }
  public void aload(int iArg) {
    super.aload(iArg);
    this.currentFrame.addStackItem(getLocal(iArg, this.currentFrame));
  }
  public void aload_0() {
    super.aload_0();
    this.currentFrame.addStackItem(getLocal(0, this.currentFrame));
  }
  public void aload_1() {
    super.aload_1();
    this.currentFrame.addStackItem(getLocal(1, this.currentFrame));
  }

  public void aload_2() {
    super.aload_2();
    this.currentFrame.addStackItem(getLocal(2, this.currentFrame));
  }
  public void aload_3() {
    super.aload_3();
    this.currentFrame.addStackItem(getLocal(3, this.currentFrame));
  }
  public void anewarray(TypeBinding typeBinding) {
    super.anewarray(typeBinding);
    char[] constantPoolName = typeBinding.constantPoolName();
    int length = constantPoolName.length;
    System.arraycopy(constantPoolName, 0, constantPoolName = new char[length + 3], 2, length);
    constantPoolName[0] = '[';
    constantPoolName[1] = 'L';
    constantPoolName[(length + 2)] = ';';
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(typeBinding.id, constantPoolName);
  }

  public void areturn() {
    super.areturn();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position));
  }
  public void arraylength() {
    super.arraylength();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
  }

  public void astore(int iArg) {
    super.astore(iArg);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(iArg, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void astore_0() { super.astore_0();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(0, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void astore_1() {
    super.astore_1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(1, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void astore_2() { super.astore_2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(2, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void astore_3() {
    super.astore_3();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(3, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void athrow() { super.athrow();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position)); }

  public void baload() {
    super.baload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void bastore() { super.bastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void bipush(byte b)
  {
    super.bipush(b);
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void caload() {
    super.caload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void castore() { super.castore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void checkcast(int baseId)
  {
    super.checkcast(baseId);
    VerificationTypeInfo info = null;
    switch (baseId) {
    case 3:
      info = new VerificationTypeInfo(26, ConstantPool.JavaLangByteConstantPoolName);
      break;
    case 4:
      info = new VerificationTypeInfo(27, ConstantPool.JavaLangShortConstantPoolName);
      break;
    case 2:
      info = new VerificationTypeInfo(28, ConstantPool.JavaLangCharacterConstantPoolName);
      break;
    case 10:
      info = new VerificationTypeInfo(29, ConstantPool.JavaLangIntegerConstantPoolName);
      break;
    case 7:
      info = new VerificationTypeInfo(30, ConstantPool.JavaLangLongConstantPoolName);
      break;
    case 9:
      info = new VerificationTypeInfo(31, ConstantPool.JavaLangFloatConstantPoolName);
      break;
    case 8:
      info = new VerificationTypeInfo(32, ConstantPool.JavaLangDoubleConstantPoolName);
      break;
    case 5:
      info = new VerificationTypeInfo(33, ConstantPool.JavaLangBooleanConstantPoolName);
    case 6:
    }int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if ((info != null) && (numberOfStackItems >= 1))
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = info;
  }

  public void checkcast(TypeBinding typeBinding) {
    super.checkcast(typeBinding);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(typeBinding);
  }

  public void d2f() {
    super.d2f();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.FLOAT);
  }

  public void d2i() {
    super.d2i();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
  }

  public void d2l() {
    super.d2l();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.LONG);
  }

  public void dadd() {
    super.dadd();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void daload() {
    super.daload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void dastore() { super.dastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void dcmpg()
  {
    super.dcmpg();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.addStackItem(TypeBinding.INT);
    }
  }

  public void dcmpl() { super.dcmpl();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.addStackItem(TypeBinding.INT);
    } }

  public void dconst_0() {
    super.dconst_0();
    this.currentFrame.addStackItem(TypeBinding.DOUBLE);
  }
  public void dconst_1() {
    super.dconst_1();
    this.currentFrame.addStackItem(TypeBinding.DOUBLE);
  }
  public void ddiv() {
    super.ddiv();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void decrStackSize(int offset) {
    super.decrStackSize(offset);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void dload(int iArg) {
    super.dload(iArg);
    this.currentFrame.addStackItem(getLocal(iArg, this.currentFrame));
  }
  public void dload_0() {
    super.dload_0();
    this.currentFrame.addStackItem(getLocal(0, this.currentFrame));
  }
  public void dload_1() {
    super.dload_1();
    this.currentFrame.addStackItem(getLocal(1, this.currentFrame));
  }
  public void dload_2() {
    super.dload_2();
    this.currentFrame.addStackItem(getLocal(2, this.currentFrame));
  }
  public void dload_3() {
    super.dload_3();
    this.currentFrame.addStackItem(getLocal(3, this.currentFrame));
  }
  public void dmul() {
    super.dmul();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void drem() {
    super.drem();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void dreturn() {
    super.dreturn();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position));
  }
  public void dstore(int iArg) {
    super.dstore(iArg);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(iArg, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void dstore_0() { super.dstore_0();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(0, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void dstore_1() {
    super.dstore_1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(1, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void dstore_2() { super.dstore_2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(2, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void dstore_3() {
    super.dstore_3();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(3, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void dsub() { super.dsub();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void dup()
  {
    super.dup();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.addStackItem(this.currentFrame.stackItems[(numberOfStackItems - 1)]);
  }

  public void dup_x1() {
    super.dup_x1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 2) {
      VerificationTypeInfo info = this.currentFrame.stackItems[(numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      VerificationTypeInfo info2 = this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.addStackItem(info);
      this.currentFrame.addStackItem(info2);
      this.currentFrame.addStackItem(info);
    }
  }

  public void dup_x2() { super.dup_x2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 2) {
      VerificationTypeInfo info = this.currentFrame.stackItems[(numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      VerificationTypeInfo info2 = this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      switch (info2.id()) {
      case 7:
      case 8:
        this.currentFrame.addStackItem(info);
        this.currentFrame.addStackItem(info2);
        this.currentFrame.addStackItem(info);
        break;
      default:
        numberOfStackItems = this.currentFrame.numberOfStackItems;
        if (numberOfStackItems >= 1) {
          VerificationTypeInfo info3 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
          this.currentFrame.numberOfStackItems -= 1;
          this.currentFrame.addStackItem(info);
          this.currentFrame.addStackItem(info3);
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
        }break;
      }
    } }

  public void dup2() {
    super.dup2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      VerificationTypeInfo info = this.currentFrame.stackItems[(numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      switch (info.id()) {
      case 7:
      case 8:
        this.currentFrame.addStackItem(info);
        this.currentFrame.addStackItem(info);
        break;
      default:
        numberOfStackItems = this.currentFrame.numberOfStackItems;
        if (numberOfStackItems >= 1) {
          VerificationTypeInfo info2 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
          this.currentFrame.numberOfStackItems -= 1;
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
        }break;
      }
    }
  }

  public void dup2_x1() { super.dup2_x1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 2) {
      VerificationTypeInfo info = this.currentFrame.stackItems[(numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      VerificationTypeInfo info2 = this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      switch (info.id()) {
      case 7:
      case 8:
        this.currentFrame.addStackItem(info);
        this.currentFrame.addStackItem(info2);
        this.currentFrame.addStackItem(info);
        break;
      default:
        numberOfStackItems = this.currentFrame.numberOfStackItems;
        if (numberOfStackItems >= 1) {
          VerificationTypeInfo info3 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
          this.currentFrame.numberOfStackItems -= 1;
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
          this.currentFrame.addStackItem(info3);
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
        }break;
      }
    } }

  public void dup2_x2() {
    super.dup2_x2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 2) {
      VerificationTypeInfo info = this.currentFrame.stackItems[(numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      VerificationTypeInfo info2 = this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)];
      this.currentFrame.numberOfStackItems -= 1;
      switch (info.id()) {
      case 7:
      case 8:
        switch (info2.id())
        {
        case 7:
        case 8:
          this.currentFrame.addStackItem(info);
          this.currentFrame.addStackItem(info2);
          this.currentFrame.addStackItem(info);
          break;
        default:
          numberOfStackItems = this.currentFrame.numberOfStackItems;
          if (numberOfStackItems >= 1) {
            VerificationTypeInfo info3 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
            this.currentFrame.numberOfStackItems -= 1;
            this.currentFrame.addStackItem(info);
            this.currentFrame.addStackItem(info3);
            this.currentFrame.addStackItem(info2);
            this.currentFrame.addStackItem(info);
          }break;
        }
        break;
      default:
        numberOfStackItems = this.currentFrame.numberOfStackItems;
        if (numberOfStackItems >= 1) {
          VerificationTypeInfo info3 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
          this.currentFrame.numberOfStackItems -= 1;
          switch (info3.id())
          {
          case 7:
          case 8:
            this.currentFrame.addStackItem(info2);
            this.currentFrame.addStackItem(info);
            this.currentFrame.addStackItem(info3);
            this.currentFrame.addStackItem(info2);
            this.currentFrame.addStackItem(info);
            break;
          default:
            numberOfStackItems = this.currentFrame.numberOfStackItems;
            if (numberOfStackItems >= 1) {
              VerificationTypeInfo info4 = this.currentFrame.stackItems[(numberOfStackItems - 1)];
              this.currentFrame.numberOfStackItems -= 1;
              this.currentFrame.addStackItem(info2);
              this.currentFrame.addStackItem(info);
              this.currentFrame.addStackItem(info4);
              this.currentFrame.addStackItem(info3);
              this.currentFrame.addStackItem(info2);
              this.currentFrame.addStackItem(info);
            }break;
          }
        }break;
      }
    }
  }

  public void exitUserScope(BlockScope currentScope) { int index = this.visibleLocalsCount - 1;
    while (index >= 0) {
      LocalVariableBinding visibleLocal = this.visibleLocals[index];
      if (visibleLocal == null) {
        index--;
      }
      else {
        if (visibleLocal.declaringScope != currentScope)
        {
          break;
        }
        if (visibleLocal.initializationCount > 0) {
          this.currentFrame.removeLocals(visibleLocal.resolvedPosition);
        }
        index--;
      }
    }
    if (currentScope != null) {
      int localIndex = currentScope.localIndex;
      if (localIndex != 0) {
        for (int i = 0; i < localIndex; i++) {
          LocalVariableBinding variableBinding = currentScope.locals[i];
          if ((variableBinding != null) && (variableBinding.useFlag == 1) && (variableBinding.resolvedPosition != -1)) {
            this.currentFrame.removeLocals(variableBinding.resolvedPosition);
          }
        }
      }
    }
    super.exitUserScope(currentScope); }

  public void exitUserScope(BlockScope currentScope, LocalVariableBinding binding) {
    int index = this.visibleLocalsCount - 1;
    while (index >= 0) {
      LocalVariableBinding visibleLocal = this.visibleLocals[index];
      if ((visibleLocal == null) || (visibleLocal == binding)) {
        index--;
      }
      else {
        if (visibleLocal.declaringScope != currentScope)
        {
          break;
        }
        if (visibleLocal.initializationCount > 0) {
          this.currentFrame.removeLocals(visibleLocal.resolvedPosition);
        }
        index--;
      }
    }
    if (currentScope != null) {
      int localIndex = currentScope.localIndex;
      if (localIndex != 0) {
        for (int i = 0; i < localIndex; i++) {
          LocalVariableBinding variableBinding = currentScope.locals[i];
          if ((variableBinding != null) && (variableBinding != binding) && (variableBinding.useFlag == 1) && (variableBinding.resolvedPosition != -1)) {
            this.currentFrame.removeLocals(variableBinding.resolvedPosition);
          }
        }
      }
    }
    storeStackMapFrame();
    super.exitUserScope(currentScope, binding);
  }
  public void f2d() {
    super.f2d();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.DOUBLE);
  }

  public void f2i() {
    super.f2i();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
  }

  public void f2l() {
    super.f2l();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.LONG);
  }

  public void fadd() {
    super.fadd();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void faload() {
    super.faload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void fastore() { super.fastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void fcmpg()
  {
    super.fcmpg();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.addStackItem(TypeBinding.INT);
    }
  }

  public void fcmpl() { super.fcmpl();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.addStackItem(TypeBinding.INT);
    } }

  public void fconst_0() {
    super.fconst_0();
    this.currentFrame.addStackItem(TypeBinding.FLOAT);
  }
  public void fconst_1() {
    super.fconst_1();
    this.currentFrame.addStackItem(TypeBinding.FLOAT);
  }
  public void fconst_2() {
    super.fconst_2();
    this.currentFrame.addStackItem(TypeBinding.FLOAT);
  }
  public void fdiv() {
    super.fdiv();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void fload(int iArg) {
    super.fload(iArg);
    this.currentFrame.addStackItem(getLocal(iArg, this.currentFrame));
  }
  public void fload_0() {
    super.fload_0();
    this.currentFrame.addStackItem(getLocal(0, this.currentFrame));
  }
  public void fload_1() {
    super.fload_1();
    this.currentFrame.addStackItem(getLocal(1, this.currentFrame));
  }
  public void fload_2() {
    super.fload_2();
    this.currentFrame.addStackItem(getLocal(2, this.currentFrame));
  }
  public void fload_3() {
    super.fload_3();
    this.currentFrame.addStackItem(getLocal(3, this.currentFrame));
  }
  public void fmul() {
    super.fmul();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void frem() {
    super.frem();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void freturn() {
    super.freturn();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position));
  }
  public void fstore(int iArg) {
    super.fstore(iArg);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(iArg, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void fstore_0() { super.fstore_0();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(0, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void fstore_1() {
    super.fstore_1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(1, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void fstore_2() { super.fstore_2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(2, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void fstore_3() {
    super.fstore_3();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(3, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void fsub() { super.fsub();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void generateBoxingConversion(int unboxedTypeID)
  {
    super.generateBoxingConversion(unboxedTypeID);
    VerificationTypeInfo info = null;
    switch (unboxedTypeID) {
    case 3:
      info = new VerificationTypeInfo(26, ConstantPool.JavaLangByteConstantPoolName);
      break;
    case 4:
      info = new VerificationTypeInfo(27, ConstantPool.JavaLangShortConstantPoolName);
      break;
    case 2:
      info = new VerificationTypeInfo(28, ConstantPool.JavaLangCharacterConstantPoolName);
      break;
    case 10:
      info = new VerificationTypeInfo(29, ConstantPool.JavaLangIntegerConstantPoolName);
      break;
    case 7:
      info = new VerificationTypeInfo(30, ConstantPool.JavaLangLongConstantPoolName);
      break;
    case 9:
      info = new VerificationTypeInfo(31, ConstantPool.JavaLangFloatConstantPoolName);
      break;
    case 8:
      info = new VerificationTypeInfo(32, ConstantPool.JavaLangDoubleConstantPoolName);
      break;
    case 5:
      info = new VerificationTypeInfo(33, ConstantPool.JavaLangBooleanConstantPoolName);
    case 6:
    }int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if ((info != null) && (numberOfStackItems >= 1))
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = info;
  }

  public void generateOuterAccess(Object[] mappingSequence, ASTNode invocationSite, Binding target, Scope scope) {
    if (mappingSequence == null) {
      if ((target instanceof LocalVariableBinding)) {
        scope.problemReporter().needImplementation();
      } else {
        scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, false);
        this.currentFrame.addStackItem((ReferenceBinding)target);
      }
      return;
    }
    if (mappingSequence == BlockScope.NoEnclosingInstanceInConstructorCall) {
      scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, true);
      this.currentFrame.addStackItem((ReferenceBinding)target);
      return;
    }if (mappingSequence == BlockScope.NoEnclosingInstanceInStaticContext) {
      scope.problemReporter().noSuchEnclosingInstance((ReferenceBinding)target, invocationSite, false);
      this.currentFrame.addStackItem((ReferenceBinding)target);
      return;
    }

    if (mappingSequence == BlockScope.EmulationPathToImplicitThis) {
      aload_0();
      return;
    }if ((mappingSequence[0] instanceof FieldBinding)) {
      FieldBinding fieldBinding = (FieldBinding)mappingSequence[0];
      aload_0();
      getfield(fieldBinding);
    } else {
      load((LocalVariableBinding)mappingSequence[0]);
    }
    int i = 1; for (int length = mappingSequence.length; i < length; i++)
      if ((mappingSequence[i] instanceof FieldBinding)) {
        FieldBinding fieldBinding = (FieldBinding)mappingSequence[i];
        getfield(fieldBinding);
      } else {
        invokestatic((MethodBinding)mappingSequence[i]);
      }
  }

  public void generateUnboxingConversion(int unboxedTypeID) {
    super.generateUnboxingConversion(unboxedTypeID);
    VerificationTypeInfo info = null;
    switch (unboxedTypeID) {
    case 3:
      info = new VerificationTypeInfo(TypeBinding.BYTE);
      break;
    case 4:
      info = new VerificationTypeInfo(TypeBinding.SHORT);
      break;
    case 2:
      info = new VerificationTypeInfo(TypeBinding.CHAR);
      break;
    case 10:
      info = new VerificationTypeInfo(TypeBinding.INT);
      break;
    case 7:
      info = new VerificationTypeInfo(TypeBinding.LONG);
      break;
    case 9:
      info = new VerificationTypeInfo(TypeBinding.FLOAT);
      break;
    case 8:
      info = new VerificationTypeInfo(TypeBinding.DOUBLE);
      break;
    case 5:
      info = new VerificationTypeInfo(TypeBinding.BOOLEAN);
    case 6:
    }int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if ((info != null) && (numberOfStackItems >= 1))
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = info;
  }

  public void getBaseTypeValue(int baseTypeID) {
    super.getBaseTypeValue(baseTypeID);
    VerificationTypeInfo info = null;
    switch (baseTypeID) {
    case 3:
      info = new VerificationTypeInfo(TypeBinding.BYTE);
      break;
    case 4:
      info = new VerificationTypeInfo(TypeBinding.SHORT);
      break;
    case 2:
      info = new VerificationTypeInfo(TypeBinding.CHAR);
      break;
    case 10:
      info = new VerificationTypeInfo(TypeBinding.INT);
      break;
    case 7:
      info = new VerificationTypeInfo(TypeBinding.LONG);
      break;
    case 9:
      info = new VerificationTypeInfo(TypeBinding.FLOAT);
      break;
    case 8:
      info = new VerificationTypeInfo(TypeBinding.DOUBLE);
      break;
    case 5:
      info = new VerificationTypeInfo(TypeBinding.BOOLEAN);
    case 6:
    }int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if ((info != null) && (numberOfStackItems >= 1))
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = info;
  }

  public void getfield(FieldBinding fieldBinding) {
    super.getfield(fieldBinding);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(fieldBinding.type);
  }

  private VerificationTypeInfo getLocal(int resolvedPosition, StackMapFrame frame) {
    VerificationTypeInfo verificationTypeInfo = frame.locals[resolvedPosition];

    if (verificationTypeInfo == null)
      return null;
    try
    {
      if ((verificationTypeInfo.tag == 6) || 
        (verificationTypeInfo.tag == 8)) {
        return verificationTypeInfo;
      }
      return (VerificationTypeInfo)verificationTypeInfo.clone(); } catch (CloneNotSupportedException localCloneNotSupportedException) {
    }
    return verificationTypeInfo;
  }

  protected int getPosition()
  {
    int pos = super.getPosition();
    this.framePositions.add(new Integer(this.position));
    storeStackMapFrame();
    return pos;
  }
  public void getstatic(FieldBinding fieldBinding) {
    super.getstatic(fieldBinding);
    this.currentFrame.addStackItem(fieldBinding.type);
  }
  public void getTYPE(int baseTypeID) {
    super.getTYPE(baseTypeID);
    this.currentFrame.addStackItem(new VerificationTypeInfo(16, ConstantPool.JavaLangClassConstantPoolName));
  }

  public void goto_(BranchLabel label)
  {
    super.goto_(label);
    this.framePositions.add(new Integer(this.position));
  }
  public void goto_w(BranchLabel label) {
    super.goto_w(label);
    this.framePositions.add(new Integer(this.position));
  }
  public void i2b() {
    super.i2b();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.BYTE);
  }

  public void i2c() {
    super.i2c();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.CHAR);
  }

  public void i2d() {
    super.i2d();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.DOUBLE);
  }

  public void i2f() {
    super.i2f();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.FLOAT);
  }

  public void i2l() {
    super.i2l();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.LONG);
  }

  public void i2s() {
    super.i2s();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.SHORT);
  }

  public void iadd() {
    super.iadd();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void iaload() {
    super.iaload();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void iand() { super.iand();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void iastore()
  {
    super.iastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3;
  }

  public void iconst_0() {
    super.iconst_0();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_1() {
    super.iconst_1();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_2() {
    super.iconst_2();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_3() {
    super.iconst_3();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_4() {
    super.iconst_4();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_5() {
    super.iconst_5();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void iconst_m1() {
    super.iconst_m1();
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void idiv() {
    super.idiv();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void if_acmpeq(BranchLabel lbl) {
    super.if_acmpeq(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_acmpne(BranchLabel lbl) {
    super.if_acmpne(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmpeq(BranchLabel lbl) {
    super.if_icmpeq(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmpge(BranchLabel lbl) {
    super.if_icmpge(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmpgt(BranchLabel lbl) {
    super.if_icmpgt(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmple(BranchLabel lbl) {
    super.if_icmple(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmplt(BranchLabel lbl) {
    super.if_icmplt(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void if_icmpne(BranchLabel lbl) {
    super.if_icmpne(lbl);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void ifeq(BranchLabel lbl) {
    super.ifeq(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifge(BranchLabel lbl) {
    super.ifge(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifgt(BranchLabel lbl) {
    super.ifgt(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifle(BranchLabel lbl) {
    super.ifle(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void iflt(BranchLabel lbl) {
    super.iflt(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifne(BranchLabel lbl) {
    super.ifne(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifnonnull(BranchLabel lbl) {
    super.ifnonnull(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ifnull(BranchLabel lbl) {
    super.ifnull(lbl);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void iload(int iArg) {
    super.iload(iArg);
    this.currentFrame.addStackItem(getLocal(iArg, this.currentFrame));
  }
  public void iload_0() {
    super.iload_0();
    this.currentFrame.addStackItem(getLocal(0, this.currentFrame));
  }
  public void iload_1() {
    super.iload_1();
    this.currentFrame.addStackItem(getLocal(1, this.currentFrame));
  }
  public void iload_2() {
    super.iload_2();
    this.currentFrame.addStackItem(getLocal(2, this.currentFrame));
  }
  public void iload_3() {
    super.iload_3();
    this.currentFrame.addStackItem(getLocal(3, this.currentFrame));
  }
  public void imul() {
    super.imul();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public boolean inlineForwardReferencesFromLabelsTargeting(BranchLabel targetLabel, int gotoLocation) {
    if (targetLabel.delegate != null) return false;
    int chaining = 0;

    boolean removeFrame = true;
    for (int i = this.countLabels - 1; i >= 0; i--) {
      BranchLabel currentLabel = this.labels[i];
      if (currentLabel.position != gotoLocation) break;
      if (currentLabel == targetLabel) {
        chaining |= 4;
      }
      else if (currentLabel.isStandardLabel()) {
        if (currentLabel.delegate == null) {
          chaining |= 2;
          if ((currentLabel.forwardReferenceCount() == 0) && ((currentLabel.tagBits & 0x2) != 0))
            removeFrame = false;
        }
      }
      else
      {
        removeFrame = false;
        chaining |= 4;
      }
    }
    if ((chaining & 0x2) != 0) {
      for (int i = this.countLabels - 1; i >= 0; i--) {
        BranchLabel currentLabel = this.labels[i];
        if (currentLabel.position != gotoLocation) break;
        if ((currentLabel != targetLabel) && 
          (currentLabel.isStandardLabel()) && 
          (currentLabel.delegate == null)) {
          targetLabel.becomeDelegateFor(currentLabel);

          if (removeFrame) {
            currentLabel.tagBits &= -3;
            removeStackFrameFor(gotoLocation);
          }
        }
      }
    }
    return (chaining & 0x6) == 2;
  }

  public void init(ClassFile targetClassFile) {
    super.init(targetClassFile);
    this.frames = null;
    this.currentFrame = null;
  }
  public void initializeMaxLocals(MethodBinding methodBinding) {
    super.initializeMaxLocals(methodBinding);
    StackMapFrame frame = new StackMapFrame();
    frame.pc = -1;

    if (this.maxLocals != 0) {
      int resolvedPosition = 0;

      boolean isConstructor = methodBinding.isConstructor();
      if (isConstructor) {
        frame.putLocal(resolvedPosition, new VerificationTypeInfo(6, methodBinding.declaringClass));
        resolvedPosition++;
      } else if (!methodBinding.isStatic()) {
        frame.putLocal(resolvedPosition, new VerificationTypeInfo(7, methodBinding.declaringClass));
        resolvedPosition++;
      }

      if (isConstructor) {
        if (methodBinding.declaringClass.isEnum()) {
          frame.putLocal(resolvedPosition, new VerificationTypeInfo(11, ConstantPool.JavaLangStringConstantPoolName));
          resolvedPosition++;
          frame.putLocal(resolvedPosition, new VerificationTypeInfo(TypeBinding.INT));
          resolvedPosition++;
        }

        if (methodBinding.declaringClass.isNestedType())
        {
          ReferenceBinding[] enclosingInstanceTypes;
          if ((enclosingInstanceTypes = methodBinding.declaringClass.syntheticEnclosingInstanceTypes()) != null) {
            int i = 0; for (int max = enclosingInstanceTypes.length; i < max; i++)
            {
              frame.putLocal(resolvedPosition, new VerificationTypeInfo(enclosingInstanceTypes[i]));
              resolvedPosition++;
            }
          }
          TypeBinding[] arguments;
          if ((arguments = methodBinding.parameters) != null) {
            int i = 0; for (int max = arguments.length; i < max; i++) {
              TypeBinding typeBinding = arguments[i];
              frame.putLocal(resolvedPosition, new VerificationTypeInfo(typeBinding));
              switch (typeBinding.id) {
              case 7:
              case 8:
                resolvedPosition += 2;
                break;
              default:
                resolvedPosition++;
              }
            }
          }
          SyntheticArgumentBinding[] syntheticArguments;
          if ((syntheticArguments = methodBinding.declaringClass.syntheticOuterLocalVariables()) != null) {
            int i = 0; for (int max = syntheticArguments.length; i < max; i++) {
              TypeBinding typeBinding = syntheticArguments[i].type;
              frame.putLocal(resolvedPosition, new VerificationTypeInfo(typeBinding));
              switch (typeBinding.id) {
              case 7:
              case 8:
                resolvedPosition += 2;
                break;
              default:
                resolvedPosition++;
              }
            }
          }
        }
        else
        {
          TypeBinding[] arguments;
          if ((arguments = methodBinding.parameters) != null) {
            int i = 0; for (int max = arguments.length; i < max; i++) {
              TypeBinding typeBinding = arguments[i];
              frame.putLocal(resolvedPosition, new VerificationTypeInfo(typeBinding));
              switch (typeBinding.id) {
              case 7:
              case 8:
                resolvedPosition += 2;
                break;
              default:
                resolvedPosition++;
              }
            }
          }
        }
      }
      else
      {
        TypeBinding[] arguments;
        if ((arguments = methodBinding.parameters) != null) {
          int i = 0; for (int max = arguments.length; i < max; i++) {
            TypeBinding typeBinding = arguments[i];
            frame.putLocal(resolvedPosition, new VerificationTypeInfo(typeBinding));
            switch (typeBinding.id) {
            case 7:
            case 8:
              resolvedPosition += 2;
              break;
            default:
              resolvedPosition++;
            }
          }
        }
      }
    }
    try {
      this.frames = new ArrayList();
      this.frames.add(frame.clone());
    } catch (CloneNotSupportedException e) {
      e.printStackTrace();
    }
    this.currentFrame = frame;
    this.framePositions = new HashSet();
    this.variablesModificationsPositions = new ArrayList();
  }
  public void instance_of(TypeBinding typeBinding) {
    super.instance_of(typeBinding);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
  }

  protected void invokeAccessibleObjectSetAccessible() {
    super.invokeAccessibleObjectSetAccessible();
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  protected void invokeArrayNewInstance() {
    super.invokeArrayNewInstance();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(1, ConstantPool.JavaLangObjectConstantPoolName);
    }
  }

  public void invokeClassForName() { super.invokeClassForName();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(16, ConstantPool.JavaLangClassConstantPoolName); }

  protected void invokeClassGetDeclaredConstructor()
  {
    super.invokeClassGetDeclaredConstructor();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(20, ConstantPool.JavaLangReflectConstructorConstantPoolName);
    }
  }

  protected void invokeClassGetDeclaredField() { super.invokeClassGetDeclaredField();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(54, ConstantPool.JAVALANGREFLECTFIELD_CONSTANTPOOLNAME);
    } }

  protected void invokeClassGetDeclaredMethod() {
    super.invokeClassGetDeclaredMethod();
    if (this.currentFrame.numberOfStackItems >= 3) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(55, ConstantPool.JAVALANGREFLECTMETHOD_CONSTANTPOOLNAME);
    }
  }

  public void invokeEnumOrdinal(char[] enumTypeConstantPoolName) { super.invokeEnumOrdinal(enumTypeConstantPoolName);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT); }

  public void invokeinterface(MethodBinding methodBinding)
  {
    super.invokeinterface(methodBinding);
    int argCount = 1;
    argCount += methodBinding.parameters.length;
    if (this.currentFrame.numberOfStackItems >= argCount) {
      this.currentFrame.numberOfStackItems -= argCount;
    }
    if (methodBinding.returnType != TypeBinding.VOID)
      this.currentFrame.addStackItem(methodBinding.returnType);
  }

  public void invokeJavaLangAssertionErrorConstructor(int typeBindingID)
  {
    super.invokeJavaLangAssertionErrorConstructor(typeBindingID);
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void invokeJavaLangAssertionErrorDefaultConstructor() { super.invokeJavaLangAssertionErrorDefaultConstructor();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void invokeJavaLangClassDesiredAssertionStatus()
  {
    super.invokeJavaLangClassDesiredAssertionStatus();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.BOOLEAN);
  }

  public void invokeJavaLangEnumvalueOf(ReferenceBinding binding)
  {
    super.invokeJavaLangEnumvalueOf(binding);
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 2;
      this.currentFrame.addStackItem(binding);
    }
  }

  public void invokeJavaLangEnumValues(TypeBinding enumBinding, ArrayBinding arrayBinding) { super.invokeJavaLangEnumValues(enumBinding, arrayBinding);
    this.currentFrame.addStackItem(arrayBinding); }

  public void invokeJavaLangErrorConstructor() {
    super.invokeJavaLangErrorConstructor();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void invokeJavaLangReflectConstructorNewInstance() { super.invokeJavaLangReflectConstructorNewInstance();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(1, ConstantPool.JavaLangObjectConstantPoolName);
    } }

  protected void invokeJavaLangReflectFieldGetter(int typeID) {
    super.invokeJavaLangReflectFieldGetter(typeID);
    VerificationTypeInfo info = null;
    switch (typeID) {
    case 10:
      info = new VerificationTypeInfo(TypeBinding.INT);
      break;
    case 3:
      info = new VerificationTypeInfo(TypeBinding.BYTE);
      break;
    case 4:
      info = new VerificationTypeInfo(TypeBinding.SHORT);
      break;
    case 7:
      info = new VerificationTypeInfo(TypeBinding.LONG);
      break;
    case 9:
      info = new VerificationTypeInfo(TypeBinding.FLOAT);
      break;
    case 8:
      info = new VerificationTypeInfo(TypeBinding.DOUBLE);
      break;
    case 2:
      info = new VerificationTypeInfo(TypeBinding.CHAR);
      break;
    case 5:
      info = new VerificationTypeInfo(TypeBinding.BOOLEAN);
      break;
    case 6:
    default:
      info = new VerificationTypeInfo(1, ConstantPool.JavaLangObjectConstantPoolName);
    }

    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = info;
    }
  }

  protected void invokeJavaLangReflectFieldSetter(int typeID) { super.invokeJavaLangReflectFieldSetter(typeID);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2; }

  public void invokeJavaLangReflectMethodInvoke()
  {
    super.invokeJavaLangReflectMethodInvoke();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3;
  }

  public void invokeJavaUtilIteratorHasNext() {
    super.invokeJavaUtilIteratorHasNext();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.BOOLEAN);
  }

  public void invokeJavaUtilIteratorNext()
  {
    super.invokeJavaUtilIteratorNext();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(1, ConstantPool.JavaLangObjectConstantPoolName);
  }

  public void invokeNoClassDefFoundErrorStringConstructor() {
    super.invokeNoClassDefFoundErrorStringConstructor();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void invokeObjectGetClass() { super.invokeObjectGetClass();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(16, ConstantPool.JavaLangClassConstantPoolName); }

  public void invokespecial(MethodBinding methodBinding)
  {
    super.invokespecial(methodBinding);

    int argCount = 0;
    if (methodBinding.isConstructor()) {
      ReferenceBinding declaringClass = methodBinding.declaringClass;
      if (declaringClass.isNestedType())
      {
        TypeBinding[] syntheticArgumentTypes = declaringClass.syntheticEnclosingInstanceTypes();
        if (syntheticArgumentTypes != null) {
          argCount += syntheticArgumentTypes.length;
        }

        SyntheticArgumentBinding[] syntheticArguments = declaringClass.syntheticOuterLocalVariables();
        if (syntheticArguments != null) {
          argCount += syntheticArguments.length;
        }
      }
      if (declaringClass.isEnum()) {
        argCount += 2;
      }
      argCount += methodBinding.parameters.length;
      if (this.currentFrame.numberOfStackItems >= argCount + 1) {
        this.currentFrame.numberOfStackItems -= argCount;
        this.currentFrame.initializeReceiver();
        this.currentFrame.numberOfStackItems -= 1;
      }
    } else {
      argCount = 1;
      argCount += methodBinding.parameters.length;
      if (this.currentFrame.numberOfStackItems >= argCount) {
        this.currentFrame.numberOfStackItems -= argCount;
      }
      if (methodBinding.returnType != TypeBinding.VOID)
        this.currentFrame.addStackItem(methodBinding.returnType);
    }
  }

  public void invokestatic(MethodBinding methodBinding) {
    super.invokestatic(methodBinding);
    int parametersLength = methodBinding.parameters.length;
    if (this.currentFrame.numberOfStackItems >= parametersLength) {
      this.currentFrame.numberOfStackItems -= parametersLength;
    }
    if (methodBinding.returnType != TypeBinding.VOID)
      this.currentFrame.addStackItem(methodBinding.returnType);
  }

  public void invokeStringConcatenationAppendForType(int typeID) {
    super.invokeStringConcatenationAppendForType(typeID);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void invokeStringConcatenationDefaultConstructor()
  {
    super.invokeStringConcatenationDefaultConstructor();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void invokeStringConcatenationStringConstructor() { super.invokeStringConcatenationStringConstructor();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.initializeReceiver();
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void invokeStringConcatenationToString() {
    super.invokeStringConcatenationToString();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(11, ConstantPool.JavaLangStringConstantPoolName);
  }

  public void invokeStringValueOf(int typeID) {
    super.invokeStringValueOf(typeID);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(11, ConstantPool.JavaLangStringConstantPoolName);
  }

  public void invokeSystemArraycopy() {
    super.invokeSystemArraycopy();
    if (this.currentFrame.numberOfStackItems >= 5)
      this.currentFrame.numberOfStackItems -= 5;
  }

  public void invokeThrowableGetMessage() {
    super.invokeThrowableGetMessage();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(11, ConstantPool.JavaLangStringConstantPoolName);
  }

  public void invokevirtual(MethodBinding methodBinding) {
    super.invokevirtual(methodBinding);
    int argCount = 1;
    argCount += methodBinding.parameters.length;
    if (this.currentFrame.numberOfStackItems >= argCount) {
      this.currentFrame.numberOfStackItems -= argCount;
    }
    if (methodBinding.returnType != TypeBinding.VOID)
      this.currentFrame.addStackItem(methodBinding.returnType);
  }

  public void ior() {
    super.ior();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void irem() {
    super.irem();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ireturn() {
    super.ireturn();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position));
  }
  public void ishl() {
    super.ishl();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ishr() {
    super.ishr();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void istore(int iArg) {
    super.istore(iArg);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(iArg, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void istore_0() { super.istore_0();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(0, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void istore_1() {
    super.istore_1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(1, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void istore_2() { super.istore_2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(2, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void istore_3() {
    super.istore_3();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(3, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void isub() { super.isub();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void iushr()
  {
    super.iushr();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void ixor() {
    super.ixor();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void l2d() {
    super.l2d();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.DOUBLE);
  }

  public void l2f() {
    super.l2f();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.FLOAT);
  }

  public void l2i() {
    super.l2i();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
  }

  public void ladd() {
    super.ladd();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void laload() {
    super.laload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void land() { super.land();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void lastore()
  {
    super.lastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3;
  }

  public void lcmp() {
    super.lcmp();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.stackItems[(this.currentFrame.numberOfStackItems - 1)] = new VerificationTypeInfo(TypeBinding.INT);
    }
  }

  public void lconst_0() { super.lconst_0();
    this.currentFrame.addStackItem(TypeBinding.LONG); }

  public void lconst_1() {
    super.lconst_1();
    this.currentFrame.addStackItem(TypeBinding.LONG);
  }
  public void ldc(float constant) {
    super.ldc(constant);
    this.currentFrame.addStackItem(TypeBinding.FLOAT);
  }
  public void ldc(int constant) {
    super.ldc(constant);
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void ldc(TypeBinding typeBinding) {
    super.ldc(typeBinding);
    this.currentFrame.addStackItem(typeBinding);
  }
  public void ldc2_w(double constant) {
    super.ldc2_w(constant);
    this.currentFrame.addStackItem(TypeBinding.DOUBLE);
  }
  public void ldc2_w(long constant) {
    super.ldc2_w(constant);
    this.currentFrame.addStackItem(TypeBinding.LONG);
  }
  public void ldcForIndex(int index, char[] constant) {
    super.ldcForIndex(index, constant);
    this.currentFrame.addStackItem(new VerificationTypeInfo(11, ConstantPool.JavaLangStringConstantPoolName));
  }
  public void ldiv() {
    super.ldiv();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lload(int iArg) {
    super.lload(iArg);
    this.currentFrame.addStackItem(getLocal(iArg, this.currentFrame));
  }
  public void lload_0() {
    super.lload_0();
    this.currentFrame.addStackItem(getLocal(0, this.currentFrame));
  }
  public void lload_1() {
    super.lload_1();
    this.currentFrame.addStackItem(getLocal(1, this.currentFrame));
  }
  public void lload_2() {
    super.lload_2();
    this.currentFrame.addStackItem(getLocal(2, this.currentFrame));
  }
  public void lload_3() {
    super.lload_3();
    this.currentFrame.addStackItem(getLocal(3, this.currentFrame));
  }
  public void lmul() {
    super.lmul();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lookupswitch(CaseLabel defaultLabel, int[] keys, int[] sortedIndexes, CaseLabel[] casesLabel) {
    super.lookupswitch(defaultLabel, keys, sortedIndexes, casesLabel);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lor() {
    super.lor();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lrem() {
    super.lrem();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lreturn() {
    super.lreturn();
    if (this.currentFrame.numberOfStackItems >= 1) {
      this.currentFrame.numberOfStackItems -= 1;
    }
    this.framePositions.add(new Integer(this.position));
  }
  public void lshl() {
    super.lshl();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lshr() {
    super.lshr();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lstore(int iArg) {
    super.lstore(iArg);
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(iArg, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void lstore_0() { super.lstore_0();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(0, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void lstore_1() {
    super.lstore_1();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(1, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void lstore_2() { super.lstore_2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(2, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    } }

  public void lstore_3() {
    super.lstore_3();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1) {
      this.currentFrame.putLocal(3, this.currentFrame.stackItems[(numberOfStackItems - 1)]);
      this.currentFrame.numberOfStackItems -= 1;
    }
  }

  public void lsub() { super.lsub();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void lushr()
  {
    super.lushr();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void lxor() {
    super.lxor();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void monitorenter() {
    super.monitorenter();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void monitorexit() {
    super.monitorexit();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void multianewarray(TypeBinding typeBinding, int dimensions) {
    super.multianewarray(typeBinding, dimensions);
    if (this.currentFrame.numberOfStackItems >= dimensions) {
      this.currentFrame.numberOfStackItems -= dimensions;
    }
    char[] brackets = new char[dimensions];
    for (int i = dimensions - 1; i >= 0; i--) brackets[i] = '[';
    char[] constantPoolName = CharOperation.concat(brackets, typeBinding.constantPoolName());
    this.currentFrame.addStackItem(new VerificationTypeInfo(typeBinding.id, constantPoolName));
  }

  public void new_(TypeBinding typeBinding) {
    int pc = this.position;
    super.new_(typeBinding);
    VerificationTypeInfo verificationTypeInfo = new VerificationTypeInfo(8, typeBinding);
    verificationTypeInfo.offset = pc;
    this.currentFrame.addStackItem(verificationTypeInfo);
  }
  public void newarray(int array_Type) {
    super.newarray(array_Type);
    char[] constantPoolName = (char[])null;
    switch (array_Type) {
    case 10:
      constantPoolName = new char[] { '[', 'I' };
      break;
    case 8:
      constantPoolName = new char[] { '[', 'B' };
      break;
    case 4:
      constantPoolName = new char[] { '[', 'Z' };
      break;
    case 9:
      constantPoolName = new char[] { '[', 'S' };
      break;
    case 5:
      constantPoolName = new char[] { '[', 'C' };
      break;
    case 11:
      constantPoolName = new char[] { '[', 'J' };
      break;
    case 6:
      constantPoolName = new char[] { '[', 'F' };
      break;
    case 7:
      constantPoolName = new char[] { '[', 'D' };
    }

    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      this.currentFrame.stackItems[(numberOfStackItems - 1)] = new VerificationTypeInfo(1, constantPoolName);
  }

  public void newJavaLangAssertionError() {
    int pc = this.position;
    super.newJavaLangAssertionError();
    VerificationTypeInfo verificationTypeInfo = new VerificationTypeInfo(35, 8, ConstantPool.JavaLangAssertionErrorConstantPoolName);
    verificationTypeInfo.offset = pc;
    this.currentFrame.addStackItem(verificationTypeInfo);
  }
  public void newJavaLangError() {
    int pc = this.position;
    super.newJavaLangError();
    VerificationTypeInfo verificationTypeInfo = new VerificationTypeInfo(19, 8, ConstantPool.JavaLangErrorConstantPoolName);
    verificationTypeInfo.offset = pc;
    this.currentFrame.addStackItem(verificationTypeInfo);
  }
  public void newNoClassDefFoundError() {
    int pc = this.position;
    super.newNoClassDefFoundError();
    VerificationTypeInfo verificationTypeInfo = new VerificationTypeInfo(22, 8, ConstantPool.JavaLangNoClassDefFoundErrorConstantPoolName);
    verificationTypeInfo.offset = pc;
    this.currentFrame.addStackItem(verificationTypeInfo);
  }
  public void newStringContatenation() {
    int pc = this.position;
    super.newStringContatenation();

    VerificationTypeInfo verificationTypeInfo = new VerificationTypeInfo(40, 8, ConstantPool.JavaLangStringBuilderConstantPoolName);
    verificationTypeInfo.offset = pc;
    this.currentFrame.addStackItem(verificationTypeInfo);
  }
  public void newWrapperFor(int typeID) {
    int pc = this.position;
    super.newWrapperFor(typeID);
    VerificationTypeInfo verificationTypeInfo = null;
    switch (typeID) {
    case 10:
      verificationTypeInfo = new VerificationTypeInfo(29, 8, ConstantPool.JavaLangIntegerConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 5:
      verificationTypeInfo = new VerificationTypeInfo(33, 8, ConstantPool.JavaLangBooleanConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 3:
      verificationTypeInfo = new VerificationTypeInfo(26, 8, ConstantPool.JavaLangByteConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 2:
      verificationTypeInfo = new VerificationTypeInfo(28, 8, ConstantPool.JavaLangCharacterConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 9:
      verificationTypeInfo = new VerificationTypeInfo(31, 8, ConstantPool.JavaLangFloatConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 8:
      verificationTypeInfo = new VerificationTypeInfo(32, 8, ConstantPool.JavaLangDoubleConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 4:
      verificationTypeInfo = new VerificationTypeInfo(27, 8, ConstantPool.JavaLangShortConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 7:
      verificationTypeInfo = new VerificationTypeInfo(30, 8, ConstantPool.JavaLangLongConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
      break;
    case 6:
      verificationTypeInfo = new VerificationTypeInfo(34, 8, ConstantPool.JavaLangVoidConstantPoolName);
      this.currentFrame.addStackItem(verificationTypeInfo);
    }
    if (verificationTypeInfo != null)
      verificationTypeInfo.offset = pc;
  }

  public void optimizeBranch(int oldPosition, BranchLabel lbl) {
    super.optimizeBranch(oldPosition, lbl);
    int frameIndex = this.frames.size() - 1;
    while (frameIndex > 0) {
      StackMapFrame frame = (StackMapFrame)this.frames.get(frameIndex);
      if (frame.pc == oldPosition) {
        if (this.framePositions.remove(new Integer(oldPosition))) {
          this.framePositions.add(new Integer(this.position));
        }
        if (this.variablesModificationsPositions.remove(new Integer(oldPosition))) {
          this.variablesModificationsPositions.add(new Integer(this.position));
        }
        frame.pc = this.position;
        StackMapFrame previousFrame = (StackMapFrame)this.frames.get(frameIndex - 1);
        if (previousFrame.pc != this.position)
          break;
        this.frames.set(frameIndex - 1, frame);
        this.frames.remove(frameIndex);

        break;
      }if (frame.pc > oldPosition) {
        return;
      }
      frameIndex--;
    }
  }

  public void pop() { super.pop();
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1; }

  public void pop2()
  {
    super.pop2();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 1)
      switch (this.currentFrame.stackItems[(numberOfStackItems - 1)].id()) {
      case 7:
      case 8:
        this.currentFrame.numberOfStackItems -= 1;
        break;
      default:
        if (numberOfStackItems >= 2)
          this.currentFrame.numberOfStackItems -= 2;
        break;
      }
  }

  public void popStateIndex() {
    this.stateIndexesCounter -= 1;
  }
  public void pushOnStack(TypeBinding binding) {
    super.pushOnStack(binding);
    this.currentFrame.addStackItem(binding);
  }
  public void putfield(FieldBinding fieldBinding) {
    super.putfield(fieldBinding);
    if (this.currentFrame.numberOfStackItems >= 2)
      this.currentFrame.numberOfStackItems -= 2;
  }

  public void pushStateIndex(int naturalExitMergeInitStateIndex)
  {
    if (this.stateIndexes == null) {
      this.stateIndexes = new int[3];
    }
    int length = this.stateIndexes.length;
    if (length == this.stateIndexesCounter)
    {
      System.arraycopy(this.stateIndexes, 0, this.stateIndexes = new int[length * 2], 0, length);
    }
    this.stateIndexes[(this.stateIndexesCounter++)] = naturalExitMergeInitStateIndex;
  }
  public void putstatic(FieldBinding fieldBinding) {
    super.putstatic(fieldBinding);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void recordExpressionType(TypeBinding typeBinding) {
    super.recordExpressionType(typeBinding);
    this.currentFrame.setTopOfStack(typeBinding);
  }
  public void removeVariable(LocalVariableBinding localBinding) {
    this.currentFrame.removeLocals(localBinding.resolvedPosition);
    super.removeVariable(localBinding);
  }
  public void removeNotDefinitelyAssignedVariables(Scope scope, int initStateIndex) {
    int index = this.visibleLocalsCount;
    for (int i = 0; i < index; i++) {
      LocalVariableBinding localBinding = this.visibleLocals[i];
      if ((localBinding != null) && (localBinding.initializationCount > 0)) {
        boolean isDefinitelyAssigned = isDefinitelyAssigned(scope, initStateIndex, localBinding);
        if (!isDefinitelyAssigned) {
          if (this.stateIndexes != null) {
            int j = 0; for (int max = this.stateIndexesCounter; j < max; j++) {
              if (isDefinitelyAssigned(scope, this.stateIndexes[j], localBinding)) {
                break;
              }
            }
          }
          this.currentFrame.removeLocals(localBinding.resolvedPosition);
          localBinding.recordInitializationEndPC(this.position);
        }
      }
    }
    Integer newValue = new Integer(this.position);
    if ((this.variablesModificationsPositions.size() == 0) || (!this.variablesModificationsPositions.get(this.variablesModificationsPositions.size() - 1).equals(newValue))) {
      this.variablesModificationsPositions.add(newValue);
    }
    storeStackMapFrame();
  }
  public void storeStackMapFrame() {
    int frameSize = this.frames.size();
    StackMapFrame mapFrame = null;
    try {
      mapFrame = (StackMapFrame)this.currentFrame.clone();
      mapFrame.pc = this.position;
    }
    catch (CloneNotSupportedException localCloneNotSupportedException) {
    }
    if (frameSize == 0) {
      this.frames.add(mapFrame);
    } else {
      StackMapFrame lastFrame = (StackMapFrame)this.frames.get(frameSize - 1);
      if (lastFrame.pc == this.position)
        this.frames.set(frameSize - 1, mapFrame);
      else
        this.frames.add(mapFrame);
    }
  }

  public void return_() {
    super.return_();
    this.framePositions.add(new Integer(this.position));
  }
  public void saload() {
    super.saload();
    if (this.currentFrame.numberOfStackItems >= 2) {
      this.currentFrame.numberOfStackItems -= 1;
      this.currentFrame.replaceWithElementType();
    }
  }

  public void sastore() { super.sastore();
    if (this.currentFrame.numberOfStackItems >= 3)
      this.currentFrame.numberOfStackItems -= 3; }

  public void sipush(int s)
  {
    super.sipush(s);
    this.currentFrame.addStackItem(TypeBinding.INT);
  }
  public void store(LocalVariableBinding localBinding, boolean valueRequired) {
    super.store(localBinding, valueRequired);
    TypeBinding typeBinding = localBinding.type;

    this.currentFrame.locals[localBinding.resolvedPosition] = new VerificationTypeInfo(typeBinding);
  }

  public void swap() {
    super.swap();
    int numberOfStackItems = this.currentFrame.numberOfStackItems;
    if (numberOfStackItems >= 2)
      try {
        VerificationTypeInfo info = (VerificationTypeInfo)this.currentFrame.stackItems[(numberOfStackItems - 1)].clone();
        VerificationTypeInfo info2 = (VerificationTypeInfo)this.currentFrame.stackItems[(numberOfStackItems - 2)].clone();
        this.currentFrame.stackItems[(numberOfStackItems - 1)] = info2;
        this.currentFrame.stackItems[(numberOfStackItems - 2)] = info;
      }
      catch (CloneNotSupportedException localCloneNotSupportedException) {
      }
  }

  public void tableswitch(CaseLabel defaultLabel, int low, int high, int[] keys, int[] sortedIndexes, CaseLabel[] casesLabel) {
    super.tableswitch(defaultLabel, low, high, keys, sortedIndexes, casesLabel);
    if (this.currentFrame.numberOfStackItems >= 1)
      this.currentFrame.numberOfStackItems -= 1;
  }

  public void throwAnyException(LocalVariableBinding anyExceptionVariable) {
    this.currentFrame.putLocal(anyExceptionVariable.resolvedPosition, new VerificationTypeInfo(7, anyExceptionVariable.type));
    super.throwAnyException(anyExceptionVariable);
    this.currentFrame.removeLocals(anyExceptionVariable.resolvedPosition);
  }
  public void removeStackFrameFor(int pos) {
  }

  public void reset(ClassFile givenClassFile) {
    super.reset(givenClassFile);
    this.frames = null;
    this.currentFrame = null;
    this.framePositions = null;
    this.variablesModificationsPositions = null;
  }
  protected void writePosition(BranchLabel label) {
    super.writePosition(label);
    this.framePositions.add(new Integer(label.position));
  }
  protected void writeWidePosition(BranchLabel label) {
    super.writeWidePosition(label);
    this.framePositions.add(new Integer(label.position));
  }
  protected void writePosition(BranchLabel label, int forwardReference) {
    super.writePosition(label, forwardReference);
    this.framePositions.add(new Integer(label.position));
  }
}