package org.eclipse.jdt.internal.compiler.env;

import org.eclipse.jdt.core.compiler.CharOperation;

public class AccessRuleSet
{
  private AccessRule[] accessRules;
  public String[] messageTemplates;
  public static final int MESSAGE_TEMPLATES_LENGTH = 4;

  public AccessRuleSet(AccessRule[] accessRules, String[] messageTemplates)
  {
    this.accessRules = accessRules;
    if ((messageTemplates != null) && (messageTemplates.length == 4))
      this.messageTemplates = messageTemplates;
    else
      this.messageTemplates = new String[] { "{0}", "{0}", "{0} {1}", "{0} {1}" };
  }

  public boolean equals(Object object)
  {
    if (this == object)
      return true;
    if (!(object instanceof AccessRuleSet))
      return false;
    AccessRuleSet otherRuleSet = (AccessRuleSet)object;
    if ((this.messageTemplates.length != 4) || 
      (otherRuleSet.messageTemplates.length != 4))
      return false;
    for (int i = 0; i < 4; i++)
      if (!this.messageTemplates[i].equals(otherRuleSet.messageTemplates[i]))
        return false;
    int rulesLength = this.accessRules.length;
    if (rulesLength != otherRuleSet.accessRules.length) return false;
    for (int i = 0; i < rulesLength; i++)
      if (!this.accessRules[i].equals(otherRuleSet.accessRules[i]))
        return false;
    return true;
  }

  public AccessRule[] getAccessRules() {
    return this.accessRules;
  }

  public AccessRestriction getViolatedRestriction(char[] targetTypeFilePath)
  {
    int i = 0; for (int length = this.accessRules.length; i < length; i++) {
      AccessRule accessRule = this.accessRules[i];
      if (CharOperation.pathMatch(accessRule.pattern, targetTypeFilePath, 
        true, '/')) {
        switch (accessRule.getProblemId()) {
        case 16777496:
        case 16777523:
          return new AccessRestriction(accessRule, this.messageTemplates);
        }
        return null;
      }
    }

    return null;
  }

  public String toString() {
    return toString(true);
  }

  public String toString(boolean wrap) {
    StringBuffer buffer = new StringBuffer(200);
    buffer.append("AccessRuleSet {");
    if (wrap)
      buffer.append('\n');
    int i = 0; for (int length = this.accessRules.length; i < length; i++) {
      if (wrap)
        buffer.append('\t');
      AccessRule accessRule = this.accessRules[i];
      buffer.append(accessRule);
      if (wrap)
        buffer.append('\n');
      else if (i < length - 1)
        buffer.append(", ");
    }
    buffer.append("} [templates:\"");
    for (int i = 0; i < this.messageTemplates.length; i++)
      buffer.append(this.messageTemplates[i]);
    buffer.append("\"]");
    return buffer.toString();
  }
}