package org.eclipse.jdt.internal.compiler.impl;

import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.Compiler;
import org.eclipse.jdt.internal.compiler.util.Util;

public class CompilerOptions
{
  public static final String OPTION_LocalVariableAttribute = "org.eclipse.jdt.core.compiler.debug.localVariable";
  public static final String OPTION_LineNumberAttribute = "org.eclipse.jdt.core.compiler.debug.lineNumber";
  public static final String OPTION_SourceFileAttribute = "org.eclipse.jdt.core.compiler.debug.sourceFile";
  public static final String OPTION_PreserveUnusedLocal = "org.eclipse.jdt.core.compiler.codegen.unusedLocal";
  public static final String OPTION_DocCommentSupport = "org.eclipse.jdt.core.compiler.doc.comment.support";
  public static final String OPTION_ReportMethodWithConstructorName = "org.eclipse.jdt.core.compiler.problem.methodWithConstructorName";
  public static final String OPTION_ReportOverridingPackageDefaultMethod = "org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod";
  public static final String OPTION_ReportDeprecation = "org.eclipse.jdt.core.compiler.problem.deprecation";
  public static final String OPTION_ReportDeprecationInDeprecatedCode = "org.eclipse.jdt.core.compiler.problem.deprecationInDeprecatedCode";
  public static final String OPTION_ReportDeprecationWhenOverridingDeprecatedMethod = "org.eclipse.jdt.core.compiler.problem.deprecationWhenOverridingDeprecatedMethod";
  public static final String OPTION_ReportHiddenCatchBlock = "org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock";
  public static final String OPTION_ReportUnusedLocal = "org.eclipse.jdt.core.compiler.problem.unusedLocal";
  public static final String OPTION_ReportUnusedParameter = "org.eclipse.jdt.core.compiler.problem.unusedParameter";
  public static final String OPTION_ReportUnusedParameterWhenImplementingAbstract = "org.eclipse.jdt.core.compiler.problem.unusedParameterWhenImplementingAbstract";
  public static final String OPTION_ReportUnusedParameterWhenOverridingConcrete = "org.eclipse.jdt.core.compiler.problem.unusedParameterWhenOverridingConcrete";
  public static final String OPTION_ReportUnusedParameterIncludeDocCommentReference = "org.eclipse.jdt.core.compiler.problem.unusedParameterIncludeDocCommentReference";
  public static final String OPTION_ReportUnusedImport = "org.eclipse.jdt.core.compiler.problem.unusedImport";
  public static final String OPTION_ReportSyntheticAccessEmulation = "org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation";
  public static final String OPTION_ReportNoEffectAssignment = "org.eclipse.jdt.core.compiler.problem.noEffectAssignment";
  public static final String OPTION_ReportLocalVariableHiding = "org.eclipse.jdt.core.compiler.problem.localVariableHiding";
  public static final String OPTION_ReportSpecialParameterHidingField = "org.eclipse.jdt.core.compiler.problem.specialParameterHidingField";
  public static final String OPTION_ReportFieldHiding = "org.eclipse.jdt.core.compiler.problem.fieldHiding";
  public static final String OPTION_ReportTypeParameterHiding = "org.eclipse.jdt.core.compiler.problem.typeParameterHiding";
  public static final String OPTION_ReportPossibleAccidentalBooleanAssignment = "org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment";
  public static final String OPTION_ReportNonExternalizedStringLiteral = "org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral";
  public static final String OPTION_ReportIncompatibleNonInheritedInterfaceMethod = "org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod";
  public static final String OPTION_ReportUnusedPrivateMember = "org.eclipse.jdt.core.compiler.problem.unusedPrivateMember";
  public static final String OPTION_ReportNoImplicitStringConversion = "org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion";
  public static final String OPTION_ReportAssertIdentifier = "org.eclipse.jdt.core.compiler.problem.assertIdentifier";
  public static final String OPTION_ReportEnumIdentifier = "org.eclipse.jdt.core.compiler.problem.enumIdentifier";
  public static final String OPTION_ReportNonStaticAccessToStatic = "org.eclipse.jdt.core.compiler.problem.staticAccessReceiver";
  public static final String OPTION_ReportIndirectStaticAccess = "org.eclipse.jdt.core.compiler.problem.indirectStaticAccess";
  public static final String OPTION_ReportEmptyStatement = "org.eclipse.jdt.core.compiler.problem.emptyStatement";
  public static final String OPTION_ReportUnnecessaryTypeCheck = "org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck";
  public static final String OPTION_ReportUnnecessaryElse = "org.eclipse.jdt.core.compiler.problem.unnecessaryElse";
  public static final String OPTION_ReportUndocumentedEmptyBlock = "org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock";
  public static final String OPTION_ReportInvalidJavadoc = "org.eclipse.jdt.core.compiler.problem.invalidJavadoc";
  public static final String OPTION_ReportInvalidJavadocTags = "org.eclipse.jdt.core.compiler.problem.invalidJavadocTags";
  public static final String OPTION_ReportInvalidJavadocTagsDeprecatedRef = "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsDeprecatedRef";
  public static final String OPTION_ReportInvalidJavadocTagsNotVisibleRef = "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsNotVisibleRef";
  public static final String OPTION_ReportInvalidJavadocTagsVisibility = "org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsVisibility";
  public static final String OPTION_ReportMissingJavadocTags = "org.eclipse.jdt.core.compiler.problem.missingJavadocTags";
  public static final String OPTION_ReportMissingJavadocTagsVisibility = "org.eclipse.jdt.core.compiler.problem.missingJavadocTagsVisibility";
  public static final String OPTION_ReportMissingJavadocTagsOverriding = "org.eclipse.jdt.core.compiler.problem.missingJavadocTagsOverriding";
  public static final String OPTION_ReportMissingJavadocComments = "org.eclipse.jdt.core.compiler.problem.missingJavadocComments";
  public static final String OPTION_ReportMissingJavadocCommentsVisibility = "org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsVisibility";
  public static final String OPTION_ReportMissingJavadocCommentsOverriding = "org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsOverriding";
  public static final String OPTION_ReportFinallyBlockNotCompletingNormally = "org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally";
  public static final String OPTION_ReportUnusedDeclaredThrownException = "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException";
  public static final String OPTION_ReportUnusedDeclaredThrownExceptionWhenOverriding = "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownExceptionWhenOverriding";
  public static final String OPTION_ReportUnqualifiedFieldAccess = "org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess";
  public static final String OPTION_ReportUncheckedTypeOperation = "org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation";
  public static final String OPTION_ReportRawTypeReference = "org.eclipse.jdt.core.compiler.problem.rawTypeReference";
  public static final String OPTION_ReportFinalParameterBound = "org.eclipse.jdt.core.compiler.problem.finalParameterBound";
  public static final String OPTION_ReportMissingSerialVersion = "org.eclipse.jdt.core.compiler.problem.missingSerialVersion";
  public static final String OPTION_ReportVarargsArgumentNeedCast = "org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast";
  public static final String OPTION_Source = "org.eclipse.jdt.core.compiler.source";
  public static final String OPTION_TargetPlatform = "org.eclipse.jdt.core.compiler.codegen.targetPlatform";
  public static final String OPTION_Compliance = "org.eclipse.jdt.core.compiler.compliance";
  public static final String OPTION_Encoding = "org.eclipse.jdt.core.encoding";
  public static final String OPTION_MaxProblemPerUnit = "org.eclipse.jdt.core.compiler.maxProblemPerUnit";
  public static final String OPTION_TaskTags = "org.eclipse.jdt.core.compiler.taskTags";
  public static final String OPTION_TaskPriorities = "org.eclipse.jdt.core.compiler.taskPriorities";
  public static final String OPTION_TaskCaseSensitive = "org.eclipse.jdt.core.compiler.taskCaseSensitive";
  public static final String OPTION_InlineJsr = "org.eclipse.jdt.core.compiler.codegen.inlineJsrBytecode";
  public static final String OPTION_ReportNullReference = "org.eclipse.jdt.core.compiler.problem.nullReference";
  public static final String OPTION_ReportPotentialNullReference = "org.eclipse.jdt.core.compiler.problem.potentialNullReference";
  public static final String OPTION_ReportRedundantNullCheck = "org.eclipse.jdt.core.compiler.problem.redundantNullCheck";
  public static final String OPTION_ReportAutoboxing = "org.eclipse.jdt.core.compiler.problem.autoboxing";
  public static final String OPTION_ReportAnnotationSuperInterface = "org.eclipse.jdt.core.compiler.problem.annotationSuperInterface";
  public static final String OPTION_ReportMissingOverrideAnnotation = "org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation";
  public static final String OPTION_ReportMissingDeprecatedAnnotation = "org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation";
  public static final String OPTION_ReportIncompleteEnumSwitch = "org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch";
  public static final String OPTION_ReportForbiddenReference = "org.eclipse.jdt.core.compiler.problem.forbiddenReference";
  public static final String OPTION_ReportDiscouragedReference = "org.eclipse.jdt.core.compiler.problem.discouragedReference";
  public static final String OPTION_SuppressWarnings = "org.eclipse.jdt.core.compiler.problem.suppressWarnings";
  public static final String OPTION_ReportUnhandledWarningToken = "org.eclipse.jdt.core.compiler.problem.unhandledWarningToken";
  public static final String OPTION_ReportUnusedLabel = "org.eclipse.jdt.core.compiler.problem.unusedLabel";
  public static final String OPTION_FatalOptionalError = "org.eclipse.jdt.core.compiler.problem.fatalOptionalError";
  public static final String OPTION_ReportParameterAssignment = "org.eclipse.jdt.core.compiler.problem.parameterAssignment";
  public static final String OPTION_ReportFallthroughCase = "org.eclipse.jdt.core.compiler.problem.fallthroughCase";
  public static final String OPTION_ReportOverridingMethodWithoutSuperInvocation = "org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation";
  public static final String OPTION_GenerateClassFiles = "org.eclipse.jdt.core.compiler.generateClassFiles";
  public static final String OPTION_Process_Annotations = "org.eclipse.jdt.core.compiler.processAnnotations";
  public static final String OPTION_ReportInvalidAnnotation = "org.eclipse.jdt.core.compiler.problem.invalidAnnotation";
  public static final String OPTION_ReportMissingAnnotation = "org.eclipse.jdt.core.compiler.problem.missingAnnotation";
  public static final String OPTION_ReportMissingJavadoc = "org.eclipse.jdt.core.compiler.problem.missingJavadoc";
  public static final String GENERATE = "generate";
  public static final String DO_NOT_GENERATE = "do not generate";
  public static final String PRESERVE = "preserve";
  public static final String OPTIMIZE_OUT = "optimize out";
  public static final String VERSION_1_1 = "1.1";
  public static final String VERSION_1_2 = "1.2";
  public static final String VERSION_1_3 = "1.3";
  public static final String VERSION_1_4 = "1.4";
  public static final String VERSION_JSR14 = "jsr14";
  public static final String VERSION_1_5 = "1.5";
  public static final String VERSION_1_6 = "1.6";
  public static final String VERSION_1_7 = "1.7";
  public static final String ERROR = "error";
  public static final String WARNING = "warning";
  public static final String IGNORE = "ignore";
  public static final String ENABLED = "enabled";
  public static final String DISABLED = "disabled";
  public static final String PUBLIC = "public";
  public static final String PROTECTED = "protected";
  public static final String DEFAULT = "default";
  public static final String PRIVATE = "private";
  public static final long MethodWithConstructorName = 1L;
  public static final long OverriddenPackageDefaultMethod = 2L;
  public static final long UsingDeprecatedAPI = 4L;
  public static final long MaskedCatchBlock = 8L;
  public static final long UnusedLocalVariable = 16L;
  public static final long UnusedArgument = 32L;
  public static final long NoImplicitStringConversion = 64L;
  public static final long AccessEmulation = 128L;
  public static final long NonExternalizedString = 256L;
  public static final long AssertUsedAsAnIdentifier = 512L;
  public static final long UnusedImport = 1024L;
  public static final long NonStaticAccessToStatic = 2048L;
  public static final long Task = 4096L;
  public static final long NoEffectAssignment = 8192L;
  public static final long IncompatibleNonInheritedInterfaceMethod = 16384L;
  public static final long UnusedPrivateMember = 32768L;
  public static final long LocalVariableHiding = 65536L;
  public static final long FieldHiding = 131072L;
  public static final long AccidentalBooleanAssign = 262144L;
  public static final long EmptyStatement = 524288L;
  public static final long MissingJavadocComments = 1048576L;
  public static final long MissingJavadocTags = 2097152L;
  public static final long UnqualifiedFieldAccess = 4194304L;
  public static final long UnusedDeclaredThrownException = 8388608L;
  public static final long FinallyBlockNotCompleting = 16777216L;
  public static final long InvalidJavadoc = 33554432L;
  public static final long UnnecessaryTypeCheck = 67108864L;
  public static final long UndocumentedEmptyBlock = 134217728L;
  public static final long IndirectStaticAccess = 268435456L;
  public static final long UnnecessaryElse = 536870912L;
  public static final long UncheckedTypeOperation = 1073741824L;
  public static final long FinalParameterBound = 2147483648L;
  public static final long MissingSerialVersion = 4294967296L;
  public static final long EnumUsedAsAnIdentifier = 8589934592L;
  public static final long ForbiddenReference = 17179869184L;
  public static final long VarargsArgumentNeedCast = 34359738368L;
  public static final long NullReference = 68719476736L;
  public static final long AutoBoxing = 137438953472L;
  public static final long AnnotationSuperInterface = 274877906944L;
  public static final long TypeHiding = 549755813888L;
  public static final long MissingOverrideAnnotation = 1099511627776L;
  public static final long IncompleteEnumSwitch = 2199023255552L;
  public static final long MissingDeprecatedAnnotation = 4398046511104L;
  public static final long DiscouragedReference = 8796093022208L;
  public static final long UnhandledWarningToken = 17592186044416L;
  public static final long RawTypeReference = 35184372088832L;
  public static final long UnusedLabel = 70368744177664L;
  public static final long ParameterAssignment = 140737488355328L;
  public static final long FallthroughCase = 281474976710656L;
  public static final long OverridingMethodWithoutSuperInvocation = 562949953421312L;
  public static final long PotentialNullReference = 1125899906842624L;
  public static final long RedundantNullCheck = 2251799813685248L;
  private static Map OptionToIrritants;
  public long errorThreshold = 0L;

  public long warningThreshold = 132833691627103L;

  public int produceDebugAttributes = 3;

  public long complianceLevel = 3145728L;
  public long sourceLevel = 3080192L;
  public long targetJDK = 3014656L;

  public String defaultEncoding = null;

  public boolean verbose = Compiler.DEBUG;

  public boolean produceReferenceInfo = false;

  public boolean preserveAllLocalVariables = false;

  public boolean parseLiteralExpressionsAsConstants = true;

  public int maxProblemsPerUnit = 100;

  public char[][] taskTags = null;
  public char[][] taskPriorites = null;
  public boolean isTaskCaseSensitive = true;

  public boolean reportDeprecationInsideDeprecatedCode = false;
  public boolean reportDeprecationWhenOverridingDeprecatedMethod = false;

  public boolean reportUnusedParameterWhenImplementingAbstract = false;
  public boolean reportUnusedParameterWhenOverridingConcrete = false;
  public boolean reportUnusedParameterIncludeDocCommentReference = true;

  public boolean reportUnusedDeclaredThrownExceptionWhenOverriding = false;

  public boolean reportSpecialParameterHidingField = false;

  public int reportInvalidJavadocTagsVisibility = 1;
  public boolean reportInvalidJavadocTags = false;
  public boolean reportInvalidJavadocTagsDeprecatedRef = false;
  public boolean reportInvalidJavadocTagsNotVisibleRef = false;

  public int reportMissingJavadocTagsVisibility = 1;
  public boolean reportMissingJavadocTagsOverriding = false;

  public int reportMissingJavadocCommentsVisibility = 1;
  public boolean reportMissingJavadocCommentsOverriding = false;

  public boolean inlineJsrBytecode = false;

  public boolean docCommentSupport = false;

  public boolean suppressWarnings = true;

  public boolean treatOptionalErrorAsFatal = true;

  public boolean performMethodsFullRecovery = true;

  public boolean performStatementsRecovery = true;

  public boolean storeAnnotations = false;

  public boolean generateClassFiles = true;

  public boolean processAnnotations = false;

  public static final String[] warningTokens = { 
    "all", 
    "boxing", 
    "cast", 
    "dep-ann", 
    "deprecation", 
    "fallthrough", 
    "finally", 
    "hiding", 
    "incomplete-switch", 
    "nls", 
    "null", 
    "restriction", 
    "serial", 
    "static-access", 
    "super", 
    "synthetic-access", 
    "unchecked", 
    "unqualified-field-access", 
    "unused" };

  public CompilerOptions()
  {
  }

  public CompilerOptions(Map settings)
  {
    if (settings == null) return;
    set(settings);
  }

  /** @deprecated */
  public CompilerOptions(Map settings, boolean parseLiteralExpressionsAsConstants)
  {
    this(settings);
    this.parseLiteralExpressionsAsConstants = parseLiteralExpressionsAsConstants;
  }

  public Map getMap() {
    Map optionsMap = new HashMap(30);
    optionsMap.put("org.eclipse.jdt.core.compiler.debug.localVariable", (this.produceDebugAttributes & 0x4) != 0 ? "generate" : "do not generate");
    optionsMap.put("org.eclipse.jdt.core.compiler.debug.lineNumber", (this.produceDebugAttributes & 0x2) != 0 ? "generate" : "do not generate");
    optionsMap.put("org.eclipse.jdt.core.compiler.debug.sourceFile", (this.produceDebugAttributes & 0x1) != 0 ? "generate" : "do not generate");
    optionsMap.put("org.eclipse.jdt.core.compiler.codegen.unusedLocal", this.preserveAllLocalVariables ? "preserve" : "optimize out");
    optionsMap.put("org.eclipse.jdt.core.compiler.doc.comment.support", this.docCommentSupport ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.methodWithConstructorName", getSeverityString(1L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod", getSeverityString(2L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.deprecation", getSeverityString(4L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.deprecationInDeprecatedCode", this.reportDeprecationInsideDeprecatedCode ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.deprecationWhenOverridingDeprecatedMethod", this.reportDeprecationWhenOverridingDeprecatedMethod ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock", getSeverityString(8L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedLocal", getSeverityString(16L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedParameter", getSeverityString(32L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedImport", getSeverityString(1024L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation", getSeverityString(128L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.noEffectAssignment", getSeverityString(8192L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral", getSeverityString(256L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion", getSeverityString(64L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.staticAccessReceiver", getSeverityString(2048L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.indirectStaticAccess", getSeverityString(268435456L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod", getSeverityString(16384L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedPrivateMember", getSeverityString(32768L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.localVariableHiding", getSeverityString(65536L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.fieldHiding", getSeverityString(131072L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.typeParameterHiding", getSeverityString(549755813888L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment", getSeverityString(262144L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.emptyStatement", getSeverityString(524288L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.assertIdentifier", getSeverityString(512L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.enumIdentifier", getSeverityString(8589934592L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock", getSeverityString(134217728L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck", getSeverityString(67108864L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unnecessaryElse", getSeverityString(536870912L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.autoboxing", getSeverityString(137438953472L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.annotationSuperInterface", getSeverityString(274877906944L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch", getSeverityString(2199023255552L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.invalidJavadoc", getSeverityString(33554432L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsVisibility", getVisibilityString(this.reportInvalidJavadocTagsVisibility));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.invalidJavadocTags", this.reportInvalidJavadocTags ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsDeprecatedRef", this.reportInvalidJavadocTagsDeprecatedRef ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsNotVisibleRef", this.reportInvalidJavadocTagsNotVisibleRef ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocTags", getSeverityString(2097152L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocTagsVisibility", getVisibilityString(this.reportMissingJavadocTagsVisibility));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocTagsOverriding", this.reportMissingJavadocTagsOverriding ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocComments", getSeverityString(1048576L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsVisibility", getVisibilityString(this.reportMissingJavadocCommentsVisibility));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsOverriding", this.reportMissingJavadocCommentsOverriding ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally", getSeverityString(16777216L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException", getSeverityString(8388608L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownExceptionWhenOverriding", this.reportUnusedDeclaredThrownExceptionWhenOverriding ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess", getSeverityString(4194304L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation", getSeverityString(1073741824L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.rawTypeReference", getSeverityString(35184372088832L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.finalParameterBound", getSeverityString(2147483648L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingSerialVersion", getSeverityString(4294967296L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.forbiddenReference", getSeverityString(17179869184L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.discouragedReference", getSeverityString(8796093022208L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast", getSeverityString(34359738368L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation", getSeverityString(1099511627776L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation", getSeverityString(4398046511104L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch", getSeverityString(2199023255552L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedLabel", getSeverityString(70368744177664L));
    optionsMap.put("org.eclipse.jdt.core.compiler.compliance", versionFromJdkLevel(this.complianceLevel));
    optionsMap.put("org.eclipse.jdt.core.compiler.source", versionFromJdkLevel(this.sourceLevel));
    optionsMap.put("org.eclipse.jdt.core.compiler.codegen.targetPlatform", versionFromJdkLevel(this.targetJDK));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.fatalOptionalError", this.treatOptionalErrorAsFatal ? "enabled" : "disabled");
    if (this.defaultEncoding != null) {
      optionsMap.put("org.eclipse.jdt.core.encoding", this.defaultEncoding);
    }
    optionsMap.put("org.eclipse.jdt.core.compiler.taskTags", this.taskTags == null ? Util.EMPTY_STRING : new String(CharOperation.concatWith(this.taskTags, ',')));
    optionsMap.put("org.eclipse.jdt.core.compiler.taskPriorities", this.taskPriorites == null ? Util.EMPTY_STRING : new String(CharOperation.concatWith(this.taskPriorites, ',')));
    optionsMap.put("org.eclipse.jdt.core.compiler.taskCaseSensitive", this.isTaskCaseSensitive ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedParameterWhenImplementingAbstract", this.reportUnusedParameterWhenImplementingAbstract ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedParameterWhenOverridingConcrete", this.reportUnusedParameterWhenOverridingConcrete ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unusedParameterIncludeDocCommentReference", this.reportUnusedParameterIncludeDocCommentReference ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.specialParameterHidingField", this.reportSpecialParameterHidingField ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.maxProblemPerUnit", String.valueOf(this.maxProblemsPerUnit));
    optionsMap.put("org.eclipse.jdt.core.compiler.codegen.inlineJsrBytecode", this.inlineJsrBytecode ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.nullReference", getSeverityString(68719476736L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.potentialNullReference", getSeverityString(1125899906842624L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.redundantNullCheck", getSeverityString(2251799813685248L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.suppressWarnings", this.suppressWarnings ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.unhandledWarningToken", getSeverityString(17592186044416L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.parameterAssignment", getSeverityString(140737488355328L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.fallthroughCase", getSeverityString(281474976710656L));
    optionsMap.put("org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation", getSeverityString(562949953421312L));
    optionsMap.put("org.eclipse.jdt.core.compiler.generateClassFiles", this.generateClassFiles ? "enabled" : "disabled");
    optionsMap.put("org.eclipse.jdt.core.compiler.processAnnotations", this.processAnnotations ? "enabled" : "disabled");
    return optionsMap;
  }

  public static String optionKeyFromIrritant(long irritant)
  {
    int irritantInt = (int)irritant;
    if (irritantInt == irritant) {
      switch (irritantInt) {
      case 1:
        return "org.eclipse.jdt.core.compiler.problem.methodWithConstructorName";
      case 2:
        return "org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod";
      case 4:
      case 33554436:
        return "org.eclipse.jdt.core.compiler.problem.deprecation";
      case 8:
        return "org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock";
      case 16:
        return "org.eclipse.jdt.core.compiler.problem.unusedLocal";
      case 32:
        return "org.eclipse.jdt.core.compiler.problem.unusedParameter";
      case 64:
        return "org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion";
      case 128:
        return "org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation";
      case 256:
        return "org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral";
      case 512:
        return "org.eclipse.jdt.core.compiler.problem.assertIdentifier";
      case 1024:
        return "org.eclipse.jdt.core.compiler.problem.unusedImport";
      case 2048:
        return "org.eclipse.jdt.core.compiler.problem.staticAccessReceiver";
      case 4096:
        return "org.eclipse.jdt.core.compiler.taskTags";
      case 8192:
        return "org.eclipse.jdt.core.compiler.problem.noEffectAssignment";
      case 16384:
        return "org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod";
      case 32768:
        return "org.eclipse.jdt.core.compiler.problem.unusedPrivateMember";
      case 65536:
        return "org.eclipse.jdt.core.compiler.problem.localVariableHiding";
      case 131072:
        return "org.eclipse.jdt.core.compiler.problem.fieldHiding";
      case 262144:
        return "org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment";
      case 524288:
        return "org.eclipse.jdt.core.compiler.problem.emptyStatement";
      case 1048576:
        return "org.eclipse.jdt.core.compiler.problem.missingJavadocComments";
      case 2097152:
        return "org.eclipse.jdt.core.compiler.problem.missingJavadocTags";
      case 4194304:
        return "org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess";
      case 8388608:
        return "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownExceptionWhenOverriding";
      case 16777216:
        return "org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally";
      case 33554432:
        return "org.eclipse.jdt.core.compiler.problem.invalidJavadoc";
      case 67108864:
        return "org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck";
      case 134217728:
        return "org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock";
      case 268435456:
        return "org.eclipse.jdt.core.compiler.problem.indirectStaticAccess";
      case 536870912:
        return "org.eclipse.jdt.core.compiler.problem.unnecessaryElse";
      case 1073741824:
        return "org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation";
      case -2147483648:
        return "org.eclipse.jdt.core.compiler.problem.finalParameterBound";
      }
    } else {
      irritantInt = (int)(irritant >>> 32);
      switch (irritantInt) {
      case 1:
        return "org.eclipse.jdt.core.compiler.problem.missingSerialVersion";
      case 2:
        return "org.eclipse.jdt.core.compiler.problem.enumIdentifier";
      case 4:
        return "org.eclipse.jdt.core.compiler.problem.forbiddenReference";
      case 8:
        return "org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast";
      case 16:
        return "org.eclipse.jdt.core.compiler.problem.nullReference";
      case 262144:
        return "org.eclipse.jdt.core.compiler.problem.potentialNullReference";
      case 524288:
        return "org.eclipse.jdt.core.compiler.problem.redundantNullCheck";
      case 32:
        return "org.eclipse.jdt.core.compiler.problem.autoboxing";
      case 64:
        return "org.eclipse.jdt.core.compiler.problem.annotationSuperInterface";
      case 128:
        return "org.eclipse.jdt.core.compiler.problem.typeParameterHiding";
      case 256:
        return "org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation";
      case 512:
        return "org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch";
      case 1024:
        return "org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation";
      case 2048:
        return "org.eclipse.jdt.core.compiler.problem.discouragedReference";
      case 4096:
        return "org.eclipse.jdt.core.compiler.problem.unhandledWarningToken";
      case 8192:
        return "org.eclipse.jdt.core.compiler.problem.rawTypeReference";
      case 16384:
        return "org.eclipse.jdt.core.compiler.problem.unusedLabel";
      case 32768:
        return "org.eclipse.jdt.core.compiler.problem.parameterAssignment";
      case 65536:
        return "org.eclipse.jdt.core.compiler.problem.fallthroughCase";
      case 131072:
        return "org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation";
      }
    }
    return null;
  }

  public static long optionKeyToIrritant(String optionName) {
    if (OptionToIrritants == null) {
      long irritant = 0L;
      for (int i = 0; i < 64; i++) {
        irritant <<= 1;
        String optionKey = optionKeyFromIrritant(irritant);
        if (optionKey != null)
          OptionToIrritants.put(optionKey, new Long(irritant));
      }
    }
    Long irritant = (Long)OptionToIrritants.get(optionName);
    return irritant == null ? 0L : irritant.longValue();
  }

  public int getSeverity(long irritant) {
    if ((this.errorThreshold & irritant) != 0L) {
      return this.treatOptionalErrorAsFatal ? 
        161 : 
        33;
    }
    if ((this.warningThreshold & irritant) != 0L)
      return 32;
    return -1;
  }

  public String getSeverityString(long irritant) {
    if ((this.warningThreshold & irritant) != 0L)
      return "warning";
    if ((this.errorThreshold & irritant) != 0L)
      return "error";
    return "ignore";
  }

  public String getVisibilityString(int level) {
    switch (level & 0x7) {
    case 1:
      return "public";
    case 4:
      return "protected";
    case 2:
      return "private";
    case 3:
    }return "default";
  }

  public void set(Map optionsMap)
  {
    Object optionValue;
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.debug.localVariable")) != null) {
      if ("generate".equals(optionValue))
        this.produceDebugAttributes |= 4;
      else if ("do not generate".equals(optionValue)) {
        this.produceDebugAttributes &= -5;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.debug.lineNumber")) != null) {
      if ("generate".equals(optionValue))
        this.produceDebugAttributes |= 2;
      else if ("do not generate".equals(optionValue)) {
        this.produceDebugAttributes &= -3;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.debug.sourceFile")) != null) {
      if ("generate".equals(optionValue))
        this.produceDebugAttributes |= 1;
      else if ("do not generate".equals(optionValue)) {
        this.produceDebugAttributes &= -2;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.codegen.unusedLocal")) != null) {
      if ("preserve".equals(optionValue))
        this.preserveAllLocalVariables = true;
      else if ("optimize out".equals(optionValue)) {
        this.preserveAllLocalVariables = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.deprecationInDeprecatedCode")) != null) {
      if ("enabled".equals(optionValue))
        this.reportDeprecationInsideDeprecatedCode = true;
      else if ("disabled".equals(optionValue)) {
        this.reportDeprecationInsideDeprecatedCode = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.deprecationWhenOverridingDeprecatedMethod")) != null) {
      if ("enabled".equals(optionValue))
        this.reportDeprecationWhenOverridingDeprecatedMethod = true;
      else if ("disabled".equals(optionValue)) {
        this.reportDeprecationWhenOverridingDeprecatedMethod = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownExceptionWhenOverriding")) != null) {
      if ("enabled".equals(optionValue))
        this.reportUnusedDeclaredThrownExceptionWhenOverriding = true;
      else if ("disabled".equals(optionValue)) {
        this.reportUnusedDeclaredThrownExceptionWhenOverriding = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.compliance")) != null) {
      long level = versionToJdkLevel(optionValue);
      if (level != 0L) this.complianceLevel = level;
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.source")) != null) {
      long level = versionToJdkLevel(optionValue);
      if (level != 0L) this.sourceLevel = level;
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.codegen.targetPlatform")) != null) {
      long level = versionToJdkLevel(optionValue);
      if (level != 0L) this.targetJDK = level;
      if (this.targetJDK >= 3211264L) this.inlineJsrBytecode = true;
    }
    if (((optionValue = optionsMap.get("org.eclipse.jdt.core.encoding")) != null) && 
      ((optionValue instanceof String))) {
      this.defaultEncoding = null;
      String stringValue = (String)optionValue;
      if (stringValue.length() > 0) {
        try {
          new InputStreamReader(new ByteArrayInputStream(new byte[0]), stringValue);
          this.defaultEncoding = stringValue;
        }
        catch (UnsupportedEncodingException localUnsupportedEncodingException)
        {
        }
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedParameterWhenImplementingAbstract")) != null) {
      if ("enabled".equals(optionValue))
        this.reportUnusedParameterWhenImplementingAbstract = true;
      else if ("disabled".equals(optionValue)) {
        this.reportUnusedParameterWhenImplementingAbstract = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedParameterWhenOverridingConcrete")) != null) {
      if ("enabled".equals(optionValue))
        this.reportUnusedParameterWhenOverridingConcrete = true;
      else if ("disabled".equals(optionValue)) {
        this.reportUnusedParameterWhenOverridingConcrete = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedParameterIncludeDocCommentReference")) != null) {
      if ("enabled".equals(optionValue))
        this.reportUnusedParameterIncludeDocCommentReference = true;
      else if ("disabled".equals(optionValue)) {
        this.reportUnusedParameterIncludeDocCommentReference = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.specialParameterHidingField")) != null) {
      if ("enabled".equals(optionValue))
        this.reportSpecialParameterHidingField = true;
      else if ("disabled".equals(optionValue)) {
        this.reportSpecialParameterHidingField = false;
      }
    }
    if (((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.maxProblemPerUnit")) != null) && 
      ((optionValue instanceof String))) {
      String stringValue = (String)optionValue;
      try {
        int val = Integer.parseInt(stringValue);
        if (val >= 0) this.maxProblemsPerUnit = val;
      }
      catch (NumberFormatException localNumberFormatException)
      {
      }
    }
    if (((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.taskTags")) != null) && 
      ((optionValue instanceof String))) {
      String stringValue = (String)optionValue;
      if (stringValue.length() == 0)
        this.taskTags = null;
      else {
        this.taskTags = CharOperation.splitAndTrimOn(',', stringValue.toCharArray());
      }
    }

    if (((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.taskPriorities")) != null) && 
      ((optionValue instanceof String))) {
      String stringValue = (String)optionValue;
      if (stringValue.length() == 0)
        this.taskPriorites = null;
      else {
        this.taskPriorites = CharOperation.splitAndTrimOn(',', stringValue.toCharArray());
      }
    }

    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.taskCaseSensitive")) != null) {
      if ("enabled".equals(optionValue))
        this.isTaskCaseSensitive = true;
      else if ("disabled".equals(optionValue)) {
        this.isTaskCaseSensitive = false;
      }
    }
    if (((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.codegen.inlineJsrBytecode")) != null) && 
      (this.targetJDK < 3211264L)) {
      if ("enabled".equals(optionValue))
        this.inlineJsrBytecode = true;
      else if ("disabled".equals(optionValue)) {
        this.inlineJsrBytecode = false;
      }
    }

    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.suppressWarnings")) != null) {
      if ("enabled".equals(optionValue))
        this.suppressWarnings = true;
      else if ("disabled".equals(optionValue)) {
        this.suppressWarnings = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.fatalOptionalError")) != null) {
      if ("enabled".equals(optionValue))
        this.treatOptionalErrorAsFatal = true;
      else if ("disabled".equals(optionValue)) {
        this.treatOptionalErrorAsFatal = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.methodWithConstructorName")) != null) updateSeverity(1L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod")) != null) updateSeverity(2L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.deprecation")) != null) updateSeverity(4L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock")) != null) updateSeverity(8L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedLocal")) != null) updateSeverity(16L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedParameter")) != null) updateSeverity(32L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedImport")) != null) updateSeverity(1024L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedPrivateMember")) != null) updateSeverity(32768L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException")) != null) updateSeverity(8388608L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion")) != null) updateSeverity(64L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation")) != null) updateSeverity(128L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.localVariableHiding")) != null) updateSeverity(65536L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.fieldHiding")) != null) updateSeverity(131072L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.typeParameterHiding")) != null) updateSeverity(549755813888L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment")) != null) updateSeverity(262144L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.emptyStatement")) != null) updateSeverity(524288L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral")) != null) updateSeverity(256L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.assertIdentifier")) != null) updateSeverity(512L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.enumIdentifier")) != null) updateSeverity(8589934592L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.staticAccessReceiver")) != null) updateSeverity(2048L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.indirectStaticAccess")) != null) updateSeverity(268435456L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod")) != null) updateSeverity(16384L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock")) != null) updateSeverity(134217728L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck")) != null) updateSeverity(67108864L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unnecessaryElse")) != null) updateSeverity(536870912L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally")) != null) updateSeverity(16777216L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess")) != null) updateSeverity(4194304L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.noEffectAssignment")) != null) updateSeverity(8192L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation")) != null) updateSeverity(1073741824L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.rawTypeReference")) != null) updateSeverity(35184372088832L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.finalParameterBound")) != null) updateSeverity(2147483648L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingSerialVersion")) != null) updateSeverity(4294967296L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.forbiddenReference")) != null) updateSeverity(17179869184L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.discouragedReference")) != null) updateSeverity(8796093022208L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast")) != null) updateSeverity(34359738368L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.nullReference")) != null) updateSeverity(68719476736L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.potentialNullReference")) != null) updateSeverity(1125899906842624L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.redundantNullCheck")) != null) updateSeverity(2251799813685248L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.autoboxing")) != null) updateSeverity(137438953472L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.annotationSuperInterface")) != null) updateSeverity(274877906944L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation")) != null) updateSeverity(1099511627776L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation")) != null) updateSeverity(4398046511104L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch")) != null) updateSeverity(2199023255552L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unhandledWarningToken")) != null) updateSeverity(17592186044416L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.unusedLabel")) != null) updateSeverity(70368744177664L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.parameterAssignment")) != null) updateSeverity(140737488355328L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.fallthroughCase")) != null) updateSeverity(281474976710656L, optionValue);
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation")) != null) updateSeverity(562949953421312L, optionValue);

    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.doc.comment.support")) != null) {
      if ("enabled".equals(optionValue))
        this.docCommentSupport = true;
      else if ("disabled".equals(optionValue)) {
        this.docCommentSupport = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.invalidJavadoc")) != null) {
      updateSeverity(33554432L, optionValue);
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsVisibility")) != null) {
      if ("public".equals(optionValue))
        this.reportInvalidJavadocTagsVisibility = 1;
      else if ("protected".equals(optionValue))
        this.reportInvalidJavadocTagsVisibility = 4;
      else if ("default".equals(optionValue))
        this.reportInvalidJavadocTagsVisibility = 0;
      else if ("private".equals(optionValue)) {
        this.reportInvalidJavadocTagsVisibility = 2;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.invalidJavadocTags")) != null) {
      if ("enabled".equals(optionValue))
        this.reportInvalidJavadocTags = true;
      else if ("disabled".equals(optionValue)) {
        this.reportInvalidJavadocTags = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsDeprecatedRef")) != null) {
      if ("enabled".equals(optionValue))
        this.reportInvalidJavadocTagsDeprecatedRef = true;
      else if ("disabled".equals(optionValue)) {
        this.reportInvalidJavadocTagsDeprecatedRef = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.invalidJavadocTagsNotVisibleRef")) != null) {
      if ("enabled".equals(optionValue))
        this.reportInvalidJavadocTagsNotVisibleRef = true;
      else if ("disabled".equals(optionValue)) {
        this.reportInvalidJavadocTagsNotVisibleRef = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocTags")) != null) {
      updateSeverity(2097152L, optionValue);
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocTagsVisibility")) != null) {
      if ("public".equals(optionValue))
        this.reportMissingJavadocTagsVisibility = 1;
      else if ("protected".equals(optionValue))
        this.reportMissingJavadocTagsVisibility = 4;
      else if ("default".equals(optionValue))
        this.reportMissingJavadocTagsVisibility = 0;
      else if ("private".equals(optionValue)) {
        this.reportMissingJavadocTagsVisibility = 2;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocTagsOverriding")) != null) {
      if ("enabled".equals(optionValue))
        this.reportMissingJavadocTagsOverriding = true;
      else if ("disabled".equals(optionValue)) {
        this.reportMissingJavadocTagsOverriding = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocComments")) != null) {
      updateSeverity(1048576L, optionValue);
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsVisibility")) != null) {
      if ("public".equals(optionValue))
        this.reportMissingJavadocCommentsVisibility = 1;
      else if ("protected".equals(optionValue))
        this.reportMissingJavadocCommentsVisibility = 4;
      else if ("default".equals(optionValue))
        this.reportMissingJavadocCommentsVisibility = 0;
      else if ("private".equals(optionValue)) {
        this.reportMissingJavadocCommentsVisibility = 2;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.problem.missingJavadocCommentsOverriding")) != null) {
      if ("enabled".equals(optionValue))
        this.reportMissingJavadocCommentsOverriding = true;
      else if ("disabled".equals(optionValue)) {
        this.reportMissingJavadocCommentsOverriding = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.generateClassFiles")) != null) {
      if ("enabled".equals(optionValue))
        this.generateClassFiles = true;
      else if ("disabled".equals(optionValue)) {
        this.generateClassFiles = false;
      }
    }
    if ((optionValue = optionsMap.get("org.eclipse.jdt.core.compiler.processAnnotations")) != null)
      if ("enabled".equals(optionValue)) {
        this.processAnnotations = true;
        this.storeAnnotations = true;
        this.docCommentSupport = true;
      } else if ("disabled".equals(optionValue)) {
        this.processAnnotations = false;
        this.storeAnnotations = false;
      }
  }

  public String toString()
  {
    StringBuffer buf = new StringBuffer("CompilerOptions:");
    buf.append("\n\t- local variables debug attributes: ").append((this.produceDebugAttributes & 0x4) != 0 ? "ON" : " OFF");
    buf.append("\n\t- line number debug attributes: ").append((this.produceDebugAttributes & 0x2) != 0 ? "ON" : " OFF");
    buf.append("\n\t- source debug attributes: ").append((this.produceDebugAttributes & 0x1) != 0 ? "ON" : " OFF");
    buf.append("\n\t- preserve all local variables: ").append(this.preserveAllLocalVariables ? "ON" : " OFF");
    buf.append("\n\t- method with constructor name: ").append(getSeverityString(1L));
    buf.append("\n\t- overridden package default method: ").append(getSeverityString(2L));
    buf.append("\n\t- deprecation: ").append(getSeverityString(4L));
    buf.append("\n\t- masked catch block: ").append(getSeverityString(8L));
    buf.append("\n\t- unused local variable: ").append(getSeverityString(16L));
    buf.append("\n\t- unused parameter: ").append(getSeverityString(32L));
    buf.append("\n\t- unused import: ").append(getSeverityString(1024L));
    buf.append("\n\t- synthetic access emulation: ").append(getSeverityString(128L));
    buf.append("\n\t- assignment with no effect: ").append(getSeverityString(8192L));
    buf.append("\n\t- non externalized string: ").append(getSeverityString(256L));
    buf.append("\n\t- static access receiver: ").append(getSeverityString(2048L));
    buf.append("\n\t- indirect static access: ").append(getSeverityString(268435456L));
    buf.append("\n\t- incompatible non inherited interface method: ").append(getSeverityString(16384L));
    buf.append("\n\t- unused private member: ").append(getSeverityString(32768L));
    buf.append("\n\t- local variable hiding another variable: ").append(getSeverityString(65536L));
    buf.append("\n\t- field hiding another variable: ").append(getSeverityString(131072L));
    buf.append("\n\t- type hiding another type: ").append(getSeverityString(549755813888L));
    buf.append("\n\t- possible accidental boolean assignment: ").append(getSeverityString(262144L));
    buf.append("\n\t- superfluous semicolon: ").append(getSeverityString(524288L));
    buf.append("\n\t- uncommented empty block: ").append(getSeverityString(134217728L));
    buf.append("\n\t- unnecessary type check: ").append(getSeverityString(67108864L));
    buf.append("\n\t- javadoc comment support: ").append(this.docCommentSupport ? "ON" : " OFF");
    buf.append("\n\t\t+ invalid javadoc: ").append(getSeverityString(33554432L));
    buf.append("\n\t\t+ report invalid javadoc tags: ").append(this.reportInvalidJavadocTags ? "enabled" : "disabled");
    buf.append("\n\t\t\t* deprecated references: ").append(this.reportInvalidJavadocTagsDeprecatedRef ? "enabled" : "disabled");
    buf.append("\n\t\t\t* not visible references: ").append(this.reportInvalidJavadocTagsNotVisibleRef ? "enabled" : "disabled");
    buf.append("\n\t\t+ visibility level to report invalid javadoc tags: ").append(getVisibilityString(this.reportInvalidJavadocTagsVisibility));
    buf.append("\n\t\t+ missing javadoc tags: ").append(getSeverityString(2097152L));
    buf.append("\n\t\t+ visibility level to report missing javadoc tags: ").append(getVisibilityString(this.reportMissingJavadocTagsVisibility));
    buf.append("\n\t\t+ report missing javadoc tags in overriding methods: ").append(this.reportMissingJavadocTagsOverriding ? "enabled" : "disabled");
    buf.append("\n\t\t+ missing javadoc comments: ").append(getSeverityString(1048576L));
    buf.append("\n\t\t+ visibility level to report missing javadoc comments: ").append(getVisibilityString(this.reportMissingJavadocCommentsVisibility));
    buf.append("\n\t\t+ report missing javadoc comments in overriding methods: ").append(this.reportMissingJavadocCommentsOverriding ? "enabled" : "disabled");
    buf.append("\n\t- finally block not completing normally: ").append(getSeverityString(16777216L));
    buf.append("\n\t- unused declared thrown exception: ").append(getSeverityString(8388608L));
    buf.append("\n\t- unused declared thrown exception when overriding: ").append(this.reportUnusedDeclaredThrownExceptionWhenOverriding ? "enabled" : "disabled");
    buf.append("\n\t- unnecessary else: ").append(getSeverityString(536870912L));
    buf.append("\n\t- JDK compliance level: " + versionFromJdkLevel(this.complianceLevel));
    buf.append("\n\t- JDK source level: " + versionFromJdkLevel(this.sourceLevel));
    buf.append("\n\t- JDK target level: " + versionFromJdkLevel(this.targetJDK));
    buf.append("\n\t- verbose : ").append(this.verbose ? "ON" : "OFF");
    buf.append("\n\t- produce reference info : ").append(this.produceReferenceInfo ? "ON" : "OFF");
    buf.append("\n\t- parse literal expressions as constants : ").append(this.parseLiteralExpressionsAsConstants ? "ON" : "OFF");
    buf.append("\n\t- encoding : ").append(this.defaultEncoding == null ? "<default>" : this.defaultEncoding);
    buf.append("\n\t- task tags: ").append(this.taskTags == null ? Util.EMPTY_STRING : new String(CharOperation.concatWith(this.taskTags, ',')));
    buf.append("\n\t- task priorities : ").append(this.taskPriorites == null ? Util.EMPTY_STRING : new String(CharOperation.concatWith(this.taskPriorites, ',')));
    buf.append("\n\t- report deprecation inside deprecated code : ").append(this.reportDeprecationInsideDeprecatedCode ? "enabled" : "disabled");
    buf.append("\n\t- report deprecation when overriding deprecated method : ").append(this.reportDeprecationWhenOverridingDeprecatedMethod ? "enabled" : "disabled");
    buf.append("\n\t- report unused parameter when implementing abstract method : ").append(this.reportUnusedParameterWhenImplementingAbstract ? "enabled" : "disabled");
    buf.append("\n\t- report unused parameter when overriding concrete method : ").append(this.reportUnusedParameterWhenOverridingConcrete ? "enabled" : "disabled");
    buf.append("\n\t- report unused parameter include doc comment reference : ").append(this.reportUnusedParameterIncludeDocCommentReference ? "enabled" : "disabled");
    buf.append("\n\t- report constructor/setter parameter hiding existing field : ").append(this.reportSpecialParameterHidingField ? "enabled" : "disabled");
    buf.append("\n\t- inline JSR bytecode : ").append(this.inlineJsrBytecode ? "enabled" : "disabled");
    buf.append("\n\t- unsafe type operation: ").append(getSeverityString(1073741824L));
    buf.append("\n\t- unsafe raw type: ").append(getSeverityString(35184372088832L));
    buf.append("\n\t- final bound for type parameter: ").append(getSeverityString(2147483648L));
    buf.append("\n\t- missing serialVersionUID: ").append(getSeverityString(4294967296L));
    buf.append("\n\t- varargs argument need cast: ").append(getSeverityString(34359738368L));
    buf.append("\n\t- forbidden reference to type with access restriction: ").append(getSeverityString(17179869184L));
    buf.append("\n\t- discouraged reference to type with access restriction: ").append(getSeverityString(8796093022208L));
    buf.append("\n\t- null reference: ").append(getSeverityString(68719476736L));
    buf.append("\n\t- potential null reference: ").append(getSeverityString(1125899906842624L));
    buf.append("\n\t- redundant null check: ").append(getSeverityString(2251799813685248L));
    buf.append("\n\t- autoboxing: ").append(getSeverityString(137438953472L));
    buf.append("\n\t- annotation super interface: ").append(getSeverityString(274877906944L));
    buf.append("\n\t- missing @Override annotation: ").append(getSeverityString(1099511627776L));
    buf.append("\n\t- missing @Deprecated annotation: ").append(getSeverityString(4398046511104L));
    buf.append("\n\t- incomplete enum switch: ").append(getSeverityString(2199023255552L));
    buf.append("\n\t- suppress warnings: ").append(this.suppressWarnings ? "enabled" : "disabled");
    buf.append("\n\t- unhandled warning token: ").append(getSeverityString(17592186044416L));
    buf.append("\n\t- unused label: ").append(getSeverityString(70368744177664L));
    buf.append("\n\t- treat optional error as fatal: ").append(this.treatOptionalErrorAsFatal ? "enabled" : "disabled");
    buf.append("\n\t- parameter assignment: ").append(getSeverityString(140737488355328L));
    buf.append("\n\t- generate class files: ").append(this.generateClassFiles ? "enabled" : "disabled");
    buf.append("\n\t- process annotations: ").append(this.processAnnotations ? "enabled" : "disabled");
    return buf.toString();
  }

  void updateSeverity(long irritant, Object severityString) {
    if ("error".equals(severityString)) {
      this.errorThreshold |= irritant;
      this.warningThreshold &= (irritant ^ 0xFFFFFFFF);
    } else if ("warning".equals(severityString)) {
      this.errorThreshold &= (irritant ^ 0xFFFFFFFF);
      this.warningThreshold |= irritant;
    } else if ("ignore".equals(severityString)) {
      this.errorThreshold &= (irritant ^ 0xFFFFFFFF);
      this.warningThreshold &= (irritant ^ 0xFFFFFFFF);
    }
  }

  public static long versionToJdkLevel(Object versionID) { if ((versionID instanceof String)) {
      String version = (String)versionID;

      if ((version.length() == 3) && (version.charAt(0) == '1') && (version.charAt(1) == '.')) {
        switch (version.charAt(2)) {
        case '1':
          return 2949123L;
        case '2':
          return 3014656L;
        case '3':
          return 3080192L;
        case '4':
          return 3145728L;
        case '5':
          return 3211264L;
        case '6':
          return 3276800L;
        case '7':
          return 3342336L;
        }
        return 0L;
      }

      if ("jsr14".equals(versionID)) {
        return 3145728L;
      }
    }
    return 0L; }

  public static String versionFromJdkLevel(long jdkLevel)
  {
    switch ((int)(jdkLevel >> 16)) {
    case 45:
      if (jdkLevel == 2949123L)
        return "1.1";
      break;
    case 46:
      if (jdkLevel == 3014656L)
        return "1.2";
      break;
    case 47:
      if (jdkLevel == 3080192L)
        return "1.3";
      break;
    case 48:
      if (jdkLevel == 3145728L)
        return "1.4";
      break;
    case 49:
      if (jdkLevel == 3211264L)
        return "1.5";
      break;
    case 50:
      if (jdkLevel == 3276800L)
        return "1.6";
      break;
    case 51:
      if (jdkLevel == 3342336L)
        return "1.7";
      break;
    }
    return Util.EMPTY_STRING;
  }

  public static String[] warningOptionNames()
  {
    String[] result = { 
      "org.eclipse.jdt.core.compiler.problem.annotationSuperInterface", 
      "org.eclipse.jdt.core.compiler.problem.assertIdentifier", 
      "org.eclipse.jdt.core.compiler.problem.autoboxing", 
      "org.eclipse.jdt.core.compiler.problem.deprecation", 
      "org.eclipse.jdt.core.compiler.problem.discouragedReference", 
      "org.eclipse.jdt.core.compiler.problem.emptyStatement", 
      "org.eclipse.jdt.core.compiler.problem.enumIdentifier", 
      "org.eclipse.jdt.core.compiler.problem.fallthroughCase", 
      "org.eclipse.jdt.core.compiler.problem.fieldHiding", 
      "org.eclipse.jdt.core.compiler.problem.finalParameterBound", 
      "org.eclipse.jdt.core.compiler.problem.finallyBlockNotCompletingNormally", 
      "org.eclipse.jdt.core.compiler.problem.forbiddenReference", 
      "org.eclipse.jdt.core.compiler.problem.hiddenCatchBlock", 
      "org.eclipse.jdt.core.compiler.problem.incompatibleNonInheritedInterfaceMethod", 
      "org.eclipse.jdt.core.compiler.problem.incompleteEnumSwitch", 
      "org.eclipse.jdt.core.compiler.problem.indirectStaticAccess", 
      "org.eclipse.jdt.core.compiler.problem.invalidJavadoc", 
      "org.eclipse.jdt.core.compiler.problem.localVariableHiding", 
      "org.eclipse.jdt.core.compiler.problem.methodWithConstructorName", 
      "org.eclipse.jdt.core.compiler.problem.missingDeprecatedAnnotation", 
      "org.eclipse.jdt.core.compiler.problem.missingJavadocComments", 
      "org.eclipse.jdt.core.compiler.problem.missingJavadocTags", 
      "org.eclipse.jdt.core.compiler.problem.missingOverrideAnnotation", 
      "org.eclipse.jdt.core.compiler.problem.missingSerialVersion", 
      "org.eclipse.jdt.core.compiler.problem.noEffectAssignment", 
      "org.eclipse.jdt.core.compiler.problem.noImplicitStringConversion", 
      "org.eclipse.jdt.core.compiler.problem.nonExternalizedStringLiteral", 
      "org.eclipse.jdt.core.compiler.problem.staticAccessReceiver", 
      "org.eclipse.jdt.core.compiler.problem.nullReference", 
      "org.eclipse.jdt.core.compiler.problem.potentialNullReference", 
      "org.eclipse.jdt.core.compiler.problem.redundantNullCheck", 
      "org.eclipse.jdt.core.compiler.problem.overridingPackageDefaultMethod", 
      "org.eclipse.jdt.core.compiler.problem.parameterAssignment", 
      "org.eclipse.jdt.core.compiler.problem.possibleAccidentalBooleanAssignment", 
      "org.eclipse.jdt.core.compiler.problem.syntheticAccessEmulation", 
      "org.eclipse.jdt.core.compiler.problem.typeParameterHiding", 
      "org.eclipse.jdt.core.compiler.problem.uncheckedTypeOperation", 
      "org.eclipse.jdt.core.compiler.problem.undocumentedEmptyBlock", 
      "org.eclipse.jdt.core.compiler.problem.unnecessaryElse", 
      "org.eclipse.jdt.core.compiler.problem.unnecessaryTypeCheck", 
      "org.eclipse.jdt.core.compiler.problem.unqualifiedFieldAccess", 
      "org.eclipse.jdt.core.compiler.problem.unusedDeclaredThrownException", 
      "org.eclipse.jdt.core.compiler.problem.unusedImport", 
      "org.eclipse.jdt.core.compiler.problem.unusedLocal", 
      "org.eclipse.jdt.core.compiler.problem.unusedParameter", 
      "org.eclipse.jdt.core.compiler.problem.unusedPrivateMember", 
      "org.eclipse.jdt.core.compiler.problem.varargsArgumentNeedCast", 
      "org.eclipse.jdt.core.compiler.problem.unhandledWarningToken", 
      "org.eclipse.jdt.core.compiler.problem.overridingMethodWithoutSuperInvocation" };

    return result;
  }

  public static String warningTokenFromIrritant(long irritant)
  {
    int irritantInt = (int)irritant;
    if (irritantInt == irritant) {
      switch (irritantInt) {
      case 4:
      case 33554436:
        return "deprecation";
      case 16777216:
        return "finally";
      case 8:
      case 65536:
      case 131072:
        return "hiding";
      case 256:
        return "nls";
      case 67108864:
        return "cast";
      case 16:
      case 32:
      case 1024:
      case 32768:
      case 8388608:
        return "unused";
      case 2048:
      case 268435456:
        return "static-access";
      case 128:
        return "synthetic-access";
      case 4194304:
        return "unqualified-field-access";
      case 1073741824:
        return "unchecked";
      }
    } else {
      irritantInt = (int)(irritant >>> 32);
      switch (irritantInt) {
      case 1:
        return "serial";
      case 32:
        return "boxing";
      case 128:
        return "hiding";
      case 512:
        return "incomplete-switch";
      case 1024:
        return "dep-ann";
      case 8192:
        return "unchecked";
      case 16384:
        return "unused";
      case 4:
      case 2048:
        return "restriction";
      case 16:
      case 262144:
      case 524288:
        return "null";
      case 65536:
        return "fallthrough";
      case 131072:
        return "super";
      }
    }
    return null;
  }

  public static long warningTokenToIrritant(String warningToken)
  {
    if ((warningToken == null) || (warningToken.length() == 0)) return 0L;
    switch (warningToken.charAt(0)) {
    case 'a':
      if ("all".equals(warningToken))
        return -1L;
      break;
    case 'b':
      if ("boxing".equals(warningToken))
        return 137438953472L;
      break;
    case 'c':
      if ("cast".equals(warningToken))
        return 67108864L;
      break;
    case 'd':
      if ("deprecation".equals(warningToken))
        return 4L;
      if ("dep-ann".equals(warningToken))
        return 4398046511104L;
      break;
    case 'f':
      if ("fallthrough".equals(warningToken))
        return 281474976710656L;
      if ("finally".equals(warningToken))
        return 16777216L;
      break;
    case 'h':
      if ("hiding".equals(warningToken))
        return 549756010504L;
    case 'i':
      if ("incomplete-switch".equals(warningToken))
        return 2199023255552L;
      break;
    case 'n':
      if ("nls".equals(warningToken))
        return 256L;
      if ("null".equals(warningToken))
        return 3377768440004608L;
      break;
    case 'r':
      if ("restriction".equals(warningToken))
        return 8813272891392L;
      break;
    case 's':
      if ("serial".equals(warningToken))
        return 4294967296L;
      if ("static-access".equals(warningToken))
        return 268437504L;
      if ("synthetic-access".equals(warningToken))
        return 128L;
      if ("super".equals(warningToken)) {
        return 562949953421312L;
      }
      break;
    case 'u':
      if ("unused".equals(warningToken))
        return 70368752600112L;
      if ("unchecked".equals(warningToken))
        return 35185445830656L;
      if ("unqualified-field-access".equals(warningToken))
        return 4194304L; break;
    case 'e':
    case 'g':
    case 'j':
    case 'k':
    case 'l':
    case 'm':
    case 'o':
    case 'p':
    case 'q':
    case 't': } return 0L;
  }
}