package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.internal.compiler.ast.Annotation;

public class AnnotationBinding
{
  ReferenceBinding type;
  ElementValuePair[] pairs;

  public static AnnotationBinding[] addStandardAnnotations(AnnotationBinding[] recordedAnnotations, long annotationTagBits, LookupEnvironment env)
  {
    int count = 0;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if ((annotationTagBits & 0x0) != 0L)
      count++;
    if (count == 0) {
      return recordedAnnotations;
    }
    int index = recordedAnnotations.length;
    AnnotationBinding[] result = new AnnotationBinding[index + count];
    System.arraycopy(recordedAnnotations, 0, result, 0, index);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildTargetAnnotation(annotationTagBits, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildRetentionAnnotation(annotationTagBits, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildMarkerAnnotation(TypeConstants.JAVA_LANG_DEPRECATED, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildMarkerAnnotation(TypeConstants.JAVA_LANG_ANNOTATION_DOCUMENTED, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildMarkerAnnotation(TypeConstants.JAVA_LANG_ANNOTATION_INHERITED, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildMarkerAnnotation(TypeConstants.JAVA_LANG_OVERRIDE, env);
    if ((annotationTagBits & 0x0) != 0L)
      result[(index++)] = buildMarkerAnnotation(TypeConstants.JAVA_LANG_SUPPRESSWARNINGS, env);
    return result;
  }

  private static AnnotationBinding buildMarkerAnnotation(char[][] compoundName, LookupEnvironment env) {
    ReferenceBinding type = env.getResolvedType(compoundName, null);
    return env.createAnnotation(type, Binding.NO_ELEMENT_VALUE_PAIRS);
  }

  private static AnnotationBinding buildRetentionAnnotation(long bits, LookupEnvironment env) {
    ReferenceBinding retentionPolicy = 
      env.getResolvedType(TypeConstants.JAVA_LANG_ANNOTATION_RETENTIONPOLICY, 
      null);
    Object value = null;
    if ((bits & 0x0) != 0L)
      value = retentionPolicy.getField(TypeConstants.UPPER_RUNTIME, true);
    else if ((bits & 0x0) != 0L)
      value = retentionPolicy.getField(TypeConstants.UPPER_CLASS, true);
    else if ((bits & 0x0) != 0L)
      value = retentionPolicy.getField(TypeConstants.UPPER_SOURCE, true);
    return env.createAnnotation(
      env.getResolvedType(TypeConstants.JAVA_LANG_ANNOTATION_RETENTION, null), 
      new ElementValuePair[] { 
      new ElementValuePair(TypeConstants.VALUE, value, null) });
  }

  private static AnnotationBinding buildTargetAnnotation(long bits, LookupEnvironment env)
  {
    ReferenceBinding target = env.getResolvedType(TypeConstants.JAVA_LANG_ANNOTATION_TARGET, null);
    if ((bits & 0x0) != 0L) {
      return new AnnotationBinding(target, Binding.NO_ELEMENT_VALUE_PAIRS);
    }
    int arraysize = 0;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    if ((bits & 0x0) != 0L)
      arraysize++;
    Object[] value = new Object[arraysize];
    if (arraysize > 0) {
      ReferenceBinding elementType = env.getResolvedType(TypeConstants.JAVA_LANG_ANNOTATION_ELEMENTTYPE, null);
      int index = 0;
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_ANNOTATION_TYPE, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_CONSTRUCTOR, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_FIELD, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_LOCAL_VARIABLE, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_METHOD, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_PACKAGE, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.UPPER_PARAMETER, true);
      if ((bits & 0x0) != 0L)
        value[(index++)] = elementType.getField(TypeConstants.TYPE, true);
    }
    return env.createAnnotation(
      target, 
      new ElementValuePair[] { 
      new ElementValuePair(TypeConstants.VALUE, value, null) });
  }

  AnnotationBinding(ReferenceBinding type, ElementValuePair[] pairs)
  {
    this.type = type;
    this.pairs = pairs;
  }

  AnnotationBinding(Annotation astAnnotation) {
    this((ReferenceBinding)astAnnotation.resolvedType, astAnnotation.computeElementValuePairs());
  }

  public ReferenceBinding getAnnotationType() {
    return this.type;
  }

  public ElementValuePair[] getElementValuePairs() {
    return this.pairs;
  }

  public static void setMethodBindings(ReferenceBinding type, ElementValuePair[] pairs)
  {
    int i = pairs.length;
    do { ElementValuePair pair = pairs[i];
      MethodBinding[] methods = type.getMethods(pair.getName());

      if ((methods != null) && (methods.length == 1))
        pair.setMethodBinding(methods[0]);
      i--; } while (i >= 0);
  }
}