package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Argument;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.ConstructorDeclaration;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference;
import org.eclipse.jdt.internal.compiler.ast.SingleNameReference;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public class MethodScope extends BlockScope
{
  public ReferenceContext referenceContext;
  public boolean isStatic;
  public boolean isConstructorCall = false;
  public FieldBinding initializedField;
  public int lastVisibleFieldID = -1;
  public int analysisIndex;
  public boolean isPropagatingInnerClassEmulation;
  public int lastIndex = 0;
  public long[] definiteInits = new long[4];
  public long[][] extraDefiniteInits = new long[4][];

  public boolean insideTypeAnnotation = false;
  public SyntheticArgumentBinding[] extraSyntheticArguments;

  public MethodScope(ClassScope parent, ReferenceContext context, boolean isStatic)
  {
    super(2, parent);
    this.locals = new LocalVariableBinding[5];
    this.referenceContext = context;
    this.isStatic = isStatic;
    this.startIndex = 0;
  }

  private void checkAndSetModifiersForConstructor(MethodBinding methodBinding)
  {
    int modifiers = methodBinding.modifiers;
    ReferenceBinding declaringClass = methodBinding.declaringClass;
    if ((modifiers & 0x400000) != 0) {
      problemReporter().duplicateModifierForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);
    }
    if ((((ConstructorDeclaration)this.referenceContext).bits & 0x80) != 0)
    {
      int flags;
      if ((flags = declaringClass.modifiers & 0x4005) != 0) {
        if ((flags & 0x4000) != 0) {
          modifiers &= -8;
          modifiers |= 2;
        } else {
          modifiers &= -8;
          modifiers |= flags;
        }
      }

    }

    int realModifiers = modifiers & 0xFFFF;

    if ((declaringClass.isEnum()) && ((((ConstructorDeclaration)this.referenceContext).bits & 0x80) == 0))
    {
      if ((realModifiers & 0xFFFFF7FD) != 0) {
        problemReporter().illegalModifierForEnumConstructor((AbstractMethodDeclaration)this.referenceContext);
        modifiers &= -63486;
      } else if ((((AbstractMethodDeclaration)this.referenceContext).modifiers & 0x800) != 0)
      {
        problemReporter().illegalModifierForMethod((AbstractMethodDeclaration)this.referenceContext);
      }
      modifiers |= 2;
    } else if ((realModifiers & 0xFFFFF7F8) != 0) {
      problemReporter().illegalModifierForMethod((AbstractMethodDeclaration)this.referenceContext);
      modifiers &= -63481;
    } else if ((((AbstractMethodDeclaration)this.referenceContext).modifiers & 0x800) != 0)
    {
      problemReporter().illegalModifierForMethod((AbstractMethodDeclaration)this.referenceContext);
    }

    int accessorBits = realModifiers & 0x7;
    if ((accessorBits & accessorBits - 1) != 0) {
      problemReporter().illegalVisibilityModifierCombinationForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);

      if ((accessorBits & 0x1) != 0) {
        if ((accessorBits & 0x4) != 0)
          modifiers &= -5;
        if ((accessorBits & 0x2) != 0)
          modifiers &= -3;
      } else if (((accessorBits & 0x4) != 0) && ((accessorBits & 0x2) != 0)) {
        modifiers &= -3;
      }

    }

    methodBinding.modifiers = modifiers;
  }

  private void checkAndSetModifiersForMethod(MethodBinding methodBinding)
  {
    int modifiers = methodBinding.modifiers;
    ReferenceBinding declaringClass = methodBinding.declaringClass;
    if ((modifiers & 0x400000) != 0) {
      problemReporter().duplicateModifierForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);
    }

    int realModifiers = modifiers & 0xFFFF;

    if (declaringClass.isInterface()) {
      if ((realModifiers & 0xFFFFFBFE) != 0) {
        if ((declaringClass.modifiers & 0x2000) != 0)
          problemReporter().illegalModifierForAnnotationMember((AbstractMethodDeclaration)this.referenceContext);
        else
          problemReporter().illegalModifierForInterfaceMethod((AbstractMethodDeclaration)this.referenceContext);
      }
      return;
    }

    if ((realModifiers & 0xFFFFF2C0) != 0) {
      problemReporter().illegalModifierForMethod((AbstractMethodDeclaration)this.referenceContext);
      modifiers &= -62145;
    }

    int accessorBits = realModifiers & 0x7;
    if ((accessorBits & accessorBits - 1) != 0) {
      problemReporter().illegalVisibilityModifierCombinationForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);

      if ((accessorBits & 0x1) != 0) {
        if ((accessorBits & 0x4) != 0)
          modifiers &= -5;
        if ((accessorBits & 0x2) != 0)
          modifiers &= -3;
      } else if (((accessorBits & 0x4) != 0) && ((accessorBits & 0x2) != 0)) {
        modifiers &= -3;
      }

    }

    if ((modifiers & 0x400) != 0) {
      int incompatibleWithAbstract = 2362;
      if ((modifiers & incompatibleWithAbstract) != 0)
        problemReporter().illegalAbstractModifierCombinationForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);
      if (!methodBinding.declaringClass.isAbstract()) {
        problemReporter().abstractMethodInAbstractClass((SourceTypeBinding)declaringClass, (AbstractMethodDeclaration)this.referenceContext);
      }

    }

    if (((modifiers & 0x100) != 0) && ((modifiers & 0x800) != 0)) {
      problemReporter().nativeMethodsCannotBeStrictfp(declaringClass, (AbstractMethodDeclaration)this.referenceContext);
    }

    if (((realModifiers & 0x8) != 0) && (declaringClass.isNestedType()) && (!declaringClass.isStatic())) {
      problemReporter().unexpectedStaticModifierForMethod(declaringClass, (AbstractMethodDeclaration)this.referenceContext);
    }
    methodBinding.modifiers = modifiers;
  }

  public void computeLocalVariablePositions(int initOffset, CodeStream codeStream)
  {
    boolean isReportingUnusedArgument = false;

    if ((this.referenceContext instanceof AbstractMethodDeclaration)) {
      AbstractMethodDeclaration methodDecl = (AbstractMethodDeclaration)this.referenceContext;
      MethodBinding method = methodDecl.binding;
      if ((!method.isAbstract()) && 
        ((!method.isImplementing()) || (compilerOptions().reportUnusedParameterWhenImplementingAbstract)) && 
        ((!method.isOverriding()) || (method.isImplementing()) || (compilerOptions().reportUnusedParameterWhenOverridingConcrete)) && 
        (!method.isMain())) {
        isReportingUnusedArgument = true;
      }
    }
    this.offset = initOffset;
    this.maxOffset = initOffset;

    int ilocal = 0; int maxLocals = this.localIndex;
    while (ilocal < maxLocals) {
      LocalVariableBinding local = this.locals[ilocal];
      if ((local == null) || ((local.tagBits & 0x400) == 0L)) {
        break;
      }
      if ((isReportingUnusedArgument) && 
        (local.useFlag == 0) && 
        ((local.declaration.bits & 0x40000000) != 0)) {
        problemReporter().unusedArgument(local.declaration);
      }

      codeStream.record(local);

      local.resolvedPosition = this.offset;

      if ((local.type == TypeBinding.LONG) || (local.type == TypeBinding.DOUBLE))
        this.offset += 2;
      else {
        this.offset += 1;
      }

      if (this.offset > 255) {
        problemReporter().noMoreAvailableSpaceForArgument(local, local.declaration);
      }
      ilocal++;
    }

    if (this.extraSyntheticArguments != null) {
      int iarg = 0; for (int maxArguments = this.extraSyntheticArguments.length; iarg < maxArguments; iarg++) {
        SyntheticArgumentBinding argument = this.extraSyntheticArguments[iarg];
        argument.resolvedPosition = this.offset;
        if ((argument.type == TypeBinding.LONG) || (argument.type == TypeBinding.DOUBLE))
          this.offset += 2;
        else {
          this.offset += 1;
        }
        if (this.offset > 255) {
          problemReporter().noMoreAvailableSpaceForArgument(argument, (ASTNode)this.referenceContext);
        }
      }
    }
    computeLocalVariablePositions(ilocal, this.offset, codeStream);
  }

  MethodBinding createMethod(AbstractMethodDeclaration method)
  {
    this.referenceContext = method;
    method.scope = this;
    SourceTypeBinding declaringClass = referenceType().binding;
    int modifiers = method.modifiers | 0x2000000;
    if (method.isConstructor()) {
      if (method.isDefaultConstructor())
        modifiers |= 67108864;
      method.binding = new MethodBinding(modifiers, null, null, declaringClass);
      checkAndSetModifiersForConstructor(method.binding);
    } else {
      if (declaringClass.isInterface())
        modifiers |= 1025;
      method.binding = 
        new MethodBinding(modifiers, method.selector, null, null, null, declaringClass);
      checkAndSetModifiersForMethod(method.binding);
    }
    this.isStatic = method.binding.isStatic();

    Argument[] argTypes = method.arguments;
    int argLength = argTypes == null ? 0 : argTypes.length;
    if ((argLength > 0) && (compilerOptions().sourceLevel >= 3211264L)) {
      if (argTypes[(--argLength)].isVarArgs())
        method.binding.modifiers |= 128;
      do {
        if (argTypes[argLength].isVarArgs())
          problemReporter().illegalVararg(argTypes[argLength], method);
        argLength--; } while (argLength >= 0);
    }

    TypeParameter[] typeParameters = method.typeParameters();

    if ((typeParameters == null) || (compilerOptions().sourceLevel < 3211264L)) {
      method.binding.typeVariables = Binding.NO_TYPE_VARIABLES;
    } else {
      method.binding.typeVariables = createTypeVariables(typeParameters, method.binding);
      method.binding.modifiers |= 1073741824;
    }
    return method.binding;
  }

  public FieldBinding findField(TypeBinding receiverType, char[] fieldName, InvocationSite invocationSite, boolean needResolve)
  {
    FieldBinding field = super.findField(receiverType, fieldName, invocationSite, needResolve);
    if (field == null)
      return null;
    if (!field.isValidBinding())
      return field;
    if (field.isStatic()) {
      return field;
    }
    if ((!this.isConstructorCall) || (receiverType != enclosingSourceType())) {
      return field;
    }
    if ((invocationSite instanceof SingleNameReference))
      return new ProblemFieldBinding(
        field, 
        field.declaringClass, 
        fieldName, 
        6);
    if ((invocationSite instanceof QualifiedNameReference))
    {
      QualifiedNameReference name = (QualifiedNameReference)invocationSite;
      if (name.binding == null)
      {
        return new ProblemFieldBinding(
          field, 
          field.declaringClass, 
          fieldName, 
          6);
      }
    }
    return field;
  }

  public boolean isInsideConstructor()
  {
    return this.referenceContext instanceof ConstructorDeclaration;
  }

  public boolean isInsideInitializer()
  {
    return this.referenceContext instanceof TypeDeclaration;
  }

  public boolean isInsideInitializerOrConstructor()
  {
    return ((this.referenceContext instanceof TypeDeclaration)) || 
      ((this.referenceContext instanceof ConstructorDeclaration));
  }

  public ProblemReporter problemReporter()
  {
    MethodScope outerMethodScope;
    if ((outerMethodScope = outerMostMethodScope()) == this) {
      ProblemReporter problemReporter = referenceCompilationUnit().problemReporter;
      problemReporter.referenceContext = this.referenceContext;
      return problemReporter;
    }
    return outerMethodScope.problemReporter();
  }

  public final int recordInitializationStates(FlowInfo flowInfo)
  {
    if ((flowInfo.tagBits & 0x1) != 0) return -1; 
UnconditionalFlowInfo unconditionalFlowInfo = flowInfo.unconditionalInitsWithoutSideEffect();
    long[] extraInits = unconditionalFlowInfo.extra == null ? 
      null : unconditionalFlowInfo.extra[0];
    long inits = unconditionalFlowInfo.definiteInits;
    int i = this.lastIndex;
    label139: 
    do { if (this.definiteInits[i] == inits) {
        long[] otherInits = this.extraDefiniteInits[i];
        int j;
        int max;
        if ((extraInits != null) && (otherInits != null)) {
          if (extraInits.length != otherInits.length)
            break label139;
          j = 0; max = extraInits.length;
        }while (extraInits[j] == otherInits[j])
        {
          j++; if (j >= max)
          {
            return i;

            if ((extraInits == null) && (otherInits == null))
              return i;
          }
        }
      }
      i--; } while (i >= 0);

    if (this.definiteInits.length == this.lastIndex)
    {
      System.arraycopy(
        this.definiteInits, 
        0, 
        this.definiteInits = new long[this.lastIndex + 20], 
        0, 
        this.lastIndex);
      System.arraycopy(
        this.extraDefiniteInits, 
        0, 
        this.extraDefiniteInits = new long[this.lastIndex + 20][], 
        0, 
        this.lastIndex);
    }
    this.definiteInits[this.lastIndex] = inits;
    if (extraInits != null) {
      this.extraDefiniteInits[this.lastIndex] = new long[extraInits.length];
      System.arraycopy(
        extraInits, 
        0, 
        this.extraDefiniteInits[this.lastIndex], 
        0, 
        extraInits.length);
    }
    return this.lastIndex++;
  }

  public AbstractMethodDeclaration referenceMethod()
  {
    if ((this.referenceContext instanceof AbstractMethodDeclaration)) return (AbstractMethodDeclaration)this.referenceContext;
    return null;
  }

  public TypeDeclaration referenceType()
  {
    return ((ClassScope)this.parent).referenceContext;
  }

  String basicToString(int tab)
  {
    String newLine = "\n";
    int i = tab;
    do { newLine = newLine + "\t";

      i--; } while (i >= 0);

    String s = newLine + "--- Method Scope ---";
    newLine = newLine + "\t";
    s = s + newLine + "locals:";
    for (int i = 0; i < this.localIndex; i++)
      s = s + newLine + "\t" + this.locals[i].toString();
    s = s + newLine + "startIndex = " + this.startIndex;
    s = s + newLine + "isConstructorCall = " + this.isConstructorCall;
    s = s + newLine + "initializedField = " + this.initializedField;
    s = s + newLine + "lastVisibleFieldID = " + this.lastVisibleFieldID;
    s = s + newLine + "referenceContext = " + this.referenceContext;
    return s;
  }
}