package org.eclipse.jdt.internal.compiler.lookup;

import org.eclipse.jdt.core.compiler.CharOperation;

public class WildcardBinding extends ReferenceBinding
{
  ReferenceBinding genericType;
  int rank;
  public TypeBinding bound;
  public TypeBinding[] otherBounds;
  char[] genericSignature;
  public int boundKind;
  ReferenceBinding superclass;
  ReferenceBinding[] superInterfaces;
  TypeVariableBinding typeVariable;
  LookupEnvironment environment;

  public WildcardBinding(ReferenceBinding genericType, int rank, TypeBinding bound, TypeBinding[] otherBounds, int boundKind, LookupEnvironment environment)
  {
    this.genericType = genericType;
    this.rank = rank;
    this.boundKind = boundKind;
    this.modifiers = 1073741825;
    this.environment = environment;
    initialize(genericType, bound, otherBounds);

    if ((genericType instanceof UnresolvedReferenceBinding))
      ((UnresolvedReferenceBinding)genericType).addWrapper(this, environment);
    if ((bound instanceof UnresolvedReferenceBinding))
      ((UnresolvedReferenceBinding)bound).addWrapper(this, environment);
    this.tagBits |= 16777216L;
  }

  public int kind() {
    return 516;
  }

  public boolean boundCheck(TypeBinding argumentType)
  {
    switch (this.boundKind) {
    case 0:
      return true;
    case 1:
      if (argumentType.isCompatibleWith(this.bound)) return true;

      int i = 0; for (int length = this.otherBounds == null ? 0 : this.otherBounds.length; i < length; i++) {
        if (argumentType.isCompatibleWith(this.otherBounds[i])) return true;
      }
      return false;
    }

    return argumentType.isCompatibleWith(this.bound);
  }

  public boolean canBeInstantiated()
  {
    return false;
  }

  public void collectSubstitutes(Scope scope, TypeBinding actualType, InferenceContext inferenceContext, int constraint)
  {
    if ((this.tagBits & 0x20000000) == 0L) return;
    if (actualType == TypeBinding.NULL) return;

    if (actualType.isCapture()) {
      CaptureBinding capture = (CaptureBinding)actualType;
      actualType = capture.wildcard;
    }

    switch (constraint) {
    case 1:
      switch (this.boundKind)
      {
      case 0:
        break;
      case 1:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 1);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++) {
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 1);
            }
            break;
          case 2:
          default:
            break;
          }
        } else { this.bound.collectSubstitutes(scope, actualType, inferenceContext, 1); }

        break;
      case 2:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            break;
          case 2:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 2);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++) {
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 2);
            }
          default:
            break;
          }
        } else { this.bound.collectSubstitutes(scope, actualType, inferenceContext, 2); }

        break;
      }
      break;
    case 0:
      switch (this.boundKind)
      {
      case 0:
        break;
      case 1:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 0);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++) {
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 0);
            }

          case 2:
          }

        }

        break;
      case 2:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            break;
          case 2:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 0);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++) {
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 0);
            }
          }
        }

        break;
      }

      break;
    case 2:
      switch (this.boundKind)
      {
      case 0:
        break;
      case 1:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 2);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++) {
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 2);
            }

          case 2:
          }

        }

        break;
      case 2:
        if (actualType.isWildcard()) {
          WildcardBinding actualWildcard = (WildcardBinding)actualType;
          switch (actualWildcard.boundKind) {
          case 0:
            break;
          case 1:
            break;
          case 2:
            this.bound.collectSubstitutes(scope, actualWildcard.bound, inferenceContext, 2);
            int i = 0; for (int length = actualWildcard.otherBounds == null ? 0 : actualWildcard.otherBounds.length; i < length; i++)
              this.bound.collectSubstitutes(scope, actualWildcard.otherBounds[i], inferenceContext, 2);
          }
        }
        break;
      }
      break;
    }
  }

  public char[] computeUniqueKey(boolean isLeaf)
  {
    char[] genericTypeKey = this.genericType.computeUniqueKey(false);
    char[] wildCardKey;
    char[] wildCardKey;
    char[] wildCardKey;
    switch (this.boundKind) {
    case 0:
      wildCardKey = TypeConstants.WILDCARD_STAR;
      break;
    case 1:
      wildCardKey = CharOperation.concat(TypeConstants.WILDCARD_PLUS, this.bound.computeUniqueKey(false));
      break;
    default:
      wildCardKey = CharOperation.concat(TypeConstants.WILDCARD_MINUS, this.bound.computeUniqueKey(false));
    }

    return CharOperation.concat(genericTypeKey, wildCardKey);
  }

  public char[] constantPoolName()
  {
    return erasure().constantPoolName();
  }

  public String debugName()
  {
    return toString();
  }

  public TypeBinding erasure()
  {
    if (this.otherBounds == null) {
      if (this.boundKind == 1)
        return this.bound.erasure();
      return typeVariable().erasure();
    }

    return this.bound.id == 1 ? 
      this.otherBounds[0].erasure() : 
      this.bound.erasure();
  }

  public char[] genericTypeSignature()
  {
    if (this.genericSignature == null) {
      switch (this.boundKind) {
      case 0:
        this.genericSignature = TypeConstants.WILDCARD_STAR;
        break;
      case 1:
        this.genericSignature = CharOperation.concat(TypeConstants.WILDCARD_PLUS, this.bound.genericTypeSignature());
        break;
      default:
        this.genericSignature = CharOperation.concat(TypeConstants.WILDCARD_MINUS, this.bound.genericTypeSignature());
      }
    }
    return this.genericSignature;
  }

  public int hashCode() {
    return this.genericType.hashCode();
  }

  void initialize(ReferenceBinding someGenericType, TypeBinding someBound, TypeBinding[] someOtherBounds) {
    this.genericType = someGenericType;
    this.bound = someBound;
    this.otherBounds = someOtherBounds;
    if (someGenericType != null) {
      this.fPackage = someGenericType.getPackage();
    }
    if (someBound != null)
      this.tagBits |= someBound.tagBits & 0x20000000;
  }

  public boolean isSuperclassOf(ReferenceBinding otherType)
  {
    if (this.boundKind == 2) {
      if ((this.bound instanceof ReferenceBinding)) {
        return ((ReferenceBinding)this.bound).isSuperclassOf(otherType);
      }
      return otherType.id == 1;
    }

    return false;
  }

  public boolean isIntersectionType()
  {
    return this.otherBounds != null;
  }

  public boolean isUnboundWildcard()
  {
    return this.boundKind == 0;
  }

  public boolean isWildcard()
  {
    return true;
  }

  public char[] readableName()
  {
    switch (this.boundKind) {
    case 0:
      return TypeConstants.WILDCARD_NAME;
    case 1:
      if (this.otherBounds == null)
        return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_EXTENDS, this.bound.readableName());
      StringBuffer buffer = new StringBuffer(10);
      buffer.append(this.bound.readableName());
      int i = 0; for (int length = this.otherBounds.length; i < length; i++)
        buffer.append('&').append(this.otherBounds[i].readableName());
      int length;
      char[] result = new char[length = buffer.length()];
      buffer.getChars(0, length, result, 0);
      return result;
    }
    return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_SUPER, this.bound.readableName());
  }

  ReferenceBinding resolve()
  {
    if ((this.tagBits & 0x1000000) == 0L) {
      return this;
    }
    this.tagBits &= -16777217L;
    BinaryTypeBinding.resolveType(this.genericType, this.environment, null, 0);
    switch (this.boundKind) {
    case 1:
    case 2:
      BinaryTypeBinding.resolveType(this.bound, this.environment, null, 0);
    case 0:
    }

    return this;
  }

  public char[] shortReadableName()
  {
    switch (this.boundKind) {
    case 0:
      return TypeConstants.WILDCARD_NAME;
    case 1:
      if (this.otherBounds == null)
        return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_EXTENDS, this.bound.shortReadableName());
      StringBuffer buffer = new StringBuffer(10);
      buffer.append(this.bound.shortReadableName());
      int i = 0; for (int length = this.otherBounds.length; i < length; i++)
        buffer.append('&').append(this.otherBounds[i].shortReadableName());
      int length;
      char[] result = new char[length = buffer.length()];
      buffer.getChars(0, length, result, 0);
      return result;
    }
    return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_SUPER, this.bound.shortReadableName());
  }

  public char[] signature()
  {
    if (this.signature == null) {
      switch (this.boundKind) {
      case 1:
        return this.bound.signature();
      }
      return typeVariable().signature();
    }

    return this.signature;
  }

  public char[] sourceName()
  {
    switch (this.boundKind) {
    case 0:
      return TypeConstants.WILDCARD_NAME;
    case 1:
      return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_EXTENDS, this.bound.sourceName());
    }
    return CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_SUPER, this.bound.sourceName());
  }

  public ReferenceBinding superclass()
  {
    if (this.superclass == null) {
      TypeBinding superType = null;
      if ((this.boundKind == 1) && (!this.bound.isInterface())) {
        superType = this.bound;
      } else {
        TypeVariableBinding variable = typeVariable();
        if (variable != null) superType = variable.firstBound;
      }
      this.superclass = (((superType instanceof ReferenceBinding)) && (!superType.isInterface()) ? 
        (ReferenceBinding)superType : 
        this.environment.getResolvedType(TypeConstants.JAVA_LANG_OBJECT, null));
    }

    return this.superclass;
  }

  public ReferenceBinding[] superInterfaces()
  {
    if (this.superInterfaces == null) {
      if (typeVariable() != null)
        this.superInterfaces = this.typeVariable.superInterfaces();
      else {
        this.superInterfaces = Binding.NO_SUPERINTERFACES;
      }
      if (this.boundKind == 1) {
        if (this.bound.isInterface())
        {
          int length = this.superInterfaces.length;
          System.arraycopy(this.superInterfaces, 0, this.superInterfaces = new ReferenceBinding[length + 1], 1, length);
          this.superInterfaces[0] = ((ReferenceBinding)this.bound);
        }
        if (this.otherBounds != null)
        {
          int length = this.superInterfaces.length;
          int otherLength = this.otherBounds.length;
          System.arraycopy(this.superInterfaces, 0, this.superInterfaces = new ReferenceBinding[length + otherLength], 0, length);
          for (int i = 0; i < otherLength; i++) {
            this.superInterfaces[(length + i)] = ((ReferenceBinding)this.otherBounds[i]);
          }
        }
      }
    }
    return this.superInterfaces;
  }

  public ReferenceBinding[] superInterfaces2() {
    if (this.superInterfaces == null) {
      if (this.boundKind == 1) {
        if (this.bound.isInterface()) {
          if (this.otherBounds != null)
          {
            int otherLength = this.otherBounds.length;
            System.arraycopy(this.otherBounds, 0, this.superInterfaces = new ReferenceBinding[otherLength + 1], 1, otherLength);
            this.superInterfaces[0] = ((ReferenceBinding)this.bound);
          } else {
            this.superInterfaces = new ReferenceBinding[] { (ReferenceBinding)this.bound };
          }
        } else if (this.otherBounds != null) {
          int otherLength = this.otherBounds.length;
          System.arraycopy(this.otherBounds, 0, this.superInterfaces = new ReferenceBinding[otherLength], 0, otherLength);
        } else {
          this.superInterfaces = Binding.NO_SUPERINTERFACES;
        }
      }
      else this.superInterfaces = Binding.NO_SUPERINTERFACES;
    }

    return this.superInterfaces;
  }

  public void swapUnresolved(UnresolvedReferenceBinding unresolvedType, ReferenceBinding resolvedType, LookupEnvironment env) {
    boolean affected = false;
    if (this.genericType == unresolvedType) {
      this.genericType = resolvedType;
      affected = true;
    } else if (this.bound == unresolvedType) {
      this.bound = env.convertUnresolvedBinaryToRawType(resolvedType);
      affected = true;
    }
    if (affected)
      initialize(this.genericType, this.bound, this.otherBounds);
  }

  public String toString()
  {
    switch (this.boundKind) {
    case 0:
      return new String(TypeConstants.WILDCARD_NAME);
    case 1:
      if (this.otherBounds == null)
        return new String(CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_EXTENDS, this.bound.debugName().toCharArray()));
      StringBuffer buffer = new StringBuffer(this.bound.debugName());
      int i = 0; for (int length = this.otherBounds.length; i < length; i++) {
        buffer.append('&').append(this.otherBounds[i].debugName());
      }
      return buffer.toString();
    }
    return new String(CharOperation.concat(TypeConstants.WILDCARD_NAME, TypeConstants.WILDCARD_SUPER, this.bound.debugName().toCharArray()));
  }

  public TypeVariableBinding typeVariable()
  {
    if (this.typeVariable == null) {
      TypeVariableBinding[] typeVariables = this.genericType.typeVariables();
      if (this.rank < typeVariables.length)
        this.typeVariable = typeVariables[this.rank];
    }
    return this.typeVariable;
  }
}