package org.eclipse.jdt.internal.compiler.parser;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.util.Util;

public class Scanner
  implements TerminalTokens
{
  public long sourceLevel;
  public long complianceLevel;
  public boolean useAssertAsAnIndentifier = false;

  public boolean containsAssertKeyword = false;

  public boolean useEnumAsAnIndentifier = false;

  public boolean recordLineSeparator = false;
  public char currentCharacter;
  public int startPosition;
  public int currentPosition;
  public int initialPosition;
  public int eofPosition;
  public boolean tokenizeComments = false;
  public boolean tokenizeWhiteSpace = false;
  public char[] source;
  public char[] withoutUnicodeBuffer;
  public int withoutUnicodePtr;
  public boolean unicodeAsBackSlash = false;

  public boolean scanningFloatLiteral = false;
  public static final int COMMENT_ARRAYS_SIZE = 30;
  public int[] commentStops = new int[30];
  public int[] commentStarts = new int[30];
  public int[] commentTagStarts = new int[30];
  public int commentPtr = -1;
  protected int lastCommentLinePosition = -1;

  public char[][] foundTaskTags = null;
  public char[][] foundTaskMessages;
  public char[][] foundTaskPriorities = null;
  public int[][] foundTaskPositions;
  public int foundTaskCount = 0;
  public char[][] taskTags = null;
  public char[][] taskPriorities = null;
  public boolean isTaskCaseSensitive = true;

  public boolean diet = false;

  public int[] lineEnds = new int['ú'];
  public int linePtr = -1;
  public boolean wasAcr = false;
  public static final String END_OF_SOURCE = "End_Of_Source";
  public static final String INVALID_HEXA = "Invalid_Hexa_Literal";
  public static final String INVALID_OCTAL = "Invalid_Octal_Literal";
  public static final String INVALID_CHARACTER_CONSTANT = "Invalid_Character_Constant";
  public static final String INVALID_ESCAPE = "Invalid_Escape";
  public static final String INVALID_INPUT = "Invalid_Input";
  public static final String INVALID_UNICODE_ESCAPE = "Invalid_Unicode_Escape";
  public static final String INVALID_FLOAT = "Invalid_Float_Literal";
  public static final String INVALID_LOW_SURROGATE = "Invalid_Low_Surrogate";
  public static final String INVALID_HIGH_SURROGATE = "Invalid_High_Surrogate";
  public static final String NULL_SOURCE_STRING = "Null_Source_String";
  public static final String UNTERMINATED_STRING = "Unterminated_String";
  public static final String UNTERMINATED_COMMENT = "Unterminated_Comment";
  public static final String INVALID_CHAR_IN_STRING = "Invalid_Char_In_String";
  public static final String INVALID_DIGIT = "Invalid_Digit";
  private static final int[] EMPTY_LINE_ENDS = Util.EMPTY_INT_ARRAY;

  static final char[] charArray_a = { 'a' };
  static final char[] charArray_b = { 'b' };
  static final char[] charArray_c = { 'c' };
  static final char[] charArray_d = { 'd' };
  static final char[] charArray_e = { 'e' };
  static final char[] charArray_f = { 'f' };
  static final char[] charArray_g = { 'g' };
  static final char[] charArray_h = { 'h' };
  static final char[] charArray_i = { 'i' };
  static final char[] charArray_j = { 'j' };
  static final char[] charArray_k = { 'k' };
  static final char[] charArray_l = { 'l' };
  static final char[] charArray_m = { 'm' };
  static final char[] charArray_n = { 'n' };
  static final char[] charArray_o = { 'o' };
  static final char[] charArray_p = { 'p' };
  static final char[] charArray_q = { 'q' };
  static final char[] charArray_r = { 'r' };
  static final char[] charArray_s = { 's' };
  static final char[] charArray_t = { 't' };
  static final char[] charArray_u = { 'u' };
  static final char[] charArray_v = { 'v' };
  static final char[] charArray_w = { 'w' };
  static final char[] charArray_x = { 'x' };
  static final char[] charArray_y = { 'y' };
  static final char[] charArray_z = { 'z' };

  static final char[] initCharArray = new char[6];
  static final int TableSize = 30;
  static final int InternalTableSize = 6;
  public static final int OptimizedLength = 7;
  public final char[][][][] charArray_length = new char[7][30][6];

  public static final char[] TAG_PREFIX = "//$NON-NLS-".toCharArray();
  public static final int TAG_PREFIX_LENGTH = TAG_PREFIX.length;
  public static final char TAG_POSTFIX = '$';
  public static final int TAG_POSTFIX_LENGTH = 1;
  private NLSTag[] nlsTags = null;
  protected int nlsTagsPtr;
  public boolean checkNonExternalizedStringLiterals;
  public boolean returnOnlyGreater = false;
  int newEntry2;
  int newEntry3;
  int newEntry4;
  int newEntry5;
  int newEntry6;
  public boolean insideRecovery;
  public static final int RoundBracket = 0;
  public static final int SquareBracket = 1;
  public static final int CurlyBracket = 2;
  public static final int BracketKinds = 3;
  public static final int LOW_SURROGATE_MIN_VALUE = 56320;
  public static final int HIGH_SURROGATE_MIN_VALUE = 55296;
  public static final int HIGH_SURROGATE_MAX_VALUE = 56319;
  public static final int LOW_SURROGATE_MAX_VALUE = 57343;

  public Scanner()
  {
    this(false, false, false, 3080192L, null, null, true);
  }

  public Scanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean checkNonExternalizedStringLiterals, long sourceLevel, long complianceLevel, char[][] taskTags, char[][] taskPriorities, boolean isTaskCaseSensitive)
  {
    for (int i = 0; i < 6; i++) {
      for (int j = 0; j < 30; j++) {
        for (int k = 0; k < 6; k++) {
          this.charArray_length[i][j][k] = initCharArray;
        }
      }
    }

    this.newEntry2 = 0;
    this.newEntry3 = 0;
    this.newEntry4 = 0;
    this.newEntry5 = 0;
    this.newEntry6 = 0;
    this.insideRecovery = false;

    this.eofPosition = 2147483647;
    this.tokenizeComments = tokenizeComments;
    this.tokenizeWhiteSpace = tokenizeWhiteSpace;
    this.sourceLevel = sourceLevel;
    this.complianceLevel = complianceLevel;
    this.checkNonExternalizedStringLiterals = checkNonExternalizedStringLiterals;
    this.taskTags = taskTags;
    this.taskPriorities = taskPriorities;
    this.isTaskCaseSensitive = isTaskCaseSensitive;
  }

  public Scanner(boolean tokenizeComments, boolean tokenizeWhiteSpace, boolean checkNonExternalizedStringLiterals, long sourceLevel, char[][] taskTags, char[][] taskPriorities, boolean isTaskCaseSensitive)
  {
    this(tokenizeComments, 
      tokenizeWhiteSpace, 
      checkNonExternalizedStringLiterals, 
      sourceLevel, 
      sourceLevel, 
      taskTags, 
      taskPriorities, 
      isTaskCaseSensitive);
  }

  public final boolean atEnd()
  {
    return this.eofPosition <= this.currentPosition;
  }

  public void checkTaskTag(int commentStart, int commentEnd)
    throws InvalidInputException
  {
    char[] src = this.source;

    if ((this.foundTaskCount > 0) && 
      (this.foundTaskPositions[(this.foundTaskCount - 1)][0] >= commentStart)) {
      return;
    }
    int foundTaskIndex = this.foundTaskCount;
    char previous = src[(commentStart + 1)];

    for (int i = commentStart + 2; (i < commentEnd) && (i < this.eofPosition); i++) {
      char[] tag = (char[])null;
      char[] priority = (char[])null;

      if (previous != '@')
        for (int itag = 0; itag < this.taskTags.length; itag++) {
          tag = this.taskTags[itag];
          int tagLength = tag.length;
          if (tagLength != 0)
          {
            if ((!ScannerHelper.isJavaIdentifierStart(tag[0])) || 
              (!ScannerHelper.isJavaIdentifierPart(previous)))
            {
              for (int t = 0; t < tagLength; t++)
              {
                int x = i + t;
                char sc;
                char tc;
                if ((x >= this.eofPosition) || (x >= commentEnd) || (
                  ((sc = src[(i + t)]) != (tc = tag[t])) && (
                  (this.isTaskCaseSensitive) || (ScannerHelper.toLowerCase(sc) != ScannerHelper.toLowerCase(tc)))))
                {
                  break;
                }
              }

              if ((i + tagLength >= commentEnd) || (!ScannerHelper.isJavaIdentifierPart(src[(i + tagLength - 1)])) || 
                (!ScannerHelper.isJavaIdentifierPart(src[(i + tagLength)])))
              {
                if (this.foundTaskTags == null) {
                  this.foundTaskTags = new char[5][];
                  this.foundTaskMessages = new char[5][];
                  this.foundTaskPriorities = new char[5][];
                  this.foundTaskPositions = new int[5][];
                } else if (this.foundTaskCount == this.foundTaskTags.length) {
                  System.arraycopy(this.foundTaskTags, 0, this.foundTaskTags = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskMessages, 0, this.foundTaskMessages = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskPriorities, 0, this.foundTaskPriorities = new char[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                  System.arraycopy(this.foundTaskPositions, 0, this.foundTaskPositions = new int[this.foundTaskCount * 2][], 0, this.foundTaskCount);
                }

                priority = (this.taskPriorities != null) && (itag < this.taskPriorities.length) ? 
                  this.taskPriorities[itag] : 
                  null;

                this.foundTaskTags[this.foundTaskCount] = tag;
                this.foundTaskPriorities[this.foundTaskCount] = priority;
                this.foundTaskPositions[this.foundTaskCount] = { i, i + tagLength - 1 };
                this.foundTaskMessages[this.foundTaskCount] = CharOperation.NO_CHAR;
                this.foundTaskCount += 1;
                i += tagLength - 1;
                break;
              }
            }
          }
        }
      previous = src[i];
    }
    boolean containsEmptyTask = false;
    for (int i = foundTaskIndex; i < this.foundTaskCount; i++)
    {
      int msgStart = this.foundTaskPositions[i][0] + this.foundTaskTags[i].length;
      int max_value = i + 1 < this.foundTaskCount ? 
        this.foundTaskPositions[(i + 1)][0] - 1 : 
        commentEnd - 1;

      if (max_value < msgStart) {
        max_value = msgStart;
      }
      int end = -1;

      for (int j = msgStart; j < max_value; j++)
      {
        char c;
        if (((c = src[j]) == '\n') || (c == '\r')) {
          end = j - 1;
          break;
        }
      }
      if (end == -1) {
        for (int j = max_value; j > msgStart; j--)
        {
          char c;
          if ((c = src[j]) == '*') {
            end = j - 1;
            break;
          }
        }
        if (end == -1)
          end = max_value;
      }
      if (msgStart == end)
      {
        containsEmptyTask = true;
      }
      else
      {
        do {
          end--;

          if (!CharOperation.isWhitespace(src[end])) break;  } while (msgStart <= end);

        while ((CharOperation.isWhitespace(src[msgStart])) && (msgStart <= end)) {
          msgStart++;
        }
        this.foundTaskPositions[i][1] = end;

        int messageLength = end - msgStart + 1;
        char[] message = new char[messageLength];
        System.arraycopy(src, msgStart, message, 0, messageLength);
        this.foundTaskMessages[i] = message;
      }
    }
    if (containsEmptyTask) {
      int i = foundTaskIndex; for (int max = this.foundTaskCount; i < max; i++)
        if (this.foundTaskMessages[i].length == 0)
          for (int j = i + 1; j < max; j++)
            if (this.foundTaskMessages[j].length != 0) {
              this.foundTaskMessages[i] = this.foundTaskMessages[j];
              this.foundTaskPositions[i][1] = this.foundTaskPositions[j][1];
              break;
            }
    }
  }

  public char[] getCurrentIdentifierSource()
  {
    char[] result;
    if (this.withoutUnicodePtr != 0)
    {
      char[] result;
      System.arraycopy(
        this.withoutUnicodeBuffer, 
        1, 
        result = new char[this.withoutUnicodePtr], 
        0, 
        this.withoutUnicodePtr);
    } else {
      int length = this.currentPosition - this.startPosition;
      if (length == this.eofPosition) return this.source;
      switch (length) {
      case 1:
        return optimizedCurrentTokenSource1();
      case 2:
        return optimizedCurrentTokenSource2();
      case 3:
        return optimizedCurrentTokenSource3();
      case 4:
        return optimizedCurrentTokenSource4();
      case 5:
        return optimizedCurrentTokenSource5();
      case 6:
        return optimizedCurrentTokenSource6();
      }

      System.arraycopy(this.source, this.startPosition, result = new char[length], 0, length);
    }

    return result;
  }
  public int getCurrentTokenEndPosition() {
    return this.currentPosition - 1;
  }

  public char[] getCurrentTokenSource()
  {
    char[] result;
    if (this.withoutUnicodePtr != 0)
    {
      char[] result;
      System.arraycopy(
        this.withoutUnicodeBuffer, 
        1, 
        result = new char[this.withoutUnicodePtr], 
        0, 
        this.withoutUnicodePtr);
    }
    else
    {
      int length;
      System.arraycopy(
        this.source, 
        this.startPosition, 
        result = new char[length = this.currentPosition - this.startPosition], 
        0, 
        length);
    }
    return result;
  }

  public final String getCurrentTokenString()
  {
    if (this.withoutUnicodePtr != 0)
    {
      return new String(
        this.withoutUnicodeBuffer, 
        1, 
        this.withoutUnicodePtr);
    }
    return new String(
      this.source, 
      this.startPosition, 
      this.currentPosition - this.startPosition);
  }

  public char[] getCurrentTokenSourceString()
  {
    char[] result;
    if (this.withoutUnicodePtr != 0)
    {
      char[] result;
      System.arraycopy(this.withoutUnicodeBuffer, 2, 
        result = new char[this.withoutUnicodePtr - 2], 0, this.withoutUnicodePtr - 2);
    }
    else
    {
      int length;
      System.arraycopy(
        this.source, 
        this.startPosition + 1, 
        result = new char[length = this.currentPosition - this.startPosition - 2], 
        0, 
        length);
    }
    return result;
  }

  public final String getCurrentStringLiteral()
  {
    if (this.withoutUnicodePtr != 0)
    {
      return new String(this.withoutUnicodeBuffer, 2, this.withoutUnicodePtr - 2);
    }
    return new String(this.source, this.startPosition + 1, this.currentPosition - this.startPosition - 2);
  }

  public final char[] getRawTokenSource() {
    int length = this.currentPosition - this.startPosition;
    char[] tokenSource = new char[length];
    System.arraycopy(this.source, this.startPosition, tokenSource, 0, length);
    return tokenSource;
  }

  public final char[] getRawTokenSourceEnd() {
    int length = this.eofPosition - this.currentPosition - 1;
    char[] sourceEnd = new char[length];
    System.arraycopy(this.source, this.currentPosition, sourceEnd, 0, length);
    return sourceEnd;
  }

  public int getCurrentTokenStartPosition() {
    return this.startPosition;
  }

  public final int getLineEnd(int lineNumber)
  {
    if ((this.lineEnds == null) || (this.linePtr == -1))
      return -1;
    if (lineNumber > this.lineEnds.length + 1)
      return -1;
    if (lineNumber <= 0)
      return -1;
    if (lineNumber == this.lineEnds.length + 1)
      return this.eofPosition;
    return this.lineEnds[(lineNumber - 1)];
  }

  public final int[] getLineEnds()
  {
    if (this.linePtr == -1)
      return EMPTY_LINE_ENDS;
    int[] copy;
    System.arraycopy(this.lineEnds, 0, copy = new int[this.linePtr + 1], 0, this.linePtr + 1);
    return copy;
  }

  public final int getLineStart(int lineNumber)
  {
    if ((this.lineEnds == null) || (this.linePtr == -1))
      return -1;
    if (lineNumber > this.lineEnds.length + 1)
      return -1;
    if (lineNumber <= 0) {
      return -1;
    }
    if (lineNumber == 1)
      return this.initialPosition;
    return this.lineEnds[(lineNumber - 2)] + 1;
  }
  public final int getNextChar() {
    try {
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
      } else {
        this.unicodeAsBackSlash = false;
        if (this.withoutUnicodePtr != 0) {
          unicodeStore();
        }
      }
      return this.currentCharacter;
    } catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      return -1; } catch (InvalidInputException localInvalidInputException) {
    }
    return -1;
  }

  public final int getNextCharWithBoundChecks() {
    if (this.currentPosition >= this.eofPosition) {
      return -1;
    }
    this.currentCharacter = this.source[(this.currentPosition++)];
    if (this.currentPosition >= this.eofPosition) {
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0) {
        unicodeStore();
      }
      return this.currentCharacter;
    }
    if ((this.currentCharacter == '\\') && (this.source[this.currentPosition] == 'u')) {
      try {
        getNextUnicodeChar();
      } catch (InvalidInputException localInvalidInputException) {
        return -1;
      }
    } else {
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0) {
        unicodeStore();
      }
    }
    return this.currentCharacter;
  }

  public final boolean getNextChar(char testedChar)
  {
    if (this.currentPosition >= this.eofPosition) {
      this.unicodeAsBackSlash = false;
      return false;
    }

    int temp = this.currentPosition;
    try {
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        if (this.currentCharacter != testedChar) {
          this.currentPosition = temp;
          this.withoutUnicodePtr -= 1;
          return false;
        }
        return true;
      }

      if (this.currentCharacter != testedChar) {
        this.currentPosition = temp;
        return false;
      }
      this.unicodeAsBackSlash = false;
      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return true;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      this.unicodeAsBackSlash = false;
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException localInvalidInputException) {
      this.unicodeAsBackSlash = false;
      this.currentPosition = temp;
    }return false;
  }

  public final int getNextChar(char testedChar1, char testedChar2)
  {
    if (this.currentPosition >= this.eofPosition) {
      return -1;
    }
    int temp = this.currentPosition;
    try
    {
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        int result;
        if (this.currentCharacter == testedChar1) {
          result = 0;
        }
        else
        {
          int result;
          if (this.currentCharacter == testedChar2) {
            result = 1;
          } else {
            this.currentPosition = temp;
            this.withoutUnicodePtr -= 1; } 
        }return -1;
      }
      int result;
      if (this.currentCharacter == testedChar1) {
        result = 0;
      }
      else
      {
        int result;
        if (this.currentCharacter == testedChar2) {
          result = 1;
        } else {
          this.currentPosition = temp;
          return -1;
        }
      }
      int result;
      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return result;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      this.currentPosition = temp;
      return -1;
    } catch (InvalidInputException localInvalidInputException) {
      this.currentPosition = temp;
    }return -1;
  }

  public final boolean getNextCharAsDigit()
    throws InvalidInputException
  {
    if (this.currentPosition >= this.eofPosition) {
      return false;
    }
    int temp = this.currentPosition;
    try {
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        if (!ScannerHelper.isDigit(this.currentCharacter)) {
          this.currentPosition = temp;
          this.withoutUnicodePtr -= 1;
          return false;
        }
        return true;
      }
      if (!ScannerHelper.isDigit(this.currentCharacter)) {
        this.currentPosition = temp;
        return false;
      }
      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return true;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException localInvalidInputException) {
      this.currentPosition = temp;
    }return false;
  }

  public final boolean getNextCharAsDigit(int radix)
  {
    if (this.currentPosition >= this.eofPosition) {
      return false;
    }
    int temp = this.currentPosition;
    try {
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        if (ScannerHelper.digit(this.currentCharacter, radix) == -1) {
          this.currentPosition = temp;
          this.withoutUnicodePtr -= 1;
          return false;
        }
        return true;
      }
      if (ScannerHelper.digit(this.currentCharacter, radix) == -1) {
        this.currentPosition = temp;
        return false;
      }
      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return true;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      this.currentPosition = temp;
      return false;
    } catch (InvalidInputException localInvalidInputException) {
      this.currentPosition = temp;
    }return false;
  }

  public boolean getNextCharAsJavaIdentifierPartWithBoundCheck()
  {
    int pos = this.currentPosition;
    if (pos >= this.eofPosition) {
      return false;
    }
    int temp2 = this.withoutUnicodePtr;
    try {
      boolean unicode = false;
      this.currentCharacter = this.source[(this.currentPosition++)];
      if ((this.currentPosition < this.eofPosition) && 
        (this.currentCharacter == '\\') && (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        unicode = true;
      }

      char c = this.currentCharacter;
      boolean isJavaIdentifierPart = false;
      if ((c >= 55296) && (c <= 56319)) {
        if (this.complianceLevel < 3211264L) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }

        char low = (char)getNextCharWithBoundChecks();
        if ((low < 56320) || (low > 57343))
        {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(c, low);
      } else {
        if ((c >= 56320) && (c <= 57343)) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(c);
      }
      if (unicode) {
        if (!isJavaIdentifierPart) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        return true;
      }
      if (!isJavaIdentifierPart) {
        this.currentPosition = pos;
        return false;
      }

      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return true;
    }
    catch (InvalidInputException localInvalidInputException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
    }return false;
  }

  public boolean getNextCharAsJavaIdentifierPart()
  {
    int pos;
    if ((pos = this.currentPosition) >= this.eofPosition) {
      return false;
    }
    int temp2 = this.withoutUnicodePtr;
    try {
      boolean unicode = false;
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
        unicode = true;
      }
      char c = this.currentCharacter;
      boolean isJavaIdentifierPart = false;
      if ((c >= 55296) && (c <= 56319)) {
        if (this.complianceLevel < 3211264L) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }

        char low = (char)getNextChar();
        if ((low < 56320) || (low > 57343))
        {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(c, low);
      } else {
        if ((c >= 56320) && (c <= 57343)) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        isJavaIdentifierPart = ScannerHelper.isJavaIdentifierPart(c);
      }
      if (unicode) {
        if (!isJavaIdentifierPart) {
          this.currentPosition = pos;
          this.withoutUnicodePtr = temp2;
          return false;
        }
        return true;
      }
      if (!isJavaIdentifierPart) {
        this.currentPosition = pos;
        return false;
      }

      if (this.withoutUnicodePtr != 0)
        unicodeStore();
      return true;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
      return false;
    } catch (InvalidInputException localInvalidInputException) {
      this.currentPosition = pos;
      this.withoutUnicodePtr = temp2;
    }return false;
  }
  public int scanIdentifier() throws InvalidInputException { int whiteStart = 0;

    this.withoutUnicodePtr = 0;

    whiteStart = this.currentPosition;
    boolean hasWhiteSpaces = false;

    boolean checkIfUnicode = false;
    int unicodePtr;
    int offset;
    boolean isWhiteSpace;
    do { unicodePtr = this.withoutUnicodePtr;
      offset = this.currentPosition;
      this.startPosition = this.currentPosition;
      if (this.currentPosition < this.eofPosition) {
        this.currentCharacter = this.source[(this.currentPosition++)];
        checkIfUnicode = (this.currentPosition < this.eofPosition) && 
          (this.currentCharacter == '\\') && 
          (this.source[this.currentPosition] == 'u'); } else {
        if ((this.tokenizeWhiteSpace) && (whiteStart != this.currentPosition - 1))
        {
          this.currentPosition -= 1;
          this.startPosition = whiteStart;
          return 1000;
        }
        return 68;
      }
      if (checkIfUnicode) {
        boolean isWhiteSpace = jumpOverUnicodeWhiteSpace();
        offset = this.currentPosition - offset;
      } else {
        offset = this.currentPosition - offset;
        boolean isWhiteSpace;
        switch (this.currentCharacter) {
        case '\t':
        case '\n':
        case '\f':
        case '\r':
        case ' ':
          isWhiteSpace = true;
          break;
        default:
          isWhiteSpace = false;
        }
      }
      if (isWhiteSpace)
        hasWhiteSpaces = true;
    }
    while (
      isWhiteSpace);
    if (hasWhiteSpaces) {
      if (this.tokenizeWhiteSpace)
      {
        this.currentPosition -= offset;
        this.startPosition = whiteStart;
        if (checkIfUnicode) {
          this.withoutUnicodePtr = unicodePtr;
        }
        return 1000;
      }if (checkIfUnicode) {
        this.withoutUnicodePtr = 0;
        unicodeStore();
      } else {
        this.withoutUnicodePtr = 0;
      }
    }
    char c = this.currentCharacter;
    if (c < '') {
      if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0) {
        return scanIdentifierOrKeywordWithBoundCheck();
      }
      return 110;
    }
    boolean isJavaIdStart;
    boolean isJavaIdStart;
    if ((c >= 55296) && (c <= 56319)) {
      if (this.complianceLevel < 3211264L) {
        throw new InvalidInputException("Invalid_Unicode_Escape");
      }

      char low = (char)getNextCharWithBoundChecks();
      if ((low < 56320) || (low > 57343))
      {
        throw new InvalidInputException("Invalid_Low_Surrogate");
      }
      isJavaIdStart = ScannerHelper.isJavaIdentifierStart(c, low); } else {
      if ((c >= 56320) && (c <= 57343)) {
        if (this.complianceLevel < 3211264L) {
          throw new InvalidInputException("Invalid_Unicode_Escape");
        }
        throw new InvalidInputException("Invalid_High_Surrogate");
      }

      isJavaIdStart = Character.isJavaIdentifierStart(c);
    }
    if (isJavaIdStart)
      return scanIdentifierOrKeywordWithBoundCheck();
    return 110; }

  public int getNextToken() throws InvalidInputException
  {
    this.wasAcr = false;
    if (this.diet) {
      jumpOverMethodBody();
      this.diet = false;
      return this.currentPosition > this.eofPosition ? 68 : 31;
    }
    int whiteStart = 0;
    try { while (true) { this.withoutUnicodePtr = 0;

        whiteStart = this.currentPosition;
        boolean hasWhiteSpaces = false;

        boolean checkIfUnicode = false;
        int unicodePtr;
        int offset;
        boolean isWhiteSpace;
        do { unicodePtr = this.withoutUnicodePtr;
          offset = this.currentPosition;
          this.startPosition = this.currentPosition;
          try {
            checkIfUnicode = ((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
              (this.source[this.currentPosition] == 'u');
          } catch (IndexOutOfBoundsException localIndexOutOfBoundsException1) {
            if ((this.tokenizeWhiteSpace) && (whiteStart != this.currentPosition - 1))
            {
              this.currentPosition -= 1;
              this.startPosition = whiteStart;
              return 1000;
            }
            if (this.currentPosition > this.eofPosition)
              return 68;
          }
          if (this.currentPosition > this.eofPosition)
            return 68;
          if (checkIfUnicode) {
            boolean isWhiteSpace = jumpOverUnicodeWhiteSpace();
            offset = this.currentPosition - offset;
          } else {
            offset = this.currentPosition - offset;
            if (((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) && 
              (this.recordLineSeparator))
              pushLineSeparator();
            boolean isWhiteSpace;
            switch (this.currentCharacter) {
            case '\t':
            case '\n':
            case '\f':
            case '\r':
            case ' ':
              isWhiteSpace = true;
              break;
            default:
              isWhiteSpace = false;
            }
          }
          if (isWhiteSpace)
            hasWhiteSpaces = true;
        }
        while (
          isWhiteSpace);
        if (hasWhiteSpaces) {
          if (this.tokenizeWhiteSpace)
          {
            this.currentPosition -= offset;
            this.startPosition = whiteStart;
            if (checkIfUnicode) {
              this.withoutUnicodePtr = unicodePtr;
            }
            return 1000;
          }if (checkIfUnicode) {
            this.withoutUnicodePtr = 0;
            unicodeStore();
          } else {
            this.withoutUnicodePtr = 0;
          }
        }

        switch (this.currentCharacter)
        {
        case '@':
          return 53;
        case '(':
          return 28;
        case ')':
          return 29;
        case '{':
          return 69;
        case '}':
          return 31;
        case '[':
          return 14;
        case ']':
          return 70;
        case ';':
          return 27;
        case ',':
          return 30;
        case '.':
          if (getNextCharAsDigit()) {
            return scanNumber(true);
          }
          int temp = this.currentPosition;
          if (getNextChar('.')) {
            if (getNextChar('.')) {
              return 107;
            }
            this.currentPosition = temp;
            return 3;
          }

          this.currentPosition = temp;
          return 3;
        case '+':
          int test;
          if ((test = getNextChar('+', '=')) == 0)
            return 9;
          if (test > 0)
            return 84;
          return 1;
        case '-':
          int test;
          if ((test = getNextChar('-', '=')) == 0)
            return 10;
          if (test > 0)
            return 85;
          return 2;
        case '~':
          return 67;
        case '!':
          if (getNextChar('='))
            return 19;
          return 66;
        case '*':
          if (getNextChar('='))
            return 86;
          return 4;
        case '%':
          if (getNextChar('='))
            return 91;
          return 5;
        case '<':
          int test;
          if ((test = getNextChar('=', '<')) == 0)
            return 16;
          if (test > 0) {
            if (getNextChar('='))
              return 92;
            return 13;
          }
          return 7;
        case '>':
          if (this.returnOnlyGreater)
            return 12;
          int test;
          if ((test = getNextChar('=', '>')) == 0)
            return 17;
          if (test > 0) {
            if ((test = getNextChar('=', '>')) == 0)
              return 93;
            if (test > 0) {
              if (getNextChar('='))
                return 94;
              return 11;
            }
            return 8;
          }
          return 12;
        case '=':
          if (getNextChar('='))
            return 18;
          return 71;
        case '&':
          int test;
          if ((test = getNextChar('&', '=')) == 0)
            return 24;
          if (test > 0)
            return 88;
          return 20;
        case '|':
          int test;
          if ((test = getNextChar('|', '=')) == 0)
            return 25;
          if (test > 0)
            return 89;
          return 22;
        case '^':
          if (getNextChar('='))
            return 90;
          return 21;
        case '?':
          return 23;
        case ':':
          return 65;
        case '\'':
          int test;
          if ((test = getNextChar('\n', '\r')) == 0) {
            throw new InvalidInputException("Invalid_Character_Constant");
          }
          if (test > 0)
          {
            for (int lookAhead = 0; lookAhead < 3; lookAhead++) {
              if (this.currentPosition + lookAhead == this.eofPosition)
                break;
              if (this.source[(this.currentPosition + lookAhead)] == '\n')
                break;
              if (this.source[(this.currentPosition + lookAhead)] == '\'') {
                this.currentPosition += lookAhead + 1;
                break;
              }
            }
            throw new InvalidInputException("Invalid_Character_Constant");
          }

          if (getNextChar('\''))
          {
            for (int lookAhead = 0; lookAhead < 3; lookAhead++) {
              if (this.currentPosition + lookAhead == this.eofPosition)
                break;
              if (this.source[(this.currentPosition + lookAhead)] == '\n')
                break;
              if (this.source[(this.currentPosition + lookAhead)] == '\'') {
                this.currentPosition += lookAhead + 1;
                break;
              }
            }
            throw new InvalidInputException("Invalid_Character_Constant");
          }
          if (getNextChar('\\')) {
            if (this.unicodeAsBackSlash)
            {
              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }
              else if (this.withoutUnicodePtr != 0)
                unicodeStore();
            }
            else
            {
              this.currentCharacter = this.source[(this.currentPosition++)];
            }
            scanEscapeCharacter();
          } else {
            this.unicodeAsBackSlash = false;
            checkIfUnicode = false;
            try {
              checkIfUnicode = ((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u');
            } catch (IndexOutOfBoundsException localIndexOutOfBoundsException2) {
              this.currentPosition -= 1;
              throw new InvalidInputException("Invalid_Character_Constant");
            }
            if (checkIfUnicode) {
              getNextUnicodeChar();
            }
            else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            }
          }

          if (getNextChar('\'')) {
            return 51;
          }
          for (int lookAhead = 0; lookAhead < 20; lookAhead++) {
            if (this.currentPosition + lookAhead == this.eofPosition)
              break;
            if (this.source[(this.currentPosition + lookAhead)] == '\n')
              break;
            if (this.source[(this.currentPosition + lookAhead)] == '\'') {
              this.currentPosition += lookAhead + 1;
              break;
            }
          }
          throw new InvalidInputException("Invalid_Character_Constant");
        case '"':
          try
          {
            this.unicodeAsBackSlash = false;
            boolean isUnicode = false;
            if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
              (this.source[this.currentPosition] == 'u')) {
              getNextUnicodeChar();
              isUnicode = true;
            }
            else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            }

            while (this.currentCharacter != '"')
            {
              if ((this.currentCharacter == '\n') || (this.currentCharacter == '\r'))
              {
                if (isUnicode) {
                  int start = this.currentPosition;
                  for (int lookAhead = 0; lookAhead < 50; lookAhead++) {
                    if (this.currentPosition >= this.eofPosition) {
                      this.currentPosition = start;
                      break;
                    }
                    if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && (this.source[this.currentPosition] == 'u')) {
                      isUnicode = true;
                      getNextUnicodeChar();
                    } else {
                      isUnicode = false;
                    }
                    if ((!isUnicode) && (this.currentCharacter == '\n')) {
                      this.currentPosition -= 1;
                      break;
                    }
                    if (this.currentCharacter == '"')
                      throw new InvalidInputException("Invalid_Char_In_String");
                  }
                }
                else {
                  this.currentPosition -= 1;
                }
                throw new InvalidInputException("Invalid_Char_In_String");
              }
              if (this.currentCharacter == '\\') {
                if (this.unicodeAsBackSlash) {
                  this.withoutUnicodePtr -= 1;

                  this.unicodeAsBackSlash = false;
                  if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && (this.source[this.currentPosition] == 'u')) {
                    getNextUnicodeChar();
                    isUnicode = true;
                    this.withoutUnicodePtr -= 1;
                  } else {
                    isUnicode = false;
                  }
                } else {
                  if (this.withoutUnicodePtr == 0) {
                    unicodeInitializeBuffer(this.currentPosition - this.startPosition);
                  }
                  this.withoutUnicodePtr -= 1;
                  this.currentCharacter = this.source[(this.currentPosition++)];
                }

                scanEscapeCharacter();
                if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                }
              }

              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
                isUnicode = true;
              } else {
                isUnicode = false;
                if (this.withoutUnicodePtr != 0)
                  unicodeStore();
              }
            }
          }
          catch (IndexOutOfBoundsException localIndexOutOfBoundsException3)
          {
            this.currentPosition -= 1;
            throw new InvalidInputException("Unterminated_String");
          } catch (InvalidInputException e) {
            if (e.getMessage().equals("Invalid_Escape"))
            {
              for (int lookAhead = 0; lookAhead < 50; lookAhead++) {
                if (this.currentPosition + lookAhead == this.eofPosition)
                  break;
                if (this.source[(this.currentPosition + lookAhead)] == '\n')
                  break;
                if (this.source[(this.currentPosition + lookAhead)] == '"') {
                  this.currentPosition += lookAhead + 1;
                  break;
                }
              }
            }

            throw e;
          }
          return 52;
        case '/':
          int test;
          if ((test = getNextChar('/', '*')) == 0) {
            this.lastCommentLinePosition = this.currentPosition;
            try {
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }

              if ((this.currentCharacter == '\\') && 
                (this.source[this.currentPosition] == '\\')) {
                this.currentPosition += 1;
              }
              boolean isUnicode = false;
              while ((this.currentCharacter != '\r') && (this.currentCharacter != '\n')) {
                this.lastCommentLinePosition = this.currentPosition;

                isUnicode = false;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                  (this.source[this.currentPosition] == 'u')) {
                  getNextUnicodeChar();
                  isUnicode = true;
                }

                if ((this.currentCharacter == '\\') && 
                  (this.source[this.currentPosition] == '\\')) {
                  this.currentPosition += 1;
                }

              }

              if ((this.currentCharacter == '\r') && 
                (this.eofPosition > this.currentPosition)) {
                if (this.source[this.currentPosition] == '\n') {
                  this.currentPosition += 1;
                  this.currentCharacter = '\n';
                } else if ((this.source[this.currentPosition] == '\\') && 
                  (this.source[(this.currentPosition + 1)] == 'u')) {
                  getNextUnicodeChar();
                  isUnicode = true;
                }
              }
              recordComment(1001);
              if (this.taskTags != null) checkTaskTag(this.startPosition, this.currentPosition);
              if ((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) {
                if (this.checkNonExternalizedStringLiterals) {
                  parseTags();
                }
                if (this.recordLineSeparator) {
                  if (isUnicode)
                    pushUnicodeLineSeparator();
                  else {
                    pushLineSeparator();
                  }
                }
              }
              if (!this.tokenizeComments) continue;
              return 1001;
            }
            catch (IndexOutOfBoundsException localIndexOutOfBoundsException4) {
              this.currentPosition -= 1;
              recordComment(1001);
              if (this.taskTags != null) checkTaskTag(this.startPosition, this.currentPosition);
              if (this.checkNonExternalizedStringLiterals) {
                parseTags();
              }
              if (this.tokenizeComments) {
                return 1001;
              }
              this.currentPosition += 1;
            }

          }
          else if (test > 0) {
            try {
              boolean isJavadoc = false; boolean star = false;
              boolean isUnicode = false;

              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
                isUnicode = true;
              } else {
                isUnicode = false;
                if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                }
              }

              if (this.currentCharacter == '*') {
                isJavadoc = true;
                star = true;
              }
              if (((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) && 
                (this.recordLineSeparator)) {
                if (isUnicode)
                  pushUnicodeLineSeparator();
                else {
                  pushLineSeparator();
                }
              }

              isUnicode = false;
              int previous = this.currentPosition;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u'))
              {
                getNextUnicodeChar();
                isUnicode = true;
              } else {
                isUnicode = false;
              }

              if ((this.currentCharacter == '\\') && 
                (this.source[this.currentPosition] == '\\')) {
                this.currentPosition += 1;
              }

              if (this.currentCharacter == '/') {
                isJavadoc = false;
              }

              int firstTag = 0;
              while ((this.currentCharacter != '/') || (!star)) {
                if (((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) && 
                  (this.recordLineSeparator)) {
                  if (isUnicode)
                    pushUnicodeLineSeparator();
                  else {
                    pushLineSeparator();
                  }
                }

                switch (this.currentCharacter) {
                case '*':
                  star = true;
                  break;
                case '@':
                  if (firstTag == 0) {
                    firstTag = previous;
                  }
                  break;
                }
                star = false;

                previous = this.currentPosition;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                  (this.source[this.currentPosition] == 'u'))
                {
                  getNextUnicodeChar();
                  isUnicode = true;
                } else {
                  isUnicode = false;
                }

                if ((this.currentCharacter == '\\') && 
                  (this.source[this.currentPosition] == '\\')) {
                  this.currentPosition += 1;
                }
              }
              int token = isJavadoc ? 1003 : 1002;
              recordComment(token);
              this.commentTagStarts[this.commentPtr] = firstTag;
              if (this.taskTags != null) checkTaskTag(this.startPosition, this.currentPosition);
              if (this.tokenizeComments)
              {
                return token;
              }
            } catch (IndexOutOfBoundsException localIndexOutOfBoundsException5) {
              this.currentPosition -= 1;
              throw new InvalidInputException("Unterminated_Comment");
            }
          }break;
        case '\032':
        } } if (getNextChar('='))
        return 87;
      return 6;

      if (atEnd()) {
        return 68;
      }
      throw new InvalidInputException("Ctrl-Z");

      char c = this.currentCharacter;
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0)
          return scanIdentifierOrKeyword();
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x4) != 0) {
          return scanNumber(false);
        }
        return 110;
      }
      boolean isJavaIdStart;
      boolean isJavaIdStart;
      if ((c >= 55296) && (c <= 56319)) {
        if (this.complianceLevel < 3211264L) {
          throw new InvalidInputException("Invalid_Unicode_Escape");
        }

        char low = (char)getNextChar();
        if ((low < 56320) || (low > 57343))
        {
          throw new InvalidInputException("Invalid_Low_Surrogate");
        }
        isJavaIdStart = ScannerHelper.isJavaIdentifierStart(c, low);
      } else {
        if ((c >= 56320) && (c <= 57343)) {
          if (this.complianceLevel < 3211264L) {
            throw new InvalidInputException("Invalid_Unicode_Escape");
          }
          throw new InvalidInputException("Invalid_High_Surrogate");
        }

        isJavaIdStart = Character.isJavaIdentifierStart(c);
      }
      if (isJavaIdStart)
        return scanIdentifierOrKeyword();
      if (ScannerHelper.isDigit(this.currentCharacter)) {
        return scanNumber(false);
      }
      return 110;
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException6)
    {
      if ((this.tokenizeWhiteSpace) && (whiteStart != this.currentPosition - 1))
      {
        this.currentPosition -= 1;
        this.startPosition = whiteStart;
        return 1000;
      }
    }
    return 68;
  }

  public void getNextUnicodeChar()
    throws InvalidInputException
  {
    int c1 = 0; int c2 = 0; int c3 = 0; int c4 = 0; int unicodeSize = 6;
    this.currentPosition += 1;
    if (this.currentPosition < this.eofPosition) {
      while (this.source[this.currentPosition] == 'u') {
        this.currentPosition += 1;
        if (this.currentPosition >= this.eofPosition) {
          this.currentPosition -= 1;
          throw new InvalidInputException("Invalid_Unicode_Escape");
        }
        unicodeSize++;
      }
    } else {
      this.currentPosition -= 1;
      throw new InvalidInputException("Invalid_Unicode_Escape");
    }

    if (this.currentPosition + 4 > this.eofPosition) {
      this.currentPosition += this.eofPosition - this.currentPosition;
      throw new InvalidInputException("Invalid_Unicode_Escape");
    }
    if (((c1 = ScannerHelper.getNumericValue(this.source[(this.currentPosition++)])) > 15) || 
      (c1 < 0) || 
      ((c2 = ScannerHelper.getNumericValue(this.source[(this.currentPosition++)])) > 15) || 
      (c2 < 0) || 
      ((c3 = ScannerHelper.getNumericValue(this.source[(this.currentPosition++)])) > 15) || 
      (c3 < 0) || 
      ((c4 = ScannerHelper.getNumericValue(this.source[(this.currentPosition++)])) > 15) || 
      (c4 < 0)) {
      throw new InvalidInputException("Invalid_Unicode_Escape");
    }
    this.currentCharacter = ((char)(((c1 * 16 + c2) * 16 + c3) * 16 + c4));

    if (this.withoutUnicodePtr == 0)
    {
      unicodeInitializeBuffer(this.currentPosition - unicodeSize - this.startPosition);
    }

    unicodeStore();
    this.unicodeAsBackSlash = (this.currentCharacter == '\\');
  }
  public NLSTag[] getNLSTags() {
    int length = this.nlsTagsPtr;
    if (length != 0) {
      NLSTag[] result = new NLSTag[length];
      System.arraycopy(this.nlsTags, 0, result, 0, length);
      this.nlsTagsPtr = 0;
      return result;
    }
    return null;
  }
  public char[] getSource() {
    return this.source;
  }

  public final void jumpOverMethodBody() {
    this.wasAcr = false;
    int found = 1;
    try {
      while (true) {
        this.withoutUnicodePtr = 0;
        boolean isWhiteSpace;
        do {
          this.startPosition = this.currentPosition;
          boolean isWhiteSpace;
          if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
            (this.source[this.currentPosition] == 'u')) {
            isWhiteSpace = jumpOverUnicodeWhiteSpace();
          } else {
            if ((this.recordLineSeparator) && (
              (this.currentCharacter == '\r') || (this.currentCharacter == '\n'))) {
              pushLineSeparator();
            }
            isWhiteSpace = CharOperation.isWhitespace(this.currentCharacter);
          }
        }
        while (
          isWhiteSpace);

        switch (this.currentCharacter) {
        case '{':
          found++;
          break;
        case '}':
          found--;
          if (found == 0)
          {
            return;
          }
          break;
        case '\'':
          boolean test = getNextChar('\\');
          if (test)
            try {
              if (this.unicodeAsBackSlash)
              {
                this.unicodeAsBackSlash = false;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && (this.source[this.currentPosition] == 'u')) {
                  getNextUnicodeChar();
                }
                else if (this.withoutUnicodePtr != 0)
                  unicodeStore();
              }
              else
              {
                this.currentCharacter = this.source[(this.currentPosition++)];
              }
              scanEscapeCharacter();
            }
            catch (InvalidInputException localInvalidInputException1) {
            }
          else {
            try {
              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }
              else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              }
            }
            catch (InvalidInputException localInvalidInputException2)
            {
            }
          }
          getNextChar('\'');
          break;
        case '"':
          try
          {
            try {
              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }
              else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              }
            }
            catch (InvalidInputException localInvalidInputException3)
            {
            }
            while (this.currentCharacter != '"') {
              if (this.currentCharacter == '\r') {
                if (this.source[this.currentPosition] != '\n') break; this.currentPosition += 1;
                break;
              }
              if (this.currentCharacter == '\n') {
                break;
              }
              if (this.currentCharacter == '\\')
                try {
                  if (this.unicodeAsBackSlash)
                  {
                    this.unicodeAsBackSlash = false;
                    if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && (this.source[this.currentPosition] == 'u')) {
                      getNextUnicodeChar();
                    }
                    else if (this.withoutUnicodePtr != 0)
                      unicodeStore();
                  }
                  else
                  {
                    this.currentCharacter = this.source[(this.currentPosition++)];
                  }
                  scanEscapeCharacter();
                }
                catch (InvalidInputException localInvalidInputException4)
                {
                }
              try {
                this.unicodeAsBackSlash = false;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                  (this.source[this.currentPosition] == 'u')) {
                  getNextUnicodeChar();
                }
                else if (this.withoutUnicodePtr != 0)
                  unicodeStore();
              }
              catch (InvalidInputException localInvalidInputException5)
              {
              }
            }
          }
          catch (IndexOutOfBoundsException localIndexOutOfBoundsException1)
          {
            return;
          }
        case '/':
          int test;
          if ((test = getNextChar('/', '*')) == 0) {
            try {
              this.lastCommentLinePosition = this.currentPosition;

              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }

              if ((this.currentCharacter == '\\') && 
                (this.source[this.currentPosition] == '\\')) {
                this.currentPosition += 1;
              }
              boolean isUnicode = false;
              while ((this.currentCharacter != '\r') && (this.currentCharacter != '\n')) {
                this.lastCommentLinePosition = this.currentPosition;

                isUnicode = false;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                  (this.source[this.currentPosition] == 'u')) {
                  isUnicode = true;
                  getNextUnicodeChar();
                }

                if ((this.currentCharacter == '\\') && 
                  (this.source[this.currentPosition] == '\\')) {
                  this.currentPosition += 1;
                }

              }

              if ((this.currentCharacter == '\r') && 
                (this.eofPosition > this.currentPosition)) {
                if (this.source[this.currentPosition] == '\n') {
                  this.currentPosition += 1;
                  this.currentCharacter = '\n';
                } else if ((this.source[this.currentPosition] == '\\') && 
                  (this.source[(this.currentPosition + 1)] == 'u')) {
                  isUnicode = true;
                  getNextUnicodeChar();
                }
              }
              recordComment(1001);
              if ((!this.recordLineSeparator) || (
                (this.currentCharacter != '\r') && (this.currentCharacter != '\n'))) continue;
              if (this.checkNonExternalizedStringLiterals) {
                parseTags();
              }
              if (!this.recordLineSeparator) continue;
              if (isUnicode) {
                pushUnicodeLineSeparator(); continue;
              }
              pushLineSeparator();
            }
            catch (IndexOutOfBoundsException localIndexOutOfBoundsException2)
            {
              this.currentPosition -= 1;
              recordComment(1001);
              if (this.checkNonExternalizedStringLiterals)
                parseTags();
            }
            if (!this.tokenizeComments) {
              this.currentPosition += 1;
            }

          }
          else if (test > 0) {
            boolean isJavadoc = false;
            try {
              boolean star = false;

              boolean isUnicode = false;

              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
                isUnicode = true;
              } else {
                isUnicode = false;
                if (this.withoutUnicodePtr != 0) {
                  unicodeStore();
                }
              }

              if (this.currentCharacter == '*') {
                isJavadoc = true;
                star = true;
              }
              if (((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) && 
                (this.recordLineSeparator)) {
                if (isUnicode)
                  pushUnicodeLineSeparator();
                else {
                  pushLineSeparator();
                }
              }

              isUnicode = false;
              int previous = this.currentPosition;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
                isUnicode = true;
              } else {
                isUnicode = false;
              }

              if ((this.currentCharacter == '\\') && 
                (this.source[this.currentPosition] == '\\')) {
                this.currentPosition += 1;
              }

              if (this.currentCharacter == '/') {
                isJavadoc = false;
              }

              int firstTag = 0;
              while ((this.currentCharacter != '/') || (!star)) {
                if (((this.currentCharacter == '\r') || (this.currentCharacter == '\n')) && 
                  (this.recordLineSeparator)) {
                  if (isUnicode)
                    pushUnicodeLineSeparator();
                  else {
                    pushLineSeparator();
                  }
                }

                switch (this.currentCharacter) {
                case '*':
                  star = true;
                  break;
                case '@':
                  if (firstTag == 0) {
                    firstTag = previous;
                  }
                  break;
                }
                star = false;

                previous = this.currentPosition;
                if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                  (this.source[this.currentPosition] == 'u')) {
                  getNextUnicodeChar();
                  isUnicode = true;
                } else {
                  isUnicode = false;
                }

                if ((this.currentCharacter == '\\') && 
                  (this.source[this.currentPosition] == '\\')) {
                  this.currentPosition += 1;
                }
              }
              recordComment(isJavadoc ? 1003 : 1002);
              this.commentTagStarts[this.commentPtr] = firstTag;
            }
            catch (IndexOutOfBoundsException localIndexOutOfBoundsException3)
            {
              return;
            }
          }
          break;
        default:
          try
          {
            char c = this.currentCharacter;
            if (c < '') {
              if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x40) != 0) {
                scanIdentifierOrKeyword();
              }
              else if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x4) != 0) {
                scanNumber(false);
              }

            }
            else if ((c >= 55296) && (c <= 56319)) {
              if (this.complianceLevel < 3211264L) {
                throw new InvalidInputException("Invalid_Unicode_Escape");
              }

              char low = (char)getNextChar();
              if ((low >= 56320) && (low <= 57343))
              {
                boolean isJavaIdStart = ScannerHelper.isJavaIdentifierStart(c, low);
              } } else if ((c < 56320) || (c > 57343))
            {
              boolean isJavaIdStart = Character.isJavaIdentifierStart(c);

              if (isJavaIdStart)
                scanIdentifierOrKeyword();
            }
          }
          catch (InvalidInputException localInvalidInputException6)
          {
          }
        }
      }
    }
    catch (IndexOutOfBoundsException localIndexOutOfBoundsException4)
    {
    }
    catch (InvalidInputException localInvalidInputException7)
    {
    }
  }

  public final boolean jumpOverUnicodeWhiteSpace()
    throws InvalidInputException
  {
    this.wasAcr = false;
    getNextUnicodeChar();
    return CharOperation.isWhitespace(this.currentCharacter);
  }

  final char[] optimizedCurrentTokenSource1()
  {
    char charOne = this.source[this.startPosition];
    switch (charOne) {
    case 'a':
      return charArray_a;
    case 'b':
      return charArray_b;
    case 'c':
      return charArray_c;
    case 'd':
      return charArray_d;
    case 'e':
      return charArray_e;
    case 'f':
      return charArray_f;
    case 'g':
      return charArray_g;
    case 'h':
      return charArray_h;
    case 'i':
      return charArray_i;
    case 'j':
      return charArray_j;
    case 'k':
      return charArray_k;
    case 'l':
      return charArray_l;
    case 'm':
      return charArray_m;
    case 'n':
      return charArray_n;
    case 'o':
      return charArray_o;
    case 'p':
      return charArray_p;
    case 'q':
      return charArray_q;
    case 'r':
      return charArray_r;
    case 's':
      return charArray_s;
    case 't':
      return charArray_t;
    case 'u':
      return charArray_u;
    case 'v':
      return charArray_v;
    case 'w':
      return charArray_w;
    case 'x':
      return charArray_x;
    case 'y':
      return charArray_y;
    case 'z':
      return charArray_z;
    }
    return new char[] { charOne };
  }

  final char[] optimizedCurrentTokenSource2()
  {
    char[] src = this.source;
    int start = this.startPosition;
    char c0;
    char c1;
    int hash = (((c0 = src[start]) << '\006') + (c1 = src[(start + 1)])) % 30;
    char[][] table = this.charArray_length[0][hash];
    int i = this.newEntry2;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && (c1 == charArray[1]))
        return charArray;
      i++; } while (i < 6);

    i = -1;
    int max = this.newEntry2;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && (c1 == charArray[1]))
        return charArray;
      i++; } while (i <= max);

    max++; if (max >= 6) max = 0;
    char[] r;
    System.arraycopy(src, start, r = new char[2], 0, 2);
    int tmp169_167 = max; this.newEntry2 = tmp169_167; return table[tmp169_167] =  = r;
  }

  final char[] optimizedCurrentTokenSource3()
  {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[(start + 1)];
    char c0;
    char c2;
    int hash = (((c0 = src[start]) << '\006') + (c2 = src[(start + 2)])) % 30;

    char[][] table = this.charArray_length[1][hash];
    int i = this.newEntry3;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && (c1 == charArray[1]) && (c2 == charArray[2]))
        return charArray;
      i++; } while (i < 6);

    i = -1;
    int max = this.newEntry3;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && (c1 == charArray[1]) && (c2 == charArray[2]))
        return charArray;
      i++; } while (i <= max);

    max++; if (max >= 6) max = 0;
    char[] r;
    System.arraycopy(src, start, r = new char[3], 0, 3);
    int tmp194_192 = max; this.newEntry3 = tmp194_192; return table[tmp194_192] =  = r;
  }

  final char[] optimizedCurrentTokenSource4()
  {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[(start + 1)]; char c3 = src[(start + 3)];
    char c0;
    char c2;
    int hash = (((c0 = src[start]) << '\006') + (c2 = src[(start + 2)])) % 30;

    char[][] table = this.charArray_length[2][hash];
    int i = this.newEntry4;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]))
        return charArray;
      i++; } while (i < 6);

    i = -1;
    int max = this.newEntry4;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]))
        return charArray;
      i++; } while (i <= max);

    max++; if (max >= 6) max = 0;
    char[] r;
    System.arraycopy(src, start, r = new char[4], 0, 4);
    int tmp219_217 = max; this.newEntry4 = tmp219_217; return table[tmp219_217] =  = r;
  }

  final char[] optimizedCurrentTokenSource5()
  {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[(start + 1)]; char c3 = src[(start + 3)];
    char c0;
    char c2;
    char c4;
    int hash = (((c0 = src[start]) << '\f') + ((c2 = src[(start + 2)]) << '\006') + (c4 = src[(start + 4)])) % 30;

    char[][] table = this.charArray_length[3][hash];
    int i = this.newEntry5;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]) && 
        (c4 == charArray[4]))
        return charArray;
      i++; } while (i < 6);

    i = -1;
    int max = this.newEntry5;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]) && 
        (c4 == charArray[4]))
        return charArray;
      i++; } while (i <= max);

    max++; if (max >= 6) max = 0;
    char[] r;
    System.arraycopy(src, start, r = new char[5], 0, 5);
    int tmp249_247 = max; this.newEntry5 = tmp249_247; return table[tmp249_247] =  = r;
  }

  final char[] optimizedCurrentTokenSource6()
  {
    char[] src = this.source;
    int start = this.startPosition;
    char c1 = src[(start + 1)]; char c3 = src[(start + 3)]; char c5 = src[(start + 5)];
    char c0;
    char c2;
    char c4;
    int hash = (((c0 = src[start]) << '\f') + ((c2 = src[(start + 2)]) << '\006') + (c4 = src[(start + 4)])) % 30;

    char[][] table = this.charArray_length[4][hash];
    int i = this.newEntry6;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]) && 
        (c4 == charArray[4]) && 
        (c5 == charArray[5]))
        return charArray;
      i++; } while (i < 6);

    i = -1;
    int max = this.newEntry6;
    do {
      char[] charArray = table[i];
      if ((c0 == charArray[0]) && 
        (c1 == charArray[1]) && 
        (c2 == charArray[2]) && 
        (c3 == charArray[3]) && 
        (c4 == charArray[4]) && 
        (c5 == charArray[5]))
        return charArray;
      i++; } while (i <= max);

    max++; if (max >= 6) max = 0;
    char[] r;
    System.arraycopy(src, start, r = new char[6], 0, 6);
    int tmp276_274 = max; this.newEntry6 = tmp276_274; return table[tmp276_274] =  = r;
  }

  private void parseTags() {
    int position = 0;
    int currentStartPosition = this.startPosition;
    int currentLinePtr = this.linePtr;
    if (currentLinePtr >= 0) {
      position = this.lineEnds[currentLinePtr] + 1;
    }
    while (ScannerHelper.isWhitespace(this.source[position])) {
      position++;
    }
    if (currentStartPosition == position)
    {
      return;
    }
    char[] s = (char[])null;
    int sourceEnd = this.currentPosition;
    int sourceStart = currentStartPosition;
    int sourceDelta = 0;
    if (this.withoutUnicodePtr != 0)
    {
      System.arraycopy(
        this.withoutUnicodeBuffer, 
        1, 
        s = new char[this.withoutUnicodePtr], 
        0, 
        this.withoutUnicodePtr);
      sourceEnd = this.withoutUnicodePtr;
      sourceStart = 1;
      sourceDelta = currentStartPosition;
    } else {
      s = this.source;
    }
    int pos = CharOperation.indexOf(TAG_PREFIX, s, true, sourceStart, sourceEnd);
    if (pos != -1) {
      if (this.nlsTags == null) {
        this.nlsTags = new NLSTag[10];
        this.nlsTagsPtr = 0;
      }
      while (pos != -1) {
        int start = pos + TAG_PREFIX_LENGTH;
        int end = CharOperation.indexOf('$', s, start, sourceEnd);
        if (end != -1) {
          NLSTag currentTag = null;
          int currentLine = currentLinePtr + 1;
          try {
            currentTag = new NLSTag(pos + sourceDelta, end + sourceDelta, currentLine, extractInt(s, start, end));
          } catch (NumberFormatException localNumberFormatException) {
            currentTag = new NLSTag(pos + sourceDelta, end + sourceDelta, currentLine, -1);
          }
          if (this.nlsTagsPtr == this.nlsTags.length)
          {
            System.arraycopy(this.nlsTags, 0, this.nlsTags = new NLSTag[this.nlsTagsPtr + 10], 0, this.nlsTagsPtr);
          }
          this.nlsTags[(this.nlsTagsPtr++)] = currentTag;
        } else {
          end = start;
        }
        pos = CharOperation.indexOf(TAG_PREFIX, s, true, end, sourceEnd);
      }
    }
  }

  private int extractInt(char[] array, int start, int end) { int value = 0;
    for (int i = start; i < end; i++) {
      char currentChar = array[i];
      int digit = 0;
      switch (currentChar) {
      case '0':
        digit = 0;
        break;
      case '1':
        digit = 1;
        break;
      case '2':
        digit = 2;
        break;
      case '3':
        digit = 3;
        break;
      case '4':
        digit = 4;
        break;
      case '5':
        digit = 5;
        break;
      case '6':
        digit = 6;
        break;
      case '7':
        digit = 7;
        break;
      case '8':
        digit = 8;
        break;
      case '9':
        digit = 9;
        break;
      default:
        throw new NumberFormatException();
      }
      value *= 10;
      if (digit < 0) throw new NumberFormatException();
      value += digit;
    }
    return value;
  }

  public final void pushLineSeparator()
  {
    if (this.currentCharacter == '\r') {
      int separatorPos = this.currentPosition - 1;
      if ((this.linePtr >= 0) && (this.lineEnds[this.linePtr] >= separatorPos)) return;
      int length = this.lineEnds.length;
      if (++this.linePtr >= length)
        System.arraycopy(this.lineEnds, 0, this.lineEnds = new int[length + 250], 0, length);
      this.lineEnds[this.linePtr] = separatorPos;
      try
      {
        if (this.source[this.currentPosition] == '\n')
        {
          this.lineEnds[this.linePtr] = this.currentPosition;
          this.currentPosition += 1;
          this.wasAcr = false;
        } else {
          this.wasAcr = true;
        }
      } catch (IndexOutOfBoundsException localIndexOutOfBoundsException) {
        this.wasAcr = true;
      }

    }
    else if (this.currentCharacter == '\n') {
      if ((this.wasAcr) && (this.lineEnds[this.linePtr] == this.currentPosition - 2))
      {
        this.lineEnds[this.linePtr] = (this.currentPosition - 1);
      } else {
        int separatorPos = this.currentPosition - 1;
        if ((this.linePtr >= 0) && (this.lineEnds[this.linePtr] >= separatorPos)) return;
        int length = this.lineEnds.length;
        if (++this.linePtr >= length)
          System.arraycopy(this.lineEnds, 0, this.lineEnds = new int[length + 250], 0, length);
        this.lineEnds[this.linePtr] = separatorPos;
      }
      this.wasAcr = false;
    }
  }

  public final void pushUnicodeLineSeparator()
  {
    if (this.currentCharacter == '\r') {
      if (this.source[this.currentPosition] == '\n')
        this.wasAcr = false;
      else {
        this.wasAcr = true;
      }

    }
    else if (this.currentCharacter == '\n')
      this.wasAcr = false;
  }

  public void recordComment(int token)
  {
    int stopPosition = this.currentPosition;
    switch (token) {
    case 1001:
      stopPosition = -this.lastCommentLinePosition;
      break;
    case 1002:
      stopPosition = -this.currentPosition;
    }

    int length = this.commentStops.length;
    if (++this.commentPtr >= length) {
      int newLength = length + 300;
      System.arraycopy(this.commentStops, 0, this.commentStops = new int[newLength], 0, length);
      System.arraycopy(this.commentStarts, 0, this.commentStarts = new int[newLength], 0, length);
      System.arraycopy(this.commentTagStarts, 0, this.commentTagStarts = new int[newLength], 0, length);
    }
    this.commentStops[this.commentPtr] = stopPosition;
    this.commentStarts[this.commentPtr] = this.startPosition;
  }

  public void resetTo(int begin, int end)
  {
    this.diet = false;
    this.initialPosition = (this.startPosition = this.currentPosition = begin);
    if ((this.source != null) && (this.source.length < end))
      this.eofPosition = this.source.length;
    else {
      this.eofPosition = (end < 2147483647 ? end + 1 : end);
    }
    this.commentPtr = -1;
    this.foundTaskCount = 0;
  }

  public final void scanEscapeCharacter()
    throws InvalidInputException
  {
    switch (this.currentCharacter) {
    case 'b':
      this.currentCharacter = '\b';
      break;
    case 't':
      this.currentCharacter = '\t';
      break;
    case 'n':
      this.currentCharacter = '\n';
      break;
    case 'f':
      this.currentCharacter = '\f';
      break;
    case 'r':
      this.currentCharacter = '\r';
      break;
    case '"':
      this.currentCharacter = '"';
      break;
    case '\'':
      this.currentCharacter = '\'';
      break;
    case '\\':
      this.currentCharacter = '\\';
      break;
    default:
      int number = ScannerHelper.getNumericValue(this.currentCharacter);
      if ((number >= 0) && (number <= 7)) {
        boolean zeroToThreeNot = number > 3;
        if (ScannerHelper.isDigit(this.currentCharacter = this.source[(this.currentPosition++)])) {
          int digit = ScannerHelper.getNumericValue(this.currentCharacter);
          if ((digit >= 0) && (digit <= 7)) {
            number = number * 8 + digit;
            if (ScannerHelper.isDigit(this.currentCharacter = this.source[(this.currentPosition++)])) {
              if (zeroToThreeNot) {
                this.currentPosition -= 1;
              } else {
                digit = ScannerHelper.getNumericValue(this.currentCharacter);
                if ((digit >= 0) && (digit <= 7))
                  number = number * 8 + digit;
                else
                  this.currentPosition -= 1;
              }
            }
            else
              this.currentPosition -= 1;
          }
          else {
            this.currentPosition -= 1;
          }
        } else {
          this.currentPosition -= 1;
        }
        if (number > 255)
          throw new InvalidInputException("Invalid_Escape");
        this.currentCharacter = ((char)number);
      } else {
        throw new InvalidInputException("Invalid_Escape");
      }
      break;
    }
  }

  public int scanIdentifierOrKeywordWithBoundCheck()
  {
    this.useAssertAsAnIndentifier = false;
    this.useEnumAsAnIndentifier = false;

    char[] src = this.source;

    int srcLength = this.eofPosition;
    int pos;
    while ((pos = this.currentPosition) < srcLength)
    {
      char c = src[pos];
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 
          0x3C) != 0) {
          if (this.withoutUnicodePtr != 0) {
            this.currentCharacter = c;
            unicodeStore();
          }
          this.currentPosition += 1;
        } else if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x102) != 0) {
          this.currentCharacter = c;
        }
        else
        {
          while (getNextCharAsJavaIdentifierPartWithBoundCheck());
        }
      }
      else
      {
        while (getNextCharAsJavaIdentifierPartWithBoundCheck());
      }
    }
    int index;
    int length;
    char[] data;
    int index;
    if (this.withoutUnicodePtr == 0)
    {
      int length;
      if ((length = this.currentPosition - this.startPosition) == 1) {
        return 26;
      }
      char[] data = this.source;
      index = this.startPosition;
    } else {
      if ((length = this.withoutUnicodePtr) == 1)
        return 26;
      data = this.withoutUnicodeBuffer;
      index = 1;
    }

    return internalScanIdentifierOrKeyword(index, length, data);
  }

  public int scanIdentifierOrKeyword()
  {
    this.useAssertAsAnIndentifier = false;
    this.useEnumAsAnIndentifier = false;

    char[] src = this.source;

    int srcLength = this.eofPosition;
    int pos;
    while ((pos = this.currentPosition) < srcLength)
    {
      char c = src[pos];
      if (c < '') {
        if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 
          0x3C) != 0) {
          if (this.withoutUnicodePtr != 0) {
            this.currentCharacter = c;
            unicodeStore();
          }
          this.currentPosition += 1;
        } else if ((ScannerHelper.OBVIOUS_IDENT_CHAR_NATURES[c] & 0x102) != 0) {
          this.currentCharacter = c;
        }
        else
        {
          while (getNextCharAsJavaIdentifierPart());
        }
      }
      else
      {
        while (getNextCharAsJavaIdentifierPart());
      }
    }
    int index;
    int length;
    char[] data;
    int index;
    if (this.withoutUnicodePtr == 0)
    {
      int length;
      if ((length = this.currentPosition - this.startPosition) == 1) {
        return 26;
      }
      char[] data = this.source;
      index = this.startPosition;
    } else {
      if ((length = this.withoutUnicodePtr) == 1)
        return 26;
      data = this.withoutUnicodeBuffer;
      index = 1;
    }

    return internalScanIdentifierOrKeyword(index, length, data);
  }

  private int internalScanIdentifierOrKeyword(int index, int length, char[] data) {
    switch (data[index]) {
    case 'a':
      switch (length) {
      case 8:
        if ((data[(++index)] == 'b') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 't')) {
          return 56;
        }
        return 26;
      case 6:
        if ((data[(++index)] == 's') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 't')) {
          if (this.sourceLevel >= 3145728L) {
            this.containsAssertKeyword = true;
            return 74;
          }
          this.useAssertAsAnIndentifier = true;
          return 26;
        }

        return 26;
      case 7:
      }
      return 26;
    case 'b':
      switch (length) {
      case 4:
        if ((data[(++index)] == 'y') && (data[(++index)] == 't') && (data[(++index)] == 'e')) {
          return 33;
        }
        return 26;
      case 5:
        if ((data[(++index)] == 'r') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'k')) {
          return 75;
        }
        return 26;
      case 7:
        if ((data[(++index)] == 'o') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'n')) {
          return 32;
        }
        return 26;
      case 6:
      }return 26;
    case 'c':
      switch (length) {
      case 4:
        if (data[(++index)] == 'a') {
          if ((data[(++index)] == 's') && (data[(++index)] == 'e')) {
            return 101;
          }
          return 26;
        }
        if ((data[index] == 'h') && (data[(++index)] == 'a') && (data[(++index)] == 'r')) {
          return 34;
        }
        return 26;
      case 5:
        if (data[(++index)] == 'a') {
          if ((data[(++index)] == 't') && (data[(++index)] == 'c') && (data[(++index)] == 'h')) {
            return 102;
          }
          return 26;
        }
        if (data[index] == 'l') {
          if ((data[(++index)] == 'a') && 
            (data[(++index)] == 's') && 
            (data[(++index)] == 's')) {
            return 72;
          }
          return 26;
        }if ((data[index] == 'o') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 't')) {
          return 108;
        }
        return 26;
      case 8:
        if ((data[(++index)] == 'o') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'u') && 
          (data[(++index)] == 'e')) {
          return 76;
        }
        return 26;
      case 6:
      case 7: } return 26;
    case 'd':
      switch (length) {
      case 2:
        if (data[(++index)] == 'o') {
          return 77;
        }
        return 26;
      case 6:
        if ((data[(++index)] == 'o') && 
          (data[(++index)] == 'u') && 
          (data[(++index)] == 'b') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'e')) {
          return 35;
        }
        return 26;
      case 7:
        if ((data[(++index)] == 'e') && 
          (data[(++index)] == 'f') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'u') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 't')) {
          return 97;
        }
        return 26;
      case 3:
      case 4:
      case 5: } return 26;
    case 'e':
      switch (length) {
      case 4:
        if ((data[(++index)] == 'l') && (data[(++index)] == 's') && (data[(++index)] == 'e'))
          return 103;
        if ((data[index] == 'n') && 
          (data[(++index)] == 'u') && 
          (data[(++index)] == 'm')) {
          if (this.sourceLevel >= 3211264L) {
            return 98;
          }
          this.useEnumAsAnIndentifier = true;
          return 26;
        }

        return 26;
      case 7:
        if ((data[(++index)] == 'x') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'd') && 
          (data[(++index)] == 's')) {
          return 99;
        }
        return 26;
      case 5:
      case 6: } return 26;
    case 'f':
      switch (length) {
      case 3:
        if ((data[(++index)] == 'o') && (data[(++index)] == 'r')) {
          return 78;
        }
        return 26;
      case 5:
        if (data[(++index)] == 'i') {
          if ((data[(++index)] == 'n') && 
            (data[(++index)] == 'a') && 
            (data[(++index)] == 'l')) {
            return 57;
          }
          return 26;
        }
        if (data[index] == 'l') {
          if ((data[(++index)] == 'o') && 
            (data[(++index)] == 'a') && 
            (data[(++index)] == 't')) {
            return 36;
          }
          return 26;
        }
        if ((data[index] == 'a') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 'e')) {
          return 44;
        }
        return 26;
      case 7:
        if ((data[(++index)] == 'i') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'y')) {
          return 104;
        }
        return 26;
      case 4:
      case 6:
      }return 26;
    case 'g':
      if ((length == 4) && 
        (data[(++index)] == 'o') && 
        (data[(++index)] == 't') && 
        (data[(++index)] == 'o')) {
        return 109;
      }

      return 26;
    case 'i':
      switch (length) {
      case 2:
        if (data[(++index)] == 'f') {
          return 79;
        }
        return 26;
      case 3:
        if ((data[(++index)] == 'n') && (data[(++index)] == 't')) {
          return 37;
        }
        return 26;
      case 6:
        if ((data[(++index)] == 'm') && 
          (data[(++index)] == 'p') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 't')) {
          return 100;
        }
        return 26;
      case 9:
        if ((data[(++index)] == 'n') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'f') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 'e')) {
          return 95;
        }
        return 26;
      case 10:
        if (data[(++index)] == 'm') {
          if ((data[(++index)] == 'p') && 
            (data[(++index)] == 'l') && 
            (data[(++index)] == 'e') && 
            (data[(++index)] == 'm') && 
            (data[(++index)] == 'e') && 
            (data[(++index)] == 'n') && 
            (data[(++index)] == 't') && 
            (data[(++index)] == 's')) {
            return 106;
          }
          return 26;
        }
        if ((data[index] == 'n') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'f')) {
          return 15;
        }
        return 26;
      case 4:
      case 5:
      case 7:
      case 8: } return 26;
    case 'l':
      if ((length == 4) && 
        (data[(++index)] == 'o') && 
        (data[(++index)] == 'n') && 
        (data[(++index)] == 'g')) {
        return 38;
      }

      return 26;
    case 'n':
      switch (length) {
      case 3:
        if ((data[(++index)] == 'e') && (data[(++index)] == 'w')) {
          return 43;
        }
        return 26;
      case 4:
        if ((data[(++index)] == 'u') && (data[(++index)] == 'l') && (data[(++index)] == 'l')) {
          return 45;
        }
        return 26;
      case 6:
        if ((data[(++index)] == 'a') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'v') && 
          (data[(++index)] == 'e')) {
          return 58;
        }
        return 26;
      case 5:
      }return 26;
    case 'p':
      switch (length) {
      case 6:
        if ((data[(++index)] == 'u') && 
          (data[(++index)] == 'b') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'c')) {
          return 61;
        }
        return 26;
      case 7:
        if (data[(++index)] == 'a') {
          if ((data[(++index)] == 'c') && 
            (data[(++index)] == 'k') && 
            (data[(++index)] == 'a') && 
            (data[(++index)] == 'g') && 
            (data[(++index)] == 'e')) {
            return 96;
          }
          return 26;
        }
        if ((data[index] == 'r') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'v') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'e')) {
          return 59;
        }
        return 26;
      case 9:
        if ((data[(++index)] == 'r') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'd')) {
          return 60;
        }
        return 26;
      case 8:
      }
      return 26;
    case 'r':
      if ((length == 6) && 
        (data[(++index)] == 'e') && 
        (data[(++index)] == 't') && 
        (data[(++index)] == 'u') && 
        (data[(++index)] == 'r') && 
        (data[(++index)] == 'n')) {
        return 80;
      }

      return 26;
    case 's':
      switch (length) {
      case 5:
        if (data[(++index)] == 'h') {
          if ((data[(++index)] == 'o') && (data[(++index)] == 'r') && (data[(++index)] == 't')) {
            return 39;
          }
          return 26;
        }
        if ((data[index] == 'u') && 
          (data[(++index)] == 'p') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'r')) {
          return 41;
        }
        return 26;
      case 6:
        if (data[(++index)] == 't') {
          if ((data[(++index)] == 'a') && 
            (data[(++index)] == 't') && 
            (data[(++index)] == 'i') && 
            (data[(++index)] == 'c')) {
            return 54;
          }
          return 26;
        }
        if ((data[index] == 'w') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 'h')) {
          return 81;
        }
        return 26;
      case 8:
        if ((data[(++index)] == 't') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'f') && 
          (data[(++index)] == 'p')) {
          return 62;
        }
        return 26;
      case 12:
        if ((data[(++index)] == 'y') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'c') && 
          (data[(++index)] == 'h') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'z') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'd')) {
          return 55;
        }
        return 26;
      case 7:
      case 9:
      case 10:
      case 11: } return 26;
    case 't':
      switch (length) {
      case 3:
        if ((data[(++index)] == 'r') && (data[(++index)] == 'y')) {
          return 83;
        }
        return 26;
      case 4:
        if (data[(++index)] == 'h') {
          if ((data[(++index)] == 'i') && (data[(++index)] == 's')) {
            return 42;
          }
          return 26;
        }
        if ((data[index] == 'r') && (data[(++index)] == 'u') && (data[(++index)] == 'e')) {
          return 46;
        }
        return 26;
      case 5:
        if ((data[(++index)] == 'h') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'w')) {
          return 82;
        }
        return 26;
      case 6:
        if ((data[(++index)] == 'h') && 
          (data[(++index)] == 'r') && 
          (data[(++index)] == 'o') && 
          (data[(++index)] == 'w') && 
          (data[(++index)] == 's')) {
          return 105;
        }
        return 26;
      case 9:
        if ((data[(++index)] == 'r') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 's') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'e') && 
          (data[(++index)] == 'n') && 
          (data[(++index)] == 't')) {
          return 63;
        }
        return 26;
      case 7:
      case 8:
      }return 26;
    case 'v':
      switch (length) {
      case 4:
        if ((data[(++index)] == 'o') && (data[(++index)] == 'i') && (data[(++index)] == 'd')) {
          return 40;
        }
        return 26;
      case 8:
        if ((data[(++index)] == 'o') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'a') && 
          (data[(++index)] == 't') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'e')) {
          return 64;
        }
        return 26;
      case 5:
      case 6:
      case 7: } return 26;
    case 'w':
      switch (length) {
      case 5:
        if ((data[(++index)] == 'h') && 
          (data[(++index)] == 'i') && 
          (data[(++index)] == 'l') && 
          (data[(++index)] == 'e')) {
          return 73;
        }
        return 26;
      }

      return 26;
    case 'h':
    case 'j':
    case 'k':
    case 'm':
    case 'o':
    case 'q':
    case 'u': } return 26;
  }

  public int scanNumber(boolean dotPrefix)
    throws InvalidInputException
  {
    boolean floating = dotPrefix;
    if ((!dotPrefix) && (this.currentCharacter == '0')) {
      if (getNextChar('x', 'X') >= 0) {
        int start = this.currentPosition;
        while (getNextCharAsDigit(16));
        int end = this.currentPosition;
        if (getNextChar('l', 'L') >= 0) {
          if (end == start) {
            throw new InvalidInputException("Invalid_Hexa_Literal");
          }
          return 48;
        }if (getNextChar('.')) {
          if (this.sourceLevel < 3211264L) {
            if (end == start) {
              throw new InvalidInputException("Invalid_Hexa_Literal");
            }
            this.currentPosition = end;
            return 47;
          }

          boolean hasNoDigitsBeforeDot = end == start;
          start = this.currentPosition;
          while (getNextCharAsDigit(16));
          end = this.currentPosition;
          if ((hasNoDigitsBeforeDot) && (end == start)) {
            throw new InvalidInputException("Invalid_Hexa_Literal");
          }

          if (getNextChar('p', 'P') >= 0) {
            this.unicodeAsBackSlash = false;
            if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
              (this.source[this.currentPosition] == 'u')) {
              getNextUnicodeChar();
            }
            else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            }

            if ((this.currentCharacter == '-') || 
              (this.currentCharacter == '+')) {
              this.unicodeAsBackSlash = false;
              if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
                (this.source[this.currentPosition] == 'u')) {
                getNextUnicodeChar();
              }
              else if (this.withoutUnicodePtr != 0) {
                unicodeStore();
              }
            }

            if (!ScannerHelper.isDigit(this.currentCharacter)) {
              throw new InvalidInputException("Invalid_Hexa_Literal");
            }
            while (getNextCharAsDigit());
            if (getNextChar('f', 'F') >= 0) {
              return 49;
            }
            if (getNextChar('d', 'D') >= 0) {
              return 50;
            }
            if (getNextChar('l', 'L') >= 0) {
              throw new InvalidInputException("Invalid_Hexa_Literal");
            }
            return 50;
          }
          throw new InvalidInputException("Invalid_Hexa_Literal");
        }
        if (getNextChar('p', 'P') >= 0) {
          if (this.sourceLevel < 3211264L)
          {
            this.currentPosition = end;
            return 47;
          }
          this.unicodeAsBackSlash = false;
          if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
            (this.source[this.currentPosition] == 'u')) {
            getNextUnicodeChar();
          }
          else if (this.withoutUnicodePtr != 0) {
            unicodeStore();
          }

          if ((this.currentCharacter == '-') || 
            (this.currentCharacter == '+')) {
            this.unicodeAsBackSlash = false;
            if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
              (this.source[this.currentPosition] == 'u')) {
              getNextUnicodeChar();
            }
            else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            }
          }

          if (!ScannerHelper.isDigit(this.currentCharacter))
            throw new InvalidInputException("Invalid_Float_Literal");
          while (getNextCharAsDigit());
          if (getNextChar('f', 'F') >= 0)
            return 49;
          if (getNextChar('d', 'D') >= 0)
            return 50;
          if (getNextChar('l', 'L') >= 0) {
            throw new InvalidInputException("Invalid_Hexa_Literal");
          }
          return 50;
        }
        if (end == start)
          throw new InvalidInputException("Invalid_Hexa_Literal");
        return 47;
      }

      if (getNextCharAsDigit()) {
        while (getNextCharAsDigit());
        if (getNextChar('l', 'L') >= 0) {
          return 48;
        }

        if (getNextChar('f', 'F') >= 0) {
          return 49;
        }

        if (getNextChar('d', 'D') >= 0) {
          return 50;
        }
        boolean isInteger = true;
        if (getNextChar('.')) { isInteger = false;
          while (getNextCharAsDigit());
        }
        if (getNextChar('e', 'E') >= 0) {
          isInteger = false;
          this.unicodeAsBackSlash = false;
          if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
            (this.source[this.currentPosition] == 'u')) {
            getNextUnicodeChar();
          }
          else if (this.withoutUnicodePtr != 0) {
            unicodeStore();
          }

          if ((this.currentCharacter == '-') || 
            (this.currentCharacter == '+')) {
            this.unicodeAsBackSlash = false;
            if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
              (this.source[this.currentPosition] == 'u')) {
              getNextUnicodeChar();
            }
            else if (this.withoutUnicodePtr != 0) {
              unicodeStore();
            }
          }

          if (!ScannerHelper.isDigit(this.currentCharacter))
            throw new InvalidInputException("Invalid_Float_Literal");
          while (getNextCharAsDigit());
        }
        if (getNextChar('f', 'F') >= 0)
          return 49;
        if ((getNextChar('d', 'D') >= 0) || (!isInteger))
          return 50;
        return 47;
      }

    }

    while (getNextCharAsDigit());
    if ((!dotPrefix) && (getNextChar('l', 'L') >= 0)) {
      return 48;
    }
    if ((!dotPrefix) && (getNextChar('.'))) {
      while (getNextCharAsDigit());
      floating = true;
    }

    if (getNextChar('e', 'E') >= 0) {
      floating = true;

      this.unicodeAsBackSlash = false;
      if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
        (this.source[this.currentPosition] == 'u')) {
        getNextUnicodeChar();
      }
      else if (this.withoutUnicodePtr != 0) {
        unicodeStore();
      }

      if ((this.currentCharacter == '-') || 
        (this.currentCharacter == '+')) {
        this.unicodeAsBackSlash = false;
        if (((this.currentCharacter = this.source[(this.currentPosition++)]) == '\\') && 
          (this.source[this.currentPosition] == 'u')) {
          getNextUnicodeChar();
        }
        else if (this.withoutUnicodePtr != 0) {
          unicodeStore();
        }
      }

      if (!ScannerHelper.isDigit(this.currentCharacter)) {
        throw new InvalidInputException("Invalid_Float_Literal");
      }
      while (getNextCharAsDigit());
    }
    if (getNextChar('d', 'D') >= 0)
      return 50;
    if (getNextChar('f', 'F') >= 0) {
      return 49;
    }

    return floating ? 50 : 47;
  }

  public final int getLineNumber(int position)
  {
    return Util.getLineNumber(position, this.lineEnds, 0, this.linePtr);
  }

  public final void setSource(char[] sourceString)
  {
    int sourceLength;
    int sourceLength;
    if (sourceString == null) {
      this.source = CharOperation.NO_CHAR;
      sourceLength = 0;
    } else {
      this.source = sourceString;
      sourceLength = sourceString.length;
    }
    this.startPosition = -1;
    this.eofPosition = sourceLength;
    this.initialPosition = (this.currentPosition = 0);
    this.containsAssertKeyword = false;
    this.linePtr = -1;
  }

  public final void setSource(char[] contents, CompilationResult compilationResult)
  {
    if (contents == null) {
      char[] cuContents = compilationResult.compilationUnit.getContents();
      setSource(cuContents);
    } else {
      setSource(contents);
    }
    int[] lineSeparatorPositions = compilationResult.lineSeparatorPositions;
    if (lineSeparatorPositions != null) {
      this.lineEnds = lineSeparatorPositions;
      this.linePtr = (lineSeparatorPositions.length - 1);
    }
  }

  public final void setSource(CompilationResult compilationResult)
  {
    setSource(null, compilationResult);
  }
  public String toString() {
    if (this.startPosition == this.eofPosition)
      return "EOF\n\n" + new String(this.source);
    if (this.currentPosition > this.eofPosition) {
      return "behind the EOF\n\n" + new String(this.source);
    }
    char[] front = new char[this.startPosition];
    System.arraycopy(this.source, 0, front, 0, this.startPosition);

    int middleLength = this.currentPosition - 1 - this.startPosition + 1;
    char[] middle;
    if (middleLength > -1) {
      char[] middle = new char[middleLength];
      System.arraycopy(
        this.source, 
        this.startPosition, 
        middle, 
        0, 
        middleLength);
    } else {
      middle = CharOperation.NO_CHAR;
    }

    char[] end = new char[this.eofPosition - (this.currentPosition - 1)];
    System.arraycopy(
      this.source, 
      this.currentPosition - 1 + 1, 
      end, 
      0, 
      this.eofPosition - (this.currentPosition - 1) - 1);

    return new String(front) + 
      "\n===============================\nStarts here -->" + 
      new String(middle) + 
      "<-- Ends here\n===============================\n" + 
      new String(end);
  }
  public String toStringAction(int act) {
    switch (act) {
    case 26:
      return "Identifier(" + new String(getCurrentTokenSource()) + ")";
    case 56:
      return "abstract";
    case 32:
      return "boolean";
    case 75:
      return "break";
    case 33:
      return "byte";
    case 101:
      return "case";
    case 102:
      return "catch";
    case 34:
      return "char";
    case 72:
      return "class";
    case 76:
      return "continue";
    case 97:
      return "default";
    case 77:
      return "do";
    case 35:
      return "double";
    case 103:
      return "else";
    case 99:
      return "extends";
    case 44:
      return "false";
    case 57:
      return "final";
    case 104:
      return "finally";
    case 36:
      return "float";
    case 78:
      return "for";
    case 79:
      return "if";
    case 106:
      return "implements";
    case 100:
      return "import";
    case 15:
      return "instanceof";
    case 37:
      return "int";
    case 95:
      return "interface";
    case 38:
      return "long";
    case 58:
      return "native";
    case 43:
      return "new";
    case 45:
      return "null";
    case 96:
      return "package";
    case 59:
      return "private";
    case 60:
      return "protected";
    case 61:
      return "public";
    case 80:
      return "return";
    case 39:
      return "short";
    case 54:
      return "static";
    case 41:
      return "super";
    case 81:
      return "switch";
    case 55:
      return "synchronized";
    case 42:
      return "this";
    case 82:
      return "throw";
    case 105:
      return "throws";
    case 63:
      return "transient";
    case 46:
      return "true";
    case 83:
      return "try";
    case 40:
      return "void";
    case 64:
      return "volatile";
    case 73:
      return "while";
    case 47:
      return "Integer(" + new String(getCurrentTokenSource()) + ")";
    case 48:
      return "Long(" + new String(getCurrentTokenSource()) + ")";
    case 49:
      return "Float(" + new String(getCurrentTokenSource()) + ")";
    case 50:
      return "Double(" + new String(getCurrentTokenSource()) + ")";
    case 51:
      return "Char(" + new String(getCurrentTokenSource()) + ")";
    case 52:
      return "String(" + new String(getCurrentTokenSource()) + ")";
    case 9:
      return "++";
    case 10:
      return "--";
    case 18:
      return "==";
    case 16:
      return "<=";
    case 17:
      return ">=";
    case 19:
      return "!=";
    case 13:
      return "<<";
    case 8:
      return ">>";
    case 11:
      return ">>>";
    case 84:
      return "+=";
    case 85:
      return "-=";
    case 86:
      return "*=";
    case 87:
      return "/=";
    case 88:
      return "&=";
    case 89:
      return "|=";
    case 90:
      return "^=";
    case 91:
      return "%=";
    case 92:
      return "<<=";
    case 93:
      return ">>=";
    case 94:
      return ">>>=";
    case 25:
      return "||";
    case 24:
      return "&&";
    case 1:
      return "+";
    case 2:
      return "-";
    case 66:
      return "!";
    case 5:
      return "%";
    case 21:
      return "^";
    case 20:
      return "&";
    case 4:
      return "*";
    case 22:
      return "|";
    case 67:
      return "~";
    case 6:
      return "/";
    case 12:
      return ">";
    case 7:
      return "<";
    case 28:
      return "(";
    case 29:
      return ")";
    case 69:
      return "{";
    case 31:
      return "}";
    case 14:
      return "[";
    case 70:
      return "]";
    case 27:
      return ";";
    case 23:
      return "?";
    case 65:
      return ":";
    case 30:
      return ",";
    case 3:
      return ".";
    case 71:
      return "=";
    case 68:
      return "EOF";
    case 1000:
      return "white_space(" + new String(getCurrentTokenSource()) + ")";
    }
    return "not-a-token";
  }

  public void unicodeInitializeBuffer(int length) {
    this.withoutUnicodePtr = length;
    if (this.withoutUnicodeBuffer == null) this.withoutUnicodeBuffer = new char[length + 11];
    int bLength = this.withoutUnicodeBuffer.length;
    if (1 + length >= bLength) {
      System.arraycopy(this.withoutUnicodeBuffer, 0, this.withoutUnicodeBuffer = new char[length + 11], 0, bLength);
    }
    System.arraycopy(this.source, this.startPosition, this.withoutUnicodeBuffer, 1, length);
  }
  public void unicodeStore() {
    int pos = ++this.withoutUnicodePtr;
    if (this.withoutUnicodeBuffer == null) this.withoutUnicodeBuffer = new char[10];
    int length = this.withoutUnicodeBuffer.length;
    if (pos == length) {
      System.arraycopy(this.withoutUnicodeBuffer, 0, this.withoutUnicodeBuffer = new char[length * 2], 0, length);
    }
    this.withoutUnicodeBuffer[pos] = this.currentCharacter;
  }
}