package org.jvnet.jaxbw.intellij.wizard;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Composite;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.LinkedList;
import java.util.StringTokenizer;
import javax.swing.Icon;
import javax.swing.ImageIcon;

public final class ImageCreator
{
  public static final Color mainUUltraLightColor = new Color(196, 220, 255);

  public static final Color mainUltraLightColor = new Color(128, 192, 255);

  public static final Color mainLightColor = new Color(0, 128, 255);

  public static final Color mainMidColor = new Color(0, 64, 196);

  public static final Color mainDarkColor = new Color(0, 32, 128);

  public static final Color mainUltraDarkColor = new Color(0, 32, 64);

  public static final Color iconClassColor = new Color(128, 255, 128);

  public static final Color iconAnnotationColor = new Color(141, 112, 255);

  public static final Color iconFieldMethodColor = new Color(32, 128, 255);

  public static final Color iconArrowColor = new Color(128, 32, 0);
  public static final int ICON_DIMENSION = 15;
  public static final int CUBE_DIMENSION = 5;
  public static final int CLOSE_DIMENSION = 4;

  public static BufferedImage getBlankImage(int width, int height)
  {
    BufferedImage image = new BufferedImage(width, height, 2);

    for (int col = 0; col < width; col++) {
      for (int row = 0; row < height; row++) {
        image.setRGB(col, row, 0);
      }
    }

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    return image;
  }

  public static Icon getWizardImage(int width, int height)
  {
    BufferedImage image = new BufferedImage(width, height, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    Color main = mainLightColor;
    Color text = Color.yellow;
    double coef = 0.9D;
    Color dark = new Color((int)(coef * main.getRed()), (int)(coef * main.getGreen()), (int)(coef * main.getBlue()));

    Color light = new Color(255 - (int)(coef * (255 - main.getRed())), 255 - (int)(coef * (255 - main.getGreen())), 255 - (int)(coef * (255 - main.getBlue())));

    graphics.drawImage(getBackgroundStripedImage(width, height, light, main, dark), 0, 0, null);

    String[] strings = { "JAXB 2.0", "XSD", "JAXB", "XML", "Binding", "<entity>", "Java", "JAXBContext", "Marshaller", "Unmarshaller" };

    int cellCount = 4;
    int cellSizeX = width / cellCount;
    int cellSizeY = height / cellCount;
    for (int i = 0; i < cellCount; i++) {
      for (int j = 0; j < cellCount; j++) {
        int fontSize = (int)(15.0D + 15.0D * Math.random());
        int strIndex = (int)(strings.length * Math.random());

        int r = (int)(text.getRed() + 20 - 40.0D * Math.random());
        r = Math.max(r, 0);
        r = Math.min(r, 255);
        int g = (int)(text.getGreen() + 20 - 40.0D * Math.random());
        g = Math.max(g, 0);
        g = Math.min(g, 255);
        int b = (int)(text.getBlue() + 20 - 40.0D * Math.random());
        b = Math.max(b, 0);
        b = Math.min(b, 255);
        graphics.setColor(new Color(r, g, b));

        graphics.setFont(new Font("Arial", 0, fontSize));
        FontRenderContext frc = graphics.getFontRenderContext();
        TextLayout mLayout = new TextLayout(strings[strIndex], graphics.getFont(), frc);

        int x = (int)((i + Math.random()) * cellSizeX - mLayout.getBounds().getWidth() / 2.0D);

        int y = (int)((j + Math.random()) * cellSizeY);
        Composite c = AlphaComposite.getInstance(3, (float)(0.2D + 0.8D * Math.random()));

        graphics.setComposite(c);
        graphics.drawString(strings[strIndex], x, y);
      }
    }

    return new ImageIcon(image);
  }

  public static Icon getTitleImage(int width, int height, String title, Color foregroundColor, Color shadowColor)
  {
    BufferedImage image = getBackgroundImage(width, height, false);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    Font font = new Font("Arial", 0, height - 8);
    graphics.setFont(font);
    FontMetrics fm = graphics.getFontMetrics();
    FontRenderContext frc = graphics.getFontRenderContext();
    TextLayout mLayout = new TextLayout(title, font, frc);

    Rectangle2D bounds = mLayout.getBounds();
    double x = (width - bounds.getWidth()) / 2.0D;
    double y = (height - bounds.getHeight() + fm.getHeight()) / 2.0D + 3.0D;

    Composite c = AlphaComposite.getInstance(3, 0.8F);

    graphics.setComposite(c);
    graphics.setColor(shadowColor);
    graphics.drawString(title, (int)x + 1, (int)y + 1);
    c = AlphaComposite.getInstance(3, 1.0F);
    graphics.setComposite(c);
    graphics.setColor(foregroundColor);
    graphics.drawString(title, (int)x, (int)y);

    return new ImageIcon(image);
  }

  public static BufferedImage getBackgroundStripedImage(int width, int height, Color lightColor, Color midColor, Color darkColor)
  {
    BufferedImage image = new BufferedImage(width, height, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    GradientPaint gpLightLeft = new GradientPaint(0.0F, 0.0F, lightColor, width / 2, 0.0F, midColor);

    GradientPaint gpLightRight = new GradientPaint(width / 2, 0.0F, midColor, width, 0.0F, lightColor);

    GradientPaint gpDarkLeft = new GradientPaint(0.0F, 0.0F, midColor, width / 2, 0.0F, darkColor);

    GradientPaint gpDarkRight = new GradientPaint(width / 2, 0.0F, darkColor, width, 0.0F, midColor);

    for (int row = 0; row < height; row++) {
      GradientPaint gpLeft = row % 2 == 0 ? gpLightLeft : gpDarkLeft;
      graphics.setPaint(gpLeft);
      graphics.drawLine(0, row, width / 2, row);
      GradientPaint gpRight = row % 2 == 0 ? gpLightRight : gpDarkRight;
      graphics.setPaint(gpRight);
      graphics.drawLine(width / 2, row, width, row);
    }

    return image;
  }

  public static BufferedImage getBackgroundImage(int width, int height, boolean hasStripeTop)
  {
    BufferedImage image = getBlankImage(width, height);
    Graphics2D graphics = (Graphics2D)image.getGraphics();

    if (hasStripeTop)
    {
      for (int col = 0; col < width; col++)
      {
        float transp = Math.abs(col - width / 2) / (width / 2);

        transp = Math.min(1.0F, (float)(transp * 1.25D));
        Composite c = AlphaComposite.getInstance(3, 1.0F - transp);

        graphics.setComposite(c);
        graphics.setColor(Color.black);
        graphics.drawLine(col, 0, col, 2);
      }
    }

    return image;
  }

  public static Icon getMultiline(String str, int width, Color foregroundColor, Color shadowColor)
  {
    BufferedImage tempImage = new BufferedImage(width, 20, 2);

    Graphics2D tempGraphics = (Graphics2D)tempImage.getGraphics();
    tempGraphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);

    Font font = new Font("Arial", 0, 12);

    tempGraphics.setFont(font);
    FontRenderContext frc = tempGraphics.getFontRenderContext();

    LinkedList lines = new LinkedList();
    StringTokenizer tokenizer = new StringTokenizer(str, " ", false);
    String currLine = "";
    while (tokenizer.hasMoreTokens()) {
      String currToken = tokenizer.nextToken() + " ";
      String newLine = currLine + currToken;
      TextLayout mLayout = new TextLayout(newLine, font, frc);

      Rectangle2D bounds = mLayout.getBounds();
      if (bounds.getWidth() > width - 20)
      {
        lines.addLast(currLine);
        currLine = currToken;
      } else {
        currLine = newLine;
      }
    }

    lines.addLast(currLine);

    int lineCount = lines.size();
    FontMetrics fm = tempGraphics.getFontMetrics();
    int height = lineCount * (fm.getHeight() + 5);

    BufferedImage image = getBackgroundImage(width, height, false);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);

    graphics.setFont(font);
    int ypos = fm.getHeight();
    for (String line : lines) {
      Composite c = AlphaComposite.getInstance(3, 0.8F);

      graphics.setComposite(c);
      graphics.setColor(shadowColor);
      graphics.drawString(line, 6, ypos + 1);
      c = AlphaComposite.getInstance(3, 1.0F);

      graphics.setComposite(c);
      graphics.setColor(foregroundColor);
      graphics.drawString(line, 5, ypos);
      ypos += fm.getHeight() + 1;
    }

    return new ImageIcon(image);
  }

  public static BufferedImage getSingleLetterImage(char letter, boolean isCollection, Color backgroundColor)
  {
    BufferedImage image = new BufferedImage(15, 15, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    letter = Character.toUpperCase(letter);
    graphics.setFont(new Font("Arial", 1, 10));

    float x = (float)((15.0D - Math.ceil(graphics.getFontMetrics().charWidth(letter))) / 2.0D);

    float y = 13.0F - (float)((15.0D - graphics.getFontMetrics().getStringBounds("" + letter, graphics).getHeight()) / 2.0D);

    graphics.setColor(backgroundColor);
    graphics.fillOval(0, 0, 14, 14);

    double id4 = 3.75D;
    double spotX = id4;
    double spotY = id4;
    for (int col = 0; col < 15; col++) {
      for (int row = 0; row < 15; row++)
      {
        double dx = col - spotX;
        double dy = row - spotY;
        double dist = Math.sqrt(dx * dx + dy * dy);

        if (dist > 15.0D) {
          dist = 15.0D;
        }

        int currColor = image.getRGB(col, row);
        int transp = currColor >>> 24 & 0xFF;
        int oldR = currColor >>> 16 & 0xFF;
        int oldG = currColor >>> 8 & 0xFF;
        int oldB = currColor >>> 0 & 0xFF;

        double coef = 0.9D - 0.9D * dist / 15.0D;
        int dr = 255 - oldR;
        int dg = 255 - oldG;
        int db = 255 - oldB;

        int newR = (int)(oldR + coef * dr);
        int newG = (int)(oldG + coef * dg);
        int newB = (int)(oldB + coef * db);

        int newColor = transp << 24 | newR << 16 | newG << 8 | newB;

        image.setRGB(col, row, newColor);
      }

    }

    graphics.setColor(Color.black);
    graphics.drawOval(0, 0, 14, 14);

    graphics.drawString("" + letter, x, y);

    if (isCollection) {
      int xp = 9;
      int yp = 3;
      graphics.setColor(new Color(255, 255, 255, 128));
      graphics.drawLine(xp - 1, yp - 2, xp + 6, yp - 2);
      graphics.drawLine(xp - 1, yp - 1, xp + 6, yp - 1);
      graphics.drawLine(xp - 1, yp, xp + 6, yp);
      graphics.drawLine(xp - 1, yp + 1, xp + 6, yp + 1);
      graphics.drawLine(xp + 1, yp - 4, xp + 1, yp + 3);
      graphics.drawLine(xp + 2, yp - 4, xp + 2, yp + 3);
      graphics.drawLine(xp + 3, yp - 4, xp + 3, yp + 3);
      graphics.drawLine(xp + 4, yp - 4, xp + 4, yp + 3);
      graphics.setColor(new Color(255, 64, 64));
      graphics.drawLine(xp, yp - 1, xp + 5, yp - 1);
      graphics.drawLine(xp, yp, xp + 5, yp);
      graphics.drawLine(xp + 2, yp - 3, xp + 2, yp + 2);
      graphics.drawLine(xp + 3, yp - 3, xp + 3, yp + 2);
    }

    return image;
  }

  public static Icon getSingleLetterIcon(char letter, boolean isCollection, Color backgroundColor)
  {
    return new ImageIcon(getSingleLetterImage(letter, isCollection, backgroundColor));
  }

  public static Icon getLetterIconWithArrow(char letter, boolean isCollection, Color backgroundColor, boolean isArrowToRight, Color arrowColor)
  {
    BufferedImage image = getSingleLetterImage(letter, isCollection, backgroundColor);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);

    BufferedImage arrowImage = getArrowImage(arrowColor, new Color(255, 255, 255, 196), 15, isArrowToRight);

    graphics.drawImage(arrowImage, 0, 15 - arrowImage.getHeight(), null);

    return new ImageIcon(image);
  }

  public static BufferedImage getRightArrow(Color color, Color haloColor, int width)
  {
    int height = 6;
    BufferedImage image = new BufferedImage(width, height, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    Polygon pol = new Polygon();
    int ya = 3;
    pol.addPoint(1, ya);
    pol.addPoint(width / 2 + 3, ya);
    pol.addPoint(width / 2 + 3, ya + 2);
    pol.addPoint(width - 1, ya);
    pol.addPoint(width / 2 + 3, ya - 2);
    pol.addPoint(width / 2 + 3, ya);

    graphics.setColor(color);
    graphics.drawPolygon(pol);

    BufferedImage fimage = new BufferedImage(width, height, 2);

    for (int col = 0; col < width; col++) {
      for (int row = 0; row < height; row++) {
        fimage.setRGB(col, row, 0);
      }
    }
    Graphics2D fgraphics = (Graphics2D)fimage.getGraphics();
    int haloOpacity = haloColor.getAlpha();
    for (int col = 0; col < width; col++) {
      int xs = Math.max(0, col - 1);
      int xe = Math.min(width - 1, col + 1);
      for (int row = 0; row < height; row++) {
        int ys = Math.max(0, row - 1);
        int ye = Math.min(height - 1, row + 1);
        int currColor = image.getRGB(col, row);
        int opacity = (int)(haloOpacity * (currColor >>> 24 & 0xFF) / 255.0D);
        if (opacity > 0)
        {
          for (int x = xs; x <= xe; x++) {
            for (int y = ys; y <= ye; y++) {
              int oldOpacity = fimage.getRGB(x, y) >>> 24 & 0xFF;
              int newOpacity = Math.max(oldOpacity, opacity);

              int newColor = newOpacity << 24 | 0xFF0000 | 0xFF00 | 0xFF;

              fimage.setRGB(x, y, newColor);
            }
          }
        }
      }

    }

    fgraphics.drawImage(image, 0, 0, null);

    return fimage;
  }

  public static BufferedImage getLeftArrow(Color arrowColor, Color haloColor, int width)
  {
    BufferedImage rimage = getRightArrow(arrowColor, haloColor, width);

    int height = rimage.getHeight();
    BufferedImage image = new BufferedImage(width, height, 2);

    for (int col = 0; col < width; col++) {
      for (int row = 0; row < height; row++) {
        image.setRGB(col, row, rimage.getRGB(width - 1 - col, row));
      }
    }
    return image;
  }

  public static BufferedImage getArrowImage(Color arrowColor, Color haloColor, int width, boolean isArrowToRight)
  {
    if (isArrowToRight) {
      return getRightArrow(arrowColor, haloColor, width);
    }
    return getLeftArrow(arrowColor, haloColor, width);
  }

  public static BufferedImage createGradientLine(int width, Color leftColor, Color rightColor, double opacity)
  {
    BufferedImage image = new BufferedImage(width, 1, 2);

    int iOpacity = (int)(255.0D * opacity);

    for (int col = 0; col < width; col++) {
      double coef = col / width;
      int r = (int)(leftColor.getRed() + coef * (rightColor.getRed() - leftColor.getRed()));

      int g = (int)(leftColor.getGreen() + coef * (rightColor.getGreen() - leftColor.getGreen()));

      int b = (int)(leftColor.getBlue() + coef * (rightColor.getBlue() - leftColor.getBlue()));

      int color = iOpacity << 24 | r << 16 | g << 8 | b;
      image.setRGB(col, 0, color);
    }
    return image;
  }

  public static BufferedImage getGutterMarker(Color themeColor, int width, int height)
  {
    BufferedImage image = new BufferedImage(width, height, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();

    int rd = themeColor.getRed() / 2;
    int gd = themeColor.getGreen() / 2;
    int bd = themeColor.getBlue() / 2;
    Color darkColor = new Color(rd, gd, bd);

    int rl = 255 - (255 - themeColor.getRed()) / 4;
    int gl = 255 - (255 - themeColor.getGreen()) / 4;
    int bl = 255 - (255 - themeColor.getBlue()) / 4;
    Color lightColor = new Color(rl, gl, bl);
    graphics.setColor(lightColor);
    graphics.fillRect(0, 0, width - 1, height - 1);
    graphics.setColor(darkColor);
    graphics.drawRect(0, 0, width - 1, height - 1);
    return image;
  }

  public static BufferedImage getGutterStatusImage(Color themeColor, int dimension)
  {
    BufferedImage image = new BufferedImage(dimension, dimension, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();

    int rud = themeColor.getRed() / 4;
    int gud = themeColor.getGreen() / 4;
    int bud = themeColor.getBlue() / 4;
    Color ultraDarkColor = new Color(rud, gud, bud);

    int rd = themeColor.getRed() / 2;
    int gd = themeColor.getGreen() / 2;
    int bd = themeColor.getBlue() / 2;
    Color darkColor = new Color(rd, gd, bd);

    int rl = 255 - (255 - themeColor.getRed()) / 4;
    int gl = 255 - (255 - themeColor.getGreen()) / 4;
    int bl = 255 - (255 - themeColor.getBlue()) / 4;
    Color lightColor = new Color(rl, gl, bl);

    GradientPaint gradient = new GradientPaint(0.0F, 0.0F, lightColor, dimension, dimension, darkColor);

    graphics.setPaint(gradient);
    Rectangle rect = new Rectangle(dimension, dimension);
    graphics.fill(rect);
    graphics.setColor(ultraDarkColor);
    graphics.drawLine(0, 0, dimension - 1, 0);
    graphics.drawLine(0, 0, 0, dimension - 1);
    graphics.setColor(lightColor);
    graphics.drawLine(0, dimension - 1, dimension - 1, dimension - 1);
    graphics.drawLine(dimension - 1, 1, dimension - 1, dimension - 1);

    return image;
  }

  public static Icon getCloseIcon(int dimension)
  {
    BufferedImage image = new BufferedImage(dimension, dimension, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    GradientPaint gradient = new GradientPaint(0.0F, 0.0F, mainUltraLightColor, dimension, dimension, Color.white);

    graphics.setPaint(gradient);
    Rectangle rect = new Rectangle(dimension, dimension);
    graphics.fill(rect);

    graphics.setColor(Color.black);
    graphics.drawRect(0, 0, dimension - 1, dimension - 1);

    graphics.setStroke(new BasicStroke(2.0F));
    graphics.setColor(mainMidColor);
    graphics.drawLine(3, 3, dimension - 4, dimension - 4);
    graphics.drawLine(3, dimension - 4, dimension - 4, 3);

    return new ImageIcon(image);
  }

  public static Icon getCloseIcon(int dimension, char letter)
  {
    BufferedImage image = new BufferedImage(dimension, dimension, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    GradientPaint gradient = new GradientPaint(dimension, 0.0F, mainDarkColor, 0.0F, dimension, mainUltraDarkColor);

    graphics.setPaint(gradient);

    Font font = new Font("Arial", 1, dimension - 2);
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    graphics.setFont(font);
    letter = Character.toUpperCase(letter);
    int charWidth = graphics.getFontMetrics().charWidth(letter);

    int x = dimension - charWidth - 2;
    FontRenderContext frc = graphics.getFontRenderContext();
    TextLayout mLayout = new TextLayout("" + letter, font, frc);
    Rectangle2D bounds = mLayout.getBounds();

    graphics.drawString("" + letter, x, (int)bounds.getHeight() + 2);

    int xs = 1;
    int xe = xs + 4;
    int ys = 8;
    int ye = ys + 4;

    graphics.setStroke(new BasicStroke(3.5F));
    graphics.setColor(new Color(255, 255, 255, 196));
    graphics.drawLine(xs, ys, xe, ye);
    graphics.drawLine(xs, ye, xe, ys);
    graphics.setStroke(new BasicStroke(2.0F));
    graphics.setColor(mainMidColor);
    graphics.drawLine(xs, ys, xe, ye);
    graphics.drawLine(xs, ye, xe, ys);

    return new ImageIcon(image);
  }

  public static Icon getEditIcon(int dimension, char letter)
  {
    BufferedImage image = new BufferedImage(dimension, dimension, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    GradientPaint gradient = new GradientPaint(dimension, 0.0F, mainDarkColor, 0.0F, dimension, mainUltraDarkColor);

    graphics.setPaint(gradient);

    Font font = new Font("Arial", 1, dimension - 2);
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

    graphics.setFont(font);
    letter = Character.toUpperCase(letter);
    int charWidth = graphics.getFontMetrics().charWidth(letter);

    int x = dimension - charWidth - 2;
    FontRenderContext frc = graphics.getFontRenderContext();
    TextLayout mLayout = new TextLayout("" + letter, font, frc);
    Rectangle2D bounds = mLayout.getBounds();

    graphics.drawString("" + letter, x, (int)bounds.getHeight() + 2);

    int xs = 1;
    int xe = xs + 4;
    int xm = (xs + xe) / 2;
    int ys = 8;
    int ye = ys + 4;
    int ym = (ys + ye) / 2;

    graphics.setStroke(new BasicStroke(3.5F));
    graphics.setColor(new Color(255, 255, 255, 196));
    graphics.drawLine(xs, ym, xe, ym);
    graphics.drawLine(xm, ys, xm, ye);
    graphics.setStroke(new BasicStroke(2.5F));
    graphics.setColor(mainMidColor);
    graphics.drawLine(xs, ym, xe, ym);
    graphics.drawLine(xm, ys, xm, ye);

    return new ImageIcon(image);
  }

  public static BufferedImage getGradientCubesImage(int width, int height, Color leftColor, Color rightColor, int transitionStart, int transitionEnd)
  {
    BufferedImage image = new BufferedImage(width, height, 2);

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    GradientPaint gradient = new GradientPaint(transitionStart, 0.0F, leftColor, transitionEnd, 0.0F, rightColor);

    graphics.setPaint(gradient);
    graphics.fillRect(transitionStart, 0, transitionEnd - transitionStart, height);

    graphics.setColor(leftColor);
    graphics.fillRect(0, 0, transitionStart, height);

    graphics.setColor(rightColor);
    graphics.fillRect(transitionEnd, 0, width - transitionEnd, height);

    int cubeCountY = height / 5;
    int cubeCountX = 1 + (transitionEnd - transitionStart) / 5;

    int cubeStartY = height % 5 / 2;
    int cubeStartX = transitionStart - (5 - (transitionEnd - transitionStart) % 5);

    for (int col = 0; col < cubeCountX; col++) {
      for (int row = 0; row < cubeCountY; row++)
      {
        if (Math.random() >= 0.5D)
        {
          double coef = 1.0D - (col / cubeCountX + 0.9D * (Math.random() - 0.5D));

          coef = Math.max(0.0D, coef);
          coef = Math.min(1.0D, coef);

          int r = (int)(coef * leftColor.getRed() + (1.0D - coef) * rightColor.getRed());

          int g = (int)(coef * leftColor.getGreen() + (1.0D - coef) * rightColor.getGreen());

          int b = (int)(coef * leftColor.getBlue() + (1.0D - coef) * rightColor.getBlue());

          graphics.setColor(new Color(r, g, b));
          graphics.fillRect(cubeStartX + col * 5, cubeStartY + row * 5, 5, 5);

          graphics.setColor(new Color(255 - (int)(0.95D * (255 - r)), 255 - (int)(0.9D * (255 - g)), 255 - (int)(0.9D * (255 - b))));

          graphics.drawRect(cubeStartX + col * 5, cubeStartY + row * 5, 5, 5);
        }

      }

    }

    return image;
  }

  public static Polygon getRoundedRectangle(Rectangle rect)
  {
    Polygon border = new Polygon();
    int offsetX = (int)rect.getX();
    int offsetY = (int)rect.getY();
    border.addPoint(offsetX, offsetY + 1);
    border.addPoint(offsetX + 1, offsetY);
    border.addPoint(offsetX + (int)rect.getWidth() - 1, offsetY);
    border.addPoint(offsetX + (int)rect.getWidth(), offsetY + 1);
    border.addPoint(offsetX + (int)rect.getWidth(), offsetY + (int)rect.getHeight() - 1);

    border.addPoint(offsetX + (int)rect.getWidth() - 1, offsetY + (int)rect.getHeight());

    border.addPoint(offsetX + 1, offsetY + (int)rect.getHeight());
    border.addPoint(offsetX, offsetY + (int)rect.getHeight() - 1);

    return border;
  }

  public static Icon overlay(Icon icon1, Icon icon2) {
    BufferedImage image = getBlankImage(icon2.getIconWidth() / 2 + icon1.getIconWidth(), icon1.getIconHeight());

    Graphics2D graphics = (Graphics2D)image.getGraphics();
    icon1.paintIcon(null, graphics, icon2.getIconWidth() / 2, 0);
    icon2.paintIcon(null, graphics, 0, icon1.getIconHeight() - icon2.getIconHeight());

    return new ImageIcon(image);
  }

  public static Image getIconImage() {
    BufferedImage result = getBlankImage(16, 16);
    Graphics2D g2 = (Graphics2D)result.getGraphics().create();

    g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

    g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);

    g2.setColor(new Color(140, 72, 170));
    g2.setFont(new Font("Tahoma", 1, 17));
    g2.drawString("J", 6.0F, 14.0F);
    g2.setColor(Color.BLACK);
    g2.setFont(new Font("Tahoma", 1, 11));
    g2.drawString("w", 2.0F, 11.0F);

    g2.dispose();
    return result;
  }
}