package org.jvnet.jaxbw.intellij.wizard;

import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;
import org.jvnet.flamingo.wizard.WizardSettings;
import org.jvnet.flamingo.wizard.WizardStep;

public final class InputAdditionalStep
  implements WizardStep
{
  private final JComponent component;
  private final JTextField classpathTf;
  private final JTextField catalogTf;
  private final JTextField useRuntimeTf;
  private final JCheckBox doStrictValidationCb;
  private final JCheckBox doAllowVendorExtensionsCb;
  private final JCheckBox doCreateReadOnlyFilesCb;

  public InputAdditionalStep(XjcWizard owner)
  {
    JPanel panel = new JPanel();
    SpringLayout layout = new SpringLayout();
    panel.setLayout(layout);

    JLabel classpathL = new JLabel("User classpath");
    panel.add(classpathL);
    JLabel catalogFilesL = new JLabel("Catalog files");
    panel.add(catalogFilesL);
    JLabel useRuntimeL = new JLabel("Use runtime");
    panel.add(useRuntimeL);

    layout.putConstraint("West", classpathL, 5, "West", panel);

    layout.putConstraint("North", classpathL, 3, "North", panel);

    layout.putConstraint("West", catalogFilesL, 5, "West", panel);

    layout.putConstraint("North", catalogFilesL, 6, "South", classpathL);

    layout.putConstraint("West", useRuntimeL, 5, "West", panel);

    layout.putConstraint("North", useRuntimeL, 6, "South", catalogFilesL);

    this.classpathTf = new JTextField();
    this.classpathTf.setColumns(20);
    panel.add(this.classpathTf);

    this.catalogTf = new JTextField();
    this.catalogTf.setColumns(20);
    panel.add(this.catalogTf);

    this.useRuntimeTf = new JTextField();
    this.useRuntimeTf.setColumns(20);
    panel.add(this.useRuntimeTf);

    layout.putConstraint("West", this.classpathTf, 5, "East", classpathL);

    layout.putConstraint("North", this.classpathTf, 0, "North", classpathL);

    layout.putConstraint("West", this.catalogTf, 5, "East", classpathL);

    layout.putConstraint("North", this.catalogTf, 0, "North", catalogFilesL);

    layout.putConstraint("West", this.useRuntimeTf, 5, "East", classpathL);

    layout.putConstraint("North", this.useRuntimeTf, 0, "North", useRuntimeL);

    this.doStrictValidationCb = new JCheckBox("Do strict validation");
    this.doStrictValidationCb.setSelected(true);
    panel.add(this.doStrictValidationCb);

    this.doAllowVendorExtensionsCb = new JCheckBox("Allow extensions");
    panel.add(this.doAllowVendorExtensionsCb);

    this.doCreateReadOnlyFilesCb = new JCheckBox("Create R/O files");
    panel.add(this.doCreateReadOnlyFilesCb);

    layout.putConstraint("West", this.doStrictValidationCb, 5, "East", this.classpathTf);

    layout.putConstraint("North", this.doStrictValidationCb, -1, "North", this.classpathTf);

    layout.putConstraint("West", this.doAllowVendorExtensionsCb, 5, "East", this.classpathTf);

    layout.putConstraint("North", this.doAllowVendorExtensionsCb, -1, "North", this.catalogTf);

    layout.putConstraint("West", this.doCreateReadOnlyFilesCb, 5, "East", this.classpathTf);

    layout.putConstraint("North", this.doCreateReadOnlyFilesCb, -1, "North", this.useRuntimeTf);

    panel.setOpaque(false);

    this.component = panel;
  }

  public final boolean canProceed()
  {
    return true;
  }

  public void exportSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    String classpath = this.classpathTf.getText();
    if ((classpath != null) && (classpath.length() > 0)) {
      settings.setUserClasspath(classpath);
    }
    String catalogFiles = this.catalogTf.getText();
    if ((catalogFiles != null) && (catalogFiles.length() > 0)) {
      settings.setExternalEntityCatalogFiles(catalogFiles);
    }
    String useRuntime = this.useRuntimeTf.getText();
    if ((useRuntime != null) && (useRuntime.length() > 0)) {
      settings.setHttpProxy(useRuntime);
    }
    settings.setDoStrictValidation(this.doStrictValidationCb.isSelected());
    settings.setDoAllowVendorExtensions(this.doAllowVendorExtensionsCb.isSelected());

    settings.setDoCreateReadOnlyFiles(this.doCreateReadOnlyFilesCb.isSelected());
  }

  public void importSettings(WizardSettings wSettings)
  {
    XjcSettings settings = (XjcSettings)wSettings;
    this.classpathTf.setText(settings.getUserClasspath());
    this.catalogTf.setText(settings.getExternalEntityCatalogFiles());
    this.useRuntimeTf.setText(settings.getHttpProxy());
    this.doStrictValidationCb.setSelected(settings.isDoStrictValidation());
    this.doAllowVendorExtensionsCb.setSelected(settings.isDoAllowVendorExtensions());
    this.doCreateReadOnlyFilesCb.setSelected(settings.isDoCreateReadOnlyFiles());
  }

  public final JComponent getComponent()
  {
    return this.component;
  }

  public final String getTitle()
  {
    return "Additional settings";
  }

  public final String getCurrentDirectory()
    throws UnsupportedOperationException
  {
    throw new UnsupportedOperationException();
  }

  public final boolean isSupportsDirectory()
  {
    return false;
  }

  public final void setCurrentDirectory(String str)
    throws UnsupportedOperationException
  {
    throw new UnsupportedOperationException();
  }

  public final String getDescription()
  {
    return "Specify additional settings for the generator.";
  }

  public Icon getStepIcon(int width, int height) {
    return ImageCreator.getWizardImage(width, height);
  }
}