package org.jvnet.jaxbw.intellij.wizard;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import org.jvnet.flamingo.wizard.WizardMainFrame;
import org.jvnet.flamingo.wizard.WizardMainFrame.WizardMode;
import org.jvnet.flamingo.wizard.WizardStep;

public final class XjcWizard extends WizardMainFrame
{
  public static final String PROP_PACKAGE_NAME = "xjc.package.name";
  public static final String PROP_OUTPUT_DIR = "xjc.output.dir";
  public static final String PROP_USER_CLASSPATH = "xjc.user.cp";
  public static final String PROP_CATALOG_FILES = "xjc.catalog.files";
  public static final String PROP_HTTP_PROXY = "xjc.http.proxy";
  public static final String PROP_DO_STRICT_VALIDATION = "xjc.strict.validation";
  public static final String PROP_ALLOW_EXTENSIONS = "xjc.allow.extensions";
  public static final String PROP_CREATE_RO_FILES = "xjc.create.ro.files";
  protected static Map<String, String> settings = new HashMap();

  public XjcWizard(int width, int height, WizardMainFrame.WizardMode mode)
  {
    super("XJC Wizard", width, height, mode);

    this.steps.addLast(new InputPackageStep(this));
    this.steps.addLast(new OutputDirectoryStep(this));
    this.steps.addLast(new InputAdditionalStep(this));

    setQuitListener(new QuitActionListener(this));

    setIconImage(ImageCreator.getIconImage());
  }

  public List<WizardStep> getSteps() {
    return Collections.unmodifiableList(this.steps);
  }

  public static void setSetting(String name, String value) {
    settings.put(name, value);
  }

  public static String getSetting(String name) {
    return (String)settings.get(name);
  }

  public static Map<String, String> getSettings() {
    return Collections.unmodifiableMap(settings);
  }

  private static final class QuitActionListener
    implements ActionListener
  {
    private final XjcWizard wmf;

    public QuitActionListener(XjcWizard wmf)
    {
      this.wmf = wmf;
    }

    public final void actionPerformed(ActionEvent e) {
      this.wmf.dispose();
    }
  }
}