package org.quartz.core.jmx;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import javax.management.openmbean.CompositeData;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.CompositeType;
import javax.management.openmbean.OpenDataException;
import javax.management.openmbean.OpenType;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.TabularData;
import javax.management.openmbean.TabularDataSupport;
import javax.management.openmbean.TabularType;
import org.quartz.JobDataMap;

public class JobDataMapSupport
{
  private static final String typeName = "JobDataMap";
  private static final String[] keyValue = { "key", "value" };
  private static final OpenType[] openTypes = { SimpleType.STRING, SimpleType.STRING };
  private static final CompositeType rowType;
  public static final TabularType TABULAR_TYPE;

  public static JobDataMap newJobDataMap(TabularData tabularData)
  {
    JobDataMap jobDataMap = new JobDataMap();
    Iterator pos;
    if (tabularData != null) {
      for (pos = tabularData.values().iterator(); pos.hasNext(); ) {
        CompositeData cData = (CompositeData)pos.next();
        jobDataMap.put((String)cData.get("key"), (String)cData.get("value"));
      }
    }

    return jobDataMap;
  }

  public static JobDataMap newJobDataMap(Map<String, Object> map) {
    JobDataMap jobDataMap = new JobDataMap();
    Iterator pos;
    if (map != null) {
      for (pos = map.keySet().iterator(); pos.hasNext(); ) {
        String key = (String)pos.next();
        jobDataMap.put(key, map.get(key));
      }
    }

    return jobDataMap;
  }

  public static CompositeData toCompositeData(String key, String value)
  {
    try
    {
      return new CompositeDataSupport(rowType, keyValue, new Object[] { key, value });
    }
    catch (OpenDataException e) {
      throw new RuntimeException(e);
    }
  }

  public static TabularData toTabularData(JobDataMap jobDataMap)
  {
    TabularData tData = new TabularDataSupport(TABULAR_TYPE);
    ArrayList list = new ArrayList();
    Iterator iter = jobDataMap.keySet().iterator();
    while (iter.hasNext()) {
      String key = (String)iter.next();
      list.add(toCompositeData(key, String.valueOf(jobDataMap.get(key))));
    }
    tData.putAll((CompositeData[])list.toArray(new CompositeData[list.size()]));
    return tData;
  }

  static
  {
    try
    {
      rowType = new CompositeType("JobDataMap", "JobDataMap", keyValue, keyValue, openTypes);

      TABULAR_TYPE = new TabularType("JobDataMap", "JobDataMap", rowType, new String[] { "key" });
    }
    catch (OpenDataException e) {
      throw new RuntimeException(e);
    }
  }
}