package org.quartz.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import org.quartz.Calendar;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.Scheduler;
import org.quartz.Trigger;
import org.quartz.spi.OperableTrigger;
import org.quartz.spi.TriggerFiredBundle;

public class JobExecutionContextImpl
  implements Serializable, JobExecutionContext
{
  private transient Scheduler scheduler;
  private Trigger trigger;
  private JobDetail jobDetail;
  private JobDataMap jobDataMap;
  private transient Job job;
  private Calendar calendar;
  private boolean recovering = false;

  private int numRefires = 0;
  private Date fireTime;
  private Date scheduledFireTime;
  private Date prevFireTime;
  private Date nextFireTime;
  private long jobRunTime = -1L;
  private Object result;
  private HashMap<Object, Object> data = new HashMap();

  public JobExecutionContextImpl(Scheduler scheduler, TriggerFiredBundle firedBundle, Job job)
  {
    this.scheduler = scheduler;
    this.trigger = firedBundle.getTrigger();
    this.calendar = firedBundle.getCalendar();
    this.jobDetail = firedBundle.getJobDetail();
    this.job = job;
    this.recovering = firedBundle.isRecovering();
    this.fireTime = firedBundle.getFireTime();
    this.scheduledFireTime = firedBundle.getScheduledFireTime();
    this.prevFireTime = firedBundle.getPrevFireTime();
    this.nextFireTime = firedBundle.getNextFireTime();

    this.jobDataMap = new JobDataMap();
    this.jobDataMap.putAll(this.jobDetail.getJobDataMap());
    this.jobDataMap.putAll(this.trigger.getJobDataMap());
  }

  public Scheduler getScheduler()
  {
    return this.scheduler;
  }

  public Trigger getTrigger()
  {
    return this.trigger;
  }

  public Calendar getCalendar()
  {
    return this.calendar;
  }

  public boolean isRecovering()
  {
    return this.recovering;
  }

  public void incrementRefireCount() {
    this.numRefires += 1;
  }

  public int getRefireCount()
  {
    return this.numRefires;
  }

  public JobDataMap getMergedJobDataMap()
  {
    return this.jobDataMap;
  }

  public JobDetail getJobDetail()
  {
    return this.jobDetail;
  }

  public Job getJobInstance()
  {
    return this.job;
  }

  public Date getFireTime()
  {
    return this.fireTime;
  }

  public Date getScheduledFireTime()
  {
    return this.scheduledFireTime;
  }

  public Date getPreviousFireTime()
  {
    return this.prevFireTime;
  }

  public Date getNextFireTime()
  {
    return this.nextFireTime;
  }

  public String toString()
  {
    return "JobExecutionContext: trigger: '" + getTrigger().getKey() + " job: " + getJobDetail().getKey() + " fireTime: '" + getFireTime() + " scheduledFireTime: " + getScheduledFireTime() + " previousFireTime: '" + getPreviousFireTime() + " nextFireTime: " + getNextFireTime() + " isRecovering: " + isRecovering() + " refireCount: " + getRefireCount();
  }

  public Object getResult()
  {
    return this.result;
  }

  public void setResult(Object result)
  {
    this.result = result;
  }

  public long getJobRunTime()
  {
    return this.jobRunTime;
  }

  public void setJobRunTime(long jobRunTime)
  {
    this.jobRunTime = jobRunTime;
  }

  public void put(Object key, Object value)
  {
    this.data.put(key, value);
  }

  public Object get(Object key)
  {
    return this.data.get(key);
  }

  public String getFireInstanceId()
  {
    return ((OperableTrigger)this.trigger).getFireInstanceId();
  }
}