package org.quartz.impl.calendar;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.TimeZone;

public class AnnualCalendar extends BaseCalendar
  implements org.quartz.Calendar, Serializable
{
  static final long serialVersionUID = 7346867105876610961L;
  private ArrayList<java.util.Calendar> excludeDays = new ArrayList();

  private boolean dataSorted = false;

  public AnnualCalendar() {
  }

  public AnnualCalendar(org.quartz.Calendar baseCalendar) {
    super(baseCalendar);
  }

  public AnnualCalendar(TimeZone timeZone) {
    super(timeZone);
  }

  public AnnualCalendar(org.quartz.Calendar baseCalendar, TimeZone timeZone) {
    super(baseCalendar, timeZone);
  }

  public Object clone()
  {
    AnnualCalendar clone = (AnnualCalendar)super.clone();
    clone.excludeDays = new ArrayList(this.excludeDays);
    return clone;
  }

  public ArrayList<java.util.Calendar> getDaysExcluded()
  {
    return this.excludeDays;
  }

  public boolean isDayExcluded(java.util.Calendar day)
  {
    if (day == null) {
      throw new IllegalArgumentException("Parameter day must not be null");
    }

    if (!super.isTimeIncluded(day.getTime().getTime())) {
      return true;
    }

    int dmonth = day.get(2);
    int dday = day.get(5);

    if (!this.dataSorted) {
      Collections.sort(this.excludeDays, new CalendarComparator());
      this.dataSorted = true;
    }

    Iterator iter = this.excludeDays.iterator();
    while (iter.hasNext()) {
      java.util.Calendar cl = (java.util.Calendar)iter.next();

      if (dmonth < cl.get(2)) {
        return false;
      }

      if ((dday == cl.get(5)) && 
        (dmonth == cl.get(2)))
      {
        return true;
      }
    }
    return false;
  }

  public void setDaysExcluded(ArrayList<java.util.Calendar> days)
  {
    if (days == null)
      this.excludeDays = new ArrayList();
    else {
      this.excludeDays = days;
    }

    this.dataSorted = false;
  }

  public void setDayExcluded(java.util.Calendar day, boolean exclude)
  {
    if (exclude) {
      if (isDayExcluded(day)) {
        return;
      }

      this.excludeDays.add(day);
      this.dataSorted = false;
    } else {
      if (!isDayExcluded(day)) {
        return;
      }

      removeExcludedDay(day, true);
    }
  }

  public void removeExcludedDay(java.util.Calendar day)
  {
    removeExcludedDay(day, false);
  }

  private void removeExcludedDay(java.util.Calendar day, boolean isChecked) {
    if ((!isChecked) && (!isDayExcluded(day)))
    {
      return;
    }

    if (this.excludeDays.remove(day)) {
      return;
    }

    int dmonth = day.get(2);
    int dday = day.get(5);

    Iterator iter = this.excludeDays.iterator();
    while (iter.hasNext()) {
      java.util.Calendar cl = (java.util.Calendar)iter.next();

      if ((dmonth == cl.get(2)) && 
        (dday == cl.get(5)))
      {
        day = cl;
      }
    }

    this.excludeDays.remove(day);
  }

  public boolean isTimeIncluded(long timeStamp)
  {
    if (!super.isTimeIncluded(timeStamp)) return false;

    java.util.Calendar day = createJavaCalendar(timeStamp);

    return !isDayExcluded(day);
  }

  public long getNextIncludedTime(long timeStamp)
  {
    long baseTime = super.getNextIncludedTime(timeStamp);
    if ((baseTime > 0L) && (baseTime > timeStamp)) {
      timeStamp = baseTime;
    }

    java.util.Calendar day = getStartOfDayJavaCalendar(timeStamp);
    if (!isDayExcluded(day)) {
      return timeStamp;
    }

    while (isDayExcluded(day) == true) {
      day.add(5, 1);
    }

    return day.getTime().getTime();
  }
}