package org.quartz.impl.calendar;

import java.text.ParseException;
import java.util.Date;
import java.util.TimeZone;
import org.quartz.Calendar;
import org.quartz.CronExpression;

public class CronCalendar extends BaseCalendar
{
  static final long serialVersionUID = -8172103999750856831L;
  CronExpression cronExpression;

  public CronCalendar(String expression)
    throws ParseException
  {
    this(null, expression, null);
  }

  public CronCalendar(Calendar baseCalendar, String expression)
    throws ParseException
  {
    this(baseCalendar, expression, null);
  }

  public CronCalendar(Calendar baseCalendar, String expression, TimeZone timeZone)
    throws ParseException
  {
    super(baseCalendar);
    this.cronExpression = new CronExpression(expression);
    this.cronExpression.setTimeZone(timeZone);
  }

  public Object clone()
  {
    CronCalendar clone = (CronCalendar)super.clone();
    clone.cronExpression = ((CronExpression)this.cronExpression.clone());
    return clone;
  }

  public TimeZone getTimeZone()
  {
    return this.cronExpression.getTimeZone();
  }

  public void setTimeZone(TimeZone timeZone)
  {
    this.cronExpression.setTimeZone(timeZone);
  }

  public boolean isTimeIncluded(long timeInMillis)
  {
    if ((getBaseCalendar() != null) && (!getBaseCalendar().isTimeIncluded(timeInMillis)))
    {
      return false;
    }

    return !this.cronExpression.isSatisfiedBy(new Date(timeInMillis));
  }

  public long getNextIncludedTime(long timeInMillis)
  {
    long nextIncludedTime = timeInMillis + 1L;

    while (!isTimeIncluded(nextIncludedTime))
    {
      if (this.cronExpression.isSatisfiedBy(new Date(nextIncludedTime))) {
        nextIncludedTime = this.cronExpression.getNextInvalidTimeAfter(new Date(nextIncludedTime)).getTime();
      }
      else if ((getBaseCalendar() != null) && (!getBaseCalendar().isTimeIncluded(nextIncludedTime)))
      {
        nextIncludedTime = getBaseCalendar().getNextIncludedTime(nextIncludedTime);
      }
      else {
        nextIncludedTime += 1L;
      }
    }

    return nextIncludedTime;
  }

  public String toString()
  {
    StringBuffer buffer = new StringBuffer();
    buffer.append("base calendar: [");
    if (getBaseCalendar() != null)
      buffer.append(getBaseCalendar().toString());
    else {
      buffer.append("null");
    }
    buffer.append("], excluded cron expression: '");
    buffer.append(this.cronExpression);
    buffer.append("'");
    return buffer.toString();
  }

  public CronExpression getCronExpression()
  {
    return this.cronExpression;
  }

  public void setCronExpression(String expression)
    throws ParseException
  {
    CronExpression newExp = new CronExpression(expression);

    this.cronExpression = newExp;
  }

  public void setCronExpression(CronExpression expression)
  {
    if (expression == null) {
      throw new IllegalArgumentException("expression cannot be null");
    }

    this.cronExpression = expression;
  }
}