package org.quartz.impl.calendar;

import java.io.Serializable;
import java.util.Collections;
import java.util.Date;
import java.util.SortedSet;
import java.util.TimeZone;
import java.util.TreeSet;

public class HolidayCalendar extends BaseCalendar
  implements org.quartz.Calendar, Serializable
{
  static final long serialVersionUID = -7590908752291814693L;
  private TreeSet<Date> dates = new TreeSet();

  public HolidayCalendar() {
  }

  public HolidayCalendar(org.quartz.Calendar baseCalendar) {
    super(baseCalendar);
  }

  public HolidayCalendar(TimeZone timeZone) {
    super(timeZone);
  }

  public HolidayCalendar(org.quartz.Calendar baseCalendar, TimeZone timeZone) {
    super(baseCalendar, timeZone);
  }

  public Object clone()
  {
    HolidayCalendar clone = (HolidayCalendar)super.clone();
    clone.dates = new TreeSet(this.dates);
    return clone;
  }

  public boolean isTimeIncluded(long timeStamp)
  {
    if (!super.isTimeIncluded(timeStamp)) {
      return false;
    }

    Date lookFor = getStartOfDayJavaCalendar(timeStamp).getTime();

    return !this.dates.contains(lookFor);
  }

  public long getNextIncludedTime(long timeStamp)
  {
    long baseTime = super.getNextIncludedTime(timeStamp);
    if ((baseTime > 0L) && (baseTime > timeStamp)) {
      timeStamp = baseTime;
    }

    java.util.Calendar day = getStartOfDayJavaCalendar(timeStamp);
    while (!isTimeIncluded(day.getTime().getTime())) {
      day.add(5, 1);
    }

    return day.getTime().getTime();
  }

  public void addExcludedDate(Date excludedDate)
  {
    Date date = getStartOfDayJavaCalendar(excludedDate.getTime()).getTime();

    this.dates.add(date);
  }

  public void removeExcludedDate(Date dateToRemove) {
    Date date = getStartOfDayJavaCalendar(dateToRemove.getTime()).getTime();
    this.dates.remove(date);
  }

  public SortedSet<Date> getExcludedDates()
  {
    return Collections.unmodifiableSortedSet(this.dates);
  }
}