package org.quartz.impl.calendar;

import java.io.Serializable;
import java.util.Date;
import java.util.TimeZone;

public class WeeklyCalendar extends BaseCalendar
  implements org.quartz.Calendar, Serializable
{
  static final long serialVersionUID = -6809298821229007586L;
  private boolean[] excludeDays = new boolean[8];

  private boolean excludeAll = false;

  public WeeklyCalendar() {
    this(null, null);
  }

  public WeeklyCalendar(org.quartz.Calendar baseCalendar) {
    this(baseCalendar, null);
  }

  public WeeklyCalendar(TimeZone timeZone) {
    super(null, timeZone);
  }

  public WeeklyCalendar(org.quartz.Calendar baseCalendar, TimeZone timeZone) {
    super(baseCalendar, timeZone);

    this.excludeDays[1] = true;
    this.excludeDays[7] = true;
    this.excludeAll = areAllDaysExcluded();
  }

  public Object clone()
  {
    WeeklyCalendar clone = (WeeklyCalendar)super.clone();
    clone.excludeDays = ((boolean[])this.excludeDays.clone());
    return clone;
  }

  public boolean[] getDaysExcluded()
  {
    return this.excludeDays;
  }

  public boolean isDayExcluded(int wday)
  {
    return this.excludeDays[wday];
  }

  public void setDaysExcluded(boolean[] weekDays)
  {
    if (weekDays == null) {
      return;
    }

    this.excludeDays = weekDays;
    this.excludeAll = areAllDaysExcluded();
  }

  public void setDayExcluded(int wday, boolean exclude)
  {
    this.excludeDays[wday] = exclude;
    this.excludeAll = areAllDaysExcluded();
  }

  public boolean areAllDaysExcluded()
  {
    return (isDayExcluded(1)) && (isDayExcluded(2)) && (isDayExcluded(3)) && (isDayExcluded(4)) && (isDayExcluded(5)) && (isDayExcluded(6)) && (isDayExcluded(7));
  }

  public boolean isTimeIncluded(long timeStamp)
  {
    if (this.excludeAll == true) {
      return false;
    }

    if (!super.isTimeIncluded(timeStamp)) return false;

    java.util.Calendar cl = createJavaCalendar(timeStamp);
    int wday = cl.get(7);

    return !isDayExcluded(wday);
  }

  public long getNextIncludedTime(long timeStamp)
  {
    if (this.excludeAll == true) {
      return 0L;
    }

    long baseTime = super.getNextIncludedTime(timeStamp);
    if ((baseTime > 0L) && (baseTime > timeStamp)) {
      timeStamp = baseTime;
    }

    java.util.Calendar cl = getStartOfDayJavaCalendar(timeStamp);
    int wday = cl.get(7);

    if (!isDayExcluded(wday)) {
      return timeStamp;
    }

    while (isDayExcluded(wday) == true) {
      cl.add(5, 1);
      wday = cl.get(7);
    }

    return cl.getTime().getTime();
  }
}