package org.quartz.impl.jdbcjobstore;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import org.quartz.JobKey;
import org.quartz.spi.ClassLoadHelper;
import org.slf4j.Logger;

public class DB2v6Delegate extends StdJDBCDelegate
{
  public static final String SELECT_NUM_JOBS = "SELECT COUNT(*) FROM {0}JOB_DETAILS WHERE SCHED_NAME = {1}";
  public static final String SELECT_NUM_TRIGGERS_FOR_JOB = "SELECT COUNT(*) FROM {0}TRIGGERS WHERE SCHED_NAME = {1} AND JOB_NAME = ? AND JOB_GROUP = ?";
  public static final String SELECT_NUM_TRIGGERS = "SELECT COUNT(*) FROM {0}TRIGGERS WHERE SCHED_NAME = {1}";
  public static final String SELECT_NUM_CALENDARS = "SELECT COUNT(*) FROM {0}CALENDARS WHERE SCHED_NAME = {1}";

  public DB2v6Delegate(Logger logger, String tablePrefix, String schedName, String instanceId, ClassLoadHelper classLoadHelper)
  {
    super(logger, tablePrefix, schedName, instanceId, classLoadHelper);
  }

  public DB2v6Delegate(Logger logger, String tablePrefix, String schedName, String instanceId, ClassLoadHelper classLoadHelper, Boolean useProperties)
  {
    super(logger, tablePrefix, schedName, instanceId, classLoadHelper, useProperties);
  }

  public int selectNumJobs(Connection conn) throws SQLException
  {
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      int count = 0;
      ps = conn.prepareStatement(rtp("SELECT COUNT(*) FROM {0}JOB_DETAILS WHERE SCHED_NAME = {1}"));
      rs = ps.executeQuery();

      if (rs.next()) {
        count = rs.getInt(1);
      }

      return count;
    } finally {
      closeResultSet(rs);
      closeStatement(ps);
    }
  }

  public int selectNumTriggersForJob(Connection conn, JobKey jobKey) throws SQLException
  {
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      ps = conn.prepareStatement(rtp("SELECT COUNT(*) FROM {0}TRIGGERS WHERE SCHED_NAME = {1} AND JOB_NAME = ? AND JOB_GROUP = ?"));
      ps.setString(1, jobKey.getName());
      ps.setString(2, jobKey.getGroup());
      rs = ps.executeQuery();
      int i;
      if (rs.next()) {
        return rs.getInt(1);
      }
      return 0;
    }
    finally {
      closeResultSet(rs);
      closeStatement(ps);
    }
  }

  public int selectNumTriggers(Connection conn) throws SQLException
  {
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      int count = 0;
      ps = conn.prepareStatement(rtp("SELECT COUNT(*) FROM {0}TRIGGERS WHERE SCHED_NAME = {1}"));
      rs = ps.executeQuery();

      if (rs.next()) {
        count = rs.getInt(1);
      }

      return count;
    } finally {
      closeResultSet(rs);
      closeStatement(ps);
    }
  }

  public int selectNumCalendars(Connection conn) throws SQLException
  {
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      int count = 0;
      ps = conn.prepareStatement(rtp("SELECT COUNT(*) FROM {0}CALENDARS WHERE SCHED_NAME = {1}"));
      rs = ps.executeQuery();

      if (rs.next()) {
        count = rs.getInt(1);
      }

      return count;
    } finally {
      closeResultSet(rs);
      closeStatement(ps);
    }
  }
}