package org.quartz.impl.matchers;

import org.quartz.Matcher;
import org.quartz.utils.Key;

public class AndMatcher<T extends Key<?>>
  implements Matcher<T>
{
  protected Matcher<T> leftOperand;
  protected Matcher<T> rightOperand;

  protected AndMatcher(Matcher<T> leftOperand, Matcher<T> rightOperand)
  {
    if ((leftOperand == null) || (rightOperand == null)) {
      throw new IllegalArgumentException("Two non-null operands required!");
    }
    this.leftOperand = leftOperand;
    this.rightOperand = rightOperand;
  }

  public static <U extends Key<?>> AndMatcher<U> and(Matcher<U> leftOperand, Matcher<U> rightOperand)
  {
    return new AndMatcher(leftOperand, rightOperand);
  }

  public boolean isMatch(T key)
  {
    return (this.leftOperand.isMatch(key)) && (this.rightOperand.isMatch(key));
  }

  public Matcher<T> getLeftOperand() {
    return this.leftOperand;
  }

  public Matcher<T> getRightOperand() {
    return this.rightOperand;
  }

  public int hashCode()
  {
    int prime = 31;
    int result = 1;
    result = 31 * result + (this.leftOperand == null ? 0 : this.leftOperand.hashCode());

    result = 31 * result + (this.rightOperand == null ? 0 : this.rightOperand.hashCode());

    return result;
  }

  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    AndMatcher other = (AndMatcher)obj;
    if (this.leftOperand == null) {
      if (other.leftOperand != null)
        return false;
    } else if (!this.leftOperand.equals(other.leftOperand))
      return false;
    if (this.rightOperand == null) {
      if (other.rightOperand != null)
        return false;
    } else if (!this.rightOperand.equals(other.rightOperand))
      return false;
    return true;
  }
}