package org.quartz.impl.matchers;

import org.quartz.Matcher;
import org.quartz.utils.Key;

public class NotMatcher<T extends Key<?>>
  implements Matcher<T>
{
  protected Matcher<T> operand;

  protected NotMatcher(Matcher<T> operand)
  {
    if (operand == null) {
      throw new IllegalArgumentException("Non-null operand required!");
    }
    this.operand = operand;
  }

  public static <U extends Key<?>> NotMatcher<U> not(Matcher<U> operand)
  {
    return new NotMatcher(operand);
  }

  public boolean isMatch(T key)
  {
    return !this.operand.isMatch(key);
  }

  public Matcher<T> getOperand() {
    return this.operand;
  }

  public int hashCode()
  {
    int prime = 31;
    int result = 1;
    result = 31 * result + (this.operand == null ? 0 : this.operand.hashCode());
    return result;
  }

  public boolean equals(Object obj)
  {
    if (this == obj)
      return true;
    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    NotMatcher other = (NotMatcher)obj;
    if (this.operand == null) {
      if (other.operand != null)
        return false;
    } else if (!this.operand.equals(other.operand))
      return false;
    return true;
  }
}