package org.quartz.impl.triggers;

import java.text.ParseException;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.TimeZone;
import org.quartz.CronExpression;
import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.ScheduleBuilder;

public class CronTriggerImpl extends AbstractTrigger<CronTrigger>
  implements CronTrigger, CoreTrigger
{
  private static final long serialVersionUID = -8644953146451592766L;
  protected static final int YEAR_TO_GIVEUP_SCHEDULING_AT = CronExpression.MAX_YEAR;

  private CronExpression cronEx = null;
  private Date startTime = null;
  private Date endTime = null;
  private Date nextFireTime = null;
  private Date previousFireTime = null;
  private transient TimeZone timeZone = null;

  public CronTriggerImpl()
  {
    setStartTime(new Date());
    setTimeZone(TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name)
  {
    this(name, null);
  }

  @Deprecated
  public CronTriggerImpl(String name, String group)
  {
    super(name, group);
    setStartTime(new Date());
    setTimeZone(TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String cronExpression)
    throws ParseException
  {
    super(name, group);

    setCronExpression(cronExpression);

    setStartTime(new Date());
    setTimeZone(TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String jobName, String jobGroup)
  {
    super(name, group, jobName, jobGroup);
    setStartTime(new Date());
    setTimeZone(TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String jobName, String jobGroup, String cronExpression)
    throws ParseException
  {
    this(name, group, jobName, jobGroup, null, null, cronExpression, TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String jobName, String jobGroup, String cronExpression, TimeZone timeZone)
    throws ParseException
  {
    this(name, group, jobName, jobGroup, null, null, cronExpression, timeZone);
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String jobName, String jobGroup, Date startTime, Date endTime, String cronExpression)
    throws ParseException
  {
    super(name, group, jobName, jobGroup);

    setCronExpression(cronExpression);

    if (startTime == null) {
      startTime = new Date();
    }
    setStartTime(startTime);
    if (endTime != null) {
      setEndTime(endTime);
    }
    setTimeZone(TimeZone.getDefault());
  }

  @Deprecated
  public CronTriggerImpl(String name, String group, String jobName, String jobGroup, Date startTime, Date endTime, String cronExpression, TimeZone timeZone)
    throws ParseException
  {
    super(name, group, jobName, jobGroup);

    setCronExpression(cronExpression);

    if (startTime == null) {
      startTime = new Date();
    }
    setStartTime(startTime);
    if (endTime != null) {
      setEndTime(endTime);
    }
    if (timeZone == null)
      setTimeZone(TimeZone.getDefault());
    else
      setTimeZone(timeZone);
  }

  public Object clone()
  {
    CronTriggerImpl copy = (CronTriggerImpl)super.clone();
    if (this.cronEx != null) {
      copy.setCronExpression((CronExpression)this.cronEx.clone());
    }
    return copy;
  }

  public void setCronExpression(String cronExpression) throws ParseException {
    TimeZone origTz = getTimeZone();
    this.cronEx = new CronExpression(cronExpression);
    this.cronEx.setTimeZone(origTz);
  }

  public String getCronExpression()
  {
    return this.cronEx == null ? null : this.cronEx.getCronExpression();
  }

  public void setCronExpression(CronExpression cronExpression)
  {
    this.cronEx = cronExpression;
    this.timeZone = cronExpression.getTimeZone();
  }

  public Date getStartTime()
  {
    return this.startTime;
  }

  public void setStartTime(Date startTime)
  {
    if (startTime == null) {
      throw new IllegalArgumentException("Start time cannot be null");
    }

    Date eTime = getEndTime();
    if ((eTime != null) && (startTime != null) && (eTime.before(startTime))) {
      throw new IllegalArgumentException("End time cannot be before start time");
    }

    java.util.Calendar cl = java.util.Calendar.getInstance();
    cl.setTime(startTime);
    cl.set(14, 0);

    this.startTime = cl.getTime();
  }

  public Date getEndTime()
  {
    return this.endTime;
  }

  public void setEndTime(Date endTime)
  {
    Date sTime = getStartTime();
    if ((sTime != null) && (endTime != null) && (sTime.after(endTime))) {
      throw new IllegalArgumentException("End time cannot be before start time");
    }

    this.endTime = endTime;
  }

  public Date getNextFireTime()
  {
    return this.nextFireTime;
  }

  public Date getPreviousFireTime()
  {
    return this.previousFireTime;
  }

  public void setNextFireTime(Date nextFireTime)
  {
    this.nextFireTime = nextFireTime;
  }

  public void setPreviousFireTime(Date previousFireTime)
  {
    this.previousFireTime = previousFireTime;
  }

  public TimeZone getTimeZone()
  {
    if (this.cronEx != null) {
      return this.cronEx.getTimeZone();
    }

    if (this.timeZone == null) {
      this.timeZone = TimeZone.getDefault();
    }
    return this.timeZone;
  }

  public void setTimeZone(TimeZone timeZone)
  {
    if (this.cronEx != null) {
      this.cronEx.setTimeZone(timeZone);
    }
    this.timeZone = timeZone;
  }

  public Date getFireTimeAfter(Date afterTime)
  {
    if (afterTime == null) {
      afterTime = new Date();
    }

    if (getStartTime().after(afterTime)) {
      afterTime = new Date(getStartTime().getTime() - 1000L);
    }

    if ((getEndTime() != null) && (afterTime.compareTo(getEndTime()) >= 0)) {
      return null;
    }

    Date pot = getTimeAfter(afterTime);
    if ((getEndTime() != null) && (pot != null) && (pot.after(getEndTime()))) {
      return null;
    }

    return pot;
  }

  public Date getFinalFireTime()
  {
    Date resultTime;
    Date resultTime;
    if (getEndTime() != null)
      resultTime = getTimeBefore(new Date(getEndTime().getTime() + 1000L));
    else {
      resultTime = this.cronEx == null ? null : this.cronEx.getFinalFireTime();
    }

    if ((resultTime != null) && (getStartTime() != null) && (resultTime.before(getStartTime()))) {
      return null;
    }

    return resultTime;
  }

  public boolean mayFireAgain()
  {
    return getNextFireTime() != null;
  }

  protected boolean validateMisfireInstruction(int misfireInstruction)
  {
    if (misfireInstruction < -1) {
      return false;
    }

    if (misfireInstruction > 2) {
      return false;
    }

    return true;
  }

  public void updateAfterMisfire(org.quartz.Calendar cal)
  {
    int instr = getMisfireInstruction();

    if (instr == -1) {
      return;
    }
    if (instr == 0) {
      instr = 1;
    }

    if (instr == 2) {
      Date newFireTime = getFireTimeAfter(new Date());

      while ((newFireTime != null) && (cal != null) && (!cal.isTimeIncluded(newFireTime.getTime()))) {
        newFireTime = getFireTimeAfter(newFireTime);
      }
      setNextFireTime(newFireTime);
    } else if (instr == 1) {
      setNextFireTime(new Date());
    }
  }

  public boolean willFireOn(java.util.Calendar test)
  {
    return willFireOn(test, false);
  }

  public boolean willFireOn(java.util.Calendar test, boolean dayOnly)
  {
    test = (java.util.Calendar)test.clone();

    test.set(14, 0);

    if (dayOnly) {
      test.set(11, 0);
      test.set(12, 0);
      test.set(13, 0);
    }

    Date testTime = test.getTime();

    Date fta = getFireTimeAfter(new Date(test.getTime().getTime() - 1000L));

    if (fta == null) {
      return false;
    }
    java.util.Calendar p = java.util.Calendar.getInstance(test.getTimeZone());
    p.setTime(fta);

    int year = p.get(1);
    int month = p.get(2);
    int day = p.get(5);

    if (dayOnly) {
      return (year == test.get(1)) && (month == test.get(2)) && (day == test.get(5));
    }

    while (fta.before(testTime)) {
      fta = getFireTimeAfter(fta);
    }

    if (fta.equals(testTime)) {
      return true;
    }

    return false;
  }

  public void triggered(org.quartz.Calendar calendar)
  {
    this.previousFireTime = this.nextFireTime;
    this.nextFireTime = getFireTimeAfter(this.nextFireTime);

    while ((this.nextFireTime != null) && (calendar != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime())))
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);
  }

  public void updateWithNewCalendar(org.quartz.Calendar calendar, long misfireThreshold)
  {
    this.nextFireTime = getFireTimeAfter(this.previousFireTime);

    if ((this.nextFireTime == null) || (calendar == null)) {
      return;
    }

    Date now = new Date();
    while ((this.nextFireTime != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime())))
    {
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);

      if (this.nextFireTime == null)
      {
        break;
      }

      java.util.Calendar c = new GregorianCalendar();
      c.setTime(this.nextFireTime);
      if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT) {
        this.nextFireTime = null;
      }

      if ((this.nextFireTime != null) && (this.nextFireTime.before(now))) {
        long diff = now.getTime() - this.nextFireTime.getTime();
        if (diff >= misfireThreshold)
          this.nextFireTime = getFireTimeAfter(this.nextFireTime);
      }
    }
  }

  public Date computeFirstFireTime(org.quartz.Calendar calendar)
  {
    this.nextFireTime = getFireTimeAfter(new Date(getStartTime().getTime() - 1000L));

    while ((this.nextFireTime != null) && (calendar != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime()))) {
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);
    }

    return this.nextFireTime;
  }

  public String getExpressionSummary()
  {
    return this.cronEx == null ? null : this.cronEx.getExpressionSummary();
  }

  public boolean hasAdditionalProperties()
  {
    return false;
  }

  public ScheduleBuilder<CronTrigger> getScheduleBuilder()
  {
    CronScheduleBuilder cb = CronScheduleBuilder.cronSchedule(getCronExpression()).inTimeZone(getTimeZone());

    switch (getMisfireInstruction()) { case 2:
      cb.withMisfireHandlingInstructionDoNothing();
      break;
    case 1:
      cb.withMisfireHandlingInstructionFireAndProceed();
    }

    return cb;
  }

  protected Date getTimeAfter(Date afterTime)
  {
    return this.cronEx == null ? null : this.cronEx.getTimeAfter(afterTime);
  }

  protected Date getTimeBefore(Date eTime)
  {
    return this.cronEx == null ? null : this.cronEx.getTimeBefore(eTime);
  }
}