package org.quartz.impl.triggers;

import java.util.Date;
import org.quartz.ScheduleBuilder;
import org.quartz.SchedulerException;
import org.quartz.SimpleScheduleBuilder;
import org.quartz.SimpleTrigger;

public class SimpleTriggerImpl extends AbstractTrigger<SimpleTrigger>
  implements SimpleTrigger, CoreTrigger
{
  private static final long serialVersionUID = -3735980074222850397L;
  private static final int YEAR_TO_GIVEUP_SCHEDULING_AT = java.util.Calendar.getInstance().get(1) + 100;

  private Date startTime = null;

  private Date endTime = null;

  private Date nextFireTime = null;

  private Date previousFireTime = null;

  private int repeatCount = 0;

  private long repeatInterval = 0L;

  private int timesTriggered = 0;

  private boolean complete = false;

  public SimpleTriggerImpl()
  {
  }

  @Deprecated
  public SimpleTriggerImpl(String name)
  {
    this(name, (String)null);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, String group)
  {
    this(name, group, new Date(), null, 0, 0L);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, int repeatCount, long repeatInterval)
  {
    this(name, null, repeatCount, repeatInterval);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, String group, int repeatCount, long repeatInterval)
  {
    this(name, group, new Date(), null, repeatCount, repeatInterval);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, Date startTime)
  {
    this(name, null, startTime);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, String group, Date startTime)
  {
    this(name, group, startTime, null, 0, 0L);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, Date startTime, Date endTime, int repeatCount, long repeatInterval)
  {
    this(name, null, startTime, endTime, repeatCount, repeatInterval);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, String group, Date startTime, Date endTime, int repeatCount, long repeatInterval)
  {
    super(name, group);

    setStartTime(startTime);
    setEndTime(endTime);
    setRepeatCount(repeatCount);
    setRepeatInterval(repeatInterval);
  }

  @Deprecated
  public SimpleTriggerImpl(String name, String group, String jobName, String jobGroup, Date startTime, Date endTime, int repeatCount, long repeatInterval)
  {
    super(name, group, jobName, jobGroup);

    setStartTime(startTime);
    setEndTime(endTime);
    setRepeatCount(repeatCount);
    setRepeatInterval(repeatInterval);
  }

  public Date getStartTime()
  {
    return this.startTime;
  }

  public void setStartTime(Date startTime)
  {
    if (startTime == null) {
      throw new IllegalArgumentException("Start time cannot be null");
    }

    Date eTime = getEndTime();
    if ((eTime != null) && (startTime != null) && (eTime.before(startTime))) {
      throw new IllegalArgumentException("End time cannot be before start time");
    }

    this.startTime = startTime;
  }

  public Date getEndTime()
  {
    return this.endTime;
  }

  public void setEndTime(Date endTime)
  {
    Date sTime = getStartTime();
    if ((sTime != null) && (endTime != null) && (sTime.after(endTime))) {
      throw new IllegalArgumentException("End time cannot be before start time");
    }

    this.endTime = endTime;
  }

  public int getRepeatCount()
  {
    return this.repeatCount;
  }

  public void setRepeatCount(int repeatCount)
  {
    if ((repeatCount < 0) && (repeatCount != -1)) {
      throw new IllegalArgumentException("Repeat count must be >= 0, use the constant REPEAT_INDEFINITELY for infinite.");
    }

    this.repeatCount = repeatCount;
  }

  public long getRepeatInterval()
  {
    return this.repeatInterval;
  }

  public void setRepeatInterval(long repeatInterval)
  {
    if (repeatInterval < 0L) {
      throw new IllegalArgumentException("Repeat interval must be >= 0");
    }

    this.repeatInterval = repeatInterval;
  }

  public int getTimesTriggered()
  {
    return this.timesTriggered;
  }

  public void setTimesTriggered(int timesTriggered)
  {
    this.timesTriggered = timesTriggered;
  }

  protected boolean validateMisfireInstruction(int misfireInstruction)
  {
    if (misfireInstruction < -1) {
      return false;
    }

    if (misfireInstruction > 5) {
      return false;
    }

    return true;
  }

  public void updateAfterMisfire(org.quartz.Calendar cal)
  {
    int instr = getMisfireInstruction();

    if (instr == -1) {
      return;
    }
    if (instr == 0) {
      if (getRepeatCount() == 0)
        instr = 1;
      else if (getRepeatCount() == -1) {
        instr = 4;
      }
      else
        instr = 2;
    }
    else if ((instr == 1) && (getRepeatCount() != 0)) {
      instr = 3;
    }

    if (instr == 1) {
      setNextFireTime(new Date());
    } else if (instr == 5) {
      Date newFireTime = getFireTimeAfter(new Date());

      while ((newFireTime != null) && (cal != null) && (!cal.isTimeIncluded(newFireTime.getTime()))) {
        newFireTime = getFireTimeAfter(newFireTime);

        if (newFireTime == null)
        {
          break;
        }
        java.util.Calendar c = java.util.Calendar.getInstance();
        c.setTime(newFireTime);
        if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT) {
          newFireTime = null;
        }
      }
      setNextFireTime(newFireTime);
    } else if (instr == 4) {
      Date newFireTime = getFireTimeAfter(new Date());

      while ((newFireTime != null) && (cal != null) && (!cal.isTimeIncluded(newFireTime.getTime()))) {
        newFireTime = getFireTimeAfter(newFireTime);

        if (newFireTime == null)
        {
          break;
        }
        java.util.Calendar c = java.util.Calendar.getInstance();
        c.setTime(newFireTime);
        if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT) {
          newFireTime = null;
        }
      }
      if (newFireTime != null) {
        int timesMissed = computeNumTimesFiredBetween(this.nextFireTime, newFireTime);

        setTimesTriggered(getTimesTriggered() + timesMissed);
      }

      setNextFireTime(newFireTime);
    } else if (instr == 2) {
      Date newFireTime = new Date();
      if ((this.repeatCount != 0) && (this.repeatCount != -1)) {
        setRepeatCount(getRepeatCount() - getTimesTriggered());
        setTimesTriggered(0);
      }

      if ((getEndTime() != null) && (getEndTime().before(newFireTime))) {
        setNextFireTime(null);
      } else {
        setStartTime(newFireTime);
        setNextFireTime(newFireTime);
      }
    } else if (instr == 3) {
      Date newFireTime = new Date();

      int timesMissed = computeNumTimesFiredBetween(this.nextFireTime, newFireTime);

      if ((this.repeatCount != 0) && (this.repeatCount != -1)) {
        int remainingCount = getRepeatCount() - (getTimesTriggered() + timesMissed);

        if (remainingCount <= 0) {
          remainingCount = 0;
        }
        setRepeatCount(remainingCount);
        setTimesTriggered(0);
      }

      if ((getEndTime() != null) && (getEndTime().before(newFireTime))) {
        setNextFireTime(null);
      } else {
        setStartTime(newFireTime);
        setNextFireTime(newFireTime);
      }
    }
  }

  public void triggered(org.quartz.Calendar calendar)
  {
    this.timesTriggered += 1;
    this.previousFireTime = this.nextFireTime;
    this.nextFireTime = getFireTimeAfter(this.nextFireTime);

    while ((this.nextFireTime != null) && (calendar != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime())))
    {
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);

      if (this.nextFireTime == null)
      {
        break;
      }
      java.util.Calendar c = java.util.Calendar.getInstance();
      c.setTime(this.nextFireTime);
      if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT)
        this.nextFireTime = null;
    }
  }

  public void updateWithNewCalendar(org.quartz.Calendar calendar, long misfireThreshold)
  {
    this.nextFireTime = getFireTimeAfter(this.previousFireTime);

    if ((this.nextFireTime == null) || (calendar == null)) {
      return;
    }

    Date now = new Date();
    while ((this.nextFireTime != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime())))
    {
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);

      if (this.nextFireTime == null)
      {
        break;
      }
      java.util.Calendar c = java.util.Calendar.getInstance();
      c.setTime(this.nextFireTime);
      if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT) {
        this.nextFireTime = null;
      }

      if ((this.nextFireTime != null) && (this.nextFireTime.before(now))) {
        long diff = now.getTime() - this.nextFireTime.getTime();
        if (diff >= misfireThreshold)
          this.nextFireTime = getFireTimeAfter(this.nextFireTime);
      }
    }
  }

  public Date computeFirstFireTime(org.quartz.Calendar calendar)
  {
    this.nextFireTime = getStartTime();

    while ((this.nextFireTime != null) && (calendar != null) && (!calendar.isTimeIncluded(this.nextFireTime.getTime()))) {
      this.nextFireTime = getFireTimeAfter(this.nextFireTime);

      if (this.nextFireTime == null)
      {
        break;
      }
      java.util.Calendar c = java.util.Calendar.getInstance();
      c.setTime(this.nextFireTime);
      if (c.get(1) > YEAR_TO_GIVEUP_SCHEDULING_AT) {
        return null;
      }
    }

    return this.nextFireTime;
  }

  public Date getNextFireTime()
  {
    return this.nextFireTime;
  }

  public Date getPreviousFireTime()
  {
    return this.previousFireTime;
  }

  public void setNextFireTime(Date nextFireTime)
  {
    this.nextFireTime = nextFireTime;
  }

  public void setPreviousFireTime(Date previousFireTime)
  {
    this.previousFireTime = previousFireTime;
  }

  public Date getFireTimeAfter(Date afterTime)
  {
    if (this.complete) {
      return null;
    }

    if ((this.timesTriggered > this.repeatCount) && (this.repeatCount != -1))
    {
      return null;
    }

    if (afterTime == null) {
      afterTime = new Date();
    }

    if ((this.repeatCount == 0) && (afterTime.compareTo(getStartTime()) >= 0)) {
      return null;
    }

    long startMillis = getStartTime().getTime();
    long afterMillis = afterTime.getTime();
    long endMillis = getEndTime() == null ? 9223372036854775807L : getEndTime().getTime();

    if (endMillis <= afterMillis) {
      return null;
    }

    if (afterMillis < startMillis) {
      return new Date(startMillis);
    }

    long numberOfTimesExecuted = (afterMillis - startMillis) / this.repeatInterval + 1L;

    if ((numberOfTimesExecuted > this.repeatCount) && (this.repeatCount != -1))
    {
      return null;
    }

    Date time = new Date(startMillis + numberOfTimesExecuted * this.repeatInterval);

    if (endMillis <= time.getTime()) {
      return null;
    }

    return time;
  }

  public Date getFireTimeBefore(Date end)
  {
    if (end.getTime() < getStartTime().getTime()) {
      return null;
    }

    int numFires = computeNumTimesFiredBetween(getStartTime(), end);

    return new Date(getStartTime().getTime() + numFires * this.repeatInterval);
  }

  public int computeNumTimesFiredBetween(Date start, Date end)
  {
    if (this.repeatInterval < 1L) {
      return 0;
    }

    long time = end.getTime() - start.getTime();

    return (int)(time / this.repeatInterval);
  }

  public Date getFinalFireTime()
  {
    if (this.repeatCount == 0) {
      return this.startTime;
    }

    if (this.repeatCount == -1) {
      return getEndTime() == null ? null : getFireTimeBefore(getEndTime());
    }

    long lastTrigger = this.startTime.getTime() + this.repeatCount * this.repeatInterval;

    if ((getEndTime() == null) || (lastTrigger < getEndTime().getTime())) {
      return new Date(lastTrigger);
    }
    return getFireTimeBefore(getEndTime());
  }

  public boolean mayFireAgain()
  {
    return getNextFireTime() != null;
  }

  public void validate()
    throws SchedulerException
  {
    super.validate();

    if ((this.repeatCount != 0) && (this.repeatInterval < 1L))
      throw new SchedulerException("Repeat Interval cannot be zero.");
  }

  public boolean hasAdditionalProperties()
  {
    return false;
  }

  public ScheduleBuilder<SimpleTrigger> getScheduleBuilder()
  {
    SimpleScheduleBuilder sb = SimpleScheduleBuilder.simpleSchedule().withIntervalInMilliseconds(getRepeatInterval()).withRepeatCount(getRepeatCount());

    switch (getMisfireInstruction()) { case 1:
      sb.withMisfireHandlingInstructionFireNow();
      break;
    case 5:
      sb.withMisfireHandlingInstructionNextWithExistingCount();
      break;
    case 4:
      sb.withMisfireHandlingInstructionNextWithRemainingCount();
      break;
    case 2:
      sb.withMisfireHandlingInstructionNowWithExistingCount();
      break;
    case 3:
      sb.withMisfireHandlingInstructionNowWithRemainingCount();
    }

    return sb;
  }
}