package org.quartz.jobs.ee.jms;

import javax.jms.Message;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.naming.Context;
import org.quartz.Job;
import org.quartz.JobDataMap;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;

public final class SendQueueMessageJob
  implements Job
{
  public void execute(JobExecutionContext jobCtx)
    throws JobExecutionException
  {
    QueueConnection conn = null;

    QueueSession sess = null;

    QueueSender sender = null;
    try
    {
      JobDataMap dataMap = jobCtx.getMergedJobDataMap();

      Context namingCtx = JmsHelper.getInitialContext(dataMap);

      QueueConnectionFactory connFactory = (QueueConnectionFactory)namingCtx.lookup(dataMap.getString("jms.connection.factory"));

      if (!JmsHelper.isDestinationSecure(dataMap)) {
        conn = connFactory.createQueueConnection();
      } else {
        String user = dataMap.getString("jms.user");

        String password = dataMap.getString("jms.password");

        conn = connFactory.createQueueConnection(user, password);
      }

      boolean useTransactions = JmsHelper.useTransaction(dataMap);

      int ackMode = dataMap.getInt("jms.acknowledge");

      sess = conn.createQueueSession(useTransactions, ackMode);

      Queue queue = (Queue)namingCtx.lookup(dataMap.getString("jms.destination"));

      sender = sess.createSender(queue);

      JmsMessageFactory msgFactory = JmsHelper.getMessageFactory(dataMap.getString("jms.message.factory.class.name"));

      Message msg = msgFactory.createMessage(dataMap, sess);

      sender.send(msg);
    } catch (Exception e) {
      throw new JobExecutionException(e.getMessage());
    } finally {
      JmsHelper.closeResource(sender);

      JmsHelper.closeResource(sess);

      JmsHelper.closeResource(conn);
    }
  }
}