package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.filer.volume.DiskInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("aggr-add")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPIMISSINGARGUMENT", "EINTERNALERROR", "EINVALIDINPUTERROR", "EVOLUMEDISKDUP", "EVOLUMEDISKSIZEINVALID", "EVOLUMEDOESNOTEXIST", "EVOLUMEGSMINVALID"})
@Family({"ontap-c", "ontap-d"})
public class AggrAdd
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String aggregate;
  public List<DiskInfo> badDisks;
  public Integer diskCount;
  public Integer diskSize;
  public String diskSizeWithUnit;
  public List<DiskInfo> disks;
  public Boolean forceSparePool;
  public String groupSelectionMode;
  public List<DiskInfo> mirrorDisks;
  public String raidGroup;

  @Input("aggregate")
  @Type("string")
  public void setAggregate(String paramString)
  {
    this.aggregate = paramString;
  }

  public String getAggregate()
  {
    return this.aggregate;
  }

  public AggrAdd withAggregate(String paramString)
  {
    this.aggregate = paramString;
    return this;
  }

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public AggrAdd withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public AggrAdd withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("disk-count")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
  }

  public Integer getDiskCount()
  {
    return this.diskCount;
  }

  public AggrAdd withDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
    return this;
  }

  @Input("disk-size")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setDiskSize(Integer paramInteger)
  {
    this.diskSize = paramInteger;
  }

  public Integer getDiskSize()
  {
    return this.diskSize;
  }

  public AggrAdd withDiskSize(Integer paramInteger)
  {
    this.diskSize = paramInteger;
    return this;
  }

  @Input("disk-size-with-unit")
  @Type("string")
  @Optional
  public void setDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
  }

  public String getDiskSizeWithUnit()
  {
    return this.diskSizeWithUnit;
  }

  public AggrAdd withDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
    return this;
  }

  @Input("disks")
  @Type("disk-info[]")
  @Optional
  public void setDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
  }

  public List<DiskInfo> getDisks()
  {
    return this.disks;
  }

  public AggrAdd withDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
    return this;
  }

  public AggrAdd withDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.disks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force-spare-pool")
  @Type("boolean")
  @Optional
  public void setForceSparePool(Boolean paramBoolean)
  {
    this.forceSparePool = paramBoolean;
  }

  public Boolean isForceSparePool()
  {
    return this.forceSparePool;
  }

  public AggrAdd withForceSparePool(Boolean paramBoolean)
  {
    this.forceSparePool = paramBoolean;
    return this;
  }

  @Input("group-selection-mode")
  @Type("string")
  @Optional
  public void setGroupSelectionMode(String paramString)
  {
    this.groupSelectionMode = paramString;
  }

  public String getGroupSelectionMode()
  {
    return this.groupSelectionMode;
  }

  public AggrAdd withGroupSelectionMode(String paramString)
  {
    this.groupSelectionMode = paramString;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public AggrAdd withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public AggrAdd withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("raid-group")
  @Type("string")
  @Optional
  public void setRaidGroup(String paramString)
  {
    this.raidGroup = paramString;
  }

  public String getRaidGroup()
  {
    return this.raidGroup;
  }

  public AggrAdd withRaidGroup(String paramString)
  {
    this.raidGroup = paramString;
    return this;
  }
}