package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.filer.volume.DiskInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("aggr-create")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPIMISSINGARGUMENT", "ECANT_USE_ALL_DISKS", "EINTERNALERROR", "EINVALIDINPUTERROR", "ESERVICENOTLICENSED"})
@Family({"ontap-c", "ontap-d"})
public class AggrCreate
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String aggregate;
  public List<DiskInfo> badDisks;
  public Long diskCount;
  public Long diskSize;
  public String diskSizeWithUnit;
  public String diskType;
  public List<DiskInfo> disks;
  public Boolean forceSparePool;
  public Boolean forceZcs;
  public Boolean mirrored;
  public Boolean snaplock;
  public String languageCode;
  public List<DiskInfo> mirrorDisks;
  public Long raidSize;
  public String raidType;
  public Long rpm;
  public String snaplockType;
  public String type;

  @Input("aggregate")
  @Type("string")
  public void setAggregate(String paramString)
  {
    this.aggregate = paramString;
  }

  public String getAggregate()
  {
    return this.aggregate;
  }

  public AggrCreate withAggregate(String paramString)
  {
    this.aggregate = paramString;
    return this;
  }

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public AggrCreate withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public AggrCreate withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("disk-count")
  @Type("integer")
  @Optional
  public void setDiskCount(Long paramLong)
  {
    this.diskCount = paramLong;
  }

  public Long getDiskCount()
  {
    return this.diskCount;
  }

  public AggrCreate withDiskCount(Long paramLong)
  {
    this.diskCount = paramLong;
    return this;
  }

  @Input("disk-size")
  @Type("integer")
  @Optional
  public void setDiskSize(Long paramLong)
  {
    this.diskSize = paramLong;
  }

  public Long getDiskSize()
  {
    return this.diskSize;
  }

  public AggrCreate withDiskSize(Long paramLong)
  {
    this.diskSize = paramLong;
    return this;
  }

  @Input("disk-size-with-unit")
  @Type("string")
  @Optional
  public void setDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
  }

  public String getDiskSizeWithUnit()
  {
    return this.diskSizeWithUnit;
  }

  public AggrCreate withDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
    return this;
  }

  @Input("disk-type")
  @Type("string")
  @Optional
  public void setDiskType(String paramString)
  {
    this.diskType = paramString;
  }

  public String getDiskType()
  {
    return this.diskType;
  }

  public AggrCreate withDiskType(String paramString)
  {
    this.diskType = paramString;
    return this;
  }

  @Input("disks")
  @Type("disk-info[]")
  @Optional
  public void setDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
  }

  public List<DiskInfo> getDisks()
  {
    return this.disks;
  }

  public AggrCreate withDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
    return this;
  }

  public AggrCreate withDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.disks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force-spare-pool")
  @Type("boolean")
  @Optional
  public void setForceSparePool(Boolean paramBoolean)
  {
    this.forceSparePool = paramBoolean;
  }

  public Boolean isForceSparePool()
  {
    return this.forceSparePool;
  }

  public AggrCreate withForceSparePool(Boolean paramBoolean)
  {
    this.forceSparePool = paramBoolean;
    return this;
  }

  @Input("force-zcs")
  @Type("boolean")
  @Optional
  public void setForceZcs(Boolean paramBoolean)
  {
    this.forceZcs = paramBoolean;
  }

  public Boolean isForceZcs()
  {
    return this.forceZcs;
  }

  public AggrCreate withForceZcs(Boolean paramBoolean)
  {
    this.forceZcs = paramBoolean;
    return this;
  }

  @Input("is-mirrored")
  @Type("boolean")
  @Optional
  public void setMirrored(Boolean paramBoolean)
  {
    this.mirrored = paramBoolean;
  }

  public Boolean isMirrored()
  {
    return this.mirrored;
  }

  public AggrCreate withMirrored(Boolean paramBoolean)
  {
    this.mirrored = paramBoolean;
    return this;
  }

  @Input("is-snaplock")
  @Type("boolean")
  @Optional
  public void setSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
  }

  public Boolean isSnaplock()
  {
    return this.snaplock;
  }

  public AggrCreate withSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
    return this;
  }

  @Input("language-code")
  @Type("string")
  @Optional
  public void setLanguageCode(String paramString)
  {
    this.languageCode = paramString;
  }

  public String getLanguageCode()
  {
    return this.languageCode;
  }

  public AggrCreate withLanguageCode(String paramString)
  {
    this.languageCode = paramString;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public AggrCreate withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public AggrCreate withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("raid-size")
  @Type("integer")
  @Optional
  public void setRaidSize(Long paramLong)
  {
    this.raidSize = paramLong;
  }

  public Long getRaidSize()
  {
    return this.raidSize;
  }

  public AggrCreate withRaidSize(Long paramLong)
  {
    this.raidSize = paramLong;
    return this;
  }

  @Input("raid-type")
  @Type("string")
  @Optional
  public void setRaidType(String paramString)
  {
    this.raidType = paramString;
  }

  public String getRaidType()
  {
    return this.raidType;
  }

  public AggrCreate withRaidType(String paramString)
  {
    this.raidType = paramString;
    return this;
  }

  @Input("rpm")
  @Type("integer")
  @Optional
  public void setRpm(Long paramLong)
  {
    this.rpm = paramLong;
  }

  public Long getRpm()
  {
    return this.rpm;
  }

  public AggrCreate withRpm(Long paramLong)
  {
    this.rpm = paramLong;
    return this;
  }

  @Input("snaplock-type")
  @Type("string")
  @Optional
  public void setSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
  }

  public String getSnaplockType()
  {
    return this.snaplockType;
  }

  public AggrCreate withSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
    return this;
  }

  @Input("type")
  @Type("string")
  @Optional
  public void setType(String paramString)
  {
    this.type = paramString;
  }

  public String getType()
  {
    return this.type;
  }

  public AggrCreate withType(String paramString)
  {
    this.type = paramString;
    return this;
  }
}