package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.filer.volume.RaidgroupSizeInfo;
import com.netapp.filer.volume.RaidtypeInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("aggr-get-filer-info")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR"})
@Family({"ontap-c", "ontap-d"})
public class AggrGetFilerInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<RaidtypeInfo> allowedRaidtypes;
  public String checksumTypes;
  public String defaultRaidtype;
  public String diskTypes;
  public Long maxNamedDisksPerRequest;
  public List<RaidgroupSizeInfo> raidgroupSize;
  public String rootVolume;
  public Integer snapshotsMax;

  @Output("allowed-raidtypes")
  @Type("raidtype-info[]")
  public void setAllowedRaidtypes(List<RaidtypeInfo> paramList)
  {
    this.allowedRaidtypes = paramList;
  }

  public List<RaidtypeInfo> getAllowedRaidtypes()
  {
    return this.allowedRaidtypes;
  }

  public AggrGetFilerInfo withAllowedRaidtypes(List<RaidtypeInfo> paramList)
  {
    this.allowedRaidtypes = paramList;
    return this;
  }

  public AggrGetFilerInfo withAllowedRaidtypes(RaidtypeInfo[] paramArrayOfRaidtypeInfo)
  {
    this.allowedRaidtypes = new ArrayList(Arrays.asList(paramArrayOfRaidtypeInfo));
    return this;
  }

  @Output("checksum-types")
  @Type("string")
  public void setChecksumTypes(String paramString)
  {
    this.checksumTypes = paramString;
  }

  public String getChecksumTypes()
  {
    return this.checksumTypes;
  }

  public AggrGetFilerInfo withChecksumTypes(String paramString)
  {
    this.checksumTypes = paramString;
    return this;
  }

  @Output("default-raidtype")
  @Type("string")
  public void setDefaultRaidtype(String paramString)
  {
    this.defaultRaidtype = paramString;
  }

  public String getDefaultRaidtype()
  {
    return this.defaultRaidtype;
  }

  public AggrGetFilerInfo withDefaultRaidtype(String paramString)
  {
    this.defaultRaidtype = paramString;
    return this;
  }

  @Output("disk-types")
  @Type("string")
  public void setDiskTypes(String paramString)
  {
    this.diskTypes = paramString;
  }

  public String getDiskTypes()
  {
    return this.diskTypes;
  }

  public AggrGetFilerInfo withDiskTypes(String paramString)
  {
    this.diskTypes = paramString;
    return this;
  }

  @Output("max-named-disks-per-request")
  @Type("integer")
  public void setMaxNamedDisksPerRequest(Long paramLong)
  {
    this.maxNamedDisksPerRequest = paramLong;
  }

  public Long getMaxNamedDisksPerRequest()
  {
    return this.maxNamedDisksPerRequest;
  }

  public AggrGetFilerInfo withMaxNamedDisksPerRequest(Long paramLong)
  {
    this.maxNamedDisksPerRequest = paramLong;
    return this;
  }

  @Output("raidgroup-size")
  @Type("raidgroup-size-info[]")
  public void setRaidgroupSize(List<RaidgroupSizeInfo> paramList)
  {
    this.raidgroupSize = paramList;
  }

  public List<RaidgroupSizeInfo> getRaidgroupSize()
  {
    return this.raidgroupSize;
  }

  public AggrGetFilerInfo withRaidgroupSize(List<RaidgroupSizeInfo> paramList)
  {
    this.raidgroupSize = paramList;
    return this;
  }

  public AggrGetFilerInfo withRaidgroupSize(RaidgroupSizeInfo[] paramArrayOfRaidgroupSizeInfo)
  {
    this.raidgroupSize = new ArrayList(Arrays.asList(paramArrayOfRaidgroupSizeInfo));
    return this;
  }

  @Output("root-volume")
  @Type("string")
  public void setRootVolume(String paramString)
  {
    this.rootVolume = paramString;
  }

  public String getRootVolume()
  {
    return this.rootVolume;
  }

  public AggrGetFilerInfo withRootVolume(String paramString)
  {
    this.rootVolume = paramString;
    return this;
  }

  @Output("snapshots-max")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setSnapshotsMax(Integer paramInteger)
  {
    this.snapshotsMax = paramInteger;
  }

  public Integer getSnapshotsMax()
  {
    return this.snapshotsMax;
  }

  public AggrGetFilerInfo withSnapshotsMax(Integer paramInteger)
  {
    this.snapshotsMax = paramInteger;
    return this;
  }
}