package com.netapp.filer.aggr;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Typedef("aggr-space-info")
public class AggrSpaceInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String aggregateName;
  public BigInteger sizeAsisUsed;
  public BigInteger sizeBssmNvlogUsed;
  public BigInteger sizeFree;
  public BigInteger sizeMetadata;
  public BigInteger sizeNominal;
  public BigInteger sizeSnapUsed;
  public BigInteger sizeUsed;
  public BigInteger sizeVolumeAllocated;
  public BigInteger sizeVolumeUsed;
  public BigInteger volumeCount;
  public List<VolumeSpaceInfo> volumes;

  @Element("aggregate-name")
  @Type("string")
  public void setAggregateName(String paramString)
  {
    this.aggregateName = paramString;
  }

  public String getAggregateName()
  {
    return this.aggregateName;
  }

  public AggrSpaceInfo withAggregateName(String paramString)
  {
    this.aggregateName = paramString;
    return this;
  }

  @Element("size-asis-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeAsisUsed(BigInteger paramBigInteger)
  {
    this.sizeAsisUsed = paramBigInteger;
  }

  public BigInteger getSizeAsisUsed()
  {
    return this.sizeAsisUsed;
  }

  public AggrSpaceInfo withSizeAsisUsed(BigInteger paramBigInteger)
  {
    this.sizeAsisUsed = paramBigInteger;
    return this;
  }

  @Element("size-bssm-nvlog-used")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setSizeBssmNvlogUsed(BigInteger paramBigInteger)
  {
    this.sizeBssmNvlogUsed = paramBigInteger;
  }

  public BigInteger getSizeBssmNvlogUsed()
  {
    return this.sizeBssmNvlogUsed;
  }

  public AggrSpaceInfo withSizeBssmNvlogUsed(BigInteger paramBigInteger)
  {
    this.sizeBssmNvlogUsed = paramBigInteger;
    return this;
  }

  @Element("size-free")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeFree(BigInteger paramBigInteger)
  {
    this.sizeFree = paramBigInteger;
  }

  public BigInteger getSizeFree()
  {
    return this.sizeFree;
  }

  public AggrSpaceInfo withSizeFree(BigInteger paramBigInteger)
  {
    this.sizeFree = paramBigInteger;
    return this;
  }

  @Element("size-metadata")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeMetadata(BigInteger paramBigInteger)
  {
    this.sizeMetadata = paramBigInteger;
  }

  public BigInteger getSizeMetadata()
  {
    return this.sizeMetadata;
  }

  public AggrSpaceInfo withSizeMetadata(BigInteger paramBigInteger)
  {
    this.sizeMetadata = paramBigInteger;
    return this;
  }

  @Element("size-nominal")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeNominal(BigInteger paramBigInteger)
  {
    this.sizeNominal = paramBigInteger;
  }

  public BigInteger getSizeNominal()
  {
    return this.sizeNominal;
  }

  public AggrSpaceInfo withSizeNominal(BigInteger paramBigInteger)
  {
    this.sizeNominal = paramBigInteger;
    return this;
  }

  @Element("size-snap-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeSnapUsed(BigInteger paramBigInteger)
  {
    this.sizeSnapUsed = paramBigInteger;
  }

  public BigInteger getSizeSnapUsed()
  {
    return this.sizeSnapUsed;
  }

  public AggrSpaceInfo withSizeSnapUsed(BigInteger paramBigInteger)
  {
    this.sizeSnapUsed = paramBigInteger;
    return this;
  }

  @Element("size-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
  }

  public BigInteger getSizeUsed()
  {
    return this.sizeUsed;
  }

  public AggrSpaceInfo withSizeUsed(BigInteger paramBigInteger)
  {
    this.sizeUsed = paramBigInteger;
    return this;
  }

  @Element("size-volume-allocated")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeVolumeAllocated(BigInteger paramBigInteger)
  {
    this.sizeVolumeAllocated = paramBigInteger;
  }

  public BigInteger getSizeVolumeAllocated()
  {
    return this.sizeVolumeAllocated;
  }

  public AggrSpaceInfo withSizeVolumeAllocated(BigInteger paramBigInteger)
  {
    this.sizeVolumeAllocated = paramBigInteger;
    return this;
  }

  @Element("size-volume-used")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setSizeVolumeUsed(BigInteger paramBigInteger)
  {
    this.sizeVolumeUsed = paramBigInteger;
  }

  public BigInteger getSizeVolumeUsed()
  {
    return this.sizeVolumeUsed;
  }

  public AggrSpaceInfo withSizeVolumeUsed(BigInteger paramBigInteger)
  {
    this.sizeVolumeUsed = paramBigInteger;
    return this;
  }

  @Element("volume-count")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setVolumeCount(BigInteger paramBigInteger)
  {
    this.volumeCount = paramBigInteger;
  }

  public BigInteger getVolumeCount()
  {
    return this.volumeCount;
  }

  public AggrSpaceInfo withVolumeCount(BigInteger paramBigInteger)
  {
    this.volumeCount = paramBigInteger;
    return this;
  }

  @Element("volumes")
  @Type("volume-space-info[]")
  public void setVolumes(List<VolumeSpaceInfo> paramList)
  {
    this.volumes = paramList;
  }

  public List<VolumeSpaceInfo> getVolumes()
  {
    return this.volumes;
  }

  public AggrSpaceInfo withVolumes(List<VolumeSpaceInfo> paramList)
  {
    this.volumes = paramList;
    return this;
  }

  public AggrSpaceInfo withVolumes(VolumeSpaceInfo[] paramArrayOfVolumeSpaceInfo)
  {
    this.volumes = new ArrayList(Arrays.asList(paramArrayOfVolumeSpaceInfo));
    return this;
  }
}