package com.netapp.filer.cifs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("cifs-homedir-paths-set")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "ESVCNOTAVAIL"})
@Family({"ontap-c", "vfiler"})
public class CifsHomedirPathsSet
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean force;
  public List<String> homedirPaths;
  public List<PathErrorInfo> pathError;

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public CifsHomedirPathsSet withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("homedir-paths")
  @Type("homedir-path-info[]")
  @Optional
  public void setHomedirPaths(List<String> paramList)
  {
    this.homedirPaths = paramList;
  }

  public List<String> getHomedirPaths()
  {
    return this.homedirPaths;
  }

  public CifsHomedirPathsSet withHomedirPaths(List<String> paramList)
  {
    this.homedirPaths = paramList;
    return this;
  }

  public CifsHomedirPathsSet withHomedirPaths(String[] paramArrayOfString)
  {
    this.homedirPaths = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Output("path-error")
  @Type("path-error-info[]")
  @Optional
  public void setPathError(List<PathErrorInfo> paramList)
  {
    this.pathError = paramList;
  }

  public List<PathErrorInfo> getPathError()
  {
    return this.pathError;
  }

  public CifsHomedirPathsSet withPathError(List<PathErrorInfo> paramList)
  {
    this.pathError = paramList;
    return this;
  }

  public CifsHomedirPathsSet withPathError(PathErrorInfo[] paramArrayOfPathErrorInfo)
  {
    this.pathError = new ArrayList(Arrays.asList(paramArrayOfPathErrorInfo));
    return this;
  }
}