package com.netapp.filer.cifs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("cifs-setup")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPILICENSE", "ECIFS_AD_CLOCK_SKEW", "ECIFS_AD_RESET_REQUIRED", "ECIFS_BIND_FAILED", "ECIFS_DC_CONNECT_FAILED", "ECIFS_DNS_REQUIRED", "ECIFS_KRB_CONFLICT", "ECIFS_LIST_UNAVAILABLE", "ECIFS_LOGIN_FAILED", "ECIFS_PASSWD_AND_GROUP_REQUIRED", "EINTERNALERROR", "EINVALIDINPUTERROR", "ESVCDISABLED", "ESVCNOTAVAIL"})
@Family({"ontap-c", "vfiler"})
public class CifsSetup
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String authType;
  public String domainName;
  public String loginPassword;
  public String loginUser;
  public String ouName;
  public String pdcIpAddress;
  public String securityStyle;
  public String serverName;
  public String siteName;

  @Input("auth-type")
  @Type("string")
  public void setAuthType(String paramString)
  {
    this.authType = paramString;
  }

  public String getAuthType()
  {
    return this.authType;
  }

  public CifsSetup withAuthType(String paramString)
  {
    this.authType = paramString;
    return this;
  }

  @Input("domain-name")
  @Type("string")
  @Optional
  public void setDomainName(String paramString)
  {
    this.domainName = paramString;
  }

  public String getDomainName()
  {
    return this.domainName;
  }

  public CifsSetup withDomainName(String paramString)
  {
    this.domainName = paramString;
    return this;
  }

  @Input("login-password")
  @Type("string")
  @Optional
  public void setLoginPassword(String paramString)
  {
    this.loginPassword = paramString;
  }

  public String getLoginPassword()
  {
    return this.loginPassword;
  }

  public CifsSetup withLoginPassword(String paramString)
  {
    this.loginPassword = paramString;
    return this;
  }

  @Input("login-user")
  @Type("string")
  @Optional
  public void setLoginUser(String paramString)
  {
    this.loginUser = paramString;
  }

  public String getLoginUser()
  {
    return this.loginUser;
  }

  public CifsSetup withLoginUser(String paramString)
  {
    this.loginUser = paramString;
    return this;
  }

  @Input("ou-name")
  @Type("string")
  @Optional
  public void setOuName(String paramString)
  {
    this.ouName = paramString;
  }

  public String getOuName()
  {
    return this.ouName;
  }

  public CifsSetup withOuName(String paramString)
  {
    this.ouName = paramString;
    return this;
  }

  @Input("pdc-ip-address")
  @Type("ip-address")
  @Optional
  public void setPdcIpAddress(String paramString)
  {
    this.pdcIpAddress = paramString;
  }

  public String getPdcIpAddress()
  {
    return this.pdcIpAddress;
  }

  public CifsSetup withPdcIpAddress(String paramString)
  {
    this.pdcIpAddress = paramString;
    return this;
  }

  @Input("security-style")
  @Type("string")
  public void setSecurityStyle(String paramString)
  {
    this.securityStyle = paramString;
  }

  public String getSecurityStyle()
  {
    return this.securityStyle;
  }

  public CifsSetup withSecurityStyle(String paramString)
  {
    this.securityStyle = paramString;
    return this;
  }

  @Input("server-name")
  @Type("string")
  public void setServerName(String paramString)
  {
    this.serverName = paramString;
  }

  public String getServerName()
  {
    return this.serverName;
  }

  public CifsSetup withServerName(String paramString)
  {
    this.serverName = paramString;
    return this;
  }

  @Input("site-name")
  @Type("string")
  @Optional
  public void setSiteName(String paramString)
  {
    this.siteName = paramString;
  }

  public String getSiteName()
  {
    return this.siteName;
  }

  public CifsSetup withSiteName(String paramString)
  {
    this.siteName = paramString;
    return this;
  }
}