package com.netapp.filer.cifs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("cifs-share-add")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPILICENSE", "EAPIMISSINGARGUMENT", "ECIFSSHARINGVIOLATION", "EINTERNALERROR", "EONTAPI_EINVAL", "EONTAPI_ENOMEM", "ESHAREACCESSDENIED", "ESHAREEXISTS", "ESHARETOOMANY", "ESVCDISABLED", "ESVCNOTAVAIL", "EVOLUMEINUSE"})
@Family({"ontap-c", "vfiler"})
public class CifsShareAdd
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String caching;
  public String comment;
  public Long dirUmask;
  public Long fileUmask;
  public String forcegroup;
  public Boolean accessBasedEnum;
  public Boolean browse;
  public Boolean symlinkStrictSecurity;
  public Boolean vscan;
  public Boolean vscanread;
  public Boolean widelink;
  public Long maxusers;
  public String path;
  public String shareName;
  public Long umask;

  @Input("caching")
  @Type("string")
  @Optional
  public void setCaching(String paramString)
  {
    this.caching = paramString;
  }

  public String getCaching()
  {
    return this.caching;
  }

  public CifsShareAdd withCaching(String paramString)
  {
    this.caching = paramString;
    return this;
  }

  @Input("comment")
  @Type("string")
  @Optional
  public void setComment(String paramString)
  {
    this.comment = paramString;
  }

  public String getComment()
  {
    return this.comment;
  }

  public CifsShareAdd withComment(String paramString)
  {
    this.comment = paramString;
    return this;
  }

  @Input("dir-umask")
  @Type("integer")
  @Optional
  public void setDirUmask(Long paramLong)
  {
    this.dirUmask = paramLong;
  }

  public Long getDirUmask()
  {
    return this.dirUmask;
  }

  public CifsShareAdd withDirUmask(Long paramLong)
  {
    this.dirUmask = paramLong;
    return this;
  }

  @Input("file-umask")
  @Type("integer")
  @Optional
  public void setFileUmask(Long paramLong)
  {
    this.fileUmask = paramLong;
  }

  public Long getFileUmask()
  {
    return this.fileUmask;
  }

  public CifsShareAdd withFileUmask(Long paramLong)
  {
    this.fileUmask = paramLong;
    return this;
  }

  @Input("forcegroup")
  @Type("string")
  @Optional
  public void setForcegroup(String paramString)
  {
    this.forcegroup = paramString;
  }

  public String getForcegroup()
  {
    return this.forcegroup;
  }

  public CifsShareAdd withForcegroup(String paramString)
  {
    this.forcegroup = paramString;
    return this;
  }

  @Input("is-access-based-enum")
  @Type("boolean")
  @Optional
  public void setAccessBasedEnum(Boolean paramBoolean)
  {
    this.accessBasedEnum = paramBoolean;
  }

  public Boolean isAccessBasedEnum()
  {
    return this.accessBasedEnum;
  }

  public CifsShareAdd withAccessBasedEnum(Boolean paramBoolean)
  {
    this.accessBasedEnum = paramBoolean;
    return this;
  }

  @Input("is-browse")
  @Type("boolean")
  @Optional
  public void setBrowse(Boolean paramBoolean)
  {
    this.browse = paramBoolean;
  }

  public Boolean isBrowse()
  {
    return this.browse;
  }

  public CifsShareAdd withBrowse(Boolean paramBoolean)
  {
    this.browse = paramBoolean;
    return this;
  }

  @Input("is-symlink-strict-security")
  @Type("boolean")
  @Optional
  public void setSymlinkStrictSecurity(Boolean paramBoolean)
  {
    this.symlinkStrictSecurity = paramBoolean;
  }

  public Boolean isSymlinkStrictSecurity()
  {
    return this.symlinkStrictSecurity;
  }

  public CifsShareAdd withSymlinkStrictSecurity(Boolean paramBoolean)
  {
    this.symlinkStrictSecurity = paramBoolean;
    return this;
  }

  @Input("is-vscan")
  @Type("boolean")
  @Optional
  public void setVscan(Boolean paramBoolean)
  {
    this.vscan = paramBoolean;
  }

  public Boolean isVscan()
  {
    return this.vscan;
  }

  public CifsShareAdd withVscan(Boolean paramBoolean)
  {
    this.vscan = paramBoolean;
    return this;
  }

  @Input("is-vscanread")
  @Type("boolean")
  @Optional
  public void setVscanread(Boolean paramBoolean)
  {
    this.vscanread = paramBoolean;
  }

  public Boolean isVscanread()
  {
    return this.vscanread;
  }

  public CifsShareAdd withVscanread(Boolean paramBoolean)
  {
    this.vscanread = paramBoolean;
    return this;
  }

  @Input("is-widelink")
  @Type("boolean")
  @Optional
  public void setWidelink(Boolean paramBoolean)
  {
    this.widelink = paramBoolean;
  }

  public Boolean isWidelink()
  {
    return this.widelink;
  }

  public CifsShareAdd withWidelink(Boolean paramBoolean)
  {
    this.widelink = paramBoolean;
    return this;
  }

  @Input("maxusers")
  @Type("integer")
  @Optional
  public void setMaxusers(Long paramLong)
  {
    this.maxusers = paramLong;
  }

  public Long getMaxusers()
  {
    return this.maxusers;
  }

  public CifsShareAdd withMaxusers(Long paramLong)
  {
    this.maxusers = paramLong;
    return this;
  }

  @Input("path")
  @Type("string")
  public void setPath(String paramString)
  {
    this.path = paramString;
  }

  public String getPath()
  {
    return this.path;
  }

  public CifsShareAdd withPath(String paramString)
  {
    this.path = paramString;
    return this;
  }

  @Input("share-name")
  @Type("string")
  public void setShareName(String paramString)
  {
    this.shareName = paramString;
  }

  public String getShareName()
  {
    return this.shareName;
  }

  public CifsShareAdd withShareName(String paramString)
  {
    this.shareName = paramString;
    return this;
  }

  @Input("umask")
  @Type("integer")
  @Optional
  public void setUmask(Long paramLong)
  {
    this.umask = paramLong;
  }

  public Long getUmask()
  {
    return this.umask;
  }

  public CifsShareAdd withUmask(Long paramLong)
  {
    this.umask = paramLong;
    return this;
  }
}