package com.netapp.filer.dlock;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("d-lock-break")
@Status(StatusEnum.PUBLIC)
@Errno({"EFILENOTFOUND", "EINTERNALERROR", "EINVALIDINPUTERROR", "EVOLUMEDOESNOTEXIST"})
@Family({"ontap-d"})
public class DLockBreak
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String fileName;
  public Boolean hasMoreLocks;
  public String host;
  public String owner;
  public Integer vif;
  public String volumeDsid;
  public Integer vserver;

  @Input("file-name")
  @Type("string")
  @Optional
  public void setFileName(String paramString)
  {
    this.fileName = paramString;
  }

  public String getFileName()
  {
    return this.fileName;
  }

  public DLockBreak withFileName(String paramString)
  {
    this.fileName = paramString;
    return this;
  }

  @Output("has-more-locks")
  @Type("boolean")
  public void setHasMoreLocks(Boolean paramBoolean)
  {
    this.hasMoreLocks = paramBoolean;
  }

  public Boolean isHasMoreLocks()
  {
    return this.hasMoreLocks;
  }

  public DLockBreak withHasMoreLocks(Boolean paramBoolean)
  {
    this.hasMoreLocks = paramBoolean;
    return this;
  }

  @Input("host")
  @Type("string")
  @Optional
  public void setHost(String paramString)
  {
    this.host = paramString;
  }

  public String getHost()
  {
    return this.host;
  }

  public DLockBreak withHost(String paramString)
  {
    this.host = paramString;
    return this;
  }

  @Input("owner")
  @Type("string")
  @Optional
  public void setOwner(String paramString)
  {
    this.owner = paramString;
  }

  public String getOwner()
  {
    return this.owner;
  }

  public DLockBreak withOwner(String paramString)
  {
    this.owner = paramString;
    return this;
  }

  @Input("vif")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setVif(Integer paramInteger)
  {
    this.vif = paramInteger;
  }

  public Integer getVif()
  {
    return this.vif;
  }

  public DLockBreak withVif(Integer paramInteger)
  {
    this.vif = paramInteger;
    return this;
  }

  @Input("volume-dsid")
  @Type("string")
  @Optional
  public void setVolumeDsid(String paramString)
  {
    this.volumeDsid = paramString;
  }

  public String getVolumeDsid()
  {
    return this.volumeDsid;
  }

  public DLockBreak withVolumeDsid(String paramString)
  {
    this.volumeDsid = paramString;
    return this;
  }

  @Input("vserver")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setVserver(Integer paramInteger)
  {
    this.vserver = paramInteger;
  }

  public Integer getVserver()
  {
    return this.vserver;
  }

  public DLockBreak withVserver(Integer paramInteger)
  {
    this.vserver = paramInteger;
    return this;
  }
}