package com.netapp.filer.dlock;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.filer.lock.LockStatusInfo;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("d-lock-status-list-iter")
@Status(StatusEnum.PUBLIC)
@Errno({"EFILENOTFOUND", "EINTERNALERROR", "EINVALIDINPUTERROR", "EONTAPI_ENOMEM", "EVOLUMEDOESNOTEXIST"})
@Family({"ontap-d"})
public class DLockStatusListIter
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String fileName;
  public String host;
  public List<LockStatusInfo> lockStatus;
  public Integer maxLocks;
  public Long nextTag;
  public Integer outputLocks;
  public String owner;
  public Long tag;
  public Integer vif;
  public String volumeDsid;
  public Integer vserver;

  @Input("file-name")
  @Type("string")
  @Optional
  public void setFileName(String paramString)
  {
    this.fileName = paramString;
  }

  public String getFileName()
  {
    return this.fileName;
  }

  public DLockStatusListIter withFileName(String paramString)
  {
    this.fileName = paramString;
    return this;
  }

  @Input("host")
  @Type("string")
  @Optional
  public void setHost(String paramString)
  {
    this.host = paramString;
  }

  public String getHost()
  {
    return this.host;
  }

  public DLockStatusListIter withHost(String paramString)
  {
    this.host = paramString;
    return this;
  }

  @Output("lock-status")
  @Type("lock-status-info[]")
  @Optional
  public void setLockStatus(List<LockStatusInfo> paramList)
  {
    this.lockStatus = paramList;
  }

  public List<LockStatusInfo> getLockStatus()
  {
    return this.lockStatus;
  }

  public DLockStatusListIter withLockStatus(List<LockStatusInfo> paramList)
  {
    this.lockStatus = paramList;
    return this;
  }

  public DLockStatusListIter withLockStatus(LockStatusInfo[] paramArrayOfLockStatusInfo)
  {
    this.lockStatus = new ArrayList(Arrays.asList(paramArrayOfLockStatusInfo));
    return this;
  }

  @Input("max-locks")
  @Type("integer")
  @Range(min="1", max="100")
  public void setMaxLocks(Integer paramInteger)
  {
    this.maxLocks = paramInteger;
  }

  public Integer getMaxLocks()
  {
    return this.maxLocks;
  }

  public DLockStatusListIter withMaxLocks(Integer paramInteger)
  {
    this.maxLocks = paramInteger;
    return this;
  }

  @Output("next-tag")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^63-1")
  public void setNextTag(Long paramLong)
  {
    this.nextTag = paramLong;
  }

  public Long getNextTag()
  {
    return this.nextTag;
  }

  public DLockStatusListIter withNextTag(Long paramLong)
  {
    this.nextTag = paramLong;
    return this;
  }

  @Output("output-locks")
  @Type("integer")
  @Range(min="0", max="100")
  public void setOutputLocks(Integer paramInteger)
  {
    this.outputLocks = paramInteger;
  }

  public Integer getOutputLocks()
  {
    return this.outputLocks;
  }

  public DLockStatusListIter withOutputLocks(Integer paramInteger)
  {
    this.outputLocks = paramInteger;
    return this;
  }

  @Input("owner")
  @Type("string")
  @Optional
  public void setOwner(String paramString)
  {
    this.owner = paramString;
  }

  public String getOwner()
  {
    return this.owner;
  }

  public DLockStatusListIter withOwner(String paramString)
  {
    this.owner = paramString;
    return this;
  }

  @Input("tag")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^63-1")
  public void setTag(Long paramLong)
  {
    this.tag = paramLong;
  }

  public Long getTag()
  {
    return this.tag;
  }

  public DLockStatusListIter withTag(Long paramLong)
  {
    this.tag = paramLong;
    return this;
  }

  @Input("vif")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setVif(Integer paramInteger)
  {
    this.vif = paramInteger;
  }

  public Integer getVif()
  {
    return this.vif;
  }

  public DLockStatusListIter withVif(Integer paramInteger)
  {
    this.vif = paramInteger;
    return this;
  }

  @Input("volume-dsid")
  @Type("string")
  @Optional
  public void setVolumeDsid(String paramString)
  {
    this.volumeDsid = paramString;
  }

  public String getVolumeDsid()
  {
    return this.volumeDsid;
  }

  public DLockStatusListIter withVolumeDsid(String paramString)
  {
    this.volumeDsid = paramString;
    return this;
  }

  @Input("vserver")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setVserver(Integer paramInteger)
  {
    this.vserver = paramInteger;
  }

  public Integer getVserver()
  {
    return this.vserver;
  }

  public DLockStatusListIter withVserver(Integer paramInteger)
  {
    this.vserver = paramInteger;
    return this;
  }
}