package com.netapp.filer.file;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("file-inode-info")
@Status(StatusEnum.PUBLIC)
@Errno({"EI2PNOTENABLED", "EINELIGIBLEINODE", "EINODENUMBERTOOLARGE", "EINODENUMBERTOOSMALL", "EINTERNALERROR", "EINVALIDINODE", "EINVALIDINPUTERROR", "EPARENTINFONOTLOADED", "ESNAPSHOTDOESNOTEXIST", "EVOLUMEDOESNOTEXIST", "EVOLUMEOFFLINE"})
@Family({"ontap-c", "ontap-d", "vfiler"})
public class FileInodeInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long inodeNumberOutput;
  public Long inodeNumberInput;
  public List<InodeParentInfo> inodePaths;
  public Long numberOfParents;
  public Boolean reportLeafName;
  public Boolean reportNoPathname;
  public Boolean reportOtherParents;
  public Boolean reportParentData;
  public Long snapId;
  public String snapName;
  public Long snapshotId;
  public String snapshotName;
  public String volumeDsidOutput;
  public String volumeDsidInput;
  public Long volumeFsidOutput;
  public Long volumeFsidInput;
  public String volumeNameOutput;
  public String volumeNameInput;
  public String volumeUuidOutput;
  public String volumeUuidInput;

  @Output("inode-number")
  @Type("integer")
  public void setInodeNumberOutput(Long paramLong)
  {
    this.inodeNumberOutput = paramLong;
  }

  public Long getInodeNumberOutput()
  {
    return this.inodeNumberOutput;
  }

  public FileInodeInfo withInodeNumberOutput(Long paramLong)
  {
    this.inodeNumberOutput = paramLong;
    return this;
  }

  @Input("inode-number")
  @Type("integer")
  public void setInodeNumberInput(Long paramLong)
  {
    this.inodeNumberInput = paramLong;
  }

  public Long getInodeNumberInput()
  {
    return this.inodeNumberInput;
  }

  public FileInodeInfo withInodeNumberInput(Long paramLong)
  {
    this.inodeNumberInput = paramLong;
    return this;
  }

  @Output("inode-paths")
  @Type("inode-parent-info[]")
  public void setInodePaths(List<InodeParentInfo> paramList)
  {
    this.inodePaths = paramList;
  }

  public List<InodeParentInfo> getInodePaths()
  {
    return this.inodePaths;
  }

  public FileInodeInfo withInodePaths(List<InodeParentInfo> paramList)
  {
    this.inodePaths = paramList;
    return this;
  }

  public FileInodeInfo withInodePaths(InodeParentInfo[] paramArrayOfInodeParentInfo)
  {
    this.inodePaths = new ArrayList(Arrays.asList(paramArrayOfInodeParentInfo));
    return this;
  }

  @Output("number-of-parents")
  @Type("integer")
  public void setNumberOfParents(Long paramLong)
  {
    this.numberOfParents = paramLong;
  }

  public Long getNumberOfParents()
  {
    return this.numberOfParents;
  }

  public FileInodeInfo withNumberOfParents(Long paramLong)
  {
    this.numberOfParents = paramLong;
    return this;
  }

  @Input("report-leaf-name")
  @Type("boolean")
  @Optional
  public void setReportLeafName(Boolean paramBoolean)
  {
    this.reportLeafName = paramBoolean;
  }

  public Boolean isReportLeafName()
  {
    return this.reportLeafName;
  }

  public FileInodeInfo withReportLeafName(Boolean paramBoolean)
  {
    this.reportLeafName = paramBoolean;
    return this;
  }

  @Input("report-no-pathname")
  @Type("boolean")
  @Optional
  public void setReportNoPathname(Boolean paramBoolean)
  {
    this.reportNoPathname = paramBoolean;
  }

  public Boolean isReportNoPathname()
  {
    return this.reportNoPathname;
  }

  public FileInodeInfo withReportNoPathname(Boolean paramBoolean)
  {
    this.reportNoPathname = paramBoolean;
    return this;
  }

  @Input("report-other-parents")
  @Type("boolean")
  @Optional
  public void setReportOtherParents(Boolean paramBoolean)
  {
    this.reportOtherParents = paramBoolean;
  }

  public Boolean isReportOtherParents()
  {
    return this.reportOtherParents;
  }

  public FileInodeInfo withReportOtherParents(Boolean paramBoolean)
  {
    this.reportOtherParents = paramBoolean;
    return this;
  }

  @Input("report-parent-data")
  @Type("boolean")
  @Optional
  public void setReportParentData(Boolean paramBoolean)
  {
    this.reportParentData = paramBoolean;
  }

  public Boolean isReportParentData()
  {
    return this.reportParentData;
  }

  public FileInodeInfo withReportParentData(Boolean paramBoolean)
  {
    this.reportParentData = paramBoolean;
    return this;
  }

  @Input("snap-id")
  @Type("integer")
  @Optional
  public void setSnapId(Long paramLong)
  {
    this.snapId = paramLong;
  }

  public Long getSnapId()
  {
    return this.snapId;
  }

  public FileInodeInfo withSnapId(Long paramLong)
  {
    this.snapId = paramLong;
    return this;
  }

  @Input("snap-name")
  @Type("string")
  @Optional
  public void setSnapName(String paramString)
  {
    this.snapName = paramString;
  }

  public String getSnapName()
  {
    return this.snapName;
  }

  public FileInodeInfo withSnapName(String paramString)
  {
    this.snapName = paramString;
    return this;
  }

  @Output("snapshot-id")
  @Type("integer")
  @Optional
  public void setSnapshotId(Long paramLong)
  {
    this.snapshotId = paramLong;
  }

  public Long getSnapshotId()
  {
    return this.snapshotId;
  }

  public FileInodeInfo withSnapshotId(Long paramLong)
  {
    this.snapshotId = paramLong;
    return this;
  }

  @Output("snapshot-name")
  @Type("string")
  @Optional
  public void setSnapshotName(String paramString)
  {
    this.snapshotName = paramString;
  }

  public String getSnapshotName()
  {
    return this.snapshotName;
  }

  public FileInodeInfo withSnapshotName(String paramString)
  {
    this.snapshotName = paramString;
    return this;
  }

  @Output("volume-dsid")
  @Type("string")
  @Optional
  public void setVolumeDsidOutput(String paramString)
  {
    this.volumeDsidOutput = paramString;
  }

  public String getVolumeDsidOutput()
  {
    return this.volumeDsidOutput;
  }

  public FileInodeInfo withVolumeDsidOutput(String paramString)
  {
    this.volumeDsidOutput = paramString;
    return this;
  }

  @Input("volume-dsid")
  @Type("string")
  @Optional
  public void setVolumeDsidInput(String paramString)
  {
    this.volumeDsidInput = paramString;
  }

  public String getVolumeDsidInput()
  {
    return this.volumeDsidInput;
  }

  public FileInodeInfo withVolumeDsidInput(String paramString)
  {
    this.volumeDsidInput = paramString;
    return this;
  }

  @Output("volume-fsid")
  @Type("integer")
  public void setVolumeFsidOutput(Long paramLong)
  {
    this.volumeFsidOutput = paramLong;
  }

  public Long getVolumeFsidOutput()
  {
    return this.volumeFsidOutput;
  }

  public FileInodeInfo withVolumeFsidOutput(Long paramLong)
  {
    this.volumeFsidOutput = paramLong;
    return this;
  }

  @Input("volume-fsid")
  @Type("integer")
  @Optional
  public void setVolumeFsidInput(Long paramLong)
  {
    this.volumeFsidInput = paramLong;
  }

  public Long getVolumeFsidInput()
  {
    return this.volumeFsidInput;
  }

  public FileInodeInfo withVolumeFsidInput(Long paramLong)
  {
    this.volumeFsidInput = paramLong;
    return this;
  }

  @Output("volume-name")
  @Type("string")
  public void setVolumeNameOutput(String paramString)
  {
    this.volumeNameOutput = paramString;
  }

  public String getVolumeNameOutput()
  {
    return this.volumeNameOutput;
  }

  public FileInodeInfo withVolumeNameOutput(String paramString)
  {
    this.volumeNameOutput = paramString;
    return this;
  }

  @Input("volume-name")
  @Type("string")
  @Optional
  public void setVolumeNameInput(String paramString)
  {
    this.volumeNameInput = paramString;
  }

  public String getVolumeNameInput()
  {
    return this.volumeNameInput;
  }

  public FileInodeInfo withVolumeNameInput(String paramString)
  {
    this.volumeNameInput = paramString;
    return this;
  }

  @Output("volume-uuid")
  @Type("string")
  public void setVolumeUuidOutput(String paramString)
  {
    this.volumeUuidOutput = paramString;
  }

  public String getVolumeUuidOutput()
  {
    return this.volumeUuidOutput;
  }

  public FileInodeInfo withVolumeUuidOutput(String paramString)
  {
    this.volumeUuidOutput = paramString;
    return this;
  }

  @Input("volume-uuid")
  @Type("string")
  @Optional
  public void setVolumeUuidInput(String paramString)
  {
    this.volumeUuidInput = paramString;
  }

  public String getVolumeUuidInput()
  {
    return this.volumeUuidInput;
  }

  public FileInodeInfo withVolumeUuidInput(String paramString)
  {
    this.volumeUuidInput = paramString;
    return this;
  }
}