package com.netapp.filer.file;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Typedef("fingerprint-info")
public class FingerprintInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String aggregateUuid;
  public String filerId;
  public String filerName;
  public String fingerprintAlgorithm;
  public Date fingerprintEndTime;
  public String fingerprintInputPath;
  public String fingerprintScope;
  public Date fingerprintStartTime;
  public String fingerprintVersion;
  public String formattedFingerprintEndTime;
  public String formattedFingerprintStartTime;
  public String formattedSnaplockComplianceClock;
  public String formattedVolumeExpiryDate;
  public Boolean volumeExpiryDateWraparound;
  public List<FileFingerprintInfo> metadataFiles;
  public BigInteger snaplockComplianceClock;
  public String snaplockLicense;
  public String volumeContainingAggregate;
  public BigInteger volumeExpiryDate;
  public String volumeName;
  public String volumeSnaplockType;
  public String volumeType;
  public String volumeUuid;

  @Element("aggregate-uuid")
  @Type("string")
  @Optional
  public void setAggregateUuid(String paramString)
  {
    this.aggregateUuid = paramString;
  }

  public String getAggregateUuid()
  {
    return this.aggregateUuid;
  }

  public FingerprintInfo withAggregateUuid(String paramString)
  {
    this.aggregateUuid = paramString;
    return this;
  }

  @Element("filer-id")
  @Type("string")
  public void setFilerId(String paramString)
  {
    this.filerId = paramString;
  }

  public String getFilerId()
  {
    return this.filerId;
  }

  public FingerprintInfo withFilerId(String paramString)
  {
    this.filerId = paramString;
    return this;
  }

  @Element("filer-name")
  @Type("string")
  @Optional
  public void setFilerName(String paramString)
  {
    this.filerName = paramString;
  }

  public String getFilerName()
  {
    return this.filerName;
  }

  public FingerprintInfo withFilerName(String paramString)
  {
    this.filerName = paramString;
    return this;
  }

  @Element("fingerprint-algorithm")
  @Type("digest-algorithm")
  public void setFingerprintAlgorithm(String paramString)
  {
    this.fingerprintAlgorithm = paramString;
  }

  public String getFingerprintAlgorithm()
  {
    return this.fingerprintAlgorithm;
  }

  public FingerprintInfo withFingerprintAlgorithm(String paramString)
  {
    this.fingerprintAlgorithm = paramString;
    return this;
  }

  @Element("fingerprint-end-time")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFingerprintEndTime(Date paramDate)
  {
    this.fingerprintEndTime = paramDate;
  }

  public Date getFingerprintEndTime()
  {
    return this.fingerprintEndTime;
  }

  public FingerprintInfo withFingerprintEndTime(Date paramDate)
  {
    this.fingerprintEndTime = paramDate;
    return this;
  }

  @Element("fingerprint-input-path")
  @Type("string")
  public void setFingerprintInputPath(String paramString)
  {
    this.fingerprintInputPath = paramString;
  }

  public String getFingerprintInputPath()
  {
    return this.fingerprintInputPath;
  }

  public FingerprintInfo withFingerprintInputPath(String paramString)
  {
    this.fingerprintInputPath = paramString;
    return this;
  }

  @Element("fingerprint-scope")
  @Type("file-scope")
  public void setFingerprintScope(String paramString)
  {
    this.fingerprintScope = paramString;
  }

  public String getFingerprintScope()
  {
    return this.fingerprintScope;
  }

  public FingerprintInfo withFingerprintScope(String paramString)
  {
    this.fingerprintScope = paramString;
    return this;
  }

  @Element("fingerprint-start-time")
  @Type("integer")
  @Range(min="0", max="2^64-1")
  public void setFingerprintStartTime(Date paramDate)
  {
    this.fingerprintStartTime = paramDate;
  }

  public Date getFingerprintStartTime()
  {
    return this.fingerprintStartTime;
  }

  public FingerprintInfo withFingerprintStartTime(Date paramDate)
  {
    this.fingerprintStartTime = paramDate;
    return this;
  }

  @Element("fingerprint-version")
  @Type("string")
  public void setFingerprintVersion(String paramString)
  {
    this.fingerprintVersion = paramString;
  }

  public String getFingerprintVersion()
  {
    return this.fingerprintVersion;
  }

  public FingerprintInfo withFingerprintVersion(String paramString)
  {
    this.fingerprintVersion = paramString;
    return this;
  }

  @Element("formatted-fingerprint-end-time")
  @Type("string")
  public void setFormattedFingerprintEndTime(String paramString)
  {
    this.formattedFingerprintEndTime = paramString;
  }

  public String getFormattedFingerprintEndTime()
  {
    return this.formattedFingerprintEndTime;
  }

  public FingerprintInfo withFormattedFingerprintEndTime(String paramString)
  {
    this.formattedFingerprintEndTime = paramString;
    return this;
  }

  @Element("formatted-fingerprint-start-time")
  @Type("string")
  public void setFormattedFingerprintStartTime(String paramString)
  {
    this.formattedFingerprintStartTime = paramString;
  }

  public String getFormattedFingerprintStartTime()
  {
    return this.formattedFingerprintStartTime;
  }

  public FingerprintInfo withFormattedFingerprintStartTime(String paramString)
  {
    this.formattedFingerprintStartTime = paramString;
    return this;
  }

  @Element("formatted-snaplock-compliance-clock")
  @Type("string")
  public void setFormattedSnaplockComplianceClock(String paramString)
  {
    this.formattedSnaplockComplianceClock = paramString;
  }

  public String getFormattedSnaplockComplianceClock()
  {
    return this.formattedSnaplockComplianceClock;
  }

  public FingerprintInfo withFormattedSnaplockComplianceClock(String paramString)
  {
    this.formattedSnaplockComplianceClock = paramString;
    return this;
  }

  @Element("formatted-volume-expiry-date")
  @Type("string")
  @Optional
  public void setFormattedVolumeExpiryDate(String paramString)
  {
    this.formattedVolumeExpiryDate = paramString;
  }

  public String getFormattedVolumeExpiryDate()
  {
    return this.formattedVolumeExpiryDate;
  }

  public FingerprintInfo withFormattedVolumeExpiryDate(String paramString)
  {
    this.formattedVolumeExpiryDate = paramString;
    return this;
  }

  @Element("is-volume-expiry-date-wraparound")
  @Type("boolean")
  @Optional
  public void setVolumeExpiryDateWraparound(Boolean paramBoolean)
  {
    this.volumeExpiryDateWraparound = paramBoolean;
  }

  public Boolean isVolumeExpiryDateWraparound()
  {
    return this.volumeExpiryDateWraparound;
  }

  public FingerprintInfo withVolumeExpiryDateWraparound(Boolean paramBoolean)
  {
    this.volumeExpiryDateWraparound = paramBoolean;
    return this;
  }

  @Element("metadata-files")
  @Type("file-fingerprint-info[]")
  public void setMetadataFiles(List<FileFingerprintInfo> paramList)
  {
    this.metadataFiles = paramList;
  }

  public List<FileFingerprintInfo> getMetadataFiles()
  {
    return this.metadataFiles;
  }

  public FingerprintInfo withMetadataFiles(List<FileFingerprintInfo> paramList)
  {
    this.metadataFiles = paramList;
    return this;
  }

  public FingerprintInfo withMetadataFiles(FileFingerprintInfo[] paramArrayOfFileFingerprintInfo)
  {
    this.metadataFiles = new ArrayList(Arrays.asList(paramArrayOfFileFingerprintInfo));
    return this;
  }

  @Element("snaplock-compliance-clock")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setSnaplockComplianceClock(BigInteger paramBigInteger)
  {
    this.snaplockComplianceClock = paramBigInteger;
  }

  public BigInteger getSnaplockComplianceClock()
  {
    return this.snaplockComplianceClock;
  }

  public FingerprintInfo withSnaplockComplianceClock(BigInteger paramBigInteger)
  {
    this.snaplockComplianceClock = paramBigInteger;
    return this;
  }

  @Element("snaplock-license")
  @Type("string")
  @Optional
  public void setSnaplockLicense(String paramString)
  {
    this.snaplockLicense = paramString;
  }

  public String getSnaplockLicense()
  {
    return this.snaplockLicense;
  }

  public FingerprintInfo withSnaplockLicense(String paramString)
  {
    this.snaplockLicense = paramString;
    return this;
  }

  @Element("volume-containing-aggregate")
  @Type("string")
  @Optional
  public void setVolumeContainingAggregate(String paramString)
  {
    this.volumeContainingAggregate = paramString;
  }

  public String getVolumeContainingAggregate()
  {
    return this.volumeContainingAggregate;
  }

  public FingerprintInfo withVolumeContainingAggregate(String paramString)
  {
    this.volumeContainingAggregate = paramString;
    return this;
  }

  @Element("volume-expiry-date")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setVolumeExpiryDate(BigInteger paramBigInteger)
  {
    this.volumeExpiryDate = paramBigInteger;
  }

  public BigInteger getVolumeExpiryDate()
  {
    return this.volumeExpiryDate;
  }

  public FingerprintInfo withVolumeExpiryDate(BigInteger paramBigInteger)
  {
    this.volumeExpiryDate = paramBigInteger;
    return this;
  }

  @Element("volume-name")
  @Type("string")
  public void setVolumeName(String paramString)
  {
    this.volumeName = paramString;
  }

  public String getVolumeName()
  {
    return this.volumeName;
  }

  public FingerprintInfo withVolumeName(String paramString)
  {
    this.volumeName = paramString;
    return this;
  }

  @Element("volume-snaplock-type")
  @Type("string")
  @Optional
  public void setVolumeSnaplockType(String paramString)
  {
    this.volumeSnaplockType = paramString;
  }

  public String getVolumeSnaplockType()
  {
    return this.volumeSnaplockType;
  }

  public FingerprintInfo withVolumeSnaplockType(String paramString)
  {
    this.volumeSnaplockType = paramString;
    return this;
  }

  @Element("volume-type")
  @Type("string")
  public void setVolumeType(String paramString)
  {
    this.volumeType = paramString;
  }

  public String getVolumeType()
  {
    return this.volumeType;
  }

  public FingerprintInfo withVolumeType(String paramString)
  {
    this.volumeType = paramString;
    return this;
  }

  @Element("volume-uuid")
  @Type("string")
  public void setVolumeUuid(String paramString)
  {
    this.volumeUuid = paramString;
  }

  public String getVolumeUuid()
  {
    return this.volumeUuid;
  }

  public FingerprintInfo withVolumeUuid(String paramString)
  {
    this.volumeUuid = paramString;
    return this;
  }
}