package com.netapp.filer.fpolicy;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Typedef("policy-info")
public class PolicyInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean adsMonitored;
  public Boolean enabled;
  public Boolean i2pEnabled;
  public Boolean offlineFilesOnly;
  public List<MonitoredOperationInfo> monitoredOperations;
  public List<MonitoredProtocolInfo> monitoredProtocols;
  public String name;
  public Long numberOfRequestsBlockedLocally;
  public Long numberOfScreenFailures;
  public Long numberOfScreenedFiles;

  @Element("is-ads-monitored")
  @Type("boolean")
  public void setAdsMonitored(Boolean paramBoolean)
  {
    this.adsMonitored = paramBoolean;
  }

  public Boolean isAdsMonitored()
  {
    return this.adsMonitored;
  }

  public PolicyInfo withAdsMonitored(Boolean paramBoolean)
  {
    this.adsMonitored = paramBoolean;
    return this;
  }

  @Element("is-enabled")
  @Type("boolean")
  public void setEnabled(Boolean paramBoolean)
  {
    this.enabled = paramBoolean;
  }

  public Boolean isEnabled()
  {
    return this.enabled;
  }

  public PolicyInfo withEnabled(Boolean paramBoolean)
  {
    this.enabled = paramBoolean;
    return this;
  }

  @Element("is-i2p-enabled")
  @Type("boolean")
  public void setI2pEnabled(Boolean paramBoolean)
  {
    this.i2pEnabled = paramBoolean;
  }

  public Boolean isI2pEnabled()
  {
    return this.i2pEnabled;
  }

  public PolicyInfo withI2pEnabled(Boolean paramBoolean)
  {
    this.i2pEnabled = paramBoolean;
    return this;
  }

  @Element("is-offline-files-only")
  @Type("boolean")
  @Optional
  public void setOfflineFilesOnly(Boolean paramBoolean)
  {
    this.offlineFilesOnly = paramBoolean;
  }

  public Boolean isOfflineFilesOnly()
  {
    return this.offlineFilesOnly;
  }

  public PolicyInfo withOfflineFilesOnly(Boolean paramBoolean)
  {
    this.offlineFilesOnly = paramBoolean;
    return this;
  }

  @Element("monitored-operations")
  @Type("monitored-operation-info[]")
  public void setMonitoredOperations(List<MonitoredOperationInfo> paramList)
  {
    this.monitoredOperations = paramList;
  }

  public List<MonitoredOperationInfo> getMonitoredOperations()
  {
    return this.monitoredOperations;
  }

  public PolicyInfo withMonitoredOperations(List<MonitoredOperationInfo> paramList)
  {
    this.monitoredOperations = paramList;
    return this;
  }

  public PolicyInfo withMonitoredOperations(MonitoredOperationInfo[] paramArrayOfMonitoredOperationInfo)
  {
    this.monitoredOperations = new ArrayList(Arrays.asList(paramArrayOfMonitoredOperationInfo));
    return this;
  }

  @Element("monitored-protocols")
  @Type("monitored-protocol-info[]")
  public void setMonitoredProtocols(List<MonitoredProtocolInfo> paramList)
  {
    this.monitoredProtocols = paramList;
  }

  public List<MonitoredProtocolInfo> getMonitoredProtocols()
  {
    return this.monitoredProtocols;
  }

  public PolicyInfo withMonitoredProtocols(List<MonitoredProtocolInfo> paramList)
  {
    this.monitoredProtocols = paramList;
    return this;
  }

  public PolicyInfo withMonitoredProtocols(MonitoredProtocolInfo[] paramArrayOfMonitoredProtocolInfo)
  {
    this.monitoredProtocols = new ArrayList(Arrays.asList(paramArrayOfMonitoredProtocolInfo));
    return this;
  }

  @Element("name")
  @Type("string")
  public void setName(String paramString)
  {
    this.name = paramString;
  }

  public String getName()
  {
    return this.name;
  }

  public PolicyInfo withName(String paramString)
  {
    this.name = paramString;
    return this;
  }

  @Element("number-of-requests-blocked-locally")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setNumberOfRequestsBlockedLocally(Long paramLong)
  {
    this.numberOfRequestsBlockedLocally = paramLong;
  }

  public Long getNumberOfRequestsBlockedLocally()
  {
    return this.numberOfRequestsBlockedLocally;
  }

  public PolicyInfo withNumberOfRequestsBlockedLocally(Long paramLong)
  {
    this.numberOfRequestsBlockedLocally = paramLong;
    return this;
  }

  @Element("number-of-screen-failures")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setNumberOfScreenFailures(Long paramLong)
  {
    this.numberOfScreenFailures = paramLong;
  }

  public Long getNumberOfScreenFailures()
  {
    return this.numberOfScreenFailures;
  }

  public PolicyInfo withNumberOfScreenFailures(Long paramLong)
  {
    this.numberOfScreenFailures = paramLong;
    return this;
  }

  @Element("number-of-screened-files")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setNumberOfScreenedFiles(Long paramLong)
  {
    this.numberOfScreenedFiles = paramLong;
  }

  public Long getNumberOfScreenedFiles()
  {
    return this.numberOfScreenedFiles;
  }

  public PolicyInfo withNumberOfScreenedFiles(Long paramLong)
  {
    this.numberOfScreenedFiles = paramLong;
    return this;
  }
}