package com.netapp.filer.lun;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("lun-get-geometry")
@Status(StatusEnum.PUBLIC)
@Errno({"EINVALIDINPUTERROR", "EVDISK_ERROR_NO_SUCH_VDISK"})
@Family({"ontap-c", "vfiler"})
public class LunGetGeometry
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Integer bytesPerSector;
  public Integer cylinders;
  public Long maxResizeSize;
  public String path;
  public Integer sectorsPerTrack;
  public Long size;
  public Integer tracksPerCylinder;

  @Output("bytes-per-sector")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setBytesPerSector(Integer paramInteger)
  {
    this.bytesPerSector = paramInteger;
  }

  public Integer getBytesPerSector()
  {
    return this.bytesPerSector;
  }

  public LunGetGeometry withBytesPerSector(Integer paramInteger)
  {
    this.bytesPerSector = paramInteger;
    return this;
  }

  @Output("cylinders")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setCylinders(Integer paramInteger)
  {
    this.cylinders = paramInteger;
  }

  public Integer getCylinders()
  {
    return this.cylinders;
  }

  public LunGetGeometry withCylinders(Integer paramInteger)
  {
    this.cylinders = paramInteger;
    return this;
  }

  @Output("max-resize-size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setMaxResizeSize(Long paramLong)
  {
    this.maxResizeSize = paramLong;
  }

  public Long getMaxResizeSize()
  {
    return this.maxResizeSize;
  }

  public LunGetGeometry withMaxResizeSize(Long paramLong)
  {
    this.maxResizeSize = paramLong;
    return this;
  }

  @Input("path")
  @Type("string")
  public void setPath(String paramString)
  {
    this.path = paramString;
  }

  public String getPath()
  {
    return this.path;
  }

  public LunGetGeometry withPath(String paramString)
  {
    this.path = paramString;
    return this;
  }

  @Output("sectors-per-track")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setSectorsPerTrack(Integer paramInteger)
  {
    this.sectorsPerTrack = paramInteger;
  }

  public Integer getSectorsPerTrack()
  {
    return this.sectorsPerTrack;
  }

  public LunGetGeometry withSectorsPerTrack(Integer paramInteger)
  {
    this.sectorsPerTrack = paramInteger;
    return this;
  }

  @Output("size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setSize(Long paramLong)
  {
    this.size = paramLong;
  }

  public Long getSize()
  {
    return this.size;
  }

  public LunGetGeometry withSize(Long paramLong)
  {
    this.size = paramLong;
    return this;
  }

  @Output("tracks-per-cylinder")
  @Type("integer")
  @Range(min="0", max="2^31-1")
  public void setTracksPerCylinder(Integer paramInteger)
  {
    this.tracksPerCylinder = paramInteger;
  }

  public Integer getTracksPerCylinder()
  {
    return this.tracksPerCylinder;
  }

  public LunGetGeometry withTracksPerCylinder(Integer paramInteger)
  {
    this.tracksPerCylinder = paramInteger;
    return this;
  }
}