package com.netapp.filer.net;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.Date;

@Api("net-ping-info")
@Status(StatusEnum.PUBLIC)
@Errno({"EHOSTNOTFOUND"})
@Family({"ontap-c", "vfiler"})
public class NetPingInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String hostNameOrIpAddress;
  public Integer packetsReceived;
  public Integer packetsTransmitted;
  public Integer pingCount;
  public Date roundTripMaximumTime;
  public Date roundTripMeanTime;
  public Date roundTripMinimumTime;

  @Input("host-name-or-ip-address")
  @Type("string")
  public void setHostNameOrIpAddress(String paramString)
  {
    this.hostNameOrIpAddress = paramString;
  }

  public String getHostNameOrIpAddress()
  {
    return this.hostNameOrIpAddress;
  }

  public NetPingInfo withHostNameOrIpAddress(String paramString)
  {
    this.hostNameOrIpAddress = paramString;
    return this;
  }

  @Output("packets-received")
  @Type("integer")
  @Optional
  @Range(min="0", max="16")
  public void setPacketsReceived(Integer paramInteger)
  {
    this.packetsReceived = paramInteger;
  }

  public Integer getPacketsReceived()
  {
    return this.packetsReceived;
  }

  public NetPingInfo withPacketsReceived(Integer paramInteger)
  {
    this.packetsReceived = paramInteger;
    return this;
  }

  @Output("packets-transmitted")
  @Type("integer")
  @Optional
  @Range(min="1", max="16")
  public void setPacketsTransmitted(Integer paramInteger)
  {
    this.packetsTransmitted = paramInteger;
  }

  public Integer getPacketsTransmitted()
  {
    return this.packetsTransmitted;
  }

  public NetPingInfo withPacketsTransmitted(Integer paramInteger)
  {
    this.packetsTransmitted = paramInteger;
    return this;
  }

  @Input("ping-count")
  @Type("integer")
  @Optional
  @Range(min="1", max="16")
  public void setPingCount(Integer paramInteger)
  {
    this.pingCount = paramInteger;
  }

  public Integer getPingCount()
  {
    return this.pingCount;
  }

  public NetPingInfo withPingCount(Integer paramInteger)
  {
    this.pingCount = paramInteger;
    return this;
  }

  @Output("round-trip-maximum-time")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setRoundTripMaximumTime(Date paramDate)
  {
    this.roundTripMaximumTime = paramDate;
  }

  public Date getRoundTripMaximumTime()
  {
    return this.roundTripMaximumTime;
  }

  public NetPingInfo withRoundTripMaximumTime(Date paramDate)
  {
    this.roundTripMaximumTime = paramDate;
    return this;
  }

  @Output("round-trip-mean-time")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setRoundTripMeanTime(Date paramDate)
  {
    this.roundTripMeanTime = paramDate;
  }

  public Date getRoundTripMeanTime()
  {
    return this.roundTripMeanTime;
  }

  public NetPingInfo withRoundTripMeanTime(Date paramDate)
  {
    this.roundTripMeanTime = paramDate;
    return this;
  }

  @Output("round-trip-minimum-time")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setRoundTripMinimumTime(Date paramDate)
  {
    this.roundTripMinimumTime = paramDate;
  }

  public Date getRoundTripMinimumTime()
  {
    return this.roundTripMinimumTime;
  }

  public NetPingInfo withRoundTripMinimumTime(Date paramDate)
  {
    this.roundTripMinimumTime = paramDate;
    return this;
  }
}