package com.netapp.filer.nfs;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("nfs-exportfs-delete-rules")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR"})
@Family({"ontap-c", "vfiler"})
public class NfsExportfsDeleteRules
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean allPathnames;
  public List<PathnameInfo> deletedPathnames;
  public List<PathnameInfo> pathnames;
  public Boolean persistent;
  public List<PathnameInfo> unexportedPathnames;
  public Boolean verbose;

  @Input("all-pathnames")
  @Type("boolean")
  @Optional
  public void setAllPathnames(Boolean paramBoolean)
  {
    this.allPathnames = paramBoolean;
  }

  public Boolean isAllPathnames()
  {
    return this.allPathnames;
  }

  public NfsExportfsDeleteRules withAllPathnames(Boolean paramBoolean)
  {
    this.allPathnames = paramBoolean;
    return this;
  }

  @Output("deleted-pathnames")
  @Type("pathname-info[]")
  @Optional
  public void setDeletedPathnames(List<PathnameInfo> paramList)
  {
    this.deletedPathnames = paramList;
  }

  public List<PathnameInfo> getDeletedPathnames()
  {
    return this.deletedPathnames;
  }

  public NfsExportfsDeleteRules withDeletedPathnames(List<PathnameInfo> paramList)
  {
    this.deletedPathnames = paramList;
    return this;
  }

  public NfsExportfsDeleteRules withDeletedPathnames(PathnameInfo[] paramArrayOfPathnameInfo)
  {
    this.deletedPathnames = new ArrayList(Arrays.asList(paramArrayOfPathnameInfo));
    return this;
  }

  @Input("pathnames")
  @Type("pathname-info[]")
  @Optional
  public void setPathnames(List<PathnameInfo> paramList)
  {
    this.pathnames = paramList;
  }

  public List<PathnameInfo> getPathnames()
  {
    return this.pathnames;
  }

  public NfsExportfsDeleteRules withPathnames(List<PathnameInfo> paramList)
  {
    this.pathnames = paramList;
    return this;
  }

  public NfsExportfsDeleteRules withPathnames(PathnameInfo[] paramArrayOfPathnameInfo)
  {
    this.pathnames = new ArrayList(Arrays.asList(paramArrayOfPathnameInfo));
    return this;
  }

  @Input("persistent")
  @Type("boolean")
  @Optional
  public void setPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
  }

  public Boolean isPersistent()
  {
    return this.persistent;
  }

  public NfsExportfsDeleteRules withPersistent(Boolean paramBoolean)
  {
    this.persistent = paramBoolean;
    return this;
  }

  @Output("unexported-pathnames")
  @Type("pathname-info[]")
  @Optional
  public void setUnexportedPathnames(List<PathnameInfo> paramList)
  {
    this.unexportedPathnames = paramList;
  }

  public List<PathnameInfo> getUnexportedPathnames()
  {
    return this.unexportedPathnames;
  }

  public NfsExportfsDeleteRules withUnexportedPathnames(List<PathnameInfo> paramList)
  {
    this.unexportedPathnames = paramList;
    return this;
  }

  public NfsExportfsDeleteRules withUnexportedPathnames(PathnameInfo[] paramArrayOfPathnameInfo)
  {
    this.unexportedPathnames = new ArrayList(Arrays.asList(paramArrayOfPathnameInfo));
    return this;
  }

  @Input("verbose")
  @Type("boolean")
  @Optional
  public void setVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
  }

  public Boolean isVerbose()
  {
    return this.verbose;
  }

  public NfsExportfsDeleteRules withVerbose(Boolean paramBoolean)
  {
    this.verbose = paramBoolean;
    return this;
  }
}