package com.netapp.filer.registry;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("registry-list-info")
@Status(StatusEnum.UNSUPPORTED)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "EONTAPI_ENOREGKEY"})
@Family({"ontap-c", "ontap-d", "vfiler"})
public class RegistryListInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean shallow;
  public String registryKey;
  public List<RegistryPair> registryPairs;

  @Input("is-shallow")
  @Type("boolean")
  @Optional
  public void setShallow(Boolean paramBoolean)
  {
    this.shallow = paramBoolean;
  }

  public Boolean isShallow()
  {
    return this.shallow;
  }

  public RegistryListInfo withShallow(Boolean paramBoolean)
  {
    this.shallow = paramBoolean;
    return this;
  }

  @Input("registry-key")
  @Type("registry-key-string")
  @Optional
  public void setRegistryKey(String paramString)
  {
    this.registryKey = paramString;
  }

  public String getRegistryKey()
  {
    return this.registryKey;
  }

  public RegistryListInfo withRegistryKey(String paramString)
  {
    this.registryKey = paramString;
    return this;
  }

  @Output("registry-pairs")
  @Type("registry-pair[]")
  public void setRegistryPairs(List<RegistryPair> paramList)
  {
    this.registryPairs = paramList;
  }

  public List<RegistryPair> getRegistryPairs()
  {
    return this.registryPairs;
  }

  public RegistryListInfo withRegistryPairs(List<RegistryPair> paramList)
  {
    this.registryPairs = paramList;
    return this;
  }

  public RegistryListInfo withRegistryPairs(RegistryPair[] paramArrayOfRegistryPair)
  {
    this.registryPairs = new ArrayList(Arrays.asList(paramArrayOfRegistryPair));
    return this;
  }
}