package com.netapp.filer.rsh;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("rsh-get-stats")
@Status(StatusEnum.PUBLIC)
@Family({"ontap-c"})
public class RshGetStats
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long activeHighSessions;
  public Long currentActiveSessions;
  public Long maxAvailableSessions;
  public List<RshSessionInfo> rshSessions;
  public Long sessionInvocations;

  @Output("active-high-sessions")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setActiveHighSessions(Long paramLong)
  {
    this.activeHighSessions = paramLong;
  }

  public Long getActiveHighSessions()
  {
    return this.activeHighSessions;
  }

  public RshGetStats withActiveHighSessions(Long paramLong)
  {
    this.activeHighSessions = paramLong;
    return this;
  }

  @Output("current-active-sessions")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setCurrentActiveSessions(Long paramLong)
  {
    this.currentActiveSessions = paramLong;
  }

  public Long getCurrentActiveSessions()
  {
    return this.currentActiveSessions;
  }

  public RshGetStats withCurrentActiveSessions(Long paramLong)
  {
    this.currentActiveSessions = paramLong;
    return this;
  }

  @Output("max-available-sessions")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setMaxAvailableSessions(Long paramLong)
  {
    this.maxAvailableSessions = paramLong;
  }

  public Long getMaxAvailableSessions()
  {
    return this.maxAvailableSessions;
  }

  public RshGetStats withMaxAvailableSessions(Long paramLong)
  {
    this.maxAvailableSessions = paramLong;
    return this;
  }

  @Output("rsh-sessions")
  @Type("rsh-session-info[]")
  @Optional
  public void setRshSessions(List<RshSessionInfo> paramList)
  {
    this.rshSessions = paramList;
  }

  public List<RshSessionInfo> getRshSessions()
  {
    return this.rshSessions;
  }

  public RshGetStats withRshSessions(List<RshSessionInfo> paramList)
  {
    this.rshSessions = paramList;
    return this;
  }

  public RshGetStats withRshSessions(RshSessionInfo[] paramArrayOfRshSessionInfo)
  {
    this.rshSessions = new ArrayList(Arrays.asList(paramArrayOfRshSessionInfo));
    return this;
  }

  @Output("session-invocations")
  @Type("integer")
  @Range(min="0", max="2^32-1")
  public void setSessionInvocations(Long paramLong)
  {
    this.sessionInvocations = paramLong;
  }

  public Long getSessionInvocations()
  {
    return this.sessionInvocations;
  }

  public RshGetStats withSessionInvocations(Long paramLong)
  {
    this.sessionInvocations = paramLong;
    return this;
  }
}