package com.netapp.filer.snaplock;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("snaplock-log-status-list-info")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "ENOENT", "EONTAPI_EWORMMNOTWORM", "EONTAPI_EWORMNOCLOCK", "EONTAPI_EWORMVOLNOTWORM"})
public class SnaplockLogStatusListInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String logBasename;
  public Long logFileCount;
  public List<LogFileInfo> logFileDetails;
  public String logVolumeOutput;
  public String logVolumeInput;

  @Input("log-basename")
  @Type("string")
  @Optional
  public void setLogBasename(String paramString)
  {
    this.logBasename = paramString;
  }

  public String getLogBasename()
  {
    return this.logBasename;
  }

  public SnaplockLogStatusListInfo withLogBasename(String paramString)
  {
    this.logBasename = paramString;
    return this;
  }

  @Output("log-file-count")
  @Type("integer")
  public void setLogFileCount(Long paramLong)
  {
    this.logFileCount = paramLong;
  }

  public Long getLogFileCount()
  {
    return this.logFileCount;
  }

  public SnaplockLogStatusListInfo withLogFileCount(Long paramLong)
  {
    this.logFileCount = paramLong;
    return this;
  }

  @Output("log-file-details")
  @Type("log-file-info[]")
  @Optional
  public void setLogFileDetails(List<LogFileInfo> paramList)
  {
    this.logFileDetails = paramList;
  }

  public List<LogFileInfo> getLogFileDetails()
  {
    return this.logFileDetails;
  }

  public SnaplockLogStatusListInfo withLogFileDetails(List<LogFileInfo> paramList)
  {
    this.logFileDetails = paramList;
    return this;
  }

  public SnaplockLogStatusListInfo withLogFileDetails(LogFileInfo[] paramArrayOfLogFileInfo)
  {
    this.logFileDetails = new ArrayList(Arrays.asList(paramArrayOfLogFileInfo));
    return this;
  }

  @Output("log-volume")
  @Type("string")
  public void setLogVolumeOutput(String paramString)
  {
    this.logVolumeOutput = paramString;
  }

  public String getLogVolumeOutput()
  {
    return this.logVolumeOutput;
  }

  public SnaplockLogStatusListInfo withLogVolumeOutput(String paramString)
  {
    this.logVolumeOutput = paramString;
    return this;
  }

  @Input("log-volume")
  @Type("string")
  @Optional
  public void setLogVolumeInput(String paramString)
  {
    this.logVolumeInput = paramString;
  }

  public String getLogVolumeInput()
  {
    return this.logVolumeInput;
  }

  public SnaplockLogStatusListInfo withLogVolumeInput(String paramString)
  {
    this.logVolumeInput = paramString;
    return this;
  }
}