package com.netapp.filer.snapshot;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("snapshot-reclaimable-info")
@Status(StatusEnum.PUBLIC)
@Errno({"ESNAPSHOTDOESNOTEXIST", "ESNAPSHOTTOOMANY", "EVOLUMEBUSY", "EVOLUMEDOESNOTEXIST", "EVOLUMEMOUNTING", "EVOLUMEOFFLINE"})
@Family({"ontap-c", "vfiler"})
public class SnapshotReclaimableInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long reclaimableSize;
  public List<String> snapshots;
  public String volume;

  @Output("reclaimable-size")
  @Type("integer")
  @Range(min="0", max="2^63-1")
  public void setReclaimableSize(Long paramLong)
  {
    this.reclaimableSize = paramLong;
  }

  public Long getReclaimableSize()
  {
    return this.reclaimableSize;
  }

  public SnapshotReclaimableInfo withReclaimableSize(Long paramLong)
  {
    this.reclaimableSize = paramLong;
    return this;
  }

  @Input("snapshots")
  @Type("snapshot-name[]")
  public void setSnapshots(List<String> paramList)
  {
    this.snapshots = paramList;
  }

  public List<String> getSnapshots()
  {
    return this.snapshots;
  }

  public SnapshotReclaimableInfo withSnapshots(List<String> paramList)
  {
    this.snapshots = paramList;
    return this;
  }

  public SnapshotReclaimableInfo withSnapshots(String[] paramArrayOfString)
  {
    this.snapshots = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("volume")
  @Type("string")
  public void setVolume(String paramString)
  {
    this.volume = paramString;
  }

  public String getVolume()
  {
    return this.volume;
  }

  public SnapshotReclaimableInfo withVolume(String paramString)
  {
    this.volume = paramString;
    return this;
  }
}