package com.netapp.filer.snapvault;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;

@Api("snapvault-primary-initiate-restore-transfer")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPILICENSE", "ESNAPVAULTBUSY", "ESNAPVAULTERR", "ESNAPVAULTRESOURCE", "ESNAPVAULTSETUP"})
@Family({"ontap-c", "vfiler"})
public class SnapvaultPrimaryInitiateRestoreTransfer
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Long maxTransferRate;
  public Boolean noLunCloneExpansion;
  public String primaryPath;
  public String secondaryPath;
  public String secondarySnapshot;
  public String secondarySystem;

  @Input("max-transfer-rate")
  @Type("integer")
  @Optional
  @Range(min="1", max="2^32-1")
  public void setMaxTransferRate(Long paramLong)
  {
    this.maxTransferRate = paramLong;
  }

  public Long getMaxTransferRate()
  {
    return this.maxTransferRate;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withMaxTransferRate(Long paramLong)
  {
    this.maxTransferRate = paramLong;
    return this;
  }

  @Input("no-lun-clone-expansion")
  @Type("boolean")
  @Optional
  public void setNoLunCloneExpansion(Boolean paramBoolean)
  {
    this.noLunCloneExpansion = paramBoolean;
  }

  public Boolean isNoLunCloneExpansion()
  {
    return this.noLunCloneExpansion;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withNoLunCloneExpansion(Boolean paramBoolean)
  {
    this.noLunCloneExpansion = paramBoolean;
    return this;
  }

  @Input("primary-path")
  @Type("string")
  public void setPrimaryPath(String paramString)
  {
    this.primaryPath = paramString;
  }

  public String getPrimaryPath()
  {
    return this.primaryPath;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withPrimaryPath(String paramString)
  {
    this.primaryPath = paramString;
    return this;
  }

  @Input("secondary-path")
  @Type("string")
  public void setSecondaryPath(String paramString)
  {
    this.secondaryPath = paramString;
  }

  public String getSecondaryPath()
  {
    return this.secondaryPath;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withSecondaryPath(String paramString)
  {
    this.secondaryPath = paramString;
    return this;
  }

  @Input("secondary-snapshot")
  @Type("string")
  @Optional
  public void setSecondarySnapshot(String paramString)
  {
    this.secondarySnapshot = paramString;
  }

  public String getSecondarySnapshot()
  {
    return this.secondarySnapshot;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withSecondarySnapshot(String paramString)
  {
    this.secondarySnapshot = paramString;
    return this;
  }

  @Input("secondary-system")
  @Type("string")
  public void setSecondarySystem(String paramString)
  {
    this.secondarySystem = paramString;
  }

  public String getSecondarySystem()
  {
    return this.secondarySystem;
  }

  public SnapvaultPrimaryInitiateRestoreTransfer withSecondarySystem(String paramString)
  {
    this.secondarySystem = paramString;
    return this;
  }
}