package com.netapp.filer.snapvault;

import com.netapp.common.zapi.annotation.Element;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Type;
import com.netapp.common.zapi.annotation.Typedef;
import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

@Typedef("snapvault-status-info")
public class SnapvaultStatusInfo
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public String baseSnapshot;
  public BigInteger compressedBytes;
  public String contents;
  public String currentTransferError;
  public String currentTransferType;
  public String destinationPath;
  public String destinationSystem;
  public Date lagTime;
  public Long lastTransferDuration;
  public String lastTransferFromPath;
  public String lastTransferFromSystem;
  public Long lastTransferSize;
  public String lastTransferType;
  public Date mirrorTimestamp;
  public String sourcePath;
  public String sourceSystem;
  public String state;
  public String status;
  public Long transferProgress;
  public BigInteger uncompressedBytes;

  @Element("base-snapshot")
  @Type("string")
  @Optional
  public void setBaseSnapshot(String paramString)
  {
    this.baseSnapshot = paramString;
  }

  public String getBaseSnapshot()
  {
    return this.baseSnapshot;
  }

  public SnapvaultStatusInfo withBaseSnapshot(String paramString)
  {
    this.baseSnapshot = paramString;
    return this;
  }

  @Element("compressed-bytes")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setCompressedBytes(BigInteger paramBigInteger)
  {
    this.compressedBytes = paramBigInteger;
  }

  public BigInteger getCompressedBytes()
  {
    return this.compressedBytes;
  }

  public SnapvaultStatusInfo withCompressedBytes(BigInteger paramBigInteger)
  {
    this.compressedBytes = paramBigInteger;
    return this;
  }

  @Element("contents")
  @Type("string")
  @Optional
  public void setContents(String paramString)
  {
    this.contents = paramString;
  }

  public String getContents()
  {
    return this.contents;
  }

  public SnapvaultStatusInfo withContents(String paramString)
  {
    this.contents = paramString;
    return this;
  }

  @Element("current-transfer-error")
  @Type("string")
  @Optional
  public void setCurrentTransferError(String paramString)
  {
    this.currentTransferError = paramString;
  }

  public String getCurrentTransferError()
  {
    return this.currentTransferError;
  }

  public SnapvaultStatusInfo withCurrentTransferError(String paramString)
  {
    this.currentTransferError = paramString;
    return this;
  }

  @Element("current-transfer-type")
  @Type("string")
  @Optional
  public void setCurrentTransferType(String paramString)
  {
    this.currentTransferType = paramString;
  }

  public String getCurrentTransferType()
  {
    return this.currentTransferType;
  }

  public SnapvaultStatusInfo withCurrentTransferType(String paramString)
  {
    this.currentTransferType = paramString;
    return this;
  }

  @Element("destination-path")
  @Type("string")
  public void setDestinationPath(String paramString)
  {
    this.destinationPath = paramString;
  }

  public String getDestinationPath()
  {
    return this.destinationPath;
  }

  public SnapvaultStatusInfo withDestinationPath(String paramString)
  {
    this.destinationPath = paramString;
    return this;
  }

  @Element("destination-system")
  @Type("string")
  public void setDestinationSystem(String paramString)
  {
    this.destinationSystem = paramString;
  }

  public String getDestinationSystem()
  {
    return this.destinationSystem;
  }

  public SnapvaultStatusInfo withDestinationSystem(String paramString)
  {
    this.destinationSystem = paramString;
    return this;
  }

  @Element("lag-time")
  @Type("integer")
  @Optional
  @Range(min="1", max="2^32-1")
  public void setLagTime(Date paramDate)
  {
    this.lagTime = paramDate;
  }

  public Date getLagTime()
  {
    return this.lagTime;
  }

  public SnapvaultStatusInfo withLagTime(Date paramDate)
  {
    this.lagTime = paramDate;
    return this;
  }

  @Element("last-transfer-duration")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setLastTransferDuration(Long paramLong)
  {
    this.lastTransferDuration = paramLong;
  }

  public Long getLastTransferDuration()
  {
    return this.lastTransferDuration;
  }

  public SnapvaultStatusInfo withLastTransferDuration(Long paramLong)
  {
    this.lastTransferDuration = paramLong;
    return this;
  }

  @Element("last-transfer-from-path")
  @Type("string")
  @Optional
  public void setLastTransferFromPath(String paramString)
  {
    this.lastTransferFromPath = paramString;
  }

  public String getLastTransferFromPath()
  {
    return this.lastTransferFromPath;
  }

  public SnapvaultStatusInfo withLastTransferFromPath(String paramString)
  {
    this.lastTransferFromPath = paramString;
    return this;
  }

  @Element("last-transfer-from-system")
  @Type("string")
  @Optional
  public void setLastTransferFromSystem(String paramString)
  {
    this.lastTransferFromSystem = paramString;
  }

  public String getLastTransferFromSystem()
  {
    return this.lastTransferFromSystem;
  }

  public SnapvaultStatusInfo withLastTransferFromSystem(String paramString)
  {
    this.lastTransferFromSystem = paramString;
    return this;
  }

  @Element("last-transfer-size")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setLastTransferSize(Long paramLong)
  {
    this.lastTransferSize = paramLong;
  }

  public Long getLastTransferSize()
  {
    return this.lastTransferSize;
  }

  public SnapvaultStatusInfo withLastTransferSize(Long paramLong)
  {
    this.lastTransferSize = paramLong;
    return this;
  }

  @Element("last-transfer-type")
  @Type("string")
  @Optional
  public void setLastTransferType(String paramString)
  {
    this.lastTransferType = paramString;
  }

  public String getLastTransferType()
  {
    return this.lastTransferType;
  }

  public SnapvaultStatusInfo withLastTransferType(String paramString)
  {
    this.lastTransferType = paramString;
    return this;
  }

  @Element("mirror-timestamp")
  @Type("integer")
  @Optional
  @Range(min="1", max="2^32-1")
  public void setMirrorTimestamp(Date paramDate)
  {
    this.mirrorTimestamp = paramDate;
  }

  public Date getMirrorTimestamp()
  {
    return this.mirrorTimestamp;
  }

  public SnapvaultStatusInfo withMirrorTimestamp(Date paramDate)
  {
    this.mirrorTimestamp = paramDate;
    return this;
  }

  @Element("source-path")
  @Type("string")
  public void setSourcePath(String paramString)
  {
    this.sourcePath = paramString;
  }

  public String getSourcePath()
  {
    return this.sourcePath;
  }

  public SnapvaultStatusInfo withSourcePath(String paramString)
  {
    this.sourcePath = paramString;
    return this;
  }

  @Element("source-system")
  @Type("string")
  public void setSourceSystem(String paramString)
  {
    this.sourceSystem = paramString;
  }

  public String getSourceSystem()
  {
    return this.sourceSystem;
  }

  public SnapvaultStatusInfo withSourceSystem(String paramString)
  {
    this.sourceSystem = paramString;
    return this;
  }

  @Element("state")
  @Type("string")
  public void setState(String paramString)
  {
    this.state = paramString;
  }

  public String getState()
  {
    return this.state;
  }

  public SnapvaultStatusInfo withState(String paramString)
  {
    this.state = paramString;
    return this;
  }

  @Element("status")
  @Type("string")
  public void setStatus(String paramString)
  {
    this.status = paramString;
  }

  public String getStatus()
  {
    return this.status;
  }

  public SnapvaultStatusInfo withStatus(String paramString)
  {
    this.status = paramString;
    return this;
  }

  @Element("transfer-progress")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^32-1")
  public void setTransferProgress(Long paramLong)
  {
    this.transferProgress = paramLong;
  }

  public Long getTransferProgress()
  {
    return this.transferProgress;
  }

  public SnapvaultStatusInfo withTransferProgress(Long paramLong)
  {
    this.transferProgress = paramLong;
    return this;
  }

  @Element("uncompressed-bytes")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^64-1")
  public void setUncompressedBytes(BigInteger paramBigInteger)
  {
    this.uncompressedBytes = paramBigInteger;
  }

  public BigInteger getUncompressedBytes()
  {
    return this.uncompressedBytes;
  }

  public SnapvaultStatusInfo withUncompressedBytes(BigInteger paramBigInteger)
  {
    this.uncompressedBytes = paramBigInteger;
    return this;
  }
}