package com.netapp.filer.vfiler;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("vfiler-dr-configure")
@Status(StatusEnum.PUBLIC)
@Errno({"EVFILERNOTFOUND"})
public class VfilerDrConfigure
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<String> dnsServerIpaddrs;
  public List<String> nisServerIpaddrs;
  public HostnamePair alternateSrcDst;
  public List<IpaddrInfo> ipaddrs;
  public Boolean synchronous;
  public AuthenticationInfo remoteAuthenticationInfo;
  public VfilerLocation remoteVfilerLocation;
  public Boolean useSecureCommandChannel;

  @Input("DNS-server-ipaddrs")
  @Type("ip-address[]")
  @Optional
  public void setDnsServerIpaddrs(List<String> paramList)
  {
    this.dnsServerIpaddrs = paramList;
  }

  public List<String> getDnsServerIpaddrs()
  {
    return this.dnsServerIpaddrs;
  }

  public VfilerDrConfigure withDnsServerIpaddrs(List<String> paramList)
  {
    this.dnsServerIpaddrs = paramList;
    return this;
  }

  public VfilerDrConfigure withDnsServerIpaddrs(String[] paramArrayOfString)
  {
    this.dnsServerIpaddrs = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("NIS-server-ipaddrs")
  @Type("ip-address[]")
  @Optional
  public void setNisServerIpaddrs(List<String> paramList)
  {
    this.nisServerIpaddrs = paramList;
  }

  public List<String> getNisServerIpaddrs()
  {
    return this.nisServerIpaddrs;
  }

  public VfilerDrConfigure withNisServerIpaddrs(List<String> paramList)
  {
    this.nisServerIpaddrs = paramList;
    return this;
  }

  public VfilerDrConfigure withNisServerIpaddrs(String[] paramArrayOfString)
  {
    this.nisServerIpaddrs = new ArrayList(Arrays.asList(paramArrayOfString));
    return this;
  }

  @Input("alternate-src-dst")
  @Type("hostname-pair")
  @Optional
  public void setAlternateSrcDst(HostnamePair paramHostnamePair)
  {
    this.alternateSrcDst = paramHostnamePair;
  }

  public HostnamePair getAlternateSrcDst()
  {
    return this.alternateSrcDst;
  }

  public VfilerDrConfigure withAlternateSrcDst(HostnamePair paramHostnamePair)
  {
    this.alternateSrcDst = paramHostnamePair;
    return this;
  }

  @Input("ipaddrs")
  @Type("ipaddr-info[]")
  public void setIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
  }

  public List<IpaddrInfo> getIpaddrs()
  {
    return this.ipaddrs;
  }

  public VfilerDrConfigure withIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
    return this;
  }

  public VfilerDrConfigure withIpaddrs(IpaddrInfo[] paramArrayOfIpaddrInfo)
  {
    this.ipaddrs = new ArrayList(Arrays.asList(paramArrayOfIpaddrInfo));
    return this;
  }

  @Input("is-synchronous")
  @Type("boolean")
  public void setSynchronous(Boolean paramBoolean)
  {
    this.synchronous = paramBoolean;
  }

  public Boolean isSynchronous()
  {
    return this.synchronous;
  }

  public VfilerDrConfigure withSynchronous(Boolean paramBoolean)
  {
    this.synchronous = paramBoolean;
    return this;
  }

  @Input("remote-authentication-info")
  @Type("authentication-info")
  public void setRemoteAuthenticationInfo(AuthenticationInfo paramAuthenticationInfo)
  {
    this.remoteAuthenticationInfo = paramAuthenticationInfo;
  }

  public AuthenticationInfo getRemoteAuthenticationInfo()
  {
    return this.remoteAuthenticationInfo;
  }

  public VfilerDrConfigure withRemoteAuthenticationInfo(AuthenticationInfo paramAuthenticationInfo)
  {
    this.remoteAuthenticationInfo = paramAuthenticationInfo;
    return this;
  }

  @Input("remote-vfiler-location")
  @Type("vfiler-location")
  public void setRemoteVfilerLocation(VfilerLocation paramVfilerLocation)
  {
    this.remoteVfilerLocation = paramVfilerLocation;
  }

  public VfilerLocation getRemoteVfilerLocation()
  {
    return this.remoteVfilerLocation;
  }

  public VfilerDrConfigure withRemoteVfilerLocation(VfilerLocation paramVfilerLocation)
  {
    this.remoteVfilerLocation = paramVfilerLocation;
    return this;
  }

  @Input("use-secure-command-channel")
  @Type("boolean")
  @Optional
  public void setUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
  }

  public Boolean isUseSecureCommandChannel()
  {
    return this.useSecureCommandChannel;
  }

  public VfilerDrConfigure withUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
    return this;
  }
}