package com.netapp.filer.vfiler;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("vfiler-migrate")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPIMISSINGARGUMENT", "EVFILERNOTFOUND", "EVFILEROPNOTALLOWED"})
public class VfilerMigrate
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Boolean force;
  public List<IpaddrInfo> ipaddrs;
  public String migrationMethod;
  public String password;
  public String remoteFiler;
  public Boolean useSecureCommandChannel;
  public String username;
  public String vfiler;

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public VfilerMigrate withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("ipaddrs")
  @Type("ipaddr-info[]")
  public void setIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
  }

  public List<IpaddrInfo> getIpaddrs()
  {
    return this.ipaddrs;
  }

  public VfilerMigrate withIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
    return this;
  }

  public VfilerMigrate withIpaddrs(IpaddrInfo[] paramArrayOfIpaddrInfo)
  {
    this.ipaddrs = new ArrayList(Arrays.asList(paramArrayOfIpaddrInfo));
    return this;
  }

  @Input("migration-method")
  @Type("string")
  @Optional
  public void setMigrationMethod(String paramString)
  {
    this.migrationMethod = paramString;
  }

  public String getMigrationMethod()
  {
    return this.migrationMethod;
  }

  public VfilerMigrate withMigrationMethod(String paramString)
  {
    this.migrationMethod = paramString;
    return this;
  }

  @Input("password")
  @Type("string")
  public void setPassword(String paramString)
  {
    this.password = paramString;
  }

  public String getPassword()
  {
    return this.password;
  }

  public VfilerMigrate withPassword(String paramString)
  {
    this.password = paramString;
    return this;
  }

  @Input("remote-filer")
  @Type("string")
  public void setRemoteFiler(String paramString)
  {
    this.remoteFiler = paramString;
  }

  public String getRemoteFiler()
  {
    return this.remoteFiler;
  }

  public VfilerMigrate withRemoteFiler(String paramString)
  {
    this.remoteFiler = paramString;
    return this;
  }

  @Input("use-secure-command-channel")
  @Type("boolean")
  @Optional
  public void setUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
  }

  public Boolean isUseSecureCommandChannel()
  {
    return this.useSecureCommandChannel;
  }

  public VfilerMigrate withUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
    return this;
  }

  @Input("username")
  @Type("string")
  public void setUsername(String paramString)
  {
    this.username = paramString;
  }

  public String getUsername()
  {
    return this.username;
  }

  public VfilerMigrate withUsername(String paramString)
  {
    this.username = paramString;
    return this;
  }

  @Input("vfiler")
  @Type("string")
  public void setVfiler(String paramString)
  {
    this.vfiler = paramString;
  }

  public String getVfiler()
  {
    return this.vfiler;
  }

  public VfilerMigrate withVfiler(String paramString)
  {
    this.vfiler = paramString;
    return this;
  }
}