package com.netapp.filer.vfiler;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("vfiler-migrate-start")
@Status(StatusEnum.PUBLIC)
@Errno({"EVFILERNOTFOUND", "EVFILEROPNOTALLOWED"})
public class VfilerMigrateStart
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<IpaddrInfo> ipaddrs;
  public AuthenticationInfo remoteAuthenticationInfo;
  public VfilerLocation remoteVfilerLocation;
  public Boolean useSecureCommandChannel;

  @Input("ipaddrs")
  @Type("ipaddr-info[]")
  public void setIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
  }

  public List<IpaddrInfo> getIpaddrs()
  {
    return this.ipaddrs;
  }

  public VfilerMigrateStart withIpaddrs(List<IpaddrInfo> paramList)
  {
    this.ipaddrs = paramList;
    return this;
  }

  public VfilerMigrateStart withIpaddrs(IpaddrInfo[] paramArrayOfIpaddrInfo)
  {
    this.ipaddrs = new ArrayList(Arrays.asList(paramArrayOfIpaddrInfo));
    return this;
  }

  @Input("remote-authentication-info")
  @Type("authentication-info")
  public void setRemoteAuthenticationInfo(AuthenticationInfo paramAuthenticationInfo)
  {
    this.remoteAuthenticationInfo = paramAuthenticationInfo;
  }

  public AuthenticationInfo getRemoteAuthenticationInfo()
  {
    return this.remoteAuthenticationInfo;
  }

  public VfilerMigrateStart withRemoteAuthenticationInfo(AuthenticationInfo paramAuthenticationInfo)
  {
    this.remoteAuthenticationInfo = paramAuthenticationInfo;
    return this;
  }

  @Input("remote-vfiler-location")
  @Type("vfiler-location")
  public void setRemoteVfilerLocation(VfilerLocation paramVfilerLocation)
  {
    this.remoteVfilerLocation = paramVfilerLocation;
  }

  public VfilerLocation getRemoteVfilerLocation()
  {
    return this.remoteVfilerLocation;
  }

  public VfilerMigrateStart withRemoteVfilerLocation(VfilerLocation paramVfilerLocation)
  {
    this.remoteVfilerLocation = paramVfilerLocation;
    return this;
  }

  @Input("use-secure-command-channel")
  @Type("boolean")
  @Optional
  public void setUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
  }

  public Boolean isUseSecureCommandChannel()
  {
    return this.useSecureCommandChannel;
  }

  public VfilerMigrateStart withUseSecureCommandChannel(Boolean paramBoolean)
  {
    this.useSecureCommandChannel = paramBoolean;
    return this;
  }
}