package com.netapp.filer.vfiler;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("vfiler-setup")
@Status(StatusEnum.PUBLIC)
@Errno({"EAPIMISSINGARGUMENT", "EVFILERNOTFOUND", "EVFILEROPNOTALLOWED"})
public class VfilerSetup
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public Adminhost adminhost;
  public String dnsdomain;
  public List<DnsserverInfo> dnsservers;
  public List<IpbindingInfo> ipbindings;
  public String nisdomain;
  public List<NisserverInfo> nisservers;
  public String password;
  public String vfiler;

  @Input("adminhost")
  @Type("adminhost")
  @Optional
  public void setAdminhost(Adminhost paramAdminhost)
  {
    this.adminhost = paramAdminhost;
  }

  public Adminhost getAdminhost()
  {
    return this.adminhost;
  }

  public VfilerSetup withAdminhost(Adminhost paramAdminhost)
  {
    this.adminhost = paramAdminhost;
    return this;
  }

  @Input("dnsdomain")
  @Type("string")
  @Optional
  public void setDnsdomain(String paramString)
  {
    this.dnsdomain = paramString;
  }

  public String getDnsdomain()
  {
    return this.dnsdomain;
  }

  public VfilerSetup withDnsdomain(String paramString)
  {
    this.dnsdomain = paramString;
    return this;
  }

  @Input("dnsservers")
  @Type("dnsserver-info[]")
  @Optional
  public void setDnsservers(List<DnsserverInfo> paramList)
  {
    this.dnsservers = paramList;
  }

  public List<DnsserverInfo> getDnsservers()
  {
    return this.dnsservers;
  }

  public VfilerSetup withDnsservers(List<DnsserverInfo> paramList)
  {
    this.dnsservers = paramList;
    return this;
  }

  public VfilerSetup withDnsservers(DnsserverInfo[] paramArrayOfDnsserverInfo)
  {
    this.dnsservers = new ArrayList(Arrays.asList(paramArrayOfDnsserverInfo));
    return this;
  }

  @Input("ipbindings")
  @Type("ipbinding-info[]")
  @Optional
  public void setIpbindings(List<IpbindingInfo> paramList)
  {
    this.ipbindings = paramList;
  }

  public List<IpbindingInfo> getIpbindings()
  {
    return this.ipbindings;
  }

  public VfilerSetup withIpbindings(List<IpbindingInfo> paramList)
  {
    this.ipbindings = paramList;
    return this;
  }

  public VfilerSetup withIpbindings(IpbindingInfo[] paramArrayOfIpbindingInfo)
  {
    this.ipbindings = new ArrayList(Arrays.asList(paramArrayOfIpbindingInfo));
    return this;
  }

  @Input("nisdomain")
  @Type("string")
  @Optional
  public void setNisdomain(String paramString)
  {
    this.nisdomain = paramString;
  }

  public String getNisdomain()
  {
    return this.nisdomain;
  }

  public VfilerSetup withNisdomain(String paramString)
  {
    this.nisdomain = paramString;
    return this;
  }

  @Input("nisservers")
  @Type("nisserver-info[]")
  @Optional
  public void setNisservers(List<NisserverInfo> paramList)
  {
    this.nisservers = paramList;
  }

  public List<NisserverInfo> getNisservers()
  {
    return this.nisservers;
  }

  public VfilerSetup withNisservers(List<NisserverInfo> paramList)
  {
    this.nisservers = paramList;
    return this;
  }

  public VfilerSetup withNisservers(NisserverInfo[] paramArrayOfNisserverInfo)
  {
    this.nisservers = new ArrayList(Arrays.asList(paramArrayOfNisserverInfo));
    return this;
  }

  @Input("password")
  @Type("string")
  @Optional
  public void setPassword(String paramString)
  {
    this.password = paramString;
  }

  public String getPassword()
  {
    return this.password;
  }

  public VfilerSetup withPassword(String paramString)
  {
    this.password = paramString;
    return this;
  }

  @Input("vfiler")
  @Type("string")
  public void setVfiler(String paramString)
  {
    this.vfiler = paramString;
  }

  public String getVfiler()
  {
    return this.vfiler;
  }

  public VfilerSetup withVfiler(String paramString)
  {
    this.vfiler = paramString;
    return this;
  }
}