package com.netapp.filer.volume;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Range;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("volume-add")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "EVOLUMEDISKDUP", "EVOLUMEDISKSIZEINVALID", "EVOLUMERGINVALID"})
public class VolumeAdd
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<DiskInfo> badDisks;
  public Integer diskCount;
  public Integer diskSize;
  public String diskSizeWithUnit;
  public List<DiskInfo> disks;
  public Boolean force;
  public List<DiskInfo> mirrorDisks;
  public String raidGroup;
  public String volume;

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public VolumeAdd withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public VolumeAdd withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("disk-count")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
  }

  public Integer getDiskCount()
  {
    return this.diskCount;
  }

  public VolumeAdd withDiskCount(Integer paramInteger)
  {
    this.diskCount = paramInteger;
    return this;
  }

  @Input("disk-size")
  @Type("integer")
  @Optional
  @Range(min="0", max="2^31-1")
  public void setDiskSize(Integer paramInteger)
  {
    this.diskSize = paramInteger;
  }

  public Integer getDiskSize()
  {
    return this.diskSize;
  }

  public VolumeAdd withDiskSize(Integer paramInteger)
  {
    this.diskSize = paramInteger;
    return this;
  }

  @Input("disk-size-with-unit")
  @Type("string")
  @Optional
  public void setDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
  }

  public String getDiskSizeWithUnit()
  {
    return this.diskSizeWithUnit;
  }

  public VolumeAdd withDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
    return this;
  }

  @Input("disks")
  @Type("disk-info[]")
  @Optional
  public void setDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
  }

  public List<DiskInfo> getDisks()
  {
    return this.disks;
  }

  public VolumeAdd withDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
    return this;
  }

  public VolumeAdd withDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.disks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public VolumeAdd withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public VolumeAdd withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public VolumeAdd withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("raid-group")
  @Type("string")
  @Optional
  public void setRaidGroup(String paramString)
  {
    this.raidGroup = paramString;
  }

  public String getRaidGroup()
  {
    return this.raidGroup;
  }

  public VolumeAdd withRaidGroup(String paramString)
  {
    this.raidGroup = paramString;
    return this;
  }

  @Input("volume")
  @Type("string")
  public void setVolume(String paramString)
  {
    this.volume = paramString;
  }

  public String getVolume()
  {
    return this.volume;
  }

  public VolumeAdd withVolume(String paramString)
  {
    this.volume = paramString;
    return this;
  }
}