package com.netapp.filer.volume;

import com.netapp.common.zapi.annotation.Api;
import com.netapp.common.zapi.annotation.Errno;
import com.netapp.common.zapi.annotation.Family;
import com.netapp.common.zapi.annotation.Input;
import com.netapp.common.zapi.annotation.Optional;
import com.netapp.common.zapi.annotation.Output;
import com.netapp.common.zapi.annotation.Status;
import com.netapp.common.zapi.annotation.StatusEnum;
import com.netapp.common.zapi.annotation.Type;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@Api("volume-create")
@Status(StatusEnum.PUBLIC)
@Errno({"EINTERNALERROR", "EINVALIDINPUTERROR", "ESERVICENOTLICENSED", "EVOLUMEDISKDUP", "EVOLUMEDISKSIZEINVALID", "EVOLUMELANGUAGEINVALID", "EVOLUMENAMEINVALID", "EVOLUMERGSIZEINVALID"})
@Family({"ontap-c", "vfiler"})
public class VolumeCreate
  implements Serializable
{
  private static final long serialVersionUID = 1L;
  public List<DiskInfo> badDisks;
  public String containingAggrName;
  public Long diskCount;
  public Long diskSize;
  public String diskSizeWithUnit;
  public List<DiskInfo> disks;
  public Boolean force;
  public Boolean mirrored;
  public Boolean snaplock;
  public String languageCode;
  public List<DiskInfo> mirrorDisks;
  public Long raidSize;
  public String remoteLocation;
  public String size;
  public String snaplockType;
  public String spaceReserve;
  public String volume;
  public String volumeRaidType;

  @Output("bad-disks")
  @Type("disk-info[]")
  @Optional
  public void setBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
  }

  public List<DiskInfo> getBadDisks()
  {
    return this.badDisks;
  }

  public VolumeCreate withBadDisks(List<DiskInfo> paramList)
  {
    this.badDisks = paramList;
    return this;
  }

  public VolumeCreate withBadDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.badDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("containing-aggr-name")
  @Type("string")
  @Optional
  public void setContainingAggrName(String paramString)
  {
    this.containingAggrName = paramString;
  }

  public String getContainingAggrName()
  {
    return this.containingAggrName;
  }

  public VolumeCreate withContainingAggrName(String paramString)
  {
    this.containingAggrName = paramString;
    return this;
  }

  @Input("disk-count")
  @Type("integer")
  @Optional
  public void setDiskCount(Long paramLong)
  {
    this.diskCount = paramLong;
  }

  public Long getDiskCount()
  {
    return this.diskCount;
  }

  public VolumeCreate withDiskCount(Long paramLong)
  {
    this.diskCount = paramLong;
    return this;
  }

  @Input("disk-size")
  @Type("integer")
  @Optional
  public void setDiskSize(Long paramLong)
  {
    this.diskSize = paramLong;
  }

  public Long getDiskSize()
  {
    return this.diskSize;
  }

  public VolumeCreate withDiskSize(Long paramLong)
  {
    this.diskSize = paramLong;
    return this;
  }

  @Input("disk-size-with-unit")
  @Type("string")
  @Optional
  public void setDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
  }

  public String getDiskSizeWithUnit()
  {
    return this.diskSizeWithUnit;
  }

  public VolumeCreate withDiskSizeWithUnit(String paramString)
  {
    this.diskSizeWithUnit = paramString;
    return this;
  }

  @Input("disks")
  @Type("disk-info[]")
  @Optional
  public void setDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
  }

  public List<DiskInfo> getDisks()
  {
    return this.disks;
  }

  public VolumeCreate withDisks(List<DiskInfo> paramList)
  {
    this.disks = paramList;
    return this;
  }

  public VolumeCreate withDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.disks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("force")
  @Type("boolean")
  @Optional
  public void setForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
  }

  public Boolean isForce()
  {
    return this.force;
  }

  public VolumeCreate withForce(Boolean paramBoolean)
  {
    this.force = paramBoolean;
    return this;
  }

  @Input("is-mirrored")
  @Type("boolean")
  @Optional
  public void setMirrored(Boolean paramBoolean)
  {
    this.mirrored = paramBoolean;
  }

  public Boolean isMirrored()
  {
    return this.mirrored;
  }

  public VolumeCreate withMirrored(Boolean paramBoolean)
  {
    this.mirrored = paramBoolean;
    return this;
  }

  @Input("is-snaplock")
  @Type("boolean")
  @Optional
  public void setSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
  }

  public Boolean isSnaplock()
  {
    return this.snaplock;
  }

  public VolumeCreate withSnaplock(Boolean paramBoolean)
  {
    this.snaplock = paramBoolean;
    return this;
  }

  @Input("language-code")
  @Type("string")
  @Optional
  public void setLanguageCode(String paramString)
  {
    this.languageCode = paramString;
  }

  public String getLanguageCode()
  {
    return this.languageCode;
  }

  public VolumeCreate withLanguageCode(String paramString)
  {
    this.languageCode = paramString;
    return this;
  }

  @Input("mirror-disks")
  @Type("disk-info[]")
  @Optional
  public void setMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
  }

  public List<DiskInfo> getMirrorDisks()
  {
    return this.mirrorDisks;
  }

  public VolumeCreate withMirrorDisks(List<DiskInfo> paramList)
  {
    this.mirrorDisks = paramList;
    return this;
  }

  public VolumeCreate withMirrorDisks(DiskInfo[] paramArrayOfDiskInfo)
  {
    this.mirrorDisks = new ArrayList(Arrays.asList(paramArrayOfDiskInfo));
    return this;
  }

  @Input("raid-size")
  @Type("integer")
  @Optional
  public void setRaidSize(Long paramLong)
  {
    this.raidSize = paramLong;
  }

  public Long getRaidSize()
  {
    return this.raidSize;
  }

  public VolumeCreate withRaidSize(Long paramLong)
  {
    this.raidSize = paramLong;
    return this;
  }

  @Input("remote-location")
  @Type("string")
  @Optional
  public void setRemoteLocation(String paramString)
  {
    this.remoteLocation = paramString;
  }

  public String getRemoteLocation()
  {
    return this.remoteLocation;
  }

  public VolumeCreate withRemoteLocation(String paramString)
  {
    this.remoteLocation = paramString;
    return this;
  }

  @Input("size")
  @Type("string")
  @Optional
  public void setSize(String paramString)
  {
    this.size = paramString;
  }

  public String getSize()
  {
    return this.size;
  }

  public VolumeCreate withSize(String paramString)
  {
    this.size = paramString;
    return this;
  }

  @Input("snaplock-type")
  @Type("string")
  @Optional
  public void setSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
  }

  public String getSnaplockType()
  {
    return this.snaplockType;
  }

  public VolumeCreate withSnaplockType(String paramString)
  {
    this.snaplockType = paramString;
    return this;
  }

  @Input("space-reserve")
  @Type("string")
  @Optional
  public void setSpaceReserve(String paramString)
  {
    this.spaceReserve = paramString;
  }

  public String getSpaceReserve()
  {
    return this.spaceReserve;
  }

  public VolumeCreate withSpaceReserve(String paramString)
  {
    this.spaceReserve = paramString;
    return this;
  }

  @Input("volume")
  @Type("string")
  public void setVolume(String paramString)
  {
    this.volume = paramString;
  }

  public String getVolume()
  {
    return this.volume;
  }

  public VolumeCreate withVolume(String paramString)
  {
    this.volume = paramString;
    return this;
  }

  @Input("volume-raid-type")
  @Type("string")
  @Optional
  public void setVolumeRaidType(String paramString)
  {
    this.volumeRaidType = paramString;
  }

  public String getVolumeRaidType()
  {
    return this.volumeRaidType;
  }

  public VolumeCreate withVolumeRaidType(String paramString)
  {
    this.volumeRaidType = paramString;
    return this;
  }
}