package devmgr.trace;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

public class Trace
  implements TraceConstants
{
  private static final int BYTES_PER_LINE = 16;
  private static final char BYTE_SEPARATOR = '.';
  private static final long MAX_WAIT_MS = 5000L;
  private static final char[] m_UpperDigitTable = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' };
  private static final char[] m_LowerDigitTable = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
  private static Trace m_Trace = null;
  private boolean m_GlobalTracing = false;
  private boolean[] m_ComponentTracing = new boolean[TraceConstants.TRACE_COMPONENTS.length];
  private String m_TraceFile = "";
  private PrintWriter m_Output = null;
  private FileOutputStream m_OutFile = null;
  private boolean m_BeginNewLine = true;
  private SimpleDateFormat m_SimpleDateFormat = null;
  private Thread m_TransactionThread;

  private Trace()
  {
    for (int i = 0; i < this.m_ComponentTracing.length; i++)
      this.m_ComponentTracing[i] = false;
    this.m_TraceFile = "";
    this.m_TransactionThread = null;
    new SimpleDateFormat();
    this.m_SimpleDateFormat = ((SimpleDateFormat)DateFormat.getDateTimeInstance());
  }

  public static Trace getTrace()
  {
    if (m_Trace == null)
      m_Trace = new Trace();
    return m_Trace;
  }

  public void finalize()
  {
    try
    {
      closeOutput();
    }
    catch (IOException localIOException)
    {
    }
  }

  public static boolean isTracing(int paramInt)
  {
    Trace localTrace = getTrace();
    return localTrace.isTracingOn(paramInt);
  }

  public static void print(int paramInt, String paramString)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.printToOutput(localTrace.formatHeader(paramInt) + paramString);
  }

  public static void println(int paramInt, String paramString)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.printlnToOutput(localTrace.formatHeader(paramInt) + paramString);
  }

  public static void println(int paramInt)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.printlnToOutput(localTrace.formatHeader(paramInt));
  }

  public static void printBuffer(int paramInt, String paramString, byte[] paramArrayOfByte)
  {
    printBuffer(paramInt, paramString, paramArrayOfByte, 0, paramArrayOfByte.length);
  }

  public static void printBuffer(int paramInt1, String paramString, byte[] paramArrayOfByte, int paramInt2, int paramInt3)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt1))
    {
      localTrace.printlnToOutput(localTrace.formatHeader(paramInt1) + paramString);
      localTrace.printBufferToOutput(paramArrayOfByte, paramInt2, paramInt3);
    }
  }

  public static String bytesToString(byte[] paramArrayOfByte)
  {
    StringBuffer localStringBuffer = new StringBuffer();
    localStringBuffer.append('(');
    localStringBuffer.append(paramArrayOfByte.length);
    localStringBuffer.append(')');
    for (int i = 0; i < paramArrayOfByte.length - 1; i++)
    {
      localStringBuffer.append(toHex(paramArrayOfByte[i], 2));
      localStringBuffer.append('.');
    }
    localStringBuffer.append(toHex(paramArrayOfByte[i], 2));
    return localStringBuffer.toString();
  }

  public static void printStackTrace(int paramInt, Throwable paramThrowable)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
    {
      localTrace.printlnToOutput(localTrace.formatHeader(paramInt));
      localTrace.printStackTraceToOutput(paramThrowable);
    }
  }

  public static void startGroup(int paramInt)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.setLock();
  }

  public static void endGroup(int paramInt)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.clearLock();
  }

  public static void printf(int paramInt, String paramString, Object paramObject)
  {
    Object[] arrayOfObject = new Object[1];
    arrayOfObject[0] = paramObject;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2)
  {
    Object[] arrayOfObject = new Object[2];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3)
  {
    Object[] arrayOfObject = new Object[3];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4)
  {
    Object[] arrayOfObject = new Object[4];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4, Object paramObject5)
  {
    Object[] arrayOfObject = new Object[5];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    arrayOfObject[4] = paramObject5;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4, Object paramObject5, Object paramObject6)
  {
    Object[] arrayOfObject = new Object[6];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    arrayOfObject[4] = paramObject5;
    arrayOfObject[5] = paramObject6;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4, Object paramObject5, Object paramObject6, Object paramObject7)
  {
    Object[] arrayOfObject = new Object[7];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    arrayOfObject[4] = paramObject5;
    arrayOfObject[5] = paramObject6;
    arrayOfObject[6] = paramObject7;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4, Object paramObject5, Object paramObject6, Object paramObject7, Object paramObject8)
  {
    Object[] arrayOfObject = new Object[8];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    arrayOfObject[4] = paramObject5;
    arrayOfObject[5] = paramObject6;
    arrayOfObject[6] = paramObject7;
    arrayOfObject[7] = paramObject8;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void printf(int paramInt, String paramString, Object paramObject1, Object paramObject2, Object paramObject3, Object paramObject4, Object paramObject5, Object paramObject6, Object paramObject7, Object paramObject8, Object paramObject9)
  {
    Object[] arrayOfObject = new Object[9];
    arrayOfObject[0] = paramObject1;
    arrayOfObject[1] = paramObject2;
    arrayOfObject[2] = paramObject3;
    arrayOfObject[3] = paramObject4;
    arrayOfObject[4] = paramObject5;
    arrayOfObject[5] = paramObject6;
    arrayOfObject[6] = paramObject7;
    arrayOfObject[7] = paramObject8;
    arrayOfObject[8] = paramObject9;
    vprintf(paramInt, paramString, arrayOfObject);
  }

  public static void vprintf(int paramInt, String paramString, Object[] paramArrayOfObject)
  {
    Trace localTrace = getTrace();
    if (localTrace.isTracingOn(paramInt))
      localTrace.printlnToOutput(localTrace.formatHeader(paramInt) + FormatString(paramString, paramArrayOfObject).toString());
  }

  public TraceConfiguration getCurrentSettings()
  {
    TraceConfiguration localTraceConfiguration = new TraceConfiguration(this.m_GlobalTracing, this.m_ComponentTracing, this.m_TraceFile);
    return localTraceConfiguration;
  }

  public synchronized void setTraceConfiguration(TraceConfiguration paramTraceConfiguration)
    throws IOException
  {
    waitLock();
    if (paramTraceConfiguration.getGlobalTracing() == true)
    {
      if (!this.m_GlobalTracing)
      {
        this.m_TraceFile = paramTraceConfiguration.getTraceFile();
        openFile();
      }
      else if (!paramTraceConfiguration.getTraceFile().equals(this.m_TraceFile))
      {
        closeOutput();
        this.m_TraceFile = paramTraceConfiguration.getTraceFile();
        openFile();
      }
      this.m_GlobalTracing = true;
    }
    else if (this.m_GlobalTracing)
    {
      this.m_GlobalTracing = false;
      closeOutput();
    }
    boolean[] arrayOfBoolean = paramTraceConfiguration.getComponentTracing();
    for (int i = 0; i < this.m_ComponentTracing.length; i++)
      this.m_ComponentTracing[i] = arrayOfBoolean[i];
  }

  public void enableComponents(int[] paramArrayOfInt, String paramString)
    throws IOException
  {
    TraceConfiguration localTraceConfiguration = new TraceConfiguration(this.m_GlobalTracing, this.m_ComponentTracing, this.m_TraceFile);
    localTraceConfiguration.setGlobalTracing(true);
    if (paramString != null)
      localTraceConfiguration.setTraceFile(paramString);
    int i;
    if (paramArrayOfInt.length == 0)
      for (i = 0; i < this.m_ComponentTracing.length; i++)
        localTraceConfiguration.setComponentTracing(i, true);
    else
      for (i = 0; i < paramArrayOfInt.length; i++)
        localTraceConfiguration.setComponentTracing(paramArrayOfInt[i], true);
    setTraceConfiguration(localTraceConfiguration);
  }

  public void disableAll()
  {
    TraceConfiguration localTraceConfiguration = new TraceConfiguration(this.m_GlobalTracing, this.m_ComponentTracing, this.m_TraceFile);
    localTraceConfiguration.setGlobalTracing(false);
    for (int i = 0; i < this.m_ComponentTracing.length; i++)
      localTraceConfiguration.setComponentTracing(i, false);
    try
    {
      setTraceConfiguration(localTraceConfiguration);
    }
    catch (IOException localIOException)
    {
    }
  }

  private boolean isTracingOn(int paramInt)
  {
    return (this.m_GlobalTracing) && (this.m_ComponentTracing[paramInt] != 0);
  }

  private synchronized void openFile()
    throws IOException
  {
    if (this.m_Output != null)
    {
      this.m_Output.flush();
      this.m_Output = null;
    }
    if (this.m_OutFile != null)
    {
      this.m_OutFile.close();
      this.m_OutFile = null;
    }
    if (this.m_TraceFile == "")
    {
      this.m_Output = new PrintWriter(System.out, true);
    }
    else
    {
      this.m_OutFile = new FileOutputStream(this.m_TraceFile, true);
      this.m_Output = new PrintWriter(this.m_OutFile, true);
    }
  }

  private synchronized void closeOutput()
    throws IOException
  {
    if (this.m_Output != null)
    {
      this.m_Output.flush();
      this.m_Output = null;
    }
    if (this.m_OutFile != null)
    {
      this.m_OutFile.close();
      this.m_OutFile = null;
    }
  }

  private synchronized void printlnToOutput(String paramString)
  {
    waitLock();
    this.m_Output.println(paramString);
    this.m_BeginNewLine = true;
  }

  private synchronized void printlnToOutput()
  {
    waitLock();
    this.m_Output.println();
    this.m_BeginNewLine = true;
  }

  private synchronized void printToOutput(String paramString)
  {
    waitLock();
    this.m_Output.print(paramString);
    this.m_BeginNewLine = false;
  }

  private synchronized void printStackTraceToOutput(Throwable paramThrowable)
  {
    waitLock();
    paramThrowable.printStackTrace(this.m_Output);
    this.m_Output.flush();
    this.m_BeginNewLine = true;
  }

  private synchronized void printBufferToOutput(byte[] paramArrayOfByte, int paramInt1, int paramInt2)
  {
    StringBuffer localStringBuffer1 = new StringBuffer();
    StringBuffer localStringBuffer2 = new StringBuffer();
    String str = null;
    int j = paramInt1;
    while (j < paramInt2)
    {
      localStringBuffer1.setLength(0);
      localStringBuffer2.setLength(0);
      str = toHex(j, 4);
      int i = 0;
      while ((i < 16) && (j < paramInt2))
      {
        localStringBuffer1.append(toHex(paramArrayOfByte[j], 2));
        if (i == 7)
          localStringBuffer1.append('-');
        else
          localStringBuffer1.append(' ');
        if ((paramArrayOfByte[j] < 32) || (paramArrayOfByte[j] > 126))
          localStringBuffer2.append('.');
        else
          localStringBuffer2.append((char)paramArrayOfByte[j]);
        i++;
        j++;
      }
      for (i = localStringBuffer1.length(); i < 48; i++)
        localStringBuffer1.append(' ');
      printlnToOutput(str + ": " + localStringBuffer1 + " " + localStringBuffer2);
    }
    this.m_BeginNewLine = true;
  }

  private static String toHex(int paramInt1, int paramInt2)
  {
    StringBuffer localStringBuffer = new StringBuffer();
    String str = Integer.toHexString(paramInt1);
    while (localStringBuffer.length() + str.length() < paramInt2)
      localStringBuffer.append('0');
    int i;
    if (str.length() > paramInt2)
      i = str.length() - paramInt2;
    else
      i = 0;
    while (localStringBuffer.length() < paramInt2)
      localStringBuffer.append(str.charAt(i++));
    return localStringBuffer.toString();
  }

  private String formatHeader(int paramInt)
  {
    String str = "";
    if (this.m_BeginNewLine)
    {
      try
      {
        str = this.m_SimpleDateFormat.format(new Date());
      }
      catch (ArrayIndexOutOfBoundsException localArrayIndexOutOfBoundsException)
      {
        str = "UNKNOWN TIMESTAMP: ";
      }
      return str + "(" + Thread.currentThread().getName() + ") " + TraceConstants.TRACE_COMPONENTS[paramInt].getComponentTag() + ": ";
    }
    return "";
  }

  private static StringBuffer FormatString(String paramString, Object[] paramArrayOfObject)
  {
    StringBuffer localStringBuffer = new StringBuffer();
    int i = 0;
    int j = paramString.length();
    try
    {
      int k = 0;
      while (k < j)
      {
        char c = paramString.charAt(k++);
        if (c != '%')
        {
          localStringBuffer.append(c);
        }
        else
        {
          c = paramString.charAt(k++);
          int m = 0;
          if ((c == '0') || (c == '.'))
          {
            m = 1;
            c = paramString.charAt(k++);
          }
          int n = 0;
          while (Character.isDigit(c))
          {
            n *= 10;
            n += Character.digit(c, 10);
            c = paramString.charAt(k++);
          }
          long l;
          String str1;
          switch (c)
          {
          case 'b':
          case 'c':
          case 'o':
          case 's':
            String str2 = paramArrayOfObject[(i++)].toString();
            localStringBuffer.append(str2);
            for (int i1 = str2.length(); i1 < n; i1++)
              localStringBuffer.append(' ');
            break;
          case 'X':
          case 'x':
            l = ((Number)paramArrayOfObject[(i++)]).longValue();
            str1 = LongToHex(l, 0, c == 'X');
            for (int i2 = str1.length(); i2 < n; i2++)
              localStringBuffer.append(m != 0 ? '0' : ' ');
            localStringBuffer.append(str1);
            break;
          case 'd':
            l = ((Number)paramArrayOfObject[(i++)]).longValue();
            str1 = Long.toString(l);
            if ((m != 0) && (str1.charAt(0) == '-') && (n > 0))
            {
              localStringBuffer.append('-');
              n--;
              str1 = str1.substring(1);
            }
            for (int i3 = str1.length(); i3 < n; i3++)
              localStringBuffer.append(m != 0 ? '0' : ' ');
            localStringBuffer.append(str1);
            break;
          case 'R':
          case 'r':
            byte[] arrayOfByte = (byte[])paramArrayOfObject[(i++)];
            int i4 = 0;
            while (true)
            {
              if (i4 > 0)
                localStringBuffer.append('.');
              localStringBuffer.append(LongToHex(arrayOfByte[i4] & 0xFF, 2, c == 'R'));
              i4++;
              if (i4 >= arrayOfByte.length)
                break;
            }
          }
        }
      }
    }
    catch (StringIndexOutOfBoundsException localStringIndexOutOfBoundsException)
    {
    }
    catch (ClassCastException localClassCastException)
    {
    }
    catch (ArrayIndexOutOfBoundsException localArrayIndexOutOfBoundsException)
    {
    }
    return localStringBuffer;
  }

  private static String LongToHex(long paramLong, int paramInt, boolean paramBoolean)
  {
    StringBuffer localStringBuffer = new StringBuffer();
    char[] arrayOfChar = paramBoolean ? m_UpperDigitTable : m_LowerDigitTable;
    if (paramLong == 0L)
    {
      localStringBuffer.append('0');
    }
    else
    {
      while (paramLong != 0L)
      {
        localStringBuffer.append(arrayOfChar[((int)(paramLong & 0xF))]);
        paramLong >>>= 4;
      }
      for (int i = localStringBuffer.length(); i < paramInt; i++)
        localStringBuffer.append('0');
      localStringBuffer.reverse();
    }
    return localStringBuffer.toString();
  }

  private synchronized void setLock()
  {
    waitLock();
    this.m_TransactionThread = Thread.currentThread();
  }

  private synchronized void waitLock()
  {
    if (Thread.currentThread() == this.m_TransactionThread)
      return;
    try
    {
      while (this.m_TransactionThread != null)
      {
        Date localDate1 = new Date();
        wait(5000L);
        Date localDate2 = new Date();
        if (localDate2.getTime() - localDate1.getTime() >= 5000L)
        {
          this.m_Output.println("*** Trace lock timer elapsed - lock has been cleared. ***");
          this.m_BeginNewLine = true;
          clearLock();
        }
      }
    }
    catch (InterruptedException localInterruptedException)
    {
    }
  }

  private synchronized void clearLock()
  {
    this.m_TransactionThread = null;
    notifyAll();
  }
}