package devmgr.trace;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.ItemSelectable;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.text.NumberFormat;
import java.util.EventObject;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.text.JTextComponent;

public class TraceDialog extends JDialog
{
  Trace m_Trace = Trace.getTrace();
  TraceConfiguration m_TraceConfig = null;
  JRadioButton m_RadioButtonOn = null;
  JRadioButton m_RadioButtonOff = null;
  JRadioButton m_RadioButtonConsole = null;
  JRadioButton m_RadioButtonFile = null;
  JScrollPane m_ScrollPane = null;
  JList m_ComponentList = null;
  JTextField m_TextFieldFileName = null;
  JButton m_ButtonBrowse = null;
  JButton m_ButtonOK = null;
  JButton m_ButtonCancel = null;
  TraceComponent[] m_SortedComponents = null;
  boolean m_ComponentsAdjusted = false;
  SymItem m_SymItem;
  SymAction m_SymAction;
  JLabel m_FreeVMMemory = new JLabel();
  JLabel m_TotalSystemMemoryUsed = new JLabel();
  UpdateLabels m_UpdateThread = new UpdateLabels();

  public TraceDialog(Window paramWindow)
  {
    SymWindow localSymWindow = new SymWindow();
    super.setModal(true);
    addWindowListener(localSymWindow);
    this.m_SymItem = new SymItem();
    this.m_SymAction = new SymAction();
    getRootPane().registerKeyboardAction(new EscapeListener(), KeyStroke.getKeyStroke(27, 0, false), 2);
    this.m_TraceConfig = this.m_Trace.getCurrentSettings();
    getContentPane().setLayout(new BorderLayout());
    setVisible(false);
    setResizable(true);
    setTitle("Trace Facility");
    buildUpperPanel();
    buildOutputPanel();
    buildButtonPanel();
    setSize(getContentPane().getPreferredSize());
    setTracingEnabled(this.m_TraceConfig.getGlobalTracing());
    this.m_UpdateThread.start();
  }

  public TraceDialog(Window paramWindow, String paramString, boolean paramBoolean)
  {
    this(paramWindow);
    setTitle(paramString);
  }

  public void dispose()
  {
    JRootPane localJRootPane = getRootPane();
    if (localJRootPane != null)
      localJRootPane.unregisterKeyboardAction(KeyStroke.getKeyStroke(27, 0, false));
    this.m_UpdateThread.interrupt();
    this.m_UpdateThread = null;
    super.dispose();
  }

  private void buildUpperPanel()
  {
    boolean[] arrayOfBoolean = null;
    int i = 0;
    int[] arrayOfInt = null;
    GridBagLayout localGridBagLayout = new GridBagLayout();
    JPanel localJPanel1 = new JPanel(localGridBagLayout);
    getContentPane().add("North", localJPanel1);
    JPanel localJPanel2 = new JPanel(new GridLayout(2, 1));
    localJPanel2.setBorder(new TitledBorder(new EtchedBorder(1), "Tracing"));
    localJPanel2.setPreferredSize(new Dimension(144, 84));
    localJPanel1.add(localJPanel2);
    GridBagConstraints localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.gridx = 0;
    localGridBagConstraints.gridy = 0;
    localGridBagConstraints.gridwidth = 1;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.anchor = 18;
    localGridBagConstraints.fill = 0;
    localGridBagConstraints.weightx = 0.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(12, 12, 5, 12);
    localGridBagLayout.setConstraints(localJPanel2, localGridBagConstraints);
    this.m_RadioButtonOn = new JRadioButton("On", this.m_TraceConfig.getGlobalTracing());
    this.m_RadioButtonOn.setBounds(8, 24, 48, 24);
    this.m_RadioButtonOn.addItemListener(this.m_SymItem);
    localJPanel2.add(this.m_RadioButtonOn);
    this.m_RadioButtonOff = new JRadioButton("Off", !this.m_TraceConfig.getGlobalTracing());
    this.m_RadioButtonOff.setBounds(8, 48, 48, 24);
    localJPanel2.add(this.m_RadioButtonOff);
    ButtonGroup localButtonGroup = new ButtonGroup();
    localButtonGroup.add(this.m_RadioButtonOn);
    localButtonGroup.add(this.m_RadioButtonOff);
    setMemoryLabelContents();
    localGridBagConstraints.gridy = -1;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.insets = new Insets(10, 12, 5, 9);
    localGridBagConstraints.anchor = 18;
    localGridBagConstraints.fill = 2;
    localJPanel1.add(this.m_FreeVMMemory, localGridBagConstraints);
    localGridBagConstraints.insets = new Insets(1, 12, 5, 9);
    localJPanel1.add(this.m_TotalSystemMemoryUsed, localGridBagConstraints);
    this.m_SortedComponents = sortComponents(TraceConstants.TRACE_COMPONENTS);
    this.m_ComponentList = new JList(this.m_SortedComponents);
    this.m_ComponentList.setFont(new Font("Dialog", 0, 12));
    this.m_ScrollPane = new JScrollPane(this.m_ComponentList);
    this.m_ScrollPane.setPreferredSize(new Dimension(268, 268));
    this.m_ScrollPane.setBorder(new TitledBorder(new EtchedBorder(1), "Components"));
    this.m_ScrollPane.setViewportBorder(new EtchedBorder(1));
    localJPanel1.add(this.m_ScrollPane);
    localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.anchor = 18;
    localGridBagConstraints.gridx = 1;
    localGridBagConstraints.gridy = 0;
    localGridBagConstraints.gridwidth = 0;
    localGridBagConstraints.gridheight = 3;
    localGridBagConstraints.fill = 1;
    localGridBagConstraints.weightx = 1.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(12, 0, 5, 12);
    localGridBagLayout.setConstraints(this.m_ScrollPane, localGridBagConstraints);
    arrayOfBoolean = this.m_TraceConfig.getComponentTracing();
    for (int j = 0; j < arrayOfBoolean.length; j++)
      if (arrayOfBoolean[j] != 0)
        i++;
    arrayOfInt = new int[i];
    int k = 0;
    int m = 0;
    while (k < this.m_SortedComponents.length)
    {
      if (arrayOfBoolean[this.m_SortedComponents[k].getComponentID()] != 0)
        arrayOfInt[(m++)] = k;
      k++;
    }
    this.m_ComponentList.setSelectedIndices(arrayOfInt);
  }

  private void buildOutputPanel()
  {
    String str = "";
    JPanel localJPanel1 = new JPanel(new FlowLayout(0, 12, 0));
    getContentPane().add("Center", localJPanel1);
    GridBagLayout localGridBagLayout = new GridBagLayout();
    JPanel localJPanel2 = new JPanel(localGridBagLayout);
    localJPanel2.setBorder(new TitledBorder(new EtchedBorder(1), "Output"));
    localJPanel1.add(localJPanel2);
    str = this.m_TraceConfig.getTraceFile();
    boolean bool;
    if (str == "")
      bool = false;
    else
      bool = true;
    this.m_RadioButtonConsole = new JRadioButton("Console", !bool);
    localJPanel2.add(this.m_RadioButtonConsole);
    GridBagConstraints localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.gridx = 0;
    localGridBagConstraints.gridy = 0;
    localGridBagConstraints.gridwidth = 0;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.anchor = 17;
    localGridBagConstraints.fill = 0;
    localGridBagConstraints.weightx = 0.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(0, 0, 0, 0);
    localGridBagLayout.setConstraints(this.m_RadioButtonConsole, localGridBagConstraints);
    this.m_RadioButtonFile = new JRadioButton("File", bool);
    this.m_RadioButtonFile.addItemListener(this.m_SymItem);
    localJPanel2.add(this.m_RadioButtonFile);
    localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.gridx = 0;
    localGridBagConstraints.gridy = 1;
    localGridBagConstraints.gridwidth = 1;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.anchor = 17;
    localGridBagConstraints.fill = 0;
    localGridBagConstraints.weightx = 0.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(0, 0, 0, 0);
    localGridBagLayout.setConstraints(this.m_RadioButtonFile, localGridBagConstraints);
    ButtonGroup localButtonGroup = new ButtonGroup();
    localButtonGroup.add(this.m_RadioButtonConsole);
    localButtonGroup.add(this.m_RadioButtonFile);
    this.m_TextFieldFileName = new JTextField();
    this.m_TextFieldFileName.setEnabled(bool);
    this.m_TextFieldFileName.setText(str);
    this.m_TextFieldFileName.addActionListener(this.m_SymAction);
    this.m_TextFieldFileName.setPreferredSize(new Dimension(408, 24));
    localJPanel2.add(this.m_TextFieldFileName);
    localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.gridx = 0;
    localGridBagConstraints.gridy = 2;
    localGridBagConstraints.gridwidth = 0;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.anchor = 17;
    localGridBagConstraints.fill = 2;
    localGridBagConstraints.weightx = 100.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(0, 12, 0, 12);
    localGridBagLayout.setConstraints(this.m_TextFieldFileName, localGridBagConstraints);
    JPanel localJPanel3 = new JPanel(new FlowLayout(2, 20, 20));
    localJPanel2.add(localJPanel3);
    localGridBagConstraints = new GridBagConstraints();
    localGridBagConstraints.gridx = 0;
    localGridBagConstraints.gridy = 3;
    localGridBagConstraints.gridwidth = 0;
    localGridBagConstraints.gridheight = 1;
    localGridBagConstraints.anchor = 17;
    localGridBagConstraints.fill = 2;
    localGridBagConstraints.weightx = 0.0D;
    localGridBagConstraints.weighty = 0.0D;
    localGridBagConstraints.insets = new Insets(0, 0, 0, 0);
    localGridBagLayout.setConstraints(localJPanel3, localGridBagConstraints);
    this.m_ButtonBrowse = new JButton();
    this.m_ButtonBrowse.setText("Browse");
    this.m_ButtonBrowse.setEnabled(bool);
    this.m_ButtonBrowse.addActionListener(this.m_SymAction);
    localJPanel3.add(this.m_ButtonBrowse);
  }

  private void setMemoryLabelContents()
  {
    Runtime localRuntime = Runtime.getRuntime();
    this.m_FreeVMMemory.setText("VM free memory: " + NumberFormat.getInstance().format(localRuntime.freeMemory()));
    this.m_TotalSystemMemoryUsed.setText("Total VM memory allocated from OS: " + NumberFormat.getInstance().format(localRuntime.totalMemory()));
  }

  private void buildButtonPanel()
  {
    JPanel localJPanel = new JPanel(new FlowLayout(1, 20, 12));
    getContentPane().add("South", localJPanel);
    this.m_ButtonOK = new JButton();
    this.m_ButtonOK.setText("OK");
    this.m_ButtonOK.addActionListener(this.m_SymAction);
    localJPanel.add(this.m_ButtonOK);
    this.m_ButtonCancel = new JButton();
    this.m_ButtonCancel.setText("Cancel");
    this.m_ButtonCancel.addActionListener(this.m_SymAction);
    localJPanel.add(this.m_ButtonCancel);
  }

  public void addNotify()
  {
    Dimension localDimension = getSize();
    super.addNotify();
    if (this.m_ComponentsAdjusted)
      return;
    setSize(getInsets().left + getInsets().right + localDimension.width, getInsets().top + getInsets().bottom + localDimension.height);
    Component[] arrayOfComponent = getContentPane().getComponents();
    for (int i = 0; i < arrayOfComponent.length; i++)
    {
      Point localPoint = arrayOfComponent[i].getLocation();
      localPoint.translate(getInsets().left, getInsets().top);
      arrayOfComponent[i].setLocation(localPoint);
    }
    this.m_ComponentsAdjusted = true;
  }

  void setTracingEnabled(boolean paramBoolean)
  {
    this.m_ScrollPane.setEnabled(paramBoolean);
    this.m_ComponentList.setEnabled(paramBoolean);
    this.m_RadioButtonConsole.setEnabled(paramBoolean);
    this.m_RadioButtonFile.setEnabled(paramBoolean);
    if ((paramBoolean == true) && (this.m_RadioButtonFile.isSelected()))
    {
      this.m_TextFieldFileName.setEnabled(true);
      this.m_ButtonBrowse.setEnabled(true);
    }
    else
    {
      this.m_TextFieldFileName.setEnabled(false);
      this.m_ButtonBrowse.setEnabled(false);
    }
  }

  TraceComponent[] sortComponents(TraceComponent[] paramArrayOfTraceComponent)
  {
    Object localObject = null;
    TraceComponent[] arrayOfTraceComponent = (TraceComponent[])paramArrayOfTraceComponent.clone();
    for (int i = 1; i < arrayOfTraceComponent.length; i++)
      for (int j = 0; j < arrayOfTraceComponent.length - 1; j++)
        if (arrayOfTraceComponent[j].compareTo(arrayOfTraceComponent[(j + 1)]) > 0)
        {
          localObject = arrayOfTraceComponent[j];
          arrayOfTraceComponent[j] = arrayOfTraceComponent[(j + 1)];
          arrayOfTraceComponent[(j + 1)] = localObject;
        }
    return arrayOfTraceComponent;
  }

  void setConfiguration()
  {
    int i = 0;
    for (int j = 0; j < this.m_SortedComponents.length; j++)
    {
      boolean bool = this.m_ComponentList.isSelectedIndex(j);
      this.m_TraceConfig.setComponentTracing(this.m_SortedComponents[j].getComponentID(), bool);
      if (bool)
        i = 1;
    }
    if (this.m_TraceConfig.getGlobalTracing() == true)
    {
      if (i == 0)
      {
        new TraceMessageBox(this, "Error", "At least one component must have tracing selected.").setVisible(true);
        return;
      }
      String str;
      if (this.m_RadioButtonFile.isSelected() == true)
      {
        str = this.m_TextFieldFileName.getText();
        if (str.equals(""))
          new TraceMessageBox(this, "Error", "Filename is not specified.").setVisible(true);
      }
      else
      {
        str = "";
      }
      this.m_TraceConfig.setTraceFile(str);
    }
    try
    {
      this.m_Trace.setTraceConfiguration(this.m_TraceConfig);
    }
    catch (IOException localIOException)
    {
      new TraceMessageBox(this, "Error", "I/O Error occurred: " + localIOException.toString()).setVisible(true);
    }
    dispatchEvent(new WindowEvent(this, 201));
  }

  class UpdateLabels extends Thread
  {
    UpdateLabels()
    {
    }

    public void run()
    {
      while (true)
      {
        try
        {
          Thread.sleep(2000L);
        }
        catch (InterruptedException localInterruptedException)
        {
          return;
        }
        SwingUtilities.invokeLater(new TraceDialog.1(this));
      }
    }
  }

  class SymItem
    implements ItemListener
  {
    SymItem()
    {
    }

    public void itemStateChanged(ItemEvent paramItemEvent)
    {
      ItemSelectable localItemSelectable = paramItemEvent.getItemSelectable();
      boolean bool;
      if (paramItemEvent.getStateChange() == 1)
        bool = true;
      else
        bool = false;
      if (localItemSelectable == TraceDialog.this.m_RadioButtonOn)
      {
        TraceDialog.this.m_TraceConfig.setGlobalTracing(bool);
        TraceDialog.this.setTracingEnabled(bool);
      }
      else if (localItemSelectable == TraceDialog.this.m_RadioButtonFile)
      {
        TraceDialog.this.m_ButtonBrowse.setEnabled(bool);
        TraceDialog.this.m_TextFieldFileName.setEnabled(bool);
      }
    }
  }

  class EscapeListener
    implements ActionListener
  {
    EscapeListener()
    {
    }

    public void actionPerformed(ActionEvent paramActionEvent)
    {
      TraceDialog.this.dispatchEvent(new WindowEvent(TraceDialog.this, 201));
    }
  }

  class SymAction
    implements ActionListener
  {
    SymAction()
    {
    }

    public void actionPerformed(ActionEvent paramActionEvent)
    {
      JFileChooser localJFileChooser = null;
      Object localObject = paramActionEvent.getSource();
      if (localObject == TraceDialog.this.m_ButtonCancel)
      {
        TraceDialog.this.dispatchEvent(new WindowEvent(TraceDialog.this, 201));
      }
      else if (localObject == TraceDialog.this.m_TextFieldFileName)
      {
        TraceDialog.this.m_TraceConfig.setTraceFile(TraceDialog.this.m_TextFieldFileName.getText());
        TraceDialog.this.m_ButtonOK.requestFocus();
      }
      else if (localObject == TraceDialog.this.m_ButtonBrowse)
      {
        localJFileChooser = new JFileChooser();
        int i = localJFileChooser.showDialog(TraceDialog.this, "OK");
        if (i == 0)
        {
          String str = localJFileChooser.getSelectedFile().getAbsolutePath();
          TraceDialog.this.m_TextFieldFileName.setText(str);
          TraceDialog.this.m_ButtonOK.requestFocus();
        }
      }
      else if (localObject == TraceDialog.this.m_ButtonOK)
      {
        TraceDialog.this.setConfiguration();
      }
    }
  }

  class SymWindow extends WindowAdapter
  {
    SymWindow()
    {
    }

    public void windowClosing(WindowEvent paramWindowEvent)
    {
      Object localObject = paramWindowEvent.getSource();
      if (localObject == TraceDialog.this)
      {
        TraceDialog.this.setVisible(false);
        TraceDialog.this.dispose();
      }
    }
  }
}