package devmgr.versioned.jrpc;

public class XDROutputStream
{
  private static final int XDR_UNIT_SIZE = 4;
  private static final int XDR_BUFFER_SIZE = 1024;
  private XDRMessageIOV m_Msg = new XDRMessageIOV();
  private byte[] m_Buf = new byte[1024];
  private int m_Pos = 0;
  private int m_BufNum = 0;
  private boolean m_Growing = true;

  public XDRMessageIOV getMessage()
  {
    finishMessage();
    return this.m_Msg;
  }

  public void putBoolean(boolean paramBoolean)
  {
    putByte(0);
    putByte(0);
    putByte(0);
    putByte(paramBoolean ? 1 : 0);
  }

  public void putChar(char paramChar)
  {
    putByte(0);
    putByte(0);
    putByte(0);
    putByte(paramChar);
  }

  public void putShort(short paramShort)
  {
    putByte(0);
    putByte(0);
    putByte(paramShort >>> 8);
    putByte(paramShort >>> 0);
  }

  public void putInt(int paramInt)
  {
    putByte(paramInt >>> 24);
    putByte(paramInt >>> 16);
    putByte(paramInt >>> 8);
    putByte(paramInt >>> 0);
  }

  public void putLong(long paramLong)
  {
    putByte((int)(paramLong >>> 56));
    putByte((int)(paramLong >>> 48));
    putByte((int)(paramLong >>> 40));
    putByte((int)(paramLong >>> 32));
    putByte((int)(paramLong >>> 24));
    putByte((int)(paramLong >>> 16));
    putByte((int)(paramLong >>> 8));
    putByte((int)(paramLong >>> 0));
  }

  public void putFloat(float paramFloat)
  {
    putInt(Float.floatToIntBits(paramFloat));
  }

  public void putDouble(double paramDouble)
  {
    putLong(Double.doubleToLongBits(paramDouble));
  }

  public void putString(String paramString)
  {
    int i = paramString.length();
    putInt(i);
    int j = calculatePad(i);
    for (int k = 0; k < i; k++)
      putByte(paramString.charAt(k));
    for (k = 0; k < j; k++)
      putByte(0);
  }

  public void putString(String paramString, int paramInt)
    throws RPCError
  {
    if (paramString.length() > paramInt)
      throw new RPCError("XDR_MAX_LENGTH");
    putString(paramString);
  }

  public void putVariableOpaque(byte[] paramArrayOfByte)
    throws RPCError
  {
    int i;
    if (paramArrayOfByte == null)
      i = 0;
    else
      i = paramArrayOfByte.length;
    putInt(i);
    if (i > 0)
      putFixedOpaque(paramArrayOfByte, i);
  }

  public void putVariableOpaque(byte[] paramArrayOfByte, int paramInt)
    throws RPCError
  {
    if (paramArrayOfByte.length > paramInt)
      throw new RPCError("XDR_MAX_LENGTH");
    putVariableOpaque(paramArrayOfByte);
  }

  public void putFixedOpaque(byte[] paramArrayOfByte, int paramInt)
    throws RPCError
  {
    int k = paramArrayOfByte.length;
    if (k != paramInt)
      throw new RPCError("XDR_MAX_LENGTH");
    int i = calculatePad(k);
    for (int j = 0; j < k; j++)
      putByte(paramArrayOfByte[j]);
    for (j = 0; j < i; j++)
      putByte(0);
  }

  public int getPosition()
  {
    return this.m_Msg.getMessageSize() + this.m_Pos;
  }

  public void setPosition(int paramInt)
  {
    finishMessage();
    this.m_BufNum = (paramInt / 1024);
    this.m_Pos = (paramInt % 1024);
    this.m_Buf = this.m_Msg.getBuffer(this.m_BufNum);
  }

  public int prepareLength()
  {
    putInt(0);
    return getPosition();
  }

  public void setLength(int paramInt)
  {
    int k = getPosition() - paramInt;
    int i = (paramInt - 4) / 1024;
    int j = (paramInt - 4) % 1024;
    byte[] arrayOfByte;
    if (i == this.m_BufNum)
      arrayOfByte = this.m_Buf;
    else
      arrayOfByte = this.m_Msg.getBuffer(i);
    arrayOfByte[(j + 0)] = ((byte)(k >>> 24));
    arrayOfByte[(j + 1)] = ((byte)(k >>> 16));
    arrayOfByte[(j + 2)] = ((byte)(k >>> 8));
    arrayOfByte[(j + 3)] = ((byte)(k >>> 0));
  }

  private void putByte(int paramInt)
  {
    if (this.m_Pos >= 1024)
      if (this.m_Growing)
      {
        this.m_Msg.addFullBuffer(this.m_Buf);
        this.m_BufNum += 1;
        this.m_Pos = 0;
        this.m_Buf = new byte[1024];
      }
      else
      {
        this.m_BufNum += 1;
        this.m_Pos = 0;
        this.m_Buf = this.m_Msg.getBuffer(this.m_BufNum);
      }
    this.m_Buf[(this.m_Pos++)] = ((byte)paramInt);
  }

  private int calculatePad(int paramInt)
  {
    int i = 0;
    int j;
    if ((j = paramInt % 4) != 0)
      i = 4 - j;
    return i;
  }

  private void finishMessage()
  {
    if ((this.m_Growing) && (this.m_Pos > 0))
    {
      this.m_Msg.addLastBuffer(this.m_Buf, this.m_Pos);
      this.m_Buf = null;
      this.m_Growing = false;
    }
  }
}