package org.eclipse.jdt.internal.compiler.ast;

import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.flow.FlowContext;
import org.eclipse.jdt.internal.compiler.flow.FlowInfo;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.Constant;
import org.eclipse.jdt.internal.compiler.lookup.ArrayBinding;
import org.eclipse.jdt.internal.compiler.lookup.BaseTypeBinding;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.LookupEnvironment;
import org.eclipse.jdt.internal.compiler.lookup.ProblemReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;
import org.eclipse.jdt.internal.compiler.lookup.Scope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;

public abstract class TypeReference extends Expression
{
  public FlowInfo analyseCode(BlockScope currentScope, FlowContext flowContext, FlowInfo flowInfo)
  {
    return flowInfo;
  }

  public void aboutToResolve(Scope scope)
  {
  }

  public static final TypeReference baseTypeReference(int baseType, int dim)
  {
    if (dim == 0) {
      switch (baseType) {
      case 6:
        return new SingleTypeReference(TypeBinding.VOID.simpleName, 0L);
      case 5:
        return new SingleTypeReference(TypeBinding.BOOLEAN.simpleName, 0L);
      case 2:
        return new SingleTypeReference(TypeBinding.CHAR.simpleName, 0L);
      case 9:
        return new SingleTypeReference(TypeBinding.FLOAT.simpleName, 0L);
      case 8:
        return new SingleTypeReference(TypeBinding.DOUBLE.simpleName, 0L);
      case 3:
        return new SingleTypeReference(TypeBinding.BYTE.simpleName, 0L);
      case 4:
        return new SingleTypeReference(TypeBinding.SHORT.simpleName, 0L);
      case 10:
        return new SingleTypeReference(TypeBinding.INT.simpleName, 0L);
      case 7:
      }return new SingleTypeReference(TypeBinding.LONG.simpleName, 0L);
    }

    switch (baseType) {
    case 6:
      return new ArrayTypeReference(TypeBinding.VOID.simpleName, dim, 0L);
    case 5:
      return new ArrayTypeReference(TypeBinding.BOOLEAN.simpleName, dim, 0L);
    case 2:
      return new ArrayTypeReference(TypeBinding.CHAR.simpleName, dim, 0L);
    case 9:
      return new ArrayTypeReference(TypeBinding.FLOAT.simpleName, dim, 0L);
    case 8:
      return new ArrayTypeReference(TypeBinding.DOUBLE.simpleName, dim, 0L);
    case 3:
      return new ArrayTypeReference(TypeBinding.BYTE.simpleName, dim, 0L);
    case 4:
      return new ArrayTypeReference(TypeBinding.SHORT.simpleName, dim, 0L);
    case 10:
      return new ArrayTypeReference(TypeBinding.INT.simpleName, dim, 0L);
    case 7:
    }return new ArrayTypeReference(TypeBinding.LONG.simpleName, dim, 0L);
  }
  public void checkBounds(Scope scope) {
  }

  public abstract TypeReference copyDims(int paramInt);

  public int dimensions() {
    return 0;
  }

  public abstract char[] getLastToken();

  public char[][] getParameterizedTypeName()
  {
    return getTypeName();
  }

  protected abstract TypeBinding getTypeBinding(Scope paramScope);

  public abstract char[][] getTypeName();

  public boolean isTypeReference() {
    return true;
  }

  public TypeBinding resolveSuperType(ClassScope scope) {
    if (resolveType(scope) == null) return null;

    if (this.resolvedType.isTypeVariable()) {
      this.resolvedType = new ProblemReferenceBinding(getTypeName(), (ReferenceBinding)this.resolvedType, 9);
      reportInvalidType(scope);
      return null;
    }
    return this.resolvedType;
  }

  public final TypeBinding resolveType(BlockScope blockScope) {
    return resolveType(blockScope, true);
  }

  public TypeBinding resolveType(BlockScope scope, boolean checkBounds)
  {
    this.constant = Constant.NotAConstant;
    if (this.resolvedType != null) {
      return this.resolvedType.isValidBinding() ? this.resolvedType : null;
    }
    TypeBinding type = this.resolvedType = getTypeBinding(scope);
    if (type == null)
      return null;
    if (!type.isValidBinding()) {
      reportInvalidType(scope);
      return null;
    }
    if ((type.isArrayType()) && (((ArrayBinding)type).leafComponentType == TypeBinding.VOID)) {
      scope.problemReporter().cannotAllocateVoidArray(this);
      return null;
    }

    if (isTypeUseDeprecated(type, scope)) {
      reportDeprecatedType(type, scope);
    }
    type = scope.environment().convertToRawType(type);
    if ((type.leafComponentType().isRawType()) && 
      ((this.bits & 0x40000000) == 0) && 
      (scope.compilerOptions().getSeverity(35184372088832L) != -1)) {
      scope.problemReporter().rawTypeReference(this, type);
    }
    return this.resolvedType = type;
  }

  public TypeBinding resolveType(ClassScope scope) {
    this.constant = Constant.NotAConstant;
    if (this.resolvedType != null) {
      return this.resolvedType.isValidBinding() ? this.resolvedType : null;
    }
    TypeBinding type = this.resolvedType = getTypeBinding(scope);
    if (type == null)
      return null;
    if (!type.isValidBinding()) {
      reportInvalidType(scope);
      return null;
    }
    if ((type.isArrayType()) && (((ArrayBinding)type).leafComponentType == TypeBinding.VOID)) {
      scope.problemReporter().cannotAllocateVoidArray(this);
      return null;
    }
    if (isTypeUseDeprecated(type, scope)) {
      reportDeprecatedType(type, scope);
    }
    type = scope.environment().convertToRawType(type);
    if ((type.leafComponentType().isRawType()) && 
      ((this.bits & 0x40000000) == 0) && 
      (scope.compilerOptions().getSeverity(35184372088832L) != -1)) {
      scope.problemReporter().rawTypeReference(this, type);
    }
    return this.resolvedType = type;
  }

  public TypeBinding resolveTypeArgument(BlockScope blockScope, ReferenceBinding genericType, int rank) {
    return resolveType(blockScope, true);
  }

  public TypeBinding resolveTypeArgument(ClassScope classScope, ReferenceBinding genericType, int rank) {
    return resolveType(classScope);
  }

  protected void reportInvalidType(Scope scope) {
    scope.problemReporter().invalidType(this, this.resolvedType);
  }
  protected void reportDeprecatedType(TypeBinding type, Scope scope) {
    scope.problemReporter().deprecatedType(type, this);
  }

  public abstract void traverse(ASTVisitor paramASTVisitor, BlockScope paramBlockScope);

  public abstract void traverse(ASTVisitor paramASTVisitor, ClassScope paramClassScope);
}