package org.eclipse.jdt.internal.compiler.parser;

import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.core.compiler.InvalidInputException;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.CompilationResult;
import org.eclipse.jdt.internal.compiler.ast.AND_AND_Expression;
import org.eclipse.jdt.internal.compiler.ast.ASTNode;
import org.eclipse.jdt.internal.compiler.ast.AbstractMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AbstractVariableDeclaration;
import org.eclipse.jdt.internal.compiler.ast.AllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.Annotation;
import org.eclipse.jdt.internal.compiler.ast.AnnotationMethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.Argument;
import org.eclipse.jdt.internal.compiler.ast.ArrayAllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.ArrayInitializer;
import org.eclipse.jdt.internal.compiler.ast.ArrayQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.ArrayReference;
import org.eclipse.jdt.internal.compiler.ast.ArrayTypeReference;
import org.eclipse.jdt.internal.compiler.ast.AssertStatement;
import org.eclipse.jdt.internal.compiler.ast.Assignment;
import org.eclipse.jdt.internal.compiler.ast.BinaryExpression;
import org.eclipse.jdt.internal.compiler.ast.Block;
import org.eclipse.jdt.internal.compiler.ast.BreakStatement;
import org.eclipse.jdt.internal.compiler.ast.CaseStatement;
import org.eclipse.jdt.internal.compiler.ast.CastExpression;
import org.eclipse.jdt.internal.compiler.ast.CharLiteral;
import org.eclipse.jdt.internal.compiler.ast.ClassLiteralAccess;
import org.eclipse.jdt.internal.compiler.ast.CombinedBinaryExpression;
import org.eclipse.jdt.internal.compiler.ast.CompilationUnitDeclaration;
import org.eclipse.jdt.internal.compiler.ast.CompoundAssignment;
import org.eclipse.jdt.internal.compiler.ast.ConditionalExpression;
import org.eclipse.jdt.internal.compiler.ast.ConstructorDeclaration;
import org.eclipse.jdt.internal.compiler.ast.ContinueStatement;
import org.eclipse.jdt.internal.compiler.ast.DoStatement;
import org.eclipse.jdt.internal.compiler.ast.DoubleLiteral;
import org.eclipse.jdt.internal.compiler.ast.EmptyStatement;
import org.eclipse.jdt.internal.compiler.ast.EqualExpression;
import org.eclipse.jdt.internal.compiler.ast.ExplicitConstructorCall;
import org.eclipse.jdt.internal.compiler.ast.Expression;
import org.eclipse.jdt.internal.compiler.ast.FalseLiteral;
import org.eclipse.jdt.internal.compiler.ast.FieldDeclaration;
import org.eclipse.jdt.internal.compiler.ast.FieldReference;
import org.eclipse.jdt.internal.compiler.ast.FloatLiteral;
import org.eclipse.jdt.internal.compiler.ast.ForStatement;
import org.eclipse.jdt.internal.compiler.ast.ForeachStatement;
import org.eclipse.jdt.internal.compiler.ast.IfStatement;
import org.eclipse.jdt.internal.compiler.ast.ImportReference;
import org.eclipse.jdt.internal.compiler.ast.Initializer;
import org.eclipse.jdt.internal.compiler.ast.InstanceOfExpression;
import org.eclipse.jdt.internal.compiler.ast.IntLiteral;
import org.eclipse.jdt.internal.compiler.ast.IntLiteralMinValue;
import org.eclipse.jdt.internal.compiler.ast.Javadoc;
import org.eclipse.jdt.internal.compiler.ast.LabeledStatement;
import org.eclipse.jdt.internal.compiler.ast.LocalDeclaration;
import org.eclipse.jdt.internal.compiler.ast.LongLiteral;
import org.eclipse.jdt.internal.compiler.ast.LongLiteralMinValue;
import org.eclipse.jdt.internal.compiler.ast.MarkerAnnotation;
import org.eclipse.jdt.internal.compiler.ast.MemberValuePair;
import org.eclipse.jdt.internal.compiler.ast.MessageSend;
import org.eclipse.jdt.internal.compiler.ast.MethodDeclaration;
import org.eclipse.jdt.internal.compiler.ast.NameReference;
import org.eclipse.jdt.internal.compiler.ast.NormalAnnotation;
import org.eclipse.jdt.internal.compiler.ast.NullLiteral;
import org.eclipse.jdt.internal.compiler.ast.OR_OR_Expression;
import org.eclipse.jdt.internal.compiler.ast.OperatorIds;
import org.eclipse.jdt.internal.compiler.ast.ParameterizedQualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.ParameterizedSingleTypeReference;
import org.eclipse.jdt.internal.compiler.ast.PostfixExpression;
import org.eclipse.jdt.internal.compiler.ast.PrefixExpression;
import org.eclipse.jdt.internal.compiler.ast.QualifiedAllocationExpression;
import org.eclipse.jdt.internal.compiler.ast.QualifiedNameReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedSuperReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedThisReference;
import org.eclipse.jdt.internal.compiler.ast.QualifiedTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Reference;
import org.eclipse.jdt.internal.compiler.ast.ReturnStatement;
import org.eclipse.jdt.internal.compiler.ast.SingleMemberAnnotation;
import org.eclipse.jdt.internal.compiler.ast.SingleNameReference;
import org.eclipse.jdt.internal.compiler.ast.SingleTypeReference;
import org.eclipse.jdt.internal.compiler.ast.Statement;
import org.eclipse.jdt.internal.compiler.ast.StringLiteral;
import org.eclipse.jdt.internal.compiler.ast.SuperReference;
import org.eclipse.jdt.internal.compiler.ast.SwitchStatement;
import org.eclipse.jdt.internal.compiler.ast.SynchronizedStatement;
import org.eclipse.jdt.internal.compiler.ast.ThisReference;
import org.eclipse.jdt.internal.compiler.ast.ThrowStatement;
import org.eclipse.jdt.internal.compiler.ast.TrueLiteral;
import org.eclipse.jdt.internal.compiler.ast.TryStatement;
import org.eclipse.jdt.internal.compiler.ast.TypeDeclaration;
import org.eclipse.jdt.internal.compiler.ast.TypeParameter;
import org.eclipse.jdt.internal.compiler.ast.TypeReference;
import org.eclipse.jdt.internal.compiler.ast.UnaryExpression;
import org.eclipse.jdt.internal.compiler.ast.WhileStatement;
import org.eclipse.jdt.internal.compiler.ast.Wildcard;
import org.eclipse.jdt.internal.compiler.env.ICompilationUnit;
import org.eclipse.jdt.internal.compiler.impl.CompilerOptions;
import org.eclipse.jdt.internal.compiler.impl.ReferenceContext;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.ClassScope;
import org.eclipse.jdt.internal.compiler.lookup.MethodScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeConstants;
import org.eclipse.jdt.internal.compiler.lookup.TypeIds;
import org.eclipse.jdt.internal.compiler.parser.diagnose.DiagnoseParser;
import org.eclipse.jdt.internal.compiler.parser.diagnose.RangeUtil;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilation;
import org.eclipse.jdt.internal.compiler.problem.AbortCompilationUnit;
import org.eclipse.jdt.internal.compiler.problem.ProblemReporter;
import org.eclipse.jdt.internal.compiler.util.Messages;
import org.eclipse.jdt.internal.compiler.util.Util;

public class Parser
  implements ParserBasicInformation, TerminalTokens, OperatorIds, TypeIds
{
  protected static final int THIS_CALL = 3;
  protected static final int SUPER_CALL = 2;
  public static char[] asb = null;
  public static char[] asr = null;
  protected static final int AstStackIncrement = 100;
  public static char[] base_action = null;
  public static final int BracketKinds = 3;
  public static short[] check_table = null;
  public static final int CurlyBracket = 2;
  private static final boolean DEBUG = false;
  private static final boolean DEBUG_AUTOMATON = false;
  private static final String EOF_TOKEN = "$eof";
  private static final String ERROR_TOKEN = "$error";
  protected static final int ExpressionStackIncrement = 100;
  protected static final int GenericsStackIncrement = 10;
  private static final String FILEPREFIX = "parser";
  public static char[] in_symb = null;
  private static final String INVALID_CHARACTER = "Invalid Character";
  public static char[] lhs = null;

  public static String[] name = null;
  public static char[] nasb = null;
  public static char[] nasr = null;
  public static char[] non_terminal_index = null;
  private static final String READABLE_NAMES_FILE = "readableNames";
  private static final String READABLE_NAMES_FILE_NAME = "org.eclipse.jdt.internal.compiler.parser.readableNames";
  public static String[] readableName = null;

  public static byte[] rhs = null;

  public static int[] reverse_index = null;
  public static char[] recovery_templates_index = null;
  public static char[] recovery_templates = null;
  public static char[] statements_recovery_filter = null;

  public static long[] rules_compliance = null;
  public static final int RoundBracket = 0;
  public static byte[] scope_la = null;
  public static char[] scope_lhs = null;

  public static char[] scope_prefix = null;
  public static char[] scope_rhs = null;
  public static char[] scope_state = null;

  public static char[] scope_state_set = null;
  public static char[] scope_suffix = null;
  public static final int SquareBracket = 1;
  protected static final int StackIncrement = 255;
  public static char[] term_action = null;
  public static byte[] term_check = null;

  public static char[] terminal_index = null;
  private static final String UNEXPECTED_EOF = "Unexpected End Of File";
  public static boolean VERBOSE_RECOVERY = false;
  protected int astLengthPtr;
  protected int[] astLengthStack;
  protected int astPtr;
  protected ASTNode[] astStack = new ASTNode[100];
  public CompilationUnitDeclaration compilationUnit;
  protected RecoveredElement currentElement;
  public int currentToken;
  protected boolean diet = false;
  protected int dietInt = 0;
  protected int endPosition;
  protected int endStatementPosition;
  protected int expressionLengthPtr;
  protected int[] expressionLengthStack;
  protected int expressionPtr;
  protected Expression[] expressionStack = new Expression[100];
  public int firstToken;
  protected int genericsIdentifiersLengthPtr;
  protected int[] genericsIdentifiersLengthStack = new int[10];
  protected int genericsLengthPtr;
  protected int[] genericsLengthStack = new int[10];
  protected int genericsPtr;
  protected ASTNode[] genericsStack = new ASTNode[10];
  protected boolean hasError;
  protected boolean hasReportedError;
  protected int identifierLengthPtr;
  protected int[] identifierLengthStack;
  protected long[] identifierPositionStack;
  protected int identifierPtr;
  protected char[][] identifierStack;
  protected boolean ignoreNextOpeningBrace;
  protected int intPtr;
  protected int[] intStack;
  public int lastAct;
  protected int lastCheckPoint;
  protected int lastErrorEndPosition;
  protected int lastErrorEndPositionBeforeRecovery = -1;
  protected int lastIgnoredToken;
  protected int nextIgnoredToken;
  protected int listLength;
  protected int listTypeParameterLength;
  protected int lParenPos;
  protected int rParenPos;
  protected int modifiers;
  protected int modifiersSourceStart;
  protected int[] nestedMethod;
  protected int nestedType;
  protected int dimensions;
  ASTNode[] noAstNodes = new ASTNode[100];
  Expression[] noExpressions = new Expression[100];

  protected boolean optimizeStringLiterals = true;
  protected CompilerOptions options;
  protected ProblemReporter problemReporter;
  protected int rBraceStart;
  protected int rBraceEnd;
  protected int rBraceSuccessorStart;
  protected int realBlockPtr;
  protected int[] realBlockStack;
  protected int recoveredStaticInitializerStart;
  public ReferenceContext referenceContext;
  public boolean reportOnlyOneSyntaxError = false;
  public boolean reportSyntaxErrorIsRequired = true;
  protected boolean restartRecovery;
  public boolean methodRecoveryActivated = false;
  protected boolean statementRecoveryActivated = false;
  protected TypeDeclaration[] recoveredTypes;
  protected int recoveredTypePtr;
  protected int nextTypeStart;
  protected TypeDeclaration pendingRecoveredType;
  public RecoveryScanner recoveryScanner;
  public Scanner scanner;
  protected int[] stack = new int['ÿ'];
  protected int stateStackTop;
  protected int synchronizedBlockSourceStart;
  protected int[] variablesCounter;
  protected boolean checkExternalizeStrings;
  protected boolean recordStringLiterals;
  public Javadoc javadoc;
  public JavadocParser javadocParser;
  protected int lastJavadocEnd;

  static
  {
    try
    {
      initTables();
    } catch (IOException ex) {
      throw new ExceptionInInitializerError(ex.getMessage());
    }
  }

  public static int asi(int state) {
    return asb[original_state(state)];
  }
  public static final short base_check(int i) {
    return check_table[(i - 696)];
  }
  private static final void buildFile(String filename, List listToDump) {
    BufferedWriter writer = null;
    try {
      writer = new BufferedWriter(new FileWriter(filename));
      for (Iterator iterator = listToDump.iterator(); iterator.hasNext(); ) {
        writer.write(String.valueOf(iterator.next()));
      }
      writer.flush();
    }
    catch (IOException localIOException1)
    {
      if (writer != null)
        try {
          writer.close();
        }
        catch (IOException localIOException2)
        {
        }
    }
    finally
    {
      if (writer != null)
        try {
          writer.close();
        }
        catch (IOException localIOException3)
        {
        }
    }
    System.out.println(filename + " creation complete");
  }
  private static final String[] buildFileForName(String filename, String contents) {
    String[] result = new String[contents.length()];
    result[0] = null;
    int resultCount = 1;

    StringBuffer buffer = new StringBuffer();

    int start = contents.indexOf("name[]");
    start = contents.indexOf('"', start);
    int end = contents.indexOf("};", start);

    contents = contents.substring(start, end);

    boolean addLineSeparator = false;
    int tokenStart = -1;
    StringBuffer currentToken = new StringBuffer();
    for (int i = 0; i < contents.length(); i++) {
      char c = contents.charAt(i);
      if (c == '"') {
        if (tokenStart == -1) {
          tokenStart = i + 1;
        } else {
          if (addLineSeparator) {
            buffer.append('\n');
            result[(resultCount++)] = currentToken.toString();
            currentToken = new StringBuffer();
          }
          String token = contents.substring(tokenStart, i);
          if (token.equals("$error"))
            token = "Invalid Character";
          else if (token.equals("$eof")) {
            token = "Unexpected End Of File";
          }
          buffer.append(token);
          currentToken.append(token);
          addLineSeparator = true;
          tokenStart = -1;
        }
      }
      if ((tokenStart == -1) && (c == '+')) {
        addLineSeparator = false;
      }
    }
    if (currentToken.length() > 0) {
      result[(resultCount++)] = currentToken.toString();
    }

    buildFileForTable(filename, buffer.toString().toCharArray());

    System.arraycopy(result, 0, result = new String[resultCount], 0, resultCount);
    return result;
  }

  private static void buildFileForReadableName(String file, char[] newLhs, char[] newNonTerminalIndex, String[] newName, String[] tokens)
  {
    ArrayList entries = new ArrayList();

    boolean[] alreadyAdded = new boolean[newName.length];

    for (int i = 0; i < tokens.length; i += 3) {
      if ("1".equals(tokens[i])) {
        int index = newNonTerminalIndex[newLhs[Integer.parseInt(tokens[(i + 1)])]];
        StringBuffer buffer = new StringBuffer();
        if (alreadyAdded[index] == 0) {
          alreadyAdded[index] = true;
          buffer.append(newName[index]);
          buffer.append('=');
          buffer.append(tokens[(i + 2)].trim());
          buffer.append('\n');
          entries.add(String.valueOf(buffer));
        }
      }
    }
    int i = 1;
    while (!"Invalid Character".equals(newName[i])) i++;
    i++;
    for (; i < alreadyAdded.length; i++) {
      if (alreadyAdded[i] == 0) {
        System.out.println(newName[i] + " has no readable name");
      }
    }
    Collections.sort(entries);
    buildFile(file, entries);
  }

  private static void buildFilesForRecoveryTemplates(String indexFilename, String templatesFilename, char[] newTerminalIndex, char[] newNonTerminalIndex, String[] newName, char[] newLhs, String[] tokens)
  {
    int[] newReverse = computeReverseTable(newTerminalIndex, newNonTerminalIndex, newName);

    char[] newRecoveyTemplatesIndex = new char[newNonTerminalIndex.length];
    char[] newRecoveyTemplates = new char[newNonTerminalIndex.length];
    int newRecoveyTemplatesPtr = 0;

    for (int i = 0; i < tokens.length; i += 3) {
      if ("3".equals(tokens[i])) {
        int length = newRecoveyTemplates.length;
        if (length == newRecoveyTemplatesPtr + 1) {
          System.arraycopy(newRecoveyTemplates, 0, newRecoveyTemplates = new char[length * 2], 0, length);
        }
        newRecoveyTemplates[(newRecoveyTemplatesPtr++)] = '\000';

        int index = newLhs[Integer.parseInt(tokens[(i + 1)])];

        newRecoveyTemplatesIndex[index] = ((char)newRecoveyTemplatesPtr);

        String token = tokens[(i + 2)].trim();
        StringTokenizer st = new StringTokenizer(new String(token), " ");
        String[] terminalNames = new String[st.countTokens()];
        int t = 0;
        while (st.hasMoreTokens()) {
          terminalNames[(t++)] = st.nextToken();
        }

        for (int j = 0; j < terminalNames.length; j++) {
          int symbol = getSymbol(terminalNames[j], newName, newReverse);
          if (symbol > -1) {
            length = newRecoveyTemplates.length;
            if (length == newRecoveyTemplatesPtr + 1) {
              System.arraycopy(newRecoveyTemplates, 0, newRecoveyTemplates = new char[length * 2], 0, length);
            }
            newRecoveyTemplates[(newRecoveyTemplatesPtr++)] = ((char)symbol);
          }
        }
      }
    }
    newRecoveyTemplates[(newRecoveyTemplatesPtr++)] = '\000';
    System.arraycopy(newRecoveyTemplates, 0, newRecoveyTemplates = new char[newRecoveyTemplatesPtr], 0, newRecoveyTemplatesPtr);

    buildFileForTable(indexFilename, newRecoveyTemplatesIndex);
    buildFileForTable(templatesFilename, newRecoveyTemplates);
  }

  private static void buildFilesForStatementsRecoveryFilter(String filename, char[] newNonTerminalIndex, char[] newLhs, String[] tokens)
  {
    char[] newStatementsRecoveryFilter = new char[newNonTerminalIndex.length];

    for (int i = 0; i < tokens.length; i += 3) {
      if ("4".equals(tokens[i])) {
        int index = newLhs[Integer.parseInt(tokens[(i + 1)])];

        newStatementsRecoveryFilter[index] = '\001';
      }
    }
    buildFileForTable(filename, newStatementsRecoveryFilter);
  }

  private static void buildFileForCompliance(String file, int length, String[] tokens)
  {
    byte[] result = new byte[length * 8];

    for (int i = 0; i < tokens.length; i += 3) {
      if ("2".equals(tokens[i])) {
        int index = Integer.parseInt(tokens[(i + 1)]);
        String token = tokens[(i + 2)].trim();
        long compliance = 0L;
        if ("1.4".equals(token))
          compliance = 3145728L;
        else if ("1.5".equals(token))
          compliance = 3211264L;
        else if ("recovery".equals(token)) {
          compliance = 9223372036854775807L;
        }

        int j = index * 8;
        result[j] = ((byte)(int)(compliance >>> 56));
        result[(j + 1)] = ((byte)(int)(compliance >>> 48));
        result[(j + 2)] = ((byte)(int)(compliance >>> 40));
        result[(j + 3)] = ((byte)(int)(compliance >>> 32));
        result[(j + 4)] = ((byte)(int)(compliance >>> 24));
        result[(j + 5)] = ((byte)(int)(compliance >>> 16));
        result[(j + 6)] = ((byte)(int)(compliance >>> 8));
        result[(j + 7)] = ((byte)(int)compliance);
      }
    }

    buildFileForTable(file, result);
  }
  private static final void buildFileForTable(String filename, byte[] bytes) {
    FileOutputStream stream = null;
    try {
      stream = new FileOutputStream(filename);
      stream.write(bytes);
    }
    catch (IOException localIOException1)
    {
      if (stream != null)
        try {
          stream.close();
        }
        catch (IOException localIOException2)
        {
        }
    }
    finally
    {
      if (stream != null)
        try {
          stream.close();
        }
        catch (IOException localIOException3)
        {
        }
    }
    System.out.println(filename + " creation complete");
  }
  private static final void buildFileForTable(String filename, char[] chars) {
    byte[] bytes = new byte[chars.length * 2];
    for (int i = 0; i < chars.length; i++) {
      bytes[(2 * i)] = ((byte)(chars[i] >>> '\b'));
      bytes[(2 * i + 1)] = ((byte)(chars[i] & 0xFF));
    }

    FileOutputStream stream = null;
    try {
      stream = new FileOutputStream(filename);
      stream.write(bytes);
    }
    catch (IOException localIOException1)
    {
      if (stream != null)
        try {
          stream.close();
        }
        catch (IOException localIOException2)
        {
        }
    }
    finally
    {
      if (stream != null)
        try {
          stream.close();
        }
        catch (IOException localIOException3)
        {
        }
    }
    System.out.println(filename + " creation complete");
  }

  private static final byte[] buildFileOfByteFor(String filename, String tag, String[] tokens)
  {
    int i = 0;

    while (!tokens[(i++)].equals(tag));
    byte[] bytes = new byte[tokens.length];
    int ic = 0;
    String token;
    while (!(token = tokens[(i++)]).equals("}"))
    {
      String token;
      int c = Integer.parseInt(token);
      bytes[(ic++)] = ((byte)c);
    }

    System.arraycopy(bytes, 0, bytes = new byte[ic], 0, ic);

    buildFileForTable(filename, bytes);
    return bytes;
  }

  private static final char[] buildFileOfIntFor(String filename, String tag, String[] tokens)
  {
    int i = 0;

    while (!tokens[(i++)].equals(tag));
    char[] chars = new char[tokens.length];
    int ic = 0;
    String token;
    while (!(token = tokens[(i++)]).equals("}"))
    {
      String token;
      int c = Integer.parseInt(token);
      chars[(ic++)] = ((char)c);
    }

    System.arraycopy(chars, 0, chars = new char[ic], 0, ic);

    buildFileForTable(filename, chars);
    return chars;
  }

  private static final void buildFileOfShortFor(String filename, String tag, String[] tokens)
  {
    int i = 0;

    while (!tokens[(i++)].equals(tag));
    char[] chars = new char[tokens.length];
    int ic = 0;
    String token;
    while (!(token = tokens[(i++)]).equals("}"))
    {
      String token;
      int c = Integer.parseInt(token);
      chars[(ic++)] = ((char)(c + 32768));
    }

    System.arraycopy(chars, 0, chars = new char[ic], 0, ic);

    buildFileForTable(filename, chars);
  }

  public static final void buildFilesFromLPG(String dataFilename, String dataFilename2)
  {
    char[] contents = CharOperation.NO_CHAR;
    try {
      contents = Util.getFileCharContent(new File(dataFilename), null);
    } catch (IOException localIOException1) {
      System.out.println(Messages.parser_incorrectPath);
      return;
    }
    StringTokenizer st = 
      new StringTokenizer(new String(contents), " \t\n\r[]={,;");
    String[] tokens = new String[st.countTokens()];
    int j = 0;
    while (st.hasMoreTokens()) {
      tokens[(j++)] = st.nextToken();
    }

    int i = 0;

    char[] newLhs = buildFileOfIntFor("parser" + ++i + ".rsc", "lhs", tokens);
    buildFileOfShortFor("parser" + ++i + ".rsc", "check_table", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "asb", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "asr", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "nasb", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "nasr", tokens);
    char[] newTerminalIndex = buildFileOfIntFor("parser" + ++i + ".rsc", "terminal_index", tokens);
    char[] newNonTerminalIndex = buildFileOfIntFor("parser" + ++i + ".rsc", "non_terminal_index", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "term_action", tokens);

    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_prefix", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_suffix", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_lhs", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_state_set", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_rhs", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "scope_state", tokens);
    buildFileOfIntFor("parser" + ++i + ".rsc", "in_symb", tokens);

    byte[] newRhs = buildFileOfByteFor("parser" + ++i + ".rsc", "rhs", tokens);
    buildFileOfByteFor("parser" + ++i + ".rsc", "term_check", tokens);
    buildFileOfByteFor("parser" + ++i + ".rsc", "scope_la", tokens);

    String[] newName = buildFileForName("parser" + ++i + ".rsc", new String(contents));

    contents = CharOperation.NO_CHAR;
    try {
      contents = Util.getFileCharContent(new File(dataFilename2), null);
    } catch (IOException localIOException2) {
      System.out.println(Messages.parser_incorrectPath);
      return;
    }
    st = new StringTokenizer(new String(contents), "\t\n\r#");
    tokens = new String[st.countTokens()];
    j = 0;
    while (st.hasMoreTokens()) {
      tokens[(j++)] = st.nextToken();
    }

    buildFileForCompliance("parser" + ++i + ".rsc", newRhs.length, tokens);
    buildFileForReadableName("readableNames.properties", newLhs, newNonTerminalIndex, newName, tokens);

    buildFilesForRecoveryTemplates(
      "parser" + ++i + ".rsc", 
      "parser" + ++i + ".rsc", 
      newTerminalIndex, 
      newNonTerminalIndex, 
      newName, 
      newLhs, 
      tokens);

    buildFilesForStatementsRecoveryFilter(
      "parser" + ++i + ".rsc", 
      newNonTerminalIndex, 
      newLhs, 
      tokens);

    System.out.println(Messages.parser_moveFiles);
  }
  public static int in_symbol(int state) {
    return in_symb[original_state(state)];
  }

  public static final void initTables() throws IOException
  {
    int i = 0;
    lhs = readTable("parser" + ++i + ".rsc");
    char[] chars = readTable("parser" + ++i + ".rsc");
    check_table = new short[chars.length];
    for (int c = chars.length; c-- > 0; ) {
      check_table[c] = ((short)(chars[c] - 32768));
    }
    asb = readTable("parser" + ++i + ".rsc");
    asr = readTable("parser" + ++i + ".rsc");
    nasb = readTable("parser" + ++i + ".rsc");
    nasr = readTable("parser" + ++i + ".rsc");
    terminal_index = readTable("parser" + ++i + ".rsc");
    non_terminal_index = readTable("parser" + ++i + ".rsc");
    term_action = readTable("parser" + ++i + ".rsc");

    scope_prefix = readTable("parser" + ++i + ".rsc");
    scope_suffix = readTable("parser" + ++i + ".rsc");
    scope_lhs = readTable("parser" + ++i + ".rsc");
    scope_state_set = readTable("parser" + ++i + ".rsc");
    scope_rhs = readTable("parser" + ++i + ".rsc");
    scope_state = readTable("parser" + ++i + ".rsc");
    in_symb = readTable("parser" + ++i + ".rsc");

    rhs = readByteTable("parser" + ++i + ".rsc");
    term_check = readByteTable("parser" + ++i + ".rsc");
    scope_la = readByteTable("parser" + ++i + ".rsc");

    name = readNameTable("parser" + ++i + ".rsc");

    rules_compliance = readLongTable("parser" + ++i + ".rsc");

    readableName = readReadableNameTable("org.eclipse.jdt.internal.compiler.parser.readableNames");

    reverse_index = computeReverseTable(terminal_index, non_terminal_index, name);

    recovery_templates_index = readTable("parser" + ++i + ".rsc");
    recovery_templates = readTable("parser" + ++i + ".rsc");

    statements_recovery_filter = readTable("parser" + ++i + ".rsc");

    base_action = lhs;
  }
  public static int nasi(int state) {
    return nasb[original_state(state)];
  }
  public static int ntAction(int state, int sym) {
    return base_action[(state + sym)];
  }
  protected static int original_state(int state) {
    return -base_check(state);
  }
  protected static int[] computeReverseTable(char[] newTerminalIndex, char[] newNonTerminalIndex, String[] newName) {
    int[] newReverseTable = new int[newName.length];
    for (int j = 0; j < newName.length; j++)
    {
      for (int k = 0; k < newTerminalIndex.length; k++) {
        if (newTerminalIndex[k] == j) {
          newReverseTable[j] = k;
          break;
        }
      }
      for (int k = 0; k < newNonTerminalIndex.length; k++) {
        if (newNonTerminalIndex[k] == j) {
          newReverseTable[j] = (-k);
          break;
        }
      }
    }

    return newReverseTable;
  }

  private static int getSymbol(String terminalName, String[] newName, int[] newReverse) {
    for (int j = 0; j < newName.length; j++) {
      if (terminalName.equals(newName[j])) {
        return newReverse[j];
      }
    }
    return -1;
  }

  protected static byte[] readByteTable(String filename)
    throws IOException
  {
    InputStream stream = Parser.class.getResourceAsStream(filename);
    if (stream == null) {
      throw new IOException(Messages.bind(Messages.parser_missingFile, filename));
    }
    byte[] bytes = (byte[])null;
    try {
      stream = new BufferedInputStream(stream);
      bytes = Util.getInputStreamAsByteArray(stream, -1);
    } finally {
      try {
        stream.close();
      }
      catch (IOException localIOException1) {
      }
    }
    return bytes;
  }

  protected static String[] readNameTable(String filename) throws IOException {
    char[] contents = readTable(filename);
    char[][] nameAsChar = CharOperation.splitOn('\n', contents);

    String[] result = new String[nameAsChar.length + 1];
    result[0] = null;
    for (int i = 0; i < nameAsChar.length; i++) {
      result[(i + 1)] = new String(nameAsChar[i]);
    }

    return result;
  }
  protected static String[] readReadableNameTable(String filename) {
    String[] result = new String[name.length];
    try
    {
      bundle = ResourceBundle.getBundle(filename, Locale.getDefault());
    }
    catch (MissingResourceException e)
    {
      ResourceBundle bundle;
      System.out.println("Missing resource : " + filename.replace('.', '/') + ".properties for locale " + Locale.getDefault());
      throw e;
    }
    ResourceBundle bundle;
    for (int i = 0; i < 111; i++) {
      result[i] = name[i];
    }
    for (int i = 110; i < name.length; i++) {
      try {
        String n = bundle.getString(name[i]);
        if ((n != null) && (n.length() > 0))
          result[i] = n;
        else
          result[i] = name[i];
      }
      catch (MissingResourceException localMissingResourceException1) {
        result[i] = name[i];
      }
    }
    return result;
  }

  protected static char[] readTable(String filename)
    throws IOException
  {
    InputStream stream = Parser.class.getResourceAsStream(filename);
    if (stream == null) {
      throw new IOException(Messages.bind(Messages.parser_missingFile, filename));
    }
    byte[] bytes = (byte[])null;
    try {
      stream = new BufferedInputStream(stream);
      bytes = Util.getInputStreamAsByteArray(stream, -1);
    } finally {
      try {
        stream.close();
      }
      catch (IOException localIOException1)
      {
      }
    }

    int length = bytes.length;
    if ((length & 0x1) != 0) {
      throw new IOException(Messages.bind(Messages.parser_corruptedFile, filename));
    }

    char[] chars = new char[length / 2];
    int i = 0;
    int charIndex = 0;
    do
    {
      chars[(charIndex++)] = ((char)(((bytes[(i++)] & 0xFF) << 8) + (bytes[(i++)] & 0xFF)));
    }while (i != length);

    return chars;
  }

  protected static long[] readLongTable(String filename)
    throws IOException
  {
    InputStream stream = Parser.class.getResourceAsStream(filename);
    if (stream == null) {
      throw new IOException(Messages.bind(Messages.parser_missingFile, filename));
    }
    byte[] bytes = (byte[])null;
    try {
      stream = new BufferedInputStream(stream);
      bytes = Util.getInputStreamAsByteArray(stream, -1);
    } finally {
      try {
        stream.close();
      }
      catch (IOException localIOException1)
      {
      }
    }

    int length = bytes.length;
    if (length % 8 != 0) {
      throw new IOException(Messages.bind(Messages.parser_corruptedFile, filename));
    }

    long[] longs = new long[length / 8];
    int i = 0;
    int longIndex = 0;
    do
    {
      longs[(longIndex++)] = 
        (((bytes[(i++)] & 0xFF) << 56) + (
        (bytes[(i++)] & 0xFF) << 48) + (
        (bytes[(i++)] & 0xFF) << 40) + (
        (bytes[(i++)] & 0xFF) << 32) + (
        (bytes[(i++)] & 0xFF) << 24) + (
        (bytes[(i++)] & 0xFF) << 16) + (
        (bytes[(i++)] & 0xFF) << 8) + (
        bytes[(i++)] & 0xFF));
    }
    while (i != length);

    return longs;
  }
  public static int tAction(int state, int sym) {
    return term_action[base_action[state]];
  }

  public Parser(ProblemReporter problemReporter, boolean optimizeStringLiterals)
  {
    this.problemReporter = problemReporter;
    this.options = problemReporter.options;
    this.optimizeStringLiterals = optimizeStringLiterals;
    initializeScanner();
    this.astLengthStack = new int[50];
    this.expressionLengthStack = new int[30];
    this.intStack = new int[50];
    this.identifierStack = new char[30][];
    this.identifierLengthStack = new int[30];
    this.nestedMethod = new int[30];
    this.realBlockStack = new int[30];
    this.identifierPositionStack = new long[30];
    this.variablesCounter = new int[30];

    this.javadocParser = createJavadocParser();
  }
  protected void annotationRecoveryCheckPoint(int start, int end) {
    if ((this.lastCheckPoint > start) && (this.lastCheckPoint < end))
      this.lastCheckPoint = (end + 1);
  }

  public void arrayInitializer(int length)
  {
    ArrayInitializer ai = new ArrayInitializer();
    if (length != 0) {
      this.expressionPtr -= length;
      System.arraycopy(this.expressionStack, this.expressionPtr + 1, ai.expressions = new Expression[length], 0, length);
    }
    pushOnExpressionStack(ai);

    ai.sourceEnd = this.endStatementPosition;
    ai.sourceStart = this.intStack[(this.intPtr--)];
  }

  protected void blockReal()
  {
    this.realBlockStack[this.realBlockPtr] += 1;
  }

  public RecoveredElement buildInitialRecoveryState()
  {
    this.lastCheckPoint = 0;
    this.lastErrorEndPositionBeforeRecovery = this.scanner.currentPosition;

    RecoveredElement element = null;
    if ((this.referenceContext instanceof CompilationUnitDeclaration)) {
      element = new RecoveredUnit(this.compilationUnit, 0, this);

      this.compilationUnit.currentPackage = null;
      this.compilationUnit.imports = null;
      this.compilationUnit.types = null;
      this.currentToken = 0;
      this.listLength = 0;
      this.listTypeParameterLength = 0;
      this.endPosition = 0;
      this.endStatementPosition = 0;
      return element;
    }

    if ((this.referenceContext instanceof AbstractMethodDeclaration)) {
      element = new RecoveredMethod((AbstractMethodDeclaration)this.referenceContext, null, 0, this);
      this.lastCheckPoint = ((AbstractMethodDeclaration)this.referenceContext).bodyStart;
      if (this.statementRecoveryActivated) {
        element = element.add(new Block(0), 0);
      }

    }
    else if ((this.referenceContext instanceof TypeDeclaration)) {
      TypeDeclaration type = (TypeDeclaration)this.referenceContext;
      for (int i = 0; i < type.fields.length; i++) {
        FieldDeclaration field = type.fields[i];
        if ((field != null) && 
          (field.getKind() == 2) && 
          (field.declarationSourceStart <= this.scanner.initialPosition) && 
          (this.scanner.initialPosition <= field.declarationSourceEnd) && 
          (this.scanner.eofPosition <= field.declarationSourceEnd + 1)) {
          element = new RecoveredInitializer(field, null, 1, this);
          this.lastCheckPoint = field.declarationSourceStart;
          break;
        }

      }

    }

    if (element == null) return element;

    for (int i = 0; i <= this.astPtr; i++) {
      ASTNode node = this.astStack[i];
      if ((node instanceof AbstractMethodDeclaration)) {
        AbstractMethodDeclaration method = (AbstractMethodDeclaration)node;
        if (method.declarationSourceEnd == 0) {
          element = element.add(method, 0);
          this.lastCheckPoint = method.bodyStart;
        } else {
          element = element.add(method, 0);
          this.lastCheckPoint = (method.declarationSourceEnd + 1);
        }

      }
      else if ((node instanceof Initializer)) {
        Initializer initializer = (Initializer)node;
        if (initializer.declarationSourceEnd == 0) {
          element = element.add(initializer, 1);
          this.lastCheckPoint = initializer.sourceStart;
        } else {
          element = element.add(initializer, 0);
          this.lastCheckPoint = (initializer.declarationSourceEnd + 1);
        }

      }
      else if ((node instanceof FieldDeclaration)) {
        FieldDeclaration field = (FieldDeclaration)node;
        if (field.declarationSourceEnd == 0) {
          element = element.add(field, 0);
          if (field.initialization == null)
            this.lastCheckPoint = (field.sourceEnd + 1);
          else
            this.lastCheckPoint = (field.initialization.sourceEnd + 1);
        }
        else {
          element = element.add(field, 0);
          this.lastCheckPoint = (field.declarationSourceEnd + 1);
        }

      }
      else if ((node instanceof TypeDeclaration)) {
        TypeDeclaration type = (TypeDeclaration)node;
        if (type.declarationSourceEnd == 0) {
          element = element.add(type, 0);
          this.lastCheckPoint = type.bodyStart;
        } else {
          element = element.add(type, 0);
          this.lastCheckPoint = (type.declarationSourceEnd + 1);
        }
      }
      else {
        if ((node instanceof ImportReference)) {
          ImportReference importRef = (ImportReference)node;
          element = element.add(importRef, 0);
          this.lastCheckPoint = (importRef.declarationSourceEnd + 1);
        }
        if (this.statementRecoveryActivated) {
          if ((node instanceof Block)) {
            Block block = (Block)node;
            element = element.add(block, 0);
            this.lastCheckPoint = (block.sourceEnd + 1);
          } else if ((node instanceof LocalDeclaration)) {
            LocalDeclaration statement = (LocalDeclaration)node;
            element = element.add(statement, 0);
            this.lastCheckPoint = (statement.sourceEnd + 1);
          } else if ((node instanceof Expression)) {
            if (((node instanceof Assignment)) || 
              ((node instanceof PrefixExpression)) || 
              ((node instanceof PostfixExpression)) || 
              ((node instanceof MessageSend)) || 
              ((node instanceof AllocationExpression)))
            {
              Expression statement = (Expression)node;
              element = element.add(statement, 0);
              if (statement.statementEnd != -1)
                this.lastCheckPoint = (statement.statementEnd + 1);
              else
                this.lastCheckPoint = (statement.sourceEnd + 1);
            }
          }
          else if ((node instanceof Statement)) {
            Statement statement = (Statement)node;
            element = element.add(statement, 0);
            this.lastCheckPoint = (statement.sourceEnd + 1);
          }
        }
      }
    }
    if ((this.statementRecoveryActivated) && 
      (this.pendingRecoveredType != null) && 
      (this.scanner.startPosition - 1 <= this.pendingRecoveredType.declarationSourceEnd))
    {
      element = element.add(this.pendingRecoveredType, 0);
      this.lastCheckPoint = (this.pendingRecoveredType.declarationSourceEnd + 1);
      this.pendingRecoveredType = null;
    }

    return element;
  }

  protected void checkAndSetModifiers(int flag)
  {
    if ((this.modifiers & flag) != 0) {
      this.modifiers |= 4194304;
    }
    this.modifiers |= flag;

    if (this.modifiersSourceStart < 0) this.modifiersSourceStart = this.scanner.startPosition;
  }

  public void checkComment()
  {
    if (((!this.diet) || (this.dietInt != 0)) && (this.scanner.commentPtr >= 0)) {
      flushCommentsDefinedPriorTo(this.endStatementPosition);
    }

    int lastComment = this.scanner.commentPtr;

    if (this.modifiersSourceStart >= 0)
    {
      while ((lastComment >= 0) && (this.scanner.commentStarts[lastComment] > this.modifiersSourceStart)) lastComment--;
    }
    if (lastComment >= 0)
    {
      this.modifiersSourceStart = this.scanner.commentStarts[0];

      while ((lastComment >= 0) && (this.scanner.commentStops[lastComment] < 0)) lastComment--;
      if ((lastComment >= 0) && (this.javadocParser != null)) {
        int commentEnd = this.scanner.commentStops[lastComment] - 1;

        this.javadocParser.reportProblems = ((this.currentElement == null) || (commentEnd > this.lastJavadocEnd));
        if (this.javadocParser.checkDeprecation(lastComment)) {
          checkAndSetModifiers(1048576);
        }
        this.javadoc = this.javadocParser.docComment;
        if (this.currentElement == null) this.lastJavadocEnd = commentEnd; 
      }
    }
  }

  protected void checkNonNLSAfterBodyEnd(int declarationEnd) { if (this.scanner.currentPosition - 1 <= declarationEnd) {
      this.scanner.eofPosition = (declarationEnd < 2147483647 ? declarationEnd + 1 : declarationEnd);
      try {
        while (this.scanner.getNextToken() != 68);
      }
      catch (InvalidInputException localInvalidInputException)
      {
      }
    }
  }

  protected void classInstanceCreation(boolean isQualified)
  {
    int length;
    if (((length = this.astLengthStack[(this.astLengthPtr--)]) == 1) && 
      (this.astStack[this.astPtr] == null))
    {
      this.astPtr -= 1;
      AllocationExpression alloc;
      AllocationExpression alloc;
      if (isQualified)
        alloc = new QualifiedAllocationExpression();
      else {
        alloc = new AllocationExpression();
      }
      alloc.sourceEnd = this.endPosition;

      if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
        this.expressionPtr -= length;
        System.arraycopy(
          this.expressionStack, 
          this.expressionPtr + 1, 
          alloc.arguments = new Expression[length], 
          0, 
          length);
      }
      alloc.type = getTypeReference(0);

      alloc.sourceStart = this.intStack[(this.intPtr--)];
      pushOnExpressionStack(alloc);
    } else {
      dispatchDeclarationInto(length);
      TypeDeclaration anonymousTypeDeclaration = (TypeDeclaration)this.astStack[this.astPtr];
      anonymousTypeDeclaration.declarationSourceEnd = this.endStatementPosition;
      anonymousTypeDeclaration.bodyEnd = this.endStatementPosition;
      if (anonymousTypeDeclaration.allocation != null) {
        anonymousTypeDeclaration.allocation.sourceEnd = this.endStatementPosition;
      }
      if ((length == 0) && (!containsComment(anonymousTypeDeclaration.bodyStart, anonymousTypeDeclaration.bodyEnd))) {
        anonymousTypeDeclaration.bits |= 8;
      }
      this.astPtr -= 1;
      this.astLengthPtr -= 1;

      markInitializersWithLocalType(anonymousTypeDeclaration);
    }
  }

  protected void concatExpressionLists() { this.expressionLengthStack[(--this.expressionLengthPtr)] += 1; }

  protected void concatGenericsLists() {
    this.genericsLengthStack[(this.genericsLengthPtr - 1)] += this.genericsLengthStack[(this.genericsLengthPtr--)];
  }

  protected void concatNodeLists()
  {
    this.astLengthStack[(this.astLengthPtr - 1)] += this.astLengthStack[(this.astLengthPtr--)];
  }
  protected void consumeAdditionalBound() {
    pushOnGenericsStack(getTypeReference(this.intStack[(this.intPtr--)]));
  }

  protected void consumeAdditionalBound1() {
  }

  protected void consumeAdditionalBoundList() {
    concatGenericsLists();
  }
  protected void consumeAdditionalBoundList1() {
    concatGenericsLists();
  }

  protected void consumeAllocationHeader()
  {
    if (this.currentElement == null) {
      return;
    }
    if (this.currentToken == 69)
    {
      TypeDeclaration anonymousType = new TypeDeclaration(this.compilationUnit.compilationResult);
      anonymousType.name = CharOperation.NO_CHAR;
      anonymousType.bits |= 768;
      anonymousType.sourceStart = this.intStack[(this.intPtr--)];
      anonymousType.sourceEnd = this.rParenPos;
      QualifiedAllocationExpression alloc = new QualifiedAllocationExpression(anonymousType);
      alloc.type = getTypeReference(0);
      alloc.sourceStart = anonymousType.sourceStart;
      alloc.sourceEnd = anonymousType.sourceEnd;
      this.lastCheckPoint = (anonymousType.bodyStart = this.scanner.currentPosition);
      this.currentElement = this.currentElement.add(anonymousType, 0);
      this.lastIgnoredToken = -1;
      this.currentToken = 0;
      return;
    }
    this.lastCheckPoint = this.scanner.startPosition;
    this.restartRecovery = true;
  }
  protected void consumeAnnotationAsModifier() {
    Expression expression = this.expressionStack[this.expressionPtr];
    int sourceStart = expression.sourceStart;
    if (this.modifiersSourceStart < 0)
      this.modifiersSourceStart = sourceStart;
  }

  protected void consumeAnnotationName() {
    if (this.currentElement != null) {
      int start = this.intStack[this.intPtr];
      int end = (int)(this.identifierPositionStack[this.identifierPtr] & 0xFFFFFFFF);
      annotationRecoveryCheckPoint(start, end);
    }
    this.recordStringLiterals = false;
  }

  protected void consumeAnnotationTypeDeclaration()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)
    {
      dispatchDeclarationInto(length);
    }

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];

    markInitializersWithLocalType(typeDecl);

    typeDecl.checkConstructors(this);

    if (this.scanner.containsAssertKeyword) {
      typeDecl.bits |= 1;
    }
    typeDecl.addClinit();
    typeDecl.bodyEnd = this.endStatementPosition;
    if ((length == 0) && (!containsComment(typeDecl.bodyStart, typeDecl.bodyEnd))) {
      typeDecl.bits |= 8;
    }
    typeDecl.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }
  protected void consumeAnnotationTypeDeclarationHeader() {
    if (this.currentElement != null) {
      this.restartRecovery = true;
    }

    this.scanner.commentPtr = -1;
  }

  protected void consumeAnnotationTypeDeclarationHeaderName()
  {
    TypeDeclaration annotationTypeDeclaration = new TypeDeclaration(this.compilationUnit.compilationResult);
    if (this.nestedMethod[this.nestedType] == 0) {
      if (this.nestedType != 0)
        annotationTypeDeclaration.bits |= 1024;
    }
    else
    {
      annotationTypeDeclaration.bits |= 256;
      markEnclosingMemberWithLocalType();
      blockReal();
    }

    long pos = this.identifierPositionStack[this.identifierPtr];
    annotationTypeDeclaration.sourceEnd = ((int)pos);
    annotationTypeDeclaration.sourceStart = ((int)(pos >>> 32));
    annotationTypeDeclaration.name = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    this.intPtr -= 1;
    this.intPtr -= 1;

    annotationTypeDeclaration.modifiersSourceStart = this.intStack[(this.intPtr--)];
    annotationTypeDeclaration.modifiers = (this.intStack[(this.intPtr--)] | 0x2000 | 0x200);
    if (annotationTypeDeclaration.modifiersSourceStart >= 0) {
      annotationTypeDeclaration.declarationSourceStart = annotationTypeDeclaration.modifiersSourceStart;
      this.intPtr -= 1;
    } else {
      int atPosition = this.intStack[(this.intPtr--)];

      annotationTypeDeclaration.declarationSourceStart = atPosition;
    }

    if (((annotationTypeDeclaration.bits & 0x400) == 0) && ((annotationTypeDeclaration.bits & 0x100) == 0) && 
      (this.compilationUnit != null) && (!CharOperation.equals(annotationTypeDeclaration.name, this.compilationUnit.getMainTypeName())))
      annotationTypeDeclaration.bits |= 4096;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        annotationTypeDeclaration.annotations = new Annotation[length], 
        0, 
        length);
    }
    annotationTypeDeclaration.bodyStart = (annotationTypeDeclaration.sourceEnd + 1);

    annotationTypeDeclaration.javadoc = this.javadoc;
    this.javadoc = null;
    pushOnAstStack(annotationTypeDeclaration);
    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      problemReporter().invalidUsageOfAnnotationDeclarations(annotationTypeDeclaration);
    }

    if (this.currentElement != null) {
      this.lastCheckPoint = annotationTypeDeclaration.bodyStart;
      this.currentElement = this.currentElement.add(annotationTypeDeclaration, 0);
      this.lastIgnoredToken = -1;
    }
  }

  protected void consumeAnnotationTypeMemberDeclaration() {
    AnnotationMethodDeclaration annotationTypeMemberDeclaration = (AnnotationMethodDeclaration)this.astStack[this.astPtr];
    annotationTypeMemberDeclaration.modifiers |= 16777216;

    int declarationEndPosition = flushCommentsDefinedPriorTo(this.endStatementPosition);
    annotationTypeMemberDeclaration.bodyStart = this.endStatementPosition;
    annotationTypeMemberDeclaration.bodyEnd = declarationEndPosition;
    annotationTypeMemberDeclaration.declarationSourceEnd = declarationEndPosition;
  }

  protected void consumeAnnotationTypeMemberDeclarations() {
    concatNodeLists();
  }

  protected void consumeArgumentList() {
    concatExpressionLists();
  }

  protected void consumeArguments()
  {
    pushOnIntStack(this.rParenPos);
  }

  protected void consumeArrayAccess(boolean unspecifiedReference)
  {
    Expression exp;
    Expression exp;
    if (unspecifiedReference) {
      exp = 
        this.expressionStack[this.expressionPtr] =  = 
        new ArrayReference(
        getUnspecifiedReferenceOptimized(), 
        this.expressionStack[this.expressionPtr]);
    } else {
      this.expressionPtr -= 1;
      this.expressionLengthPtr -= 1;
      exp = 
        this.expressionStack[this.expressionPtr] =  = 
        new ArrayReference(
        this.expressionStack[this.expressionPtr], 
        this.expressionStack[(this.expressionPtr + 1)]);
    }
    exp.sourceEnd = this.endStatementPosition;
  }

  protected void consumeArrayCreationExpressionWithInitializer()
  {
    ArrayAllocationExpression arrayAllocation = new ArrayAllocationExpression();
    this.expressionLengthPtr -= 1;
    arrayAllocation.initializer = ((ArrayInitializer)this.expressionStack[(this.expressionPtr--)]);

    arrayAllocation.type = getTypeReference(0);
    arrayAllocation.type.bits |= 1073741824;
    int length = this.expressionLengthStack[(this.expressionLengthPtr--)];
    this.expressionPtr -= length;
    System.arraycopy(
      this.expressionStack, 
      this.expressionPtr + 1, 
      arrayAllocation.dimensions = new Expression[length], 
      0, 
      length);
    arrayAllocation.sourceStart = this.intStack[(this.intPtr--)];
    if (arrayAllocation.initializer == null)
      arrayAllocation.sourceEnd = this.endStatementPosition;
    else {
      arrayAllocation.sourceEnd = arrayAllocation.initializer.sourceEnd;
    }
    pushOnExpressionStack(arrayAllocation);
  }

  protected void consumeArrayCreationExpressionWithoutInitializer()
  {
    ArrayAllocationExpression arrayAllocation = new ArrayAllocationExpression();
    arrayAllocation.type = getTypeReference(0);
    arrayAllocation.type.bits |= 1073741824;
    int length = this.expressionLengthStack[(this.expressionLengthPtr--)];
    this.expressionPtr -= length;
    System.arraycopy(
      this.expressionStack, 
      this.expressionPtr + 1, 
      arrayAllocation.dimensions = new Expression[length], 
      0, 
      length);
    arrayAllocation.sourceStart = this.intStack[(this.intPtr--)];
    if (arrayAllocation.initializer == null)
      arrayAllocation.sourceEnd = this.endStatementPosition;
    else {
      arrayAllocation.sourceEnd = arrayAllocation.initializer.sourceEnd;
    }
    pushOnExpressionStack(arrayAllocation);
  }

  protected void consumeArrayCreationHeader()
  {
  }

  protected void consumeArrayInitializer()
  {
    arrayInitializer(this.expressionLengthStack[(this.expressionLengthPtr--)]);
  }
  protected void consumeArrayTypeWithTypeArgumentsName() {
    this.genericsIdentifiersLengthStack[this.genericsIdentifiersLengthPtr] += this.identifierLengthStack[this.identifierLengthPtr];
    pushOnGenericsLengthStack(0);
  }

  protected void consumeAssertStatement() {
    this.expressionLengthPtr -= 2;
    pushOnAstStack(new AssertStatement(this.expressionStack[(this.expressionPtr--)], this.expressionStack[(this.expressionPtr--)], this.intStack[(this.intPtr--)]));
  }

  protected void consumeAssignment()
  {
    int op = this.intStack[(this.intPtr--)];

    this.expressionPtr -= 1; this.expressionLengthPtr -= 1;
    this.expressionStack[this.expressionPtr] = 
      (op != 30 ? 
      new CompoundAssignment(
      this.expressionStack[this.expressionPtr], 
      this.expressionStack[(this.expressionPtr + 1)], 
      op, 
      this.scanner.startPosition - 1) : 
      new Assignment(
      this.expressionStack[this.expressionPtr], 
      this.expressionStack[(this.expressionPtr + 1)], 
      this.scanner.startPosition - 1));

    if (this.pendingRecoveredType != null)
    {
      if ((this.pendingRecoveredType.allocation != null) && 
        (this.scanner.startPosition - 1 <= this.pendingRecoveredType.declarationSourceEnd)) {
        this.expressionStack[this.expressionPtr] = this.pendingRecoveredType.allocation;
        this.pendingRecoveredType = null;
        return;
      }
      this.pendingRecoveredType = null;
    }
  }

  protected void consumeAssignmentOperator(int pos)
  {
    pushOnIntStack(pos);
  }

  protected void consumeBinaryExpression(int op)
  {
    this.expressionPtr -= 1;
    this.expressionLengthPtr -= 1;
    Expression expr1 = this.expressionStack[this.expressionPtr];
    Expression expr2 = this.expressionStack[(this.expressionPtr + 1)];
    switch (op) {
    case 1:
      this.expressionStack[this.expressionPtr] = 
        new OR_OR_Expression(
        expr1, 
        expr2, 
        op);
      break;
    case 0:
      this.expressionStack[this.expressionPtr] = 
        new AND_AND_Expression(
        expr1, 
        expr2, 
        op);
      break;
    case 14:
      if (this.optimizeStringLiterals) {
        if ((expr1 instanceof StringLiteral)) {
          if ((expr2 instanceof CharLiteral))
            this.expressionStack[this.expressionPtr] = 
              ((StringLiteral)expr1).extendWith((CharLiteral)expr2);
          else if ((expr2 instanceof StringLiteral))
            this.expressionStack[this.expressionPtr] = 
              ((StringLiteral)expr1).extendWith((StringLiteral)expr2);
          else
            this.expressionStack[this.expressionPtr] = new BinaryExpression(expr1, expr2, 14);
        }
        else if ((expr1 instanceof CombinedBinaryExpression))
        {
          CombinedBinaryExpression cursor;
          if ((cursor = (CombinedBinaryExpression)expr1).arity < 
            cursor.arityMax) {
            cursor.left = new BinaryExpression(cursor.left, 
              cursor.right, 14);
            cursor.arity += 1;
          } else {
            cursor.left = new CombinedBinaryExpression(cursor.left, 
              cursor.right, 14, cursor.arity);
            cursor.arity = 0;
            cursor.tuneArityMax();
          }
          cursor.right = expr2;
          cursor.sourceEnd = expr2.sourceEnd;
          this.expressionStack[this.expressionPtr] = cursor;
        }
        else
        {
          if ((expr1 instanceof BinaryExpression))
          {
            if ((expr1.bits & 0xFC0) >> 
              6 == 14) {
              this.expressionStack[this.expressionPtr] = 
                new CombinedBinaryExpression(expr1, expr2, 14, 1);

              break;
            }
          }
          this.expressionStack[this.expressionPtr] = 
            new BinaryExpression(expr1, expr2, 14);
        }
      } else if ((expr1 instanceof StringLiteral)) {
        if ((expr2 instanceof StringLiteral))
        {
          this.expressionStack[this.expressionPtr] = 
            ((StringLiteral)expr1).extendsWith((StringLiteral)expr2);
        }
        else
          this.expressionStack[this.expressionPtr] = 
            new BinaryExpression(expr1, expr2, 14);
      }
      else if ((expr1 instanceof CombinedBinaryExpression))
      {
        int numberOfParens = (expr1.bits & 0x1FE00000) >> 21;
        CombinedBinaryExpression cursor;
        if ((cursor = (CombinedBinaryExpression)expr1).arity < 
          cursor.arityMax) {
          cursor.left = new BinaryExpression(cursor.left, 
            cursor.right, 14);
          if (numberOfParens != 0)
          {
            cursor.bits &= -534773761;
            cursor.left.bits &= -534773761;
            cursor.left.bits |= numberOfParens << 21;
          }
          cursor.arity += 1;
        } else {
          cursor.left = new CombinedBinaryExpression(cursor.left, 
            cursor.right, 14, cursor.arity);
          if (numberOfParens != 0)
          {
            cursor.bits &= -534773761;
            cursor.left.bits &= -534773761;
            cursor.left.bits |= numberOfParens << 21;
          }
          cursor.arity = 0;
          cursor.tuneArityMax();
        }
        cursor.right = expr2;
        cursor.sourceEnd = expr2.sourceEnd;

        this.expressionStack[this.expressionPtr] = cursor;
      } else if (((expr1 instanceof BinaryExpression)) && 
        ((expr1.bits & 0xFC0) >> 
        6 == 14))
      {
        this.expressionStack[this.expressionPtr] = 
          new CombinedBinaryExpression(expr1, expr2, 14, 1);
      } else {
        this.expressionStack[this.expressionPtr] = 
          new BinaryExpression(expr1, expr2, 14);
      }
      break;
    case 4:
      this.intPtr -= 1;
      this.expressionStack[this.expressionPtr] = 
        new BinaryExpression(
        expr1, 
        expr2, 
        op);
      break;
    default:
      this.expressionStack[this.expressionPtr] = 
        new BinaryExpression(
        expr1, 
        expr2, 
        op);
    }
  }

  protected void consumeBinaryExpressionWithName(int op)
  {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
    this.expressionPtr -= 1;
    this.expressionLengthPtr -= 1;

    Expression expr1 = this.expressionStack[(this.expressionPtr + 1)];
    Expression expr2 = this.expressionStack[this.expressionPtr];

    switch (op) {
    case 1:
      this.expressionStack[this.expressionPtr] = 
        new OR_OR_Expression(
        expr1, 
        expr2, 
        op);
      break;
    case 0:
      this.expressionStack[this.expressionPtr] = 
        new AND_AND_Expression(
        expr1, 
        expr2, 
        op);
      break;
    case 14:
      if (this.optimizeStringLiterals) {
        if ((expr1 instanceof StringLiteral)) {
          if ((expr2 instanceof CharLiteral))
            this.expressionStack[this.expressionPtr] = 
              ((StringLiteral)expr1).extendWith((CharLiteral)expr2);
          else if ((expr2 instanceof StringLiteral))
            this.expressionStack[this.expressionPtr] = 
              ((StringLiteral)expr1).extendWith((StringLiteral)expr2);
          else
            this.expressionStack[this.expressionPtr] = new BinaryExpression(expr1, expr2, 14);
        }
        else
          this.expressionStack[this.expressionPtr] = new BinaryExpression(expr1, expr2, 14);
      }
      else if ((expr1 instanceof StringLiteral)) {
        if ((expr2 instanceof StringLiteral))
        {
          this.expressionStack[this.expressionPtr] = 
            ((StringLiteral)expr1).extendsWith((StringLiteral)expr2);
        }
        else this.expressionStack[this.expressionPtr] = 
            new BinaryExpression(
            expr1, 
            expr2, 
            op);
      }
      else {
        this.expressionStack[this.expressionPtr] = 
          new BinaryExpression(
          expr1, 
          expr2, 
          op);
      }
      break;
    case 4:
      this.intPtr -= 1;
      this.expressionStack[this.expressionPtr] = 
        new BinaryExpression(
        expr1, 
        expr2, 
        op);
      break;
    default:
      this.expressionStack[this.expressionPtr] = 
        new BinaryExpression(
        expr1, 
        expr2, 
        op);
    }
  }

  protected void consumeBlock()
  {
    int statementsLength = this.astLengthStack[(this.astLengthPtr--)];
    Block block;
    if (statementsLength == 0) {
      Block block = new Block(0);
      block.sourceStart = this.intStack[(this.intPtr--)];
      block.sourceEnd = this.endStatementPosition;

      if (!containsComment(block.sourceStart, block.sourceEnd)) {
        block.bits |= 8;
      }
      this.realBlockPtr -= 1;
    } else {
      block = new Block(this.realBlockStack[(this.realBlockPtr--)]);
      this.astPtr -= statementsLength;
      System.arraycopy(
        this.astStack, 
        this.astPtr + 1, 
        block.statements = new Statement[statementsLength], 
        0, 
        statementsLength);
      block.sourceStart = this.intStack[(this.intPtr--)];
      block.sourceEnd = this.endStatementPosition;
    }
    pushOnAstStack(block);
  }

  protected void consumeBlockStatements() {
    concatNodeLists();
  }

  protected void consumeCaseLabel() {
    this.expressionLengthPtr -= 1;
    Expression expression = this.expressionStack[(this.expressionPtr--)];
    pushOnAstStack(new CaseStatement(expression, expression.sourceEnd, this.intStack[(this.intPtr--)]));
  }

  protected void consumeCastExpressionLL1()
  {
    this.expressionPtr -= 1;
    Expression exp;
    void tmp51_48 = new CastExpression(
      exp = this.expressionStack[(this.expressionPtr + 1)], 
      getTypeReference(this.expressionStack[this.expressionPtr]));

    Expression cast = tmp51_48;

    this.expressionStack[this.expressionPtr] = 
      tmp51_48;

    this.expressionLengthPtr -= 1;
    updateSourcePosition(cast);
    cast.sourceEnd = exp.sourceEnd;
  }

  protected void consumeCastExpressionWithGenericsArray()
  {
    int end = this.intStack[(this.intPtr--)];

    int dim = this.intStack[(this.intPtr--)];
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    Expression exp;
    Expression castType;
    void tmp83_80 = new CastExpression(exp = this.expressionStack[this.expressionPtr], castType = getTypeReference(dim)); Expression cast = tmp83_80; this.expressionStack[this.expressionPtr] = tmp83_80;
    this.intPtr -= 1;
    castType.sourceEnd = (end - 1);
    castType.sourceStart = ((cast.sourceStart = this.intStack[(this.intPtr--)]) + 1);
    cast.sourceEnd = exp.sourceEnd;
  }

  protected void consumeCastExpressionWithNameArray()
  {
    int end = this.intStack[(this.intPtr--)];

    pushOnGenericsLengthStack(0);
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    Expression exp;
    Expression castType;
    void tmp84_81 = new CastExpression(exp = this.expressionStack[this.expressionPtr], castType = getTypeReference(this.intStack[(this.intPtr--)])); Expression cast = tmp84_81; this.expressionStack[this.expressionPtr] = tmp84_81;
    castType.sourceEnd = (end - 1);
    castType.sourceStart = ((cast.sourceStart = this.intStack[(this.intPtr--)]) + 1);
    cast.sourceEnd = exp.sourceEnd;
  }

  protected void consumeCastExpressionWithPrimitiveType()
  {
    int end = this.intStack[(this.intPtr--)];
    Expression exp;
    Expression castType;
    void tmp66_63 = new CastExpression(exp = this.expressionStack[this.expressionPtr], castType = getTypeReference(this.intStack[(this.intPtr--)])); Expression cast = tmp66_63; this.expressionStack[this.expressionPtr] = tmp66_63;
    castType.sourceEnd = (end - 1);
    castType.sourceStart = ((cast.sourceStart = this.intStack[(this.intPtr--)]) + 1);
    cast.sourceEnd = exp.sourceEnd;
  }
  protected ParameterizedQualifiedTypeReference computeQualifiedGenericsFromRightSide(TypeReference rightSide, int dim) {
    int nameSize = this.identifierLengthStack[this.identifierLengthPtr];
    int tokensSize = nameSize;
    if ((rightSide instanceof ParameterizedSingleTypeReference))
      tokensSize++;
    else if ((rightSide instanceof SingleTypeReference))
      tokensSize++;
    else if ((rightSide instanceof ParameterizedQualifiedTypeReference))
      tokensSize += ((QualifiedTypeReference)rightSide).tokens.length;
    else if ((rightSide instanceof QualifiedTypeReference)) {
      tokensSize += ((QualifiedTypeReference)rightSide).tokens.length;
    }
    TypeReference[][] typeArguments = new TypeReference[tokensSize][];
    char[][] tokens = new char[tokensSize][];
    long[] positions = new long[tokensSize];
    if ((rightSide instanceof ParameterizedSingleTypeReference)) {
      ParameterizedSingleTypeReference singleParameterizedTypeReference = (ParameterizedSingleTypeReference)rightSide;
      tokens[nameSize] = singleParameterizedTypeReference.token;
      positions[nameSize] = ((singleParameterizedTypeReference.sourceStart << 32) + singleParameterizedTypeReference.sourceEnd);
      typeArguments[nameSize] = singleParameterizedTypeReference.typeArguments;
    } else if ((rightSide instanceof SingleTypeReference)) {
      SingleTypeReference singleTypeReference = (SingleTypeReference)rightSide;
      tokens[nameSize] = singleTypeReference.token;
      positions[nameSize] = ((singleTypeReference.sourceStart << 32) + singleTypeReference.sourceEnd);
    } else if ((rightSide instanceof ParameterizedQualifiedTypeReference)) {
      ParameterizedQualifiedTypeReference parameterizedTypeReference = (ParameterizedQualifiedTypeReference)rightSide;
      TypeReference[][] rightSideTypeArguments = parameterizedTypeReference.typeArguments;
      System.arraycopy(rightSideTypeArguments, 0, typeArguments, nameSize, rightSideTypeArguments.length);
      char[][] rightSideTokens = parameterizedTypeReference.tokens;
      System.arraycopy(rightSideTokens, 0, tokens, nameSize, rightSideTokens.length);
      long[] rightSidePositions = parameterizedTypeReference.sourcePositions;
      System.arraycopy(rightSidePositions, 0, positions, nameSize, rightSidePositions.length);
    } else if ((rightSide instanceof QualifiedTypeReference)) {
      QualifiedTypeReference qualifiedTypeReference = (QualifiedTypeReference)rightSide;
      char[][] rightSideTokens = qualifiedTypeReference.tokens;
      System.arraycopy(rightSideTokens, 0, tokens, nameSize, rightSideTokens.length);
      long[] rightSidePositions = qualifiedTypeReference.sourcePositions;
      System.arraycopy(rightSidePositions, 0, positions, nameSize, rightSidePositions.length);
    }

    int currentTypeArgumentsLength = this.genericsLengthStack[(this.genericsLengthPtr--)];
    TypeReference[] currentTypeArguments = new TypeReference[currentTypeArgumentsLength];
    this.genericsPtr -= currentTypeArgumentsLength;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, currentTypeArguments, 0, currentTypeArgumentsLength);

    if (nameSize == 1) {
      tokens[0] = this.identifierStack[this.identifierPtr];
      positions[0] = this.identifierPositionStack[(this.identifierPtr--)];
      typeArguments[0] = currentTypeArguments;
    } else {
      this.identifierPtr -= nameSize;
      System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, nameSize);
      System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, nameSize);
      typeArguments[(nameSize - 1)] = currentTypeArguments;
    }
    this.identifierLengthPtr -= 1;
    return new ParameterizedQualifiedTypeReference(tokens, typeArguments, dim, positions);
  }

  protected void consumeCastExpressionWithQualifiedGenericsArray()
  {
    int end = this.intStack[(this.intPtr--)];

    int dim = this.intStack[(this.intPtr--)];
    TypeReference rightSide = getTypeReference(0);

    ParameterizedQualifiedTypeReference qualifiedParameterizedTypeReference = computeQualifiedGenericsFromRightSide(rightSide, dim);
    this.intPtr -= 1;
    Expression exp;
    Expression castType;
    void tmp93_90 = new CastExpression(exp = this.expressionStack[this.expressionPtr], castType = qualifiedParameterizedTypeReference); Expression cast = tmp93_90; this.expressionStack[this.expressionPtr] = tmp93_90;
    castType.sourceEnd = (end - 1);
    castType.sourceStart = ((cast.sourceStart = this.intStack[(this.intPtr--)]) + 1);
    cast.sourceEnd = exp.sourceEnd;
  }

  protected void consumeCatches() {
    optimizedConcatNodeLists();
  }

  protected void consumeCatchHeader()
  {
    if (this.currentElement == null) {
      return;
    }

    if (!(this.currentElement instanceof RecoveredBlock)) {
      if (!(this.currentElement instanceof RecoveredMethod)) {
        return;
      }
      RecoveredMethod rMethod = (RecoveredMethod)this.currentElement;
      if ((rMethod.methodBody != null) || (rMethod.bracketBalance <= 0)) {
        return;
      }
    }

    Argument arg = (Argument)this.astStack[(this.astPtr--)];

    LocalDeclaration localDeclaration = new LocalDeclaration(arg.name, arg.sourceStart, arg.sourceEnd);
    localDeclaration.type = arg.type;
    localDeclaration.declarationSourceStart = arg.declarationSourceStart;
    localDeclaration.declarationSourceEnd = arg.declarationSourceEnd;

    this.currentElement = this.currentElement.add(localDeclaration, 0);
    this.lastCheckPoint = this.scanner.startPosition;
    this.restartRecovery = true;
    this.lastIgnoredToken = -1;
  }

  protected void consumeClassBodyDeclaration()
  {
    this.nestedMethod[this.nestedType] -= 1;
    Block block = (Block)this.astStack[this.astPtr];
    if (this.diet) block.bits &= -9;
    Initializer initializer = new Initializer(block, 0);
    this.intPtr -= 1;
    initializer.bodyStart = this.intStack[(this.intPtr--)];
    this.realBlockPtr -= 1;
    int javadocCommentStart = this.intStack[(this.intPtr--)];
    if (javadocCommentStart != -1) {
      initializer.declarationSourceStart = javadocCommentStart;
      initializer.javadoc = this.javadoc;
      this.javadoc = null;
    }
    this.astStack[this.astPtr] = initializer;
    initializer.bodyEnd = this.endPosition;
    initializer.sourceEnd = this.endStatementPosition;
    initializer.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeClassBodyDeclarations() {
    concatNodeLists();
  }

  protected void consumeClassBodyDeclarationsopt() {
    this.nestedType -= 1;
  }
  protected void consumeAnnotationTypeMemberDeclarationsopt() {
    this.nestedType -= 1;
  }

  protected void consumeClassBodyopt() {
    pushOnAstStack(null);
    this.endPosition = this.rParenPos;
  }

  protected void consumeClassDeclaration()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)
    {
      dispatchDeclarationInto(length);
    }

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];

    markInitializersWithLocalType(typeDecl);

    boolean hasConstructor = typeDecl.checkConstructors(this);

    if (!hasConstructor) {
      switch (TypeDeclaration.kind(typeDecl.modifiers)) {
      case 1:
      case 3:
        boolean insideFieldInitializer = false;
        if (this.diet) {
          for (int i = this.nestedType; i > 0; i--) {
            if (this.variablesCounter[i] > 0) {
              insideFieldInitializer = true;
              break;
            }
          }
        }
        typeDecl.createDefaultConstructor((!this.diet) || (insideFieldInitializer), true);
      case 2:
      }
    }
    if (this.scanner.containsAssertKeyword) {
      typeDecl.bits |= 1;
    }
    typeDecl.addClinit();
    typeDecl.bodyEnd = this.endStatementPosition;
    if ((length == 0) && (!containsComment(typeDecl.bodyStart, typeDecl.bodyEnd))) {
      typeDecl.bits |= 8;
    }

    typeDecl.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeClassHeader()
  {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (this.currentToken == 69) {
      typeDecl.bodyStart = this.scanner.currentPosition;
    }
    if (this.currentElement != null) {
      this.restartRecovery = true;
    }

    this.scanner.commentPtr = -1;
  }

  protected void consumeClassHeaderExtends()
  {
    TypeReference superClass = getTypeReference(0);

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    typeDecl.superclass = superClass;
    superClass.bits |= 16;
    typeDecl.bodyStart = (typeDecl.superclass.sourceEnd + 1);

    if (this.currentElement != null)
      this.lastCheckPoint = typeDecl.bodyStart;
  }

  protected void consumeClassHeaderImplements()
  {
    int length = this.astLengthStack[(this.astLengthPtr--)];

    this.astPtr -= length;

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    System.arraycopy(
      this.astStack, 
      this.astPtr + 1, 
      typeDecl.superInterfaces = new TypeReference[length], 
      0, 
      length);
    int i = 0; for (int max = typeDecl.superInterfaces.length; i < max; i++) {
      typeDecl.superInterfaces[i].bits |= 16;
    }
    typeDecl.bodyStart = (typeDecl.superInterfaces[(length - 1)].sourceEnd + 1);
    this.listLength = 0;

    if (this.currentElement != null)
      this.lastCheckPoint = typeDecl.bodyStart;
  }

  protected void consumeClassHeaderName1()
  {
    TypeDeclaration typeDecl = new TypeDeclaration(this.compilationUnit.compilationResult);
    if (this.nestedMethod[this.nestedType] == 0) {
      if (this.nestedType != 0)
        typeDecl.bits |= 1024;
    }
    else
    {
      typeDecl.bits |= 256;
      markEnclosingMemberWithLocalType();
      blockReal();
    }

    long pos = this.identifierPositionStack[this.identifierPtr];
    typeDecl.sourceEnd = ((int)pos);
    typeDecl.sourceStart = ((int)(pos >>> 32));
    typeDecl.name = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    typeDecl.declarationSourceStart = this.intStack[(this.intPtr--)];
    this.intPtr -= 1;

    typeDecl.modifiersSourceStart = this.intStack[(this.intPtr--)];
    typeDecl.modifiers = this.intStack[(this.intPtr--)];
    if (typeDecl.modifiersSourceStart >= 0) {
      typeDecl.declarationSourceStart = typeDecl.modifiersSourceStart;
    }

    if (((typeDecl.bits & 0x400) == 0) && ((typeDecl.bits & 0x100) == 0) && 
      (this.compilationUnit != null) && (!CharOperation.equals(typeDecl.name, this.compilationUnit.getMainTypeName())))
      typeDecl.bits |= 4096;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        typeDecl.annotations = new Annotation[length], 
        0, 
        length);
    }
    typeDecl.bodyStart = (typeDecl.sourceEnd + 1);
    pushOnAstStack(typeDecl);

    this.listLength = 0;

    if (this.currentElement != null) {
      this.lastCheckPoint = typeDecl.bodyStart;
      this.currentElement = this.currentElement.add(typeDecl, 0);
      this.lastIgnoredToken = -1;
    }

    typeDecl.javadoc = this.javadoc;
    this.javadoc = null;
  }

  protected void consumeTypeHeaderNameWithTypeParameters()
  {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, typeDecl.typeParameters = new TypeParameter[length], 0, length);

    typeDecl.bodyStart = (typeDecl.typeParameters[(length - 1)].declarationSourceEnd + 1);

    this.listTypeParameterLength = 0;

    if (this.currentElement != null) {
      RecoveredType recoveredType = (RecoveredType)this.currentElement;
      recoveredType.pendingTypeParameters = null;

      this.lastCheckPoint = typeDecl.bodyStart;
    }
  }

  protected void consumeClassInstanceCreationExpression() {
    classInstanceCreation(false);
  }

  protected void consumeClassInstanceCreationExpressionName() {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
  }

  protected void consumeClassInstanceCreationExpressionQualified()
  {
    classInstanceCreation(true);

    this.expressionLengthPtr -= 1;
    QualifiedAllocationExpression qae = 
      (QualifiedAllocationExpression)this.expressionStack[(this.expressionPtr--)];
    qae.enclosingInstance = this.expressionStack[this.expressionPtr];
    this.expressionStack[this.expressionPtr] = qae;
    qae.sourceStart = qae.enclosingInstance.sourceStart;
  }

  protected void consumeClassInstanceCreationExpressionQualifiedWithTypeArguments()
  {
    int length;
    if (((length = this.astLengthStack[(this.astLengthPtr--)]) == 1) && (this.astStack[this.astPtr] == null))
    {
      this.astPtr -= 1;
      QualifiedAllocationExpression alloc = new QualifiedAllocationExpression();
      alloc.sourceEnd = this.endPosition;

      if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
        this.expressionPtr -= length;
        System.arraycopy(
          this.expressionStack, 
          this.expressionPtr + 1, 
          alloc.arguments = new Expression[length], 
          0, 
          length);
      }
      alloc.type = getTypeReference(0);

      length = this.genericsLengthStack[(this.genericsLengthPtr--)];
      this.genericsPtr -= length;
      System.arraycopy(this.genericsStack, this.genericsPtr + 1, alloc.typeArguments = new TypeReference[length], 0, length);
      this.intPtr -= 1;

      alloc.sourceStart = this.intStack[(this.intPtr--)];
      pushOnExpressionStack(alloc);
    } else {
      dispatchDeclarationInto(length);
      TypeDeclaration anonymousTypeDeclaration = (TypeDeclaration)this.astStack[this.astPtr];
      anonymousTypeDeclaration.declarationSourceEnd = this.endStatementPosition;
      anonymousTypeDeclaration.bodyEnd = this.endStatementPosition;
      if ((length == 0) && (!containsComment(anonymousTypeDeclaration.bodyStart, anonymousTypeDeclaration.bodyEnd))) {
        anonymousTypeDeclaration.bits |= 8;
      }
      this.astPtr -= 1;
      this.astLengthPtr -= 1;

      QualifiedAllocationExpression allocationExpression = anonymousTypeDeclaration.allocation;
      if (allocationExpression != null) {
        allocationExpression.sourceEnd = this.endStatementPosition;

        length = this.genericsLengthStack[(this.genericsLengthPtr--)];
        this.genericsPtr -= length;
        System.arraycopy(this.genericsStack, this.genericsPtr + 1, allocationExpression.typeArguments = new TypeReference[length], 0, length);
        allocationExpression.sourceStart = this.intStack[(this.intPtr--)];
      }

      markInitializersWithLocalType(anonymousTypeDeclaration);
    }

    this.expressionLengthPtr -= 1;
    QualifiedAllocationExpression qae = 
      (QualifiedAllocationExpression)this.expressionStack[(this.expressionPtr--)];
    qae.enclosingInstance = this.expressionStack[this.expressionPtr];
    this.expressionStack[this.expressionPtr] = qae;
    qae.sourceStart = qae.enclosingInstance.sourceStart;
  }

  protected void consumeClassInstanceCreationExpressionWithTypeArguments()
  {
    int length;
    if (((length = this.astLengthStack[(this.astLengthPtr--)]) == 1) && 
      (this.astStack[this.astPtr] == null))
    {
      this.astPtr -= 1;
      AllocationExpression alloc = new AllocationExpression();
      alloc.sourceEnd = this.endPosition;

      if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
        this.expressionPtr -= length;
        System.arraycopy(
          this.expressionStack, 
          this.expressionPtr + 1, 
          alloc.arguments = new Expression[length], 
          0, 
          length);
      }
      alloc.type = getTypeReference(0);

      length = this.genericsLengthStack[(this.genericsLengthPtr--)];
      this.genericsPtr -= length;
      System.arraycopy(this.genericsStack, this.genericsPtr + 1, alloc.typeArguments = new TypeReference[length], 0, length);
      this.intPtr -= 1;

      alloc.sourceStart = this.intStack[(this.intPtr--)];
      pushOnExpressionStack(alloc);
    } else {
      dispatchDeclarationInto(length);
      TypeDeclaration anonymousTypeDeclaration = (TypeDeclaration)this.astStack[this.astPtr];
      anonymousTypeDeclaration.declarationSourceEnd = this.endStatementPosition;
      anonymousTypeDeclaration.bodyEnd = this.endStatementPosition;
      if ((length == 0) && (!containsComment(anonymousTypeDeclaration.bodyStart, anonymousTypeDeclaration.bodyEnd))) {
        anonymousTypeDeclaration.bits |= 8;
      }
      this.astPtr -= 1;
      this.astLengthPtr -= 1;

      QualifiedAllocationExpression allocationExpression = anonymousTypeDeclaration.allocation;
      if (allocationExpression != null) {
        allocationExpression.sourceEnd = this.endStatementPosition;

        length = this.genericsLengthStack[(this.genericsLengthPtr--)];
        this.genericsPtr -= length;
        System.arraycopy(this.genericsStack, this.genericsPtr + 1, allocationExpression.typeArguments = new TypeReference[length], 0, length);
        allocationExpression.sourceStart = this.intStack[(this.intPtr--)];
      }

      markInitializersWithLocalType(anonymousTypeDeclaration);
    }
  }

  protected void consumeClassOrInterface() { this.genericsIdentifiersLengthStack[this.genericsIdentifiersLengthPtr] += this.identifierLengthStack[this.identifierLengthPtr];
    pushOnGenericsLengthStack(0); }

  protected void consumeClassOrInterfaceName() {
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    pushOnGenericsLengthStack(0);
  }

  protected void consumeClassTypeElt() {
    pushOnAstStack(getTypeReference(0));

    this.listLength += 1;
  }

  protected void consumeClassTypeList() {
    optimizedConcatNodeLists();
  }

  protected void consumeCompilationUnit()
  {
  }

  protected void consumeConditionalExpression(int op)
  {
    this.intPtr -= 2;
    this.expressionPtr -= 2;
    this.expressionLengthPtr -= 2;
    this.expressionStack[this.expressionPtr] = 
      new ConditionalExpression(
      this.expressionStack[this.expressionPtr], 
      this.expressionStack[(this.expressionPtr + 1)], 
      this.expressionStack[(this.expressionPtr + 2)]);
  }

  protected void consumeConditionalExpressionWithName(int op)
  {
    this.intPtr -= 2;
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
    this.expressionPtr -= 2;
    this.expressionLengthPtr -= 2;
    this.expressionStack[this.expressionPtr] = 
      new ConditionalExpression(
      this.expressionStack[(this.expressionPtr + 2)], 
      this.expressionStack[this.expressionPtr], 
      this.expressionStack[(this.expressionPtr + 1)]);
  }

  protected void consumeConstructorBlockStatements() {
    concatNodeLists();
  }

  protected void consumeConstructorBody()
  {
    this.nestedMethod[this.nestedType] -= 1;
  }

  protected void consumeConstructorDeclaration()
  {
    this.intPtr -= 1;
    this.intPtr -= 1;

    this.realBlockPtr -= 1;
    ExplicitConstructorCall constructorCall = null;
    Statement[] statements = (Statement[])null;
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      this.astPtr -= length;
      if ((this.astStack[(this.astPtr + 1)] instanceof ExplicitConstructorCall))
      {
        System.arraycopy(
          this.astStack, 
          this.astPtr + 2, 
          statements = new Statement[length - 1], 
          0, 
          length - 1);
        constructorCall = (ExplicitConstructorCall)this.astStack[(this.astPtr + 1)];
      } else {
        System.arraycopy(
          this.astStack, 
          this.astPtr + 1, 
          statements = new Statement[length], 
          0, 
          length);
        constructorCall = SuperReference.implicitSuperConstructorCall();
      }
    } else {
      boolean insideFieldInitializer = false;
      if (this.diet) {
        for (int i = this.nestedType; i > 0; i--) {
          if (this.variablesCounter[i] > 0) {
            insideFieldInitializer = true;
            break;
          }
        }
      }

      if ((!this.diet) || (insideFieldInitializer))
      {
        constructorCall = SuperReference.implicitSuperConstructorCall();
      }

    }

    ConstructorDeclaration cd = (ConstructorDeclaration)this.astStack[this.astPtr];
    cd.constructorCall = constructorCall;
    cd.statements = statements;

    if ((constructorCall != null) && (cd.constructorCall.sourceEnd == 0)) {
      cd.constructorCall.sourceEnd = cd.sourceEnd;
      cd.constructorCall.sourceStart = cd.sourceStart;
    }

    if (((!this.diet) || (this.dietInt != 0)) && 
      (statements == null) && 
      ((constructorCall == null) || (constructorCall.isImplicitSuper())) && 
      (!containsComment(cd.bodyStart, this.endPosition))) {
      cd.bits |= 8;
    }

    cd.bodyEnd = this.endPosition;
    cd.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeConstructorHeader()
  {
    AbstractMethodDeclaration method = (AbstractMethodDeclaration)this.astStack[this.astPtr];

    if (this.currentToken == 69) {
      method.bodyStart = this.scanner.currentPosition;
    }

    if (this.currentElement != null) {
      if (this.currentToken == 27) {
        method.modifiers |= 16777216;
        method.declarationSourceEnd = (this.scanner.currentPosition - 1);
        method.bodyEnd = (this.scanner.currentPosition - 1);
        if ((this.currentElement.parseTree() == method) && (this.currentElement.parent != null)) {
          this.currentElement = this.currentElement.parent;
        }
      }
      this.restartRecovery = true;
    }
  }

  protected void consumeConstructorHeaderName()
  {
    if ((this.currentElement != null) && 
      (this.lastIgnoredToken == 43)) {
      this.lastCheckPoint = this.scanner.startPosition;
      this.restartRecovery = true;
      return;
    }

    ConstructorDeclaration cd = new ConstructorDeclaration(this.compilationUnit.compilationResult);

    cd.selector = this.identifierStack[this.identifierPtr];
    long selectorSource = this.identifierPositionStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    cd.declarationSourceStart = this.intStack[(this.intPtr--)];
    cd.modifiers = this.intStack[(this.intPtr--)];
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        cd.annotations = new Annotation[length], 
        0, 
        length);
    }

    cd.javadoc = this.javadoc;
    this.javadoc = null;

    cd.sourceStart = ((int)(selectorSource >>> 32));
    pushOnAstStack(cd);
    cd.sourceEnd = this.lParenPos;
    cd.bodyStart = (this.lParenPos + 1);
    this.listLength = 0;

    if (this.currentElement != null) {
      this.lastCheckPoint = cd.bodyStart;
      if ((((this.currentElement instanceof RecoveredType)) && (this.lastIgnoredToken != 3)) || 
        (cd.modifiers != 0)) {
        this.currentElement = this.currentElement.add(cd, 0);
        this.lastIgnoredToken = -1;
      }
    }
  }

  protected void consumeConstructorHeaderNameWithTypeParameters()
  {
    if ((this.currentElement != null) && 
      (this.lastIgnoredToken == 43)) {
      this.lastCheckPoint = this.scanner.startPosition;
      this.restartRecovery = true;
      return;
    }

    ConstructorDeclaration cd = new ConstructorDeclaration(this.compilationUnit.compilationResult);

    cd.selector = this.identifierStack[this.identifierPtr];
    long selectorSource = this.identifierPositionStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, cd.typeParameters = new TypeParameter[length], 0, length);

    cd.declarationSourceStart = this.intStack[(this.intPtr--)];
    cd.modifiers = this.intStack[(this.intPtr--)];

    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        cd.annotations = new Annotation[length], 
        0, 
        length);
    }

    cd.javadoc = this.javadoc;
    this.javadoc = null;

    cd.sourceStart = ((int)(selectorSource >>> 32));
    pushOnAstStack(cd);
    cd.sourceEnd = this.lParenPos;
    cd.bodyStart = (this.lParenPos + 1);
    this.listLength = 0;

    if (this.currentElement != null) {
      this.lastCheckPoint = cd.bodyStart;
      if ((((this.currentElement instanceof RecoveredType)) && (this.lastIgnoredToken != 3)) || 
        (cd.modifiers != 0)) {
        this.currentElement = this.currentElement.add(cd, 0);
        this.lastIgnoredToken = -1;
      }
    }
  }

  protected void consumeDefaultLabel() {
    pushOnAstStack(new CaseStatement(null, this.intStack[(this.intPtr--)], this.intStack[(this.intPtr--)]));
  }
  protected void consumeDefaultModifiers() {
    checkComment();
    pushOnIntStack(this.modifiers);
    pushOnIntStack(
      this.modifiersSourceStart >= 0 ? this.modifiersSourceStart : this.scanner.startPosition);
    resetModifiers();
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeDiet() {
    checkComment();
    pushOnIntStack(this.modifiersSourceStart);
    resetModifiers();
    jumpOverMethodBody();
  }

  protected void consumeDims() {
    pushOnIntStack(this.dimensions);
    this.dimensions = 0;
  }

  protected void consumeDimWithOrWithOutExpr() {
    pushOnExpressionStack(null);

    if ((this.currentElement != null) && (this.currentToken == 69)) {
      this.ignoreNextOpeningBrace = true;
      this.currentElement.bracketBalance += 1;
    }
  }

  protected void consumeDimWithOrWithOutExprs() {
    concatExpressionLists();
  }

  protected void consumeEmptyAnnotationTypeMemberDeclarationsopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyArgumentListopt() {
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeEmptyArguments() {
    FieldDeclaration fieldDeclaration = (FieldDeclaration)this.astStack[this.astPtr];
    pushOnIntStack(fieldDeclaration.sourceEnd);
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeEmptyArrayInitializer() {
    arrayInitializer(0);
  }

  protected void consumeEmptyArrayInitializeropt() {
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeEmptyBlockStatementsopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyCatchesopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyClassBodyDeclarationsopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyMethodHeaderDefaultValue() {
    AbstractMethodDeclaration method = (AbstractMethodDeclaration)this.astStack[this.astPtr];
    if (method.isAnnotationMethod()) {
      pushOnExpressionStackLengthStack(0);
    }
    this.recordStringLiterals = true;
  }

  protected void consumeEmptyDimsopt() {
    pushOnIntStack(0);
  }

  protected void consumeEmptyEnumDeclarations() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyExpression() {
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeEmptyForInitopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyForUpdateopt() {
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumeEmptyInterfaceMemberDeclarationsopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyInternalCompilationUnit()
  {
    if (this.compilationUnit.isPackageInfo()) {
      this.compilationUnit.types = new TypeDeclaration[1];

      TypeDeclaration declaration = new TypeDeclaration(this.compilationUnit.compilationResult);
      declaration.name = TypeConstants.PACKAGE_INFO_NAME;
      declaration.modifiers = 512;
      this.compilationUnit.types[0] = declaration;
      declaration.javadoc = this.compilationUnit.javadoc;
    }
  }

  protected void consumeEmptyMemberValuePairsopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyMemberValueArrayInitializer()
  {
    arrayInitializer(0);
  }

  protected void consumeEmptyStatement() {
    char[] source = this.scanner.source;
    if (source[this.endStatementPosition] == ';') {
      pushOnAstStack(new EmptyStatement(this.endStatementPosition, this.endStatementPosition));
    } else {
      if (source.length > 5) {
        int c1 = 0; int c2 = 0; int c3 = 0; int c4 = 0;
        int pos = this.endStatementPosition - 4;
        while (source[pos] == 'u') {
          pos--;
        }
        if ((source[pos] == '\\') && 
          ((c1 = ScannerHelper.getNumericValue(source[(this.endStatementPosition - 3)])) <= 15) && 
          (c1 >= 0) && 
          ((c2 = ScannerHelper.getNumericValue(source[(this.endStatementPosition - 2)])) <= 15) && 
          (c2 >= 0) && 
          ((c3 = ScannerHelper.getNumericValue(source[(this.endStatementPosition - 1)])) <= 15) && 
          (c3 >= 0) && 
          ((c4 = ScannerHelper.getNumericValue(source[this.endStatementPosition])) <= 15) && 
          (c4 >= 0) && 
          ((char)(((c1 * 16 + c2) * 16 + c3) * 16 + c4) == ';'))
        {
          pushOnAstStack(new EmptyStatement(pos, this.endStatementPosition));
          return;
        }
      }
      pushOnAstStack(new EmptyStatement(this.endStatementPosition, this.endStatementPosition));
    }
  }

  protected void consumeEmptySwitchBlock() {
    pushOnAstLengthStack(0);
  }

  protected void consumeEmptyTypeDeclaration()
  {
    pushOnAstLengthStack(0);
    if (!this.statementRecoveryActivated) problemReporter().superfluousSemicolon(this.endPosition + 1, this.endStatementPosition);
    flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeEnhancedForStatementHeaderInit(boolean hasModifiers)
  {
    char[] identifierName = this.identifierStack[this.identifierPtr];
    long namePosition = this.identifierPositionStack[this.identifierPtr];

    LocalDeclaration localDeclaration = createLocalDeclaration(identifierName, (int)(namePosition >>> 32), (int)namePosition);
    localDeclaration.declarationSourceEnd = localDeclaration.declarationEnd;

    int extraDims = this.intStack[(this.intPtr--)];
    this.identifierPtr -= 1;
    this.identifierLengthPtr -= 1;

    int declarationSourceStart = 0;
    int modifiersValue = 0;
    if (hasModifiers) {
      declarationSourceStart = this.intStack[(this.intPtr--)];
      modifiersValue = this.intStack[(this.intPtr--)];
    } else {
      this.intPtr -= 2;
    }

    TypeReference type = getTypeReference(this.intStack[(this.intPtr--)] + extraDims);
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        localDeclaration.annotations = new Annotation[length], 
        0, 
        length);
    }
    if (hasModifiers) {
      localDeclaration.declarationSourceStart = declarationSourceStart;
      localDeclaration.modifiers = modifiersValue;
    } else {
      localDeclaration.declarationSourceStart = type.sourceStart;
    }
    localDeclaration.type = type;

    ForeachStatement iteratorForStatement = 
      new ForeachStatement(
      localDeclaration, 
      this.intStack[(this.intPtr--)]);
    pushOnAstStack(iteratorForStatement);

    iteratorForStatement.sourceEnd = localDeclaration.declarationSourceEnd;
  }

  protected void consumeEnhancedForStatementHeader() {
    ForeachStatement statement = (ForeachStatement)this.astStack[this.astPtr];

    this.expressionLengthPtr -= 1;
    Expression collection = this.expressionStack[(this.expressionPtr--)];
    statement.collection = collection;
    statement.sourceEnd = this.rParenPos;

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition))
      problemReporter().invalidUsageOfForeachStatements(statement.elementVariable, collection);
  }

  protected void consumeEnhancedForStatement()
  {
    this.astLengthPtr -= 1;
    Statement statement = (Statement)this.astStack[(this.astPtr--)];

    ForeachStatement foreachStatement = (ForeachStatement)this.astStack[this.astPtr];
    foreachStatement.action = statement;

    if ((statement instanceof EmptyStatement)) statement.bits |= 1;

    foreachStatement.sourceEnd = this.endStatementPosition;
  }

  protected void consumeEnterAnonymousClassBody() {
    TypeReference typeReference = getTypeReference(0);

    TypeDeclaration anonymousType = new TypeDeclaration(this.compilationUnit.compilationResult);
    anonymousType.name = CharOperation.NO_CHAR;
    anonymousType.bits |= 768;
    QualifiedAllocationExpression alloc = new QualifiedAllocationExpression(anonymousType);
    markEnclosingMemberWithLocalType();
    pushOnAstStack(anonymousType);

    alloc.sourceEnd = this.rParenPos;
    int argumentLength;
    if ((argumentLength = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      this.expressionPtr -= argumentLength;
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr + 1, 
        alloc.arguments = new Expression[argumentLength], 
        0, 
        argumentLength);
    }
    alloc.type = typeReference;

    anonymousType.sourceEnd = alloc.sourceEnd;

    anonymousType.sourceStart = (anonymousType.declarationSourceStart = alloc.type.sourceStart);
    alloc.sourceStart = this.intStack[(this.intPtr--)];
    pushOnExpressionStack(alloc);

    anonymousType.bodyStart = this.scanner.currentPosition;
    this.listLength = 0;

    this.scanner.commentPtr = -1;

    if (this.currentElement != null) {
      this.lastCheckPoint = anonymousType.bodyStart;
      this.currentElement = this.currentElement.add(anonymousType, 0);
      this.currentToken = 0;
      this.lastIgnoredToken = -1;
    }
  }

  protected void consumeEnterCompilationUnit()
  {
  }

  protected void consumeEnterMemberValue()
  {
  }

  protected void consumeEnterMemberValueArrayInitializer() {
    if (this.currentElement != null) {
      this.ignoreNextOpeningBrace = true;
      this.currentElement.bracketBalance += 1;
    }
  }

  protected void consumeEnterVariable()
  {
    char[] identifierName = this.identifierStack[this.identifierPtr];
    long namePosition = this.identifierPositionStack[this.identifierPtr];
    int extendedDimension = this.intStack[(this.intPtr--)];

    boolean isLocalDeclaration = this.nestedMethod[this.nestedType] != 0;
    AbstractVariableDeclaration declaration;
    AbstractVariableDeclaration declaration;
    if (isLocalDeclaration)
    {
      declaration = 
        createLocalDeclaration(identifierName, (int)(namePosition >>> 32), (int)namePosition);
    }
    else {
      declaration = 
        createFieldDeclaration(identifierName, (int)(namePosition >>> 32), (int)namePosition);
    }

    this.identifierPtr -= 1;
    this.identifierLengthPtr -= 1;

    int variableIndex = this.variablesCounter[this.nestedType];
    int typeDim = 0;
    TypeReference type;
    if (variableIndex == 0)
    {
      if (isLocalDeclaration) {
        declaration.declarationSourceStart = this.intStack[(this.intPtr--)];
        declaration.modifiers = this.intStack[(this.intPtr--)];
        int length;
        if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
          System.arraycopy(
            this.expressionStack, 
            this.expressionPtr -= length + 1, 
            declaration.annotations = new Annotation[length], 
            0, 
            length);
        }
        TypeReference type = getTypeReference(typeDim = this.intStack[(this.intPtr--)]);
        if (declaration.declarationSourceStart == -1)
        {
          declaration.declarationSourceStart = type.sourceStart;
        }
        pushOnAstStack(type);
      } else {
        TypeReference type = getTypeReference(typeDim = this.intStack[(this.intPtr--)]);
        pushOnAstStack(type);
        declaration.declarationSourceStart = this.intStack[(this.intPtr--)];
        declaration.modifiers = this.intStack[(this.intPtr--)];
        int length;
        if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
          System.arraycopy(
            this.expressionStack, 
            this.expressionPtr -= length + 1, 
            declaration.annotations = new Annotation[length], 
            0, 
            length);
        }

        FieldDeclaration fieldDeclaration = (FieldDeclaration)declaration;
        fieldDeclaration.javadoc = this.javadoc;
        this.javadoc = null;
      }
    } else {
      type = (TypeReference)this.astStack[(this.astPtr - variableIndex)];
      typeDim = type.dimensions();
      AbstractVariableDeclaration previousVariable = 
        (AbstractVariableDeclaration)this.astStack[this.astPtr];
      declaration.declarationSourceStart = previousVariable.declarationSourceStart;
      declaration.modifiers = previousVariable.modifiers;
      Annotation[] annotations = previousVariable.annotations;
      if (annotations != null) {
        int annotationsLength = annotations.length;
        System.arraycopy(annotations, 0, declaration.annotations = new Annotation[annotationsLength], 0, annotationsLength);
      }
    }

    if (extendedDimension == 0) {
      declaration.type = type;
    } else {
      int dimension = typeDim + extendedDimension;
      declaration.type = copyDims(type, dimension);
    }
    this.variablesCounter[this.nestedType] += 1;
    pushOnAstStack(declaration);

    if (this.currentElement != null) {
      if ((!(this.currentElement instanceof RecoveredType)) && (
        (this.currentToken == 3) || 
        (Util.getLineNumber(declaration.type.sourceStart, this.scanner.lineEnds, 0, this.scanner.linePtr) != 
        Util.getLineNumber((int)(namePosition >>> 32), this.scanner.lineEnds, 0, this.scanner.linePtr)))) {
        this.lastCheckPoint = ((int)(namePosition >>> 32));
        this.restartRecovery = true;
        return;
      }
      if (isLocalDeclaration) {
        LocalDeclaration localDecl = (LocalDeclaration)this.astStack[this.astPtr];
        this.lastCheckPoint = (localDecl.sourceEnd + 1);
        this.currentElement = this.currentElement.add(localDecl, 0);
      } else {
        FieldDeclaration fieldDecl = (FieldDeclaration)this.astStack[this.astPtr];
        this.lastCheckPoint = (fieldDecl.sourceEnd + 1);
        this.currentElement = this.currentElement.add(fieldDecl, 0);
      }
      this.lastIgnoredToken = -1;
    }
  }

  protected void consumeEnumBodyNoConstants()
  {
  }

  protected void consumeEnumBodyWithConstants() {
    concatNodeLists();
  }
  protected void consumeEnumConstantHeaderName() {
    if ((this.currentElement != null) && (
      ((!(this.currentElement instanceof RecoveredType)) && (
      (!(this.currentElement instanceof RecoveredField)) || (((RecoveredField)this.currentElement).fieldDeclaration.type != null))) || 
      (this.lastIgnoredToken == 3))) {
      this.lastCheckPoint = this.scanner.startPosition;
      this.restartRecovery = true;
      return;
    }

    long namePosition = this.identifierPositionStack[this.identifierPtr];
    char[] constantName = this.identifierStack[this.identifierPtr];
    int sourceEnd = (int)namePosition;
    FieldDeclaration enumConstant = createFieldDeclaration(constantName, (int)(namePosition >>> 32), sourceEnd);
    this.identifierPtr -= 1;
    this.identifierLengthPtr -= 1;
    enumConstant.modifiersSourceStart = this.intStack[(this.intPtr--)];
    enumConstant.modifiers = this.intStack[(this.intPtr--)];
    enumConstant.declarationSourceStart = enumConstant.modifiersSourceStart;

    if (((enumConstant.bits & 0x400) == 0) && ((enumConstant.bits & 0x100) == 0) && 
      (this.compilationUnit != null) && (!CharOperation.equals(enumConstant.name, this.compilationUnit.getMainTypeName())))
      enumConstant.bits |= 4096;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        enumConstant.annotations = new Annotation[length], 
        0, 
        length);
    }
    pushOnAstStack(enumConstant);
    if (this.currentElement != null) {
      this.lastCheckPoint = (enumConstant.sourceEnd + 1);
      this.currentElement = this.currentElement.add(enumConstant, 0);
    }

    enumConstant.javadoc = this.javadoc;
    this.javadoc = null;
  }
  protected void consumeEnumConstantHeader() {
    FieldDeclaration enumConstant = (FieldDeclaration)this.astStack[this.astPtr];
    boolean foundOpeningBrace = this.currentToken == 69;
    if (foundOpeningBrace)
    {
      TypeDeclaration anonymousType = new TypeDeclaration(this.compilationUnit.compilationResult);
      anonymousType.name = CharOperation.NO_CHAR;
      anonymousType.bits |= 768;
      int start = this.scanner.startPosition;
      anonymousType.declarationSourceStart = start;
      anonymousType.sourceStart = start;
      anonymousType.sourceEnd = start;
      anonymousType.modifiers = 0;
      anonymousType.bodyStart = this.scanner.currentPosition;
      markEnclosingMemberWithLocalType();
      pushOnAstStack(anonymousType);
      QualifiedAllocationExpression allocationExpression = new QualifiedAllocationExpression(anonymousType);
      allocationExpression.enumConstant = enumConstant;
      int length;
      if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
        this.expressionPtr -= length;
        System.arraycopy(
          this.expressionStack, 
          this.expressionPtr + 1, 
          allocationExpression.arguments = new Expression[length], 
          0, 
          length);
      }
      enumConstant.initialization = allocationExpression;
    } else {
      AllocationExpression allocationExpression = new AllocationExpression();
      allocationExpression.enumConstant = enumConstant;
      int length;
      if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
        this.expressionPtr -= length;
        System.arraycopy(
          this.expressionStack, 
          this.expressionPtr + 1, 
          allocationExpression.arguments = new Expression[length], 
          0, 
          length);
      }
      enumConstant.initialization = allocationExpression;
    }

    if (this.currentElement != null)
      if (foundOpeningBrace) {
        TypeDeclaration anonymousType = (TypeDeclaration)this.astStack[this.astPtr];
        this.currentElement = this.currentElement.add(anonymousType, 0);
        this.lastCheckPoint = anonymousType.bodyStart;
        this.lastIgnoredToken = -1;
        this.currentToken = 0;
      } else {
        if (this.currentToken == 27) {
          RecoveredType currentType = currentRecoveryType();
          if (currentType != null) {
            currentType.insideEnumConstantPart = false;
          }
        }
        this.lastCheckPoint = this.scanner.startPosition;
        this.lastIgnoredToken = -1;
        this.restartRecovery = true;
      }
  }

  protected void consumeEnumConstantNoClassBody()
  {
    int endOfEnumConstant = this.intStack[(this.intPtr--)];
    FieldDeclaration fieldDeclaration = (FieldDeclaration)this.astStack[this.astPtr];
    fieldDeclaration.declarationEnd = endOfEnumConstant;
    fieldDeclaration.declarationSourceEnd = endOfEnumConstant;
  }
  protected void consumeEnumConstants() {
    concatNodeLists();
  }
  protected void consumeEnumConstantWithClassBody() {
    dispatchDeclarationInto(this.astLengthStack[(this.astLengthPtr--)]);
    TypeDeclaration anonymousType = (TypeDeclaration)this.astStack[(this.astPtr--)];
    this.astLengthPtr -= 1;
    anonymousType.bodyEnd = this.endPosition;
    anonymousType.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
    FieldDeclaration fieldDeclaration = (FieldDeclaration)this.astStack[this.astPtr];
    fieldDeclaration.declarationEnd = this.endStatementPosition;
    fieldDeclaration.declarationSourceEnd = anonymousType.declarationSourceEnd;
    this.intPtr -= 1;
  }

  protected void consumeEnumDeclaration()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)
    {
      dispatchDeclarationIntoEnumDeclaration(length);
    }

    TypeDeclaration enumDeclaration = (TypeDeclaration)this.astStack[this.astPtr];

    markInitializersWithLocalType(enumDeclaration);

    boolean hasConstructor = enumDeclaration.checkConstructors(this);

    if (!hasConstructor) {
      boolean insideFieldInitializer = false;
      if (this.diet) {
        for (int i = this.nestedType; i > 0; i--) {
          if (this.variablesCounter[i] > 0) {
            insideFieldInitializer = true;
            break;
          }
        }
      }
      enumDeclaration.createDefaultConstructor((!this.diet) || (insideFieldInitializer), true);
    }

    if (this.scanner.containsAssertKeyword) {
      enumDeclaration.bits |= 1;
    }
    enumDeclaration.addClinit();
    enumDeclaration.bodyEnd = this.endStatementPosition;
    if ((length == 0) && (!containsComment(enumDeclaration.bodyStart, enumDeclaration.bodyEnd))) {
      enumDeclaration.bits |= 8;
    }

    enumDeclaration.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }
  protected void consumeEnumDeclarations() {
  }

  protected void consumeEnumHeader() {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (this.currentToken == 69) {
      typeDecl.bodyStart = this.scanner.currentPosition;
    }

    if (this.currentElement != null) {
      this.restartRecovery = true;
    }

    this.scanner.commentPtr = -1;
  }

  protected void consumeEnumHeaderName() {
    TypeDeclaration enumDeclaration = new TypeDeclaration(this.compilationUnit.compilationResult);
    if (this.nestedMethod[this.nestedType] == 0) {
      if (this.nestedType != 0) {
        enumDeclaration.bits |= 1024;
      }
    }
    else
    {
      blockReal();
    }

    long pos = this.identifierPositionStack[this.identifierPtr];
    enumDeclaration.sourceEnd = ((int)pos);
    enumDeclaration.sourceStart = ((int)(pos >>> 32));
    enumDeclaration.name = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    enumDeclaration.declarationSourceStart = this.intStack[(this.intPtr--)];
    this.intPtr -= 1;

    enumDeclaration.modifiersSourceStart = this.intStack[(this.intPtr--)];
    enumDeclaration.modifiers = (this.intStack[(this.intPtr--)] | 0x4000);
    if (enumDeclaration.modifiersSourceStart >= 0)
      enumDeclaration.declarationSourceStart = enumDeclaration.modifiersSourceStart;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        enumDeclaration.annotations = new Annotation[length], 
        0, 
        length);
    }

    enumDeclaration.bodyStart = (enumDeclaration.sourceEnd + 1);
    pushOnAstStack(enumDeclaration);

    this.listLength = 0;

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition))
    {
      problemReporter().invalidUsageOfEnumDeclarations(enumDeclaration);
    }

    if (this.currentElement != null) {
      this.lastCheckPoint = enumDeclaration.bodyStart;
      this.currentElement = this.currentElement.add(enumDeclaration, 0);
      this.lastIgnoredToken = -1;
    }

    enumDeclaration.javadoc = this.javadoc;
    this.javadoc = null;
  }

  protected void consumeEqualityExpression(int op)
  {
    this.expressionPtr -= 1;
    this.expressionLengthPtr -= 1;
    this.expressionStack[this.expressionPtr] = 
      new EqualExpression(
      this.expressionStack[this.expressionPtr], 
      this.expressionStack[(this.expressionPtr + 1)], 
      op);
  }

  protected void consumeEqualityExpressionWithName(int op)
  {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
    this.expressionPtr -= 1;
    this.expressionLengthPtr -= 1;
    this.expressionStack[this.expressionPtr] = 
      new EqualExpression(
      this.expressionStack[(this.expressionPtr + 1)], 
      this.expressionStack[this.expressionPtr], 
      op);
  }

  protected void consumeExitMemberValue()
  {
  }

  protected void consumeExitTryBlock() {
    if (this.currentElement != null)
      this.restartRecovery = true;
  }

  protected void consumeExitVariableWithInitialization()
  {
    this.expressionLengthPtr -= 1;
    AbstractVariableDeclaration variableDecl = (AbstractVariableDeclaration)this.astStack[this.astPtr];
    variableDecl.initialization = this.expressionStack[(this.expressionPtr--)];

    variableDecl.declarationSourceEnd = variableDecl.initialization.sourceEnd;
    variableDecl.declarationEnd = variableDecl.initialization.sourceEnd;

    recoveryExitFromVariable();
  }

  protected void consumeExitVariableWithoutInitialization()
  {
    AbstractVariableDeclaration variableDecl = (AbstractVariableDeclaration)this.astStack[this.astPtr];
    variableDecl.declarationSourceEnd = variableDecl.declarationEnd;
    if ((this.currentElement != null) && ((this.currentElement instanceof RecoveredField)) && 
      (this.endStatementPosition > variableDecl.sourceEnd)) {
      this.currentElement.updateSourceEndIfNecessary(this.endStatementPosition);
    }

    recoveryExitFromVariable();
  }

  protected void consumeExplicitConstructorInvocation(int flag, int recFlag)
  {
    int startPosition = this.intStack[(this.intPtr--)];
    ExplicitConstructorCall ecc = new ExplicitConstructorCall(recFlag);
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      this.expressionPtr -= length;
      System.arraycopy(this.expressionStack, this.expressionPtr + 1, ecc.arguments = new Expression[length], 0, length);
    }
    switch (flag) {
    case 0:
      ecc.sourceStart = startPosition;
      break;
    case 1:
      this.expressionLengthPtr -= 1;
      ecc.sourceStart = (ecc.qualification = this.expressionStack[(this.expressionPtr--)]).sourceStart;
      break;
    case 2:
      ecc.sourceStart = (ecc.qualification = getUnspecifiedReferenceOptimized()).sourceStart;
    }

    pushOnAstStack(ecc);
    ecc.sourceEnd = this.endStatementPosition;
  }

  protected void consumeExplicitConstructorInvocationWithTypeArguments(int flag, int recFlag)
  {
    int startPosition = this.intStack[(this.intPtr--)];
    ExplicitConstructorCall ecc = new ExplicitConstructorCall(recFlag);

    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      this.expressionPtr -= length;
      System.arraycopy(this.expressionStack, this.expressionPtr + 1, ecc.arguments = new Expression[length], 0, length);
    }
    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, ecc.typeArguments = new TypeReference[length], 0, length);
    ecc.typeArgumentsSourceStart = this.intStack[(this.intPtr--)];

    switch (flag) {
    case 0:
      ecc.sourceStart = startPosition;
      break;
    case 1:
      this.expressionLengthPtr -= 1;
      ecc.sourceStart = (ecc.qualification = this.expressionStack[(this.expressionPtr--)]).sourceStart;
      break;
    case 2:
      ecc.sourceStart = (ecc.qualification = getUnspecifiedReferenceOptimized()).sourceStart;
    }

    pushOnAstStack(ecc);
    ecc.sourceEnd = this.endStatementPosition;
  }

  protected void consumeExpressionStatement() {
    this.expressionLengthPtr -= 1;
    Expression expression = this.expressionStack[(this.expressionPtr--)];
    expression.statementEnd = this.endStatementPosition;
    pushOnAstStack(expression);
  }

  protected void consumeFieldAccess(boolean isSuperAccess)
  {
    FieldReference fr = 
      new FieldReference(
      this.identifierStack[this.identifierPtr], 
      this.identifierPositionStack[(this.identifierPtr--)]);
    this.identifierLengthPtr -= 1;
    if (isSuperAccess)
    {
      fr.sourceStart = this.intStack[(this.intPtr--)];
      fr.receiver = new SuperReference(fr.sourceStart, this.endPosition);
      pushOnExpressionStack(fr);
    }
    else {
      fr.receiver = this.expressionStack[this.expressionPtr];

      fr.sourceStart = fr.receiver.sourceStart;
      this.expressionStack[this.expressionPtr] = fr;
    }
  }

  protected void consumeFieldDeclaration()
  {
    int variableDeclaratorsCounter = this.astLengthStack[this.astLengthPtr];

    for (int i = variableDeclaratorsCounter - 1; i >= 0; i--) {
      FieldDeclaration fieldDeclaration = (FieldDeclaration)this.astStack[(this.astPtr - i)];
      fieldDeclaration.declarationSourceEnd = this.endStatementPosition;
      fieldDeclaration.declarationEnd = this.endStatementPosition;
    }

    updateSourceDeclarationParts(variableDeclaratorsCounter);
    int endPos = flushCommentsDefinedPriorTo(this.endStatementPosition);
    if (endPos != this.endStatementPosition) {
      for (int i = 0; i < variableDeclaratorsCounter; i++) {
        FieldDeclaration fieldDeclaration = (FieldDeclaration)this.astStack[(this.astPtr - i)];
        fieldDeclaration.declarationSourceEnd = endPos;
      }
    }

    int startIndex = this.astPtr - this.variablesCounter[this.nestedType] + 1;
    System.arraycopy(
      this.astStack, 
      startIndex, 
      this.astStack, 
      startIndex - 1, 
      variableDeclaratorsCounter);
    this.astPtr -= 1;
    this.astLengthStack[(--this.astLengthPtr)] = variableDeclaratorsCounter;

    if (this.currentElement != null) {
      this.lastCheckPoint = (endPos + 1);
      if ((this.currentElement.parent != null) && ((this.currentElement instanceof RecoveredField)) && 
        (!(this.currentElement instanceof RecoveredInitializer))) {
        this.currentElement = this.currentElement.parent;
      }

      this.restartRecovery = true;
    }
    this.variablesCounter[this.nestedType] = 0;
  }

  protected void consumeForceNoDiet() {
    this.dietInt += 1;
  }

  protected void consumeForInit() {
    pushOnAstLengthStack(-1);
  }

  protected void consumeFormalParameter(boolean isVarArgs)
  {
    this.identifierLengthPtr -= 1;
    char[] identifierName = this.identifierStack[this.identifierPtr];
    long namePositions = this.identifierPositionStack[(this.identifierPtr--)];
    int extendedDimensions = this.intStack[(this.intPtr--)];
    int endOfEllipsis = 0;
    if (isVarArgs) {
      endOfEllipsis = this.intStack[(this.intPtr--)];
    }
    int firstDimensions = this.intStack[(this.intPtr--)];
    int typeDimensions = firstDimensions + extendedDimensions;
    TypeReference type = getTypeReference(typeDimensions);
    if (isVarArgs) {
      type = copyDims(type, typeDimensions + 1);
      if (extendedDimensions == 0) {
        type.sourceEnd = endOfEllipsis;
      }
      type.bits |= 16384;
    }
    int modifierPositions = this.intStack[(this.intPtr--)];
    this.intPtr -= 1;
    Argument arg = 
      new Argument(
      identifierName, 
      namePositions, 
      type, 
      this.intStack[(this.intPtr + 1)] & 0xFFEFFFFF);
    arg.declarationSourceStart = modifierPositions;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        arg.annotations = new Annotation[length], 
        0, 
        length);
    }
    pushOnAstStack(arg);

    this.listLength += 1;

    if (isVarArgs)
      if ((!this.statementRecoveryActivated) && 
        (this.options.sourceLevel < 3211264L) && 
        (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition))
        problemReporter().invalidUsageOfVarargs(arg);
      else if ((!this.statementRecoveryActivated) && 
        (extendedDimensions > 0))
        problemReporter().illegalExtendedDimensions(arg);
  }

  protected void consumeFormalParameterList()
  {
    optimizedConcatNodeLists();
  }

  protected void consumeFormalParameterListopt() {
    pushOnAstLengthStack(0);
  }

  protected void consumeGenericType()
  {
  }

  protected void consumeGenericTypeArrayType()
  {
  }

  protected void consumeGenericTypeNameArrayType()
  {
  }

  protected void consumeImportDeclaration() {
    ImportReference impt = (ImportReference)this.astStack[this.astPtr];

    impt.declarationEnd = this.endStatementPosition;
    impt.declarationSourceEnd = 
      flushCommentsDefinedPriorTo(impt.declarationSourceEnd);

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.currentElement = this.currentElement.add(impt, 0);
      this.lastIgnoredToken = -1;
      this.restartRecovery = true;
    }
  }

  protected void consumeImportDeclarations()
  {
    optimizedConcatNodeLists();
  }

  protected void consumeInsideCastExpression() {
  }

  protected void consumeInsideCastExpressionLL1() {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
  }

  protected void consumeInsideCastExpressionWithQualifiedGenerics()
  {
  }

  protected void consumeInstanceOfExpression()
  {
    void tmp44_41 = 
      new InstanceOfExpression(
      this.expressionStack[this.expressionPtr], 
      getTypeReference(this.intStack[(this.intPtr--)]));

    Expression exp = tmp44_41; this.expressionStack[this.expressionPtr] = tmp44_41;

    if (exp.sourceEnd == 0)
    {
      exp.sourceEnd = (this.scanner.startPosition - 1);
    }
  }

  protected void consumeInstanceOfExpressionWithName()
  {
    TypeReference reference = getTypeReference(this.intStack[(this.intPtr--)]);
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
    void tmp54_51 = 
      new InstanceOfExpression(
      this.expressionStack[this.expressionPtr], 
      reference);

    Expression exp = tmp54_51; this.expressionStack[this.expressionPtr] = tmp54_51;

    if (exp.sourceEnd == 0)
    {
      exp.sourceEnd = (this.scanner.startPosition - 1);
    }
  }

  protected void consumeInterfaceDeclaration()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)
    {
      dispatchDeclarationInto(length);
    }

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];

    markInitializersWithLocalType(typeDecl);

    typeDecl.checkConstructors(this);

    if (this.scanner.containsAssertKeyword) {
      typeDecl.bits |= 1;
    }
    typeDecl.addClinit();
    typeDecl.bodyEnd = this.endStatementPosition;
    if ((length == 0) && (!containsComment(typeDecl.bodyStart, typeDecl.bodyEnd))) {
      typeDecl.bits |= 8;
    }
    typeDecl.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeInterfaceHeader()
  {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (this.currentToken == 69) {
      typeDecl.bodyStart = this.scanner.currentPosition;
    }
    if (this.currentElement != null) {
      this.restartRecovery = true;
    }

    this.scanner.commentPtr = -1;
  }

  protected void consumeInterfaceHeaderExtends() {
    int length = this.astLengthStack[(this.astLengthPtr--)];

    this.astPtr -= length;
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    System.arraycopy(
      this.astStack, 
      this.astPtr + 1, 
      typeDecl.superInterfaces = new TypeReference[length], 
      0, 
      length);
    int i = 0; for (int max = typeDecl.superInterfaces.length; i < max; i++) {
      typeDecl.superInterfaces[i].bits |= 16;
    }
    typeDecl.bodyStart = (typeDecl.superInterfaces[(length - 1)].sourceEnd + 1);
    this.listLength = 0;

    if (this.currentElement != null)
      this.lastCheckPoint = typeDecl.bodyStart;
  }

  protected void consumeInterfaceHeaderName1()
  {
    TypeDeclaration typeDecl = new TypeDeclaration(this.compilationUnit.compilationResult);

    if (this.nestedMethod[this.nestedType] == 0) {
      if (this.nestedType != 0)
        typeDecl.bits |= 1024;
    }
    else
    {
      typeDecl.bits |= 256;
      markEnclosingMemberWithLocalType();
      blockReal();
    }

    long pos = this.identifierPositionStack[this.identifierPtr];
    typeDecl.sourceEnd = ((int)pos);
    typeDecl.sourceStart = ((int)(pos >>> 32));
    typeDecl.name = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    typeDecl.declarationSourceStart = this.intStack[(this.intPtr--)];
    this.intPtr -= 1;
    typeDecl.modifiersSourceStart = this.intStack[(this.intPtr--)];
    typeDecl.modifiers = (this.intStack[(this.intPtr--)] | 0x200);
    if (typeDecl.modifiersSourceStart >= 0) {
      typeDecl.declarationSourceStart = typeDecl.modifiersSourceStart;
    }

    if (((typeDecl.bits & 0x400) == 0) && ((typeDecl.bits & 0x100) == 0) && 
      (this.compilationUnit != null) && (!CharOperation.equals(typeDecl.name, this.compilationUnit.getMainTypeName())))
      typeDecl.bits |= 4096;
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        typeDecl.annotations = new Annotation[length], 
        0, 
        length);
    }
    typeDecl.bodyStart = (typeDecl.sourceEnd + 1);
    pushOnAstStack(typeDecl);
    this.listLength = 0;

    if (this.currentElement != null) {
      this.lastCheckPoint = typeDecl.bodyStart;
      this.currentElement = this.currentElement.add(typeDecl, 0);
      this.lastIgnoredToken = -1;
    }

    typeDecl.javadoc = this.javadoc;
    this.javadoc = null;
  }

  protected void consumeInterfaceMemberDeclarations() {
    concatNodeLists();
  }

  protected void consumeInterfaceMemberDeclarationsopt() {
    this.nestedType -= 1;
  }

  protected void consumeInterfaceType() {
    pushOnAstStack(getTypeReference(0));

    this.listLength += 1;
  }

  protected void consumeInterfaceTypeList() {
    optimizedConcatNodeLists();
  }

  protected void consumeInternalCompilationUnit()
  {
    if (this.compilationUnit.isPackageInfo()) {
      this.compilationUnit.types = new TypeDeclaration[1];

      TypeDeclaration declaration = new TypeDeclaration(this.compilationUnit.compilationResult);
      declaration.name = TypeConstants.PACKAGE_INFO_NAME;
      declaration.modifiers = 512;
      this.compilationUnit.types[0] = declaration;
      declaration.javadoc = this.compilationUnit.javadoc;
    }
  }

  protected void consumeInternalCompilationUnitWithTypes()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)
      if (this.compilationUnit.isPackageInfo()) {
        this.compilationUnit.types = new TypeDeclaration[length + 1];
        this.astPtr -= length;
        System.arraycopy(this.astStack, this.astPtr + 1, this.compilationUnit.types, 1, length);

        TypeDeclaration declaration = new TypeDeclaration(this.compilationUnit.compilationResult);
        declaration.name = TypeConstants.PACKAGE_INFO_NAME;
        declaration.modifiers = 512;
        this.compilationUnit.types[0] = declaration;
        declaration.javadoc = this.compilationUnit.javadoc;
      } else {
        this.compilationUnit.types = new TypeDeclaration[length];
        this.astPtr -= length;
        System.arraycopy(this.astStack, this.astPtr + 1, this.compilationUnit.types, 0, length);
      }
  }

  protected void consumeInvalidAnnotationTypeDeclaration()
  {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (!this.statementRecoveryActivated) problemReporter().illegalLocalTypeDeclaration(typeDecl);

    this.astPtr -= 1;
    pushOnAstLengthStack(-1);
    concatNodeLists();
  }

  protected void consumeInvalidConstructorDeclaration()
  {
    ConstructorDeclaration cd = (ConstructorDeclaration)this.astStack[this.astPtr];

    cd.bodyEnd = this.endPosition;
    cd.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);

    cd.modifiers |= 16777216;
  }

  protected void consumeInvalidConstructorDeclaration(boolean hasBody)
  {
    if (hasBody)
    {
      this.intPtr -= 1;
    }

    if (hasBody)
      this.realBlockPtr -= 1;
    int length;
    if ((hasBody) && ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)) {
      this.astPtr -= length;
    }
    ConstructorDeclaration constructorDeclaration = (ConstructorDeclaration)this.astStack[this.astPtr];
    constructorDeclaration.bodyEnd = this.endStatementPosition;
    constructorDeclaration.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
    if (!hasBody)
      constructorDeclaration.modifiers |= 16777216;
  }

  protected void consumeInvalidInterfaceDeclaration()
  {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (!this.statementRecoveryActivated) problemReporter().illegalLocalTypeDeclaration(typeDecl);

    this.astPtr -= 1;
    pushOnAstLengthStack(-1);
    concatNodeLists();
  }

  protected void consumeInvalidEnumDeclaration() {
    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (!this.statementRecoveryActivated) problemReporter().illegalLocalTypeDeclaration(typeDecl);

    this.astPtr -= 1;
    pushOnAstLengthStack(-1);
    concatNodeLists();
  }

  protected void consumeInvalidMethodDeclaration()
  {
    this.intPtr -= 1;

    this.realBlockPtr -= 1;
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      this.astPtr -= length;
    }

    MethodDeclaration md = (MethodDeclaration)this.astStack[this.astPtr];
    md.bodyEnd = this.endPosition;
    md.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);

    if (!this.statementRecoveryActivated) problemReporter().abstractMethodNeedingNoBody(md); 
  }

  protected void consumeLabel()
  {
  }

  protected void consumeLeftParen() { pushOnIntStack(this.lParenPos); }


  protected void consumeLocalVariableDeclaration()
  {
    int variableDeclaratorsCounter = this.astLengthStack[this.astLengthPtr];

    int startIndex = this.astPtr - this.variablesCounter[this.nestedType] + 1;
    System.arraycopy(
      this.astStack, 
      startIndex, 
      this.astStack, 
      startIndex - 1, 
      variableDeclaratorsCounter);
    this.astPtr -= 1;
    this.astLengthStack[(--this.astLengthPtr)] = variableDeclaratorsCounter;
    this.variablesCounter[this.nestedType] = 0;
  }

  protected void consumeLocalVariableDeclarationStatement()
  {
    this.realBlockStack[this.realBlockPtr] += 1;

    int variableDeclaratorsCounter = this.astLengthStack[this.astLengthPtr];
    for (int i = variableDeclaratorsCounter - 1; i >= 0; i--) {
      LocalDeclaration localDeclaration = (LocalDeclaration)this.astStack[(this.astPtr - i)];
      localDeclaration.declarationSourceEnd = this.endStatementPosition;
      localDeclaration.declarationEnd = this.endStatementPosition;
    }
  }

  protected void consumeMarkerAnnotation()
  {
    MarkerAnnotation markerAnnotation = null;
    TypeReference typeReference = getAnnotationType();
    markerAnnotation = new MarkerAnnotation(typeReference, this.intStack[(this.intPtr--)]);
    markerAnnotation.declarationSourceEnd = markerAnnotation.sourceEnd;
    pushOnExpressionStack(markerAnnotation);
    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      problemReporter().invalidUsageOfAnnotation(markerAnnotation);
    }
    this.recordStringLiterals = true;
  }

  protected void consumeMemberValueArrayInitializer()
  {
    arrayInitializer(this.expressionLengthStack[(this.expressionLengthPtr--)]);
  }
  protected void consumeMemberValueAsName() {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
  }

  protected void consumeMemberValuePair() {
    char[] simpleName = this.identifierStack[this.identifierPtr];
    long position = this.identifierPositionStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;
    int end = (int)position;
    int start = (int)(position >>> 32);
    Expression value = this.expressionStack[(this.expressionPtr--)];
    this.expressionLengthPtr -= 1;
    MemberValuePair memberValuePair = new MemberValuePair(simpleName, start, end, value);
    pushOnAstStack(memberValuePair);
  }

  protected void consumeMemberValuePairs() {
    concatNodeLists();
  }

  protected void consumeMemberValues() {
    concatExpressionLists();
  }

  protected void consumeMethodBody() {
    this.nestedMethod[this.nestedType] -= 1;
  }

  protected void consumeMethodDeclaration(boolean isNotAbstract)
  {
    if (isNotAbstract)
    {
      this.intPtr -= 1;
      this.intPtr -= 1;
    }

    int explicitDeclarations = 0;
    Statement[] statements = (Statement[])null;
    if (isNotAbstract)
    {
      explicitDeclarations = this.realBlockStack[(this.realBlockPtr--)];
      int length;
      if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
        System.arraycopy(
          this.astStack, 
          this.astPtr -= length + 1, 
          statements = new Statement[length], 
          0, 
          length);
      }

    }

    MethodDeclaration md = (MethodDeclaration)this.astStack[this.astPtr];
    md.statements = statements;
    md.explicitDeclarations = explicitDeclarations;

    if (!isNotAbstract)
      md.modifiers |= 16777216;
    else if (((!this.diet) || (this.dietInt != 0)) && (statements == null) && (!containsComment(md.bodyStart, this.endPosition))) {
      md.bits |= 8;
    }

    md.bodyEnd = this.endPosition;
    md.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
  }

  protected void consumeMethodHeader()
  {
    AbstractMethodDeclaration method = (AbstractMethodDeclaration)this.astStack[this.astPtr];

    if (this.currentToken == 69) {
      method.bodyStart = this.scanner.currentPosition;
    }

    if (this.currentElement != null)
    {
      if (this.currentToken == 27) {
        method.modifiers |= 16777216;
        method.declarationSourceEnd = (this.scanner.currentPosition - 1);
        method.bodyEnd = (this.scanner.currentPosition - 1);
        if ((this.currentElement.parseTree() == method) && (this.currentElement.parent != null))
          this.currentElement = this.currentElement.parent;
      }
      else if ((this.currentToken == 69) && 
        ((this.currentElement instanceof RecoveredMethod)) && 
        (((RecoveredMethod)this.currentElement).methodDeclaration != method)) {
        this.ignoreNextOpeningBrace = true;
        this.currentElement.bracketBalance += 1;
      }

      this.restartRecovery = true;
    }
  }

  protected void consumeMethodHeaderDefaultValue() {
    MethodDeclaration md = (MethodDeclaration)this.astStack[this.astPtr];

    int length = this.expressionLengthStack[(this.expressionLengthPtr--)];
    if (length == 1) {
      this.intPtr -= 1;
      this.intPtr -= 1;
      if (md.isAnnotationMethod()) {
        ((AnnotationMethodDeclaration)md).defaultValue = this.expressionStack[this.expressionPtr];
        md.modifiers |= 131072;
      }
      this.expressionPtr -= 1;
      this.recordStringLiterals = true;
    }

    if ((this.currentElement != null) && 
      (md.isAnnotationMethod()))
      this.currentElement.updateSourceEndIfNecessary(((AnnotationMethodDeclaration)md).defaultValue.sourceEnd);
  }

  protected void consumeMethodHeaderExtendedDims()
  {
    MethodDeclaration md = (MethodDeclaration)this.astStack[this.astPtr];
    int extendedDims = this.intStack[(this.intPtr--)];
    if (md.isAnnotationMethod()) {
      ((AnnotationMethodDeclaration)md).extendedDimensions = extendedDims;
    }
    if (extendedDims != 0) {
      TypeReference returnType = md.returnType;
      md.sourceEnd = this.endPosition;
      int dims = returnType.dimensions() + extendedDims;
      md.returnType = copyDims(returnType, dims);
      if (this.currentToken == 69) {
        md.bodyStart = (this.endPosition + 1);
      }

      if (this.currentElement != null)
        this.lastCheckPoint = md.bodyStart;
    }
  }

  protected void consumeMethodHeaderName(boolean isAnnotationMethod)
  {
    MethodDeclaration md = null;
    if (isAnnotationMethod) {
      md = new AnnotationMethodDeclaration(this.compilationUnit.compilationResult);
      this.recordStringLiterals = false;
    } else {
      md = new MethodDeclaration(this.compilationUnit.compilationResult);
    }

    md.selector = this.identifierStack[this.identifierPtr];
    long selectorSource = this.identifierPositionStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    md.returnType = getTypeReference(this.intStack[(this.intPtr--)]);

    md.declarationSourceStart = this.intStack[(this.intPtr--)];
    md.modifiers = this.intStack[(this.intPtr--)];
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        md.annotations = new Annotation[length], 
        0, 
        length);
    }

    md.javadoc = this.javadoc;
    this.javadoc = null;

    md.sourceStart = ((int)(selectorSource >>> 32));
    pushOnAstStack(md);
    md.sourceEnd = this.lParenPos;
    md.bodyStart = (this.lParenPos + 1);
    this.listLength = 0;

    if (this.currentElement != null)
      if (((this.currentElement instanceof RecoveredType)) || 
        (Util.getLineNumber(md.returnType.sourceStart, this.scanner.lineEnds, 0, this.scanner.linePtr) == 
        Util.getLineNumber(md.sourceStart, this.scanner.lineEnds, 0, this.scanner.linePtr))) {
        this.lastCheckPoint = md.bodyStart;
        this.currentElement = this.currentElement.add(md, 0);
        this.lastIgnoredToken = -1;
      } else {
        this.lastCheckPoint = md.sourceStart;
        this.restartRecovery = true;
      }
  }

  protected void consumeMethodHeaderNameWithTypeParameters(boolean isAnnotationMethod)
  {
    MethodDeclaration md = null;
    if (isAnnotationMethod) {
      md = new AnnotationMethodDeclaration(this.compilationUnit.compilationResult);
      this.recordStringLiterals = false;
    } else {
      md = new MethodDeclaration(this.compilationUnit.compilationResult);
    }

    md.selector = this.identifierStack[this.identifierPtr];
    long selectorSource = this.identifierPositionStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    md.returnType = getTypeReference(this.intStack[(this.intPtr--)]);

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, md.typeParameters = new TypeParameter[length], 0, length);

    md.declarationSourceStart = this.intStack[(this.intPtr--)];
    md.modifiers = this.intStack[(this.intPtr--)];

    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        md.annotations = new Annotation[length], 
        0, 
        length);
    }

    md.javadoc = this.javadoc;
    this.javadoc = null;

    md.sourceStart = ((int)(selectorSource >>> 32));
    pushOnAstStack(md);
    md.sourceEnd = this.lParenPos;
    md.bodyStart = (this.lParenPos + 1);
    this.listLength = 0;

    if (this.currentElement != null)
    {
      boolean isType;
      if (((isType = this.currentElement instanceof RecoveredType)) || 
        (Util.getLineNumber(md.returnType.sourceStart, this.scanner.lineEnds, 0, this.scanner.linePtr) == 
        Util.getLineNumber(md.sourceStart, this.scanner.lineEnds, 0, this.scanner.linePtr))) {
        if (isType) {
          ((RecoveredType)this.currentElement).pendingTypeParameters = null;
        }
        this.lastCheckPoint = md.bodyStart;
        this.currentElement = this.currentElement.add(md, 0);
        this.lastIgnoredToken = -1;
      } else {
        this.lastCheckPoint = md.sourceStart;
        this.restartRecovery = true;
      }
    }
  }

  protected void consumeMethodHeaderRightParen() {
    int length = this.astLengthStack[(this.astLengthPtr--)];
    this.astPtr -= length;
    AbstractMethodDeclaration md = (AbstractMethodDeclaration)this.astStack[this.astPtr];
    md.sourceEnd = this.rParenPos;

    if (length != 0) {
      System.arraycopy(
        this.astStack, 
        this.astPtr + 1, 
        md.arguments = new Argument[length], 
        0, 
        length);
    }
    md.bodyStart = (this.rParenPos + 1);
    this.listLength = 0;

    if (this.currentElement != null) {
      this.lastCheckPoint = md.bodyStart;
      if (this.currentElement.parseTree() == md) return;

      if ((md.isConstructor()) && (
        (length != 0) || 
        (this.currentToken == 69) || 
        (this.currentToken == 105))) {
        this.currentElement = this.currentElement.add(md, 0);
        this.lastIgnoredToken = -1;
      }
    }
  }

  protected void consumeMethodHeaderThrowsClause()
  {
    int length = this.astLengthStack[(this.astLengthPtr--)];
    this.astPtr -= length;
    AbstractMethodDeclaration md = (AbstractMethodDeclaration)this.astStack[this.astPtr];
    System.arraycopy(
      this.astStack, 
      this.astPtr + 1, 
      md.thrownExceptions = new TypeReference[length], 
      0, 
      length);
    md.sourceEnd = md.thrownExceptions[(length - 1)].sourceEnd;
    md.bodyStart = (md.thrownExceptions[(length - 1)].sourceEnd + 1);
    this.listLength = 0;

    if (this.currentElement != null)
      this.lastCheckPoint = md.bodyStart;
  }

  protected void consumeMethodInvocationName()
  {
    MessageSend m = newMessageSend();
    m.sourceEnd = this.rParenPos;
    m.sourceStart = 
      ((int)((m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr]) >>> 32));
    m.selector = this.identifierStack[(this.identifierPtr--)];
    if (this.identifierLengthStack[this.identifierLengthPtr] == 1) {
      m.receiver = ThisReference.implicitThis();
      this.identifierLengthPtr -= 1;
    } else {
      this.identifierLengthStack[this.identifierLengthPtr] -= 1;
      m.receiver = getUnspecifiedReference();
      m.sourceStart = m.receiver.sourceStart;
    }
    pushOnExpressionStack(m);
  }

  protected void consumeMethodInvocationNameWithTypeArguments()
  {
    MessageSend m = newMessageSendWithTypeArguments();
    m.sourceEnd = this.rParenPos;
    m.sourceStart = 
      ((int)((m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr]) >>> 32));
    m.selector = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, m.typeArguments = new TypeReference[length], 0, length);
    this.intPtr -= 1;

    m.receiver = getUnspecifiedReference();
    m.sourceStart = m.receiver.sourceStart;
    pushOnExpressionStack(m);
  }

  protected void consumeMethodInvocationPrimary()
  {
    MessageSend m = newMessageSend();
    m.sourceStart = 
      ((int)((m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr]) >>> 32));
    m.selector = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;
    m.receiver = this.expressionStack[this.expressionPtr];
    m.sourceStart = m.receiver.sourceStart;
    m.sourceEnd = this.rParenPos;
    this.expressionStack[this.expressionPtr] = m;
  }

  protected void consumeMethodInvocationPrimaryWithTypeArguments()
  {
    MessageSend m = newMessageSendWithTypeArguments();
    m.sourceStart = 
      ((int)((m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr]) >>> 32));
    m.selector = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, m.typeArguments = new TypeReference[length], 0, length);
    this.intPtr -= 1;

    m.receiver = this.expressionStack[this.expressionPtr];
    m.sourceStart = m.receiver.sourceStart;
    m.sourceEnd = this.rParenPos;
    this.expressionStack[this.expressionPtr] = m;
  }

  protected void consumeMethodInvocationSuper()
  {
    MessageSend m = newMessageSend();
    m.sourceStart = this.intStack[(this.intPtr--)];
    m.sourceEnd = this.rParenPos;
    m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr];
    m.selector = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;
    m.receiver = new SuperReference(m.sourceStart, this.endPosition);
    pushOnExpressionStack(m);
  }

  protected void consumeMethodInvocationSuperWithTypeArguments()
  {
    MessageSend m = newMessageSendWithTypeArguments();
    this.intPtr -= 1;
    m.sourceEnd = this.rParenPos;
    m.nameSourcePosition = this.identifierPositionStack[this.identifierPtr];
    m.selector = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;

    int length = this.genericsLengthStack[(this.genericsLengthPtr--)];
    this.genericsPtr -= length;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, m.typeArguments = new TypeReference[length], 0, length);
    m.sourceStart = this.intStack[(this.intPtr--)];

    m.receiver = new SuperReference(m.sourceStart, this.endPosition);
    pushOnExpressionStack(m);
  }
  protected void consumeModifiers() {
    int savedModifiersSourceStart = this.modifiersSourceStart;
    checkComment();
    pushOnIntStack(this.modifiers);
    if (this.modifiersSourceStart >= savedModifiersSourceStart) {
      this.modifiersSourceStart = savedModifiersSourceStart;
    }
    pushOnIntStack(this.modifiersSourceStart);
    resetModifiers();
  }
  protected void consumeModifiers2() {
    this.expressionLengthStack[(this.expressionLengthPtr - 1)] += this.expressionLengthStack[(this.expressionLengthPtr--)];
  }
  protected void consumeNameArrayType() {
    pushOnGenericsLengthStack(0);
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
  }

  protected void consumeNestedMethod() {
    jumpOverMethodBody();
    this.nestedMethod[this.nestedType] += 1;
    pushOnIntStack(this.scanner.currentPosition);
    consumeOpenBlock();
  }

  protected void consumeNestedType() {
    int length = this.nestedMethod.length;
    if (++this.nestedType >= length) {
      System.arraycopy(
        this.nestedMethod, 0, 
        this.nestedMethod = new int[length + 30], 0, 
        length);

      System.arraycopy(
        this.variablesCounter, 0, 
        this.variablesCounter = new int[length + 30], 0, 
        length);
    }
    this.nestedMethod[this.nestedType] = 0;
    this.variablesCounter[this.nestedType] = 0;
  }

  protected void consumeNormalAnnotation() {
    NormalAnnotation normalAnnotation = null;
    TypeReference typeReference = getAnnotationType();
    normalAnnotation = new NormalAnnotation(typeReference, this.intStack[(this.intPtr--)]);
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      System.arraycopy(
        this.astStack, 
        this.astPtr -= length + 1, 
        normalAnnotation.memberValuePairs = new MemberValuePair[length], 
        0, 
        length);
    }
    normalAnnotation.declarationSourceEnd = this.rParenPos;
    pushOnExpressionStack(normalAnnotation);

    if (this.currentElement != null) {
      annotationRecoveryCheckPoint(normalAnnotation.sourceStart, normalAnnotation.declarationSourceEnd);
    }

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      problemReporter().invalidUsageOfAnnotation(normalAnnotation);
    }
    this.recordStringLiterals = true;
  }

  protected void consumeOneDimLoop() {
    this.dimensions += 1;
  }

  protected void consumeOnlySynchronized() {
    pushOnIntStack(this.synchronizedBlockSourceStart);
    resetModifiers();
    this.expressionLengthPtr -= 1;
  }
  protected void consumeOnlyTypeArguments() {
    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      int length = this.genericsLengthStack[this.genericsLengthPtr];
      problemReporter().invalidUsageOfTypeArguments(
        (TypeReference)this.genericsStack[(this.genericsPtr - length + 1)], 
        (TypeReference)this.genericsStack[this.genericsPtr]);
    }
  }

  protected void consumeOnlyTypeArgumentsForCastExpression()
  {
  }

  protected void consumeOpenBlock() {
    pushOnIntStack(this.scanner.startPosition);
    int stackLength = this.realBlockStack.length;
    if (++this.realBlockPtr >= stackLength) {
      System.arraycopy(
        this.realBlockStack, 0, 
        this.realBlockStack = new int[stackLength + 255], 0, 
        stackLength);
    }
    this.realBlockStack[this.realBlockPtr] = 0;
  }

  protected void consumePackageComment() {
    if (this.options.sourceLevel >= 3211264L) {
      checkComment();
      resetModifiers();
    }
  }

  protected void consumePackageDeclaration()
  {
    ImportReference impt = this.compilationUnit.currentPackage;
    this.compilationUnit.javadoc = this.javadoc;
    this.javadoc = null;

    impt.declarationEnd = this.endStatementPosition;
    impt.declarationSourceEnd = flushCommentsDefinedPriorTo(impt.declarationSourceEnd);
  }

  protected void consumePackageDeclarationName()
  {
    int length;
    char[][] tokens = 
      new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, ++this.identifierPtr, tokens, 0, length);
    System.arraycopy(
      this.identifierPositionStack, 
      this.identifierPtr--, 
      positions, 
      0, 
      length);

    ImportReference impt = new ImportReference(tokens, positions, true, 0);
    this.compilationUnit.currentPackage = impt;

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    impt.declarationSourceStart = this.intStack[(this.intPtr--)];

    if (this.javadoc != null) {
      impt.declarationSourceStart = this.javadoc.sourceStart;
    }

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.restartRecovery = true;
    }
  }

  protected void consumePackageDeclarationNameWithModifiers()
  {
    int length;
    char[][] tokens = 
      new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, ++this.identifierPtr, tokens, 0, length);
    System.arraycopy(
      this.identifierPositionStack, 
      this.identifierPtr--, 
      positions, 
      0, 
      length);

    int packageModifiersSourceStart = this.intStack[(this.intPtr--)];
    int packageModifiers = this.intStack[(this.intPtr--)];

    ImportReference impt = new ImportReference(tokens, positions, true, packageModifiers);
    this.compilationUnit.currentPackage = impt;

    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr -= length + 1, 
        impt.annotations = new Annotation[length], 
        0, 
        length);
      impt.declarationSourceStart = packageModifiersSourceStart;
      this.intPtr -= 1;
    } else {
      impt.declarationSourceStart = this.intStack[(this.intPtr--)];

      if (this.javadoc != null) {
        impt.declarationSourceStart = this.javadoc.sourceStart;
      }
    }

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.restartRecovery = true;
    }
  }

  protected void consumePostfixExpression() {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
  }

  protected void consumePrimaryNoNewArray() {
    Expression parenthesizedExpression = this.expressionStack[this.expressionPtr];
    updateSourcePosition(parenthesizedExpression);
    int numberOfParenthesis = (parenthesizedExpression.bits & 0x1FE00000) >> 21;
    parenthesizedExpression.bits &= -534773761;
    parenthesizedExpression.bits |= numberOfParenthesis + 1 << 21;
  }

  protected void consumePrimaryNoNewArrayArrayType() {
    this.intPtr -= 1;

    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    pushOnGenericsLengthStack(0);

    pushOnExpressionStack(
      new ClassLiteralAccess(this.intStack[(this.intPtr--)], getTypeReference(this.intStack[(this.intPtr--)])));
  }

  protected void consumePrimaryNoNewArrayName() {
    this.intPtr -= 1;

    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    pushOnGenericsLengthStack(0);
    TypeReference typeReference = getTypeReference(0);

    pushOnExpressionStack(
      new ClassLiteralAccess(this.intStack[(this.intPtr--)], typeReference));
  }

  protected void consumePrimaryNoNewArrayNameSuper()
  {
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    pushOnGenericsLengthStack(0);
    TypeReference typeReference = getTypeReference(0);

    pushOnExpressionStack(
      new QualifiedSuperReference(
      typeReference, 
      this.intStack[(this.intPtr--)], 
      this.endPosition));
  }

  protected void consumePrimaryNoNewArrayNameThis()
  {
    pushOnGenericsIdentifiersLengthStack(this.identifierLengthStack[this.identifierLengthPtr]);
    pushOnGenericsLengthStack(0);

    TypeReference typeReference = getTypeReference(0);

    pushOnExpressionStack(
      new QualifiedThisReference(
      typeReference, 
      this.intStack[(this.intPtr--)], 
      this.endPosition));
  }

  protected void consumePrimaryNoNewArrayPrimitiveArrayType() {
    this.intPtr -= 1;
    pushOnExpressionStack(
      new ClassLiteralAccess(this.intStack[(this.intPtr--)], getTypeReference(this.intStack[(this.intPtr--)])));
  }

  protected void consumePrimaryNoNewArrayPrimitiveType() {
    this.intPtr -= 1;
    pushOnExpressionStack(
      new ClassLiteralAccess(this.intStack[(this.intPtr--)], getTypeReference(0)));
  }

  protected void consumePrimaryNoNewArrayThis() {
    pushOnExpressionStack(new ThisReference(this.intStack[(this.intPtr--)], this.endPosition));
  }

  protected void consumePrimaryNoNewArrayWithName() {
    pushOnExpressionStack(getUnspecifiedReferenceOptimized());
    Expression parenthesizedExpression = this.expressionStack[this.expressionPtr];
    updateSourcePosition(parenthesizedExpression);
    int numberOfParenthesis = (parenthesizedExpression.bits & 0x1FE00000) >> 21;
    parenthesizedExpression.bits &= -534773761;
    parenthesizedExpression.bits |= numberOfParenthesis + 1 << 21;
  }

  protected void consumePrimitiveArrayType()
  {
  }

  protected void consumePrimitiveType() {
    pushOnIntStack(0);
  }
  protected void consumePushLeftBrace() {
    pushOnIntStack(this.endPosition);
  }
  protected void consumePushRealModifiers() {
    checkComment();
    pushOnIntStack(this.modifiers);
    pushOnIntStack(this.modifiersSourceStart);
    resetModifiers();
  }
  protected void consumePushModifiers() {
    pushOnIntStack(this.modifiers);
    pushOnIntStack(this.modifiersSourceStart);
    resetModifiers();
    pushOnExpressionStackLengthStack(0);
  }
  protected void consumePushModifiersForHeader() {
    checkComment();
    pushOnIntStack(this.modifiers);
    pushOnIntStack(this.modifiersSourceStart);
    resetModifiers();
    pushOnExpressionStackLengthStack(0);
  }

  protected void consumePushPosition()
  {
    pushOnIntStack(this.endPosition);
  }

  protected void consumeQualifiedName()
  {
    this.identifierLengthStack[(--this.identifierLengthPtr)] += 1;
  }

  protected void consumeRecoveryMethodHeaderName() {
    boolean isAnnotationMethod = false;
    if ((this.currentElement instanceof RecoveredType)) {
      isAnnotationMethod = (((RecoveredType)this.currentElement).typeDeclaration.modifiers & 0x2000) != 0;
    } else {
      RecoveredType recoveredType = this.currentElement.enclosingType();
      if (recoveredType != null) {
        isAnnotationMethod = (recoveredType.typeDeclaration.modifiers & 0x2000) != 0;
      }
    }
    consumeMethodHeaderName(isAnnotationMethod);
  }

  protected void consumeRecoveryMethodHeaderNameWithTypeParameters() {
    boolean isAnnotationMethod = false;
    if ((this.currentElement instanceof RecoveredType)) {
      isAnnotationMethod = (((RecoveredType)this.currentElement).typeDeclaration.modifiers & 0x2000) != 0;
    } else {
      RecoveredType recoveredType = this.currentElement.enclosingType();
      if (recoveredType != null) {
        isAnnotationMethod = (recoveredType.typeDeclaration.modifiers & 0x2000) != 0;
      }
    }
    consumeMethodHeaderNameWithTypeParameters(isAnnotationMethod);
  }

  protected void consumeReduceImports()
  {
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      this.astPtr -= length;
      System.arraycopy(
        this.astStack, 
        this.astPtr + 1, 
        this.compilationUnit.imports = new ImportReference[length], 
        0, 
        length);
    }
  }

  protected void consumeReferenceType() { pushOnIntStack(0); }

  protected void consumeReferenceType1() {
    pushOnGenericsStack(getTypeReference(this.intStack[(this.intPtr--)]));
  }
  protected void consumeReferenceType2() {
    pushOnGenericsStack(getTypeReference(this.intStack[(this.intPtr--)]));
  }
  protected void consumeReferenceType3() {
    pushOnGenericsStack(getTypeReference(this.intStack[(this.intPtr--)]));
  }

  protected void consumeRestoreDiet() {
    this.dietInt -= 1;
  }

  protected void consumeRightParen() {
    pushOnIntStack(this.rParenPos);
  }

  protected void consumeRule(int act) {
    switch (act) {
    case 30:
      consumePrimitiveType();
      break;
    case 44:
      consumeReferenceType();
      break;
    case 48:
      consumeClassOrInterfaceName();
      break;
    case 49:
      consumeClassOrInterface();
      break;
    case 50:
      consumeGenericType();
      break;
    case 51:
      consumeArrayTypeWithTypeArgumentsName();
      break;
    case 52:
      consumePrimitiveArrayType();
      break;
    case 53:
      consumeNameArrayType();
      break;
    case 54:
      consumeGenericTypeNameArrayType();
      break;
    case 55:
      consumeGenericTypeArrayType();
      break;
    case 60:
      consumeQualifiedName();
      break;
    case 61:
      consumeCompilationUnit();
      break;
    case 62:
      consumeInternalCompilationUnit();
      break;
    case 63:
      consumeInternalCompilationUnit();
      break;
    case 64:
      consumeInternalCompilationUnitWithTypes();
      break;
    case 65:
      consumeInternalCompilationUnitWithTypes();
      break;
    case 66:
      consumeInternalCompilationUnit();
      break;
    case 67:
      consumeInternalCompilationUnitWithTypes();
      break;
    case 68:
      consumeInternalCompilationUnitWithTypes();
      break;
    case 69:
      consumeEmptyInternalCompilationUnit();
      break;
    case 70:
      consumeReduceImports();
      break;
    case 71:
      consumeEnterCompilationUnit();
      break;
    case 87:
      consumeCatchHeader();
      break;
    case 89:
      consumeImportDeclarations();
      break;
    case 91:
      consumeTypeDeclarations();
      break;
    case 92:
      consumePackageDeclaration();
      break;
    case 93:
      consumePackageDeclarationNameWithModifiers();
      break;
    case 94:
      consumePackageDeclarationName();
      break;
    case 95:
      consumePackageComment();
      break;
    case 100:
      consumeImportDeclaration();
      break;
    case 101:
      consumeSingleTypeImportDeclarationName();
      break;
    case 102:
      consumeImportDeclaration();
      break;
    case 103:
      consumeTypeImportOnDemandDeclarationName();
      break;
    case 106:
      consumeEmptyTypeDeclaration();
      break;
    case 110:
      consumeModifiers2();
      break;
    case 122:
      consumeAnnotationAsModifier();
      break;
    case 123:
      consumeClassDeclaration();
      break;
    case 124:
      consumeClassHeader();
      break;
    case 125:
      consumeTypeHeaderNameWithTypeParameters();
      break;
    case 127:
      consumeClassHeaderName1();
      break;
    case 128:
      consumeClassHeaderExtends();
      break;
    case 129:
      consumeClassHeaderImplements();
      break;
    case 131:
      consumeInterfaceTypeList();
      break;
    case 132:
      consumeInterfaceType();
      break;
    case 135:
      consumeClassBodyDeclarations();
      break;
    case 139:
      consumeClassBodyDeclaration();
      break;
    case 140:
      consumeDiet();
      break;
    case 141:
      consumeClassBodyDeclaration();
      break;
    case 148:
      consumeEmptyTypeDeclaration();
      break;
    case 151:
      consumeFieldDeclaration();
      break;
    case 153:
      consumeVariableDeclarators();
      break;
    case 156:
      consumeEnterVariable();
      break;
    case 157:
      consumeExitVariableWithInitialization();
      break;
    case 158:
      consumeExitVariableWithoutInitialization();
      break;
    case 159:
      consumeForceNoDiet();
      break;
    case 160:
      consumeRestoreDiet();
      break;
    case 165:
      consumeMethodDeclaration(true);
      break;
    case 166:
      consumeMethodDeclaration(false);
      break;
    case 167:
      consumeMethodHeader();
      break;
    case 168:
      consumeMethodHeaderNameWithTypeParameters(false);
      break;
    case 169:
      consumeMethodHeaderName(false);
      break;
    case 170:
      consumeMethodHeaderRightParen();
      break;
    case 171:
      consumeMethodHeaderExtendedDims();
      break;
    case 172:
      consumeMethodHeaderThrowsClause();
      break;
    case 173:
      consumeConstructorHeader();
      break;
    case 174:
      consumeConstructorHeaderNameWithTypeParameters();
      break;
    case 175:
      consumeConstructorHeaderName();
      break;
    case 177:
      consumeFormalParameterList();
      break;
    case 178:
      consumeFormalParameter(false);
      break;
    case 179:
      consumeFormalParameter(true);
      break;
    case 181:
      consumeClassTypeList();
      break;
    case 182:
      consumeClassTypeElt();
      break;
    case 183:
      consumeMethodBody();
      break;
    case 184:
      consumeNestedMethod();
      break;
    case 185:
      consumeStaticInitializer();
      break;
    case 186:
      consumeStaticOnly();
      break;
    case 187:
      consumeConstructorDeclaration();
      break;
    case 188:
      consumeInvalidConstructorDeclaration();
      break;
    case 189:
      consumeExplicitConstructorInvocation(0, 3);
      break;
    case 190:
      consumeExplicitConstructorInvocationWithTypeArguments(0, 3);
      break;
    case 191:
      consumeExplicitConstructorInvocation(0, 2);
      break;
    case 192:
      consumeExplicitConstructorInvocationWithTypeArguments(0, 2);
      break;
    case 193:
      consumeExplicitConstructorInvocation(1, 2);
      break;
    case 194:
      consumeExplicitConstructorInvocationWithTypeArguments(1, 2);
      break;
    case 195:
      consumeExplicitConstructorInvocation(2, 2);
      break;
    case 196:
      consumeExplicitConstructorInvocationWithTypeArguments(2, 2);
      break;
    case 197:
      consumeExplicitConstructorInvocation(1, 3);
      break;
    case 198:
      consumeExplicitConstructorInvocationWithTypeArguments(1, 3);
      break;
    case 199:
      consumeExplicitConstructorInvocation(2, 3);
      break;
    case 200:
      consumeExplicitConstructorInvocationWithTypeArguments(2, 3);
      break;
    case 201:
      consumeInterfaceDeclaration();
      break;
    case 202:
      consumeInterfaceHeader();
      break;
    case 203:
      consumeTypeHeaderNameWithTypeParameters();
      break;
    case 205:
      consumeInterfaceHeaderName1();
      break;
    case 206:
      consumeInterfaceHeaderExtends();
      break;
    case 209:
      consumeInterfaceMemberDeclarations();
      break;
    case 210:
      consumeEmptyTypeDeclaration();
      break;
    case 212:
      consumeInvalidMethodDeclaration();
      break;
    case 213:
      consumeInvalidConstructorDeclaration(true);
      break;
    case 214:
      consumeInvalidConstructorDeclaration(false);
      break;
    case 222:
      consumePushLeftBrace();
      break;
    case 223:
      consumeEmptyArrayInitializer();
      break;
    case 224:
      consumeArrayInitializer();
      break;
    case 225:
      consumeArrayInitializer();
      break;
    case 227:
      consumeVariableInitializers();
      break;
    case 228:
      consumeBlock();
      break;
    case 229:
      consumeOpenBlock();
      break;
    case 231:
      consumeBlockStatements();
      break;
    case 235:
      consumeInvalidInterfaceDeclaration();
      break;
    case 236:
      consumeInvalidAnnotationTypeDeclaration();
      break;
    case 237:
      consumeInvalidEnumDeclaration();
      break;
    case 238:
      consumeLocalVariableDeclarationStatement();
      break;
    case 239:
      consumeLocalVariableDeclaration();
      break;
    case 240:
      consumeLocalVariableDeclaration();
      break;
    case 241:
      consumePushModifiers();
      break;
    case 242:
      consumePushModifiersForHeader();
      break;
    case 243:
      consumePushRealModifiers();
      break;
    case 269:
      consumeEmptyStatement();
      break;
    case 270:
      consumeStatementLabel();
      break;
    case 271:
      consumeStatementLabel();
      break;
    case 272:
      consumeLabel();
      break;
    case 273:
      consumeExpressionStatement();
      break;
    case 282:
      consumeStatementIfNoElse();
      break;
    case 283:
      consumeStatementIfWithElse();
      break;
    case 284:
      consumeStatementIfWithElse();
      break;
    case 285:
      consumeStatementSwitch();
      break;
    case 286:
      consumeEmptySwitchBlock();
      break;
    case 289:
      consumeSwitchBlock();
      break;
    case 291:
      consumeSwitchBlockStatements();
      break;
    case 292:
      consumeSwitchBlockStatement();
      break;
    case 294:
      consumeSwitchLabels();
      break;
    case 295:
      consumeCaseLabel();
      break;
    case 296:
      consumeDefaultLabel();
      break;
    case 297:
      consumeStatementWhile();
      break;
    case 298:
      consumeStatementWhile();
      break;
    case 299:
      consumeStatementDo();
      break;
    case 300:
      consumeStatementFor();
      break;
    case 301:
      consumeStatementFor();
      break;
    case 302:
      consumeForInit();
      break;
    case 306:
      consumeStatementExpressionList();
      break;
    case 307:
      consumeSimpleAssertStatement();
      break;
    case 308:
      consumeAssertStatement();
      break;
    case 309:
      consumeStatementBreak();
      break;
    case 310:
      consumeStatementBreakWithLabel();
      break;
    case 311:
      consumeStatementContinue();
      break;
    case 312:
      consumeStatementContinueWithLabel();
      break;
    case 313:
      consumeStatementReturn();
      break;
    case 314:
      consumeStatementThrow();
      break;
    case 315:
      consumeStatementSynchronized();
      break;
    case 316:
      consumeOnlySynchronized();
      break;
    case 317:
      consumeStatementTry(false);
      break;
    case 318:
      consumeStatementTry(true);
      break;
    case 320:
      consumeExitTryBlock();
      break;
    case 322:
      consumeCatches();
      break;
    case 323:
      consumeStatementCatch();
      break;
    case 325:
      consumeLeftParen();
      break;
    case 326:
      consumeRightParen();
      break;
    case 331:
      consumePrimaryNoNewArrayThis();
      break;
    case 332:
      consumePrimaryNoNewArray();
      break;
    case 333:
      consumePrimaryNoNewArrayWithName();
      break;
    case 336:
      consumePrimaryNoNewArrayNameThis();
      break;
    case 337:
      consumePrimaryNoNewArrayNameSuper();
      break;
    case 338:
      consumePrimaryNoNewArrayName();
      break;
    case 339:
      consumePrimaryNoNewArrayArrayType();
      break;
    case 340:
      consumePrimaryNoNewArrayPrimitiveArrayType();
      break;
    case 341:
      consumePrimaryNoNewArrayPrimitiveType();
      break;
    case 344:
      consumeAllocationHeader();
      break;
    case 345:
      consumeClassInstanceCreationExpressionWithTypeArguments();
      break;
    case 346:
      consumeClassInstanceCreationExpression();
      break;
    case 347:
      consumeClassInstanceCreationExpressionQualifiedWithTypeArguments();
      break;
    case 348:
      consumeClassInstanceCreationExpressionQualified();
      break;
    case 349:
      consumeClassInstanceCreationExpressionQualified();
      break;
    case 350:
      consumeClassInstanceCreationExpressionQualifiedWithTypeArguments();
      break;
    case 351:
      consumeClassInstanceCreationExpressionName();
      break;
    case 352:
      consumeClassBodyopt();
      break;
    case 354:
      consumeEnterAnonymousClassBody();
      break;
    case 356:
      consumeArgumentList();
      break;
    case 357:
      consumeArrayCreationHeader();
      break;
    case 358:
      consumeArrayCreationHeader();
      break;
    case 359:
      consumeArrayCreationExpressionWithoutInitializer();
      break;
    case 360:
      consumeArrayCreationExpressionWithInitializer();
      break;
    case 361:
      consumeArrayCreationExpressionWithoutInitializer();
      break;
    case 362:
      consumeArrayCreationExpressionWithInitializer();
      break;
    case 364:
      consumeDimWithOrWithOutExprs();
      break;
    case 366:
      consumeDimWithOrWithOutExpr();
      break;
    case 367:
      consumeDims();
      break;
    case 370:
      consumeOneDimLoop();
      break;
    case 371:
      consumeFieldAccess(false);
      break;
    case 372:
      consumeFieldAccess(true);
      break;
    case 373:
      consumeMethodInvocationName();
      break;
    case 374:
      consumeMethodInvocationNameWithTypeArguments();
      break;
    case 375:
      consumeMethodInvocationPrimaryWithTypeArguments();
      break;
    case 376:
      consumeMethodInvocationPrimary();
      break;
    case 377:
      consumeMethodInvocationSuperWithTypeArguments();
      break;
    case 378:
      consumeMethodInvocationSuper();
      break;
    case 379:
      consumeArrayAccess(true);
      break;
    case 380:
      consumeArrayAccess(false);
      break;
    case 381:
      consumeArrayAccess(false);
      break;
    case 383:
      consumePostfixExpression();
      break;
    case 386:
      consumeUnaryExpression(14, true);
      break;
    case 387:
      consumeUnaryExpression(13, true);
      break;
    case 388:
      consumePushPosition();
      break;
    case 391:
      consumeUnaryExpression(14);
      break;
    case 392:
      consumeUnaryExpression(13);
      break;
    case 394:
      consumeUnaryExpression(14, false);
      break;
    case 395:
      consumeUnaryExpression(13, false);
      break;
    case 397:
      consumeUnaryExpression(12);
      break;
    case 398:
      consumeUnaryExpression(11);
      break;
    case 400:
      consumeCastExpressionWithPrimitiveType();
      break;
    case 401:
      consumeCastExpressionWithGenericsArray();
      break;
    case 402:
      consumeCastExpressionWithQualifiedGenericsArray();
      break;
    case 403:
      consumeCastExpressionLL1();
      break;
    case 404:
      consumeCastExpressionWithNameArray();
      break;
    case 405:
      consumeOnlyTypeArgumentsForCastExpression();
      break;
    case 406:
      consumeInsideCastExpression();
      break;
    case 407:
      consumeInsideCastExpressionLL1();
      break;
    case 408:
      consumeInsideCastExpressionWithQualifiedGenerics();
      break;
    case 410:
      consumeBinaryExpression(15);
      break;
    case 411:
      consumeBinaryExpression(9);
      break;
    case 412:
      consumeBinaryExpression(16);
      break;
    case 414:
      consumeBinaryExpression(14);
      break;
    case 415:
      consumeBinaryExpression(13);
      break;
    case 417:
      consumeBinaryExpression(10);
      break;
    case 418:
      consumeBinaryExpression(17);
      break;
    case 419:
      consumeBinaryExpression(19);
      break;
    case 421:
      consumeBinaryExpression(4);
      break;
    case 422:
      consumeBinaryExpression(6);
      break;
    case 423:
      consumeBinaryExpression(5);
      break;
    case 424:
      consumeBinaryExpression(7);
      break;
    case 426:
      consumeInstanceOfExpression();
      break;
    case 428:
      consumeEqualityExpression(18);
      break;
    case 429:
      consumeEqualityExpression(29);
      break;
    case 431:
      consumeBinaryExpression(2);
      break;
    case 433:
      consumeBinaryExpression(8);
      break;
    case 435:
      consumeBinaryExpression(3);
      break;
    case 437:
      consumeBinaryExpression(0);
      break;
    case 439:
      consumeBinaryExpression(1);
      break;
    case 441:
      consumeConditionalExpression(23);
      break;
    case 444:
      consumeAssignment();
      break;
    case 446:
      ignoreExpressionAssignment();
      break;
    case 447:
      consumeAssignmentOperator(30);
      break;
    case 448:
      consumeAssignmentOperator(15);
      break;
    case 449:
      consumeAssignmentOperator(9);
      break;
    case 450:
      consumeAssignmentOperator(16);
      break;
    case 451:
      consumeAssignmentOperator(14);
      break;
    case 452:
      consumeAssignmentOperator(13);
      break;
    case 453:
      consumeAssignmentOperator(10);
      break;
    case 454:
      consumeAssignmentOperator(17);
      break;
    case 455:
      consumeAssignmentOperator(19);
      break;
    case 456:
      consumeAssignmentOperator(2);
      break;
    case 457:
      consumeAssignmentOperator(8);
      break;
    case 458:
      consumeAssignmentOperator(3);
      break;
    case 462:
      consumeEmptyExpression();
      break;
    case 467:
      consumeEmptyClassBodyDeclarationsopt();
      break;
    case 468:
      consumeClassBodyDeclarationsopt();
      break;
    case 469:
      consumeDefaultModifiers();
      break;
    case 470:
      consumeModifiers();
      break;
    case 471:
      consumeEmptyBlockStatementsopt();
      break;
    case 473:
      consumeEmptyDimsopt();
      break;
    case 475:
      consumeEmptyArgumentListopt();
      break;
    case 479:
      consumeFormalParameterListopt();
      break;
    case 483:
      consumeEmptyInterfaceMemberDeclarationsopt();
      break;
    case 484:
      consumeInterfaceMemberDeclarationsopt();
      break;
    case 485:
      consumeNestedType();
      break;
    case 486:
      consumeEmptyForInitopt();
      break;
    case 488:
      consumeEmptyForUpdateopt();
      break;
    case 492:
      consumeEmptyCatchesopt();
      break;
    case 494:
      consumeEnumDeclaration();
      break;
    case 495:
      consumeEnumHeader();
      break;
    case 496:
      consumeEnumHeaderName();
      break;
    case 497:
      consumeEnumBodyNoConstants();
      break;
    case 498:
      consumeEnumBodyNoConstants();
      break;
    case 499:
      consumeEnumBodyWithConstants();
      break;
    case 500:
      consumeEnumBodyWithConstants();
      break;
    case 502:
      consumeEnumConstants();
      break;
    case 503:
      consumeEnumConstantHeaderName();
      break;
    case 504:
      consumeEnumConstantHeader();
      break;
    case 505:
      consumeEnumConstantWithClassBody();
      break;
    case 506:
      consumeEnumConstantNoClassBody();
      break;
    case 507:
      consumeArguments();
      break;
    case 508:
      consumeEmptyArguments();
      break;
    case 510:
      consumeEnumDeclarations();
      break;
    case 511:
      consumeEmptyEnumDeclarations();
      break;
    case 513:
      consumeEnhancedForStatement();
      break;
    case 514:
      consumeEnhancedForStatement();
      break;
    case 515:
      consumeEnhancedForStatementHeaderInit(false);
      break;
    case 516:
      consumeEnhancedForStatementHeaderInit(true);
      break;
    case 517:
      consumeEnhancedForStatementHeader();
      break;
    case 518:
      consumeImportDeclaration();
      break;
    case 519:
      consumeSingleStaticImportDeclarationName();
      break;
    case 520:
      consumeImportDeclaration();
      break;
    case 521:
      consumeStaticImportOnDemandDeclarationName();
      break;
    case 522:
      consumeTypeArguments();
      break;
    case 523:
      consumeOnlyTypeArguments();
      break;
    case 525:
      consumeTypeArgumentList1();
      break;
    case 527:
      consumeTypeArgumentList();
      break;
    case 528:
      consumeTypeArgument();
      break;
    case 532:
      consumeReferenceType1();
      break;
    case 533:
      consumeTypeArgumentReferenceType1();
      break;
    case 535:
      consumeTypeArgumentList2();
      break;
    case 538:
      consumeReferenceType2();
      break;
    case 539:
      consumeTypeArgumentReferenceType2();
      break;
    case 541:
      consumeTypeArgumentList3();
      break;
    case 544:
      consumeReferenceType3();
      break;
    case 545:
      consumeWildcard();
      break;
    case 546:
      consumeWildcardWithBounds();
      break;
    case 547:
      consumeWildcardBoundsExtends();
      break;
    case 548:
      consumeWildcardBoundsSuper();
      break;
    case 549:
      consumeWildcard1();
      break;
    case 550:
      consumeWildcard1WithBounds();
      break;
    case 551:
      consumeWildcardBounds1Extends();
      break;
    case 552:
      consumeWildcardBounds1Super();
      break;
    case 553:
      consumeWildcard2();
      break;
    case 554:
      consumeWildcard2WithBounds();
      break;
    case 555:
      consumeWildcardBounds2Extends();
      break;
    case 556:
      consumeWildcardBounds2Super();
      break;
    case 557:
      consumeWildcard3();
      break;
    case 558:
      consumeWildcard3WithBounds();
      break;
    case 559:
      consumeWildcardBounds3Extends();
      break;
    case 560:
      consumeWildcardBounds3Super();
      break;
    case 561:
      consumeTypeParameterHeader();
      break;
    case 562:
      consumeTypeParameters();
      break;
    case 564:
      consumeTypeParameterList();
      break;
    case 566:
      consumeTypeParameterWithExtends();
      break;
    case 567:
      consumeTypeParameterWithExtendsAndBounds();
      break;
    case 569:
      consumeAdditionalBoundList();
      break;
    case 570:
      consumeAdditionalBound();
      break;
    case 572:
      consumeTypeParameterList1();
      break;
    case 573:
      consumeTypeParameter1();
      break;
    case 574:
      consumeTypeParameter1WithExtends();
      break;
    case 575:
      consumeTypeParameter1WithExtendsAndBounds();
      break;
    case 577:
      consumeAdditionalBoundList1();
      break;
    case 578:
      consumeAdditionalBound1();
      break;
    case 584:
      consumeUnaryExpression(14);
      break;
    case 585:
      consumeUnaryExpression(13);
      break;
    case 588:
      consumeUnaryExpression(12);
      break;
    case 589:
      consumeUnaryExpression(11);
      break;
    case 592:
      consumeBinaryExpression(15);
      break;
    case 593:
      consumeBinaryExpressionWithName(15);
      break;
    case 594:
      consumeBinaryExpression(9);
      break;
    case 595:
      consumeBinaryExpressionWithName(9);
      break;
    case 596:
      consumeBinaryExpression(16);
      break;
    case 597:
      consumeBinaryExpressionWithName(16);
      break;
    case 599:
      consumeBinaryExpression(14);
      break;
    case 600:
      consumeBinaryExpressionWithName(14);
      break;
    case 601:
      consumeBinaryExpression(13);
      break;
    case 602:
      consumeBinaryExpressionWithName(13);
      break;
    case 604:
      consumeBinaryExpression(10);
      break;
    case 605:
      consumeBinaryExpressionWithName(10);
      break;
    case 606:
      consumeBinaryExpression(17);
      break;
    case 607:
      consumeBinaryExpressionWithName(17);
      break;
    case 608:
      consumeBinaryExpression(19);
      break;
    case 609:
      consumeBinaryExpressionWithName(19);
      break;
    case 611:
      consumeBinaryExpression(4);
      break;
    case 612:
      consumeBinaryExpressionWithName(4);
      break;
    case 613:
      consumeBinaryExpression(6);
      break;
    case 614:
      consumeBinaryExpressionWithName(6);
      break;
    case 615:
      consumeBinaryExpression(5);
      break;
    case 616:
      consumeBinaryExpressionWithName(5);
      break;
    case 617:
      consumeBinaryExpression(7);
      break;
    case 618:
      consumeBinaryExpressionWithName(7);
      break;
    case 620:
      consumeInstanceOfExpressionWithName();
      break;
    case 621:
      consumeInstanceOfExpression();
      break;
    case 623:
      consumeEqualityExpression(18);
      break;
    case 624:
      consumeEqualityExpressionWithName(18);
      break;
    case 625:
      consumeEqualityExpression(29);
      break;
    case 626:
      consumeEqualityExpressionWithName(29);
      break;
    case 628:
      consumeBinaryExpression(2);
      break;
    case 629:
      consumeBinaryExpressionWithName(2);
      break;
    case 631:
      consumeBinaryExpression(8);
      break;
    case 632:
      consumeBinaryExpressionWithName(8);
      break;
    case 634:
      consumeBinaryExpression(3);
      break;
    case 635:
      consumeBinaryExpressionWithName(3);
      break;
    case 637:
      consumeBinaryExpression(0);
      break;
    case 638:
      consumeBinaryExpressionWithName(0);
      break;
    case 640:
      consumeBinaryExpression(1);
      break;
    case 641:
      consumeBinaryExpressionWithName(1);
      break;
    case 643:
      consumeConditionalExpression(23);
      break;
    case 644:
      consumeConditionalExpressionWithName(23);
      break;
    case 648:
      consumeAnnotationTypeDeclarationHeaderName();
      break;
    case 649:
      consumeAnnotationTypeDeclarationHeaderName();
      break;
    case 650:
      consumeAnnotationTypeDeclarationHeader();
      break;
    case 651:
      consumeAnnotationTypeDeclaration();
      break;
    case 653:
      consumeEmptyAnnotationTypeMemberDeclarationsopt();
      break;
    case 654:
      consumeAnnotationTypeMemberDeclarationsopt();
      break;
    case 656:
      consumeAnnotationTypeMemberDeclarations();
      break;
    case 657:
      consumeMethodHeaderNameWithTypeParameters(true);
      break;
    case 658:
      consumeMethodHeaderName(true);
      break;
    case 659:
      consumeEmptyMethodHeaderDefaultValue();
      break;
    case 660:
      consumeMethodHeaderDefaultValue();
      break;
    case 661:
      consumeMethodHeader();
      break;
    case 662:
      consumeAnnotationTypeMemberDeclaration();
      break;
    case 670:
      consumeAnnotationName();
      break;
    case 671:
      consumeNormalAnnotation();
      break;
    case 672:
      consumeEmptyMemberValuePairsopt();
      break;
    case 675:
      consumeMemberValuePairs();
      break;
    case 676:
      consumeMemberValuePair();
      break;
    case 677:
      consumeEnterMemberValue();
      break;
    case 678:
      consumeExitMemberValue();
      break;
    case 680:
      consumeMemberValueAsName();
      break;
    case 683:
      consumeMemberValueArrayInitializer();
      break;
    case 684:
      consumeMemberValueArrayInitializer();
      break;
    case 685:
      consumeEmptyMemberValueArrayInitializer();
      break;
    case 686:
      consumeEmptyMemberValueArrayInitializer();
      break;
    case 687:
      consumeEnterMemberValueArrayInitializer();
      break;
    case 689:
      consumeMemberValues();
      break;
    case 690:
      consumeMarkerAnnotation();
      break;
    case 691:
      consumeSingleMemberAnnotation();
      break;
    case 692:
      consumeRecoveryMethodHeaderNameWithTypeParameters();
      break;
    case 693:
      consumeRecoveryMethodHeaderName();
      break;
    case 694:
      consumeMethodHeader();
      break;
    case 695:
      consumeMethodHeader();
    case 31:
    case 32:
    case 33:
    case 34:
    case 35:
    case 36:
    case 37:
    case 38:
    case 39:
    case 40:
    case 41:
    case 42:
    case 43:
    case 45:
    case 46:
    case 47:
    case 56:
    case 57:
    case 58:
    case 59:
    case 72:
    case 73:
    case 74:
    case 75:
    case 76:
    case 77:
    case 78:
    case 79:
    case 80:
    case 81:
    case 82:
    case 83:
    case 84:
    case 85:
    case 86:
    case 88:
    case 90:
    case 96:
    case 97:
    case 98:
    case 99:
    case 104:
    case 105:
    case 107:
    case 108:
    case 109:
    case 111:
    case 112:
    case 113:
    case 114:
    case 115:
    case 116:
    case 117:
    case 118:
    case 119:
    case 120:
    case 121:
    case 126:
    case 130:
    case 133:
    case 134:
    case 136:
    case 137:
    case 138:
    case 142:
    case 143:
    case 144:
    case 145:
    case 146:
    case 147:
    case 149:
    case 150:
    case 152:
    case 154:
    case 155:
    case 161:
    case 162:
    case 163:
    case 164:
    case 176:
    case 180:
    case 204:
    case 207:
    case 208:
    case 211:
    case 215:
    case 216:
    case 217:
    case 218:
    case 219:
    case 220:
    case 221:
    case 226:
    case 230:
    case 232:
    case 233:
    case 234:
    case 244:
    case 245:
    case 246:
    case 247:
    case 248:
    case 249:
    case 250:
    case 251:
    case 252:
    case 253:
    case 254:
    case 255:
    case 256:
    case 257:
    case 258:
    case 259:
    case 260:
    case 261:
    case 262:
    case 263:
    case 264:
    case 265:
    case 266:
    case 267:
    case 268:
    case 274:
    case 275:
    case 276:
    case 277:
    case 278:
    case 279:
    case 280:
    case 281:
    case 287:
    case 288:
    case 290:
    case 293:
    case 303:
    case 304:
    case 305:
    case 319:
    case 321:
    case 324:
    case 327:
    case 328:
    case 329:
    case 330:
    case 334:
    case 335:
    case 342:
    case 343:
    case 353:
    case 355:
    case 363:
    case 365:
    case 368:
    case 369:
    case 382:
    case 384:
    case 385:
    case 389:
    case 390:
    case 393:
    case 396:
    case 399:
    case 409:
    case 413:
    case 416:
    case 420:
    case 425:
    case 427:
    case 430:
    case 432:
    case 434:
    case 436:
    case 438:
    case 440:
    case 442:
    case 443:
    case 445:
    case 459:
    case 460:
    case 461:
    case 463:
    case 464:
    case 465:
    case 466:
    case 472:
    case 474:
    case 476:
    case 477:
    case 478:
    case 480:
    case 481:
    case 482:
    case 487:
    case 489:
    case 490:
    case 491:
    case 493:
    case 501:
    case 509:
    case 512:
    case 524:
    case 526:
    case 529:
    case 530:
    case 531:
    case 534:
    case 536:
    case 537:
    case 540:
    case 542:
    case 543:
    case 563:
    case 565:
    case 568:
    case 571:
    case 576:
    case 579:
    case 580:
    case 581:
    case 582:
    case 583:
    case 586:
    case 587:
    case 590:
    case 591:
    case 598:
    case 603:
    case 610:
    case 619:
    case 622:
    case 627:
    case 630:
    case 633:
    case 636:
    case 639:
    case 642:
    case 645:
    case 646:
    case 647:
    case 652:
    case 655:
    case 663:
    case 664:
    case 665:
    case 666:
    case 667:
    case 668:
    case 669:
    case 673:
    case 674:
    case 679:
    case 681:
    case 682:
    case 688: }  } 
  protected void consumeSimpleAssertStatement() { this.expressionLengthPtr -= 1;
    pushOnAstStack(new AssertStatement(this.expressionStack[(this.expressionPtr--)], this.intStack[(this.intPtr--)])); }

  protected void consumeSingleMemberAnnotation()
  {
    SingleMemberAnnotation singleMemberAnnotation = null;
    TypeReference typeReference = getAnnotationType();
    singleMemberAnnotation = new SingleMemberAnnotation(typeReference, this.intStack[(this.intPtr--)]);
    singleMemberAnnotation.memberValue = this.expressionStack[(this.expressionPtr--)];
    this.expressionLengthPtr -= 1;
    singleMemberAnnotation.declarationSourceEnd = this.rParenPos;
    pushOnExpressionStack(singleMemberAnnotation);

    if (this.currentElement != null) {
      annotationRecoveryCheckPoint(singleMemberAnnotation.sourceStart, singleMemberAnnotation.declarationSourceEnd);
    }

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      problemReporter().invalidUsageOfAnnotation(singleMemberAnnotation);
    }
    this.recordStringLiterals = true;
  }

  protected void consumeSingleStaticImportDeclarationName()
  {
    int length;
    char[][] tokens = new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
    ImportReference impt;
    pushOnAstStack(impt = new ImportReference(tokens, positions, false, 8));

    this.modifiers = 0;
    this.modifiersSourceStart = -1;

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    impt.declarationSourceStart = this.intStack[(this.intPtr--)];

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      impt.modifiers = 0;
      problemReporter().invalidUsageOfStaticImports(impt);
    }

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.currentElement = this.currentElement.add(impt, 0);
      this.lastIgnoredToken = -1;
      this.restartRecovery = true;
    }
  }

  protected void consumeSingleTypeImportDeclarationName()
  {
    int length;
    char[][] tokens = new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
    ImportReference impt;
    pushOnAstStack(impt = new ImportReference(tokens, positions, false, 0));

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    impt.declarationSourceStart = this.intStack[(this.intPtr--)];

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.currentElement = this.currentElement.add(impt, 0);
      this.lastIgnoredToken = -1;
      this.restartRecovery = true;
    }
  }

  protected void consumeStatementBreak()
  {
    pushOnAstStack(new BreakStatement(null, this.intStack[(this.intPtr--)], this.endStatementPosition));

    if (this.pendingRecoveredType != null)
    {
      if ((this.pendingRecoveredType.allocation == null) && 
        (this.endPosition <= this.pendingRecoveredType.declarationSourceEnd)) {
        this.astStack[this.astPtr] = this.pendingRecoveredType;
        this.pendingRecoveredType = null;
        return;
      }
      this.pendingRecoveredType = null;
    }
  }

  protected void consumeStatementBreakWithLabel()
  {
    pushOnAstStack(
      new BreakStatement(
      this.identifierStack[(this.identifierPtr--)], 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition));
    this.identifierLengthPtr -= 1;
  }

  protected void consumeStatementCatch()
  {
    this.astLengthPtr -= 1;
    this.listLength = 0;
  }

  protected void consumeStatementContinue()
  {
    pushOnAstStack(
      new ContinueStatement(
      null, 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition));
  }

  protected void consumeStatementContinueWithLabel()
  {
    pushOnAstStack(
      new ContinueStatement(
      this.identifierStack[(this.identifierPtr--)], 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition));
    this.identifierLengthPtr -= 1;
  }

  protected void consumeStatementDo()
  {
    this.intPtr -= 1;

    Statement statement = (Statement)this.astStack[this.astPtr];
    this.expressionLengthPtr -= 1;
    this.astStack[this.astPtr] = 
      new DoStatement(
      this.expressionStack[(this.expressionPtr--)], 
      statement, 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition);
  }

  protected void consumeStatementExpressionList() {
    concatExpressionLists();
  }

  protected void consumeStatementFor()
  {
    Expression cond = null;

    boolean scope = true;

    this.astLengthPtr -= 1;
    Statement statement = (Statement)this.astStack[(this.astPtr--)];
    int length;
    Statement[] updates;
    Statement[] updates;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) == 0) {
      updates = (Statement[])null;
    } else {
      this.expressionPtr -= length;
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr + 1, 
        updates = new Statement[length], 
        0, 
        length);
    }

    if (this.expressionLengthStack[(this.expressionLengthPtr--)] != 0)
      cond = this.expressionStack[(this.expressionPtr--)];
    Statement[] inits;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) == 0) {
      Statement[] inits = (Statement[])null;
      scope = false;
    }
    else if (length == -1) {
      scope = false;
      length = this.expressionLengthStack[(this.expressionLengthPtr--)];
      this.expressionPtr -= length;
      Statement[] inits;
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr + 1, 
        inits = new Statement[length], 
        0, 
        length);
    } else {
      this.astPtr -= length;
      System.arraycopy(
        this.astStack, 
        this.astPtr + 1, 
        inits = new Statement[length], 
        0, 
        length);
    }

    pushOnAstStack(
      new ForStatement(
      inits, 
      cond, 
      updates, 
      statement, 
      scope, 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition));
  }

  protected void consumeStatementIfNoElse()
  {
    this.expressionLengthPtr -= 1;
    Statement thenStatement = (Statement)this.astStack[this.astPtr];
    this.astStack[this.astPtr] = 
      new IfStatement(
      this.expressionStack[(this.expressionPtr--)], 
      thenStatement, 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition);
  }

  protected void consumeStatementIfWithElse()
  {
    this.expressionLengthPtr -= 1;

    this.astLengthPtr -= 1;

    this.astStack[(--this.astPtr)] = 
      new IfStatement(
      this.expressionStack[(this.expressionPtr--)], 
      (Statement)this.astStack[this.astPtr], 
      (Statement)this.astStack[(this.astPtr + 1)], 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition);
  }

  protected void consumeStatementLabel()
  {
    Statement statement = (Statement)this.astStack[this.astPtr];
    this.astStack[this.astPtr] = 
      new LabeledStatement(
      this.identifierStack[this.identifierPtr], 
      statement, 
      this.identifierPositionStack[(this.identifierPtr--)], 
      this.endStatementPosition);
    this.identifierLengthPtr -= 1;
  }

  protected void consumeStatementReturn()
  {
    if (this.expressionLengthStack[(this.expressionLengthPtr--)] != 0) {
      pushOnAstStack(
        new ReturnStatement(
        this.expressionStack[(this.expressionPtr--)], 
        this.intStack[(this.intPtr--)], 
        this.endStatementPosition));
    }
    else
      pushOnAstStack(new ReturnStatement(null, this.intStack[(this.intPtr--)], this.endStatementPosition));
  }

  protected void consumeStatementSwitch()
  {
    SwitchStatement switchStatement = new SwitchStatement();
    this.expressionLengthPtr -= 1;
    switchStatement.expression = this.expressionStack[(this.expressionPtr--)];
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      this.astPtr -= length;
      System.arraycopy(
        this.astStack, 
        this.astPtr + 1, 
        switchStatement.statements = new Statement[length], 
        0, 
        length);
    }
    switchStatement.explicitDeclarations = this.realBlockStack[(this.realBlockPtr--)];
    pushOnAstStack(switchStatement);
    switchStatement.blockStart = this.intStack[(this.intPtr--)];
    switchStatement.sourceStart = this.intStack[(this.intPtr--)];
    switchStatement.sourceEnd = this.endStatementPosition;
    if ((length == 0) && (!containsComment(switchStatement.blockStart, switchStatement.sourceEnd)))
      switchStatement.bits |= 8;
  }

  protected void consumeStatementSynchronized()
  {
    if (this.astLengthStack[this.astLengthPtr] == 0) {
      this.astLengthStack[this.astLengthPtr] = 1;
      this.expressionLengthPtr -= 1;
      this.astStack[(++this.astPtr)] = 
        new SynchronizedStatement(
        this.expressionStack[(this.expressionPtr--)], 
        null, 
        this.intStack[(this.intPtr--)], 
        this.endStatementPosition);
    } else {
      this.expressionLengthPtr -= 1;
      this.astStack[this.astPtr] = 
        new SynchronizedStatement(
        this.expressionStack[(this.expressionPtr--)], 
        (Block)this.astStack[this.astPtr], 
        this.intStack[(this.intPtr--)], 
        this.endStatementPosition);
    }
    resetModifiers();
  }

  protected void consumeStatementThrow() {
    this.expressionLengthPtr -= 1;
    pushOnAstStack(new ThrowStatement(this.expressionStack[(this.expressionPtr--)], this.intStack[(this.intPtr--)], this.endStatementPosition));
  }

  protected void consumeStatementTry(boolean withFinally)
  {
    TryStatement tryStmt = new TryStatement();

    if (withFinally) {
      this.astLengthPtr -= 1;
      tryStmt.finallyBlock = ((Block)this.astStack[(this.astPtr--)]);
    }
    int length;
    if ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0) {
      if (length == 1) {
        tryStmt.catchBlocks = new Block[] { (Block)this.astStack[(this.astPtr--)] };
        tryStmt.catchArguments = new Argument[] { (Argument)this.astStack[(this.astPtr--)] };
      } else {
        Block[] bks = tryStmt.catchBlocks = new Block[length];
        Argument[] args = tryStmt.catchArguments = new Argument[length];
        while (length-- > 0) {
          bks[length] = ((Block)this.astStack[(this.astPtr--)]);
          args[length] = ((Argument)this.astStack[(this.astPtr--)]);
        }
      }
    }

    this.astLengthPtr -= 1;
    tryStmt.tryBlock = ((Block)this.astStack[(this.astPtr--)]);

    tryStmt.sourceEnd = this.endStatementPosition;
    tryStmt.sourceStart = this.intStack[(this.intPtr--)];
    pushOnAstStack(tryStmt);
  }

  protected void consumeStatementWhile()
  {
    this.expressionLengthPtr -= 1;
    Statement statement = (Statement)this.astStack[this.astPtr];
    this.astStack[this.astPtr] = 
      new WhileStatement(
      this.expressionStack[(this.expressionPtr--)], 
      statement, 
      this.intStack[(this.intPtr--)], 
      this.endStatementPosition);
  }

  protected void consumeStaticImportOnDemandDeclarationName()
  {
    int length;
    char[][] tokens = new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
    ImportReference impt;
    pushOnAstStack(impt = new ImportReference(tokens, positions, true, 8));

    this.modifiers = 0;
    this.modifiersSourceStart = -1;

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    impt.declarationSourceStart = this.intStack[(this.intPtr--)];

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      impt.modifiers = 0;
      problemReporter().invalidUsageOfStaticImports(impt);
    }

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.currentElement = this.currentElement.add(impt, 0);
      this.lastIgnoredToken = -1;
      this.restartRecovery = true;
    }
  }

  protected void consumeStaticInitializer()
  {
    Block block = (Block)this.astStack[this.astPtr];
    if (this.diet) block.bits &= -9;
    Initializer initializer = new Initializer(block, 8);
    this.astStack[this.astPtr] = initializer;
    initializer.sourceEnd = this.endStatementPosition;
    initializer.declarationSourceEnd = flushCommentsDefinedPriorTo(this.endStatementPosition);
    this.nestedMethod[this.nestedType] -= 1;
    initializer.declarationSourceStart = this.intStack[(this.intPtr--)];
    initializer.bodyStart = this.intStack[(this.intPtr--)];
    initializer.bodyEnd = this.endPosition;

    initializer.javadoc = this.javadoc;
    this.javadoc = null;

    if (this.currentElement != null) {
      this.lastCheckPoint = initializer.declarationSourceEnd;
      this.currentElement = this.currentElement.add(initializer, 0);
      this.lastIgnoredToken = -1;
    }
  }

  protected void consumeStaticOnly() {
    int savedModifiersSourceStart = this.modifiersSourceStart;
    checkComment();
    if (this.modifiersSourceStart >= savedModifiersSourceStart) {
      this.modifiersSourceStart = savedModifiersSourceStart;
    }
    pushOnIntStack(this.scanner.currentPosition);
    pushOnIntStack(
      this.modifiersSourceStart >= 0 ? this.modifiersSourceStart : this.scanner.startPosition);
    jumpOverMethodBody();
    this.nestedMethod[this.nestedType] += 1;
    resetModifiers();
    this.expressionLengthPtr -= 1;

    if (this.currentElement != null)
      this.recoveredStaticInitializerStart = this.intStack[this.intPtr];
  }

  protected void consumeSwitchBlock()
  {
    concatNodeLists();
  }

  protected void consumeSwitchBlockStatement() {
    concatNodeLists();
  }

  protected void consumeSwitchBlockStatements() {
    concatNodeLists();
  }

  protected void consumeSwitchLabels() {
    optimizedConcatNodeLists();
  }

  protected void consumeToken(int type)
  {
    switch (type) {
    case 26:
      pushIdentifier();
      if ((this.scanner.useAssertAsAnIndentifier) && 
        (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
        long positions = this.identifierPositionStack[this.identifierPtr];
        if (!this.statementRecoveryActivated) problemReporter().useAssertAsAnIdentifier((int)(positions >>> 32), (int)positions);
      }
      if ((this.scanner.useEnumAsAnIndentifier) && 
        (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
        long positions = this.identifierPositionStack[this.identifierPtr];
        if (!this.statementRecoveryActivated) problemReporter().useEnumAsAnIdentifier((int)(positions >>> 32), (int)positions);
      }
      break;
    case 95:
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 56:
      checkAndSetModifiers(1024);
      pushOnExpressionStackLengthStack(0);
      break;
    case 62:
      checkAndSetModifiers(2048);
      pushOnExpressionStackLengthStack(0);
      break;
    case 57:
      checkAndSetModifiers(16);
      pushOnExpressionStackLengthStack(0);
      break;
    case 58:
      checkAndSetModifiers(256);
      pushOnExpressionStackLengthStack(0);
      break;
    case 59:
      checkAndSetModifiers(2);
      pushOnExpressionStackLengthStack(0);
      break;
    case 60:
      checkAndSetModifiers(4);
      pushOnExpressionStackLengthStack(0);
      break;
    case 61:
      checkAndSetModifiers(1);
      pushOnExpressionStackLengthStack(0);
      break;
    case 63:
      checkAndSetModifiers(128);
      pushOnExpressionStackLengthStack(0);
      break;
    case 64:
      checkAndSetModifiers(64);
      pushOnExpressionStackLengthStack(0);
      break;
    case 54:
      checkAndSetModifiers(8);
      pushOnExpressionStackLengthStack(0);
      break;
    case 55:
      this.synchronizedBlockSourceStart = this.scanner.startPosition;
      checkAndSetModifiers(32);
      pushOnExpressionStackLengthStack(0);
      break;
    case 40:
      pushIdentifier(-6);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 32:
      pushIdentifier(-5);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 33:
      pushIdentifier(-3);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 34:
      pushIdentifier(-2);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 35:
      pushIdentifier(-8);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 36:
      pushIdentifier(-9);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 37:
      pushIdentifier(-10);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 38:
      pushIdentifier(-7);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 39:
      pushIdentifier(-4);
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 47:
      pushOnExpressionStack(
        new IntLiteral(
        this.scanner.getCurrentTokenSource(), 
        this.scanner.startPosition, 
        this.scanner.currentPosition - 1));
      break;
    case 48:
      pushOnExpressionStack(
        new LongLiteral(
        this.scanner.getCurrentTokenSource(), 
        this.scanner.startPosition, 
        this.scanner.currentPosition - 1));
      break;
    case 49:
      pushOnExpressionStack(
        new FloatLiteral(
        this.scanner.getCurrentTokenSource(), 
        this.scanner.startPosition, 
        this.scanner.currentPosition - 1));
      break;
    case 50:
      pushOnExpressionStack(
        new DoubleLiteral(
        this.scanner.getCurrentTokenSource(), 
        this.scanner.startPosition, 
        this.scanner.currentPosition - 1));
      break;
    case 51:
      pushOnExpressionStack(
        new CharLiteral(
        this.scanner.getCurrentTokenSource(), 
        this.scanner.startPosition, 
        this.scanner.currentPosition - 1));
      break;
    case 52:
      StringLiteral stringLiteral;
      if ((this.recordStringLiterals) && (this.checkExternalizeStrings) && (!this.statementRecoveryActivated)) {
        StringLiteral stringLiteral = createStringLiteral(
          this.scanner.getCurrentTokenSourceString(), 
          this.scanner.startPosition, 
          this.scanner.currentPosition - 1, 
          Util.getLineNumber(this.scanner.startPosition, this.scanner.lineEnds, 0, this.scanner.linePtr));
        this.compilationUnit.recordStringLiteral(stringLiteral);
      } else {
        stringLiteral = createStringLiteral(
          this.scanner.getCurrentTokenSourceString(), 
          this.scanner.startPosition, 
          this.scanner.currentPosition - 1, 
          0);
      }
      pushOnExpressionStack(stringLiteral);
      break;
    case 44:
      pushOnExpressionStack(
        new FalseLiteral(this.scanner.startPosition, this.scanner.currentPosition - 1));
      break;
    case 46:
      pushOnExpressionStack(
        new TrueLiteral(this.scanner.startPosition, this.scanner.currentPosition - 1));
      break;
    case 45:
      pushOnExpressionStack(
        new NullLiteral(this.scanner.startPosition, this.scanner.currentPosition - 1));
      break;
    case 41:
    case 42:
      this.endPosition = (this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 73:
    case 74:
    case 75:
    case 76:
    case 77:
    case 78:
    case 79:
    case 80:
    case 81:
    case 82:
    case 83:
    case 96:
    case 100:
    case 101:
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 43:
      resetModifiers();
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 72:
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 98:
      pushOnIntStack(this.scanner.currentPosition - 1);
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 97:
      pushOnIntStack(this.scanner.startPosition);
      pushOnIntStack(this.scanner.currentPosition - 1);
      break;
    case 70:
      this.endPosition = this.scanner.startPosition;
      this.endStatementPosition = (this.scanner.currentPosition - 1);
      break;
    case 69:
      this.endStatementPosition = (this.scanner.currentPosition - 1);
    case 1:
    case 2:
    case 66:
    case 67:
      this.endPosition = this.scanner.startPosition;
      break;
    case 9:
    case 10:
      this.endPosition = this.scanner.startPosition;
      this.endStatementPosition = (this.scanner.currentPosition - 1);
      break;
    case 27:
    case 31:
      this.endStatementPosition = (this.scanner.currentPosition - 1);
      this.endPosition = (this.scanner.startPosition - 1);

      break;
    case 29:
      this.rParenPos = (this.scanner.currentPosition - 1);
      break;
    case 28:
      this.lParenPos = this.scanner.startPosition;
      break;
    case 53:
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 23:
      pushOnIntStack(this.scanner.startPosition);
      pushOnIntStack(this.scanner.currentPosition - 1);
      break;
    case 7:
      pushOnIntStack(this.scanner.startPosition);
      break;
    case 107:
      pushOnIntStack(this.scanner.currentPosition - 1);
    case 3:
    case 4:
    case 5:
    case 6:
    case 8:
    case 11:
    case 12:
    case 13:
    case 14:
    case 15:
    case 16:
    case 17:
    case 18:
    case 19:
    case 20:
    case 21:
    case 22:
    case 24:
    case 25:
    case 30:
    case 65:
    case 68:
    case 71:
    case 84:
    case 85:
    case 86:
    case 87:
    case 88:
    case 89:
    case 90:
    case 91:
    case 92:
    case 93:
    case 94:
    case 99:
    case 102:
    case 103:
    case 104:
    case 105:
    case 106:
    }
  }

  protected void consumeTypeArgument()
  {
    pushOnGenericsStack(getTypeReference(this.intStack[(this.intPtr--)]));
  }
  protected void consumeTypeArgumentList() {
    concatGenericsLists();
  }
  protected void consumeTypeArgumentList1() {
    concatGenericsLists();
  }
  protected void consumeTypeArgumentList2() {
    concatGenericsLists();
  }
  protected void consumeTypeArgumentList3() {
    concatGenericsLists();
  }
  protected void consumeTypeArgumentReferenceType1() {
    concatGenericsLists();
    pushOnGenericsStack(getTypeReference(0));
    this.intPtr -= 1;
  }
  protected void consumeTypeArgumentReferenceType2() {
    concatGenericsLists();
    pushOnGenericsStack(getTypeReference(0));
    this.intPtr -= 1;
  }
  protected void consumeTypeArguments() {
    concatGenericsLists();
    this.intPtr -= 1;

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      int length = this.genericsLengthStack[this.genericsLengthPtr];
      problemReporter().invalidUsageOfTypeArguments(
        (TypeReference)this.genericsStack[(this.genericsPtr - length + 1)], 
        (TypeReference)this.genericsStack[this.genericsPtr]);
    }
  }

  protected void consumeTypeDeclarations() {
    concatNodeLists();
  }

  protected void consumeTypeImportOnDemandDeclarationName()
  {
    int length;
    char[][] tokens = new char[length = this.identifierLengthStack[(this.identifierLengthPtr--)]][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
    ImportReference impt;
    pushOnAstStack(impt = new ImportReference(tokens, positions, true, 0));

    if (this.currentToken == 27)
      impt.declarationSourceEnd = (this.scanner.currentPosition - 1);
    else {
      impt.declarationSourceEnd = impt.sourceEnd;
    }
    impt.declarationEnd = impt.declarationSourceEnd;

    impt.declarationSourceStart = this.intStack[(this.intPtr--)];

    if (this.currentElement != null) {
      this.lastCheckPoint = (impt.declarationSourceEnd + 1);
      this.currentElement = this.currentElement.add(impt, 0);
      this.lastIgnoredToken = -1;
      this.restartRecovery = true;
    }
  }

  protected void consumeTypeParameterHeader() {
    TypeParameter typeParameter = new TypeParameter();
    long pos = this.identifierPositionStack[this.identifierPtr];
    int end = (int)pos;
    typeParameter.declarationSourceEnd = end;
    typeParameter.sourceEnd = end;
    int start = (int)(pos >>> 32);
    typeParameter.declarationSourceStart = start;
    typeParameter.sourceStart = start;
    typeParameter.name = this.identifierStack[(this.identifierPtr--)];
    this.identifierLengthPtr -= 1;
    pushOnGenericsStack(typeParameter);

    this.listTypeParameterLength += 1;
  }

  protected void consumeTypeParameter1() {
  }

  protected void consumeTypeParameter1WithExtends() {
    TypeReference superType = (TypeReference)this.genericsStack[(this.genericsPtr--)];
    this.genericsLengthPtr -= 1;
    TypeParameter typeParameter = (TypeParameter)this.genericsStack[this.genericsPtr];
    typeParameter.declarationSourceEnd = superType.sourceEnd;
    typeParameter.type = superType;
    superType.bits |= 16;
    this.genericsStack[this.genericsPtr] = typeParameter;
  }

  protected void consumeTypeParameter1WithExtendsAndBounds() {
    int additionalBoundsLength = this.genericsLengthStack[(this.genericsLengthPtr--)];
    TypeReference[] bounds = new TypeReference[additionalBoundsLength];
    this.genericsPtr -= additionalBoundsLength;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, bounds, 0, additionalBoundsLength);
    TypeReference superType = getTypeReference(this.intStack[(this.intPtr--)]);
    TypeParameter typeParameter = (TypeParameter)this.genericsStack[this.genericsPtr];
    typeParameter.declarationSourceEnd = bounds[(additionalBoundsLength - 1)].sourceEnd;
    typeParameter.type = superType;
    superType.bits |= 16;
    typeParameter.bounds = bounds;
    int i = 0; for (int max = bounds.length; i < max; i++)
      bounds[i].bits |= 16;
  }

  protected void consumeTypeParameterList()
  {
    concatGenericsLists();
  }

  protected void consumeTypeParameterList1() {
    concatGenericsLists();
  }
  protected void consumeTypeParameters() {
    int startPos = this.intStack[(this.intPtr--)];

    if ((this.currentElement != null) && 
      ((this.currentElement instanceof RecoveredType))) {
      RecoveredType recoveredType = (RecoveredType)this.currentElement;
      int length = this.genericsLengthStack[this.genericsLengthPtr];
      TypeParameter[] typeParameters = new TypeParameter[length];
      System.arraycopy(this.genericsStack, this.genericsPtr - length + 1, typeParameters, 0, length);

      recoveredType.add(typeParameters, startPos);
    }

    if ((!this.statementRecoveryActivated) && 
      (this.options.sourceLevel < 3211264L) && 
      (this.lastErrorEndPositionBeforeRecovery < this.scanner.currentPosition)) {
      int length = this.genericsLengthStack[this.genericsLengthPtr];
      problemReporter().invalidUsageOfTypeParameters(
        (TypeParameter)this.genericsStack[(this.genericsPtr - length + 1)], 
        (TypeParameter)this.genericsStack[this.genericsPtr]);
    }
  }

  protected void consumeTypeParameterWithExtends() {
    TypeReference superType = getTypeReference(this.intStack[(this.intPtr--)]);
    TypeParameter typeParameter = (TypeParameter)this.genericsStack[this.genericsPtr];
    typeParameter.declarationSourceEnd = superType.sourceEnd;
    typeParameter.type = superType;
    superType.bits |= 16;
  }

  protected void consumeTypeParameterWithExtendsAndBounds() {
    int additionalBoundsLength = this.genericsLengthStack[(this.genericsLengthPtr--)];
    TypeReference[] bounds = new TypeReference[additionalBoundsLength];
    this.genericsPtr -= additionalBoundsLength;
    System.arraycopy(this.genericsStack, this.genericsPtr + 1, bounds, 0, additionalBoundsLength);
    TypeReference superType = getTypeReference(this.intStack[(this.intPtr--)]);
    TypeParameter typeParameter = (TypeParameter)this.genericsStack[this.genericsPtr];
    typeParameter.type = superType;
    superType.bits |= 16;
    typeParameter.bounds = bounds;
    typeParameter.declarationSourceEnd = bounds[(additionalBoundsLength - 1)].sourceEnd;
    int i = 0; for (int max = bounds.length; i < max; i++)
      bounds[i].bits |= 16;
  }

  protected void consumeUnaryExpression(int op)
  {
    Expression exp = this.expressionStack[this.expressionPtr];
    Expression r;
    Expression r;
    if (op == 13)
    {
      Expression r;
      if (((exp instanceof IntLiteral)) && (((IntLiteral)exp).mayRepresentMIN_VALUE())) {
        r = this.expressionStack[this.expressionPtr] =  = new IntLiteralMinValue();
      }
      else
      {
        Expression r;
        if (((exp instanceof LongLiteral)) && (((LongLiteral)exp).mayRepresentMIN_VALUE()))
          r = this.expressionStack[this.expressionPtr] =  = new LongLiteralMinValue();
        else
          r = this.expressionStack[this.expressionPtr] =  = new UnaryExpression(exp, op);
      }
    }
    else {
      r = this.expressionStack[this.expressionPtr] =  = new UnaryExpression(exp, op);
    }
    r.sourceStart = this.intStack[(this.intPtr--)];
    r.sourceEnd = exp.sourceEnd;
  }

  protected void consumeUnaryExpression(int op, boolean post)
  {
    Expression leftHandSide = this.expressionStack[this.expressionPtr];
    if ((leftHandSide instanceof Reference))
    {
      if (post)
        this.expressionStack[this.expressionPtr] = 
          new PostfixExpression(
          leftHandSide, 
          IntLiteral.One, 
          op, 
          this.endStatementPosition);
      else
        this.expressionStack[this.expressionPtr] = 
          new PrefixExpression(
          leftHandSide, 
          IntLiteral.One, 
          op, 
          this.intStack[(this.intPtr--)]);
    }
    else
    {
      if (!post) {
        this.intPtr -= 1;
      }
      if (!this.statementRecoveryActivated) problemReporter().invalidUnaryExpression(leftHandSide); 
    }
  }

  protected void consumeVariableDeclarators()
  {
    optimizedConcatNodeLists();
  }

  protected void consumeVariableInitializers() {
    concatExpressionLists();
  }
  protected void consumeWildcard() {
    Wildcard wildcard = new Wildcard(0);
    wildcard.sourceEnd = this.intStack[(this.intPtr--)];
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }
  protected void consumeWildcard1() {
    Wildcard wildcard = new Wildcard(0);
    wildcard.sourceEnd = this.intStack[(this.intPtr--)];
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }

  protected void consumeWildcard1WithBounds() {
  }

  protected void consumeWildcard2() {
    Wildcard wildcard = new Wildcard(0);
    wildcard.sourceEnd = this.intStack[(this.intPtr--)];
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }

  protected void consumeWildcard2WithBounds() {
  }

  protected void consumeWildcard3() {
    Wildcard wildcard = new Wildcard(0);
    wildcard.sourceEnd = this.intStack[(this.intPtr--)];
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }

  protected void consumeWildcard3WithBounds() {
  }

  protected void consumeWildcardBounds1Extends() {
    Wildcard wildcard = new Wildcard(1);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBounds1Super() {
    Wildcard wildcard = new Wildcard(2);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    this.intPtr -= 1;
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBounds2Extends() {
    Wildcard wildcard = new Wildcard(1);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBounds2Super() {
    Wildcard wildcard = new Wildcard(2);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    this.intPtr -= 1;
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBounds3Extends() {
    Wildcard wildcard = new Wildcard(1);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBounds3Super() {
    Wildcard wildcard = new Wildcard(2);
    wildcard.bound = ((TypeReference)this.genericsStack[this.genericsPtr]);
    this.intPtr -= 1;
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    this.genericsStack[this.genericsPtr] = wildcard;
  }
  protected void consumeWildcardBoundsExtends() {
    Wildcard wildcard = new Wildcard(1);
    wildcard.bound = getTypeReference(this.intStack[(this.intPtr--)]);
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }
  protected void consumeWildcardBoundsSuper() {
    Wildcard wildcard = new Wildcard(2);
    wildcard.bound = getTypeReference(this.intStack[(this.intPtr--)]);
    this.intPtr -= 1;
    wildcard.sourceEnd = wildcard.bound.sourceEnd;
    this.intPtr -= 1;
    wildcard.sourceStart = this.intStack[(this.intPtr--)];
    pushOnGenericsStack(wildcard);
  }

  protected void consumeWildcardWithBounds()
  {
  }

  public boolean containsComment(int sourceStart, int sourceEnd)
  {
    for (int iComment = this.scanner.commentPtr; 
      iComment >= 0; iComment--) {
      int commentStart = this.scanner.commentStarts[iComment];

      if (commentStart >= sourceStart)
      {
        if (commentStart <= sourceEnd)
          return true; 
      }
    }
    return false;
  }
  public MethodDeclaration convertToMethodDeclaration(ConstructorDeclaration c, CompilationResult compilationResult) {
    MethodDeclaration m = new MethodDeclaration(compilationResult);
    m.typeParameters = c.typeParameters;
    m.sourceStart = c.sourceStart;
    m.sourceEnd = c.sourceEnd;
    m.bodyStart = c.bodyStart;
    m.bodyEnd = c.bodyEnd;
    m.declarationSourceEnd = c.declarationSourceEnd;
    m.declarationSourceStart = c.declarationSourceStart;
    m.selector = c.selector;
    m.statements = c.statements;
    m.modifiers = c.modifiers;
    m.annotations = c.annotations;
    m.arguments = c.arguments;
    m.thrownExceptions = c.thrownExceptions;
    m.explicitDeclarations = c.explicitDeclarations;
    m.returnType = null;
    m.javadoc = c.javadoc;
    return m;
  }
  protected TypeReference copyDims(TypeReference typeRef, int dim) {
    return typeRef.copyDims(dim);
  }
  protected FieldDeclaration createFieldDeclaration(char[] fieldDeclarationName, int sourceStart, int sourceEnd) {
    return new FieldDeclaration(fieldDeclarationName, sourceStart, sourceEnd);
  }
  protected JavadocParser createJavadocParser() {
    return new JavadocParser(this);
  }
  protected LocalDeclaration createLocalDeclaration(char[] localDeclarationName, int sourceStart, int sourceEnd) {
    return new LocalDeclaration(localDeclarationName, sourceStart, sourceEnd);
  }
  protected StringLiteral createStringLiteral(char[] token, int start, int end, int lineNumber) {
    return new StringLiteral(token, start, end, lineNumber);
  }
  protected RecoveredType currentRecoveryType() {
    if (this.currentElement != null) {
      if ((this.currentElement instanceof RecoveredType)) {
        return (RecoveredType)this.currentElement;
      }
      return this.currentElement.enclosingType();
    }

    return null;
  }
  public CompilationUnitDeclaration dietParse(ICompilationUnit sourceUnit, CompilationResult compilationResult) {
    boolean old = this.diet;
    CompilationUnitDeclaration parsedUnit;
    try {
      this.diet = true;
      parsedUnit = parse(sourceUnit, compilationResult);
    }
    finally
    {
      CompilationUnitDeclaration parsedUnit;
      this.diet = old;
    }
    return parsedUnit;
  }

  protected void dispatchDeclarationInto(int length)
  {
    if (length == 0)
      return;
    int[] flag = new int[length + 1];
    int size1 = 0; int size2 = 0; int size3 = 0;
    boolean hasAbstractMethods = false;
    for (int i = length - 1; i >= 0; i--) {
      ASTNode astNode = this.astStack[(this.astPtr--)];
      if ((astNode instanceof AbstractMethodDeclaration))
      {
        flag[i] = 2;
        size2++;
        if (((AbstractMethodDeclaration)astNode).isAbstract())
          hasAbstractMethods = true;
      }
      else if ((astNode instanceof TypeDeclaration)) {
        flag[i] = 3;
        size3++;
      }
      else {
        flag[i] = 1;
        size1++;
      }

    }

    TypeDeclaration typeDecl = (TypeDeclaration)this.astStack[this.astPtr];
    if (size1 != 0) {
      typeDecl.fields = new FieldDeclaration[size1];
    }
    if (size2 != 0) {
      typeDecl.methods = new AbstractMethodDeclaration[size2];
      if (hasAbstractMethods) typeDecl.bits |= 2048;
    }
    if (size3 != 0) {
      typeDecl.memberTypes = new TypeDeclaration[size3];
    }

    size1 = size2 = size3 = 0;
    int flagI = flag[0]; int start = 0;

    for (int end = 0; end <= length; end++)
    {
      if (flagI != flag[end])
      {
        switch (flagI)
        {
        case 1:
          int length2;
          size1 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            typeDecl.fields, 
            size1 - length2, 
            length2);
          break;
        case 2:
          int length2;
          size2 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            typeDecl.methods, 
            size2 - length2, 
            length2);
          break;
        case 3:
          int length2;
          size3 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            typeDecl.memberTypes, 
            size3 - length2, 
            length2);
        }

        flagI = flag[(start = end)];
      }
    }

    if (typeDecl.memberTypes != null)
      for (int i = typeDecl.memberTypes.length - 1; i >= 0; i--)
        typeDecl.memberTypes[i].enclosingType = typeDecl;
  }

  protected void dispatchDeclarationIntoEnumDeclaration(int length)
  {
    if (length == 0)
      return;
    int[] flag = new int[length + 1];
    int size1 = 0; int size2 = 0; int size3 = 0;
    TypeDeclaration enumDeclaration = (TypeDeclaration)this.astStack[(this.astPtr - length)];
    boolean hasAbstractMethods = false;
    for (int i = length - 1; i >= 0; i--) {
      ASTNode astNode = this.astStack[(this.astPtr--)];
      if ((astNode instanceof AbstractMethodDeclaration))
      {
        flag[i] = 2;
        size2++;
        if (((AbstractMethodDeclaration)astNode).isAbstract())
          hasAbstractMethods = true;
      }
      else if ((astNode instanceof TypeDeclaration)) {
        flag[i] = 3;
        size3++;
      } else if ((astNode instanceof FieldDeclaration)) {
        flag[i] = 1;
        size1++;
      }

    }

    if (size1 != 0) {
      enumDeclaration.fields = new FieldDeclaration[size1];
    }
    if (size2 != 0) {
      enumDeclaration.methods = new AbstractMethodDeclaration[size2];
      if (hasAbstractMethods) enumDeclaration.bits |= 2048;
    }
    if (size3 != 0) {
      enumDeclaration.memberTypes = new TypeDeclaration[size3];
    }

    size1 = size2 = size3 = 0;
    int flagI = flag[0]; int start = 0;

    for (int end = 0; end <= length; end++)
    {
      if (flagI != flag[end])
      {
        switch (flagI)
        {
        case 1:
          int length2;
          size1 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            enumDeclaration.fields, 
            size1 - length2, 
            length2);
          break;
        case 2:
          int length2;
          size2 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            enumDeclaration.methods, 
            size2 - length2, 
            length2);
          break;
        case 3:
          int length2;
          size3 += (length2 = end - start);
          System.arraycopy(
            this.astStack, 
            this.astPtr + start + 1, 
            enumDeclaration.memberTypes, 
            size3 - length2, 
            length2);
        }

        flagI = flag[(start = end)];
      }
    }

    if (enumDeclaration.memberTypes != null)
      for (int i = enumDeclaration.memberTypes.length - 1; i >= 0; i--)
        enumDeclaration.memberTypes[i].enclosingType = enumDeclaration;
  }

  protected CompilationUnitDeclaration endParse(int act)
  {
    this.lastAct = act;

    if (this.statementRecoveryActivated) {
      RecoveredElement recoveredElement = buildInitialRecoveryState();
      recoveredElement.topElement().updateParseTree();
      if (this.hasError) resetStacks(); 
    }
    else if (this.currentElement != null) {
      if (VERBOSE_RECOVERY) {
        System.out.print(Messages.parser_syntaxRecovery);
        System.out.println("--------------------------");
        System.out.println(this.compilationUnit);
        System.out.println("----------------------------------");
      }
      this.currentElement.topElement().updateParseTree();
    }
    else if ((this.diet & VERBOSE_RECOVERY)) {
      System.out.print(Messages.parser_regularParse);
      System.out.println("--------------------------");
      System.out.println(this.compilationUnit);
      System.out.println("----------------------------------");
    }

    persistLineSeparatorPositions();
    for (int i = 0; i < this.scanner.foundTaskCount; i++) {
      if (!this.statementRecoveryActivated) problemReporter()
          .task(new String(this.scanner.foundTaskTags[i]), 
          new String(this.scanner.foundTaskMessages[i]), 
          this.scanner.foundTaskPriorities[i] == null ? null : new String(this.scanner.foundTaskPriorities[i]), 
          this.scanner.foundTaskPositions[i][0], 
          this.scanner.foundTaskPositions[i][1]);
    }
    return this.compilationUnit;
  }

  public int flushCommentsDefinedPriorTo(int position)
  {
    int lastCommentIndex = this.scanner.commentPtr;
    if (lastCommentIndex < 0) return position;

    int index = lastCommentIndex;
    int validCount = 0;
    while (index >= 0) {
      int commentEnd = this.scanner.commentStops[index];
      if (commentEnd < 0) commentEnd = -commentEnd;
      if (commentEnd <= position) {
        break;
      }
      index--;
      validCount++;
    }

    if (validCount > 0) {
      int immediateCommentEnd = -this.scanner.commentStops[(index + 1)];
      if (immediateCommentEnd > 0)
      {
        immediateCommentEnd--;
        if (Util.getLineNumber(position, this.scanner.lineEnds, 0, this.scanner.linePtr) == 
          Util.getLineNumber(immediateCommentEnd, this.scanner.lineEnds, 0, this.scanner.linePtr)) {
          position = immediateCommentEnd;
          validCount--;
          index++;
        }
      }
    }

    if (index < 0) return position;

    switch (validCount)
    {
    case 0:
      break;
    case 2:
      this.scanner.commentStarts[0] = this.scanner.commentStarts[(index + 1)];
      this.scanner.commentStops[0] = this.scanner.commentStops[(index + 1)];
      this.scanner.commentTagStarts[0] = this.scanner.commentTagStarts[(index + 1)];
      this.scanner.commentStarts[1] = this.scanner.commentStarts[(index + 2)];
      this.scanner.commentStops[1] = this.scanner.commentStops[(index + 2)];
      this.scanner.commentTagStarts[1] = this.scanner.commentTagStarts[(index + 2)];
      break;
    case 1:
      this.scanner.commentStarts[0] = this.scanner.commentStarts[(index + 1)];
      this.scanner.commentStops[0] = this.scanner.commentStops[(index + 1)];
      this.scanner.commentTagStarts[0] = this.scanner.commentTagStarts[(index + 1)];
      break;
    default:
      System.arraycopy(this.scanner.commentStarts, index + 1, this.scanner.commentStarts, 0, validCount);
      System.arraycopy(this.scanner.commentStops, index + 1, this.scanner.commentStops, 0, validCount);
      System.arraycopy(this.scanner.commentTagStarts, index + 1, this.scanner.commentTagStarts, 0, validCount);
    }
    this.scanner.commentPtr = (validCount - 1);
    return position;
  }
  protected TypeReference getAnnotationType() {
    int length = this.identifierLengthStack[(this.identifierLengthPtr--)];
    if (length == 1) {
      return new SingleTypeReference(
        this.identifierStack[this.identifierPtr], 
        this.identifierPositionStack[(this.identifierPtr--)]);
    }
    char[][] tokens = new char[length][];
    this.identifierPtr -= length;
    long[] positions = new long[length];
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    System.arraycopy(
      this.identifierPositionStack, 
      this.identifierPtr + 1, 
      positions, 
      0, 
      length);
    return new QualifiedTypeReference(tokens, positions);
  }

  public int getFirstToken()
  {
    return this.firstToken;
  }

  public int[] getJavaDocPositions()
  {
    int javadocCount = 0;
    int i = 0; for (int max = this.scanner.commentPtr; i <= max; i++)
    {
      if (this.scanner.commentStops[i] > 0) {
        javadocCount++;
      }
    }
    if (javadocCount == 0) return null;

    int[] positions = new int[2 * javadocCount];
    int index = 0;
    int i = 0; for (int max = this.scanner.commentPtr; i <= max; i++)
    {
      if (this.scanner.commentStops[i] > 0) {
        positions[(index++)] = this.scanner.commentStarts[i];
        positions[(index++)] = (this.scanner.commentStops[i] - 1);
      }
    }
    return positions;
  }

  public void getMethodBodies(CompilationUnitDeclaration unit)
  {
    if (unit == null) return;

    if (unit.ignoreMethodBodies) {
      unit.ignoreFurtherInvestigation = true;
      return;
    }

    if ((unit.bits & 0x10) != 0) {
      return;
    }

    int[] oldLineEnds = this.scanner.lineEnds;
    int oldLinePtr = this.scanner.linePtr;

    CompilationResult compilationResult = unit.compilationResult;
    char[] contents = compilationResult.compilationUnit.getContents();
    this.scanner.setSource(contents, compilationResult);

    if ((this.javadocParser != null) && (this.javadocParser.checkDocComment)) {
      this.javadocParser.scanner.setSource(contents);
    }
    if (unit.types != null) {
      int i = unit.types.length;
      do { unit.types[i].parseMethod(this, unit);

        i--; } while (i >= 0);
    }

    unit.bits |= 16;

    this.scanner.lineEnds = oldLineEnds;
    this.scanner.linePtr = oldLinePtr;
  }

  protected char getNextCharacter(char[] comment, int[] index)
  {
    int tmp3_2 = 0;
    int[] tmp3_1 = index;
    int tmp5_4 = tmp3_1[tmp3_2]; tmp3_1[tmp3_2] = (tmp5_4 + 1); char nextCharacter = comment[tmp5_4];
    switch (nextCharacter)
    {
    case '\\':
      index[0] += 1;
      while (comment[index[0]] == 'u') index[0] += 1;
      int tmp62_61 = 0;
      int[] tmp62_60 = index;
      int tmp64_63 = tmp62_60[tmp62_61]; tmp62_60[tmp62_61] = (tmp64_63 + 1);
      int c1;
      if (((c1 = ScannerHelper.getNumericValue(comment[tmp64_63])) <= 15) && 
        (c1 >= 0))
      {
        int tmp88_87 = 0;
        int[] tmp88_86 = index;
        int tmp90_89 = tmp88_86[tmp88_87]; tmp88_86[tmp88_87] = (tmp90_89 + 1);
        int c2;
        if (((c2 = ScannerHelper.getNumericValue(comment[tmp90_89])) <= 15) && (c2 >= 0))
        {
          int tmp114_113 = 0;
          int[] tmp114_112 = index;
          int tmp116_115 = tmp114_112[tmp114_113]; tmp114_112[tmp114_113] = (tmp116_115 + 1);
          int c3;
          if (((c3 = ScannerHelper.getNumericValue(comment[tmp116_115])) <= 15) && (c3 >= 0))
          {
            int tmp140_139 = 0;
            int[] tmp140_138 = index;
            int tmp142_141 = tmp140_138[tmp140_139]; tmp140_138[tmp140_139] = (tmp142_141 + 1);
            int c4;
            if (((c4 = ScannerHelper.getNumericValue(comment[tmp142_141])) <= 15) && (c4 >= 0))
              nextCharacter = (char)(((c1 * 16 + c2) * 16 + c3) * 16 + c4); 
          }
        }
      }
      break;
    }
    return nextCharacter;
  }

  protected Expression getTypeReference(Expression exp) {
    exp.bits &= -8;
    exp.bits |= 4;
    return exp;
  }

  protected TypeReference getTypeReference(int dim)
  {
    int length = this.identifierLengthStack[(this.identifierLengthPtr--)];
    TypeReference ref;
    if (length < 0) {
      TypeReference ref = TypeReference.baseTypeReference(-length, dim);
      ref.sourceStart = this.intStack[(this.intPtr--)];
      if (dim == 0) {
        ref.sourceEnd = this.intStack[(this.intPtr--)];
      } else {
        this.intPtr -= 1;
        ref.sourceEnd = this.endPosition;
      }
    } else {
      int numberOfIdentifiers = this.genericsIdentifiersLengthStack[(this.genericsIdentifiersLengthPtr--)];
      TypeReference ref;
      if ((length != numberOfIdentifiers) || (this.genericsLengthStack[this.genericsLengthPtr] != 0))
      {
        ref = getTypeReferenceForGenericType(dim, length, numberOfIdentifiers);
      } else if (length == 1)
      {
        this.genericsLengthPtr -= 1;
        TypeReference ref;
        if (dim == 0) {
          ref = 
            new SingleTypeReference(
            this.identifierStack[this.identifierPtr], 
            this.identifierPositionStack[(this.identifierPtr--)]);
        } else {
          TypeReference ref = 
            new ArrayTypeReference(
            this.identifierStack[this.identifierPtr], 
            dim, 
            this.identifierPositionStack[(this.identifierPtr--)]);
          ref.sourceEnd = this.endPosition;
        }
      } else {
        this.genericsLengthPtr -= 1;

        char[][] tokens = new char[length][];
        this.identifierPtr -= length;
        long[] positions = new long[length];
        System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
        System.arraycopy(
          this.identifierPositionStack, 
          this.identifierPtr + 1, 
          positions, 
          0, 
          length);
        TypeReference ref;
        if (dim == 0) {
          ref = new QualifiedTypeReference(tokens, positions);
        } else {
          ref = new ArrayQualifiedTypeReference(tokens, dim, positions);
          ref.sourceEnd = this.endPosition;
        }
      }
    }
    return ref;
  }
  protected TypeReference getTypeReferenceForGenericType(int dim, int identifierLength, int numberOfIdentifiers) {
    if ((identifierLength == 1) && (numberOfIdentifiers == 1)) {
      int currentTypeArgumentsLength = this.genericsLengthStack[(this.genericsLengthPtr--)];
      TypeReference[] typeArguments = new TypeReference[currentTypeArgumentsLength];
      this.genericsPtr -= currentTypeArgumentsLength;
      System.arraycopy(this.genericsStack, this.genericsPtr + 1, typeArguments, 0, currentTypeArgumentsLength);
      ParameterizedSingleTypeReference parameterizedSingleTypeReference = new ParameterizedSingleTypeReference(this.identifierStack[this.identifierPtr], typeArguments, dim, this.identifierPositionStack[(this.identifierPtr--)]);
      if (dim != 0) {
        parameterizedSingleTypeReference.sourceEnd = this.endStatementPosition;
      }
      return parameterizedSingleTypeReference;
    }
    TypeReference[][] typeArguments = new TypeReference[numberOfIdentifiers][];
    char[][] tokens = new char[numberOfIdentifiers][];
    long[] positions = new long[numberOfIdentifiers];
    int index = numberOfIdentifiers;
    int currentIdentifiersLength = identifierLength;
    while (index > 0) {
      int currentTypeArgumentsLength = this.genericsLengthStack[(this.genericsLengthPtr--)];
      if (currentTypeArgumentsLength != 0) {
        this.genericsPtr -= currentTypeArgumentsLength;
        System.arraycopy(this.genericsStack, this.genericsPtr + 1, typeArguments[(index - 1)] =  = new TypeReference[currentTypeArgumentsLength], 0, currentTypeArgumentsLength);
      }
      switch (currentIdentifiersLength)
      {
      case 1:
        tokens[(index - 1)] = this.identifierStack[this.identifierPtr];
        positions[(index - 1)] = this.identifierPositionStack[(this.identifierPtr--)];
        break;
      default:
        this.identifierPtr -= currentIdentifiersLength;
        System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, index - currentIdentifiersLength, currentIdentifiersLength);
        System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, index - currentIdentifiersLength, currentIdentifiersLength);
      }
      index -= currentIdentifiersLength;
      if (index > 0) {
        currentIdentifiersLength = this.identifierLengthStack[(this.identifierLengthPtr--)];
      }
    }
    ParameterizedQualifiedTypeReference parameterizedQualifiedTypeReference = new ParameterizedQualifiedTypeReference(tokens, typeArguments, dim, positions);
    if (dim != 0) {
      parameterizedQualifiedTypeReference.sourceEnd = this.endStatementPosition;
    }
    return parameterizedQualifiedTypeReference;
  }

  protected NameReference getUnspecifiedReference()
  {
    int length;
    NameReference ref;
    NameReference ref;
    if ((length = this.identifierLengthStack[(this.identifierLengthPtr--)]) == 1)
    {
      ref = 
        new SingleNameReference(
        this.identifierStack[this.identifierPtr], 
        this.identifierPositionStack[(this.identifierPtr--)]);
    }
    else
    {
      char[][] tokens = new char[length][];
      this.identifierPtr -= length;
      System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
      long[] positions = new long[length];
      System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
      ref = 
        new QualifiedNameReference(tokens, 
        positions, 
        (int)(this.identifierPositionStack[(this.identifierPtr + 1)] >> 32), 
        (int)this.identifierPositionStack[(this.identifierPtr + length)]);
    }
    return ref;
  }

  protected NameReference getUnspecifiedReferenceOptimized()
  {
    int length;
    if ((length = this.identifierLengthStack[(this.identifierLengthPtr--)]) == 1)
    {
      NameReference ref = 
        new SingleNameReference(
        this.identifierStack[this.identifierPtr], 
        this.identifierPositionStack[(this.identifierPtr--)]);
      ref.bits &= -8;
      ref.bits |= 3;
      return ref;
    }

    char[][] tokens = new char[length][];
    this.identifierPtr -= length;
    System.arraycopy(this.identifierStack, this.identifierPtr + 1, tokens, 0, length);
    long[] positions = new long[length];
    System.arraycopy(this.identifierPositionStack, this.identifierPtr + 1, positions, 0, length);
    NameReference ref = new QualifiedNameReference(
      tokens, 
      positions, 
      (int)(this.identifierPositionStack[(this.identifierPtr + 1)] >> 32), 
      (int)this.identifierPositionStack[(this.identifierPtr + length)]);
    ref.bits &= -8;
    ref.bits |= 3;
    return ref;
  }

  public void goForBlockStatementsopt()
  {
    this.firstToken = 67;
    this.scanner.recordLineSeparator = false;
  }

  public void goForBlockStatementsOrCatchHeader()
  {
    this.firstToken = 4;
    this.scanner.recordLineSeparator = false;
  }

  public void goForClassBodyDeclarations()
  {
    this.firstToken = 20;
    this.scanner.recordLineSeparator = true;
  }

  public void goForCompilationUnit()
  {
    this.firstToken = 9;
    this.scanner.foundTaskCount = 0;
    this.scanner.recordLineSeparator = true;
  }

  public void goForExpression()
  {
    this.firstToken = 5;
    this.scanner.recordLineSeparator = true;
  }

  public void goForFieldDeclaration()
  {
    this.firstToken = 24;
    this.scanner.recordLineSeparator = true;
  }

  public void goForGenericMethodDeclaration()
  {
    this.firstToken = 6;
    this.scanner.recordLineSeparator = true;
  }

  public void goForHeaders() {
    RecoveredType currentType = currentRecoveryType();
    if ((currentType != null) && (currentType.insideEnumConstantPart))
      this.firstToken = 66;
    else {
      this.firstToken = 11;
    }
    this.scanner.recordLineSeparator = true;
  }

  public void goForImportDeclaration()
  {
    this.firstToken = 25;
    this.scanner.recordLineSeparator = true;
  }

  public void goForInitializer()
  {
    this.firstToken = 8;
    this.scanner.recordLineSeparator = false;
  }

  public void goForMemberValue()
  {
    this.firstToken = 25;
    this.scanner.recordLineSeparator = true;
  }

  public void goForMethodBody()
  {
    this.firstToken = 10;
    this.scanner.recordLineSeparator = false;
  }

  public void goForPackageDeclaration()
  {
    this.firstToken = 23;
    this.scanner.recordLineSeparator = true;
  }

  public void goForTypeDeclaration()
  {
    this.firstToken = 1;
    this.scanner.recordLineSeparator = true;
  }

  protected void ignoreExpressionAssignment()
  {
    this.intPtr -= 1;
    ArrayInitializer arrayInitializer = (ArrayInitializer)this.expressionStack[(this.expressionPtr--)];
    this.expressionLengthPtr -= 1;

    if (!this.statementRecoveryActivated) problemReporter().arrayConstantsOnlyInArrayInitializers(arrayInitializer.sourceStart, arrayInitializer.sourceEnd); 
  }

  public void initialize() { initialize(false); }


  public void initialize(boolean initializeNLS)
  {
    this.astPtr = -1;
    this.astLengthPtr = -1;
    this.expressionPtr = -1;
    this.expressionLengthPtr = -1;
    this.identifierPtr = -1;
    this.identifierLengthPtr = -1;
    this.intPtr = -1;
    int tmp41_40 = 0; this.nestedType = tmp41_40; this.nestedMethod[tmp41_40] = 0;
    this.variablesCounter[this.nestedType] = 0;
    this.dimensions = 0;
    this.realBlockPtr = -1;
    this.compilationUnit = null;
    this.referenceContext = null;
    this.endStatementPosition = 0;

    int astLength = this.astStack.length;
    if (this.noAstNodes.length < astLength) {
      this.noAstNodes = new ASTNode[astLength];
    }

    System.arraycopy(this.noAstNodes, 0, this.astStack, 0, astLength);

    int expressionLength = this.expressionStack.length;
    if (this.noExpressions.length < expressionLength) {
      this.noExpressions = new Expression[expressionLength];
    }

    System.arraycopy(this.noExpressions, 0, this.expressionStack, 0, expressionLength);

    this.scanner.commentPtr = -1;
    this.scanner.foundTaskCount = 0;
    this.scanner.eofPosition = 2147483647;
    this.recordStringLiterals = true;
    boolean checkNLS = this.options.getSeverity(256L) != -1;
    this.checkExternalizeStrings = checkNLS;
    this.scanner.checkNonExternalizedStringLiterals = ((initializeNLS) && (checkNLS));

    resetModifiers();

    this.lastCheckPoint = -1;
    this.currentElement = null;
    this.restartRecovery = false;
    this.hasReportedError = false;
    this.recoveredStaticInitializerStart = 0;
    this.lastIgnoredToken = -1;
    this.lastErrorEndPosition = -1;
    this.lastErrorEndPositionBeforeRecovery = -1;
    this.lastJavadocEnd = -1;
    this.listLength = 0;
    this.listTypeParameterLength = 0;

    this.rBraceStart = 0;
    this.rBraceEnd = 0;
    this.rBraceSuccessorStart = 0;

    this.genericsIdentifiersLengthPtr = -1;
    this.genericsLengthPtr = -1;
    this.genericsPtr = -1;
  }
  public void initializeScanner() {
    this.scanner = new Scanner(
      false, 
      false, 
      false, 
      this.options.sourceLevel, 
      this.options.complianceLevel, 
      this.options.taskTags, 
      this.options.taskPriorites, 
      this.options.isTaskCaseSensitive);
  }

  public void jumpOverMethodBody()
  {
    if ((this.diet) && (this.dietInt == 0))
      this.scanner.diet = true; 
  }

  private void jumpOverType() { if ((this.recoveredTypes != null) && (this.nextTypeStart > -1) && (this.nextTypeStart < this.scanner.currentPosition))
    {
      TypeDeclaration typeDeclaration = this.recoveredTypes[this.recoveredTypePtr];
      boolean isAnonymous = typeDeclaration.allocation != null;

      int end = this.scanner.eofPosition;
      this.scanner.resetTo(typeDeclaration.declarationSourceEnd + 1, end - 1);
      if (!isAnonymous)
        ((RecoveryScanner)this.scanner).setPendingTokens(new int[] { 27, 75 });
      else {
        ((RecoveryScanner)this.scanner).setPendingTokens(new int[] { 26, 71, 26 });
      }

      this.pendingRecoveredType = typeDeclaration;
      try
      {
        this.currentToken = this.scanner.getNextToken();
      }
      catch (InvalidInputException localInvalidInputException)
      {
      }
      if (++this.recoveredTypePtr < this.recoveredTypes.length) {
        TypeDeclaration nextTypeDeclaration = this.recoveredTypes[this.recoveredTypePtr];
        this.nextTypeStart = 
          (nextTypeDeclaration.allocation == null ? 
          nextTypeDeclaration.declarationSourceStart : 
          nextTypeDeclaration.allocation.sourceStart);
      } else {
        this.nextTypeStart = 2147483647;
      }
    } }

  protected void markEnclosingMemberWithLocalType() {
    if (this.currentElement != null) return;
    for (int i = this.astPtr; i >= 0; i--) {
      ASTNode node = this.astStack[i];
      if ((!(node instanceof AbstractMethodDeclaration)) && 
        (!(node instanceof FieldDeclaration))) {
        if ((node instanceof TypeDeclaration)) {
          if (((TypeDeclaration)node).declarationSourceEnd != 0);
        }
      } else { node.bits |= 2;
        return;
      }
    }

    if (((this.referenceContext instanceof AbstractMethodDeclaration)) || 
      ((this.referenceContext instanceof TypeDeclaration)))
      ((ASTNode)this.referenceContext).bits |= 2;
  }

  protected void markInitializersWithLocalType(TypeDeclaration type) {
    if ((type.fields == null) || ((type.bits & 0x2) == 0)) return;
    int i = 0; for (int length = type.fields.length; i < length; i++) {
      FieldDeclaration field = type.fields[i];
      if ((field instanceof Initializer))
        field.bits |= 2;
    }
  }

  protected boolean moveRecoveryCheckpoint()
  {
    int pos = this.lastCheckPoint;

    this.scanner.startPosition = pos;
    this.scanner.currentPosition = pos;
    this.scanner.diet = false;

    if (this.restartRecovery) {
      this.lastIgnoredToken = -1;
      this.scanner.insideRecovery = true;
      return true;
    }

    this.lastIgnoredToken = this.nextIgnoredToken;
    this.nextIgnoredToken = -1;
    do
      try {
        this.nextIgnoredToken = this.scanner.getNextToken();
        if (this.scanner.currentPosition == this.scanner.startPosition) {
          this.scanner.currentPosition += 1;
          this.nextIgnoredToken = -1;
        }
      }
      catch (InvalidInputException localInvalidInputException) {
        pos = this.scanner.currentPosition;
      }
    while (this.nextIgnoredToken < 0);

    if ((this.nextIgnoredToken == 68) && 
      (this.currentToken == 68)) {
      return false;
    }

    this.lastCheckPoint = this.scanner.currentPosition;

    this.scanner.startPosition = pos;
    this.scanner.currentPosition = pos;
    this.scanner.commentPtr = -1;
    this.scanner.foundTaskCount = 0;
    return true;
  }

  protected MessageSend newMessageSend()
  {
    MessageSend m = new MessageSend();
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      this.expressionPtr -= length;
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr + 1, 
        m.arguments = new Expression[length], 
        0, 
        length);
    }
    return m;
  }
  protected MessageSend newMessageSendWithTypeArguments() {
    MessageSend m = new MessageSend();
    int length;
    if ((length = this.expressionLengthStack[(this.expressionLengthPtr--)]) != 0) {
      this.expressionPtr -= length;
      System.arraycopy(
        this.expressionStack, 
        this.expressionPtr + 1, 
        m.arguments = new Expression[length], 
        0, 
        length);
    }
    return m;
  }

  protected void optimizedConcatNodeLists()
  {
    this.astLengthStack[(--this.astLengthPtr)] += 1;
  }

  protected void parse()
  {
    boolean isDietParse = this.diet;
    int oldFirstToken = getFirstToken();
    this.hasError = false;

    this.hasReportedError = false;
    int act = 767;
    this.stateStackTop = -1;
    this.currentToken = getFirstToken();
    while (true) {
      int stackLength = this.stack.length;
      if (++this.stateStackTop >= stackLength) {
        System.arraycopy(
          this.stack, 0, 
          this.stack = new int[stackLength + 255], 0, 
          stackLength);
      }
      this.stack[this.stateStackTop] = act;

      act = tAction(act, this.currentToken);
      if ((act == 12794) || (this.restartRecovery))
      {
        int errorPos = this.scanner.currentPosition;
        if (!this.hasReportedError) {
          this.hasError = true;
        }
        if (resumeOnSyntaxError()) {
          if (act == 12794) this.lastErrorEndPosition = errorPos;
          act = 767;
          this.stateStackTop = -1;
          this.currentToken = getFirstToken();
        }
        else {
          act = 12794;
          break;
        }
      } else { if (act <= 695) {
          this.stateStackTop -= 1;
        }
        else if (act > 12794) {
          consumeToken(this.currentToken);
          if (this.currentElement != null) recoveryTokenCheck(); try
          {
            this.currentToken = this.scanner.getNextToken();
          } catch (InvalidInputException e) {
            if (!this.hasReportedError) {
              problemReporter().scannerError(this, e.getMessage());
              this.hasReportedError = true;
            }
            this.lastCheckPoint = this.scanner.currentPosition;
            this.restartRecovery = true;
          }
          if (this.statementRecoveryActivated) {
            jumpOverType();
          }
          act -= 12794;
        }
        else
        {
          if (act >= 12793) break;
          consumeToken(this.currentToken);
          if (this.currentElement != null) recoveryTokenCheck(); try
          {
            this.currentToken = this.scanner.getNextToken();
          } catch (InvalidInputException e) {
            if (!this.hasReportedError) {
              problemReporter().scannerError(this, e.getMessage());
              this.hasReportedError = true;
            }
            this.lastCheckPoint = this.scanner.currentPosition;
            this.restartRecovery = true;
          }
          if (!this.statementRecoveryActivated) continue;
          jumpOverType();

          continue;
        }

        do
        {
          consumeRule(act);
          this.stateStackTop -= rhs[act] - 1;
          act = ntAction(this.stack[this.stateStackTop], lhs[act]);
        }

        while (act <= 695);
      }

    }

    endParse(act);

    NLSTag[] tags = this.scanner.getNLSTags();
    if (tags != null) {
      this.compilationUnit.nlsTags = tags;
    }

    this.scanner.checkNonExternalizedStringLiterals = false;
    if ((this.reportSyntaxErrorIsRequired) && (this.hasError) && (!this.statementRecoveryActivated))
      if (!this.options.performStatementsRecovery) {
        reportSyntaxErrors(isDietParse, oldFirstToken);
      } else {
        RecoveryScannerData data = this.referenceContext.compilationResult().recoveryScannerData;

        if (this.recoveryScanner == null)
          this.recoveryScanner = new RecoveryScanner(this.scanner, data);
        else {
          this.recoveryScanner.setData(data);
        }

        this.recoveryScanner.setSource(this.scanner.source);
        this.recoveryScanner.lineEnds = this.scanner.lineEnds;
        this.recoveryScanner.linePtr = this.scanner.linePtr;

        reportSyntaxErrors(isDietParse, oldFirstToken);

        if (data == null) {
          this.referenceContext.compilationResult().recoveryScannerData = 
            this.recoveryScanner.getData();
        }

        if ((this.methodRecoveryActivated) && (this.options.performStatementsRecovery)) {
          this.methodRecoveryActivated = false;
          recoverStatements();
          this.methodRecoveryActivated = true;

          this.lastAct = 12794;
        }
      }
  }

  public void parse(ConstructorDeclaration cd, CompilationUnitDeclaration unit)
  {
    parse(cd, unit, false);
  }

  public void parse(ConstructorDeclaration cd, CompilationUnitDeclaration unit, boolean recordLineSeparator)
  {
    boolean oldMethodRecoveryActivated = this.methodRecoveryActivated;
    if (this.options.performMethodsFullRecovery) {
      this.methodRecoveryActivated = true;
    }

    initialize();
    goForBlockStatementsopt();
    if (recordLineSeparator) {
      this.scanner.recordLineSeparator = true;
    }
    this.nestedMethod[this.nestedType] += 1;
    pushOnRealBlockStack(0);

    this.referenceContext = cd;
    this.compilationUnit = unit;

    this.scanner.resetTo(cd.bodyStart, cd.bodyEnd);
    try {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
      if (this.options.performStatementsRecovery) {
        this.methodRecoveryActivated = oldMethodRecoveryActivated;
      }
    }

    checkNonNLSAfterBodyEnd(cd.declarationSourceEnd);

    if (this.lastAct == 12794) {
      initialize();
      return;
    }

    cd.explicitDeclarations = this.realBlockStack[(this.realBlockPtr--)];
    int length;
    if ((this.astLengthPtr > -1) && ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)) {
      this.astPtr -= length;
      if ((this.astStack[(this.astPtr + 1)] instanceof ExplicitConstructorCall))
      {
        System.arraycopy(
          this.astStack, 
          this.astPtr + 2, 
          cd.statements = new Statement[length - 1], 
          0, 
          length - 1);
        cd.constructorCall = ((ExplicitConstructorCall)this.astStack[(this.astPtr + 1)]);
      } else {
        System.arraycopy(
          this.astStack, 
          this.astPtr + 1, 
          cd.statements = new Statement[length], 
          0, 
          length);
        cd.constructorCall = SuperReference.implicitSuperConstructorCall();
      }
    } else {
      cd.constructorCall = SuperReference.implicitSuperConstructorCall();
      if (!containsComment(cd.bodyStart, cd.bodyEnd)) {
        cd.bits |= 8;
      }
    }

    if (cd.constructorCall.sourceEnd == 0) {
      cd.constructorCall.sourceEnd = cd.sourceEnd;
      cd.constructorCall.sourceStart = cd.sourceStart;
    }
  }

  public void parse(FieldDeclaration field, TypeDeclaration type, CompilationUnitDeclaration unit, char[] initializationSource)
  {
    initialize();
    goForExpression();
    this.nestedMethod[this.nestedType] += 1;

    this.referenceContext = type;
    this.compilationUnit = unit;

    this.scanner.setSource(initializationSource);
    this.scanner.resetTo(0, initializationSource.length - 1);
    try {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
    }

    if (this.lastAct == 12794) {
      return;
    }

    field.initialization = this.expressionStack[this.expressionPtr];

    if ((type.bits & 0x2) != 0)
      field.bits |= 2;
  }

  public CompilationUnitDeclaration parse(ICompilationUnit sourceUnit, CompilationResult compilationResult)
  {
    return parse(sourceUnit, compilationResult, -1, -1);
  }

  public CompilationUnitDeclaration parse(ICompilationUnit sourceUnit, CompilationResult compilationResult, int start, int end)
  {
    CompilationUnitDeclaration unit;
    try
    {
      initialize(true);
      goForCompilationUnit();

      this.referenceContext = 
        (this.compilationUnit = 
        new CompilationUnitDeclaration(
        this.problemReporter, 
        compilationResult, 
        0));
      char[] contents;
      try
      {
        contents = sourceUnit.getContents();
      }
      catch (AbortCompilationUnit abortException)
      {
        char[] contents;
        problemReporter().cannotReadSource(this.compilationUnit, abortException, this.options.verbose);
        contents = CharOperation.NO_CHAR;
      }
      this.scanner.setSource(contents);
      this.compilationUnit.sourceEnd = (this.scanner.source.length - 1);
      if (end != -1) this.scanner.resetTo(start, end);
      if ((this.javadocParser != null) && (this.javadocParser.checkDocComment)) {
        this.javadocParser.scanner.setSource(contents);
        if (end != -1) {
          this.javadocParser.scanner.resetTo(start, end);
        }
      }

      parse();
    } finally {
      CompilationUnitDeclaration unit = this.compilationUnit;
      this.compilationUnit = null;

      if (!this.diet) unit.bits |= 16;
    }
    return unit;
  }

  public void parse(Initializer initializer, TypeDeclaration type, CompilationUnitDeclaration unit)
  {
    boolean oldMethodRecoveryActivated = this.methodRecoveryActivated;
    if (this.options.performMethodsFullRecovery) {
      this.methodRecoveryActivated = true;
    }

    initialize();
    goForBlockStatementsopt();
    this.nestedMethod[this.nestedType] += 1;
    pushOnRealBlockStack(0);

    this.referenceContext = type;
    this.compilationUnit = unit;

    this.scanner.resetTo(initializer.bodyStart, initializer.bodyEnd);
    try {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
      if (this.options.performStatementsRecovery) {
        this.methodRecoveryActivated = oldMethodRecoveryActivated;
      }
    }

    checkNonNLSAfterBodyEnd(initializer.declarationSourceEnd);

    if (this.lastAct == 12794) {
      return;
    }

    initializer.block.explicitDeclarations = this.realBlockStack[(this.realBlockPtr--)];
    int length;
    if ((this.astLengthPtr > -1) && ((length = this.astLengthStack[(this.astLengthPtr--)]) > 0)) {
      System.arraycopy(this.astStack, this.astPtr -= length + 1, initializer.block.statements = new Statement[length], 0, length);
    }
    else if (!containsComment(initializer.block.sourceStart, initializer.block.sourceEnd)) {
      initializer.block.bits |= 8;
    }

    if ((type.bits & 0x2) != 0)
      initializer.bits |= 2;
  }

  public void parse(MethodDeclaration md, CompilationUnitDeclaration unit)
  {
    if (md.isAbstract())
      return;
    if (md.isNative())
      return;
    if ((md.modifiers & 0x1000000) != 0) {
      return;
    }
    boolean oldMethodRecoveryActivated = this.methodRecoveryActivated;
    if (this.options.performMethodsFullRecovery) {
      this.methodRecoveryActivated = true;
      this.rParenPos = md.sourceEnd;
    }
    initialize();
    goForBlockStatementsopt();
    this.nestedMethod[this.nestedType] += 1;
    pushOnRealBlockStack(0);

    this.referenceContext = md;
    this.compilationUnit = unit;

    this.scanner.resetTo(md.bodyStart, md.bodyEnd);
    try
    {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
      if (this.options.performStatementsRecovery) {
        this.methodRecoveryActivated = oldMethodRecoveryActivated;
      }
    }

    checkNonNLSAfterBodyEnd(md.declarationSourceEnd);

    if (this.lastAct == 12794) {
      return;
    }

    md.explicitDeclarations = this.realBlockStack[(this.realBlockPtr--)];
    int length;
    if ((this.astLengthPtr > -1) && ((length = this.astLengthStack[(this.astLengthPtr--)]) != 0)) {
      System.arraycopy(
        this.astStack, 
        this.astPtr -= length + 1, 
        md.statements = new Statement[length], 
        0, 
        length);
    }
    else if (!containsComment(md.bodyStart, md.bodyEnd))
      md.bits |= 8;
  }

  public ASTNode[] parseClassBodyDeclarations(char[] source, int offset, int length, CompilationUnitDeclaration unit)
  {
    initialize();
    goForClassBodyDeclarations();

    this.scanner.setSource(source);
    this.scanner.resetTo(offset, offset + length - 1);
    if ((this.javadocParser != null) && (this.javadocParser.checkDocComment)) {
      this.javadocParser.scanner.setSource(source);
      this.javadocParser.scanner.resetTo(offset, offset + length - 1);
    }

    this.nestedType = 1;

    this.referenceContext = unit;
    this.compilationUnit = unit;
    try
    {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    }

    if (this.lastAct == 12794)
      return null;
    int astLength;
    if ((this.astLengthPtr > -1) && ((astLength = this.astLengthStack[(this.astLengthPtr--)]) != 0)) {
      ASTNode[] result = new ASTNode[astLength];
      this.astPtr -= astLength;
      System.arraycopy(this.astStack, this.astPtr + 1, result, 0, astLength);
      return result;
    }
    return null;
  }

  public Expression parseExpression(char[] source, int offset, int length, CompilationUnitDeclaration unit) {
    initialize();
    goForExpression();
    this.nestedMethod[this.nestedType] += 1;

    this.referenceContext = unit;
    this.compilationUnit = unit;

    this.scanner.setSource(source);
    this.scanner.resetTo(offset, offset + length - 1);
    try {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
    }

    if (this.lastAct == 12794) {
      return null;
    }

    return this.expressionStack[this.expressionPtr];
  }

  public Expression parseMemberValue(char[] source, int offset, int length, CompilationUnitDeclaration unit) {
    initialize();
    goForMemberValue();
    this.nestedMethod[this.nestedType] += 1;

    this.referenceContext = unit;
    this.compilationUnit = unit;

    this.scanner.setSource(source);
    this.scanner.resetTo(offset, offset + length - 1);
    try {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
    }

    if (this.lastAct == 12794) {
      return null;
    }

    return this.expressionStack[this.expressionPtr];
  }
  public void parseStatements(ReferenceContext rc, int start, int end, TypeDeclaration[] types, CompilationUnitDeclaration unit) {
    boolean oldStatementRecoveryEnabled = this.statementRecoveryActivated;
    this.statementRecoveryActivated = true;

    initialize();

    goForBlockStatementsopt();
    this.nestedMethod[this.nestedType] += 1;
    pushOnRealBlockStack(0);

    pushOnAstLengthStack(0);

    this.referenceContext = rc;
    this.compilationUnit = unit;

    this.pendingRecoveredType = null;

    if ((types != null) && (types.length > 0)) {
      this.recoveredTypes = types;
      this.recoveredTypePtr = 0;
      this.nextTypeStart = 
        (this.recoveredTypes[0].allocation == null ? 
        this.recoveredTypes[0].declarationSourceStart : 
        this.recoveredTypes[0].allocation.sourceStart);
    } else {
      this.recoveredTypes = null;
      this.recoveredTypePtr = -1;
      this.nextTypeStart = -1;
    }

    this.scanner.resetTo(start, end);

    this.lastCheckPoint = this.scanner.initialPosition;

    this.stateStackTop = -1;
    try
    {
      parse();
    } catch (AbortCompilation localAbortCompilation) {
      this.lastAct = 12794;
    } finally {
      this.nestedMethod[this.nestedType] -= 1;
      this.recoveredTypes = null;
      this.statementRecoveryActivated = oldStatementRecoveryEnabled;
    }

    checkNonNLSAfterBodyEnd(end);
  }
  public void persistLineSeparatorPositions() {
    if (this.scanner.recordLineSeparator)
      this.compilationUnit.compilationResult.lineSeparatorPositions = this.scanner.getLineEnds();
  }

  protected void prepareForBlockStatements()
  {
    int tmp6_5 = 0; this.nestedType = tmp6_5; this.nestedMethod[tmp6_5] = 1;
    this.variablesCounter[this.nestedType] = 0;
    int tmp28_27 = 1; this.realBlockPtr = tmp28_27; this.realBlockStack[tmp28_27] = 0;
  }

  public ProblemReporter problemReporter()
  {
    if (this.scanner.recordLineSeparator) {
      this.compilationUnit.compilationResult.lineSeparatorPositions = this.scanner.getLineEnds();
    }
    this.problemReporter.referenceContext = this.referenceContext;
    return this.problemReporter;
  }

  protected void pushIdentifier()
  {
    int stackLength = this.identifierStack.length;
    if (++this.identifierPtr >= stackLength) {
      System.arraycopy(
        this.identifierStack, 0, 
        this.identifierStack = new char[stackLength + 20][], 0, 
        stackLength);
      System.arraycopy(
        this.identifierPositionStack, 0, 
        this.identifierPositionStack = new long[stackLength + 20], 0, 
        stackLength);
    }
    this.identifierStack[this.identifierPtr] = this.scanner.getCurrentIdentifierSource();
    this.identifierPositionStack[this.identifierPtr] = 
      ((this.scanner.startPosition << 32) + (this.scanner.currentPosition - 1));

    stackLength = this.identifierLengthStack.length;
    if (++this.identifierLengthPtr >= stackLength) {
      System.arraycopy(
        this.identifierLengthStack, 0, 
        this.identifierLengthStack = new int[stackLength + 10], 0, 
        stackLength);
    }
    this.identifierLengthStack[this.identifierLengthPtr] = 1;
  }

  protected void pushIdentifier(int flag)
  {
    int stackLength = this.identifierLengthStack.length;
    if (++this.identifierLengthPtr >= stackLength) {
      System.arraycopy(
        this.identifierLengthStack, 0, 
        this.identifierLengthStack = new int[stackLength + 10], 0, 
        stackLength);
    }
    this.identifierLengthStack[this.identifierLengthPtr] = flag;
  }

  protected void pushOnAstLengthStack(int pos) {
    int stackLength = this.astLengthStack.length;
    if (++this.astLengthPtr >= stackLength) {
      System.arraycopy(
        this.astLengthStack, 0, 
        this.astLengthStack = new int[stackLength + 255], 0, 
        stackLength);
    }
    this.astLengthStack[this.astLengthPtr] = pos;
  }

  protected void pushOnAstStack(ASTNode node)
  {
    int stackLength = this.astStack.length;
    if (++this.astPtr >= stackLength) {
      System.arraycopy(
        this.astStack, 0, 
        this.astStack = new ASTNode[stackLength + 100], 0, 
        stackLength);
      this.astPtr = stackLength;
    }
    this.astStack[this.astPtr] = node;

    stackLength = this.astLengthStack.length;
    if (++this.astLengthPtr >= stackLength) {
      System.arraycopy(
        this.astLengthStack, 0, 
        this.astLengthStack = new int[stackLength + 100], 0, 
        stackLength);
    }
    this.astLengthStack[this.astLengthPtr] = 1;
  }

  protected void pushOnExpressionStack(Expression expr) {
    int stackLength = this.expressionStack.length;
    if (++this.expressionPtr >= stackLength) {
      System.arraycopy(
        this.expressionStack, 0, 
        this.expressionStack = new Expression[stackLength + 100], 0, 
        stackLength);
    }
    this.expressionStack[this.expressionPtr] = expr;

    stackLength = this.expressionLengthStack.length;
    if (++this.expressionLengthPtr >= stackLength) {
      System.arraycopy(
        this.expressionLengthStack, 0, 
        this.expressionLengthStack = new int[stackLength + 100], 0, 
        stackLength);
    }
    this.expressionLengthStack[this.expressionLengthPtr] = 1;
  }

  protected void pushOnExpressionStackLengthStack(int pos) {
    int stackLength = this.expressionLengthStack.length;
    if (++this.expressionLengthPtr >= stackLength) {
      System.arraycopy(
        this.expressionLengthStack, 0, 
        this.expressionLengthStack = new int[stackLength + 255], 0, 
        stackLength);
    }
    this.expressionLengthStack[this.expressionLengthPtr] = pos;
  }

  protected void pushOnGenericsStack(ASTNode node)
  {
    int stackLength = this.genericsStack.length;
    if (++this.genericsPtr >= stackLength) {
      System.arraycopy(
        this.genericsStack, 0, 
        this.genericsStack = new ASTNode[stackLength + 10], 0, 
        stackLength);
    }
    this.genericsStack[this.genericsPtr] = node;

    stackLength = this.genericsLengthStack.length;
    if (++this.genericsLengthPtr >= stackLength) {
      System.arraycopy(
        this.genericsLengthStack, 0, 
        this.genericsLengthStack = new int[stackLength + 10], 0, 
        stackLength);
    }
    this.genericsLengthStack[this.genericsLengthPtr] = 1;
  }
  protected void pushOnGenericsIdentifiersLengthStack(int pos) {
    int stackLength = this.genericsIdentifiersLengthStack.length;
    if (++this.genericsIdentifiersLengthPtr >= stackLength) {
      System.arraycopy(
        this.genericsIdentifiersLengthStack, 0, 
        this.genericsIdentifiersLengthStack = new int[stackLength + 10], 0, 
        stackLength);
    }
    this.genericsIdentifiersLengthStack[this.genericsIdentifiersLengthPtr] = pos;
  }
  protected void pushOnGenericsLengthStack(int pos) {
    int stackLength = this.genericsLengthStack.length;
    if (++this.genericsLengthPtr >= stackLength) {
      System.arraycopy(
        this.genericsLengthStack, 0, 
        this.genericsLengthStack = new int[stackLength + 10], 0, 
        stackLength);
    }
    this.genericsLengthStack[this.genericsLengthPtr] = pos;
  }

  protected void pushOnIntStack(int pos) {
    int stackLength = this.intStack.length;
    if (++this.intPtr >= stackLength) {
      System.arraycopy(
        this.intStack, 0, 
        this.intStack = new int[stackLength + 255], 0, 
        stackLength);
    }
    this.intStack[this.intPtr] = pos;
  }

  protected void pushOnRealBlockStack(int i) {
    int stackLength = this.realBlockStack.length;
    if (++this.realBlockPtr >= stackLength) {
      System.arraycopy(
        this.realBlockStack, 0, 
        this.realBlockStack = new int[stackLength + 255], 0, 
        stackLength);
    }
    this.realBlockStack[this.realBlockPtr] = i;
  }

  protected void recoverStatements()
  {
    1.MethodVisitor methodVisitor = new 1.MethodVisitor();
    1.TypeVisitor typeVisitor = new 1.TypeVisitor();
    methodVisitor.typeVisitor = typeVisitor;
    typeVisitor.methodVisitor = methodVisitor;

    if ((this.referenceContext instanceof AbstractMethodDeclaration)) {
      ((AbstractMethodDeclaration)this.referenceContext).traverse(methodVisitor, null);
    } else if ((this.referenceContext instanceof TypeDeclaration)) {
      TypeDeclaration typeContext = (TypeDeclaration)this.referenceContext;

      int length = typeContext.fields.length;
      for (int i = 0; i < length; i++) {
        FieldDeclaration fieldDeclaration = typeContext.fields[i];
        switch (fieldDeclaration.getKind()) {
        case 2:
          methodVisitor.enclosingType = typeContext;
          ((Initializer)fieldDeclaration).traverse(methodVisitor, null);
        }
      }
    }
  }

  public void recoveryExitFromVariable()
  {
    if ((this.currentElement != null) && (this.currentElement.parent != null))
      if ((this.currentElement instanceof RecoveredLocalVariable))
      {
        int end = ((RecoveredLocalVariable)this.currentElement).localDeclaration.sourceEnd;
        this.currentElement.updateSourceEndIfNecessary(end);
        this.currentElement = this.currentElement.parent;
      } else if (((this.currentElement instanceof RecoveredField)) && 
        (!(this.currentElement instanceof RecoveredInitializer)))
      {
        int end = ((RecoveredField)this.currentElement).fieldDeclaration.sourceEnd;
        this.currentElement.updateSourceEndIfNecessary(end);
        this.currentElement = this.currentElement.parent;
      }
  }

  public void recoveryTokenCheck()
  {
    switch (this.currentToken) {
    case 69:
      RecoveredElement newElement = null;
      if (!this.ignoreNextOpeningBrace) {
        newElement = this.currentElement.updateOnOpeningBrace(this.scanner.startPosition - 1, this.scanner.currentPosition - 1);
      }
      this.lastCheckPoint = this.scanner.currentPosition;
      if (newElement == null) break label266;
      this.restartRecovery = true;
      this.currentElement = newElement;

      break;
    case 31:
      this.rBraceStart = (this.scanner.startPosition - 1);
      this.rBraceEnd = (this.scanner.currentPosition - 1);
      this.endPosition = flushCommentsDefinedPriorTo(this.rBraceEnd);
      RecoveredElement newElement = 
        this.currentElement.updateOnClosingBrace(this.scanner.startPosition, this.rBraceEnd);
      this.lastCheckPoint = this.scanner.currentPosition;
      if (newElement == this.currentElement) break label266;
      this.currentElement = newElement;

      break;
    case 27:
      this.endStatementPosition = (this.scanner.currentPosition - 1);
      this.endPosition = (this.scanner.startPosition - 1);
      RecoveredType currentType = currentRecoveryType();
      if (currentType != null) {
        currentType.insideEnumConstantPart = false;
      }
      break;
    }
    if ((this.rBraceEnd > this.rBraceSuccessorStart) && (this.scanner.currentPosition != this.scanner.startPosition)) {
      this.rBraceSuccessorStart = this.scanner.startPosition;
    }

    label266: this.ignoreNextOpeningBrace = false;
  }

  protected void reportSyntaxErrors(boolean isDietParse, int oldFirstToken) {
    if ((this.referenceContext instanceof MethodDeclaration)) {
      MethodDeclaration methodDeclaration = (MethodDeclaration)this.referenceContext;
      if ((methodDeclaration.bits & 0x20) != 0) {
        return;
      }
    }
    this.compilationUnit.compilationResult.lineSeparatorPositions = this.scanner.getLineEnds();
    this.scanner.recordLineSeparator = false;

    int start = this.scanner.initialPosition;
    int end = this.scanner.eofPosition == 2147483647 ? this.scanner.eofPosition : this.scanner.eofPosition - 1;
    if (isDietParse) {
      TypeDeclaration[] types = this.compilationUnit.types;
      int[][] intervalToSkip = RangeUtil.computeDietRange(types);
      DiagnoseParser diagnoseParser = new DiagnoseParser(this, oldFirstToken, start, end, intervalToSkip[0], intervalToSkip[1], intervalToSkip[2], this.options);
      diagnoseParser.diagnoseParse(false);

      reportSyntaxErrorsForSkippedMethod(types);
      this.scanner.resetTo(start, end);
    } else {
      DiagnoseParser diagnoseParser = new DiagnoseParser(this, oldFirstToken, start, end, this.options);
      diagnoseParser.diagnoseParse(this.options.performStatementsRecovery);
    }
  }

  private void reportSyntaxErrorsForSkippedMethod(TypeDeclaration[] types) { if (types != null)
      for (int i = 0; i < types.length; i++) {
        TypeDeclaration[] memberTypes = types[i].memberTypes;
        if (memberTypes != null) {
          reportSyntaxErrorsForSkippedMethod(memberTypes);
        }

        AbstractMethodDeclaration[] methods = types[i].methods;
        if (methods != null) {
          for (int j = 0; j < methods.length; j++) {
            AbstractMethodDeclaration method = methods[j];
            if ((method.bits & 0x20) != 0) {
              if (method.isAnnotationMethod()) {
                DiagnoseParser diagnoseParser = new DiagnoseParser(this, 23, method.declarationSourceStart, method.declarationSourceEnd, this.options);
                diagnoseParser.diagnoseParse(this.options.performStatementsRecovery);
              } else {
                DiagnoseParser diagnoseParser = new DiagnoseParser(this, 6, method.declarationSourceStart, method.declarationSourceEnd, this.options);
                diagnoseParser.diagnoseParse(this.options.performStatementsRecovery);
              }
            }
          }

        }

        FieldDeclaration[] fields = types[i].fields;
        if (fields != null) {
          int length = fields.length;
          for (int j = 0; j < length; j++)
            if ((fields[j] instanceof Initializer)) {
              Initializer initializer = (Initializer)fields[j];
              if ((initializer.bits & 0x20) != 0) {
                DiagnoseParser diagnoseParser = new DiagnoseParser(this, 8, initializer.declarationSourceStart, initializer.declarationSourceEnd, this.options);
                diagnoseParser.diagnoseParse(this.options.performStatementsRecovery);
              }
            }
        }
      }
  }

  protected void resetModifiers()
  {
    this.modifiers = 0;
    this.modifiersSourceStart = -1;
    this.scanner.commentPtr = -1;
  }

  protected void resetStacks()
  {
    this.astPtr = -1;
    this.astLengthPtr = -1;
    this.expressionPtr = -1;
    this.expressionLengthPtr = -1;
    this.identifierPtr = -1;
    this.identifierLengthPtr = -1;
    this.intPtr = -1;
    int tmp41_40 = 0; this.nestedType = tmp41_40; this.nestedMethod[tmp41_40] = 0;
    this.variablesCounter[this.nestedType] = 0;
    this.dimensions = 0;
    int tmp68_67 = 0; this.realBlockPtr = tmp68_67; this.realBlockStack[tmp68_67] = 0;
    this.recoveredStaticInitializerStart = 0;
    this.listLength = 0;
    this.listTypeParameterLength = 0;

    this.genericsIdentifiersLengthPtr = -1;
    this.genericsLengthPtr = -1;
    this.genericsPtr = -1;
  }

  protected boolean resumeAfterRecovery()
  {
    if ((!this.methodRecoveryActivated) && (!this.statementRecoveryActivated))
    {
      resetStacks();
      resetModifiers();

      if (!moveRecoveryCheckpoint()) {
        return false;
      }

      if ((this.referenceContext instanceof CompilationUnitDeclaration)) {
        goForHeaders();
        this.diet = true;
        return true;
      }

      return false;
    }if (!this.statementRecoveryActivated)
    {
      resetStacks();
      resetModifiers();

      if (!moveRecoveryCheckpoint()) {
        return false;
      }

      goForHeaders();
      return true;
    }
    return false;
  }

  protected boolean resumeOnSyntaxError() {
    this.checkExternalizeStrings = false;
    this.scanner.checkNonExternalizedStringLiterals = false;

    if (this.currentElement == null)
    {
      this.javadoc = null;

      if (this.statementRecoveryActivated) return false;

      this.currentElement = buildInitialRecoveryState();
    }

    if (this.currentElement == null) return false;

    if (this.restartRecovery) {
      this.restartRecovery = false;
    }

    updateRecoveryState();

    return resumeAfterRecovery();
  }
  public void setMethodsFullRecovery(boolean enabled) {
    this.options.performMethodsFullRecovery = enabled;
  }
  public void setStatementsRecovery(boolean enabled) {
    if (enabled) this.options.performMethodsFullRecovery = true;
    this.options.performStatementsRecovery = enabled;
  }

  public String toString()
  {
    String s = "lastCheckpoint : int = " + String.valueOf(this.lastCheckPoint) + "\n";
    s = s + "identifierStack : char[" + (this.identifierPtr + 1) + "][] = {";
    for (int i = 0; i <= this.identifierPtr; i++) {
      s = s + "\"" + String.valueOf(this.identifierStack[i]) + "\",";
    }
    s = s + "}\n";

    s = s + "identifierLengthStack : int[" + (this.identifierLengthPtr + 1) + "] = {";
    for (int i = 0; i <= this.identifierLengthPtr; i++) {
      s = s + this.identifierLengthStack[i] + ",";
    }
    s = s + "}\n";

    s = s + "astLengthStack : int[" + (this.astLengthPtr + 1) + "] = {";
    for (int i = 0; i <= this.astLengthPtr; i++) {
      s = s + this.astLengthStack[i] + ",";
    }
    s = s + "}\n";
    s = s + "astPtr : int = " + String.valueOf(this.astPtr) + "\n";

    s = s + "intStack : int[" + (this.intPtr + 1) + "] = {";
    for (int i = 0; i <= this.intPtr; i++) {
      s = s + this.intStack[i] + ",";
    }
    s = s + "}\n";

    s = s + "expressionLengthStack : int[" + (this.expressionLengthPtr + 1) + "] = {";
    for (int i = 0; i <= this.expressionLengthPtr; i++) {
      s = s + this.expressionLengthStack[i] + ",";
    }
    s = s + "}\n";

    s = s + "expressionPtr : int = " + String.valueOf(this.expressionPtr) + "\n";

    s = s + "genericsIdentifiersLengthStack : int[" + (this.genericsIdentifiersLengthPtr + 1) + "] = {";
    for (int i = 0; i <= this.genericsIdentifiersLengthPtr; i++) {
      s = s + this.genericsIdentifiersLengthStack[i] + ",";
    }
    s = s + "}\n";

    s = s + "genericsLengthStack : int[" + (this.genericsLengthPtr + 1) + "] = {";
    for (int i = 0; i <= this.genericsLengthPtr; i++) {
      s = s + this.genericsLengthStack[i] + ",";
    }
    s = s + "}\n";

    s = s + "genericsPtr : int = " + String.valueOf(this.genericsPtr) + "\n";

    s = s + "\n\n\n----------------Scanner--------------\n" + this.scanner.toString();
    return s;
  }

  protected void updateRecoveryState()
  {
    this.currentElement.updateFromParserState();

    recoveryTokenCheck();
  }

  protected void updateSourceDeclarationParts(int variableDeclaratorsCounter)
  {
    int endTypeDeclarationPosition = 
      -1 + this.astStack[(this.astPtr - variableDeclaratorsCounter + 1)].sourceStart;
    for (int i = 0; i < variableDeclaratorsCounter - 1; i++)
    {
      FieldDeclaration field = (FieldDeclaration)this.astStack[(this.astPtr - i - 1)];
      field.endPart1Position = endTypeDeclarationPosition;
      field.endPart2Position = (-1 + this.astStack[(this.astPtr - i)].sourceStart);
    }
    FieldDeclaration field;
    (field = (FieldDeclaration)this.astStack[this.astPtr]).endPart1Position = 
      endTypeDeclarationPosition;
    field.endPart2Position = field.declarationSourceEnd;
  }

  protected void updateSourcePosition(Expression exp)
  {
    exp.sourceEnd = this.intStack[(this.intPtr--)];
    exp.sourceStart = this.intStack[(this.intPtr--)];
  }

  class 1$MethodVisitor extends ASTVisitor
  {
    public ASTVisitor typeVisitor;
    TypeDeclaration enclosingType;
    TypeDeclaration[] types = new TypeDeclaration[0];
    int typePtr = -1;

    1$MethodVisitor() {  } 
    public boolean visit(ConstructorDeclaration constructorDeclaration, ClassScope scope) { this.typePtr = -1;
      return true; }

    public boolean visit(Initializer initializer, MethodScope scope) {
      this.typePtr = -1;
      return true;
    }
    public boolean visit(MethodDeclaration methodDeclaration, ClassScope scope) {
      this.typePtr = -1;
      return true;
    }
    public boolean visit(TypeDeclaration typeDeclaration, BlockScope scope) {
      return visit(typeDeclaration);
    }
    public boolean visit(TypeDeclaration typeDeclaration, ClassScope scope) {
      return visit(typeDeclaration);
    }
    private boolean visit(TypeDeclaration typeDeclaration) {
      if (this.types.length <= ++this.typePtr) {
        int length = this.typePtr;
        System.arraycopy(this.types, 0, this.types = new TypeDeclaration[length * 2 + 1], 0, length);
      }
      this.types[this.typePtr] = typeDeclaration;
      return false;
    }
    public void endVisit(ConstructorDeclaration constructorDeclaration, ClassScope scope) {
      endVisitMethod(constructorDeclaration, scope);
    }
    public void endVisit(MethodDeclaration methodDeclaration, ClassScope scope) {
      endVisitMethod(methodDeclaration, scope);
    }
    private void endVisitMethod(AbstractMethodDeclaration methodDeclaration, ClassScope scope) {
      TypeDeclaration[] foundTypes = (TypeDeclaration[])null;
      int length = 0;
      if (this.typePtr > -1) {
        length = this.typePtr + 1;
        foundTypes = new TypeDeclaration[length];
        System.arraycopy(this.types, 0, foundTypes, 0, length);
      }
      ReferenceContext oldContext = Parser.this.referenceContext;
      Parser.this.recoveryScanner.resetTo(methodDeclaration.bodyStart, methodDeclaration.bodyEnd);
      Scanner oldScanner = Parser.this.scanner;
      Parser.this.scanner = Parser.this.recoveryScanner;
      Parser.this.parseStatements(
        methodDeclaration, 
        methodDeclaration.bodyStart, 
        methodDeclaration.bodyEnd, 
        foundTypes, 
        Parser.this.compilationUnit);
      Parser.this.scanner = oldScanner;
      Parser.this.referenceContext = oldContext;

      for (int i = 0; i < length; i++)
        foundTypes[i].traverse(this.typeVisitor, scope);
    }

    public void endVisit(Initializer initializer, MethodScope scope) {
      TypeDeclaration[] foundTypes = (TypeDeclaration[])null;
      int length = 0;
      if (this.typePtr > -1) {
        length = this.typePtr + 1;
        foundTypes = new TypeDeclaration[length];
        System.arraycopy(this.types, 0, foundTypes, 0, length);
      }
      ReferenceContext oldContext = Parser.this.referenceContext;
      Parser.this.recoveryScanner.resetTo(initializer.bodyStart, initializer.bodyEnd);
      Scanner oldScanner = Parser.this.scanner;
      Parser.this.scanner = Parser.this.recoveryScanner;
      Parser.this.parseStatements(
        this.enclosingType, 
        initializer.bodyStart, 
        initializer.bodyEnd, 
        foundTypes, 
        Parser.this.compilationUnit);
      Parser.this.scanner = oldScanner;
      Parser.this.referenceContext = oldContext;

      for (int i = 0; i < length; i++)
        foundTypes[i].traverse(this.typeVisitor, scope);
    }
  }

  class 1$TypeVisitor extends ASTVisitor
  {
    public Parser.1.MethodVisitor methodVisitor;
    TypeDeclaration[] types = new TypeDeclaration[0];
    int typePtr = -1;

    1$TypeVisitor() {  } 
    public void endVisit(TypeDeclaration typeDeclaration, BlockScope scope) { endVisitType(); }

    public void endVisit(TypeDeclaration typeDeclaration, ClassScope scope) {
      endVisitType();
    }
    private void endVisitType() {
      this.typePtr -= 1;
    }
    public boolean visit(TypeDeclaration typeDeclaration, BlockScope scope) {
      return visit(typeDeclaration);
    }
    public boolean visit(TypeDeclaration typeDeclaration, ClassScope scope) {
      return visit(typeDeclaration);
    }
    private boolean visit(TypeDeclaration typeDeclaration) {
      if (this.types.length <= ++this.typePtr) {
        int length = this.typePtr;
        System.arraycopy(this.types, 0, this.types = new TypeDeclaration[length * 2 + 1], 0, length);
      }
      this.types[this.typePtr] = typeDeclaration;
      return true;
    }
    public boolean visit(ConstructorDeclaration constructorDeclaration, ClassScope scope) {
      if (constructorDeclaration.isDefaultConstructor()) return false;

      constructorDeclaration.traverse(this.methodVisitor, scope);
      return false;
    }
    public boolean visit(Initializer initializer, MethodScope scope) {
      this.methodVisitor.enclosingType = this.types[this.typePtr];
      initializer.traverse(this.methodVisitor, scope);
      return false;
    }
    public boolean visit(MethodDeclaration methodDeclaration, ClassScope scope) {
      methodDeclaration.traverse(this.methodVisitor, scope);
      return false;
    }
  }
}