package org.quartz;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.spi.JobFactory;

public abstract interface Scheduler
{
  public static final String DEFAULT_GROUP = "DEFAULT";
  public static final String DEFAULT_RECOVERY_GROUP = "RECOVERING_JOBS";
  public static final String DEFAULT_FAIL_OVER_GROUP = "FAILED_OVER_JOBS";
  public static final String FAILED_JOB_ORIGINAL_TRIGGER_NAME = "QRTZ_FAILED_JOB_ORIG_TRIGGER_NAME";
  public static final String FAILED_JOB_ORIGINAL_TRIGGER_GROUP = "QRTZ_FAILED_JOB_ORIG_TRIGGER_GROUP";
  public static final String FAILED_JOB_ORIGINAL_TRIGGER_FIRETIME_IN_MILLISECONDS = "QRTZ_FAILED_JOB_ORIG_TRIGGER_FIRETIME_IN_MILLISECONDS_AS_STRING";

  public abstract String getSchedulerName()
    throws SchedulerException;

  public abstract String getSchedulerInstanceId()
    throws SchedulerException;

  public abstract SchedulerContext getContext()
    throws SchedulerException;

  public abstract void start()
    throws SchedulerException;

  public abstract void startDelayed(int paramInt)
    throws SchedulerException;

  public abstract boolean isStarted()
    throws SchedulerException;

  public abstract void standby()
    throws SchedulerException;

  public abstract boolean isInStandbyMode()
    throws SchedulerException;

  public abstract void shutdown()
    throws SchedulerException;

  public abstract void shutdown(boolean paramBoolean)
    throws SchedulerException;

  public abstract boolean isShutdown()
    throws SchedulerException;

  public abstract SchedulerMetaData getMetaData()
    throws SchedulerException;

  public abstract List<JobExecutionContext> getCurrentlyExecutingJobs()
    throws SchedulerException;

  public abstract void setJobFactory(JobFactory paramJobFactory)
    throws SchedulerException;

  public abstract ListenerManager getListenerManager()
    throws SchedulerException;

  public abstract Date scheduleJob(JobDetail paramJobDetail, Trigger paramTrigger)
    throws SchedulerException;

  public abstract Date scheduleJob(Trigger paramTrigger)
    throws SchedulerException;

  public abstract void scheduleJobs(Map<JobDetail, List<Trigger>> paramMap, boolean paramBoolean)
    throws SchedulerException;

  public abstract boolean unscheduleJob(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract boolean unscheduleJobs(List<TriggerKey> paramList)
    throws SchedulerException;

  public abstract Date rescheduleJob(TriggerKey paramTriggerKey, Trigger paramTrigger)
    throws SchedulerException;

  public abstract void addJob(JobDetail paramJobDetail, boolean paramBoolean)
    throws SchedulerException;

  public abstract boolean deleteJob(JobKey paramJobKey)
    throws SchedulerException;

  public abstract boolean deleteJobs(List<JobKey> paramList)
    throws SchedulerException;

  public abstract void triggerJob(JobKey paramJobKey)
    throws SchedulerException;

  public abstract void triggerJob(JobKey paramJobKey, JobDataMap paramJobDataMap)
    throws SchedulerException;

  public abstract void pauseJob(JobKey paramJobKey)
    throws SchedulerException;

  public abstract void pauseJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract void pauseTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract void pauseTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract void resumeJob(JobKey paramJobKey)
    throws SchedulerException;

  public abstract void resumeJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract void resumeTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract void resumeTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract void pauseAll()
    throws SchedulerException;

  public abstract void resumeAll()
    throws SchedulerException;

  public abstract List<String> getJobGroupNames()
    throws SchedulerException;

  public abstract Set<JobKey> getJobKeys(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract List<? extends Trigger> getTriggersOfJob(JobKey paramJobKey)
    throws SchedulerException;

  public abstract List<String> getTriggerGroupNames()
    throws SchedulerException;

  public abstract Set<TriggerKey> getTriggerKeys(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException;

  public abstract Set<String> getPausedTriggerGroups()
    throws SchedulerException;

  public abstract JobDetail getJobDetail(JobKey paramJobKey)
    throws SchedulerException;

  public abstract Trigger getTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract Trigger.TriggerState getTriggerState(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract void addCalendar(String paramString, Calendar paramCalendar, boolean paramBoolean1, boolean paramBoolean2)
    throws SchedulerException;

  public abstract boolean deleteCalendar(String paramString)
    throws SchedulerException;

  public abstract Calendar getCalendar(String paramString)
    throws SchedulerException;

  public abstract List<String> getCalendarNames()
    throws SchedulerException;

  public abstract boolean interrupt(JobKey paramJobKey)
    throws UnableToInterruptJobException;

  public abstract boolean interrupt(String paramString)
    throws UnableToInterruptJobException;

  public abstract boolean checkExists(JobKey paramJobKey)
    throws SchedulerException;

  public abstract boolean checkExists(TriggerKey paramTriggerKey)
    throws SchedulerException;

  public abstract void clear()
    throws SchedulerException;
}