package org.quartz.core;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.quartz.Calendar;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobKey;
import org.quartz.SchedulerContext;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.Trigger.TriggerState;
import org.quartz.TriggerKey;
import org.quartz.UnableToInterruptJobException;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.spi.OperableTrigger;

public abstract interface RemotableQuartzScheduler extends Remote
{
  public abstract String getSchedulerName()
    throws RemoteException;

  public abstract String getSchedulerInstanceId()
    throws RemoteException;

  public abstract SchedulerContext getSchedulerContext()
    throws SchedulerException, RemoteException;

  public abstract void start()
    throws SchedulerException, RemoteException;

  public abstract void startDelayed(int paramInt)
    throws SchedulerException, RemoteException;

  public abstract void standby()
    throws RemoteException;

  public abstract boolean isInStandbyMode()
    throws RemoteException;

  public abstract void shutdown()
    throws RemoteException;

  public abstract void shutdown(boolean paramBoolean)
    throws RemoteException;

  public abstract boolean isShutdown()
    throws RemoteException;

  public abstract Date runningSince()
    throws RemoteException;

  public abstract String getVersion()
    throws RemoteException;

  public abstract int numJobsExecuted()
    throws RemoteException;

  public abstract Class<?> getJobStoreClass()
    throws RemoteException;

  public abstract boolean supportsPersistence()
    throws RemoteException;

  public abstract boolean isClustered()
    throws RemoteException;

  public abstract Class<?> getThreadPoolClass()
    throws RemoteException;

  public abstract int getThreadPoolSize()
    throws RemoteException;

  public abstract void clear()
    throws SchedulerException, RemoteException;

  public abstract List<JobExecutionContext> getCurrentlyExecutingJobs()
    throws SchedulerException, RemoteException;

  public abstract Date scheduleJob(JobDetail paramJobDetail, Trigger paramTrigger)
    throws SchedulerException, RemoteException;

  public abstract Date scheduleJob(Trigger paramTrigger)
    throws SchedulerException, RemoteException;

  public abstract void addJob(JobDetail paramJobDetail, boolean paramBoolean)
    throws SchedulerException, RemoteException;

  public abstract boolean deleteJob(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract boolean unscheduleJob(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract Date rescheduleJob(TriggerKey paramTriggerKey, Trigger paramTrigger)
    throws SchedulerException, RemoteException;

  public abstract void triggerJob(JobKey paramJobKey, JobDataMap paramJobDataMap)
    throws SchedulerException, RemoteException;

  public abstract void triggerJob(OperableTrigger paramOperableTrigger)
    throws SchedulerException, RemoteException;

  public abstract void pauseTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract void pauseTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract void pauseJob(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract void pauseJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract void resumeTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract void resumeTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract Set<String> getPausedTriggerGroups()
    throws SchedulerException, RemoteException;

  public abstract void resumeJob(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract void resumeJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract void pauseAll()
    throws SchedulerException, RemoteException;

  public abstract void resumeAll()
    throws SchedulerException, RemoteException;

  public abstract List<String> getJobGroupNames()
    throws SchedulerException, RemoteException;

  public abstract Set<JobKey> getJobKeys(GroupMatcher<JobKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract List<? extends Trigger> getTriggersOfJob(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract List<String> getTriggerGroupNames()
    throws SchedulerException, RemoteException;

  public abstract Set<TriggerKey> getTriggerKeys(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SchedulerException, RemoteException;

  public abstract JobDetail getJobDetail(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract Trigger getTrigger(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract Trigger.TriggerState getTriggerState(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract void addCalendar(String paramString, Calendar paramCalendar, boolean paramBoolean1, boolean paramBoolean2)
    throws SchedulerException, RemoteException;

  public abstract boolean deleteCalendar(String paramString)
    throws SchedulerException, RemoteException;

  public abstract Calendar getCalendar(String paramString)
    throws SchedulerException, RemoteException;

  public abstract List<String> getCalendarNames()
    throws SchedulerException, RemoteException;

  public abstract boolean interrupt(JobKey paramJobKey)
    throws UnableToInterruptJobException, RemoteException;

  public abstract boolean interrupt(String paramString)
    throws UnableToInterruptJobException, RemoteException;

  public abstract boolean checkExists(JobKey paramJobKey)
    throws SchedulerException, RemoteException;

  public abstract boolean checkExists(TriggerKey paramTriggerKey)
    throws SchedulerException, RemoteException;

  public abstract boolean deleteJobs(List<JobKey> paramList)
    throws SchedulerException, RemoteException;

  public abstract void scheduleJobs(Map<JobDetail, List<Trigger>> paramMap, boolean paramBoolean)
    throws SchedulerException, RemoteException;

  public abstract boolean unscheduleJobs(List<TriggerKey> paramList)
    throws SchedulerException, RemoteException;
}