package org.quartz.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.management.AttributeList;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import org.quartz.Calendar;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobKey;
import org.quartz.ListenerManager;
import org.quartz.Scheduler;
import org.quartz.SchedulerContext;
import org.quartz.SchedulerException;
import org.quartz.SchedulerMetaData;
import org.quartz.Trigger;
import org.quartz.Trigger.TriggerState;
import org.quartz.TriggerKey;
import org.quartz.UnableToInterruptJobException;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.spi.JobFactory;

public abstract class RemoteMBeanScheduler
  implements Scheduler
{
  private ObjectName schedulerObjectName;

  protected ObjectName getSchedulerObjectName()
  {
    return this.schedulerObjectName;
  }

  public void setSchedulerObjectName(String schedulerObjectName)
    throws SchedulerException
  {
    try
    {
      this.schedulerObjectName = new ObjectName(schedulerObjectName);
    } catch (MalformedObjectNameException e) {
      throw new SchedulerException("Failed to parse Scheduler MBean name: " + schedulerObjectName, e);
    }
  }

  public void setSchedulerObjectName(ObjectName schedulerObjectName)
    throws SchedulerException
  {
    this.schedulerObjectName = schedulerObjectName;
  }

  public abstract void initialize()
    throws SchedulerException;

  protected abstract Object getAttribute(String paramString)
    throws SchedulerException;

  protected abstract AttributeList getAttributes(String[] paramArrayOfString)
    throws SchedulerException;

  protected abstract Object invoke(String paramString, Object[] paramArrayOfObject, String[] paramArrayOfString)
    throws SchedulerException;

  public String getSchedulerName()
    throws SchedulerException
  {
    return (String)getAttribute("schedulerName");
  }

  public String getSchedulerInstanceId()
    throws SchedulerException
  {
    return (String)getAttribute("schedulerInstanceId");
  }

  public SchedulerMetaData getMetaData() throws SchedulerException {
    AttributeList attributeList = getAttributes(new String[] { "schedulerName", "schedulerInstanceId", "inStandbyMode", "shutdown", "jobStoreClass", "threadPoolClass", "threadPoolSize", "version" });

    return new SchedulerMetaData((String)attributeList.get(0), (String)attributeList.get(1), getClass(), true, isStarted(), ((Boolean)attributeList.get(2)).booleanValue(), ((Boolean)attributeList.get(3)).booleanValue(), (Date)invoke("runningSince", new Object[0], new String[0]), ((Integer)invoke("numJobsExecuted", new Object[0], new String[0])).intValue(), (Class)attributeList.get(4), ((Boolean)invoke("supportsPersistence", new Object[0], new String[0])).booleanValue(), ((Boolean)invoke("isClustered", new Object[0], new String[0])).booleanValue(), (Class)attributeList.get(5), ((Integer)attributeList.get(6)).intValue(), (String)attributeList.get(7));
  }

  public SchedulerContext getContext()
    throws SchedulerException
  {
    return (SchedulerContext)getAttribute("schedulerContext");
  }

  public void start()
    throws SchedulerException
  {
    invoke("start", new Object[0], new String[0]);
  }

  public void startDelayed(int seconds)
    throws SchedulerException
  {
    invoke("startDelayed", new Object[] { Integer.valueOf(seconds) }, new String[] { Integer.TYPE.getName() });
  }

  public void standby()
    throws SchedulerException
  {
    invoke("standby", new Object[0], new String[0]);
  }

  public boolean isStarted()
    throws SchedulerException
  {
    return invoke("runningSince", new Object[0], new String[0]) != null;
  }

  public boolean isInStandbyMode()
    throws SchedulerException
  {
    return ((Boolean)getAttribute("inStandbyMode")).booleanValue();
  }

  public void shutdown()
    throws SchedulerException
  {
    String schedulerName = getSchedulerName();

    invoke("shutdown", new Object[0], new String[0]);
    SchedulerRepository.getInstance().remove(schedulerName);
  }

  public void shutdown(boolean waitForJobsToComplete)
    throws SchedulerException
  {
    String schedulerName = getSchedulerName();

    invoke("shutdown", new Object[] { toBoolean(waitForJobsToComplete) }, new String[] { Boolean.TYPE.getName() });

    SchedulerRepository.getInstance().remove(schedulerName);
  }

  public boolean isShutdown()
    throws SchedulerException
  {
    return ((Boolean)getAttribute("shutdown")).booleanValue();
  }

  public List<JobExecutionContext> getCurrentlyExecutingJobs()
    throws SchedulerException
  {
    return (List)invoke("getCurrentlyExecutingJobs", new Object[0], new String[0]);
  }

  public Date scheduleJob(JobDetail jobDetail, Trigger trigger)
    throws SchedulerException
  {
    return (Date)invoke("scheduleJob", new Object[] { jobDetail, trigger }, new String[] { JobDetail.class.getName(), Trigger.class.getName() });
  }

  public Date scheduleJob(Trigger trigger)
    throws SchedulerException
  {
    return (Date)invoke("scheduleJob", new Object[] { trigger }, new String[] { Trigger.class.getName() });
  }

  public void addJob(JobDetail jobDetail, boolean replace)
    throws SchedulerException
  {
    invoke("addJob", new Object[] { jobDetail, toBoolean(replace) }, new String[] { JobDetail.class.getName(), Boolean.TYPE.getName() });
  }

  public boolean deleteJob(JobKey jobKey)
    throws SchedulerException
  {
    return ((Boolean)invoke("deleteJob", new Object[] { jobKey }, new String[] { JobKey.class.getName() })).booleanValue();
  }

  public boolean unscheduleJob(TriggerKey triggerKey)
    throws SchedulerException
  {
    return ((Boolean)invoke("unscheduleJob", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() })).booleanValue();
  }

  public boolean deleteJobs(List<JobKey> jobKeys)
    throws SchedulerException
  {
    throw new SchedulerException("Operation not supported for remote schedulers.");
  }

  public void scheduleJobs(Map<JobDetail, List<Trigger>> triggersAndJobs, boolean replace) throws SchedulerException {
    throw new SchedulerException("Operation not supported for remote schedulers.");
  }

  public boolean unscheduleJobs(List<TriggerKey> triggerKeys) throws SchedulerException {
    throw new SchedulerException("Operation not supported for remote schedulers.");
  }

  public Date rescheduleJob(TriggerKey triggerKey, Trigger newTrigger)
    throws SchedulerException
  {
    return (Date)invoke("unscheduleJob", new Object[] { triggerKey, newTrigger }, new String[] { TriggerKey.class.getName(), Trigger.class.getName() });
  }

  public void triggerJob(JobKey jobKey)
    throws SchedulerException
  {
    triggerJob(jobKey, null);
  }

  public void triggerJob(JobKey jobKey, JobDataMap data)
    throws SchedulerException
  {
    invoke("triggerJob", new Object[] { jobKey, data }, new String[] { JobKey.class.getName(), JobDataMap.class.getName() });
  }

  public void pauseTrigger(TriggerKey triggerKey)
    throws SchedulerException
  {
    invoke("pauseTrigger", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() });
  }

  public void pauseTriggers(GroupMatcher<TriggerKey> matcher)
    throws SchedulerException
  {
    invoke("pauseTriggers", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });
  }

  public void pauseJob(JobKey jobKey)
    throws SchedulerException
  {
    invoke("pauseJob", new Object[] { jobKey }, new String[] { JobKey.class.getName() });
  }

  public void pauseJobs(GroupMatcher<JobKey> matcher)
    throws SchedulerException
  {
    invoke("pauseJobs", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });
  }

  public void resumeTrigger(TriggerKey triggerKey)
    throws SchedulerException
  {
    invoke("resumeTrigger", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() });
  }

  public void resumeTriggers(GroupMatcher<TriggerKey> matcher)
    throws SchedulerException
  {
    invoke("resumeTriggers", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });
  }

  public void resumeJob(JobKey jobKey)
    throws SchedulerException
  {
    invoke("resumeJob", new Object[] { jobKey }, new String[] { JobKey.class.getName() });
  }

  public void resumeJobs(GroupMatcher<JobKey> matcher)
    throws SchedulerException
  {
    invoke("resumeJobs", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });
  }

  public void pauseAll()
    throws SchedulerException
  {
    invoke("pauseAll", new Object[0], new String[0]);
  }

  public void resumeAll()
    throws SchedulerException
  {
    invoke("resumeAll", new Object[0], new String[0]);
  }

  public List<String> getJobGroupNames()
    throws SchedulerException
  {
    return (List)invoke("getJobGroupNames", new Object[0], new String[0]);
  }

  public Set<JobKey> getJobKeys(GroupMatcher<JobKey> matcher)
    throws SchedulerException
  {
    Set keys = (Set)invoke("getJobNames", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });

    return keys;
  }

  public List<Trigger> getTriggersOfJob(JobKey jobKey)
    throws SchedulerException
  {
    return (List)invoke("getTriggersOfJob", new Object[] { jobKey }, new String[] { JobKey.class.getName() });
  }

  public List<String> getTriggerGroupNames()
    throws SchedulerException
  {
    return (List)invoke("getTriggerGroupNames", new Object[0], new String[0]);
  }

  public Set<TriggerKey> getTriggerKeys(GroupMatcher<TriggerKey> matcher)
    throws SchedulerException
  {
    Set keys = (Set)invoke("getTriggerKeys", new Object[] { matcher }, new String[] { GroupMatcher.class.getName() });

    return keys;
  }

  public JobDetail getJobDetail(JobKey jobKey)
    throws SchedulerException
  {
    return (JobDetail)invoke("getJobDetail", new Object[] { jobKey }, new String[] { JobKey.class.getName() });
  }

  public Trigger getTrigger(TriggerKey triggerKey)
    throws SchedulerException
  {
    return (Trigger)invoke("getTrigger", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() });
  }

  public boolean checkExists(JobKey jobKey)
    throws SchedulerException
  {
    return ((Boolean)invoke("checkExists", new Object[] { jobKey }, new String[] { JobKey.class.getName() })).booleanValue();
  }

  public boolean checkExists(TriggerKey triggerKey)
    throws SchedulerException
  {
    return ((Boolean)invoke("checkExists", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() })).booleanValue();
  }

  public void clear()
    throws SchedulerException
  {
    invoke("clear", new Object[0], new String[0]);
  }

  public Trigger.TriggerState getTriggerState(TriggerKey triggerKey)
    throws SchedulerException
  {
    return (Trigger.TriggerState)invoke("getTriggerState", new Object[] { triggerKey }, new String[] { TriggerKey.class.getName() });
  }

  public void addCalendar(String calName, Calendar calendar, boolean replace, boolean updateTriggers)
    throws SchedulerException
  {
    invoke("addCalendar", new Object[] { calName, calendar, toBoolean(replace), toBoolean(updateTriggers) }, new String[] { String.class.getName(), Calendar.class.getName(), Boolean.TYPE.getName(), Boolean.TYPE.getName() });
  }

  public boolean deleteCalendar(String calName)
    throws SchedulerException
  {
    return ((Boolean)invoke("getTriggerState", new Object[] { calName }, new String[] { String.class.getName() })).booleanValue();
  }

  public Calendar getCalendar(String calName)
    throws SchedulerException
  {
    return (Calendar)invoke("getCalendar", new Object[] { calName }, new String[] { String.class.getName() });
  }

  public List<String> getCalendarNames()
    throws SchedulerException
  {
    return (List)invoke("getCalendarNames", new Object[0], new String[0]);
  }

  public Set<String> getPausedTriggerGroups()
    throws SchedulerException
  {
    return (Set)invoke("getPausedTriggerGroups", new Object[0], new String[0]);
  }

  public ListenerManager getListenerManager()
    throws SchedulerException
  {
    throw new SchedulerException("Operation not supported for remote schedulers.");
  }

  public boolean interrupt(JobKey jobKey)
    throws UnableToInterruptJobException
  {
    try
    {
      return ((Boolean)invoke("interrupt", new Object[] { jobKey }, new String[] { JobKey.class.getName() })).booleanValue();
    }
    catch (SchedulerException se)
    {
      throw new UnableToInterruptJobException(se);
    }
  }

  public boolean interrupt(String fireInstanceId)
    throws UnableToInterruptJobException
  {
    try
    {
      return ((Boolean)invoke("interrupt", new Object[] { fireInstanceId }, new String[] { String.class.getName() })).booleanValue();
    }
    catch (SchedulerException se)
    {
      throw new UnableToInterruptJobException(se);
    }
  }

  public void setJobFactory(JobFactory factory)
    throws SchedulerException
  {
    throw new SchedulerException("Operation not supported for remote schedulers.");
  }

  protected Boolean toBoolean(boolean bool)
  {
    return bool ? Boolean.TRUE : Boolean.FALSE;
  }
}