package org.quartz.impl.jdbcjobstore;

import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.List;
import java.util.Set;
import org.quartz.Calendar;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.JobPersistenceException;
import org.quartz.TriggerKey;
import org.quartz.impl.matchers.GroupMatcher;
import org.quartz.spi.ClassLoadHelper;
import org.quartz.spi.OperableTrigger;
import org.quartz.utils.Key;

public abstract interface DriverDelegate
{
  public abstract void initialize(String paramString)
    throws NoSuchDelegateException;

  public abstract int updateTriggerStatesFromOtherStates(Connection paramConnection, String paramString1, String paramString2, String paramString3)
    throws SQLException;

  public abstract List<TriggerKey> selectMisfiredTriggers(Connection paramConnection, long paramLong)
    throws SQLException;

  public abstract List<TriggerKey> selectMisfiredTriggersInState(Connection paramConnection, String paramString, long paramLong)
    throws SQLException;

  public abstract boolean hasMisfiredTriggersInState(Connection paramConnection, String paramString, long paramLong, int paramInt, List<TriggerKey> paramList)
    throws SQLException;

  public abstract int countMisfiredTriggersInState(Connection paramConnection, String paramString, long paramLong)
    throws SQLException;

  public abstract List<TriggerKey> selectMisfiredTriggersInGroupInState(Connection paramConnection, String paramString1, String paramString2, long paramLong)
    throws SQLException;

  public abstract List<OperableTrigger> selectTriggersForRecoveringJobs(Connection paramConnection)
    throws SQLException, IOException, ClassNotFoundException;

  public abstract int deleteFiredTriggers(Connection paramConnection)
    throws SQLException;

  public abstract int deleteFiredTriggers(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int insertJobDetail(Connection paramConnection, JobDetail paramJobDetail)
    throws IOException, SQLException;

  public abstract int updateJobDetail(Connection paramConnection, JobDetail paramJobDetail)
    throws IOException, SQLException;

  public abstract List<TriggerKey> selectTriggerKeysForJob(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract int deleteJobDetail(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract boolean isJobNonConcurrent(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract boolean jobExists(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract int updateJobData(Connection paramConnection, JobDetail paramJobDetail)
    throws IOException, SQLException;

  public abstract JobDetail selectJobDetail(Connection paramConnection, JobKey paramJobKey, ClassLoadHelper paramClassLoadHelper)
    throws ClassNotFoundException, IOException, SQLException;

  public abstract int selectNumJobs(Connection paramConnection)
    throws SQLException;

  public abstract List<String> selectJobGroups(Connection paramConnection)
    throws SQLException;

  public abstract Set<JobKey> selectJobsInGroup(Connection paramConnection, GroupMatcher<JobKey> paramGroupMatcher)
    throws SQLException;

  public abstract int insertTrigger(Connection paramConnection, OperableTrigger paramOperableTrigger, String paramString, JobDetail paramJobDetail)
    throws SQLException, IOException;

  public abstract int updateTrigger(Connection paramConnection, OperableTrigger paramOperableTrigger, String paramString, JobDetail paramJobDetail)
    throws SQLException, IOException;

  public abstract boolean triggerExists(Connection paramConnection, TriggerKey paramTriggerKey)
    throws SQLException;

  public abstract int updateTriggerState(Connection paramConnection, TriggerKey paramTriggerKey, String paramString)
    throws SQLException;

  public abstract int updateTriggerStateFromOtherState(Connection paramConnection, TriggerKey paramTriggerKey, String paramString1, String paramString2)
    throws SQLException;

  public abstract int updateTriggerStateFromOtherStates(Connection paramConnection, TriggerKey paramTriggerKey, String paramString1, String paramString2, String paramString3, String paramString4)
    throws SQLException;

  public abstract int updateTriggerGroupStateFromOtherStates(Connection paramConnection, GroupMatcher<TriggerKey> paramGroupMatcher, String paramString1, String paramString2, String paramString3, String paramString4)
    throws SQLException;

  public abstract int updateTriggerGroupStateFromOtherState(Connection paramConnection, GroupMatcher<TriggerKey> paramGroupMatcher, String paramString1, String paramString2)
    throws SQLException;

  public abstract int updateTriggerStatesForJob(Connection paramConnection, JobKey paramJobKey, String paramString)
    throws SQLException;

  public abstract int updateTriggerStatesForJobFromOtherState(Connection paramConnection, JobKey paramJobKey, String paramString1, String paramString2)
    throws SQLException;

  public abstract int deleteTrigger(Connection paramConnection, TriggerKey paramTriggerKey)
    throws SQLException;

  public abstract int selectNumTriggersForJob(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract JobDetail selectJobForTrigger(Connection paramConnection, ClassLoadHelper paramClassLoadHelper, TriggerKey paramTriggerKey)
    throws ClassNotFoundException, SQLException;

  public abstract List<OperableTrigger> selectTriggersForJob(Connection paramConnection, JobKey paramJobKey)
    throws SQLException, ClassNotFoundException, IOException, JobPersistenceException;

  public abstract List<OperableTrigger> selectTriggersForCalendar(Connection paramConnection, String paramString)
    throws SQLException, ClassNotFoundException, IOException, JobPersistenceException;

  public abstract OperableTrigger selectTrigger(Connection paramConnection, TriggerKey paramTriggerKey)
    throws SQLException, ClassNotFoundException, IOException, JobPersistenceException;

  public abstract JobDataMap selectTriggerJobDataMap(Connection paramConnection, String paramString1, String paramString2)
    throws SQLException, ClassNotFoundException, IOException;

  public abstract String selectTriggerState(Connection paramConnection, TriggerKey paramTriggerKey)
    throws SQLException;

  public abstract TriggerStatus selectTriggerStatus(Connection paramConnection, TriggerKey paramTriggerKey)
    throws SQLException;

  public abstract int selectNumTriggers(Connection paramConnection)
    throws SQLException;

  public abstract List<String> selectTriggerGroups(Connection paramConnection)
    throws SQLException;

  public abstract List<String> selectTriggerGroups(Connection paramConnection, GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SQLException;

  public abstract Set<TriggerKey> selectTriggersInGroup(Connection paramConnection, GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SQLException;

  public abstract List<TriggerKey> selectTriggersInState(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int insertPausedTriggerGroup(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int deletePausedTriggerGroup(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int deletePausedTriggerGroup(Connection paramConnection, GroupMatcher<TriggerKey> paramGroupMatcher)
    throws SQLException;

  public abstract int deleteAllPausedTriggerGroups(Connection paramConnection)
    throws SQLException;

  public abstract boolean isTriggerGroupPaused(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract Set<String> selectPausedTriggerGroups(Connection paramConnection)
    throws SQLException;

  public abstract boolean isExistingTriggerGroup(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int insertCalendar(Connection paramConnection, String paramString, Calendar paramCalendar)
    throws IOException, SQLException;

  public abstract int updateCalendar(Connection paramConnection, String paramString, Calendar paramCalendar)
    throws IOException, SQLException;

  public abstract boolean calendarExists(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract Calendar selectCalendar(Connection paramConnection, String paramString)
    throws ClassNotFoundException, IOException, SQLException;

  public abstract boolean calendarIsReferenced(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int deleteCalendar(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int selectNumCalendars(Connection paramConnection)
    throws SQLException;

  public abstract List<String> selectCalendars(Connection paramConnection)
    throws SQLException;

  /** @deprecated */
  public abstract long selectNextFireTime(Connection paramConnection)
    throws SQLException;

  public abstract Key<?> selectTriggerForFireTime(Connection paramConnection, long paramLong)
    throws SQLException;

  /** @deprecated */
  public abstract List<TriggerKey> selectTriggerToAcquire(Connection paramConnection, long paramLong1, long paramLong2)
    throws SQLException;

  public abstract List<TriggerKey> selectTriggerToAcquire(Connection paramConnection, long paramLong1, long paramLong2, int paramInt)
    throws SQLException;

  public abstract int insertFiredTrigger(Connection paramConnection, OperableTrigger paramOperableTrigger, String paramString, JobDetail paramJobDetail)
    throws SQLException;

  public abstract int updateFiredTrigger(Connection paramConnection, OperableTrigger paramOperableTrigger, String paramString, JobDetail paramJobDetail)
    throws SQLException;

  public abstract List<FiredTriggerRecord> selectFiredTriggerRecords(Connection paramConnection, String paramString1, String paramString2)
    throws SQLException;

  public abstract List<FiredTriggerRecord> selectFiredTriggerRecordsByJob(Connection paramConnection, String paramString1, String paramString2)
    throws SQLException;

  public abstract List<FiredTriggerRecord> selectInstancesFiredTriggerRecords(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract Set<String> selectFiredTriggerInstanceNames(Connection paramConnection)
    throws SQLException;

  public abstract int deleteFiredTrigger(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int selectJobExecutionCount(Connection paramConnection, JobKey paramJobKey)
    throws SQLException;

  public abstract int insertSchedulerState(Connection paramConnection, String paramString, long paramLong1, long paramLong2)
    throws SQLException;

  public abstract int deleteSchedulerState(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract int updateSchedulerState(Connection paramConnection, String paramString, long paramLong)
    throws SQLException;

  public abstract List<SchedulerStateRecord> selectSchedulerStateRecords(Connection paramConnection, String paramString)
    throws SQLException;

  public abstract void clearData(Connection paramConnection)
    throws SQLException;
}