package org.quartz.spi;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import org.quartz.Calendar;
import org.quartz.JobDetail;
import org.quartz.JobKey;
import org.quartz.JobPersistenceException;
import org.quartz.ObjectAlreadyExistsException;
import org.quartz.SchedulerConfigException;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.Trigger.CompletedExecutionInstruction;
import org.quartz.Trigger.TriggerState;
import org.quartz.TriggerKey;
import org.quartz.impl.matchers.GroupMatcher;

public abstract interface JobStore
{
  public abstract void initialize(ClassLoadHelper paramClassLoadHelper, SchedulerSignaler paramSchedulerSignaler)
    throws SchedulerConfigException;

  public abstract void schedulerStarted()
    throws SchedulerException;

  public abstract void schedulerPaused();

  public abstract void schedulerResumed();

  public abstract void shutdown();

  public abstract boolean supportsPersistence();

  public abstract long getEstimatedTimeToReleaseAndAcquireTrigger();

  public abstract boolean isClustered();

  public abstract void storeJobAndTrigger(JobDetail paramJobDetail, OperableTrigger paramOperableTrigger)
    throws ObjectAlreadyExistsException, JobPersistenceException;

  public abstract void storeJob(JobDetail paramJobDetail, boolean paramBoolean)
    throws ObjectAlreadyExistsException, JobPersistenceException;

  public abstract void storeJobsAndTriggers(Map<JobDetail, List<Trigger>> paramMap, boolean paramBoolean)
    throws ObjectAlreadyExistsException, JobPersistenceException;

  public abstract boolean removeJob(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract boolean removeJobs(List<JobKey> paramList)
    throws JobPersistenceException;

  public abstract JobDetail retrieveJob(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract void storeTrigger(OperableTrigger paramOperableTrigger, boolean paramBoolean)
    throws ObjectAlreadyExistsException, JobPersistenceException;

  public abstract boolean removeTrigger(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract boolean removeTriggers(List<TriggerKey> paramList)
    throws JobPersistenceException;

  public abstract boolean replaceTrigger(TriggerKey paramTriggerKey, OperableTrigger paramOperableTrigger)
    throws JobPersistenceException;

  public abstract OperableTrigger retrieveTrigger(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract boolean checkExists(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract boolean checkExists(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract void clearAllSchedulingData()
    throws JobPersistenceException;

  public abstract void storeCalendar(String paramString, Calendar paramCalendar, boolean paramBoolean1, boolean paramBoolean2)
    throws ObjectAlreadyExistsException, JobPersistenceException;

  public abstract boolean removeCalendar(String paramString)
    throws JobPersistenceException;

  public abstract Calendar retrieveCalendar(String paramString)
    throws JobPersistenceException;

  public abstract int getNumberOfJobs()
    throws JobPersistenceException;

  public abstract int getNumberOfTriggers()
    throws JobPersistenceException;

  public abstract int getNumberOfCalendars()
    throws JobPersistenceException;

  public abstract Set<JobKey> getJobKeys(GroupMatcher<JobKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract Set<TriggerKey> getTriggerKeys(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract List<String> getJobGroupNames()
    throws JobPersistenceException;

  public abstract List<String> getTriggerGroupNames()
    throws JobPersistenceException;

  public abstract List<String> getCalendarNames()
    throws JobPersistenceException;

  public abstract List<OperableTrigger> getTriggersForJob(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract Trigger.TriggerState getTriggerState(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract void pauseTrigger(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract Collection<String> pauseTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract void pauseJob(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract Collection<String> pauseJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract void resumeTrigger(TriggerKey paramTriggerKey)
    throws JobPersistenceException;

  public abstract Collection<String> resumeTriggers(GroupMatcher<TriggerKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract Set<String> getPausedTriggerGroups()
    throws JobPersistenceException;

  public abstract void resumeJob(JobKey paramJobKey)
    throws JobPersistenceException;

  public abstract Collection<String> resumeJobs(GroupMatcher<JobKey> paramGroupMatcher)
    throws JobPersistenceException;

  public abstract void pauseAll()
    throws JobPersistenceException;

  public abstract void resumeAll()
    throws JobPersistenceException;

  public abstract List<OperableTrigger> acquireNextTriggers(long paramLong1, int paramInt, long paramLong2)
    throws JobPersistenceException;

  public abstract void releaseAcquiredTrigger(OperableTrigger paramOperableTrigger)
    throws JobPersistenceException;

  public abstract List<TriggerFiredResult> triggersFired(List<OperableTrigger> paramList)
    throws JobPersistenceException;

  public abstract void triggeredJobComplete(OperableTrigger paramOperableTrigger, JobDetail paramJobDetail, Trigger.CompletedExecutionInstruction paramCompletedExecutionInstruction)
    throws JobPersistenceException;

  public abstract void setInstanceId(String paramString);

  public abstract void setInstanceName(String paramString);

  public abstract void setThreadPoolSize(int paramInt);
}